#define _POSIX_SOURCE

#include <stdio.h>
#include <sys/types.h>
#include <sys/times.h>



#ifdef RS6000
#define TICKS 100.0
#endif
#ifdef DEC_ALPHA
#define TICKS 60.0
#endif

#define N 999

#define GETTIMER    times (&local);
#define USER_TIME   (float)(local.tms_utime)
#define SYSTEM_TIME (float)(local.tms_stime)
#define ELAPSED_TIME USER_TIME + SYSTEM_TIME

float     time_sum[N],     time_start[N],     time_end[N];
float cpu_time_sum[N], cpu_time_start[N], cpu_time_end[N];
float sys_time_sum[N], sys_time_start[N], sys_time_end[N];

int timer_rank;

struct tms local;

#if defined(DEC_ALPHA)
void clock_(){
}
#endif

#if defined(RS6000)
void timerclear (int *i){
#else
void timerclear_ (int *i){
#endif

  time_sum    [*i] = 0; time_start    [*i] = 0; time_end     [*i] = 0;
  cpu_time_sum[*i] = 0; cpu_time_start[*i] = 0; cpu_time_end [*i] = 0;
  sys_time_sum[*i] = 0; sys_time_start[*i] = 0; sys_time_end [*i] = 0;
}

#if defined(RS6000)
void timerinit (int *rank){
  int i;
  timer_rank = *rank;
  for (i=0; i < N; i++)
    timerclear (&i);
}
#else
void timerinit_ (int *rank){
  int i;
  timer_rank = *rank;
  for (i=0; i < N; i++)
    timerclear_ (&i);
}
#endif

#if defined(RS6000)
void timerstart (int *i) {
#else
void timerstart_ (int *i) {
#endif

  GETTIMER;

  time_start[*i]     =  ELAPSED_TIME;

  cpu_time_start[*i] =  USER_TIME;
  sys_time_start[*i] =  SYSTEM_TIME;
}

#if defined(RS6000)
void timerstop (int *i) {
#else
void timerstop_ (int *i) {
#endif

  GETTIMER;

  time_end[*i] = ELAPSED_TIME - time_start[*i];
  cpu_time_end[*i] = USER_TIME   - cpu_time_start[*i];
  sys_time_end[*i] = SYSTEM_TIME - sys_time_start[*i];
}

#if defined(RS6000)
void timerprint (int *i, char *what){
#else
void timerprint_ (int *i, char *what){
#endif

  fprintf (stderr,"%3d: TIMER %3d: %5.3f = %5.3f+%5.3f (%s)\n",
	  timer_rank,*i,time_end[*i]/TICKS, 
	  cpu_time_end[*i]/TICKS, 
	  sys_time_end[*i]/TICKS, what);
}

#if defined(RS6000)
void timerget (int *i, float *e, float *u, float *s){
#else
void timerget_ (int *i, float *e, float *u, float *s){
#endif
	  *e = time_end[*i]/TICKS; 
	  *u = cpu_time_end[*i]/TICKS; 
	  *s = sys_time_end[*i]/TICKS;
}

#if defined(RS6000)
void timerdump (int *i, char *what){
  timerstop (i);
  timerprint(i, what);
}

#else
void timerdump_ (int *i, char *what){
  timerstop_ (i);
  timerprint_(i, what);
}

#endif

/*
void main() {

  int i;
  double a;

  int n;
  char c[255];
  c[0]="\0";

  n = 1;

  timerinit_();
  timerstart_(&n);

  for (i=0; i<100000000; i++){
    a = 4/a  ;
  }

  timerdump_(&n,&c);


}
*/
