!     Gregor von Laszewski
!     Northeast Parallel Architectures Center
!     Syracuse NY
!     1995
!     ------------------------------------------------------------
!     ------------------------------------------------------------
      subroutine write1d (unit, name, a, n)

      implicit none

      integer unit
      character*8 name
      real a(*)
      integer n

      integer i

      write (unit) name(1:8)
      write (unit) 1
      write (unit) n
      do i=1,n
         write (unit) a(i)
      end do
      end
!     ------------------------------------------------------------
      subroutine read1d (unit, name, a, n)

      implicit none

      integer unit
      character*8 name
      real a(n)
      integer n

      integer i, dimension

      read (unit) name(1:8)
      read (unit) dimension
      read (unit) n
      do i=1,n
         read (unit) a(i)
      end do
      end
!     ------------------------------------------------------------
!     ------------------------------------------------------------




      subroutine write2d (unit, name, a, n,m)

      implicit none

      integer unit
      character*8 name
      real a(n,m)
      integer n,m

      integer i,j

      write (unit) name(1:8)
      write (unit) 2
      write (unit) n,m
      do i=1,n
         do j=1,m
            write (unit) a(i,j)
         end do
      end do
      end
!     ------------------------------------------------------------
      subroutine read2d (unit, name, a, n,m)

      implicit none

      integer unit
      character*8 name
      real a(n,m)
      integer n,m

      integer i,j,dimension

      read (unit) name(1:8)
      read (unit) dimension
      read (unit) n,m
      do i=1,n
         do j=1,m
            read (unit) a(i,j)
         end do
      end do
      end
!     ------------------------------------------------------------
!     ------------------------------------------------------------
      subroutine write3d (unit, name, a, n,m,k)

      implicit none

      integer unit
      character*8 name
      real a(n,m,k)
      integer n,m,k

      integer i,j,l

      write (unit) name(1:8)
      write (unit) 3
      write (unit) n,m,k
      do i=1,n
         do j=1,m
            do l=1,k
               write (unit) a(i,j,l)
            end do
         end do
      end do
      end
!     ------------------------------------------------------------
      subroutine read3d (unit, name, a, n,m,k)

      implicit none

      integer unit
      character*8 name
      real a(n,m,k)
      integer n,m,k

      integer i,j,l, dimension

      read (unit) name(1:8)
      read (unit) dimension
      read (unit) n,m,k
      do i=1,n
         do j=1,m
            do l=1,k
               read (unit) a(i,j,l)
            end do
         end do
      end do
      end

!     ------------------------------------------------------------
!     ------------------------------------------------------------




      program oneoi

#include <vrslv.com>
#include <oirslv.com>
#include <vmetric.com>
#include <vstate.com>
#include <iau.com>

      parameter (tmpunit = 49 )

      parameter    ( zero = 0.0 )
      integer      im, jnp, nymdb, nhmsb, nymde, nhmse, ndbud,
     &             ndout, ndt, ndrsw, ndrsh, errorcode
      real         alpha, dummy, dummy2
      real         plevs(50)
      character*60 message
      character*80 xlabel
      character* 8 exp_id, atime, adate
      character* 4 jobno(2), jobnx(2), scheme
 
      dimension ndfreq (ndiagt)
      dimension nfreqz (ndiagt)
      dimension ndindx (ndiagt)


c Local variables used by the model and assimilation driver
c =========================================================

      logical      qbudget, qrestart_hist,
     &             qrestart_write, qbegin, qend,
     &             leap, mats, repack, qqinit,
     &             qprog, qbud, 
     &             qendseg


c Unit numbers for I/O
c ====================

      integer budg_unit
      integer prog_unit
      integer diag_unit (10)
      data    budg_unit /14/
      data    prog_unit /35/
      data    diag_unit /41,42,43,44,45,46,47,48,49,50/

      data kbc   /16/
      data krsh  /36/
      data krsw  /37/
      data kdgn  /38/
      data kiau  /39/
      data koip  /95/


c HHMMSS event timestep counter
c =============================
      alarm( mhms ) = nalarm ( mhms, nymd, nhms, nymd0, nhms0 )

c  Frequency to Seconds Converter
c  ==============================
      nsecf(n) = n/10000*3600 + mod(n,10000)/100*60 + mod(n,100)


C **********************************************************************
C ****                     INITIALIZATION                           ****
C **********************************************************************

      dummy  = 0.
      dummy2 = 0.
      call wnlline(1)
      call phnxdate( adate, atime )
      write(6,1000)  adate, atime

C **********************************************************************
C ****        READ AND INITIALIZE MODEL SETUP AND DIAGNOSTICS       ****
C **********************************************************************


      CALL SETUP ( JOBNO   , XLABEL  , NYMDB   , NHMSB   ,
     .             NYMDE   , NHMSE   , NDBUD   , NDOUT   , NDSEG  ,
     .             NDT     , NDRSW   , NDRSH   , LEAP    , NDPNT  ,
     .             MATS    , ALPHA   , QQINIT  , PLEVS   , NULEV   )

      if (leap) scheme = 'LEAP'
      if (mats) scheme = 'MATS'

      call initial  ( imr, jmr+1, nlayr, nlevoi, pmand, repack )
      call cortable ( pmand,nlevoi )
      call trigset

C **********************************************************************
C ****                GET INITIAL/RESTART CONDITIONS                ****
C **********************************************************************

      irs = 1
      call restart ( krsw,irs,nymd,nhms,nb,nymd0,nhms0,jobnx,nrtype )
      write(6,2000)  krsw, nymd, nhms, nymd0, nhms0, nrtype 

C **********************************************************************
C ****                GET BOUNDARY CONDITIONS                       ****
C **********************************************************************

      call prtbcs( kbc,nymd,nhms )

C **********************************************************************
C ****                 Perform OI Analysis                          ****
C **********************************************************************

c Convert to A-Grid and Load A-Grid First-Guess into IAU Common
c -------------------------------------------------------------
      call ctoaset ( nb )
      call cload ( nb,'FG' )


      call write2d (tmpunit, "phisz   ", phisz       , imr,jmr+1     )         
      call write2d (tmpunit, "pz      ", pz(1,1,  nb), imr,jmr+1     )         
      call write3d (tmpunit, "uz      ", uz(1,1,1,nb), imr,jmr+1,nlayr )         
      call write3d (tmpunit, "vz      ", vz(1,1,1,nb), imr,jmr+1,nlayr )         
      call write3d (tmpunit, "tz      ", tz(1,1,1,nb), imr,jmr+1,nlayr )         
      call write3d (tmpunit, "qz      ", qz(1,1,1,nb), imr,jmr+1,nlayr )         
      call write3d (tmpunit, "pkht    ", pkht(1,1,1), imr,jmr+1,nlayr  )         

      close (tmpunit)

      stop 

c Perform OI Analysis to obtain A-Grid After Analysis
c ---------------------------------------------------
      qbudget = ndbud.ne.0
      call zeitbeg('GLASSIM')

      
      call glassim (repack, leap, mats, nymdb, nymde, nhmsb,
     &              nhmse, ndt, jobno, nb, nd, nymd, nhms, 
     &              qbudget )



      call zeitend

c Compute A-Grid IAU Increments and Convert to C-Grid
c ---------------------------------------------------
      call cload ( nb,'AI' )
      call atoc  ( duanal, duanal, imr, jmr+1, nlayr, 1 )
      call atoc  ( dvanal, dvanal, imr, jmr+1, nlayr, 2 )

c Archive IAU Increments
c ----------------------
      nrtype = 1
      call iauout (kiau,nymd,nhms,nb,nymd0,nhms0,xlabel,jobno,nrtype)

      call zeitend
      call zeitpri(6)
      call flash ( 0, message, 2 ) 
      stop 'Egress.'

 1000 FORMAT (1x,//,
     .        1x,'Goddard Earth Observing System (GEOS)',/,
     .        1X,'Potential Enstrophy and Energy Conserving GCM',/,
     .        1X,'Optimum Interpolation Analysis System',//,
     .        1X,'Starting Computation at Local Time: ',A8,' on ',A8,/)

 2000 FORMAT (1X,'Read    Restart From   Unit:  ',I2,/,
     .        1X,'Current Restart Record Date:  ',I6,1X,I6,/,
     .        1X,'  Experiment Beginning Date:  ',I6,1X,I6,/,
     .        1X,'               Restart Type:  ',I6,/,
     .        1X,' Read Diagnostics from Unit:  ',I2,/)

 3000 FORMAT (1X,'Write   Restart To     Unit:  ',I2,/,
     .        1X,'Current Restart Record Date:  ',I6,1X,I6,/,
     .        1X,'  Experiment Beginning Date:  ',I6,1X,I6,/,
     .        1X,'               Restart Type:  ',I6,/,
     .        1X,'  Write Diagnostics to Unit:  ',I2,/)


      end
      SUBROUTINE SETUP ( JOBX    , XLABX   , NYMDBX  , NHMSBX  ,
     .                   NYMDEX  , NHMSEX  , NDBUDX  , NDOUTX  , NDSEGX,
     .                   NDTX    , NDRSWX  , NDRSHX  , LEAPX   , NDPNTX,
     .                   MATSX   , ALPHAX  , QQINITX , PLEVSX  ,
     .                   NULEVX  )
C***********************************************************************
C
C  PURPOSE
C     SET CONTROL FOR MODEL EXECUTION AND OUTPUT
C  I/O DDNAME  DESCRIPTION
C     11       REWINDABLE COPY OF CARD-IMAGE DATA FROM INPUT
C
C***********************************************************************
C******               DATA ASSIMILATION OFFICE                    ******
C***********************************************************************

      DIMENSION    PLEVSX(50), PLEVS(50)
      CHARACTER*40 XLAB(2), XLABX(2)
      CHARACTER*8  JOB, JOBX
      LOGICAL      LEAP, MATS, QQINIT, LEAPX, MATSX, QQINITX
      PARAMETER  ( ZERO = 0.0 )

C  SETUP INPUT NAMELIST
C  ====================
      NAMELIST /INPUT/  JOB     , XLAB    , NYMDB   , NHMSB   ,
     .                  NYMDE   , NHMSE   , NDBUD   , NDOUT   ,
     .                  NDT     , NDRSW   , NDRSH   , LEAP    ,
     .                  MATS    , ALPHA   , QQINIT  , PLEVS   ,

     .                  NDSEG   , NDPNT

      NAMELIST /INPTZ/  JOB     , XLAB    , NYMDB   , NHMSB   ,
     .                  NYMDE   , NHMSE   , NDBUD   , NDOUT   ,
     .                  NDT     , NDRSW   , NDRSH   , LEAP    ,
     .                  MATS    , ALPHA   , QQINIT  , NDSEG   ,
     .                  NDPNT
       
C  VARIABLE     TYPE                    DESCRIPTION
C  ========  ===========   ==========================================
C  JOB        CHARACTER      EXPERIMENT IDENTIFIER
C  XLAB       CHARACTER      EXPERIMENT DESCRIPTION LABEL
C  NYMDB        INTEGER      BEGINNING YYMMDD
C  NHMSB        INTEGER      BEGINNING HHMMSS
C  NYMDE        INTEGER      ENDING    YYMMDD
C  NHMSE        INTEGER      ENDING    YYMMDD
C  NDSEG        INTEGER      HHMMSS FREQUENCY OF PRODUCTION  SEGMENT
C  NDBUD        INTEGER      HHMMSS FREQUENCY OF BUDGET      OUTPUT
C  NDOUT        INTEGER      HHMMSS FREQUENCY OF GENERIC GLA OUTPUT
C  NDRSW        INTEGER      HHMMSS FREQUENCY OF RESTART     OUTPUT
C  NDRSH        INTEGER      HHMMSS FREQUENCY OF RESTART     HISTORY
C  NDPNT        INTEGER      HHMMSS FREQUENCY OF POINT BY POINT DIAG
C  NDT          INTEGER      TIME-STEP (SECONDS)
C  MATS         LOGICAL      LOGICAL FLAG FOR MATSUNO  TIME-SCHEME
C  LEAP         LOGICAL      LOGICAL FLAG FOR LEAPFROG TIME-SCHEME
C  ALPHA           REAL      TIME-FILTER COEFFICIENT FOR LEAPFROG
C  QQINIT       LOGICAL      LOGICAL FLAG TO INITIALIZE QQ FIELD
C  PLEVS           REAL      PRESSURE LEVELS USED FOR OUTPUT (50 MAX)
C     
C **********************************************************************
C ****              INITIALIZE SETUP NAMELIST VARIABLES             ****
C **********************************************************************
       
      DATA PLEVS /50*0.0/
       
      LEAP   = .FALSE.
      MATS   = .FALSE.
      QQINIT = .FALSE.
      ALPHA  = ZERO
      NDT    = 0
      NDBUD  = 0
      NDOUT  = 0
      NDSEG  = 0
      NDRSW  = 0
      NDRSH  = 0
      NDPNT  = 0
      NYMDB  = 0
      NHMSB  = 0
      NYMDE  = 0
      NHMSE  = 0
       
C **********************************************************************
C ****                     READ SETUP NAMELIST                      ****
C **********************************************************************
       
      REWIND 11
      WRITE (6 ,1000)
      READ  (11,INPUT)
      WRITE (6 ,INPTZ)
       
      NULEV  = 0
      DO 10 I=1,50
          PLEVSX(I) = PLEVS(I)
      IF( PLEVSX(I).NE.0.0 ) NULEV = NULEV + 1
   10 CONTINUE
      IF( NULEV.NE.0 ) THEN
      PRINT *, 'PRESSURE LEVELS FOR OUTPUT:'
      WRITE(6,100) (PLEVSX(I),I=1,NULEV)
  100 FORMAT(1X,10(F6.1,2X))
      ENDIF
       
C **********************************************************************
C ****                        ASSIGN VALUES                         ****
C **********************************************************************
       
      JOBX     = JOB
      XLABX(1) = XLAB(1)
      XLABX(2) = XLAB(2)
      LEAPX    = LEAP
      MATSX    = MATS
      QQINITX  = QQINIT
      ALPHAX   = ALPHA
      NDTX     = NDT
      NDSEGX   = NDSEG
      NDBUDX   = NDBUD
      NDOUTX   = NDOUT
      NDRSWX   = NDRSW
      NDRSHX   = NDRSH
      NDPNTX   = NDPNT
      NYMDBX   = NYMDB
      NHMSBX   = NHMSB
      NYMDEX   = NYMDE
      NHMSEX   = NHMSE
      NULEVX   = NULEV
       
 1000 FORMAT(1X,'READING SETUP NAMELIST:')
      RETURN
      END
      subroutine chkdiag (ndfreq,nfreq,nymd,nhms,nymd0,nhms0,
     .                    ndindx,ndsum,nymdt,nhmst,ndiagt)
      dimension ndfreq(ndiagt)
      dimension ndindx(ndiagt)
      nsecf(n) = n/10000*3600 + MOD(n,10000)/100*60 + MOD(n,100)
 
      nymdt = nymd
      nhmst = nhms
      nymdd = nymd0
      nhmsd = nhms0
      if(nfreq.le.0) then
      ntick    = nsecf( abs(nfreq) )/2
      call tick ( nymdd,nhmsd,-ntick,ndum )
      call tick ( nymdt,nhmst,-ntick,ndum )
      endif

c Initialize Diagnostic Index
c ---------------------------
      do n=1,ndiagt
      ndindx(n) = 0
      enddo

c Fill Current Diagnostic Index
c -----------------------------
      ndsum = 0
      do n=1,ndiagt
      nafreq = abs(ndfreq(n))
      if(         ndfreq(n).eq.nfreq .and.
     .    nalarm( nafreq,nymd,nhms,nymdd,nhmsd ).eq.0 ) then
      ndsum = ndsum + 1
      ndindx(ndsum) = n
      endif
      enddo
      return
      end
      SUBROUTINE CLOAD (NB,IAFLG)                                  
C                                                                       
C.... ROUTINE TO LOAD 6H FCST FIELDS OR ANALYSIS INCREMENTS             
C     INTO IAI COMMON BLOCK (CONTROLLED BY IAFLG, 'FG' OR 'AI')         
                                                                        
#include <vrslv.com>
#include <vstate.com>
#include <iau.com>

      PARAMETER( NTOT  = IMR*(JMR+1)*NLAYR )                            
      PARAMETER( IMJMR = IMR*(JMR+1) )                                  
                                                                        
      PARAMETER( ZERO  = 0.0 )                                          

      CHARACTER*2 IAFLG                                                 
C                                                                       
      IM  = IMR                                                         
      JNP = JMR + 1                                                     
                                                                        
      IF(IAFLG .EQ. 'FG') THEN                                          
         WRITE(6,403)                                                   
 403     FORMAT(' LOADING FG INTO INCREMENT COMMON')                    
         DO 20 L=1,NLAYR                                                
             DO 10 I=1,IMJMR                                            
                 DUANAL(I,1,L) = UZ(I,1,L,NB)                           
                 DVANAL(I,1,L) = VZ(I,1,L,NB)                           
                 DTANAL(I,1,L) = TZ(I,1,L,NB)*pz(i,1,nb)                
                 DQANAL(I,1,L) = QZ(I,1,L,NB)*pz(i,1,nb)                
 10          CONTINUE                                                   
 20      CONTINUE                                                       
                                                                        
         DO 30 I=1,IMJMR                                                
             DPANAL(I,1) = PZ(I,1,NB)                                   
 30      CONTINUE                                                       
                                                                        
      ELSE IF(IAFLG .EQ. 'AI') THEN                                     
         WRITE(6,404)                                                   
 404     FORMAT(' LOADING OI-FG INTO INCREMENT COMMON')                 
                                                                        
C********************************************************************   
C*****   HERE WE WANT THE TENDENCIES IN IAICOM                          
C********************************************************************   
C                                                                       
         DO 50 L=1,NLAYR                                                
           DO 40 I=1,IMJMR                                              
                                                                        
              DUANAL(I,1,L) = UZ(I,1,L,NB) - DUANAL(I,1,L)              
              DVANAL(I,1,L) = VZ(I,1,L,NB) - DVANAL(I,1,L)              
              DTANAL(I,1,L) = TZ(I,1,L,NB)*pz(i,1,nb) - DTANAL(I,1,L)
              DQANAL(I,1,L) = QZ(I,1,L,NB)*pz(i,1,nb) - DQANAL(I,1,L)
                                                                        
 40        CONTINUE                                                     
 50      CONTINUE                                                       
                                                                        
         DO 60 I=1,IMJMR                                                
           DPANAL(I,1) = ( PZ(I,1,NB) - DPANAL(I,1))/(6.*3600.)           
 60      CONTINUE                                                       

         do layers = 1,nlayr
          do i = 1,imjmr

            duanal(i,1,layers) = duanal(i,1,layers) / (6.*3600.)
            dvanal(i,1,layers) = dvanal(i,1,layers) / (6.*3600.)
            dtanal(i,1,layers) = dtanal(i,1,layers) / (6.*3600.)
            dqanal(i,1,layers) = dqanal(i,1,layers) / (6.*3600.)

          end do
         end do

      END IF                                                            
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE  CLRINDX ( INDXLIST )
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     DRIVER TO CLEAR DIAGNOSTICS SPECIFIED IN DIAGNOSTIC INDEX LIST
C
C  ARGUMENT DESCRIPTION                                                         
C     INDXLIST.. INTEGER DIAGNOSTIC INDEX LIST
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vstate.com>                                                           

      INTEGER INDXLIST (NDIAGT)
                                                                                
      DO  INDEX=1,NDIAGT                                                          
      N = INDXLIST (index)
      IF( N.NE.0 .AND. IDIAG(N).NE.0 ) THEN                                                
      CALL CLRDIAG (N)                                                      
      ENDIF                                                                     
      ENDDO
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE  IAUOUT (IUNIT,NYMD,NHMS,NB,NYMD0,NHMS0,XLABEL,JOB,            
     .                    NRTYPE)                                           
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     ROUTINE TO WRITE SIGMA LEVEL OUTPUT FOR IAU INCREMENTS
C     IN STANDARD PHOENIX FORMAT         
C  ARGUMENT DESCRIPTION                                                         
C     IUNIT .... OUTPUT UNIT NUMBER                                             
C     NYMD ..... CURRENT   DATE (YYMMDD) OF FORECAST                            
C     NHMS ..... CURRENT   TIME (HHMMSS) OF FORECAST                            
C     NB ....... CURRENT   TIME INDES (1 OR 2)                                  
C     NYMD0 .... BEGINNING DATE (YYMMDD) OF FORECAST                            
C     NHMS0 .... BEGINNING TIME (HHMMSS) OF FORECAST                            
C     XLABEL ... CHARACTER DESCRIPTION OF EXPERIMENT                            
C     JOB ...... EXPERIMENT JOB IDENTIFIER                                      
C     NRTYPE ... OUTPUT RECORD TYPE:  0   STRAIGHT FORECAST
C                                    -1   BEFORE   ANALYSIS
C                                     1   AFTER    ANALYSIS INITIALIZED
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <iau.com>                                                           
#include <vmetric.com>                                                          
                                                                                
      PARAMETER  ( NSFLDR  = 1  )                                               
      PARAMETER  ( NUFLDR  = 4  )                                               
                                                                                
      PARAMETER  (IMJMR   =  IMR*(JMR+1)  )                                     
      PARAMETER  (IMJMM1R =  IMR* JMR     )                                     
      PARAMETER  (IMJMM2R =  IMR*(JMR-1)  )                                     
      PARAMETER  (ONE     =  1.           )                                     
                                                                                
      COMMON /SPACE/   TEM (IMR,JMR+1)                                         
      COMMON /SPACE/ ZLEVID(NLAYR)                                           
                                                                                
      LOGICAL FIRST                                                             
      DATA    FIRST /.TRUE./                                                    
                                                                                
      CHARACTER*8    JOB
      CHARACTER*8    XLABEL(10)
      CHARACTER*8    NAMES(NSFLDR)
      CHARACTER*8    NAMEU(NUFLDR)
      CHARACTER*8    NAMED(NDIAGT)
      CHARACTER*40   DESCS(NSFLDR)
      CHARACTER*40   DESCU(NUFLDR)
      CHARACTER*40   DESCD(NDIAGT)
                                                                                
      INTEGER        NDLEV(NDIAGT)                                              
                                                                                
      PARAMETER    (NDUM = 2 )
      INTEGER       IDUM(NDUM)
      REAL          RDUM(NDUM)
      CHARACTER*40  DDUM(NDUM)
      CHARACTER*8   CDUM(NDUM)

      IF (FIRST) THEN                                                           
      CALL SETMET                                                               
      FIRST = .FALSE.                                                           
      ENDIF                                                                     
                                                                                
C *********************************************************************         
C *****                 SET HEADER INFORMATION                     ****         
C *********************************************************************         
                                                                                
      NDATE  = NYMD                                                            
      NTIME  = NHMS                                                            
      NDATE0 = NYMD0                                                           
      NTIME0 = NHMS0                                                           
                                                                                
      NSFLD = NSFLDR                                                          
      NUFLD = NUFLDR                                                          
      NDFLD = 0                                                                 
                                                                                
      NAMES(1) = CDIAG(83)
      DESCS(1) = TDIAG(83)
  
      NAMEU(1) = CDIAG(54)
      NAMEU(2) = CDIAG(55)
      NAMEU(3) = CDIAG(56)
      NAMEU(4) = CDIAG(57)

      DESCU(1) = TDIAG(54)
      DESCU(2) = TDIAG(55)
      DESCU(3) = TDIAG(56)
      DESCU(4) = TDIAG(57)

      DO L=1,NLAYR
      ZLEVID(L) = nlayr-L+1
      enddo

C *********************************************************************
C *****            SET USER-DEFINED HEADER VARIABLES               ****
C *********************************************************************

      UNDEF   = getcon('UNDEF')

      ddum(1) = 'OUTPUT RECORD TYPE (INTEGER)'
      idum(1) =  nrtype
      rdum(1) =  undef
      cdum(1) = 'LOG8R'

      ddum(2) = 'UNDEFINED FILL VALUE (REAL)'
      idum(2) =  0
      rdum(2) =  undef
      cdum(2) = 'UNDEF'

C *********************************************************************         
C *****                     HEADER RECORD                          ****         
C *********************************************************************         
                                                                                
      WRITE (IUNIT) JOB, NDATE, NTIME, NDATE0, NTIME0,                          
     .              XLABEL, IM, JNP,                                            
     .              NSFLD, NUFLD, NDFLD,                                        
     .              PTOP, NLAYR, (ZLEVID(K),K=1,NLAYR),                           
     .              NLAY, (SIGE(K),K=1,NLAYP1),                                 
     .              NDUM, (DDUM(N),IDUM(N),RDUM(N),CDUM(N),N=1,NDUM)

                                                                                
      WRITE (IUNIT) (NAMES(N),DESCS(N),         N=1,NSFLD),                     
     .              (NAMEU(N),DESCU(N),         N=1,NUFLD),                     
     .              (NAMED(N),DESCD(N),NDLEV(N),N=1,NDFLD)                      
                                                                                
      write(6,1000) iunit,nymd,nhms

 1000 format(1x,'Writing IAU Increments to Unit ',i2,' with a Time',
     .         ' Stamp of ',i6,2x,i6)

C *********************************************************************
C *****                     WRITE IAU INCREMENTS                   ****
C *********************************************************************
       
      ZLEV = 0.
      CALL  CRAWRT ( DPANAL(1,1)  , IMJMR, IUNIT, TEM, ZLEV, NAMES(1) )

      DO     L = NLAYR,1,-1
      ZLEV = L
      CALL  CRAWRT ( DUANAL(1,1,L), IMJMR, IUNIT, TEM, ZLEV, NAMEU(1) )
      CALL  CRAWRT ( DVANAL(1,1,L), IMJMR, IUNIT, TEM, ZLEV, NAMEU(2) )
      CALL  CRAWRT ( DTANAL(1,1,L), IMJMR, IUNIT, TEM, ZLEV, NAMEU(3) )
      CALL  CRAWRT ( DQANAL(1,1,L), IMJMR, IUNIT, TEM, ZLEV, NAMEU(4) )
      ENDDO
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE  CRAWRT  (A, IMJM, KTP, TEM, ZLEV, NAME)
      DIMENSION   A   (IMJM)
      DIMENSION   TEM (IMJM)
      CHARACTER*8 NAME

      DO  I=1,IMJM
      TEM (I) = A(I)
      ENDDO

      WRITE(KTP)  ZLEV, NAME, TEM

      RETURN
      END
      subroutine glassim( repack, leap, mats, nymdb, nymde, nhmsb,
     &                    nhmse, ndt, exp_id, nb, nd, nymd, nhms, 
     &                    qbudget_switch )
C
C***********************************************************************
C************       NASA/GODDARD SPACE FLIGHT CENTER        ************
C********              LABORATORY FOR ATMOSPHERES               ********
C******                                                           ******
C******          Data Assimilation Office  -  Code 910.3          ******
C********       Optimum Interpolation Assimilation System       ********
C************         Frozen Version 2.0 - October 1994     ************
C***********************************************************************
#include <vrslv.com>
#include <oirslv.com>
#include <vstate.com>
#include <vmetric.com>
#include <vbudget.com>
C
      	

      common /contrl/   im0,     jnp0,    nb0,     ndalt0,  nhms0,
     &                  nlay0,   nlaym10, nlayp10, nymd0,   strat,
     &                  idimen0,  slp,    fdcos,   fexp,   saveae,
     &                  sigdat,  maxrep
      logical           strat,   slp,     fdcos,   fexp,   saveae,
     &                  sigdat, first, qbudget_switch, offlev
 
      character*8   dataname
      character*24  obsname
      parameter         ( mxqobs=200 )
      common /obs/ maxdex, nfgge, lun, dataname(5), hh(13,13,20),
     &             vcoroe(18,18,4), obserr(mxqobs,4,19),
     &             ww(13,13,20), obsname(mxqobs), qobs(mxqobs,5),
     &             offlev(mxqobs)
      common /scan/ maxobs, rscan, onestage, max_data
      real  rscan, data( 1500000       ), sat_wind( 150000   ),
     &      ztime(150000), ustar( imr,jmr+1 )
      logical onestage, repack, suspect( 150000  , 2 ), skippuv,
     &        skipmix, skipzuv, qobs, seasat, usesas, dalflg, 
     &        ldown
      integer bq( 150000   )
      data first /.true./
      save first, istart

      common /dalctl/ seasat, usesas, dalflg
 
! Unit numbers for I/O.

      data krsw/37/, kbud/14/, kbc/16/, kout/35/, krsh/36/,
     &     koip/95/, iread/1/, iwrit/2/, ndgoi/060000/


      real              maxvalue, minvalue
      EXTERNAL          MIXCOV
      EXTERNAL          PUVCOV
      EXTERNAL          ZUVCOV
 
C***********************************************************************
C*       FIELDS ON ANLAYSIS LEVELS
C***********************************************************************
C
C     lwflag_oi (imr,jmr+1)              LAND/WATER FLAG
C     psfc_oi   (imr,jmr+1)              SURFACE PRESSURE
C     tsfc_oi   (imr,jmr+1)              SURFACE TEMPERATURE (EXTRAPOLATED)
C     tvsfc_oi  (imr,jmr+1)              SURFACE VIRTUAL TEMPERATURE
C     psl_oi    (imr,jmr+1)              SEA LEVEL PRESSURE
C     usl_oi    (imr,jmr+1)              SEA LEVEL U WIND
C     vsl_oi    (imr,jmr+1)              SEA LEVEL V WIND
C     u_oi      (imr,jmr+1,nlevoi)       U WIND
C     v_oi      (imr,jmr+1,nlevoi)       V WIND
C     tv_oi     (imr,jmr+1,nlevoi)       VIRTUAL TEMPERATURE
C     t_oi      (imr,jmr+1,nlevoi)       DRY BULB TEMPERATURE
C     tgrnd     (imr,jmr+1)              GROUND TEMPERATURE
C     rh_oi     (imr,jmr+1,nlevoi)       RELATIVE HUMIDITY
C     rmix_oi   (imr,jmr+1,nlevoi)       MIXING RATIO
C     z_oi      (imr,jmr+1,nlevoi)       GEOPOTENTIAL HEIGHT

      real lwflag_oi (imr,jmr+1)
      real psfc_oi   (imr,jmr+1)
      real tsfc_oi   (imr,jmr+1)
      real tvsfc_oi  (imr,jmr+1)
      real psl_oi    (imr,jmr+1)
      real phis      (imr,jmr)
      real usl_oi    (imr,jmr+1)
      real vsl_oi    (imr,jmr+1)
      real u_oi      (imr,jmr+1,nlevoi)
      real v_oi      (imr,jmr+1,nlevoi)
      real tv_oi     (imr,jmr+1,nlevoi)
      real tgnd      (imr,jmr+1)
      real t_oi      (imr,jmr+1,nlevoi)
      real rh_oi     (imr,jmr+1,nlevoi)
      real rmix_oi   (imr,jmr+1,nlevoi)
      real z_oi      (imr,jmr+1,nlevoi)
      real zlwflag   (imr,jmr+1)

C***********************************************************************
C*       ANALYSIS INCREMENTS
C***********************************************************************
 
      real usl_inc    (imr,jmr+1)          !    SEA LEVEL U WIND
      real vsl_inc    (imr,jmr+1)          !    SEA LEVEL V WIND
      real u_inc      (imr,jmr+1,nlevoi)   !    U WIND
      real v_inc      (imr,jmr+1,nlevoi)   !    V WIND
      real rmix_inc   (imr,jmr+1,nlevoi)   !    MIXING RATIO
      real tv_inc     (imr,jmr+1,nlevoi)   !    TEMPERATURE
C
C***********************************************************************
C*       FIRST GUESS/ANALYSIS ERROR STANDARD DEVIATIONS                *
C***********************************************************************
 
C     psl_err  (imr,jmr+1)              SEA LEVEL PRESSURE
C     tsl_err  (imr,jmr+1)              SEA LEVEL TEMPERATURE
C     rmix_err (imr,jmr+1,nlevoi)       MIXING RATIO
!     u_err    (imr,jmr+1,nlevoi)       U-WIND                    <1.3>  
!     v_err    (imr,jmr+1,nlevoi)       V_WIND                    <1.3>  
!     usl_err  (imr,jmr+1)              SEA-LEVEL U-WIND          <1.3>
!     vsl_err  (imr,jmr+1)              SEA-LEVEL V-WIND          <1.3>
 
      real psl_err    (imr,jmr+1)
      real tsl_err    (imr,jmr+1)
      real rmix_err   (imr,jmr+1,nlevoi+1)
      real u_err      (imr,jmr+1,nlevoi+1)
      real v_err      (imr,jmr+1,nlevoi+1)
      real mass_err   (imr,jmr+1,nlevoi+1)
 
      save psl_err, tsl_err, rmix_err, mass_err, u_err, 
     &     v_err, usl_err, vsl_err

C Use model common /SPACE/ to store those analysis fields that are only
C needed during the execution of the OI.  Note that the analysis errors
C are required by subsequent analyses; they are kept out of common /SPACE/
C since it is overwritten during the execution of the GCM.
 
#ifdef CRAY
!gvl      common /space/ lwflag_oi
!gvl      common /space/ psfc_oi
!gvl      common /space/ tsfc_oi
!gvl      common /space/ tvsfc_oi
!gvl      common /space/ psl_oi
!gvl      common /space/ usl_oi
!gvl      common /space/ vsl_oi
!gvl      common /space/ u_oi
!gvl      common /space/ v_oi
!gvl      common /space/ tv_oi
!gvl      common /space/ t_oi
!gvl      common /space/ rh_oi
!gvl      common /space/ rmix_oi
!gvl      common /space/ z_oi
!gvl 
!gvl      common /space/ usl_inc
!gvl      common /space/ vsl_inc
!gvl      common /space/ u_inc
!gvl      common /space/ v_inc
!gvl      common /space/ rmix_inc
!gvl      common /space/ tv_inc
#endif
C
C***********************************************************************
C*       WORKSPACE                                                     *
C***********************************************************************
C
      real WKSPACE (IMR*(JMR+1)*(NLEVOI+1)*4 + 60000 + IMR*(JMR+1))
#ifdef CRAY
!gvl      common /space/ wkspace
#endif

      real prep_space (IMR*(JMR+1)*NLAYR*4+IMR*(JMR+1)*8)
#ifdef CRAY
!gvl      common /diag/ prep_space
#endif
 
C***********************************************************************
C*       LOCAL VARIABLES                                               *
C***********************************************************************
C
      data fill /1.e10/
 
      PARAMETER ( ZERO = 0.0  )
      LOGICAL         QBUD,  QOUT, QRSH, QRSW, QPNT
      LOGICAL         QBEG, QEND, LEAP, MATS
      LOGICAL         QQINIT,QGOI
 
      character*8     atime, adate, xlabel(10), err_exp_id
      character*4     job(2), exp_id(2)
      character*60    message
 
 
C  GRID VOLUME SETUP FOR OI VERSIONS 1.2 AND LATER
C
C     Grid volume information is contained in dataset 'mvtable', unit 88.
C     This dataset contains the following:
C
C     NAME            TYPE                DESCRIPTION
C
C     ngv             simple int          Number of grid volumes
C     volcentre_x     array real          Longitude volume centres
C     volcentre_y     array real          Latitude volume centres
C     glatup          array real          Volume reference latitude
C     glonup          array real          Volume reference longitude
C     nh              array int           No. horiz gridpoints in volume
C
      common / sparam / lsolve, ns, nr, nv, nh(13104), nq, kq(3),
     &                  volcentre_x(13104), ngv,
     &                  volcentre_y(13104), glatup(13104), glonup(13104)
 
     
      skippuv = .false.
      skipmix = .false.
      skipzuv = .false.

      write (6,'(//," GLASSIM: Ingress.")')
      write(6,'(//)')
      write(6,'(1x," NASA GODDARD EARTH OBSERVING SYSTEM (GEOS)",/,
     &        1X," DATA ANALYSIS SYSTEM - VERSION 2.0 - OCT 1994",//)')
c
c**********************************************************************
c
c        Read dataset containing minivolume information.
c
c**********************************************************************
 
 
      read( 88 ) ngv
      write( 6,'(/," Number of horizontal grid volumes:",i6)') ngv
      read( 88 ) (volcentre_x(ii), ii = 1,ngv)
      read( 88 ) (volcentre_y(ii), ii = 1,ngv)
      read( 88 ) (glatup(ii), ii = 1,ngv)
      read( 88 ) (glonup(ii), ii = 1,ngv)
      read( 88 ) (nh(ii), ii = 1,ngv)
      close( 88 )
C
C***********************************************************************
C        WORKSPACE POINTERS AND ANALYSIS PARAMETERS
C***********************************************************************
C
      IAQUAN = 1
      IZE    = iaquan + imr*(jmr+1)*(NLEVOI+1)*3
      IZELON = IZE    + 20000
      IZELAT = IZELON + 20000
      IZ1000 = IZELAT + 20000
 
      CALL GETCONH( 'DLAT    ', DLAT )

C***********************************************************************
C*       UPDATE ANALYSIS ERROR STANDARD DEVIATIONS WITH MODEL          *
C*       ERROR GROWTH (ONLY IF NOT A BEGIN CASE)                       *
C***********************************************************************
C
      if ( first ) then
         CALL RDERROR(NYMD,NHMS,psl_err,tsl_err,rmix_err,
     .        mass_err,ISTART,exp_id)
         CALL ITPINIT(IMR,JMR+1,NLEVOI,NLAYR,SIGE)
         first = .false.
      end if
      IF (ISTART.NE.0) then
         CALL GROWTH(psl_err,tsl_err,rmix_err,mass_err)
      end if
      ISTART = 1
C
C

C***********************************************************************
C*       OBTAIN FIRST GUESS FOR OI ON PRESSURE LEVELS                  *
C***********************************************************************
C
      CALL OIPREP( nb, PZ(1,1,nb), ptop, ZLWI,
     .     lwflag_oi, tsfc_oi, tvsfc_oi,
     .     psfc_oi, psl_oi,
     .     usl_oi, vsl_oi, u_oi, v_oi,
     .     tv_oi, z_oi, rmix_oi, rh_oi,
     .     PMAND, PMIDM, IMR, JMR+1, NLEVOI, nlayr,
     .     prep_space,ustar)
      
 
C Save pressure level first guess
 
      usl_inc = usl_oi
      vsl_inc = vsl_oi
      u_inc = u_oi
      v_inc = v_oi
      rmix_inc = rmix_oi
      tv_inc = tv_oi
C
C
C***********************************************************************
C*       COMPUTATION OF PRESSURE LEVEL DRY BULB TEMPERATURES           *
C*       BEFORE ANALYSIS PRESSURE LEVEL HISTORY WRITE                  *
C***********************************************************************
C
      CALL TDRYBULB(tv_oi,rmix_oi,t_oi,PMAND,PMIDM)
 
!  Compute the first guess wind standard deviations.

      call werrinit ( mass_err, imr, jmr+1, nlevoi, u_err, v_err )

      call oihiswrt(koip,nymd,nhms,nymd0,nhms0,imr,jmr+1,nlevoi,-1,
     .     xlabel,exp_id,pmand,
     .     phisz, psl_oi, psfc_oi, tvsfc_oi, usl_oi, vsl_oi,
     .     u_oi, v_oi, t_oi, z_oi, rmix_oi, rh_oi,
     .     psl_err, u_err, v_err, mass_err, rmix_err  )

C
C        Position REPACK dataset for this synoptic time.
C        Bypass sea-level temperature data.
C
      if ( repack ) then
         call ptndat( nymd,nhms  )
         do logical_units = 1,nfgge
            call rdthru ( 1, lun-1+logical_units )
         end do
      end if
 
!**********************************************************************
!
!        Sea level pressure and wind analysis.
!
!*********************************************************************
 
      if ( .not. skippuv ) then

         call dragon( phisz, lwflag_oi, usl_oi, vsl_oi, tsfc_oi, tgz )
 
         nv = 1
         nq = 3

         call rdslp ( data, nobs, suspect )

         if ( nobs .gt. 0 ) then
            if ( seasat ) then
!gvl               call nscat(psl_err,lwflag_oi,psl_oi,usl_oi,vsl_oi,tsfc_oi,
!gvl     &              puvcov, wkspace(iaquan), mass_err, t_oi,
!gvl     &              wkspace(ize), wkspace(izelat), wkspace(izelon),
!gvl     &              nze, psfc_oi, data, nobs, suspect, type )
            end if

            call qcslp ( phisz, psl_oi, usl_oi, vsl_oi, data, nobs,
     &           t_oi, wkspace(ize), wkspace(izelat),
     &           wkspace(izelon), nze, psfc_oi, suspect,
     &           psl_err )

            mass_err(1:imr,1:jmr+1,nlevoi+1) = psl_err(1:imr,1:jmr+1)

            call puvanl( mass_err, u_err,v_err,phisz, psl_oi, usl_oi,
     &           vsl_oi,tsfc_oi,puvcov, wkspace(iaquan), t_oi,
     &           wkspace(ize), wkspace(izelat), wkspace(izelon),
     &           nze, psfc_oi, data, nobs, suspect, ztime)
            
            psl_err(1:imr,1:jmr+1) = mass_err(1:imr,1:jmr+1,nlevoi+1)
    
         else


            call flash ( 3,'GLASSIM: No slp observations.',1)
            nze=0

         end if
 
      else  ! skip the sea level pressure/wind analysis
 
         if ( repack ) then
            do logical_units = 1,nfgge
               call rdthru( 2, lun-1+logical_units )
            end do
         end if
 
      end if
 
!
!**********************************************************************
!
!      Upper air water vapour mixing ratio analysis
!
!**********************************************************************
 
      if ( .not. skipmix ) then
 
         nv = 2
         nq = 1
         mmix = 6
         call rdmix( nymd, nhms, data, bq, nobs, pmand, psfc_oi,
     &        imr, jmr+1, nlevoi, mmix )

         if ( nobs .gt. 0 ) then
            call delmix( nymd, nhms, rh_oi, rmix_oi, pmand,
     &           psfc_oi, data, nobs, imr, jmr+1, nlevoi, mmix,
     &           sigdat, bq, t_oi ) 
            call mixanl(rmix_err,u_err,v_err, rh_oi, rmix_oi, pmand, 
     &           psfc_oi, mixcov,  wkspace(iaquan), imr, jmr+1,
     &           nlevoi, mmix, nobs, data, bq, ztime)

         else
            call flash (3,'GLASSIM: No mix observations.',1)
         end if
 
      else ! skip the mixing ratio analysis
 
         if ( repack ) then
            do logical_units = 1,nfgge
               call rdthru( 3, lun-1+logical_units )
            end do
         end if
 
      end if
 
 
!
!**********************************************************************
!
!      Upper air height and wind analysis
!
!**********************************************************************
!
      if ( .not. skipzuv ) then
!
!  Recalculation of 1000mb heights using analyzed sea level analysis.
!

         call chgz1000(psfc_oi,tvsfc_oi,psl_oi,phisz,wkspace(iz1000),
     &        pmand, imr, jmr+1, nlevoi )
         nv = 2
         nq = 3
         call rdhuv ( z_oi, tv_oi, wkspace(iz1000), data, nobs,
     &        suspect, wkspace(ize), wkspace(izelat),
     &        wkspace(izelon), nze )
         if ( nobs.gt.0 ) then
            call delhuv( z_oi, u_oi, v_oi, tv_oi, psfc_oi, 
     &		 wkspace(iz1000),
     &           mass_err, u_err, v_err, data, nobs, wkspace(ize), 
     &           wkspace(izelat), wkspace(izelon), nze, nymd,
     &           nhms, suspect, ztime )
            call zuvanl ( mass_err, u_err, v_err, z_oi, u_oi, v_oi, 
     &		 t_oi,
     &		 psfc_oi, wkspace(iz1000), zuvcov, wkspace(iaquan),
     &           wkspace(ize), wkspace(izelat),
     &           wkspace(izelon), nze, data, nobs, ztime,
     &           nymd, nhms)
         else
            call flash(3,'GLASSIM: No zuv observations.',1)
         end if
 
 
      end if
 
      CALL PRPTLY

C
C***********************************************************************
C*       COMPUTE ANALYSIS INCREMENTS                                   *
C***********************************************************************
 
      do i=1,imr*(jmr+1)
         if (usl_inc(i,1).ne.fill) then
            usl_inc(i,1)=usl_oi(i,1)-usl_inc(i,1)
         end if
      end do

      do i=1,imr*(jmr+1)
         if (vsl_inc(i,1).ne.fill) then
            vsl_inc(i,1)=vsl_oi(i,1)-vsl_inc(i,1)
         end if
      end do


      do i=1,imr*(jmr+1)*nlevoi
         if (u_inc(i,1,1).ne.fill) then
            u_inc(i,1,1)=u_oi(i,1,1)-u_inc(i,1,1)
         end if
      end do
 
      do i=1,imr*(jmr+1)*nlevoi
         if (v_inc(i,1,1).ne.fill) then
            v_inc(i,1,1)=v_oi(i,1,1)-v_inc(i,1,1)
         end if
      end do
 
      do i=1,imr*(jmr+1)*nlevoi
         if (rmix_inc(i,1,1).ne.fill) then
            rmix_inc(i,1,1)=rmix_oi(i,1,1)-rmix_inc(i,1,1)
         end if
      end do
 
      call ztot(z_oi,tv_oi,PMAND,PMIDM,imr,jmr+1,nlevoi)
 
      do i=1,imr*(jmr+1)*nlevoi
         tv_inc(i,1,1)=tv_oi(i,1,1)-tv_inc(i,1,1)
      end do
C
C***********************************************************************
C*       COMPUTATION OF PRESSURE LEVEL DRY BULB TEMPERATURES           *
C*       AFTER ANALYSIS PRESSURE LEVEL HISTORY WRITE                   *
C***********************************************************************
C
      CALL TDRYBULB(tv_oi,rmix_oi,t_oi,PMAND,PMIDM)
 
      call werrinit ( mass_err, imr, jmr+1, nlevoi, u_err, v_err )

      call mixtorh ( rmix_oi, t_oi, pmand, imr, jmr+1, nlevoi,
     &     rh_oi, fill )

      do i = 1,imr
         do j = 1,jmr+1
            do k = 1,nlevoi

               if ( rh_oi(i,j,k) .ne. fill ) then
                  rh_oi(i,j,k) = rh_oi(i,j,k)*100.
               end if

            end do
         end do
      end do


      call oihiswrt(koip,nymd,nhms,nymd0,nhms0,imr,jmr+1,nlevoi,0,
     .     xlabel,exp_id,pmand,
     .     phisz, psl_oi, psfc_oi, tvsfc_oi, usl_oi, vsl_oi,
     .     u_oi, v_oi, t_oi, z_oi, rmix_oi, rh_oi,
     .     psl_err, u_err, v_err, mass_err, rmix_err  )
C
C***********************************************************************
C*       WRITE ANALYSIS ERROR STANDARD DEVIATIONS                      *
C***********************************************************************
C
      REWIND 89
      CALL OIERRWRT(89,exp_id,NYMD,NHMS,
     .     psl_err,tsl_err,rmix_err,mass_err,
     .     IMR,JMR+1,NLEVOI,WKSPACE,ISTAT)
      IF( ISTAT.NE.0 ) then
         print*,' FATAL: GLASSIM> Error in analysis error data conver',
     &        'sion.'
         stop 'Fatal.'
      end if
C
C***********************************************************************
C*       INTERPOLATE ANALYSIS INCREMENTS TO MODEL LEVELS AND UPDATE    *
C*       TIME STEP NB                                                  *
C***********************************************************************
C
C SET TIME STEP INDICIES
C
      IF(NB .EQ. 1) ND = 2
      IF(NB .EQ. 2) ND = 1
C
      CALL OIPOST(phisz, tvsfc_oi, psl_oi, usl_inc,  vsl_inc,
     .     u_inc, v_inc,    tv_inc, rmix_inc,
     .     PMAND, PMIDM,
     .     PZ(1,1,NB)  ,PZ(1,1,ND)  ,UZ(1,1,1,NB),UZ(1,1,1,ND),
     .     VZ(1,1,1,NB),VZ(1,1,1,ND),TZ(1,1,1,NB),TZ(1,1,1,ND),
     .     QZ(1,1,1,NB),QZ(1,1,1,ND),
     .     SIGE      ,  lwflag_oi  , PTOP      , PKHT        ,
     .     IMR        , JMR+1      , NLAYR     , NLEVOI      ,
     .     qbudget_switch, ZANALP     , ZANALU    , ZANALV      ,
     .     ZANALT     , ZANALQ    ,
     .     zdpdt      , zdpudt    , zdpvdt      ,
     .     zdptdt     , zdpqdt    , dsig )
 
          
      END
