      SUBROUTINE  IAUOUT (IUNIT,NYMD,NHMS,NB,NYMD0,NHMS0,XLABEL,JOB,
     .                    NRTYPE)
C***********************************************************************
C
C  PURPOSE
C     ROUTINE TO WRITE SIGMA LEVEL OUTPUT FOR IAU INCREMENTS
C     IN STANDARD PHOENIX FORMAT
C  ARGUMENT DESCRIPTION
C     IUNIT .... OUTPUT UNIT NUMBER
C     NYMD ..... CURRENT   DATE (YYMMDD) OF FORECAST
C     NHMS ..... CURRENT   TIME (HHMMSS) OF FORECAST
C     NB ....... CURRENT   TIME INDES (1 OR 2)
C     NYMD0 .... BEGINNING DATE (YYMMDD) OF FORECAST
C     NHMS0 .... BEGINNING TIME (HHMMSS) OF FORECAST
C     XLABEL ... CHARACTER DESCRIPTION OF EXPERIMENT
C     JOB ...... EXPERIMENT JOB IDENTIFIER
C     NRTYPE ... OUTPUT RECORD TYPE:  0   STRAIGHT FORECAST
C                                    -1   BEFORE   ANALYSIS
C                                     1   AFTER    ANALYSIS INITIALIZED
C
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
C
#include <vrslv.com>
#include <vstate.com>
#include <iau.com>
#include <vmetric.com>

      PARAMETER  ( NSFLDR  = 1  )
      PARAMETER  ( NUFLDR  = 4  )

      PARAMETER  (IMJMR   =  IMR*(JMR+1)  )
      PARAMETER  (IMJMM1R =  IMR* JMR     )
      PARAMETER  (IMJMM2R =  IMR*(JMR-1)  )
      PARAMETER  (ONE     =  1.           )

      COMMON /SPACE/   TEM (IMR,JMR+1)
      COMMON /SPACE/ ZLEVID(NLAYR)

      LOGICAL FIRST
      DATA    FIRST /.TRUE./

      CHARACTER*8    JOB
      CHARACTER*8    XLABEL(10)
      CHARACTER*8    NAMES(NSFLDR)
      CHARACTER*8    NAMEU(NUFLDR)
      CHARACTER*8    NAMED(NDIAGT)
      CHARACTER*40   DESCS(NSFLDR)
      CHARACTER*40   DESCU(NUFLDR)
      CHARACTER*40   DESCD(NDIAGT)

      INTEGER        NDLEV(NDIAGT)

      PARAMETER    (NDUM = 2 )
      INTEGER       IDUM(NDUM)
      REAL          RDUM(NDUM)
      CHARACTER*40  DDUM(NDUM)
      CHARACTER*8   CDUM(NDUM)

      IF (FIRST) THEN
      CALL SETMET
      FIRST = .FALSE.
      ENDIF

C *********************************************************************
C *****                 SET HEADER INFORMATION                     ****
C *********************************************************************

      NDATE  = NYMD
      NTIME  = NHMS
      NDATE0 = NYMD0
      NTIME0 = NHMS0

      NSFLD = NSFLDR
      NUFLD = NUFLDR
      NDFLD = 0

      NAMES(1) = CDIAG(83)
      DESCS(1) = TDIAG(83)

      NAMEU(1) = CDIAG(54)
      NAMEU(2) = CDIAG(55)
      NAMEU(3) = CDIAG(56)
      NAMEU(4) = CDIAG(57)

      DESCU(1) = TDIAG(54)
      DESCU(2) = TDIAG(55)
      DESCU(3) = TDIAG(56)
      DESCU(4) = TDIAG(57)

      DO L=1,NLAYR
      ZLEVID(L) = nlayr-L+1
      enddo

C *********************************************************************
C *****            SET USER-DEFINED HEADER VARIABLES               ****
C *********************************************************************

      UNDEF   = getcon('UNDEF')

      ddum(1) = 'OUTPUT RECORD TYPE (INTEGER)'
      idum(1) =  nrtype
      rdum(1) =  undef
      cdum(1) = 'LOG8R'

      ddum(2) = 'UNDEFINED FILL VALUE (REAL)'
      idum(2) =  0
      rdum(2) =  undef
      cdum(2) = 'UNDEF'

C *********************************************************************
C *****                     HEADER RECORD                          ****
C *********************************************************************

      WRITE (IUNIT) JOB, NDATE, NTIME, NDATE0, NTIME0,
     .              XLABEL, IM, JNP,
     .              NSFLD, NUFLD, NDFLD,
     .              PTOP, NLAYR, (ZLEVID(K),K=1,NLAYR),
     .              NLAY, (SIGE(K),K=1,NLAYP1),
     .              NDUM, (DDUM(N),IDUM(N),RDUM(N),CDUM(N),N=1,NDUM)


      WRITE (IUNIT) (NAMES(N),DESCS(N),         N=1,NSFLD),
     .              (NAMEU(N),DESCU(N),         N=1,NUFLD),
     .              (NAMED(N),DESCD(N),NDLEV(N),N=1,NDFLD)

      write(6,1000) iunit,nymd,nhms

 1000 format(1x,'Writing IAU Increments to Unit ',i2,' with a Time',
     .         ' Stamp of ',i6,2x,i6)

C *********************************************************************
C *****                     WRITE IAU INCREMENTS                   ****
C *********************************************************************

      ZLEV = 0.
      CALL  CRAWRT ( DPANAL(1,1)  , IMJMR, IUNIT, TEM, ZLEV, NAMES(1) )

      DO     L = NLAYR,1,-1
      ZLEV = L
      CALL  CRAWRT ( DUANAL(1,1,L), IMJMR, IUNIT, TEM, ZLEV, NAMEU(1) )
      CALL  CRAWRT ( DVANAL(1,1,L), IMJMR, IUNIT, TEM, ZLEV, NAMEU(2) )
      CALL  CRAWRT ( DTANAL(1,1,L), IMJMR, IUNIT, TEM, ZLEV, NAMEU(3) )
      CALL  CRAWRT ( DQANAL(1,1,L), IMJMR, IUNIT, TEM, ZLEV, NAMEU(4) )
      ENDDO

      RETURN
      END
      SUBROUTINE  CRAWRT  (A, IMJM, KTP, TEM, ZLEV, NAME)
      DIMENSION   A   (IMJM)
      DIMENSION   TEM (IMJM)
      CHARACTER*8 NAME

      DO  I=1,IMJM
      TEM (I) = A(I)
      ENDDO

      WRITE(KTP)  ZLEV, NAME, TEM

      RETURN
      END
