      program oneoi

#include <vrslv.com>
#include <oirslv.com>
#include <vmetric.com>
#include <vstate.com>
#include <iau.com>

#ifdef PARALLEL
#include <parenv.h>
#include <mpif.h>

#endif      	

      parameter    ( zero = 0.0 )
      integer      im, jnp, nymdb, nhmsb, nymde, nhmse, ndbud,
     &             ndout, ndt, ndrsw, ndrsh, errorcode
      real         alpha, dummy, dummy2
      real         plevs(50)
      character*60 message
      character*80 xlabel
      character* 8 exp_id, atime, adate
      character* 4 jobno(2), jobnx(2), scheme
 
      dimension ndfreq (ndiagt)
      dimension nfreqz (ndiagt)
      dimension ndindx (ndiagt)


c Local variables used by the model and assimilation driver
c =========================================================

      logical      qbudget, qrestart_hist,
     &             qrestart_write, qbegin, qend,
     &             leap, mats, repack, qqinit,
     &             qprog, qbud, 
     &             qendseg


c Unit numbers for I/O
c ====================

      integer budg_unit
      integer prog_unit
      integer diag_unit (10)
      data    budg_unit /14/
      data    prog_unit /35/
      data    diag_unit /41,42,43,44,45,46,47,48,49,50/

      integer tmpunit
      data    tmpunit / 69 /


      data kbc   /16/
      data krsh  /36/
      data krsw  /37/
      data kdgn  /38/
      data kiau  /39/
      data koip  /95/


c HHMMSS event timestep counter
c =============================
      alarm( mhms ) = nalarm ( mhms, nymd, nhms, nymd0, nhms0 )

c  Frequency to Seconds Converter
c  ==============================
      nsecf(n) = n/10000*3600 + mod(n,10000)/100*60 + mod(n,100)


#ifdef PARALLEL
	call PAR_INIT
	decomposition = 1
#endif
      call timerinit(rank)

C **********************************************************************
C ****                     INITIALIZATION                           ****
C **********************************************************************

      
      dummy  = 0.
      dummy2 = 0.
!      call wnlline(1)
      call phnxdate( adate, atime )
      write(6,1000)  adate, atime

C **********************************************************************
C ****        READ AND INITIALIZE MODEL SETUP AND DIAGNOSTICS       ****
C **********************************************************************
      call mpi_barrier (MPI_COMM_WORLD, mpi_err)

      call timerstart(801)
      CALL SETUP ( JOBNO   , XLABEL  , NYMDB   , NHMSB   ,
     .             NYMDE   , NHMSE   , NDBUD   , NDOUT   , NDSEG  ,
     .             NDT     , NDRSW   , NDRSH   , LEAP    , NDPNT  ,
     .             MATS    , ALPHA   , QQINIT  , PLEVS   , NULEV   )
      call timerdump(801,"setup\0")
      call mpi_barrier (MPI_COMM_WORLD, mpi_err)

      if (leap) scheme = 'LEAP'
      if (mats) scheme = 'MATS'

      call timerstart(802)
      call initial  ( imr, jmr+1, nlayr, nlevoi, pmand, repack )
      call timerdump(802,"initial\0")
      call mpi_barrier (MPI_COMM_WORLD, mpi_err)

      call timerstart(803)
      call cortable ( pmand,nlevoi )
      call trigset
      call timerdump(803,"cortab and trigset\0")
      call mpi_barrier (MPI_COMM_WORLD, mpi_err)

C **********************************************************************
C ****                GET INITIAL/RESTART CONDITIONS                ****
C **********************************************************************

      irs = 1
      call timerstart(804)
      call restart ( krsw,irs,nymd,nhms,nb,nymd0,nhms0,jobnx,nrtype )
      call timerdump(804,"restart\0")
      call mpi_barrier (MPI_COMM_WORLD, mpi_err)
      write(6,2000)  krsw, nymd, nhms, nymd0, nhms0, nrtype 

C **********************************************************************
C ****                GET BOUNDARY CONDITIONS                       ****
C **********************************************************************

      call prtbcs( kbc,nymd,nhms )

C **********************************************************************
C ****                 Perform OI Analysis                          ****
C **********************************************************************

c Convert to A-Grid and Load A-Grid First-Guess into IAU Common
c -------------------------------------------------------------


c      call ctoaset ( nb )
c      call cload ( nb,'FG' )


      call timerstart(1)
      write (6,*) "GVL: read fields"
      call read2d(tmpunit,"phisz   ",phisz       ,imr,jmr+1     )         
      call read2d(tmpunit,"pz      ",pz(1,1,  nb),imr,jmr+1     )         
      call read3d(tmpunit,"uz      ",uz(1,1,1,nb),imr,jmr+1,nlayr)      
      call read3d(tmpunit,"vz      ",vz(1,1,1,nb),imr,jmr+1,nlayr)      
      call read3d(tmpunit,"tz      ",tz(1,1,1,nb),imr,jmr+1,nlayr)      
      call read3d(tmpunit,"qz      ",qz(1,1,1,nb),imr,jmr+1,nlayr)      
      call read3d(tmpunit,"pkht    ",pkht(1,1,1) ,imr,jmr+1,nlayr)    

      
      close ( tmpunit )
      call timerdump(1,"readfield\0")
      write (6,*) "GVL: field unit closed: ", tmpunit
      


c Perform OI Analysis to obtain A-Grid After Analysis
c ---------------------------------------------------I../com-
c      qbudget = ndbud.ne.0
c      call zeitbeg('GLASSIM')

      
      call glassim (repack, leap, mats, nymdb, nymde, nhmsb,
     &              nhmse, ndt, jobno, nb, nd, nymd, nhms, 
     &              qbudget )



C      call zeitend

c Compute A-Grid IAU Increments and Convert to C-Grid
c ---------------------------------------------------
c      call cload ( nb,'AI' )
c      call atoc  ( duanal, duanal, imr, jmr+1, nlayr, 1 )
c      call atoc  ( dvanal, dvanal, imr, jmr+1, nlayr, 2 )

c Archive IAU Increments
c ----------------------
      nrtype = 1
c      call iauout (kiau,nymd,nhms,nb,nymd0,nhms0,xlabel,jobno,nrtype)

      call zeitend
      call zeitpri(6)
      call flash ( 6, message, 2 ) 
#ifdef PARALLEL
      call PAR_EXIT
#endif
      stop 'Egress.'

 1000 FORMAT (1x,//,
     .        1x,'Goddard Earth Observing System (GEOS)',/,
     .        1X,'Potential Enstrophy and Energy Conserving GCM',/,
     .        1X,'Optimum Interpolation Analysis System',//,
     .        1X,'Starting Computation at Local Time: ',A8,' on ',A8,/)

 2000 FORMAT (1X,'Read    Restart From   Unit:  ',I2,/,
     .        1X,'Current Restart Record Date:  ',I6,1X,I6,/,
     .        1X,'  Experiment Beginning Date:  ',I6,1X,I6,/,
     .        1X,'               Restart Type:  ',I6,/,
     .        1X,' Read Diagnostics from Unit:  ',I2,/)

 3000 FORMAT (1X,'Write   Restart To     Unit:  ',I2,/,
     .        1X,'Current Restart Record Date:  ',I6,1X,I6,/,
     .        1X,'  Experiment Beginning Date:  ',I6,1X,I6,/,
     .        1X,'               Restart Type:  ',I6,/,
     .        1X,'  Write Diagnostics to Unit:  ',I2,/)



      end


!----------------------------------------------------------------------

