      program oneoi

#include <vrslv.com>
#include <oirslv.com>
#include <vmetric.com>
#include <vstate.com>
#include <iau.com>


      parameter    ( zero = 0.0 )
      integer      im, jnp, nymdb, nhmsb, nymde, nhmse, ndbud,
     &             ndout, ndt, ndrsw, ndrsh, errorcode
      real         alpha, dummy, dummy2
      real         plevs(50)
      character*60 message
      character*80 xlabel
      character* 8 exp_id, atime, adate
      character* 4 jobno(2), jobnx(2), scheme
 
      dimension ndfreq (ndiagt)
      dimension nfreqz (ndiagt)
      dimension ndindx (ndiagt)


c Local variables used by the model and assimilation driver
c =========================================================

      logical      qbudget, qrestart_hist,
     &             qrestart_write, qbegin, qend,
     &             leap, mats, repack, qqinit,
     &             qprog, qbud, 
     &             qendseg


c Unit numbers for I/O
c ====================

      integer budg_unit
      integer prog_unit
      integer diag_unit (10)
      data    budg_unit /14/
      data    prog_unit /35/
      data    diag_unit /41,42,43,44,45,46,47,48,49,50/

      integer tmpunit
      data    tmpunit / 69 /


      data kbc   /16/
      data krsh  /36/
      data krsw  /37/
      data kdgn  /38/
      data kiau  /39/
      data koip  /95/


c HHMMSS event timestep counter
c =============================
      alarm( mhms ) = nalarm ( mhms, nymd, nhms, nymd0, nhms0 )

c  Frequency to Seconds Converter
c  ==============================
      nsecf(n) = n/10000*3600 + mod(n,10000)/100*60 + mod(n,100)


C **********************************************************************
C ****                     INITIALIZATION                           ****
C **********************************************************************

      dummy  = 0.
      dummy2 = 0.
      call wnlline(1)
      call phnxdate( adate, atime )
      write(6,1000)  adate, atime

C **********************************************************************
C ****        READ AND INITIALIZE MODEL SETUP AND DIAGNOSTICS       ****
C **********************************************************************


      CALL SETUP ( JOBNO   , XLABEL  , NYMDB   , NHMSB   ,
     .             NYMDE   , NHMSE   , NDBUD   , NDOUT   , NDSEG  ,
     .             NDT     , NDRSW   , NDRSH   , LEAP    , NDPNT  ,
     .             MATS    , ALPHA   , QQINIT  , PLEVS   , NULEV   )

      if (leap) scheme = 'LEAP'
      if (mats) scheme = 'MATS'

      call initial  ( imr, jmr+1, nlayr, nlevoi, pmand, repack )
      call cortable ( pmand,nlevoi )
      call trigset

C **********************************************************************
C ****                GET INITIAL/RESTART CONDITIONS                ****
C **********************************************************************

      irs = 1
      call restart ( krsw,irs,nymd,nhms,nb,nymd0,nhms0,jobnx,nrtype )
      write(6,2000)  krsw, nymd, nhms, nymd0, nhms0, nrtype 

C **********************************************************************
C ****                GET BOUNDARY CONDITIONS                       ****
C **********************************************************************

      call prtbcs( kbc,nymd,nhms )

C **********************************************************************
C ****                 Perform OI Analysis                          ****
C **********************************************************************

c Convert to A-Grid and Load A-Grid First-Guess into IAU Common
c -------------------------------------------------------------
      call ctoaset ( nb )
      call cload ( nb,'FG' )

      write (6,*) "GVL: write fields"
      call wr2d(tmpunit,"phisz   ",phisz       ,imr,jmr+1     )         
      call wr2d(tmpunit,"pz      ",pz(1,1,  nb),imr,jmr+1     )         
      call wr3d(tmpunit,"uz      ",uz(1,1,1,nb),imr,jmr+1,nlayr)      
      call wr3d(tmpunit,"vz      ",vz(1,1,1,nb),imr,jmr+1,nlayr)      
      call wr3d(tmpunit,"tz      ",tz(1,1,1,nb),imr,jmr+1,nlayr)      
      call wr3d(tmpunit,"qz      ",qz(1,1,1,nb),imr,jmr+1,nlayr)      
      call wr3d(tmpunit,"pkht    ",pkht(1,1,1) ,imr,jmr+1,nlayr)    


      close ( tmpunit )
      write (6,*) "GVL: field unit closed: ", tmpunit


c Perform OI Analysis to obtain A-Grid After Analysis
c ---------------------------------------------------
C      qbudget = ndbud.ne.0
C      call zeitbeg('GLASSIM')

      
C      call glassim (repack, leap, mats, nymdb, nymde, nhmsb,
C     &              nhmse, ndt, jobno, nb, nd, nymd, nhms, 
C     &              qbudget )



C      call zeitend

c Compute A-Grid IAU Increments and Convert to C-Grid
c ---------------------------------------------------
C      call cload ( nb,'AI' )
C      call atoc  ( duanal, duanal, imr, jmr+1, nlayr, 1 )
C      call atoc  ( dvanal, dvanal, imr, jmr+1, nlayr, 2 )

c Archive IAU Increments
c ----------------------
C      nrtype = 1
C      call iauout (kiau,nymd,nhms,nb,nymd0,nhms0,xlabel,jobno,nrtype)

C      call zeitend
C      call zeitpri(6)
C      call flash ( 0, message, 2 ) 
      stop 'Egress.'

 1000 FORMAT (1x,//,
     .        1x,'Goddard Earth Observing System (GEOS)',/,
     .        1X,'Potential Enstrophy and Energy Conserving GCM',/,
     .        1X,'Optimum Interpolation Analysis System',//,
     .        1X,'Starting Computation at Local Time: ',A8,' on ',A8,/)

 2000 FORMAT (1X,'Read    Restart From   Unit:  ',I2,/,
     .        1X,'Current Restart Record Date:  ',I6,1X,I6,/,
     .        1X,'  Experiment Beginning Date:  ',I6,1X,I6,/,
     .        1X,'               Restart Type:  ',I6,/,
     .        1X,' Read Diagnostics from Unit:  ',I2,/)

 3000 FORMAT (1X,'Write   Restart To     Unit:  ',I2,/,
     .        1X,'Current Restart Record Date:  ',I6,1X,I6,/,
     .        1X,'  Experiment Beginning Date:  ',I6,1X,I6,/,
     .        1X,'               Restart Type:  ',I6,/,
     .        1X,'  Write Diagnostics to Unit:  ',I2,/)


      end


!----------------------------------------------------------------------

      SUBROUTINE SETUP ( JOBX    , XLABX   , NYMDBX  , NHMSBX  ,
     .                   NYMDEX  , NHMSEX  , NDBUDX  , NDOUTX  , NDSEGX,
     .                   NDTX    , NDRSWX  , NDRSHX  , LEAPX   , NDPNTX,
     .                   MATSX   , ALPHAX  , QQINITX , PLEVSX  ,
     .                   NULEVX  )
C***********************************************************************
C
C  PURPOSE
C     SET CONTROL FOR MODEL EXECUTION AND OUTPUT
C  I/O DDNAME  DESCRIPTION
C     11       REWINDABLE COPY OF CARD-IMAGE DATA FROM INPUT
C
C***********************************************************************
C******               DATA ASSIMILATION OFFICE                    ******
C***********************************************************************

      DIMENSION    PLEVSX(50), PLEVS(50)
      CHARACTER*40 XLAB(2), XLABX(2)
      CHARACTER*8  JOB, JOBX
      LOGICAL      LEAP, MATS, QQINIT, LEAPX, MATSX, QQINITX
      PARAMETER  ( ZERO = 0.0 )

C  SETUP INPUT NAMELIST
C  ====================
      NAMELIST /INPUT/  JOB     , XLAB    , NYMDB   , NHMSB   ,
     .                  NYMDE   , NHMSE   , NDBUD   , NDOUT   ,
     .                  NDT     , NDRSW   , NDRSH   , LEAP    ,
     .                  MATS    , ALPHA   , QQINIT  , PLEVS   ,

     .                  NDSEG   , NDPNT

      NAMELIST /INPTZ/  JOB     , XLAB    , NYMDB   , NHMSB   ,
     .                  NYMDE   , NHMSE   , NDBUD   , NDOUT   ,
     .                  NDT     , NDRSW   , NDRSH   , LEAP    ,
     .                  MATS    , ALPHA   , QQINIT  , NDSEG   ,
     .                  NDPNT
       
C  VARIABLE     TYPE                    DESCRIPTION
C  ========  ===========   ==========================================
C  JOB        CHARACTER      EXPERIMENT IDENTIFIER
C  XLAB       CHARACTER      EXPERIMENT DESCRIPTION LABEL
C  NYMDB        INTEGER      BEGINNING YYMMDD
C  NHMSB        INTEGER      BEGINNING HHMMSS
C  NYMDE        INTEGER      ENDING    YYMMDD
C  NHMSE        INTEGER      ENDING    YYMMDD
C  NDSEG        INTEGER      HHMMSS FREQUENCY OF PRODUCTION  SEGMENT
C  NDBUD        INTEGER      HHMMSS FREQUENCY OF BUDGET      OUTPUT
C  NDOUT        INTEGER      HHMMSS FREQUENCY OF GENERIC GLA OUTPUT
C  NDRSW        INTEGER      HHMMSS FREQUENCY OF RESTART     OUTPUT
C  NDRSH        INTEGER      HHMMSS FREQUENCY OF RESTART     HISTORY
C  NDPNT        INTEGER      HHMMSS FREQUENCY OF POINT BY POINT DIAG
C  NDT          INTEGER      TIME-STEP (SECONDS)
C  MATS         LOGICAL      LOGICAL FLAG FOR MATSUNO  TIME-SCHEME
C  LEAP         LOGICAL      LOGICAL FLAG FOR LEAPFROG TIME-SCHEME
C  ALPHA           REAL      TIME-FILTER COEFFICIENT FOR LEAPFROG
C  QQINIT       LOGICAL      LOGICAL FLAG TO INITIALIZE QQ FIELD
C  PLEVS           REAL      PRESSURE LEVELS USED FOR OUTPUT (50 MAX)
C     
C **********************************************************************
C ****              INITIALIZE SETUP NAMELIST VARIABLES             ****
C **********************************************************************
       
      DATA PLEVS /50*0.0/
       
      LEAP   = .FALSE.
      MATS   = .FALSE.
      QQINIT = .FALSE.
      ALPHA  = ZERO
      NDT    = 0
      NDBUD  = 0
      NDOUT  = 0
      NDSEG  = 0
      NDRSW  = 0
      NDRSH  = 0
      NDPNT  = 0
      NYMDB  = 0
      NHMSB  = 0
      NYMDE  = 0
      NHMSE  = 0
       
C **********************************************************************
C ****                     READ SETUP NAMELIST                      ****
C **********************************************************************
       
      REWIND 11
      WRITE (6 ,1000)
      READ  (11,INPUT)
      WRITE (6 ,INPTZ)
       
      NULEV  = 0
      DO 10 I=1,50
          PLEVSX(I) = PLEVS(I)
      IF( PLEVSX(I).NE.0.0 ) NULEV = NULEV + 1
   10 CONTINUE
      IF( NULEV.NE.0 ) THEN
      PRINT *, 'PRESSURE LEVELS FOR OUTPUT:'
      WRITE(6,100) (PLEVSX(I),I=1,NULEV)
  100 FORMAT(1X,10(F6.1,2X))
      ENDIF
       
C **********************************************************************
C ****                        ASSIGN VALUES                         ****
C **********************************************************************
       
      JOBX     = JOB
      XLABX(1) = XLAB(1)
      XLABX(2) = XLAB(2)
      LEAPX    = LEAP
      MATSX    = MATS
      QQINITX  = QQINIT
      ALPHAX   = ALPHA
      NDTX     = NDT
      NDSEGX   = NDSEG
      NDBUDX   = NDBUD
      NDOUTX   = NDOUT
      NDRSWX   = NDRSW
      NDRSHX   = NDRSH
      NDPNTX   = NDPNT
      NYMDBX   = NYMDB
      NHMSBX   = NHMSB
      NYMDEX   = NYMDE
      NHMSEX   = NHMSE
      NULEVX   = NULEV
       
 1000 FORMAT(1X,'READING SETUP NAMELIST:')
      RETURN
      END

!----------------------------------------------------------------------

      subroutine chkdiag (ndfreq,nfreq,nymd,nhms,nymd0,nhms0,
     .                    ndindx,ndsum,nymdt,nhmst,ndiagt)
      dimension ndfreq(ndiagt)
      dimension ndindx(ndiagt)
      nsecf(n) = n/10000*3600 + MOD(n,10000)/100*60 + MOD(n,100)
 
      nymdt = nymd
      nhmst = nhms
      nymdd = nymd0
      nhmsd = nhms0
      if(nfreq.le.0) then
      ntick    = nsecf( abs(nfreq) )/2
      call tick ( nymdd,nhmsd,-ntick,ndum )
      call tick ( nymdt,nhmst,-ntick,ndum )
      endif

c Initialize Diagnostic Index
c ---------------------------
      do n=1,ndiagt
      ndindx(n) = 0
      enddo

c Fill Current Diagnostic Index
c -----------------------------
      ndsum = 0
      do n=1,ndiagt
      nafreq = abs(ndfreq(n))
      if(         ndfreq(n).eq.nfreq .and.
     .    nalarm( nafreq,nymd,nhms,nymdd,nhmsd ).eq.0 ) then
      ndsum = ndsum + 1
      ndindx(ndsum) = n
      endif
      enddo
      return
      end
      SUBROUTINE CLOAD (NB,IAFLG)                                  
C                                                                       
C.... ROUTINE TO LOAD 6H FCST FIELDS OR ANALYSIS INCREMENTS             
C     INTO IAI COMMON BLOCK (CONTROLLED BY IAFLG, 'FG' OR 'AI')         
                                                                        
#include <vrslv.com>
#include <vstate.com>
#include <iau.com>

      PARAMETER( NTOT  = IMR*(JMR+1)*NLAYR )                            
      PARAMETER( IMJMR = IMR*(JMR+1) )                                  
                                                                        
      PARAMETER( ZERO  = 0.0 )                                          

      CHARACTER*2 IAFLG                                                 
C                                                                       
      IM  = IMR                                                         
      JNP = JMR + 1                                                     
                                                                        
      IF(IAFLG .EQ. 'FG') THEN                                          
         WRITE(6,403)                                                   
 403     FORMAT(' LOADING FG INTO INCREMENT COMMON')                    
         DO 20 L=1,NLAYR                                                
             DO 10 I=1,IMJMR                                            
                 DUANAL(I,1,L) = UZ(I,1,L,NB)                           
                 DVANAL(I,1,L) = VZ(I,1,L,NB)                           
                 DTANAL(I,1,L) = TZ(I,1,L,NB)*pz(i,1,nb)                
                 DQANAL(I,1,L) = QZ(I,1,L,NB)*pz(i,1,nb)                
 10          CONTINUE                                                   
 20      CONTINUE                                                       
                                                                        
         DO 30 I=1,IMJMR                                                
             DPANAL(I,1) = PZ(I,1,NB)                                   
 30      CONTINUE                                                       
                                                                        
      ELSE IF(IAFLG .EQ. 'AI') THEN                                     
         WRITE(6,404)                                                   
 404     FORMAT(' LOADING OI-FG INTO INCREMENT COMMON')                 
                                                                        
C********************************************************************   
C*****   HERE WE WANT THE TENDENCIES IN IAICOM                          
C********************************************************************   
C                                                                       
         DO 50 L=1,NLAYR                                                
           DO 40 I=1,IMJMR                                              
                                                                        
              DUANAL(I,1,L) = UZ(I,1,L,NB) - DUANAL(I,1,L)              
              DVANAL(I,1,L) = VZ(I,1,L,NB) - DVANAL(I,1,L)              
              DTANAL(I,1,L) = TZ(I,1,L,NB)*pz(i,1,nb) - DTANAL(I,1,L)
              DQANAL(I,1,L) = QZ(I,1,L,NB)*pz(i,1,nb) - DQANAL(I,1,L)
                                                                        
 40        CONTINUE                                                     
 50      CONTINUE                                                       
                                                                        
         DO 60 I=1,IMJMR                                                
           DPANAL(I,1) = ( PZ(I,1,NB) - DPANAL(I,1))/(6.*3600.)           
 60      CONTINUE                                                       

         do layers = 1,nlayr
          do i = 1,imjmr

            duanal(i,1,layers) = duanal(i,1,layers) / (6.*3600.)
            dvanal(i,1,layers) = dvanal(i,1,layers) / (6.*3600.)
            dtanal(i,1,layers) = dtanal(i,1,layers) / (6.*3600.)
            dqanal(i,1,layers) = dqanal(i,1,layers) / (6.*3600.)

          end do
         end do

      END IF                                                            
                                                                        
      RETURN                                                            
      END                                                               

!----------------------------------------------------------------------

      SUBROUTINE  CLRINDX ( INDXLIST )
C***********************************************************************      
C                                                                             
C  PURPOSE                                                                     
C     DRIVER TO CLEAR DIAGNOSTICS SPECIFIED IN DIAGNOSTIC INDEX LISTC
C  ARGUMENT DESCRIPTION                                                        
C     INDXLIST.. INTEGER DIAGNOSTIC INDEX LISTC                                                                            
C***********************************************************************       
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *       
C***********************************************************************       
C                                                                              
#include <vrslv.com>                                                           
#include <vstate.com>                                                          
      INTEGER INDXLIST (NDIAGT)
                                                                              
      DO  INDEX=1,NDIAGT                                                     
      N = INDXLIST (index)
      IF( N.NE.0 .AND. IDIAG(N).NE.0 ) THEN                                   
      CALL CLRDIAG (N)                                                      
      ENDIF                                                                   
      ENDDO
                                                                             
      RETURN                                                                 
      END                                                                    
