      program oneoi

#include <vrslv.com>
#include <oirslv.com>
#include <vmetric.com>
#include <vstate.com>
#include <iau.com>

      parameter    ( zero = 0.0 )
      integer      im, jnp, nymdb, nhmsb, nymde, nhmse, ndbud,
     &             ndout, ndt, ndrsw, ndrsh, errorcode
      real         alpha, dummy, dummy2
      real         plevs(50)
      character*60 message
      character*80 xlabel
      character* 8 exp_id, atime, adate
      character* 4 jobno(2), jobnx(2), scheme
 
      dimension ndfreq (ndiagt)
      dimension nfreqz (ndiagt)
      dimension ndindx (ndiagt)


c Local variables used by the model and assimilation driver
c =========================================================

      logical      qbudget, qrestart_hist,
     &             qrestart_write, qbegin, qend,
     &             leap, mats, repack, qqinit,
     &             qprog, qbud, 
     &             qendseg


c Unit numbers for I/O
c ====================

      integer budg_unit
      integer prog_unit
      integer diag_unit (10)
      data    budg_unit /14/
      data    prog_unit /35/
      data    diag_unit /41,42,43,44,45,46,47,48,49,50/

      data kbc   /16/
      data krsh  /36/
      data krsw  /37/
      data kdgn  /38/
      data kiau  /39/
      data koip  /95/


c HHMMSS event timestep counter
c =============================
      alarm( mhms ) = nalarm ( mhms, nymd, nhms, nymd0, nhms0 )

c  Frequency to Seconds Converter
c  ==============================
      nsecf(n) = n/10000*3600 + mod(n,10000)/100*60 + mod(n,100)


C **********************************************************************
C ****                     INITIALIZATION                           ****
C **********************************************************************

      dummy  = 0.
      dummy2 = 0.
      call wnlline(1)
      call phnxdate( adate, atime )
      write(6,1000)  adate, atime

C **********************************************************************
C ****        READ AND INITIALIZE MODEL SETUP AND DIAGNOSTICS       ****
C **********************************************************************


      CALL SETUP ( JOBNO   , XLABEL  , NYMDB   , NHMSB   ,
     .             NYMDE   , NHMSE   , NDBUD   , NDOUT   , NDSEG  ,
     .             NDT     , NDRSW   , NDRSH   , LEAP    , NDPNT  ,
     .             MATS    , ALPHA   , QQINIT  , PLEVS   , NULEV   )

      if (leap) scheme = 'LEAP'
      if (mats) scheme = 'MATS'

      call initial  ( imr, jmr+1, nlayr, nlevoi, pmand, repack )
      call cortable ( pmand,nlevoi )
      call trigset

C **********************************************************************
C ****                GET INITIAL/RESTART CONDITIONS                ****
C **********************************************************************

      irs = 1
      call restart ( krsw,irs,nymd,nhms,nb,nymd0,nhms0,jobnx,nrtype )
      write(6,2000)  krsw, nymd, nhms, nymd0, nhms0, nrtype 

C **********************************************************************
C ****                GET BOUNDARY CONDITIONS                       ****
C **********************************************************************

      call prtbcs( kbc,nymd,nhms )

C **********************************************************************
C ****                 Perform OI Analysis                          ****
C **********************************************************************

c Convert to A-Grid and Load A-Grid First-Guess into IAU Common
c -------------------------------------------------------------
      call ctoaset ( nb )
      call cload ( nb,'FG' )
          
c Perform OI Analysis to obtain A-Grid After Analysis
c ---------------------------------------------------
      qbudget = ndbud.ne.0
      call zeitbeg('GLASSIM')

      
      call glassim (repack, leap, mats, nymdb, nymde, nhmsb,
     &              nhmse, ndt, jobno, nb, nd, nymd, nhms, 
     &              qbudget )



      call zeitend

c Compute A-Grid IAU Increments and Convert to C-Grid
c ---------------------------------------------------
      call cload ( nb,'AI' )
      call atoc  ( duanal, duanal, imr, jmr+1, nlayr, 1 )
      call atoc  ( dvanal, dvanal, imr, jmr+1, nlayr, 2 )

c Archive IAU Increments
c ----------------------
      nrtype = 1
      call iauout (kiau,nymd,nhms,nb,nymd0,nhms0,xlabel,jobno,nrtype)

      call zeitend
      call zeitpri(6)
      call flash ( 0, message, 2 ) 
      stop 'Egress.'

 1000 FORMAT (1x,//,
     .        1x,'Goddard Earth Observing System (GEOS)',/,
     .        1X,'Potential Enstrophy and Energy Conserving GCM',/,
     .        1X,'Optimum Interpolation Analysis System',//,
     .        1X,'Starting Computation at Local Time: ',A8,' on ',A8,/)

 2000 FORMAT (1X,'Read    Restart From   Unit:  ',I2,/,
     .        1X,'Current Restart Record Date:  ',I6,1X,I6,/,
     .        1X,'  Experiment Beginning Date:  ',I6,1X,I6,/,
     .        1X,'               Restart Type:  ',I6,/,
     .        1X,' Read Diagnostics from Unit:  ',I2,/)

 3000 FORMAT (1X,'Write   Restart To     Unit:  ',I2,/,
     .        1X,'Current Restart Record Date:  ',I6,1X,I6,/,
     .        1X,'  Experiment Beginning Date:  ',I6,1X,I6,/,
     .        1X,'               Restart Type:  ',I6,/,
     .        1X,'  Write Diagnostics to Unit:  ',I2,/)


      end
