      SUBROUTINE ASTRO ( NYMD,NHMS, ALAT,ALON, IM,JM, COSZ,RA )
C***********************************************************************
C
C  INPUT:
C  ======
C    NYMD     : CURRENT YYMMDD
C    NHMS     : CURRENT HHMMSS
C    ALAT(JM) : LATITUDES  IN DEGREES.
C    ALON(IM) : LONGITUDES IN DEGREES. (0 = GREENWICH, + = EAST).
C    IM, JM   : # OF LONS AND LATS.
C
C  OUTPUT:
C  =======
C    COSZ(IM,JM) : COSINE OF ZENITH ANGLE.
C    RA          : EARTH-SUN DISTANCE IN UNITS OF
C                  THE ORBIT'S SEMI-MAJOR AXIS.
C
C  NOTE:
C  =====
C  THE INSOLATION AT THE TOP OF THE ATMOSPHERE IS:
C
C  S(I,J) = (SOLAR CONSTANT)*(1/RA**2)*COSZ(I,J),
C
C  WHERE:
C  RA AND COSZ(I,J) ARE THE TWO OUTPUTS OF THIS SUBROUTINE.
C
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
C
#include <vrslv.com>
#ifdef CRAY
      PARAMETER ( IMJMR = IMR*(JMR+1) )
#endif
#if defined(RS6000) || defined (DEC_ALPHA)
      integer imjmr
#endif
      DIMENSION COSZ(IM,JM), ALAT(JM), ALON(IM)
      INTEGER YEAR, DAY, SEC

      REAL PI, ZERO, ONE, TWO, SIX, DG2RD, YRLEN, EQNX, OB, ECC, PER
      REAL DAYLEN, FAC, THM, THP, THNOW, ZS, ZC, SJ, CJ
      PARAMETER (PI = 3.1415926535898)
      PARAMETER (ZERO=0.,ONE=1.,TWO=2.,SIX=6.)
      PARAMETER (DG2RD=PI/180.)

      INTEGER    DAYSCY
      PARAMETER (YRLEN=365.25,DAYSCY=365*4+1)
      PARAMETER (EQNX=80.9028)
      PARAMETER (OB=23.45*DG2RD, ECC=.0167, PER=102.0*DG2RD)
      PARAMETER (DAYLEN=86400.)

      REAL      TH(DAYSCY),T0,T1,T2,T3,T4,FUN,Y,HC(600),MNDY(12,4)

      LOGICAL FIRST
      DATA    FIRST/.TRUE./
      SAVE

      DATA MNDY /0,31,60,91,121,152,182,213,244,274,305,335,366,
     .           397,34*0 /

      FUN(Y) = (TWO*PI/((ONE-ECC**2)**1.5))*(ONE/YRLEN)
     .       * (ONE - ECC*COS(Y-PER)) ** 2
      NSECF(N) = N/10000*3600 + MOD(N,10000)/100* 60 + MOD(N,100)
C


#if defined(RS6000) || defined(DEC_ALPHA)
      IMJMR = IMR*(JMR+1)
#endif

C***********************************************************************
C*                          SET CURRENT TIME                           *
C***********************************************************************
C
      YEAR  = NYMD / 10000
      MONTH = MOD(NYMD,10000) / 100
      DAY   = MOD(NYMD,100)
      SEC   = NSECF(NHMS)
C
C***********************************************************************
C*                 COMPUTE DAY-ANGLES FOR 4-YEAR CYCLE                 *
C***********************************************************************
C
      IF(FIRST) THEN
           DO 100 I=15,48
           MNDY(I,1) = MNDY(I-12,1) + 365
100        CONTINUE

           KM  = INT(EQNX) + 1
           FAC = KM-EQNX
           T0 = ZERO
           T1 = FUN(T0         )*FAC
           T2 = FUN(ZERO+T1/TWO)*FAC
           T3 = FUN(ZERO+T2/TWO)*FAC
           T4 = FUN(ZERO+T3    )*FAC
           TH(KM) = (T1 + TWO*(T2 + T3) + T4) / SIX

           DO 200 K=2,DAYSCY
           T1 = FUN(TH(KM)       )
           T2 = FUN(TH(KM)+T1/TWO)
           T3 = FUN(TH(KM)+T2/TWO)
           T4 = FUN(TH(KM)+T3    )
           KP = MOD(KM,DAYSCY) + 1
           TH(KP) = TH(KM) + (T1 + TWO*(T2 + T3) + T4) / SIX
           KM = KP
 200       CONTINUE

           FIRST=.FALSE.
      ENDIF
C
C***********************************************************************
C*            COMPUTE EARTH-SUN DISTANCE TO CURRENT SECOND             *
C***********************************************************************
C
      IDAY   = DAY + MNDY(MONTH,MOD(YEAR,4)+1)
      IDAYP1 = MOD( IDAY,DAYSCY) + 1
      THM    = MOD( TH(IDAY)  ,TWO*PI)
      THP    = MOD( TH(IDAYP1),TWO*PI)

      IF(THP.LT.THM) THP = THP + TWO*PI
      FAC   = FLOAT(SEC)/DAYLEN
      THNOW = THM*(ONE-FAC) + THP*FAC

      ZS = SIN(THNOW) * SIN(OB)
      ZC = SQRT(ONE-ZS*ZS)
      RA = (1.-ECC*ECC) / ( ONE-ECC*COS(THNOW-PER) )
C
C***********************************************************************
C*                 COMPUTE COSINE OF THE ZENITH ANGLE                  *
C***********************************************************************
C
           FAC = FAC*TWO*PI + PI
      DO 300 I=1,IMR
      HC(I) = COS( FAC+ALON(I)*DG2RD )
  300 CONTINUE

      DO 400 J=1,JM
                SJ = SIN(ALAT(J)*DG2RD)
                CJ = SQRT(ONE-SJ*SJ)
      DO 400 I=1,IMR
      COSZ(I,J) = SJ*ZS + CJ*ZC*HC(I)
400   CONTINUE

      DO 500 I=1,IMJMR
      IF ( COSZ(I,1).LT.ZERO ) COSZ(I,1) = ZERO
500   CONTINUE

      RETURN
      END
