      SUBROUTINE CTOA ( qc,qa,im,jm,km,itype )                          
C                                                                       
C ******************************************************************    
C ****                                                          ****    
C ****  This program converts 'C' gridded data                  ****    
C ****                     to 'A' gridded data.                 ****    
C ****                                                          ****    
C ****  A direct transfer           is made      for itype = 0  ****    
C ****  An FFT shift transformation is made in x for itype = 1  ****    
C ****  An FFT shift transformation is made in y for itype = 2  ****    
C ****                                                          ****    
C ******************************************************************    
                                                                        
#include <vrslv.com>                                                    
                                                                        
      DIMENSION qa   (im,jm,km)                                         
      DIMENSION qc   (im,jm,km)                                         
      DIMENSION qz   (im,jm,km)                                         
                                                                        
      DIMENSION qcx  (   im+2 ,km)                                          
      DIMENSION  cx  (2*(im+2),km)                                          
      DIMENSION qcy  (   2*jm ,km)                                          
      DIMENSION  cy  (2*(2*jm),km)                                          
                                                                        
      DIMENSION cosx (imr/2), sinx(imr/2)                               
      DIMENSION cosy (jmr)  , siny(jmr)                                 
      DIMENSION IFX  (100) , IFY (100)                                    
      REAL      TRIGX(3*(IMR+1))                                        
      REAL      TRIGY(3*(2*JMR+1))                                          
                                                                        
      save IFX  , IFY
      save TRIGX
      save TRIGY
      save cosx , sinx
      save cosy , siny

      logical first                                                     
      data    first /.true./                                            
                                                                        
      parameter ( zero  = 0.0 )                                         
      parameter ( ahalf = 0.5 )                                         
                                                                        
      imh   = IM/2                                                      
      JMM1  = JM-1                                                      
      JMM2  = JM-2                                                      
      JMM3  = JM-3                                                      
      JP    = 2*jmm1                                                    
                                                                        
      PI    = 4.*ATAN(1.)                                               
      DX    = 2*PI/IM                                                   
      DY    = PI/JMM1                                                   
                                                                        
C *********************************************************             
C ****     define sinx, cosx and trigs coefficients    ****             
C *********************************************************             
                                                                        
      if (first) then                                                   
                                                                        
      call fftfax (im,ifx,trigx)                                        
      do k=1,imh                                                        
      thx = k*dx*ahalf                                                  
      cosx(k) = cos(thx)                                                
      sinx(k) = sin(thx)                                                
      enddo                                                             
                                                                        
      call fftfax (jp,ify,trigy)                                        
      do l=1,jmm1                                                       
      thy = l*dy*ahalf                                                  
      cosy(l) = cos(thy)                                                
      siny(l) = sin(thy)                                                
      enddo                                                             
                                                                        
      first = .false.                                                   
      endif                                                             
                                                                        
C *********************************************************             
C ****       load input array into temporary array     ****             
C *********************************************************             
                                                                        
      do i = 1,im*jm*km                                                 
      qz(i,1,1) = qc(i,1,1)                                             
      enddo                                                             
                                                                        
C *********************************************************             
C ****                mass points                      ****             
C *********************************************************             
                                                                        
      if (itype.eq.0) then                     
      c1  =  4./3.
      c2  = -1./3.                         
      do I=1,im*jm*km                                                   
      qa(I,1,1) = qz(I,1,1)                                             
      enddo                                                             
      do L=1,km                                                         

      do i=1,im
      qa(i,1 ,L) = c1*qz(i,2   ,L) + c2*qz(i,3   ,L)
      qa(i,jm,L) = c1*qz(i,jm-1,L) + c2*qz(i,jm-2,L)
      enddo 

      qnp = 0.                                                          
      qsp = 0.                                                          
      do i=1,im                                                         
      qnp = qnp + qa(i,jm,L)                                          
      qsp = qsp + qa(i,1 ,L)                                          
      enddo                                                             
      qnp = qnp / im                                                    
      qsp = qsp / im                                                    
      do i=1,im                                                         
      qa(I,jm,L) = qnp                                                  
      qa(I,1 ,L) = qsp                                                  
      enddo                                                             
      enddo                                                             
      endif                                                             
                                                                        
C *********************************************************             
C ****           stagger in x-direction                ****             
C *********************************************************             
                                                                        
      if (itype.eq.1) then                                              
      do j=1,jm                                                         
                                                                        
      do l=1,km                                                         
      do i=1,im                                                         
      qcx(i,l) = qz(i,j,l)                                              
      enddo                                                             
      enddo                                                             
      call rfftmlt (qcx,cx,trigx,ifx,1 ,im+2,im,km,-1)                   
                                                                        
      do l=1,km                                                         
      do k=1,imh                                                        
      kr = 2*k+1                                                        
      ki = 2*k+2                                                        
      crprime = qcx(kr,l)*cosx(k) + qcx(ki,l)*sinx(k)                   
      ciprime = qcx(ki,l)*cosx(k) - qcx(kr,l)*sinx(k)                   
      qcx(kr,l) = crprime                                               
      qcx(ki,l) = ciprime                                               
      enddo                                                             
      enddo                                                             
                                                                        
      call rfftmlt (qcx,cx,trigx,ifx,1 ,im+2,im,km, 1)                   
      do l=1,km                                                         
      do i=1,im                                                         
      qa(i,j,l) = qcx(i,l)                                              
      enddo                                                             
      enddo                                                             
      enddo                                                             
                                                                        
      endif                                                             
                                                                        
C *********************************************************             
C ****           stagger in y-direction                ****             
C *********************************************************             
                                                                        
      if (itype.eq.2) then                                              
                                                                        
      do i=1,imh                                                        
                                                                        
      do l=1,km                                                         
      do j=1,jmm1                                                       
      qcy(j,l)      =  qz(i,j,l)                                        
      qcy(j+jmm1,l) = -qz(i+imh,jmm1-j+1,l)                            
      enddo                                                             
      enddo                                                             
                                                                        
      call rfftmlt (qcy,cy,trigy,ify,1 ,jp+2,jp,km,-1)                   
                                                                        
      do l=1,km                                                         
      do k=1,jmm1                                                       
      kr = 2*k+1                                                        
      ki = 2*k+2                                                        
      crprime = qcy(kr,l)*cosy(k) + qcy(ki,l)*siny(k)                   
      ciprime = qcy(ki,l)*cosy(k) - qcy(kr,l)*siny(k)                   
      qcy(kr,l) = crprime                                               
      qcy(ki,l) = ciprime                                               
      enddo                                                             
      enddo                                                             
                                                                        
      call rfftmlt (qcy,cy,trigy,ify,1 ,jp+2,jp,km, 1)                   
                                                                        
      do l=1,km                                                         
      do j=1,jmm1                                                       
      qa(i,j,l)          =  qcy(j,l)                                   
      qa(i+imh,jm-j+1,l) = -qcy(j+jmm1,l)                              
      enddo                                                             
      enddo                                                             
      enddo                                                             
                                                                        
      do l=1,km                                                         
      do i=1,imh                                                       
      qa(i+imh,1,l) = -qa(i,1,l)                                       
      qa(i,jm,l)    = -qa(i+imh,jm,l)                                 
      enddo                                                             
      enddo                                                             
      endif                                                             
                                                                        
      return                                                            
      END                                                               
