      SUBROUTINE DIAGDRV                                                
C***********************************************************************
C                                                                       
C  PURPOSE                                                              
C     SET POINTER LOCATIONS FOR DIAGNOSTICS TURNED ON IN NAMELIST       
C  I/O DDNAME  DESCRIPTION                                              
C     11       REWINDABLE COPY OF CARD-IMAGE DATA FROM UNIT 5 (TEMPNL)  
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
C                                                                       
#include <vrslv.com>                                                    
C                                                                       
      PARAMETER ( NDIAGT  = 500)                                        
                                                                        
      LOGICAL LDIAG (NDIAGT)                                            
      COMMON  /DIAG/ NDIAGMX, QDIAG (IMR,JMR+1,1)     
C                                                                       
C ======================================================================
C ====               D I A G N O S T I C    M E N U                 ====
C ======================================================================
C                                                                       
C   #    NAME      UNITS   LEVELS             DESCRIPTION               
C  ===  =======   =======  ======  =====================================
C   1    UFLUX    N/M**2      1    U-MOMENTUM SURFACE STRESS
C   2    VFLUX    N/M**2      1    V-MOMENTUM SURFACE STRESS
C   3    HFLUX    W/M**2      1    SURFACE FLUX OF SENSIBLE    HEAT     
C   4    EFLUX    W/M**2      1    SURFACE FLUX OF EVAPORATIVE HEAT     
C   5    QICE     W/M**2      1    HEAT CONDUCTION THROUGH SEA-ICE      
C   6    RADLWG   W/M**2      1    NET UPWARD   LW RADIATION AT GROUND  
C   7    RADSWG   W/M**2      1    NET DOWNWARD SW RADIATION AT GROUND  
C   8    RI                 NLAY   RICHARDSON NUMBER               
C   9    CT          M/SEC    1    SURFACE DRAG COEFFICIENT FOR T AND Q 
C  10    CU          M/SEC    1    SURFACE DRAG COEFFICIENT FOR U AND V 
C  11    ET       M**2/SEC  NLAY   DIFFUSIVITY  COEFFICIENT FOR T AND Q 
C  12    EU       M**2/SEC  NLAY   DIFFUSIVITY  COEFFICIENT FOR U AND V 
C  13    TURBU   M/SEC/DAY  NLAY   U-MOMENTUM  CHANGES DUE TO TURBULENCE
C  14    TURBV   M/SEC/DAY  NLAY   V-MOMENTUM  CHANGES DUE TO TURBULENCE
C  15    TURBT     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO TURBULENCE
C  16    TURBQ    G/DK/DAY  NLAY   MOISTURE    CHANGES DUE TO TURBULENCE
C  17    MOISTT    DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO MOIST PROC
C  18    MOISTQ   G/DK/DAY  NLAY   MOISTURE    CHANGES DUE TO MOIST PROC
C  19    RADLW     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO LW RAD.   
C  20    RADSW     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO SW RAD.   
C  21    PREACC     MM/DAY   1     TOTAL PRECIPITATION                  
C  22    PRECON     MM/DAY   1     CONVECTIVE PRECIPITATION             
C  23    TUFLUX     N/M**2  NLAY   TURBULENT FLUX OF U-MOMENTUM         
C  24    TVFLUX     N/M**2  NLAY   TURBULENT FLUX OF V-MOMENTUM         
C  25    TTFLUX     W/M**2  NLAY   TURBULENT FLUX OF TEMPERATURE        
C  26    TQFLUX     W/M**2  NLAY   TURBULENT FLUX OF MOISTURE           
C  27    CN          M/SEC   1     NEUTRAL DRAG COEFFICIENT             
C  28    WINDS       M/SEC   1     SURFACE WIND SPEED                   
C  29    DTSRF         DEG   1     AIR/SURFACE VIRTUAL TEMPERATURE DIFF.
C  30    TG            DEG   1     GROUND TEMPERATURE                   
C  31    TS            DEG   1     SURFACE AIR TEMPERATURE              
C  32    DTG           DEG   1     GROUND TEMPERATURE ADJUSTMENT        
C  33    QG           G/KG   1     GROUND SPECIFIC HUMIDITY             
C  34    QS           G/KG   1     SATURATION SURFACE SPECIFIC HUMIDITY 
C  35    TGRLW         DEG   1     GROUND TEMP SEEN BY LONGWAVE RAD     
C  36    ST4        W/M**2   1     SIGMA T**4                           
C  37    OLR        W/M**2   1     OUTGOING LONGWAVE RADIATION          
C  38    OLRCLR     W/M**2   1     OUTGOING LONGWAVE RADIATION CLEAR SKY
C  39    LWGCLR     W/M**2   1     SURFACE LONGWAVE FLUX CLEAR SKY      
C  40    LWCLR     DEG/DAY  NLAY   LW CLEAR SKY HEATING RATES           
C  41    TLW           DEG  NLAY   TEMPERATURE SEEN BY LONGWAVE RAD     
C  42    SHLW          G/G  NLAY   SPEC. HUM.  SEEN BY LONGWAVE RAD     
C  43    OZLW          G/G  NLAY   OZONE       SEEN BY LONGWAVE RAD     
C  44    CLMOLW             NLAY   MAXIMUM OVERLAP CLOUD FRACTION FOR LW
C  45    CLROLW             NLAY   RANDOM  OVERLAP CLOUD FRACTION FOR LW
C  46    CLMOSW             NLAY   MAXIMUM OVERLAP CLOUD FRACTION FOR SW
C  47    CLROSW             NLAY   RANDOM  OVERLAP CLOUD FRACTION FOR SW
C  48    RADSWT              1     INCIDENT SW RADIATION AT TOP OF ATMS.
C  49    FREQMD              1     FREQUENCY OF MID-LEVEL CONVECTION    
C  50    FREQPN             NLAY   FREQUENCY OF PEN-LEVEL CONVECTION    
C  51    FREQLW              1     FREQUENCY OF LOW-LEVEL CONVECTION    
C  52    EVAP       MM/DAY   1     SURFACE EVAPORATION                  
C  53    DPDT       MB/SEC   1     SURFACE PRESSURE TENDENCY            
C  54    ANALU     M/S/SEC  NLAY   ANALYSIS INCREMENT OF U              
C  55    ANALV     M/S/SEC  NLAY   ANALYSIS INCREMENT OF V              
C  56    ANALT MB*THET/SEC  NLAY   ANALYSIS INCREMENT OF PI*THETA/PKAPPA    
C  57    ANALQ MB*G/G /SEC  NLAY   ANALYSIS INCREMENT OF PI*Q           
C  58    OMEGA      NB/SEC  NLAY   VERTICAL VELOCITY OMEGA (DPDT)       
C  59    DUDT      M/S/DAY  NLAY   TOTAL  U TENDENCY    (C-GRID)           
C  60    DVDT      M/S/DAY  NLAY   TOTAL  V TENDENCY    (C-GRID)           
C  61    DTDT      DEG/DAY  NLAY   TOTAL  T TENDENCY    (C-GRID)           
C  62    DQDT     G/KG/DAY  NLAY   TOTAL  Q TENDENCY    (C-GRID)           
C  63    VORT      SEC**-1  NLAY   RELATIVE VORTICITY   (C-GRID)          
C  64    PGV       M/S/DAY  NLAY   V PRESSURE GRADIENT TENDENCY (A-GRID)
C  65    CORU      M/S/DAY  NLAY   U CORIOLIS TENDENCY  (A-GRID)         
C  66    CORV      M/S/DAY  NLAY   V CORIOLIS TENDENCY  (A-GRID)         
C  67    USTAR     M/S      1      USTAR                                
C  68    Z0        M        1      SURFACE ROUGHNESS                    
C  69    FRQTRB             NLAY-1 FREQUENCY OF TURBULENCE              
C  70    PBL       MB       1      PBL DEPTH                            
C  71    SWCLR     DEG/DAY  NLAY   CLEAR-SKY SW HEATING RATES           
C  72    OSR       W/M**2   1      OUTGOING SW RADIATION                
C  73    OSRCLR    W/M**2   1      CLEAR-SKY OSR                        
C  74    CLDMAS    KG*M/S   NLAY   CLOUD MASS FLUX                      
C  75    UAVE      M/SEC    NLAY   TIME-AVERAGED U                      
C  76    VAVE      M/SEC    NLAY   TIME-AVERAGED V                      
C  77    TAVE      DEG      NLAY   TIME-AVERAGED T                      
C  78    QAVE      G/G      NLAY   TIME-AVERAGED Q                      
C  79    MDIV      MB/DAY   NLAY   MASS DIVERGENCE                      
C  80    PAVE      MB       1      TIME-AVERAGED PS-PTOP                
C  81    QQAVE     (M/S)**2 NLAY   TIME-AVERAGE QQ                      
C  82    SWGCLR    W/M**2   1      SURFACE SHORTWAVE FLUX CLEAR SKY     
C  83    ANALP     MB/DAY   1      ANALYSIS INCREMENT OF P              
C  84    SDIAG1             1      USER-DEFINED SURFACE   DIAGNOSTIC-1    
C  85    SDIAG2             1      USER-DEFINED SURFACE   DIAGNOSTIC-2    
C  86    UDIAG1             NLAY   USER-DEFINED UPPER-AIR DIAGNOSTIC-1    
C  87    UDIAG2             NLAY   USER-DEFINED UPPER-AIR DIAGNOSTIC-2    
C  88    DIABU     M/S/DAY  NLAY   TOTAL DIABATIC FORCING ON U-WIND       
C  89    DIABV     M/S/DAY  NLAY   TOTAL DIABATIC FORCING ON V-WIND       
C  90    DIABT     DEG/DAY  NLAY   TOTAL DIABATIC FORCING ON TEMPERATURE  
C  91    DIABQ    G/KG/DAY  NLAY   TOTAL DIABATIC FORCING ON SPEC. HUM.   
C  92    PSIGDOT    MB/DAY  NLAY   VERTICAL SIGMA VELOCITY PI*D(SIGMA)/DT   
C  93    PGU       M/S/DAY  NLAY   U PRESSURE GRADIENT TENDENCY (A-GRID)
C  94    PUFLX     M/S/DAY  NLAY   3-D FLUX CONVERGENCE OF PU   (A-GRID)
C  95    PVFLX     M/S/DAY  NLAY   3-D FLUX CONVERGENCE OF PV   (A-GRID)
C  96    PTFLX     DEG/DAY  NLAY   3-D FLUX CONVERGENCE OF PT   (A-GRID)
C  97    PQFLX    G/KG/DAY  NLAY   3-D FLUX CONVERGENCE OF PQ   (A-GRID)
C  98    DPUDT     M/S/DAY  NLAY   TOTAL PU TENDENCY            (A-GRID)
C  99    DPVDT     M/S/DAY  NLAY   TOTAL PV TENDENCY            (A-GRID)
C 100    DPTDT     DEG/DAY  NLAY   TOTAL PT TENDENCY            (A-GRID)
C 101    DPQDT    G/KG/DAY  NLAY   TOTAL PQ TENDENCY            (A-GRID)
C 102    VINTUQ M/SEC G/KG  1      VERTICALLY INTEGRATED U*Q    (C-GRID)
C 103    VINTVQ M/SEC G/KG  1      VERTICALLY INTEGRATED V*Q    (C-GRID)
C 104    VINTUT M/SEC  DEG  1      VERTICALLY INTEGRATED U*T    (C-GRID)
C 105    VINTVT M/SEC  DEG  1      VERTICALLY INTEGRATED V*T    (C-GRID)
C 106    CLDFRC             1      2-DIMENSIONAL TOTAL CLOUD FRACTION   
C 107    QINT               1      PRECIPITABLE WATER   (GM/CM**2)
C 108    U2M                1      U AT  2 METERS (M/SEC)
C 109    V2M                1      V AT  2 METERS (M/SEC)
C 110    T2M                1      T AT  2 METERS (DEG)
C 111    Q2M                1      Q AT  2 METERS (G/KG)
C 112    U10M               1      U AT 10 METERS (M/SEC)
C 113    V10M               1      V AT 10 METERS (M/SEC)
C 114    T10M               1      T AT 10 METERS (DEG)
C 115    Q10M               1      Q AT 10 METERS (G/KG)
C 116    DTRAIN    KG*M/S   NLAY   CONVECTIVE CLOUD DETRAINMENT               
C 117    QFILL     G/KG/DAY NLAY   FILLING OF NEGATIVE SPECIFIC HUMIDITY            
C                                                                       
C **********************************************************************
C **********************************************************************

      LOGICAL   LUFLUX                                                  
      LOGICAL   LVFLUX                                                  
      LOGICAL   LHFLUX                                                  
      LOGICAL   LEFLUX                                                  
      LOGICAL   LQICE                                                   
      LOGICAL   LRADLWG                                                 
      LOGICAL   LRADSWG                                                 
      LOGICAL   LRI                                                     
      LOGICAL   LCT                                                     
      LOGICAL   LCU                                                     
      LOGICAL   LET                                                     
      LOGICAL   LEU                                                     
      LOGICAL   LTURBU                                                  
      LOGICAL   LTURBV                                                  
      LOGICAL   LTURBT                                                  
      LOGICAL   LTURBQ                                                  
      LOGICAL   LMOISTT                                                 
      LOGICAL   LMOISTQ                                                 
      LOGICAL   LRADLW                                                  
      LOGICAL   LRADSW                                                  
      LOGICAL   LPREACC                                                 
      LOGICAL   LPRECON                                                 
      LOGICAL   LTUFLUX                                                 
      LOGICAL   LTVFLUX                                                 
      LOGICAL   LTTFLUX                                                 
      LOGICAL   LTQFLUX                                                 
      LOGICAL   LCN                                                     
      LOGICAL   LWINDS                                                  
      LOGICAL   LDTSRF                                                  
      LOGICAL   LTG                                                     
      LOGICAL   LTS                                                     
      LOGICAL   LDTG                                                    
      LOGICAL   LQG                                                     
      LOGICAL   LQS                                                     
      LOGICAL   LTGRLW                                                  
      LOGICAL   LST4                                                    
      LOGICAL   LOLR                                                    
      LOGICAL   LOLRCLR                                                 
      LOGICAL   LLWGCLR                                                 
      LOGICAL   LLWCLR                                                  
      LOGICAL   LTLW                                                    
      LOGICAL   LSHLW                                                   
      LOGICAL   LOZLW                                                   
      LOGICAL   LCLMOLW                                                 
      LOGICAL   LCLROLW                                                 
      LOGICAL   LCLMOSW                                                 
      LOGICAL   LCLROSW                                                 
      LOGICAL   LRADSWT                                                 
      LOGICAL   LFREQMD                                                 
      LOGICAL   LFREQPN                                                 
      LOGICAL   LFREQLW                                                 
      LOGICAL   LEVAP                                                   
      LOGICAL   LDPDT                                                   
      LOGICAL   LANALU                                                  
      LOGICAL   LANALV                                                  
      LOGICAL   LANALT                                                  
      LOGICAL   LANALQ                                                  
      LOGICAL   LOMEGA                                                  
      LOGICAL   LDUDT                                                  
      LOGICAL   LDVDT                                                  
      LOGICAL   LDTDT                                                  
      LOGICAL   LDQDT                                                  
      LOGICAL   LVORT                                                   
      LOGICAL   LPGV                                                    
      LOGICAL   LCORU                                                   
      LOGICAL   LCORV                                                   
      LOGICAL   LUSTAR                                                  
      LOGICAL   LZ0                                                     
      LOGICAL   LFRQTRB                                                 
      LOGICAL   LPBL                                                    
      LOGICAL   LSWCLR                                                  
      LOGICAL   LOSR                                                    
      LOGICAL   LOSRCLR                                                 
      LOGICAL   LCLDMAS                                                 
      LOGICAL   LUAVE                                                   
      LOGICAL   LVAVE                                                   
      LOGICAL   LTAVE                                                   
      LOGICAL   LQAVE                                                   
      LOGICAL   LMDIV                                                   
      LOGICAL   LPAVE                                                   
      LOGICAL   LQQAVE                                                  
      LOGICAL   LSWGCLR                                                 
      LOGICAL   LANALP                                                  
      LOGICAL   LSDIAG1                                                  
      LOGICAL   LSDIAG2                                                  
      LOGICAL   LUDIAG1                                                  
      LOGICAL   LUDIAG2                                                  
      LOGICAL   LDIABU                                                   
      LOGICAL   LDIABV                                                   
      LOGICAL   LDIABT                                                   
      LOGICAL   LDIABQ                                                   
      LOGICAL   LPSIGDOT                                                 
      LOGICAL   LPGU                                                     
      LOGICAL   LPUFLX                                                   
      LOGICAL   LPVFLX                                                   
      LOGICAL   LPTFLX                                                   
      LOGICAL   LPQFLX                                                   
      LOGICAL   LDPUDT                                                   
      LOGICAL   LDPVDT                                                   
      LOGICAL   LDPTDT                                                   
      LOGICAL   LDPQDT                                                   
      LOGICAL   LVINTUQ                                                  
      LOGICAL   LVINTVQ                                                  
      LOGICAL   LVINTUT                                                  
      LOGICAL   LVINTVT                                                  
      LOGICAL   LCLDFRC                                                  
      LOGICAL   LQINT                                                    
      LOGICAL   LU2M                                                    
      LOGICAL   LV2M                                                    
      LOGICAL   LT2M                                                    
      LOGICAL   LQ2M                                                    
      LOGICAL   LU10M                                                    
      LOGICAL   LV10M                                                    
      LOGICAL   LT10M                                                    
      LOGICAL   LQ10M                                                    
      LOGICAL   LDTRAIN                                                  
      LOGICAL   LQFILL                                                   
C                                                                       
      EQUIVALENCE ( LDIAG( 1) ,  LUFLUX  )                              
      EQUIVALENCE ( LDIAG( 2) ,  LVFLUX  )                              
      EQUIVALENCE ( LDIAG( 3) ,  LHFLUX  )                              
      EQUIVALENCE ( LDIAG( 4) ,  LEFLUX  )                              
      EQUIVALENCE ( LDIAG( 5) ,  LQICE   )                              
      EQUIVALENCE ( LDIAG( 6) ,  LRADLWG )                              
      EQUIVALENCE ( LDIAG( 7) ,  LRADSWG )                              
      EQUIVALENCE ( LDIAG( 8) ,  LRI     )                              
      EQUIVALENCE ( LDIAG( 9) ,  LCT     )                              
      EQUIVALENCE ( LDIAG(10) ,  LCU     )                              
      EQUIVALENCE ( LDIAG(11) ,  LET     )                              
      EQUIVALENCE ( LDIAG(12) ,  LEU     )                              
      EQUIVALENCE ( LDIAG(13) ,  LTURBU  )                              
      EQUIVALENCE ( LDIAG(14) ,  LTURBV  )                              
      EQUIVALENCE ( LDIAG(15) ,  LTURBT  )                              
      EQUIVALENCE ( LDIAG(16) ,  LTURBQ  )                              
      EQUIVALENCE ( LDIAG(17) ,  LMOISTT )                              
      EQUIVALENCE ( LDIAG(18) ,  LMOISTQ )                              
      EQUIVALENCE ( LDIAG(19) ,  LRADLW  )                              
      EQUIVALENCE ( LDIAG(20) ,  LRADSW  )                              
      EQUIVALENCE ( LDIAG(21) ,  LPREACC )                              
      EQUIVALENCE ( LDIAG(22) ,  LPRECON )                              
      EQUIVALENCE ( LDIAG(23) ,  LTUFLUX )                              
      EQUIVALENCE ( LDIAG(24) ,  LTVFLUX )                              
      EQUIVALENCE ( LDIAG(25) ,  LTTFLUX )                              
      EQUIVALENCE ( LDIAG(26) ,  LTQFLUX )                              
      EQUIVALENCE ( LDIAG(27) ,  LCN     )                              
      EQUIVALENCE ( LDIAG(28) ,  LWINDS  )                              
      EQUIVALENCE ( LDIAG(29) ,  LDTSRF  )                              
      EQUIVALENCE ( LDIAG(30) ,  LTG     )                              
      EQUIVALENCE ( LDIAG(31) ,  LTS     )                              
      EQUIVALENCE ( LDIAG(32) ,  LDTG    )                              
      EQUIVALENCE ( LDIAG(33) ,  LQG     )                              
      EQUIVALENCE ( LDIAG(34) ,  LQS     )                              
      EQUIVALENCE ( LDIAG(35) ,  LTGRLW  )                              
      EQUIVALENCE ( LDIAG(36) ,  LST4    )                              
      EQUIVALENCE ( LDIAG(37) ,  LOLR    )                              
      EQUIVALENCE ( LDIAG(38) ,  LOLRCLR )                              
      EQUIVALENCE ( LDIAG(39) ,  LLWGCLR )                              
      EQUIVALENCE ( LDIAG(40) ,  LLWCLR  )                              
      EQUIVALENCE ( LDIAG(41) ,  LTLW    )                              
      EQUIVALENCE ( LDIAG(42) ,  LSHLW   )                              
      EQUIVALENCE ( LDIAG(43) ,  LOZLW   )                              
      EQUIVALENCE ( LDIAG(44) ,  LCLMOLW )                              
      EQUIVALENCE ( LDIAG(45) ,  LCLROLW )                              
      EQUIVALENCE ( LDIAG(46) ,  LCLMOSW )                              
      EQUIVALENCE ( LDIAG(47) ,  LCLROSW )                              
      EQUIVALENCE ( LDIAG(48) ,  LRADSWT )                              
      EQUIVALENCE ( LDIAG(49) ,  LFREQMD )                              
      EQUIVALENCE ( LDIAG(50) ,  LFREQPN )                              
      EQUIVALENCE ( LDIAG(51) ,  LFREQLW )                              
      EQUIVALENCE ( LDIAG(52) ,  LEVAP   )                              
      EQUIVALENCE ( LDIAG(53) ,  LDPDT   )                              
      EQUIVALENCE ( LDIAG(54) ,  LANALU  )                              
      EQUIVALENCE ( LDIAG(55) ,  LANALV  )                              
      EQUIVALENCE ( LDIAG(56) ,  LANALT  )                              
      EQUIVALENCE ( LDIAG(57) ,  LANALQ  )                              
      EQUIVALENCE ( LDIAG(58) ,  LOMEGA  )                              
      EQUIVALENCE ( LDIAG(59) ,  LDUDT   )                              
      EQUIVALENCE ( LDIAG(60) ,  LDVDT   )                              
      EQUIVALENCE ( LDIAG(61) ,  LDTDT   )                              
      EQUIVALENCE ( LDIAG(62) ,  LDQDT   )                              
      EQUIVALENCE ( LDIAG(63) ,  LVORT   )                              
      EQUIVALENCE ( LDIAG(64) ,  LPGV    )                              
      EQUIVALENCE ( LDIAG(65) ,  LCORU   )                              
      EQUIVALENCE ( LDIAG(66) ,  LCORV   )                              
      EQUIVALENCE ( LDIAG(67) ,  LUSTAR  )                              
      EQUIVALENCE ( LDIAG(68) ,  LZ0     )                              
      EQUIVALENCE ( LDIAG(69) ,  LFRQTRB )                              
      EQUIVALENCE ( LDIAG(70) ,  LPBL    )                              
      EQUIVALENCE ( LDIAG(71) ,  LSWCLR  )                              
      EQUIVALENCE ( LDIAG(72) ,  LOSR    )                              
      EQUIVALENCE ( LDIAG(73) ,  LOSRCLR )                              
      EQUIVALENCE ( LDIAG(74) ,  LCLDMAS )                              
      EQUIVALENCE ( LDIAG(75) ,  LUAVE   )                              
      EQUIVALENCE ( LDIAG(76) ,  LVAVE   )                              
      EQUIVALENCE ( LDIAG(77) ,  LTAVE   )                              
      EQUIVALENCE ( LDIAG(78) ,  LQAVE   )                              
      EQUIVALENCE ( LDIAG(79) ,  LMDIV   )                              
      EQUIVALENCE ( LDIAG(80) ,  LPAVE   )                              
      EQUIVALENCE ( LDIAG(81) ,  LQQAVE  )                              
      EQUIVALENCE ( LDIAG(82) ,  LSWGCLR )                              
      EQUIVALENCE ( LDIAG(83) ,  LANALP  )                              
      EQUIVALENCE ( LDIAG(84) ,  LSDIAG1 )                              
      EQUIVALENCE ( LDIAG(85) ,  LSDIAG2 )                              
      EQUIVALENCE ( LDIAG(86) ,  LUDIAG1 )                              
      EQUIVALENCE ( LDIAG(87) ,  LUDIAG2 )                              
      EQUIVALENCE ( LDIAG(88) ,  LDIABU  )                              
      EQUIVALENCE ( LDIAG(89) ,  LDIABV  )                              
      EQUIVALENCE ( LDIAG(90) ,  LDIABT  )                              
      EQUIVALENCE ( LDIAG(91) ,  LDIABQ  )                              
      EQUIVALENCE ( LDIAG(92) ,  LPSIGDOT)                              
      EQUIVALENCE ( LDIAG(93) ,  LPGU    )                              
      EQUIVALENCE ( LDIAG(94) ,  LPUFLX  )                              
      EQUIVALENCE ( LDIAG(95) ,  LPVFLX  )                              
      EQUIVALENCE ( LDIAG(96) ,  LPTFLX  )                              
      EQUIVALENCE ( LDIAG(97) ,  LPQFLX  )                              
      EQUIVALENCE ( LDIAG(98) ,  LDPUDT  )                              
      EQUIVALENCE ( LDIAG(99) ,  LDPVDT  )                              
      EQUIVALENCE ( LDIAG(100),  LDPTDT  )                              
      EQUIVALENCE ( LDIAG(101),  LDPQDT  )                              
      EQUIVALENCE ( LDIAG(102),  LVINTUQ )                              
      EQUIVALENCE ( LDIAG(103),  LVINTVQ )                              
      EQUIVALENCE ( LDIAG(104),  LVINTUT )                              
      EQUIVALENCE ( LDIAG(105),  LVINTVT )                              
      EQUIVALENCE ( LDIAG(106),  LCLDFRC )                              
      EQUIVALENCE ( LDIAG(107),  LQINT   )                              
      EQUIVALENCE ( LDIAG(108),  LU2M    )                              
      EQUIVALENCE ( LDIAG(109),  LV2M    )                              
      EQUIVALENCE ( LDIAG(110),  LT2M    )                              
      EQUIVALENCE ( LDIAG(111),  LQ2M    )                              
      EQUIVALENCE ( LDIAG(112),  LU10M   )                              
      EQUIVALENCE ( LDIAG(113),  LV10M   )                              
      EQUIVALENCE ( LDIAG(114),  LT10M   )                              
      EQUIVALENCE ( LDIAG(115),  LQ10M   )                              
      EQUIVALENCE ( LDIAG(116),  LDTRAIN )                              
      EQUIVALENCE ( LDIAG(117),  LQFILL  )                              
C                                                                       
C  DIAGNOSTICS INPUT NAMELIST                                           
C  ==========================                                           
C                                                                       
      NAMELIST /INDIAG/ LUFLUX  , LVFLUX  , LHFLUX  , LEFLUX  ,         
     .                  LQICE   , LRADLWG , LRADSWG , LRI     ,         
     .                  LCT     , LCU     , LET     , LEU     ,         
     .                  LTURBU  , LTURBV  , LTURBT  , LTURBQ  ,         
     .                  LMOISTT , LMOISTQ , LRADLW  , LRADSW  ,         
     .                  LPREACC , LPRECON , LTUFLUX , LTVFLUX ,         
     .                  LTTFLUX , LTQFLUX , LCN     , LWINDS  ,         
     .                  LDTSRF  , LTG     , LTS     , LDTG    ,         
     .                  LQG     , LQS     , LTGRLW  , LST4    ,         
     .                  LOLR    , LOLRCLR , LLWGCLR , LLWCLR  ,         
     .                  LTLW    , LSHLW   , LOZLW   , LCLMOLW ,         
     .                  LCLROLW , LCLMOSW , LCLROSW , LRADSWT ,         
     .                  LFREQMD , LFREQPN , LFREQLW , LEVAP   ,         
     .                  LDPDT   , LANALU  , LANALV  , LANALT  ,         
     .                  LANALQ  , LOMEGA  , LDUDT   , LDVDT   ,         
     .                  LDTDT   , LDQDT   , LVORT   , LPGV    ,         
     .                  LCORU   , LCORV   , LUSTAR  , LZ0     ,         
     .                  LFRQTRB , LPBL    , LSWCLR  , LOSR    ,         
     .                  LOSRCLR , LCLDMAS , LUAVE   , LVAVE   ,         
     .                  LTAVE   , LQAVE   , LMDIV   , LPAVE   ,         
     .                  LQQAVE  , LSWGCLR , LANALP  , LSDIAG1 ,
     .                  LSDIAG2 , LUDIAG1 , LUDIAG2 , LDIABU  ,
     .                  LDIABV  , LDIABT  , LDIABQ  , LPSIGDOT,
     .                  LPGU    , LPUFLX  , LPVFLX  , LPTFLX  ,
     .                  LPQFLX  , LDPUDT  , LDPVDT  , LDPTDT  ,
     .                  LDPQDT  , LVINTUQ , LVINTVQ , LVINTUT ,
     .                  LVINTVT , LCLDFRC , LQINT   , LU2M    ,
     .                  LV2M    , LT2M    , LQ2M    , LU10M   ,
     .                  LV10M   , LT10M   , LQ10M   , LDTRAIN ,
     .                  LQFILL
                                                                        
C **********************************************************************
C ****               INITIALIZE DIAGNOSTIC LOGICALS                 ****
C **********************************************************************
                                                                        
      PRINT *                                                           
      PRINT *, 'Total Number of Diagnostics Allowed (NDIAGMX)'
      PRINT *, 'Defined by Users Application:  ',NDIAGMX
      PRINT *                                                           
                                                                        
      DO 10 N = 1,NDIAGT                                                
      LDIAG(N) = .FALSE.                                                
   10 CONTINUE                                                          
                                                                        
C **********************************************************************
C ****                  READ DIAGNOSTIC NAMELIST                    ****
C **********************************************************************
                                                                        
      REWIND 11                                                         
      WRITE (6 ,1000)                                                   
      READ  (11,INDIAG)                                                 
      WRITE (6 ,INDIAG)                                                 
                                                                        
C **********************************************************************
C ****       CALCULATE POINTERS FOR DIAGNOSTICS SET TO .TRUE.       ****
C **********************************************************************
                                                                        
      DO 20 N = 1,NDIAGT                                                
      IF(     LDIAG(N) ) THEN                                           
      CALL SETDIAG (N)                                                  
      CALL CLRDIAG (N)                                                  
      ENDIF                                                             
   20 CONTINUE                                                          
                                                                        
 1000 FORMAT(1X,'READING DIAGNOSTIC NAMELIST:')                         
      RETURN                                                            
      END                                                               
