      SUBROUTINE DIAGDRVN (NFREQ)                                               
C***********************************************************************
C                                                                       
C  PURPOSE                                                              
C     Set Pointer Locations for Diagnostics turned on in NAMELIST       
C  OUTPUT
C     NDIAG(500)    HHMMSS Frequencies of Diagnostic Output
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
C                                                                       
#include <vrslv.com>                                                    
C                                                                       
      PARAMETER ( NDIAGT  = 500)                                        
                                                                        
      INTEGER NFREQ (NDIAGT)                                            
      INTEGER NDIAG (NDIAGT)                                            
      COMMON  /DIAG/ NDIAGMX, QDIAG (IMR,JMR+1,1)     
C                                                                       
C ======================================================================
C ====               D I A G N O S T I C    M E N U                 ====
C ======================================================================
C                                                                       
C   #    NAME      UNITS   LEVELS             DESCRIPTION               
C  ===  =======   =======  ======  =====================================
C   1    UFLUX    N/M**2      1    U-MOMENTUM SURFACE STRESS
C   2    VFLUX    N/M**2      1    V-MOMENTUM SURFACE STRESS
C   3    HFLUX    W/M**2      1    SURFACE FLUX OF SENSIBLE    HEAT     
C   4    EFLUX    W/M**2      1    SURFACE FLUX OF EVAPORATIVE HEAT     
C   5    QICE     W/M**2      1    HEAT CONDUCTION THROUGH SEA-ICE      
C   6    RADLWG   W/M**2      1    NET UPWARD   LW RADIATION AT GROUND  
C   7    RADSWG   W/M**2      1    NET DOWNWARD SW RADIATION AT GROUND  
C   8    RI                 NLAY   RICHARDSON NUMBER               
C   9    CT          M/SEC    1    SURFACE DRAG COEFFICIENT FOR T AND Q 
C  10    CU          M/SEC    1    SURFACE DRAG COEFFICIENT FOR U AND V 
C  11    ET       M**2/SEC  NLAY   DIFFUSIVITY  COEFFICIENT FOR T AND Q 
C  12    EU       M**2/SEC  NLAY   DIFFUSIVITY  COEFFICIENT FOR U AND V 
C  13    TURBU   M/SEC/DAY  NLAY   U-MOMENTUM  CHANGES DUE TO TURBULENCE
C  14    TURBV   M/SEC/DAY  NLAY   V-MOMENTUM  CHANGES DUE TO TURBULENCE
C  15    TURBT     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO TURBULENCE
C  16    TURBQ    G/DK/DAY  NLAY   MOISTURE    CHANGES DUE TO TURBULENCE
C  17    MOISTT    DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO MOIST PROC
C  18    MOISTQ   G/DK/DAY  NLAY   MOISTURE    CHANGES DUE TO MOIST PROC
C  19    RADLW     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO LW RAD.   
C  20    RADSW     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO SW RAD.   
C  21    PREACC     MM/DAY   1     TOTAL PRECIPITATION                  
C  22    PRECON     MM/DAY   1     CONVECTIVE PRECIPITATION             
C  23    TUFLUX     N/M**2  NLAY   TURBULENT FLUX OF U-MOMENTUM         
C  24    TVFLUX     N/M**2  NLAY   TURBULENT FLUX OF V-MOMENTUM         
C  25    TTFLUX     W/M**2  NLAY   TURBULENT FLUX OF TEMPERATURE        
C  26    TQFLUX     W/M**2  NLAY   TURBULENT FLUX OF MOISTURE           
C  27    CN          M/SEC   1     NEUTRAL DRAG COEFFICIENT             
C  28    WINDS       M/SEC   1     SURFACE WIND SPEED                   
C  29    DTSRF         DEG   1     AIR/SURFACE VIRTUAL TEMPERATURE DIFF.
C  30    TG            DEG   1     GROUND TEMPERATURE                   
C  31    TS            DEG   1     SURFACE AIR TEMPERATURE              
C  32    DTG           DEG   1     GROUND TEMPERATURE ADJUSTMENT        
C  33    QG           G/KG   1     GROUND SPECIFIC HUMIDITY             
C  34    QS           G/KG   1     SATURATION SURFACE SPECIFIC HUMIDITY 
C  35    TGRLW         DEG   1     GROUND TEMP SEEN BY LONGWAVE RAD     
C  36    ST4        W/M**2   1     SIGMA T**4                           
C  37    OLR        W/M**2   1     OUTGOING LONGWAVE RADIATION          
C  38    OLRCLR     W/M**2   1     OUTGOING LONGWAVE RADIATION CLEAR SKY
C  39    LWGCLR     W/M**2   1     SURFACE LONGWAVE FLUX CLEAR SKY      
C  40    LWCLR     DEG/DAY  NLAY   LW CLEAR SKY HEATING RATES           
C  41    TLW           DEG  NLAY   TEMPERATURE SEEN BY LONGWAVE RAD     
C  42    SHLW          G/G  NLAY   SPEC. HUM.  SEEN BY LONGWAVE RAD     
C  43    OZLW          G/G  NLAY   OZONE       SEEN BY LONGWAVE RAD     
C  44    CLMOLW             NLAY   MAXIMUM OVERLAP CLOUD FRACTION FOR LW
C  45    CLROLW             NLAY   RANDOM  OVERLAP CLOUD FRACTION FOR LW
C  46    CLMOSW             NLAY   MAXIMUM OVERLAP CLOUD FRACTION FOR SW
C  47    CLROSW             NLAY   RANDOM  OVERLAP CLOUD FRACTION FOR SW
C  48    RADSWT              1     INCIDENT SW RADIATION AT TOP OF ATMS.
C  49    FREQMD              1     FREQUENCY OF MID-LEVEL CONVECTION    
C  50    FREQPN             NLAY   FREQUENCY OF PEN-LEVEL CONVECTION    
C  51    FREQLW              1     FREQUENCY OF LOW-LEVEL CONVECTION    
C  52    EVAP       MM/DAY   1     SURFACE EVAPORATION                  
C  53    DPDT       MB/SEC   1     SURFACE PRESSURE TENDENCY            
C  54    ANALU     M/S/SEC  NLAY   ANALYSIS INCREMENT OF U              
C  55    ANALV     M/S/SEC  NLAY   ANALYSIS INCREMENT OF V              
C  56    ANALT MB*THET/SEC  NLAY   ANALYSIS INCREMENT OF PI*THETA/PKAPPA    
C  57    ANALQ MB*G/G /SEC  NLAY   ANALYSIS INCREMENT OF PI*Q           
C  58    OMEGA      NB/SEC  NLAY   VERTICAL VELOCITY OMEGA (DPDT)       
C  59    DUDT      M/S/DAY  NLAY   TOTAL  U TENDENCY    (C-GRID)           
C  60    DVDT      M/S/DAY  NLAY   TOTAL  V TENDENCY    (C-GRID)           
C  61    DTDT      DEG/DAY  NLAY   TOTAL  T TENDENCY    (C-GRID)           
C  62    DQDT     G/KG/DAY  NLAY   TOTAL  Q TENDENCY    (C-GRID)           
C  63    VORT      SEC**-1  NLAY   RELATIVE VORTICITY   (C-GRID)          
C  64    PGV       M/S/DAY  NLAY   V PRESSURE GRADIENT TENDENCY (A-GRID)
C  65    CORU      M/S/DAY  NLAY   U CORIOLIS TENDENCY  (A-GRID)         
C  66    CORV      M/S/DAY  NLAY   V CORIOLIS TENDENCY  (A-GRID)         
C  67    USTAR     M/S      1      USTAR                                
C  68    Z0        M        1      SURFACE ROUGHNESS                    
C  69    FRQTRB             NLAY-1 FREQUENCY OF TURBULENCE              
C  70    PBL       MB       1      PBL DEPTH                            
C  71    SWCLR     DEG/DAY  NLAY   CLEAR-SKY SW HEATING RATES           
C  72    OSR       W/M**2   1      OUTGOING SW RADIATION                
C  73    OSRCLR    W/M**2   1      CLEAR-SKY OSR                        
C  74    CLDMAS    KG*M/S   NLAY   CLOUD MASS FLUX                      
C  75    UAVE      M/SEC    NLAY   TIME-AVERAGED U                      
C  76    VAVE      M/SEC    NLAY   TIME-AVERAGED V                      
C  77    TAVE      DEG      NLAY   TIME-AVERAGED T                      
C  78    QAVE      G/G      NLAY   TIME-AVERAGED Q                      
C  79    MDIV      MB/DAY   NLAY   MASS DIVERGENCE                      
C  80    PAVE      MB       1      TIME-AVERAGED PS-PTOP                
C  81    QQAVE     (M/S)**2 NLAY   TIME-AVERAGE QQ                      
C  82    SWGCLR    W/M**2   1      SURFACE SHORTWAVE FLUX CLEAR SKY     
C  83    ANALP     MB/DAY   1      ANALYSIS INCREMENT OF P              
C  84    SDIAG1             1      USER-DEFINED SURFACE   DIAGNOSTIC-1    
C  85    SDIAG2             1      USER-DEFINED SURFACE   DIAGNOSTIC-2    
C  86    UDIAG1             NLAY   USER-DEFINED UPPER-AIR DIAGNOSTIC-1    
C  87    UDIAG2             NLAY   USER-DEFINED UPPER-AIR DIAGNOSTIC-2    
C  88    DIABU     M/S/DAY  NLAY   TOTAL DIABATIC FORCING ON U-WIND       
C  89    DIABV     M/S/DAY  NLAY   TOTAL DIABATIC FORCING ON V-WIND       
C  90    DIABT     DEG/DAY  NLAY   TOTAL DIABATIC FORCING ON TEMPERATURE  
C  91    DIABQ    G/KG/DAY  NLAY   TOTAL DIABATIC FORCING ON SPEC. HUM.   
C  92    PSIGDOT    MB/DAY  NLAY   VERTICAL SIGMA VELOCITY PI*D(SIGMA)/DT   
C  93    PGU       M/S/DAY  NLAY   U PRESSURE GRADIENT TENDENCY (A-GRID)
C  94    PUFLX     M/S/DAY  NLAY   3-D FLUX CONVERGENCE OF PU   (A-GRID)
C  95    PVFLX     M/S/DAY  NLAY   3-D FLUX CONVERGENCE OF PV   (A-GRID)
C  96    PTFLX     DEG/DAY  NLAY   3-D FLUX CONVERGENCE OF PT   (A-GRID)
C  97    PQFLX    G/KG/DAY  NLAY   3-D FLUX CONVERGENCE OF PQ   (A-GRID)
C  98    DPUDT     M/S/DAY  NLAY   TOTAL PU TENDENCY            (A-GRID)
C  99    DPVDT     M/S/DAY  NLAY   TOTAL PV TENDENCY            (A-GRID)
C 100    DPTDT     DEG/DAY  NLAY   TOTAL PT TENDENCY            (A-GRID)
C 101    DPQDT    G/KG/DAY  NLAY   TOTAL PQ TENDENCY            (A-GRID)
C 102    VINTUQ M/SEC G/KG  1      VERTICALLY INTEGRATED U*Q    (C-GRID)
C 103    VINTVQ M/SEC G/KG  1      VERTICALLY INTEGRATED V*Q    (C-GRID)
C 104    VINTUT M/SEC  DEG  1      VERTICALLY INTEGRATED U*T    (C-GRID)
C 105    VINTVT M/SEC  DEG  1      VERTICALLY INTEGRATED V*T    (C-GRID)
C 106    CLDFRC             1      2-DIMENSIONAL TOTAL CLOUD FRACTION   
C 107    QINT               1      PRECIPITABLE WATER   (GM/CM**2)
C 108    U2M                1      U AT  2 METERS (M/SEC)
C 109    V2M                1      V AT  2 METERS (M/SEC)
C 110    T2M                1      T AT  2 METERS (DEG)
C 111    Q2M                1      Q AT  2 METERS (G/KG)
C 112    U10M               1      U AT 10 METERS (M/SEC)
C 113    V10M               1      V AT 10 METERS (M/SEC)
C 114    T10M               1      T AT 10 METERS (DEG)
C 115    Q10M               1      Q AT 10 METERS (G/KG)
C 116    DTRAIN    KG*M/S   NLAY   CONVECTIVE CLOUD DETRAINMENT               
C 117    QFILL     G/KG/DAY NLAY   FILLING OF NEGATIVE SPECIFIC HUMIDITY            
C                                                                       
C **********************************************************************
C **********************************************************************
C                                                                       
      INTEGER   NUFLUX                                                  
      INTEGER   NVFLUX                                                  
      INTEGER   NHFLUX                                                  
      INTEGER   NEFLUX                                                  
      INTEGER   NQICE                                                   
      INTEGER   NRADLWG                                                 
      INTEGER   NRADSWG                                                 
      INTEGER   NRI                                                     
      INTEGER   NCT                                                     
      INTEGER   NCU                                                     
      INTEGER   NET                                                     
      INTEGER   NEU                                                     
      INTEGER   NTURBU                                                  
      INTEGER   NTURBV                                                  
      INTEGER   NTURBT                                                  
      INTEGER   NTURBQ                                                  
      INTEGER   NMOISTT                                                 
      INTEGER   NMOISTQ                                                 
      INTEGER   NRADLW                                                  
      INTEGER   NRADSW                                                  
      INTEGER   NPREACC                                                 
      INTEGER   NPRECON                                                 
      INTEGER   NTUFLUX                                                 
      INTEGER   NTVFLUX                                                 
      INTEGER   NTTFLUX                                                 
      INTEGER   NTQFLUX                                                 
      INTEGER   NCN                                                     
      INTEGER   NWINDS                                                  
      INTEGER   NDTSRF                                                  
      INTEGER   NTG                                                     
      INTEGER   NTS                                                     
      INTEGER   NDTG                                                    
      INTEGER   NQG                                                     
      INTEGER   NQS                                                     
      INTEGER   NTGRLW                                                  
      INTEGER   NST4                                                    
      INTEGER   NOLR                                                    
      INTEGER   NOLRCLR                                                 
      INTEGER   NLWGCLR                                                 
      INTEGER   NLWCLR                                                  
      INTEGER   NTLW                                                    
      INTEGER   NSHLW                                                   
      INTEGER   NOZLW                                                   
      INTEGER   NCLMOLW                                                 
      INTEGER   NCLROLW                                                 
      INTEGER   NCLMOSW                                                 
      INTEGER   NCLROSW                                                 
      INTEGER   NRADSWT                                                 
      INTEGER   NFREQMD                                                 
      INTEGER   NFREQPN                                                 
      INTEGER   NFREQLW                                                 
      INTEGER   NEVAP                                                   
      INTEGER   NDPDT                                                   
      INTEGER   NANALU                                                  
      INTEGER   NANALV                                                  
      INTEGER   NANALT                                                  
      INTEGER   NANALQ                                                  
      INTEGER   NOMEGA                                                  
      INTEGER   NDUDT                                                  
      INTEGER   NDVDT                                                  
      INTEGER   NDTDT                                                  
      INTEGER   NDQDT                                                  
      INTEGER   NVORT                                                   
      INTEGER   NPGV                                                    
      INTEGER   NCORU                                                   
      INTEGER   NCORV                                                   
      INTEGER   NUSTAR                                                  
      INTEGER   NZ0                                                     
      INTEGER   NFRQTRB                                                 
      INTEGER   NPBL                                                    
      INTEGER   NSWCLR                                                  
      INTEGER   NOSR                                                    
      INTEGER   NOSRCLR                                                 
      INTEGER   NCLDMAS                                                 
      INTEGER   NUAVE                                                   
      INTEGER   NVAVE                                                   
      INTEGER   NTAVE                                                   
      INTEGER   NQAVE                                                   
      INTEGER   NMDIV                                                   
      INTEGER   NPAVE                                                   
      INTEGER   NQQAVE                                                  
      INTEGER   NSWGCLR                                                 
      INTEGER   NANALP                                                  
      INTEGER   NSDIAG1                                                  
      INTEGER   NSDIAG2                                                  
      INTEGER   NUDIAG1                                                  
      INTEGER   NUDIAG2                                                  
      INTEGER   NDIABU                                                   
      INTEGER   NDIABV                                                   
      INTEGER   NDIABT                                                   
      INTEGER   NDIABQ                                                   
      INTEGER   NPSIGDOT                                                 
      INTEGER   NPGU                                                     
      INTEGER   NPUFLX                                                   
      INTEGER   NPVFLX                                                   
      INTEGER   NPTFLX                                                   
      INTEGER   NPQFLX                                                   
      INTEGER   NDPUDT                                                   
      INTEGER   NDPVDT                                                   
      INTEGER   NDPTDT                                                   
      INTEGER   NDPQDT                                                   
      INTEGER   NVINTUQ                                                  
      INTEGER   NVINTVQ                                                  
      INTEGER   NVINTUT                                                  
      INTEGER   NVINTVT                                                  
      INTEGER   NCLDFRC                                                  
      INTEGER   NQINT                                                    
      INTEGER   NU2M                                                    
      INTEGER   NV2M                                                    
      INTEGER   NT2M                                                    
      INTEGER   NQ2M                                                    
      INTEGER   NU10M                                                    
      INTEGER   NV10M                                                    
      INTEGER   NT10M                                                    
      INTEGER   NQ10M                                                    
      INTEGER   NDTRAIN                                                  
      INTEGER   NQFILL                                                   
C                                                                       
      EQUIVALENCE ( NDIAG( 1) ,  NUFLUX  )                              
      EQUIVALENCE ( NDIAG( 2) ,  NVFLUX  )                              
      EQUIVALENCE ( NDIAG( 3) ,  NHFLUX  )                              
      EQUIVALENCE ( NDIAG( 4) ,  NEFLUX  )                              
      EQUIVALENCE ( NDIAG( 5) ,  NQICE   )                              
      EQUIVALENCE ( NDIAG( 6) ,  NRADLWG )                              
      EQUIVALENCE ( NDIAG( 7) ,  NRADSWG )                              
      EQUIVALENCE ( NDIAG( 8) ,  NRI     )                              
      EQUIVALENCE ( NDIAG( 9) ,  NCT     )                              
      EQUIVALENCE ( NDIAG(10) ,  NCU     )                              
      EQUIVALENCE ( NDIAG(11) ,  NET     )                              
      EQUIVALENCE ( NDIAG(12) ,  NEU     )                              
      EQUIVALENCE ( NDIAG(13) ,  NTURBU  )                              
      EQUIVALENCE ( NDIAG(14) ,  NTURBV  )                              
      EQUIVALENCE ( NDIAG(15) ,  NTURBT  )                              
      EQUIVALENCE ( NDIAG(16) ,  NTURBQ  )                              
      EQUIVALENCE ( NDIAG(17) ,  NMOISTT )                              
      EQUIVALENCE ( NDIAG(18) ,  NMOISTQ )                              
      EQUIVALENCE ( NDIAG(19) ,  NRADLW  )                              
      EQUIVALENCE ( NDIAG(20) ,  NRADSW  )                              
      EQUIVALENCE ( NDIAG(21) ,  NPREACC )                              
      EQUIVALENCE ( NDIAG(22) ,  NPRECON )                              
      EQUIVALENCE ( NDIAG(23) ,  NTUFLUX )                              
      EQUIVALENCE ( NDIAG(24) ,  NTVFLUX )                              
      EQUIVALENCE ( NDIAG(25) ,  NTTFLUX )                              
      EQUIVALENCE ( NDIAG(26) ,  NTQFLUX )                              
      EQUIVALENCE ( NDIAG(27) ,  NCN     )                              
      EQUIVALENCE ( NDIAG(28) ,  NWINDS  )                              
      EQUIVALENCE ( NDIAG(29) ,  NDTSRF  )                              
      EQUIVALENCE ( NDIAG(30) ,  NTG     )                              
      EQUIVALENCE ( NDIAG(31) ,  NTS     )                              
      EQUIVALENCE ( NDIAG(32) ,  NDTG    )                              
      EQUIVALENCE ( NDIAG(33) ,  NQG     )                              
      EQUIVALENCE ( NDIAG(34) ,  NQS     )                              
      EQUIVALENCE ( NDIAG(35) ,  NTGRLW  )                              
      EQUIVALENCE ( NDIAG(36) ,  NST4    )                              
      EQUIVALENCE ( NDIAG(37) ,  NOLR    )                              
      EQUIVALENCE ( NDIAG(38) ,  NOLRCLR )                              
      EQUIVALENCE ( NDIAG(39) ,  NLWGCLR )                              
      EQUIVALENCE ( NDIAG(40) ,  NLWCLR  )                              
      EQUIVALENCE ( NDIAG(41) ,  NTLW    )                              
      EQUIVALENCE ( NDIAG(42) ,  NSHLW   )                              
      EQUIVALENCE ( NDIAG(43) ,  NOZLW   )                              
      EQUIVALENCE ( NDIAG(44) ,  NCLMOLW )                              
      EQUIVALENCE ( NDIAG(45) ,  NCLROLW )                              
      EQUIVALENCE ( NDIAG(46) ,  NCLMOSW )                              
      EQUIVALENCE ( NDIAG(47) ,  NCLROSW )                              
      EQUIVALENCE ( NDIAG(48) ,  NRADSWT )                              
      EQUIVALENCE ( NDIAG(49) ,  NFREQMD )                              
      EQUIVALENCE ( NDIAG(50) ,  NFREQPN )                              
      EQUIVALENCE ( NDIAG(51) ,  NFREQLW )                              
      EQUIVALENCE ( NDIAG(52) ,  NEVAP   )                              
      EQUIVALENCE ( NDIAG(53) ,  NDPDT   )                              
      EQUIVALENCE ( NDIAG(54) ,  NANALU  )                              
      EQUIVALENCE ( NDIAG(55) ,  NANALV  )                              
      EQUIVALENCE ( NDIAG(56) ,  NANALT  )                              
      EQUIVALENCE ( NDIAG(57) ,  NANALQ  )                              
      EQUIVALENCE ( NDIAG(58) ,  NOMEGA  )                              
      EQUIVALENCE ( NDIAG(59) ,  NDUDT   )                              
      EQUIVALENCE ( NDIAG(60) ,  NDVDT   )                              
      EQUIVALENCE ( NDIAG(61) ,  NDTDT   )                              
      EQUIVALENCE ( NDIAG(62) ,  NDQDT   )                              
      EQUIVALENCE ( NDIAG(63) ,  NVORT   )                              
      EQUIVALENCE ( NDIAG(64) ,  NPGV    )                              
      EQUIVALENCE ( NDIAG(65) ,  NCORU   )                              
      EQUIVALENCE ( NDIAG(66) ,  NCORV   )                              
      EQUIVALENCE ( NDIAG(67) ,  NUSTAR  )                              
      EQUIVALENCE ( NDIAG(68) ,  NZ0     )                              
      EQUIVALENCE ( NDIAG(69) ,  NFRQTRB )                              
      EQUIVALENCE ( NDIAG(70) ,  NPBL    )                              
      EQUIVALENCE ( NDIAG(71) ,  NSWCLR  )                              
      EQUIVALENCE ( NDIAG(72) ,  NOSR    )                              
      EQUIVALENCE ( NDIAG(73) ,  NOSRCLR )                              
      EQUIVALENCE ( NDIAG(74) ,  NCLDMAS )                              
      EQUIVALENCE ( NDIAG(75) ,  NUAVE   )                              
      EQUIVALENCE ( NDIAG(76) ,  NVAVE   )                              
      EQUIVALENCE ( NDIAG(77) ,  NTAVE   )                              
      EQUIVALENCE ( NDIAG(78) ,  NQAVE   )                              
      EQUIVALENCE ( NDIAG(79) ,  NMDIV   )                              
      EQUIVALENCE ( NDIAG(80) ,  NPAVE   )                              
      EQUIVALENCE ( NDIAG(81) ,  NQQAVE  )                              
      EQUIVALENCE ( NDIAG(82) ,  NSWGCLR )                              
      EQUIVALENCE ( NDIAG(83) ,  NANALP  )                              
      EQUIVALENCE ( NDIAG(84) ,  NSDIAG1 )                              
      EQUIVALENCE ( NDIAG(85) ,  NSDIAG2 )                              
      EQUIVALENCE ( NDIAG(86) ,  NUDIAG1 )                              
      EQUIVALENCE ( NDIAG(87) ,  NUDIAG2 )                              
      EQUIVALENCE ( NDIAG(88) ,  NDIABU  )                              
      EQUIVALENCE ( NDIAG(89) ,  NDIABV  )                              
      EQUIVALENCE ( NDIAG(90) ,  NDIABT  )                              
      EQUIVALENCE ( NDIAG(91) ,  NDIABQ  )                              
      EQUIVALENCE ( NDIAG(92) ,  NPSIGDOT)                              
      EQUIVALENCE ( NDIAG(93) ,  NPGU    )                              
      EQUIVALENCE ( NDIAG(94) ,  NPUFLX  )                              
      EQUIVALENCE ( NDIAG(95) ,  NPVFLX  )                              
      EQUIVALENCE ( NDIAG(96) ,  NPTFLX  )                              
      EQUIVALENCE ( NDIAG(97) ,  NPQFLX  )                              
      EQUIVALENCE ( NDIAG(98) ,  NDPUDT  )                              
      EQUIVALENCE ( NDIAG(99) ,  NDPVDT  )                              
      EQUIVALENCE ( NDIAG(100),  NDPTDT  )                              
      EQUIVALENCE ( NDIAG(101),  NDPQDT  )                              
      EQUIVALENCE ( NDIAG(102),  NVINTUQ )                              
      EQUIVALENCE ( NDIAG(103),  NVINTVQ )                              
      EQUIVALENCE ( NDIAG(104),  NVINTUT )                              
      EQUIVALENCE ( NDIAG(105),  NVINTVT )                              
      EQUIVALENCE ( NDIAG(106),  NCLDFRC )                              
      EQUIVALENCE ( NDIAG(107),  NQINT   )                              
      EQUIVALENCE ( NDIAG(108),  NU2M    )                              
      EQUIVALENCE ( NDIAG(109),  NV2M    )                              
      EQUIVALENCE ( NDIAG(110),  NT2M    )                              
      EQUIVALENCE ( NDIAG(111),  NQ2M    )                              
      EQUIVALENCE ( NDIAG(112),  NU10M   )                              
      EQUIVALENCE ( NDIAG(113),  NV10M   )                              
      EQUIVALENCE ( NDIAG(114),  NT10M   )                              
      EQUIVALENCE ( NDIAG(115),  NQ10M   )                              
      EQUIVALENCE ( NDIAG(116),  NDTRAIN )                              
      EQUIVALENCE ( NDIAG(117),  NQFILL  )                              
C                                                                       
C  DIAGNOSTICS INPUT NAMELIST                                           
C  ==========================                                           
C                                                                       
      NAMELIST /INDIAG/ NUFLUX  , NVFLUX  , NHFLUX  , NEFLUX  ,         
     .                  NQICE   , NRADLWG , NRADSWG , NRI     ,         
     .                  NCT     , NCU     , NET     , NEU     ,         
     .                  NTURBU  , NTURBV  , NTURBT  , NTURBQ  ,         
     .                  NMOISTT , NMOISTQ , NRADLW  , NRADSW  ,         
     .                  NPREACC , NPRECON , NTUFLUX , NTVFLUX ,         
     .                  NTTFLUX , NTQFLUX , NCN     , NWINDS  ,         
     .                  NDTSRF  , NTG     , NTS     , NDTG    ,         
     .                  NQG     , NQS     , NTGRLW  , NST4    ,         
     .                  NOLR    , NOLRCLR , NLWGCLR , NLWCLR  ,         
     .                  NTLW    , NSHLW   , NOZLW   , NCLMOLW ,         
     .                  NCLROLW , NCLMOSW , NCLROSW , NRADSWT ,         
     .                  NFREQMD , NFREQPN , NFREQLW , NEVAP   ,         
     .                  NDPDT   , NANALU  , NANALV  , NANALT  ,         
     .                  NANALQ  , NOMEGA  , NDUDT   , NDVDT   ,         
     .                  NDTDT   , NDQDT   , NVORT   , NPGV    ,         
     .                  NCORU   , NCORV   , NUSTAR  , NZ0     ,         
     .                  NFRQTRB , NPBL    , NSWCLR  , NOSR    ,         
     .                  NOSRCLR , NCLDMAS , NUAVE   , NVAVE   ,         
     .                  NTAVE   , NQAVE   , NMDIV   , NPAVE   ,         
     .                  NQQAVE  , NSWGCLR , NANALP  , NSDIAG1 ,
     .                  NSDIAG2 , NUDIAG1 , NUDIAG2 , NDIABU  ,
     .                  NDIABV  , NDIABT  , NDIABQ  , NPSIGDOT,
     .                  NPGU    , NPUFLX  , NPVFLX  , NPTFLX  ,
     .                  NPQFLX  , NDPUDT  , NDPVDT  , NDPTDT  ,
     .                  NDPQDT  , NVINTUQ , NVINTVQ , NVINTUT ,
     .                  NVINTVT , NCLDFRC , NQINT   , NU2M    ,
     .                  NV2M    , NT2M    , NQ2M    , NU10M   ,
     .                  NV10M   , NT10M   , NQ10M   , NDTRAIN ,
     .                  NQFILL
                                                                        
C **********************************************************************
C ****               INITIALIZE DIAGNOSTIC LOGICALS                 ****
C **********************************************************************
                                                                        
      PRINT *                                                           
      PRINT *, 'Total Number of Diagnostics Allowed (NDIAGMX)'
      PRINT *, 'Defined by Users Application:  ',NDIAGMX
      PRINT *                                                           
                                                                        
      DO N = 1,NDIAGT                                                
      NDIAG(N) = 0
      ENDDO                                                             
                                                                        
C **********************************************************************
C ****                  READ DIAGNOSTIC NAMELIST                    ****
C **********************************************************************
                                                                        
      REWIND 11                                                         
      WRITE (6 ,1000)                                                   
      READ  (11,INDIAG)                                                 
      WRITE (6 ,INDIAG)                                                 
                                                                        
C **********************************************************************
C **** CALCULATE POINTERS FOR DIAGNOSTICS SET TO NON-ZERO FREQUENCY ****
C **********************************************************************
                                                                        
      DO   N = 1,NDIAGT                                                
           NFREQ(N) =  NDIAG(N)
      IF(  NDIAG(N).NE.0 ) THEN                                           
      CALL SETDIAG (N)                                                  
      CALL CLRDIAG (N)                                                  
      ENDIF                                                             
      ENDDO                                                             
                                                                        
 1000 FORMAT(1X,'READING DIAGNOSTIC NAMELIST:')                         
      RETURN                                                            
      END                                                               
