      SUBROUTINE GETBCS (KBC, NYMD, NHMS, TGZIJ,                                
     .                   PHIIJ, TVAIJ, SOLIJ,                                   
     .                   SSTIJ, ALBIJ, WETIJ, THKIJ, SRFIJ, ZLWIJ,              
     .                   IMJM)                                                  
C***********************************************************************        
C  PURPOSE                                                                      
C     TO READ AND UPDATE GCM BOUNDARY CONDITIONS                                
C                                                                               
C  ARGUMENTS   DESCRIPTION                                                      
C     KBC .... UNIT TO READ BCS                                                 
C     NYMD ... YYMMDD TO INTERPOLATE TO                                         
C     NHMS ... HHMMSS TO INTERPOLATE TO                                         
C     TGZIJ .. GROUND TEMPERATURES (TO BE UPDATED OVER OCEAN)                   
C     PHIIJ .. SURFACE GEOPOTENTIAL                                             
C     TVAIJ .. SURFACE GEOPOTENTIAL VARIANCE                                    
C     SOLIJ .. INCIDENT SOLAR RADIATION AT THE TOP OF THE ATMOSPHERE            
C     SSTIJ .. SEA-SURFACE TEMPERATURE                                          
C     ALBIJ .. SURFACE ALBEDO         (0-1)                                     
C     WETIJ .. SURFACE GROUND WETNESS (0-1)                                     
C     THKIJ .. SEA-ICE THICKNESS                                                
C     SRFIJ .. SURFACE ROUGHNESS                                                
C     ZLWIJ .. LAND-WATER-GLACIER FLAG                                          
C     IMJM ... HORIZONTAL DIMENSION OF BOUNDARY CONDITIONS                      
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
                                                                                
      PARAMETER ( IRUN  = IMR*(JMR+1) )                                         
      PARAMETER ( ZERO   = 0.0 )                                                
      PARAMETER ( ONE    = 1.0 )                                                
      PARAMETER ( TWO    = 2.0 )                                                
      PARAMETER ( THREE  = 3.0 )                                                
      PARAMETER ( FOUR   = 4.0 )                                                
      PARAMETER ( FIVE   = 5.0 )                                                
      PARAMETER ( ALBSNO = 0.4 )                                                
      PARAMETER ( ALBMIN = 0.4 )                                                
      PARAMETER ( ALBMAX = 1.0 )                                                
      PARAMETER ( TICE   = 273.16 )                                             
      PARAMETER ( SSTMIN = 273.16 )                                             
      PARAMETER ( SSTMAX = 310.00 )                                             
                                                                                
      INTEGER   MONTH, DAY, SEC, KBC                                            
                                                                                
      DIMENSION TGZIJ(IMJM), PHIIJ(IMJM), TVAIJ(IMJM)                           
      DIMENSION SSTIJ(IMJM), ALBIJ(IMJM), WETIJ(IMJM)                           
      DIMENSION THKIJ(IMJM), SRFIJ(IMJM), ZLWIJ(IMJM)                           
      DIMENSION SOLIJ(IMJM)                                                     
         
      character*4 year                                                         
      character*3 cmon1, cmon2                                                       
      character*4 cnam  
                                                               
      INTEGER   DAYS(13)                                                        
      DATA      DAYS /31,31,28,31,30,31,30,31,31,30,31,30,31/                   
      DIMENSION ALAT(JMR+1), ALON(IMR)                                          
                                                                                
      COMMON /BCTEMP/ TOPG(IRUN), TOPV(IRUN), FLWI(IRUN)                        
      COMMON /BCTEMP/ SSTM(IRUN), SSTP(IRUN)                                    
      COMMON /BCTEMP/ WETM(IRUN), WETP(IRUN)                                    
      COMMON /BCTEMP/ ALBM(IRUN), ALBP(IRUN)                                    
      COMMON /BCTEMP/ THKM(IRUN), THKP(IRUN)                                    
      COMMON /BCTEMP/ SRFM(IRUN), SRFP(IRUN)                                    
                                                                                
      COMMON /SPACE/   COSZ(IRUN)                                               
      COMMON /SPACE/    VT1(IRUN)                                               
                                                                                
      DATA IMONP/0/, IMONM/0/, IKBC/0/                                          
      SAVE    imonp, imonm, ikbc                                                
                                                                                
      NSECF(N) = N/10000*3600 + MOD(N,10000)/100* 60 + MOD(N,100)               
      NMONF(N) = MOD(N,10000)/100                                               
      NDAYF(N) = MOD(N,100)                                                     
                                                                                
C*********************************************************************          
C****     FIND PROPER MONTH BOUNDARIES FROM 14-MONTH DATASET      ****          
C*********************************************************************          
                                                                                
      SEC    = NSECF(NHMS)                                                      
      MONTH  = NMONF(NYMD)                                                      
      DAY    = NDAYF(NYMD)                                                      
                                                                                
      MIDMON = DAYS(MONTH+1)/2 + 1                                                
                                                                                
      IF(DAY.LT.MIDMON) THEN
         imnm = month
         imnp = month + 1
      ELSE                                                                      
         IMNM = MONTH + 1                                                       
         IMNP = MONTH + 2                                                       
      ENDIF                                                                     
                                                                                
C*********************************************************************          
C****           READ DATA SET TWICE TO GET BOTH MONTHS            ****          
C*********************************************************************          
                                                                                
      IF(IMONM.NE.IMNM .OR. IMONP.NE.IMNP .OR. IKBC.NE.KBC) THEN                
                                                                                
         IMONM = IMNM                                                           
         IMONP = IMNP                                                           
         IKBC  = KBC                                                            
                                                                                
C  FIRST READ OF DATA                                                           
C====================                                                           
         REWIND KBC                                                             
         READ  (KBC) IMJMBC,  year                                                     
         IF( IMJMBC.NE.IMJM ) THEN                                              
         PRINT *, 'BAD B.C. DATASET'                                            
         PRINT *, 'MODEL DIMENSION ',IMJM                                       
         PRINT *, 'DATA  DIMENSION ',IMJMBC                                     
         PRINT *, 'YEAR ',YEAR                                     
         STOP                                                                   
         ENDIF                                                                  
         READ  (KBC)  cnam, TOPG                                                      
         READ  (KBC)  cnam, TOPV                                                      
         READ  (KBC)  cnam, FLWI                                                      
                                                                                
         DO 200 K=1,IMONM                                                       
         READ (KBC) cmon1, cnam, SSTM                                                      
         READ (KBC) cmon1, cnam, ALBM                                                      
         READ (KBC) cmon1, cnam, WETM                                                      
         READ (KBC) cmon1, cnam, THKM                                                      
         READ (KBC) cmon1, cnam, SRFM                                                      
 200     CONTINUE                                                               
                                                                                
C  SECOND READ OF DATA                                                          
C=====================                                                          
         REWIND KBC                                                             
         READ  (KBC)  IMJMBC, year                                                     
         READ  (KBC)  cnam, TOPG                                                      
         READ  (KBC)  cnam, TOPV                                                      
         READ  (KBC)  cnam, FLWI                                                      
                                                                                
         DO 300 K=1,IMONP                                                       
         READ (KBC) cmon2, cnam, SSTP                                                      
         READ (KBC) cmon2, cnam, ALBP                                                      
         READ (KBC) cmon2, cnam, WETP                                                      
         READ (KBC) cmon2, cnam, THKP                                                      
         READ (KBC) cmon2, cnam, SRFP                                                      
 300     CONTINUE 
      
      mbefor = imonm-1
      mafter = imonp-1                                                        
      WRITE(6,1000) NYMD,NHMS, mbefor,cmon1, mafter,cmon2, year, KBC                               
 1000 FORMAT(1X,'Updating Boundary Conditions for   NYMD: ',I6,2X,              
     .        'NHMS: ',I6,/,1X,'Reading Months ',I2,' (',a3,') and ',
     .         I2,' (',a3,')',2X,'Year: ',a4,' From Unit ',I2)                                                  
      ENDIF                                                                     
                                                                                
C*********************************************************************          
C****             INTERPOLATE DATA TO CURRENT TIME                ****          
C*********************************************************************          
                                                                                
      IF(DAY.LT.MIDMON) THEN                                                    
               ISC  = (DAY+DAYS(IMONM))*86400 + SEC                             
      ELSE                                                                      
               ISC  = (DAY            )*86400 + SEC                             
      ENDIF                                                                     
                                                                                
      ISCM = (DAYS(IMONM)/2+1               )*86400                             
      ISCP = (DAYS(IMONP)/2+1 + DAYS(IMONM) )*86400                             
      
      FACP = (ISC-ISCM) / REAL(ISCP-ISCM)                                       
      FACM = ONE-FACP                                                           
                                                                                
      DO 20 I=1,IRUN                                                            
      PHIIJ(I) = TOPG(I)                                                        
      TVAIJ(I) = TOPV(I)                                                        
      ZLWIJ(I) = FLWI(I)                                                        
      VT1  (I) = SRFM(I) * FACM                                                 
      SRFIJ(I) = VT1 (I) + SRFP(I) * FACP                                       
      VT1  (I) = ALBM(I) * FACM                                                 
      ALBIJ(I) = VT1 (I) + ALBP(I) * FACP                                       
      VT1  (I) = WETM(I) * FACM                                                 
      WETIJ(I) = VT1 (I) + WETP(I) * FACP                                       
      VT1  (I) = THKM(I) * FACM                                                 
      THKIJ(I) = VT1 (I) + THKP(I) * FACP                                       
      VT1  (I) = SSTM(I) * FACM                                                 
      SSTIJ(I) = VT1 (I) + SSTP(I) * FACP                                       
   20 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****     COMPUTE INCIDENT SOLAR RADIATION AT TOP OF ATMOSPHERE    ****        
C **********************************************************************        
C                                                                               
      S0 = GETCON('S0')                                                         
      DO 400 J=1,JMR+1                                                          
      ALAT(J) = - 90. + (J-1)*180./JMR                                          
  400 CONTINUE                                                                  
      DO 500 I=1,IMR                                                            
      ALON(I) = -180. + (I-1)*360./IMR                                          
  500 CONTINUE                                                                  
                                                                                
      CALL ASTRO ( NYMD,NHMS, ALAT,ALON, IMR,JMR+1, COSZ,RA )                   
      DO 30 I=1,IRUN                                                            
      SOLIJ(I) = S0 * (ONE/RA**2) * COSZ(I)                                     
   30 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****     RESOLVE INCONSISTANCIES IN NEW SURFACE FIELD SETTINGS    ****        
C **********************************************************************        
C                                                                               
C LAND & SEA ICE COVERED POINTS HAVE ALBEDO BETWEEN 0.40 AND 1.00               
C ===============================================================               
                                                                                
      DO 40 I=1,IRUN                                                            
      IF( (ZLWIJ(I).EQ.THREE .OR.                                               
     .     THKIJ(I).GT.ZERO) .AND.                                              
     .     ALBIJ(I).LT.ALBMIN ) ALBIJ(I) = ALBMIN                               
                                                                                
      IF( (ZLWIJ(I).EQ.THREE .OR.                                               
     .     THKIJ(I).GT.ZERO) .AND.                                              
     .     ALBIJ(I).GT.ALBMAX ) ALBIJ(I) = ALBMAX                               
C                                                                               
C OPEN WATER POINTS HAVE SSTS BETWEEN TICE AND 310.0                            
C ==================================================                            
                                                                                
      IF(  ZLWIJ(I).EQ.ONE  .AND.                                               
     .     THKIJ(I).EQ.ZERO .AND.                                               
     .     SSTIJ(I).LT.SSTMIN ) SSTIJ(I) = SSTMIN                               
                                                                                
      IF(  ZLWIJ(I).EQ.ONE  .AND.                                               
     .     THKIJ(I).EQ.ZERO .AND.                                               
     .     SSTIJ(I).GT.SSTMAX ) SSTIJ(I) = SSTMAX                               
   40 CONTINUE                                                                  
C                                                                               
C*********************************************************************          
C****   UPDATE GROUND TEMPERATURE OVER OCEANS AND GROUND WETNESS  ****          
C*********************************************************************          
                                                                                
      DO 50 I=1,IRUN                                                            
      IF( ZLWIJ(I).EQ.ONE .AND. THKIJ(I).EQ.ZERO )   TGZIJ(I) = SSTIJ(I)        
      IF( ZLWIJ(I).EQ.ONE .OR.  ZLWIJ(I).EQ.THREE .OR.                          
     .    ZLWIJ(I).EQ.TWO .AND. ALBIJ(I).GT.ALBSNO ) WETIJ(I) = ONE             
   50 CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
