      SUBROUTINE GETFG ( UP,VP,HP,QP,RHP,SLP,SLU,SLV,TSRF,TVSRF,
     .                   PLEV,IRUN,NLEV,NB,
     .                   PKZ,TD,THV,RH,
     .                   WRK )      
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     To obtain FIRST-GUESS Fields on Pressure-Levels for Analysis
C
C  INPUT  DESCRIPTION                                                         
C     PLEV ..... Analysis Pressure Levels
C     IRUN ..... Horizontal Dimension
C     NLEV ..... Number of Analysis Pressure Levels
C     NB ....... Current Time Index
C     PKZ ...... Upper Air Work Array (IRUN,NLAY)
C     TD ....... Upper Air Work Array (IRUN,NLAY)
C     THV ...... Upper Air Work Array (IRUN,NLAY)
C     RH ....... Upper Air Work Array (IRUN,NLAY)
C     WRK ...... Misc.     Work Array (IRUN,8)
C
C  OUTPUT DESCRIPTION                                                         
C     UP ....... Zonal      U-Wind on Pressure                                       
C     VP ....... Meridional V-Wind on Pressure                                       
C     HP ....... Heights (Virtual) on Pressure                                       
C     QP ....... Mixing Ratio      on Pressure                                       
C     RHP ...... Relative Humidity on Pressure                                       
C     SLP ...... Sea-Level Pressure
C     SLU ...... Sea-Level U-Wind
C     SLV ...... Sea-Level V-Wind
C     TSRF ..... Surface   Temperature
C     TVSRF .... Surface   Temperature (Virtual)
C
C  NOTE:
C     STATE COMMON is assumed to be on A-GRID
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <vmetric.com>                                                          
                                                                                
      PARAMETER  (IMJMR   =  IMR*(JMR+1)  )                                     
      PARAMETER  (ONE     =  1.           )                                     
      PARAMETER  (TWO     =  2.           )                                     
      PARAMETER  (THREE   =  3.           )                                     
                                                                                
      Dimension    UP (irun,nlev)
      Dimension    VP (irun,nlev)
      Dimension    HP (irun,nlev)
      Dimension    QP (irun,nlev)
      Dimension   RHP (irun,nlev)
      Dimension   SLP (irun)
      Dimension   SLU (irun)
      Dimension   SLV (irun)
      Dimension  TSRF (irun)
      Dimension TVSRF (irun)
      Dimension  PLEV (nlev)

c Work Arrays
c -----------
      Dimension    PKZ (IRUN,NLAYR)
      Dimension    TD  (IRUN,NLAYR)
      Dimension    THV (IRUN,NLAYR)
      Dimension    RH  (IRUN,NLAYR)
      Dimension    WRK (IRUN,8)

                                                                                
C *********************************************************************
C *****           CHECK HORIZONTAL CONSISTANCY                     ****
C *********************************************************************

      if( irun.ne.imjmr ) then
      Print *
      Print *, 'Inconsistant Horizontal Resolution'
      Print *
      Print *, 'GCM Compiled with IMJMR = ',imjmr
      Print *, 'GETFG called with IRUN  = ',irun
      Print *
      Stop
      endif

      UNDEF = getcon('UNDEF')
      grav  = getcon('GRAVITY')
      AKAP  = getcon('KAPPA')                                                    
                                                                                
C *********************************************************************         
C *****        CONVERT POTENTIAL TEMPERATURE TO TEMPERATURE        ****         
C *****       CONVERT SPECIFIC HUMIDITY TO RELATIVE HUMIDITY       ****         
C *********************************************************************         

      call pkappa ( pz(1,1,nb),pkht,pkz,imr,jmr+1,nlayr )

      DO 40 L = 1,NLAYR                                                         
      DO    I = 1,IMJMR                                                         
      TD (I,L) = TZ(I,1,L,nb)*PKZ(I,L)                                     
      THV(I,L) = TZ(I,1,L,nb)*(1. + 0.609*QZ(I,1,L,nb) )                        
      WRK(I,1) = PZ(I,1  ,nb)*SIG(L) + PTOP                                  
      ENDDO
                                                                                
      CALL QSTARS ( TD(1,L),wrk(1,1),                                             
     .             wrk(1,2),wrk(1,3),wrk(1,4),wrk(1,5),wrk(1,6),
     .             IMJMR,RH(1,L) )                                                  
                                                                                
      DO I = 1,IMJMR                                                         
      RH(I,L) = QZ(I,1,L,nb) / RH(I,L)                                      
      ENDDO
   40 CONTINUE                                                                  
                                                                                

C *********************************************************************         
C *****          COMPUTE SEA-LEVEL AND SURFACE QUANTITIES          ****         
C *********************************************************************         
                                                                                
      CALL SLPRS  ( SLP,PZ(1,1,NB),PTOP,PHISZ,THV,DSIG,WRK )                                                 
      CALL TSPBL  ( PZ,PTOP,TZ(1,1,1,NB),DSIG,IMR,JMR+1,NLAYR,TSRF  )
      CALL TSPBL  ( PZ,PTOP,THV         ,DSIG,IMR,JMR+1,NLAYR,TVSRF )
 
      do I=1,IMJMR
      SLU(I) = UZ(I,1,NLAYR,NB)
      SLV(I) = VZ(I,1,NLAYR,NB)
      ENDDO


C *********************************************************************         
C *****          INTERPOLATE TO ANALYSIS PRESSURE LEVELS           ****         
C *********************************************************************         
                                                                                
      DO   L = 1, NLEV
      CALL INTERP  ( up(1,L),uz(1,1,1,nb),PKHT,PKZ,PLEV(L),UNDEF )               
      CALL INTERP  ( vp(1,L),vz(1,1,1,nb),PKHT,PKZ,PLEV(L),UNDEF )               
      CALL INTERP  (rhp(1,L),rh          ,PKHT,PKZ,PLEV(L),UNDEF )               
                                                                                
      CALL INTGEOP ( hp(1,L),THV,PKHT,PKZ,                          
     .               PHISZ,PLEV(L),UNDEF,WRK )                            
                                                                               
      CALL INTERQ  ( wrk(1,8), QP(1,L), RH, TD, QZ(1,1,1,nb),                   
     .               PKHT, PKZ, PLEV(L),                                         
     .               UNDEF, wrk(1,1),wrk(1,3),wrk(1,4),
     .                      wrk(1,5),wrk(1,6),wrk(1,7) )                      


c Scale Relative Humidity and Compute Mixing Ratio
c ------------------------------------------------
      DO I=1,IMJMR
      IF( RHP(I,L).NE.UNDEF ) RHP(I,L) =  100 * RHP(I,L)
      IF(  QP(I,L).NE.UNDEF )  QP(I,L) = 1000 *  QP(I,L)/(1.-QP(I,L))
      ENDDO

      ENDDO
                                                                                

C *********************************************************************         
C *****             FILL AND MASK UNDERGROUND VALUES               ****         
C *********************************************************************         
                                                                                
      DO I=1,IMJMR
      wrk(I,1) = PZ(I,1,nb)+PTOP
      wrk(I,2) = PHISZ(I,1)/grav
      IF( ZLWI(I,1).NE.ONE ) THEN
      SLU(I) = UNDEF
      SLV(I) = UNDEF
      ENDIF
      ENDDO

      CALL ZUNDERG ( HP,TVSRF,wrk(1,1),wrk(1,2),PLEV,
     .               UNDEF,IMR,JMR+1,NLEV )

      RETURN                                                                    
      END                                                                       
