      SUBROUTINE INTGEOP (PHI,THB,PKE,PKO,PHISS,PSTR,UNDEF,TEM)                 
C***********************************************************************        
C                                                                               
C INPUT                                                                         
C   THB .... POTENTIAL TEMPERATURE (T/PKO) AT FULL LEVELS                       
C   PKE .... P TO THE KAPPA AT THE BASE OF EACH LAYER                           
C   PKO .... P TO THE KAPPA AT FULL LEVELS                                      
C   PHISS .. SURFACE GEOPOTENTIAL                                               
C   PSTR ... OUTPUT PRESSURE LEVEL (SCALAR) IN MB                               
C   UNDEF .. UNDEFINED VALUE                                                    
C   TEM .... SCRATCH ARRAY                                                      
C                                                                               
C OUTPUT                                                                        
C   PHI .... GEOPOTENTIAL HEIGHT (M) AT PSTR                                    
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
                                                                                
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
                                                                                
      PARAMETER (HALF=.5)                                                       
      PARAMETER (ONE=1.)                                                        
      PARAMETER (TWO=2.)                                                        
                                                                                
      DIMENSION THB(IMJMR,NLAYR), PKE(IMJMR,NLAYR), PKO(IMJMR,NLAYR)            
      DIMENSION PHI(IMJMR)      , PHISS(IMJMR)    , TEM(IMJMR,5)                
                                                                                
      G   = GETCON('GRAVITY')                                                   
      CP  = GETCON('CP')                                                        
      AKP = GETCON('KAPPA')                                                     
                                                                                
      PKSTR  = PSTR ** AKP                                                      
      DO 10 I=1,IMJMR                                                           
      PHI(I) = UNDEF                                                            
   10 CONTINUE                                                                  
                                                                                
      PKMIN = PKO(1,NLAYR)                                                      
      DO 20 I=2,IMJMR                                                           
      IF( PKO(I,NLAYR).LT.PKMIN ) PKMIN = PKO(I,NLAYR)                          
   20 CONTINUE                                                                  
                                                                                
      IF( PKSTR.GE.PKMIN ) THEN                                                 
      DO 30 I=1,IMJMR                                                           
      IF (PKSTR .LE. PKE(I,NLAYR) .AND. PKSTR .GE. PKO(I,NLAYR)) THEN           
      PHI(I) = PHISS(I) + (PKE(I,NLAYR)-PKSTR) * THB(I,NLAYR)*CP                
      ENDIF                                                                     
   30 CONTINUE                                                                  
      ENDIF                                                                     
                                                                                
      DO 40 I=1,IMJMR                                                           
      TEM(I,5) = PHISS(I)                                                       
     .         +  (PKE(I,NLAYR)-PKO(I,NLAYR)) * THB(I,NLAYR)*CP                 
   40 CONTINUE                                                                  
                                                                                
      DO 310 L=NLAYR,2,-1                                                       
                                                                                
      PKMAX = PKO(1,L)                                                          
      DO 50 I=2,IMJMR                                                           
      IF( PKO(I,L).GT.PKMAX ) PKMAX = PKO(I,L)                                  
   50 CONTINUE                                                                  
      IF( PKSTR.GE.PKMAX ) GO TO 320                                            
                                                                                
      DO 60 I=1,IMJMR                                                           
      TEM(I,1) = THB(I,L-1)*(PKE(I,L-1)-PKO(I,L-1))                             
     .         + THB(I,L  )*(PKO(I,L  )-PKE(I,L-1))                             
                                                                                
      IF( PKSTR.GE.PKE(I,L-1) ) THEN                                            
      TEM(I,3) = PKO(I,L)                                                       
      TEM(I,2) = TEM(I,5)                                                       
      TEM(I,4) = THB(I,L)                                                       
      ENDIF                                                                     
                                                                                
      TEM(I,5) = TEM(I,5) + TEM(I,1) * CP                                       
                                                                                
      IF( PKSTR.LT.PKE(I,L-1) ) THEN                                            
      TEM(I,3) = PKO(I,L-1)                                                     
      TEM(I,2) = TEM(I,5  )                                                     
      TEM(I,4) = THB(I,L-1)                                                     
      ENDIF                                                                     
                                                                                
      IF( PKSTR.GE.PKO(I,L-1) .AND. PKSTR.LT.PKO(I,L) ) THEN                    
      PHI(I) = TEM(I,2)                                                         
      ENDIF                                                                     
                                                                                
      IF(L.EQ.2) THEN                                                           
      IF( PKSTR.LT.PKO(I,1) ) PHI(I) = TEM(I,2)                                 
      ENDIF                                                                     
                                                                                
      TEM(I,1) =  TEM(I,1)        / (PKO(I,L)-PKO(I,L-1))                       
      TEM(I,2) = (TEM(I,3)-PKSTR) / (TEM(I,3)-PKE(I,L-1))                       
                                                                                
      TEM(I,1) =  TEM(I,1) *      TEM(I,2)                                      
     .         +  TEM(I,4) * (TWO-TEM(I,2))                                     
      TEM(I,1) =  TEM(I,1) * (TEM(I,3)-PKSTR)                                   
                                                                                
      IF( PKSTR.GE.PKO(I,L-1) .AND. PKSTR.LT.PKO(I,L)) THEN                     
      PHI(I) = PHI(I) + TEM(I,1) * (CP*HALF)                                    
      ENDIF                                                                     
                                                                                
      IF(L.EQ.2) THEN                                                           
      IF( PKSTR.LT.PKO(I,1) ) PHI(I) = PHI(I) + TEM(I,1)*(CP*HALF)              
      ENDIF                                                                     
                                                                                
   60 CONTINUE                                                                  
  310 CONTINUE                                                                  
  320 CONTINUE                                                                  
                                                                                
      DO 70 I=1,IMJMR                                                           
      IF( PHI(I).NE.UNDEF ) PHI(I) = PHI(I) * (ONE/G)                           
   70 CONTINUE                                                                  
      RETURN                                                                    
      END                                                                       
