      SUBROUTINE PKAPPA (PZ,PKHT,PKZ,IMAX,JMAX,KMAX)                            
C***********************************************************************        
C  PURPOSE                                                                      
C     CALCULATE PHILLIPS P**KAPPA                                               
C  ARGUMENTS                                                                    
C     PZ ..... SURFACE PRESSURE - PTOP                                          
C     PKHT ... SIGMA EDGE-LEVEL PRESSURE**KAPPA                                 
C     PKZ .... SIGMA      LEVEL PRESSURE**KAPPA                                 
C     IMAX ... LONGITUDE  DIMENSION                                             
C     JMAX ... LATITUDE   DIMENSION                                             
C     KMAX ... VERTICAL   DIMENSION                                             
C  SUBPROGRAMS NEEDED                                                           
C     EXPBYK   COMPUTES  P**KAPPA  AND KAPPA=.2861328125                        
C  RECORD OF MODIFICATIONS                                                      
C  REMARKS:                                                                     
C     NOTE:  PERTURBATION GEOPOTENTIALS ARE USED WITH                           
C             TSTD = 280 DEG. K.    ,  PSTD = 1000 MB.                          
C            PKSTD = PSTD ** KAPPA  , THSTD = TSTD / PKSTD                      
C                                                                               
C***********************************************************************        
C*                GODDARD LABORATORY FOR ATMOSPHERES                            
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vmetric.com>                                                          
C                                                                               
      PARAMETER (  ONE   = 1.0 )                                                
      PARAMETER (   CP   = 1004.16  )                                           
      PARAMETER ( RGAS   = 287.0   )                                            
      PARAMETER ( ROCP   = RGAS/CP )                                            
      PARAMETER ( ROCPP1 = ROCP + 1 )                                           
      PARAMETER ( TSTD   =  280.0  )                                            
      PARAMETER ( PSTD   = 1000.0  )                                            
      PARAMETER ( IMJMR  = IMR*(JMR+1) )                                        
C                                                                               
      DIMENSION PZ  (IMAX,JMAX)                                                 
      DIMENSION PKHT(IMAX,JMAX,KMAX)                                            
      DIMENSION PKZ (IMAX,JMAX,KMAX)                                            
C                                                                               
      DIMENSION PINV(IMR,JMR+1)                                                 
      DIMENSION PLE (IMR,JMR+1,2)                                               
      DIMENSION PKLE(IMR,JMR+1,2)                                               
C                                                                               
C **********************************************************************        
C ****                      INITIALIZATION                          ****        
C **********************************************************************        
C                                                                               
      DO 10 I=1,IMJMR                                                           
      PINV(I,1) = ONE / ( ROCPP1*PZ(I,1) )                                      
   10 CONTINUE                                                                  
C                                                                               
C **********************************************************************        
C ****                  COMPUTE PHILLIPS P**KAPPA                   ****        
C **********************************************************************        
C                                                                               
      LUP = 1                                                                   
      LDN = 2                                                                   
C                                                                               
      PKTOP = EXPBYK(PTOP)                                                      
      DO 20 I=1,IMJMR                                                           
      PLE (I,1,LUP) = PTOP                                                      
      PKLE(I,1,LUP) = PKTOP                                                     
   20 CONTINUE                                                                  
                                                                                
      DO 1000 L = 1,NLAY                                                        
      DSIGINV = ONE/DSIG(L)                                                     
      DO 30 I=1,IMJMR                                                           
      PLE (I,1,LDN) = SIGE(L+1)*PZ(I,1) + PTOP                                  
      PKLE(I,1,LDN) = PKHT(I,1,L)                                               
C                                                                               
      PKZ(I,1,L) = ( PLE (I,1,LDN)*PKLE(I,1,LDN)                                
     .             - PLE (I,1,LUP)*PKLE(I,1,LUP) )                              
     .           *   PINV(I,1)*DSIGINV                                          
   30 CONTINUE                                                                  
C                                                                               
      LSV = LUP                                                                 
      LUP = LDN                                                                 
      LDN = LSV                                                                 
 1000 CONTINUE                                                                  
C                                                                               
      RETURN                                                                    
      END                                                                       
