      SUBROUTINE PKAPPC (PZ,PKHT,PKZ,IMAX,JMAX,KMAX)
C***********************************************************************
C  PURPOSE
C     CALCULATE PHILLIPS P**KAPPA ON C-GRID
C  ARGUMENTS
C     PZ ..... SURFACE PRESSURE - PTOP
C     PKHT ... SIGMA EDGE-LEVEL PRESSURE**KAPPA
C     PKZ .... SIGMA      LEVEL PRESSURE**KAPPA
C     IMAX ... LONGITUDE  DIMENSION
C     JMAX ... LATITUDE   DIMENSION
C     KMAX ... VERTICAL   DIMENSION
C  REMARKS:
C     NOTE:  PERTURBATION GEOPOTENTIALS ARE USED WITH
C             TSTD = 280 DEG. K.    ,  PSTD = 1000 MB.
C            PKSTD = PSTD ** KAPPA  , THSTD = TSTD / PKSTD
C
C***********************************************************************
C*                GODDARD LABORATORY FOR ATMOSPHERES
C***********************************************************************

#include <vreal.com>
#include <vrslv.com>
#include <vmetric.com>

      PARAMETER (  ONE    = 1.0 )
      PARAMETER ( IMJMR   = IMR*(JMR+1) )
      PARAMETER ( IMJMM1R = IMR* JMR    )
      PARAMETER ( IMJMM2R = IMR*(JMR-1) )
                  
      DIMENSION PZ  (IMAX,JMAX)
      DIMENSION PKHT(IMAX,JMAX,KMAX)
      DIMENSION PKZ (IMAX,JMAX,KMAX)
                  
      DIMENSION PINV(IMR,JMR+1)
      DIMENSION PLE (IMR,JMR+1,2)
      DIMENSION PKLE(IMR,JMR+1,2)
 
C **********************************************************************
C ****                      INITIALIZATION                          ****
C **********************************************************************
                  
      akap1 = one + getcon('KAPPA')
                  
      DO 10 I=1,IMJMM2R
      PINV(I,2) = ONE / ( akap1*PZ(I,2) )
   10 CONTINUE   
                  
C **********************************************************************
C ****                  COMPUTE PHILLIPS P**KAPPA                   ****
C **********************************************************************
                  
      LUP = 1    
      LDN = 2    
                  
      PKTOP = EXPBYK(PTOP)
      DO 20 I=1,IMJMM2R
      PLE (I,2,LUP) = PTOP
      PKLE(I,2,LUP) = PKTOP
   20 CONTINUE   
                  
      DO 1000 L = 1,NLAYR
      DSIGINV = ONE/DSIG(L)
      DO 30 I=1,IMJMM2R
      PLE (I,2,LDN) = SIGE(L+1)*PZ(I,2) + PTOP
      PKLE(I,2,LDN) = PKHT(I,2,L)
                  
      PKZ(I,2,L) = ( PLE (I,2,LDN)*PKLE(I,2,LDN)
     .             - PLE (I,2,LUP)*PKLE(I,2,LUP) )
     .           *   PINV(I,2)*DSIGINV
   30 CONTINUE   
                  
      LSV = LUP  
      LUP = LDN  
      LDN = LSV  
 1000 CONTINUE   
                  
      RETURN     
      END   
