      SUBROUTINE PRTBCS (KBC,NYMD,NHMS)                                         
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     PRINTS OUT SURFACE CONDITION FROM BOUNDARY DATA SETS                      
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
C                                                                               
      PARAMETER ( IM    = IMR    )                                              
      PARAMETER ( JNP   = JMR+1  )                                              
                                                                                
      PARAMETER ( ZERO  = 0.0    )                                              
      PARAMETER ( ONE   = 1.0    )                                              
      PARAMETER ( TWO   = 2.0    )                                              
      PARAMETER ( THREE = 3.0    )                                              
      PARAMETER ( FOUR  = 4.0    )                                              
      PARAMETER ( FIVE  = 5.0    )                                              
      PARAMETER (  ZP4  = 0.4    )                                              
      PARAMETER ( TICE  = 273.16 )                                              
      PARAMETER ( GRAV  = 9.81   )                                              
      PARAMETER ( AGRAV = ONE/GRAV )                                            
C                                                                               
      COMMON /SPACE/  TOPOG(IMR,JMR+1)                                          
      COMMON /SPACE/  FLAGS(IMR,JMR+1)                                          
C                                                                               
      CHARACTER*1 FLAGS, WFLAG, IFLAG, SFLAG, FFLAG, GFLAG, LFLAG               
      DATA        WFLAG /'W'/                                                   
      DATA        IFLAG /'I'/                                                   
      DATA        SFLAG /'S'/                                                   
      DATA        FFLAG /'F'/                                                   
      DATA        GFLAG /'G'/                                                   
      DATA        LFLAG /' '/                                                   
C                                                                               
      IMJM = IM*JNP                                                             
C                                                                               
C **********************************************************************        
C ****             INITIALIZE BOUNDARY CONDITIONS                   ****        
C **********************************************************************        
C                                                                               
      CALL GETBCS (KBC, NYMD, NHMS, TGZ, PHISZ, TVAZ, RADSWT,                   
     .             SSTZ, ALBZ, GWZ, THKZ, SRZ, ZLWI, IMJM)                      
C                                                                               
C **********************************************************************        
C ****              SET SURFACE FLAGS AND TOPOGRAPHY                ****        
C **********************************************************************        
C                                                                               
      DO 10 J = 1,JNP                                                           
      DO 10 I = 1,IM                                                            
          TOPOG(I,J) =         PHISZ(I,J)*AGRAV                                 
      IF ( ZLWI(I,J).EQ.ONE   )                      FLAGS(I,J) = WFLAG         
      IF ( ZLWI(I,J).EQ.TWO   )                      FLAGS(I,J) = LFLAG         
      IF ( ZLWI(I,J).EQ.THREE )                      FLAGS(I,J) = GFLAG         
      IF ( THKZ(I,J).GT.ZERO  )                      FLAGS(I,J) = IFLAG         
      IF ( ZLWI(I,J).EQ.TWO ) THEN                                              
      IF ( ALBZ(I,J).GT.ZP4 )                        FLAGS(I,J) = SFLAG         
      IF ( ALBZ(I,J).LE.ZP4 .AND. TGZ(I,J).LE.TICE ) FLAGS(I,J) = FFLAG         
      ENDIF                                                                     
   10 CONTINUE                                                                  
C                                                                               
C **********************************************************************        
C ****                      PRINT OUTPUT                            ****        
C **********************************************************************        
C                                                                               
      WRITE(6,6052)                                                             
 6052 FORMAT(//)                                                                
C                                                                               
      WRITE(6,1001)                                                             
      NPASS = 1 + (IM-1)/130                                                    
      DO 100 N = 1,NPASS                                                        
      IBEG  = 1+(N-1)*130                                                       
      ICHK  = IM - N *130                                                       
      IF(ICHK.GT.0) THEN                                                        
      IEND  =      N *130                                                       
      ELSE                                                                      
      IEND  = IM                                                                
      ENDIF                                                                     
      DO 110 JJ = 1,JNP                                                         
      J = JNP-JJ+1                                                              
      WRITE(6,1002) (FLAGS(I,J),I=IBEG,IEND)                                    
  110 CONTINUE                                                                  
      PRINT *                                                                   
  100 CONTINUE                                                                  
 1001 FORMAT(//1X,'LAND-WATER-ICE-SNOW-FROST POINTS',/)                         
 1002 FORMAT(1X,130A1)                                                          
C                                                                               
C     CALL PRINCE( TOPOG,'PHIS',IM,JNP,'DISP')                                  
C     CALL PRINCE(  TVAZ,'TVA ',IM,JNP,'DISP')                                  
C     CALL PRINCE(   TGZ,'GT  ',IM,JNP,'DISP')                                  
C     CALL PRINCE(   GWZ,'GW  ',IM,JNP,'DISP')                                  
C     CALL PRINCE(  ALBZ,'ALB ',IM,JNP,'DISP')                                  
C     CALL PRINCE(   SRZ,'SRF ',IM,JNP,'DISP')                                  
C     CALL PRINCE(  THKZ,'ICE ',IM,JNP,'DISP')                                  
C                                                                               
      RETURN                                                                    
      END 
