      SUBROUTINE RESTART (LU,IOFLAG,NYMD,NHMS,NB,NYMD0,NHMS0,JOB,NTYPE)         
C***********************************************************************        
C  PURPOSE                                                                      
C     Reads or Writes a Model Restart                                           
C  ARGUMENTS   DESCRIPTION                                                      
C     LU       Unit to read or write                                            
C     IOFLAG   Flag for IO:  1 to Read  a Restart                               
C                            2 to Write a Restart                               
C                           -2 to Write a Short Restart (No Tendencies)         
C     NYMD     CURRENT YYMMDD                                                   
C     NHMS     CURRENT HHMMSS                                                   
C     NB       CURRENT TIME INDEX                                               
C     NYMD0    YYMMDD OF BEGINNING OF EXPERIMENT                                
C     NHMS0    HHMMSS OF BEGINNING OF EXPERIMENT                                
C     JOB      CHARACTER*8 JOB IDENTIFIER                                       
C     NTYPE    RESTART RECORD TYPE:  -1  BEFORE ANALYSIS                        
C                                     0  AFTER  ANALYSIS                        
C                                     1  AFTER  ANALYSIS AND INITIALIZED        
C                                    1_  ANALYSIS TENDENCIES ON RESTART         
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
                                                                                
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <vphyscom.com>                                                         
#include <vmetric.com>                                                          
#include <iau.com>                                                          
                                                                                
      PARAMETER ( ZERO  = 0.0 )                                                 
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
      CHARACTER*8     JOB                                                       
      logical         dimchk                                                    
      REAL            QT, ZSIGE, ZPTOP, ZPMEAN                                  
      COMMON /SPACE/  QT(IMR,JMR+1)                                             
      COMMON /SPACE/ ZSIGE(NLAYR+1)                                             
                                                                                
C **********************************************************************        
C ****                 READ THE RESTART HEADER                      ****        
C **********************************************************************        
                                                                                
      IF(IOFLAG.EQ.1) THEN                                                      
      READ (LU) NYMD, NHMS, NB, NYMD0, NHMS0, IMAX, JMAXM1, KMAX,               
     .          KPROGS1, KPROGU1, KPROGS2, KPROGU2,                             
     .          KPHYS, NLWCLD, NSWCLD, JOB, NTYPE,         
     .          ZSIGE, ZPTOP, ZPMEAN, IRAS   
                                                                                
C **********************************************************************        
C ****                   CHECK CONSISTANCY                          ****        
C **********************************************************************        
                                                                                
      dimchk = imax.eq.imr .and. jmaxm1.eq.jmr .and. kmax.eq.nlayr              
                                                                                
      IF ( .not.dimchk ) then                                                   
      WRITE(6,9999) IMAX,    IMR,                                               
     .              JMAXM1,  JMR,                                               
     .              KMAX,    NLAYR                                              
 9999 FORMAT(1X,//,1x,'********* N O T E ***************',//,                   
     .       1X,'RESTART horizontal dimensions not consistant with ',           
     .          'loaded TEXT module:',/,                                        
     .       1X,'Read IMAX    as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read JMAX-1  as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KMAX    as ',I3,', Dimensioned as ',I3,//)                
      stop                                                                      
      endif                                                                     
                                                                                
      IF ( KPROGS1.NE.NPROGS1 .OR. KPROGU1.NE.NPROGU1 .OR.                      
     .     KPROGS2.NE.NPROGS2 .OR. KPROGU2.NE.NPROGU2 .OR.                      
     .     KPHYS .NE.NPHYS ) THEN                                               
      WRITE(6,9998) KPROGS1, NPROGS1,                                           
     .              KPROGU1, NPROGU1,                                           
     .              KPROGS2, NPROGS2,                                           
     .              KPROGU2, NPROGU2,                                           
     .              KPHYS,   NPHYS                                              
 9998 FORMAT(1X,//,1x,'********* N O T E ***************',//,                   
     .       1X,'RESTART record fields and/or tendencies ',/,                   
     .       1X,'        not consistant with loaded TEXT module.',/,            
     .       1X,'        SKIPPING or ZERO FILL values will be used:',//,        
     .       1X,'Read KPROGS1 as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KPROGU1 as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KPROGS2 as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KPROGU2 as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KPHYS   as ',I3,', Dimensioned as ',I3,/)                 
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                    LOAD SIGMA LEVELS                         ****        
C **********************************************************************        
                                                                                
      PRINT *                                                                   
      PRINT *, 'Reading Initial Conditions for ',NLAYR,' Sigma levels'          
      PRINT *, 'Model Top Pressure: ',ZPTOP,' mb.'                              
      PRINT *                                                                   
      PTOP    = ZPTOP                                                           
      DO 5 L=1,NLAYR+1                                                          
      SIGE(L) = ZSIGE(L)                                                        
      WRITE(6,8888) L,SIGE(L)                                                   
    5 CONTINUE                                                                  
 8888 FORMAT(1X,'L = ',I2,2X,'SIGE = ',F8.6)                                    
      PRINT *                                                                   
                                                                                
C **********************************************************************        
C ****       READ THE 1 TIME LEVEL SURFACE PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      nqmax = min( kprogs1,nprogs1 )                                            
                                                                                
      DO 10 NQ=1,nqmax                                                          
      READ (LU)  QT                                                             
      DO 10 I =1,IMJMR                                                          
      PROGS1(I,1,NQ) = QT(I,1)                                                  
   10 CONTINUE                                                                  
                                                                                
      if( kprogs1.gt.nprogs1 ) then                                             
      DO 11 NQ=1,kprogs1-nprogs1                                                
      READ (LU)  QT                                                             
   11 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nprogs1.gt.kprogs1 ) then                                             
      DO 12 NQ=1,nprogs1-kprogs1                                                
      DO 12 I =1,IMJMR                                                          
      PROGS1(I,1,NQ+nqmax) = zero                                               
   12 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****     READ THE 1 TIME LEVEL UPPER-AIR PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      nqmax = min( kprogu1,nprogu1 )                                            
                                                                                
      DO 20 NQ=1,nqmax                                                          
      DO 20  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
      DO 20 I =1,IMJMR                                                          
      PROGU1(I,1,L,NQ) = QT(I,1)                                                
   20 CONTINUE                                                                  
                                                                                
      if( kprogu1.gt.nprogu1 ) then                                             
      DO 21 NQ=1,kprogu1-nprogu1                                                
      DO 21  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
   21 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nprogu1.gt.kprogu1 ) then                                             
      DO 22 NQ=1,nprogu1-kprogu1                                                
      DO 22  L=1,NLAYR                                                          
      DO 22  I=1,IMJMR                                                          
      PROGU1(I,1,L,NQ+nqmax) = zero                                             
   22 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****       READ THE 2 TIME LEVEL SURFACE PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      nqmax = min( kprogs2,nprogs2 )                                            
                                                                                
      DO 30 NT=1,2                                                              
      DO 30 NQ=1,nqmax                                                          
      READ (LU)  QT                                                             
      DO 30 I =1,IMJMR                                                          
      PROGS2(I,1,NQ,NT) = QT(I,1)                                               
   30 CONTINUE                                                                  
                                                                                
      if( kprogs2.gt.nprogs2 ) then                                             
      DO 31 NT=1,2                                                              
      DO 31 NQ=1,kprogs2-nprogs2                                                
      READ (LU)  QT                                                             
   31 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nprogs2.gt.kprogs2 ) then                                             
      DO 32 NT=1,2                                                              
      DO 32 NQ=1,nprogs2-kprogs2                                                
      DO 32  I=1,IMJMR                                                          
      PROGS2(I,1,NQ,NT+nqmax) = zero                                            
   32 CONTINUE                                                                  
      endif                                                                     
                                                                                
C COMPUTE MEAN SURFACE PRESSURE                                                 
C -----------------------------                                                 
      CALL SETMET                                                               
      IF (ZPMEAN.EQ.ZERO) THEN                                                  
      PIMEAN =  PMEAN( PZ(1,1,NB),IMR,JMR+1 )                                   
      ELSE                                                                      
      PIMEAN = ZPMEAN                                                           
      ENDIF                                                                     
                                                                                
      PRINT *                                                                   
      PRINT *, 'Mean Surface Pressure: ',PIMEAN,' mb.'                          
      PRINT *                                                                   
                                                                                
C **********************************************************************        
C ****     READ THE 2 TIME LEVEL UPPER-AIR PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      nqmax = min( kprogu2,nprogu2 )                                            
                                                                                
      DO 40 NT=1,2                                                              
      DO 40 NQ=1,nqmax                                                          
      DO 40  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
      DO 40 I =1,IMJMR                                                          
      PROGU2(I,1,L,NQ,NT) = QT(I,1)                                             
   40 CONTINUE                                                                  
                                                                                
      if( kprogu2.gt.nprogu2 ) then                                             
      DO 41 NT=1,2                                                              
      DO 41 NQ=1,kprogu2-nprogu2                                                
      DO 41  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
   41 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nprogu2.gt.kprogu2 ) then                                             
      DO 42 NT=1,2                                                              
      DO 42 NQ=1,nprogu2-kprogu2                                                
      DO 42  L=1,NLAYR                                                          
      DO 42  I=1,IMJMR                                                          
      PROGU2(I,1,L,NQ,NT+nqmax) = zero                                          
   42 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****                  READ THE PHYSICS TENDENCIES                 ****        
C **********************************************************************        
                                                                                
      nqmax = min( kphys,nphys )                                                
                                                                                
      if( nqmax.ne.0 ) then                                                     
      DO 50 NQ=1,nqmax                                                          
      DO 50  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
      DO 50 I =1,IMJMR                                                          
      PHYDUM(I,1,L,NQ) = QT(I,1)                                                
   50 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( kphys.gt.nphys ) then                                                 
      DO 51 NQ=1,kphys-nphys                                                    
      DO 51  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
   51 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nphys.gt.kphys ) then                                                 
      DO 52 NQ=1,nphys-kphys                                                    
      DO 52  L=1,NLAYR                                                          
      DO 52  I=1,IMJMR                                                          
      PHYDUM(I,1,L,NQ+nqmax) = zero                                             
   52 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****        READ THE NET RADIATION (LW,SW) AT THE GROUND          ****        
C **********************************************************************        
                                                                                
      READ (LU)  QT                                                             
      DO 60 I =1,IMJMR                                                          
      RADLWG(I,1) = QT(I,1)                                                     
   60 CONTINUE                                                                  
      READ (LU)  QT                                                             
      DO 70 I =1,IMJMR                                                          
      RADSWG(I,1) = QT(I,1)                                                     
   70 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****                 READ THE ANALYSIS TENDENCIES                 ****        
C **********************************************************************        
                                                                                
      if( abs(ntype/10).eq.1 ) then
      PRINT *, 'Reading Analysis Tendencies'                         
      PRINT *                                                                   
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DPANAL(I,1) = QT(I,1)                                                     
      ENDDO                                                                     

      DO L = 1,NLAYR
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DUANAL(I,1,L) = QT(I,1)                                                     
      ENDDO                                                                     
      ENDDO                                                                     

      DO L = 1,NLAYR
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DVANAL(I,1,L) = QT(I,1)                                                     
      ENDDO                                                                     
      ENDDO                                                                     

      DO L = 1,NLAYR
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DTANAL(I,1,L) = QT(I,1)                                                     
      ENDDO                                                                     
      ENDDO                                                                     

      DO L = 1,NLAYR
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DQANAL(I,1,L) = QT(I,1)                                                     
      ENDDO                                                                     
      ENDDO                                                                     
      else
      PRINT *, 'No Analysis Tendencies on RESTART'                         
      PRINT *                                                                   
      endif
                                                                                
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                 WRITE THE RESTART HEADER                     ****        
C **********************************************************************        
                                                                                
      IF(IOFLAG.EQ. 2 .or. IOFLAG.EQ.-2) THEN                                   
      IF(IOFLAG.EQ. 2) MPHYS = NPHYS                                            
      IF(IOFLAG.EQ.-2) MPHYS = 0                                                
                                                                                
      ZPMEAN   = PIMEAN                                                         
      ZPTOP    = PTOP                                                           
      DO 205 L=1,NLAYR+1                                                        
      ZSIGE(L) = SIGE(L)                                                        
  205 CONTINUE                                                                  
                                                                                
      WRITE(LU) NYMD, NHMS, NB, NYMD0, NHMS0, IMR, JMR, NLAYR,                  
     .          NPROGS1, NPROGU1, NPROGS2, NPROGU2,                             
     .          MPHYS, NLWCLD, NSWCLD, JOB, NTYPE,         
     .          ZSIGE, ZPTOP, ZPMEAN, IRAS   
                                                                                
C **********************************************************************        
C ****      WRITE THE 1 TIME LEVEL SURFACE PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      DO 210 NQ=1,NPROGS1                                                       
      DO 215 I =1,IMJMR                                                         
      QT(I,1)  = PROGS1(I,1,NQ)                                                 
  215 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  210 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****    WRITE THE 1 TIME LEVEL UPPER-AIR PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      DO 220 NQ=1,NPROGU1                                                       
      DO 220  L=1,NLAYR                                                         
      DO 225 I =1,IMJMR                                                         
      QT(I,1)  = PROGU1(I,1,L,NQ)                                               
  225 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  220 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****      WRITE THE 2 TIME LEVEL SURFACE PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      DO 230 NT=1,2                                                             
      DO 230 NQ=1,NPROGS2                                                       
      DO 235 I =1,IMJMR                                                         
      QT(I,1)  = PROGS2(I,1,NQ,NT)                                              
  235 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  230 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****    WRITE THE 2 TIME LEVEL UPPER-AIR PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      DO 240 NT=1,2                                                             
      DO 240 NQ=1,NPROGU2                                                       
      DO 240  L=1,NLAYR                                                         
      DO 245 I =1,IMJMR                                                         
      QT(I,1)  = PROGU2(I,1,L,NQ,NT)                                            
  245 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  240 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****                 WRITE THE PHYSICS TENDENCIES                 ****        
C **********************************************************************        
                                                                                
      IF (IOFLAG.eq.2) then                                                     
      DO 250 NQ=1,NPHYS                                                         
      DO 250  L=1,NLAYR                                                         
      DO 255 I =1,IMJMR                                                         
      QT(I,1)  = PHYDUM(I,1,L,NQ)                                               
  255 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  250 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****       WRITE THE NET RADIATION (LW,SW) AT THE GROUND          ****        
C **********************************************************************        
                                                                                
      DO 260 I =1,IMJMR                                                         
      QT(I,1)  = RADLWG(I,1)                                                    
  260 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
      DO 270 I =1,IMJMR                                                         
      QT(I,1)  = RADSWG(I,1)                                                    
  270 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
                                                                                
C **********************************************************************        
C ****                WRITE THE ANALYSIS TENDENCIES                 ****        
C **********************************************************************        
                                                                                
      if( abs(ntype/10).eq.1 ) then
      DO I = 1,IMJMR                                                          
      QT(I,1) = DPANAL(I,1)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             

      DO L = 1,NLAYR
      DO I = 1,IMJMR                                                          
      QT(I,1) = DUANAL(I,1,L)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             
      ENDDO                                                                     

      DO L = 1,NLAYR
      DO I = 1,IMJMR                                                          
      QT(I,1) = DVANAL(I,1,L)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             
      ENDDO                                                                     

      DO L = 1,NLAYR
      DO I = 1,IMJMR                                                          
      QT(I,1) = DTANAL(I,1,L)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             
      ENDDO                                                                     

      DO L = 1,NLAYR
      DO I = 1,IMJMR                                                          
      QT(I,1) = DQANAL(I,1,L)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             
      ENDDO                                                                     
      endif
                                                                                
      ENDIF                                                                     
      RETURN                                                                    
      END                                                                       
