      SUBROUTINE SETMET                                                         
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     INITIALIZE RESOLUTION DEPENDENT TERMS IN MODEL                            
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
                                                                                
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <vmetric.com>                                                          
                                                                                
      PARAMETER ( IMJMR   = IMR*(JMR+1) )                                       
      PARAMETER ( IMJMM1R = IMR* JMR    )                                       
      PARAMETER ( IMJMM2R = IMR*(JMR-1) )                                       
      PARAMETER ( IMD2R   = IMR/2       )                                       
                                                                                
      PARAMETER ( ZERO   = 0.0 )                                                
      PARAMETER ( AHALF  = 0.5 )                                                
      PARAMETER ( ONE    = 1.0 )                                                
      PARAMETER ( TWO    = 2.0 )                                                
      PARAMETER ( FOUR   = 4.0 )                                                
      PARAMETER ( ROTSEC =   86164.09 )                                         
                                                                                
C *********************************************************                     
C ****            COMPUTE RESOLUTION FACTORS           ****                     
C *********************************************************                     
                                                                                
      IM     = IMR                                                              
      JNP    = JMR+1                                                            
      NLAY   = NLAYR                                                            
                                                                                
      IMH    = IM/2                                                             
      IMM1   = IM-1                                                             
      IMM2   = IM-2                                                             
      IMM3   = IM-3                                                             
      JMM1   = JNP-1                                                            
      JMM2   = JNP-2                                                            
      JMM3   = JNP-3                                                            
      JMM5   = JNP-5                                                            
      IMJM   = IM*JNP                                                           
      IMJMM1 = IM*JMM1                                                          
      IMJMM2 = IM*JMM2                                                          
      IMJMM3 = IM*JMM3                                                          
      NLAYP1 = NLAY+1                                                           
      NLAYM1 = NLAY-1                                                           
                                                                                
C *********************************************************                     
C ****            COMPUTE GRID INCREMENTS              ****                     
C *********************************************************                     
                                                                                
      A    = GETCON('EARTH RADIUS')                                             
      PI   = FOUR * ATAN(ONE)                                                   
      DL   = TWO*PI/IM                                                          
      DP   = PI/JMM1                                                            
      ACAP = TWO*PI *A**2 * (ONE-COS(AHALF*DP))                                 
                                                                                
C *********************************************************                     
C ****        COMPUTE TRIGONOMETRIC FACTORS            ****                     
C *********************************************************                     
                                                                                
      DO 10 I=1,IMD2R                                                           
      COSLON(I)     = -COS((I-1)*DL)                                            
      COSLON(I+IMH) = -COSLON(I)                                                
      SINLON(I)     = -SIN((I-1)*DL)                                            
      SINLON(I+IMH) = -SINLON(I)                                                
   10 CONTINUE                                                                  
                                                                                
      FJEQ = ( JNP+1 )*AHALF                                                    
      DO 20 J=2,JMM1                                                            
           PHI    = DP * (J - FJEQ)                                             
       COSP(J)    =     COS(PHI)                                                
       SINP(J)    =     SIN(PHI)                                                
      ACOSP(J)    = ONE/COSP(J)                                                 
      DO 20 I=1,IMR                                                             
       COSIJ(I,J) =     COSP(J)                                                 
       SINIJ(I,J) =     SINP(J)                                                 
      SINLIJ(I,J) =   SINLON(I)                                                 
      COSLIJ(I,J) =   COSLON(I)                                                 
   20 CONTINUE                                                                  
      DO 30 I=1,IMR                                                             
      SINLIJ(I,1  ) = SINLON(I)                                                 
      COSLIJ(I,1  ) = COSLON(I)                                                 
      SINLIJ(I,JNP) = SINLON(I)                                                 
      COSLIJ(I,JNP) = COSLON(I)                                                 
   30 CONTINUE                                                                  
                                                                                
       SGNP(1)      = -ONE                                                      
       SGNP(2)      =  ONE                                                      
       COSP(1)      = ZERO                                                      
       SINP(1)      = -ONE                                                      
       COSP(JNP)    = ZERO                                                      
       SINP(JNP)    =  ONE                                                      
      DO 40 I=1,IMR                                                             
       SINIJ(I,  1) = -ONE                                                      
       SINIJ(I,JNP) =  ONE                                                      
       COSIJ(I,  1) = ZERO                                                      
       COSIJ(I,JNP) = ZERO                                                      
      ACOSIJ(I,  1) = ZERO                                                      
      ACOSIJ(I,JNP) = ZERO                                                      
   40 CONTINUE                                                                  
                                                                                
                 OMEGA   = TWO*PI / ROTSEC                                      
      DO 50 I=1,IMJMR                                                           
         FIJ(I,1)   = TWO*OMEGA*SINIJ(I,1)                                      
   50 CONTINUE                                                                  
      DO 60 I=1,IMJMM2R                                                         
      ACOSIJ(I,2) = ONE/COSIJ(I,2)                                              
      ATANIJ(I,2) = SINIJ(I,2)*ACOSIJ(I,2)/A                                    
   60 CONTINUE                                                                  
                                                                                
C *********************************************************                     
C ****       COMPUTE VERTICAL GRID INCREMENTS          ****                     
C ****         (NOTE: SIGE IS FROM RESTART)            ****                     
C *********************************************************                     
                                                                                
      DO 70 L=1,NLAYR                                                           
      DSIG(L) =  SIGE(L+1) - SIGE(L)                                            
       SIG(L) = (SIGE(L+1) + SIGE(L))*AHALF                                     
   70 CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
