      SUBROUTINE SLPRS (SLP,PZ,PTOP,PHIS,THZ,DSIG,TM)                           
C***********************************************************************        
C                                                                               
C  INPUT                                                                        
C    PZ ...... SURFACE   PRESSURE (MB) - PTOP                                   
C    PTOP .... MODEL TOP PRESSURE (MB)                                          
C    PHIS .... SURFACE GEOPOTENTIAL (M2/S2)                                     
C    THZ ..... POTENTIAL TEMPERATURE (K) ON  SIGMA LEVELS                       
C    DSIG .... SIGMA INCREMENTS                                                 
C    TM ...... TEMPORARY SPACE                                                  
C                                                                               
C  OUTPUT                                                                       
C    SLP ..... SEA LEVEL PRESSURE (MB)                                          
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
                                                                                
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
                                                                                
      PARAMETER ( ZERO = 0.0   )                                                
      PARAMETER ( ONE  = 1.0   )                                                
      PARAMETER ( TWO  = 2.0   )                                                
      PARAMETER ( BETA = 0.0065)                                                
                                                                                
      DIMENSION  SLP (IMJMR),  PZ   (IMJMR)                                     
      DIMENSION  PHIS(IMJMR),  THZ  (IMJMR,NLAYR)                               
      DIMENSION  TM  (IMJMR)                                                    
      DIMENSION  DSIG(NLAYR)                                                    
                                                                                
      G  = GETCON('GRAVITY')                                                    
      R  = GETCON('RGAS')                                                       
      AK = GETCON('KAPPA')                                                      
                                                                                
      DO 10 I= 1,IMJMR                                                          
      TM(I)  = ZERO                                                             
   10 CONTINUE                                                                  
                                                                                
C***********************************************************************        
C*                COMPUTE MEAN THETA IN PBL (100 MB)                   *        
C***********************************************************************        
                                                                                
      DELP   = ZERO                                                             
      DELS   = ZERO                                                             
      DO 20 L=NLAYR,1,-1                                                        
      DELP   = DELP + (1000.-ptop)*DSIG(L)                                              
                                                                                
      IF (DELP.LE.100.) THEN                                                    
      DELS   = DELS + DSIG(L)                                                   
      DO 30 I= 1,IMJMR                                                          
      TM(I)  = TM(I) + THZ(I,L)*DSIG(L)                                         
   30 CONTINUE                                                                  
      ENDIF                                                                     
                                                                                
   20 CONTINUE                                                                  
      DO 40 I= 1,IMJMR                                                          
      TM(I)  = TM(I) / DELS                                                     
   40 CONTINUE                                                                  
                                                                                
C***********************************************************************        
C*                   COMPUTE SEA LEVEL PRESSURE                        *        
C***********************************************************************        
                                                                                
      DO 50 I=1,IMJMR                                                           
      TM (I) =   TM(I) * ( PZ(I)+PTOP )**AK + BETA * PHIS(I)/(TWO*G)            
      SLP(I) = PHIS(I) / ( R*TM(I) )                                            
      SLP(I) = ( PZ(I)+PTOP ) * EXP( SLP(I) )                                   
   50 CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
