      SUBROUTINE TSPBL (PZ,PTOP,THZ,DSIG,IM,JNP,NLAY,ts_new)
C***********************************************************************        
C
C  TSPBL: "New" surface temperature derived from mean PBL potential 
C         temperature
C                                                                               
C  INPUT                                                                        
C    PZ ...... SURFACE   PRESSURE (MB) - PTOP                                   
C    PTOP .... MODEL TOP PRESSURE (MB)                                          
C    THZ ..... POTENTIAL TEMPERATURE (K) ON  SIGMA LEVELS (T/P**KAPPA)
C    DSIG .... SIGMA INCREMENTS                                                 
C    IM ...... NUMBER OF LONGITUDINAL GRID POINTS
C    JNP ..... NUMBER OF LATITUDINAL GRID POINTS
C    NLAY .... NUMBER OF LEVELS
C                                                                               
C  OUTPUT                                                                       
C    ts_new .. Temperature (K) at surface pressure.  ts_new is derived
C              by computing the mean potential temperature in (approximately)
C              the lowest 100 mb of the atmosphere (this represents the PBL)
C              and calculating what temperature this mean potential 
C              temperature represents at the surface pressure. 
C
C  NOTE: TSPBL is a modification of Phoenix model subroutine SLPRS.
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
      dimension  pz     (im,jnp)
      dimension  thz    (im,jnp,nlay)
      dimension  ts_new (im,jnp)
      dimension  dsig   (nlay)
                                                                                
      ak = getcon('KAPPA')                                                      
 
      do i = 1,im*jnp
      ts_new(i,1) = 0.
      end do
                                                                                
C***********************************************************************        
C*                COMPUTE MEAN THETA IN PBL (100 MB)                   *        
C***********************************************************************        
                                                                                
      delp = 0.
      dels = 0.
 
      do L = nlay,1,-1                                                        
      delp = delp + (1000.-ptop)*dsig(L)
                                                                                
      if (delp.le.100.) then
          dels = dels + dsig(L)
          do i = 1,im*jnp
          ts_new(i,1) = ts_new(i,1) + thz(i,1,L)*dsig(L)
          end do
      endif 
                                                                                
      enddo
 
      do i = 1,im*jnp
      ts_new(i,1) = ts_new(i,1) / dels
      end do
                                                                                
C***********************************************************************        
C*                Compute new surface temperature                      *        
C***********************************************************************        
                                                                                
      do i = 1,im*jnp
      ts_new(i,1) = ts_new(i,1) * (pz(i,1)+ptop)**ak
      end do
                                                                                
      return
      end
