      SUBROUTINE polewnd (uz,vz,ustr,vstr,im,jm)
C***********************************************************************
C  PURPOSE     To compute fictitious winds at and near the pole
C  ARGUMENTS   DESCRIPTION
C     Uz ..... U-WIND on C-Grid
C     Vz ..... V-WIND on C-Grid
C     Ustr ... U-WIND on C-Grid at   pole
C     Vstr ... V-WIND on C-Grid near pole
C
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************


      PARAMETER ( ZERO  = 0.0     )
      PARAMETER ( AHALF = 0.5     )
      PARAMETER ( ONE   = 1.0     )
      PARAMETER ( TWO   = 2.0     )
      PARAMETER ( FOUR  = 4.0     )

      DIMENSION Uz   (im,jm)
      DIMENSION Vz   (im,jm)
      dimension ustr (im,jm)
      dimension vstr (im,jm)
 
c Local Dynamic Space
c -------------------
      dimension cosu (im)
      dimension cosv (im)
      dimension sinu (im)
      dimension sinv (im)
      dimension   vp (im)
       
      imh    = im/2
      imjmm2 = im*(jm-2)

      PI  = FOUR*ATAN(ONE)                                            
      DL  = TWO*PI/IM                                               

      do i=1,im
      cosu(i) = -cos( (i-1+.5)*dl )
      sinu(i) = -sin( (i-1+.5)*dl )
      enddo

      DO I=1,IMH
      COSV(I)     = -COS((I-1)*DL)
      COSV(I+IMH) = -COSV(I)
      SINV(I)     = -SIN((I-1)*DL)
      SINV(I+IMH) = -SINV(I)
      ENDDO

C **********************************************************************
C ****                  compute winds at the pole                   ****
C **********************************************************************
       
      a1 =  2./3.
      a2 = -1./6.
      a3 =  1./9.
       
      do i=1,imjmm2
      ustr(i,2) = uz(i,2)
      vstr(i,2) = vz(i,2)
      enddo
       
      do m=1,2
      ju1 = 1 + (m-1)*(jm-1)
      ju2 = 2 + (m-1)*(jm-3)
      ju3 = 3 + (m-1)*(jm-5)
      jv1 = 1 + (m-1)*(jm-2)
      jv2 = 2 + (m-1)*(jm-4)
       
      uc = 0.0
      us = 0.0
      do i=1,imh
      uc = uc + ( a1*(ustr(i,ju2)-ustr(i+imh,ju2))
     .          + a2*(ustr(i,ju3)-ustr(i+imh,ju3)) )*cosu(i)
      us = us + ( a1*(ustr(i,ju2)-ustr(i+imh,ju2))
     .          + a2*(ustr(i,ju3)-ustr(i+imh,ju3)) )*sinu(i)
      enddo
      do i=1,imh
      uc = uc + ( a1*(ustr(i+imh,ju2)-ustr(i,ju2))
     .          + a2*(ustr(i+imh,ju3)-ustr(i,ju3)) )*cosu(i+imh)
      us = us + ( a1*(ustr(i+imh,ju2)-ustr(i,ju2))
     .          + a2*(ustr(i+imh,ju3)-ustr(i,ju3)) )*sinu(i+imh)
      enddo
       
      uc = uc * 2./im
      us = us * 2./im
      vc =   (-1)**m * us
      vs = - (-1)**m * uc
      do i=1,im
      ustr(i,ju1) = uc*cosu(i) + us*sinu(i)
        vp(i)     = vc*cosv(i) + vs*sinv(i)
      enddo
      do i=1,imh
      vstr(i,jv1) = ( 8.*vp(i) + 2.*vstr(i,jv2) + vstr(i+imh,jv2) )*a3
      enddo
      do i=imh+1,im
      vstr(i,jv1) = ( 8.*vp(i) + 2.*vstr(i,jv2) + vstr(i-imh,jv2) )*a3
      enddo
      enddo
       
      RETURN
      END
