! getvars: Gets all requested variables from next Phoenix record for tograds.
!
! To be used with tograds version 3.
!
! Modification log:
!
! 940601 DJL - Some diag data sets may have a strange level ID for single level 
!              diagnostics.  Ignore any level ID for single level diagnostics.
!              (Requires patch to phxread also.)
! 950310 DJL - Ver 3.1.  Returns eof or error code if phxskip hits an
!              eof or error during skip.  Done primarily for ECMWF analysis 
!              tapes converted from GRIB that were missing last field at
!              end of every month.   This will let tograds continue with 
!              processing by either exiting gracefully or opening next input
!              file.  
! 950320 DJL - Does NOT return the eof code from phxskip.  If eof encountered
!              during skip, then result considered to be ok.  Meaning that 
!              the desired fields were obtained; only the remaining file
!              elements at the end were not right.  Otherwise data that were 
!              read on the last, short record (in the above ecmwf case) are 
!              not used.

      subroutine getvars(lu, nx, ny, nz, 
     .                 max_vars, max_fields,
     .                 nplevs, plevs, nvars, vars, 
     .                 im, jnp, nplevs_kept, plevs_kept, 
     .                 nvars_kept, kept_var, level_var, descr_var, q,
     .                 nrdate,nrtime,nrtype,rmiss,error_flag)

      real q(nx,ny,max_fields) 
      real plevs(nz)
      real plevs_kept(nz)

      integer level_var(max_vars)
      integer nvars_kept
      integer error_flag

      character*8 vars(max_vars)
      character*8 kept_var(max_vars)
      character*40 descr_var(max_vars)

      logical found
      logical first
C                                                                               
      REAL      UA_LEVELS(50)                                                   
                                                                                
      INTEGER   NUM_LEV_DIAG(100)                                               
                                                                                
      CHARACTER*8   SFC_NAME(100), UA_FIELD_NAME(100),                          
     &              DIAG_FIELD_NAME(100),                                       
     &              JOB                                                         
                                                                                
      CHARACTER*40  SFC_DESCR(100), UA_FIELD_DESCR(100),                        
     &              DIAG_FIELD_DESCR(100)                                       
      CHARACTER*80  EXPERIMENT_DESCR                                            

      CALL PHXHDR(lu, IDATE, ITIME, LOG8R, IM, JNP,                     
     &   NUM_UA_LEVELS, UA_LEVELS, JOB, EXPERIMENT_DESCR,                    
     &   NUM_SFC_FIELDS, NUM_UA_FIELDS, NUM_DIAG_FIELDS,                     
     &   SFC_NAME, SFC_DESCR,                                                
     &   UA_FIELD_NAME, UA_FIELD_DESCR,                                      
     &   DIAG_FIELD_NAME, DIAG_FIELD_DESCR, NUM_LEV_DIAG,
     &   rmiss,  ierr)

      print *, ' ierr from phxhdr = ', ierr

      if (ierr.eq.0) then
                                                                                
! Header read successfully; extract desired quantities

         IPXREC=1                                                               

         PRINT 6025, JOB,IDATE,ITIME,LOG8R,IM,JNP,NUM_UA_LEVELS                 
 6025    FORMAT(' --> PHOENIX HEADER READ FOR ',A8,6I9)                         


      nrdate=idate
      nrtime=itime
      nrtype=log8r

      if (nplevs.le.0) then
         nplevs_kept=NUM_UA_LEVELS
         do l=1,nplevs_kept
            plevs_kept(l)=UA_LEVELS(l)
         end do
      else
         nplevs_kept=nplevs
         do l=1,nplevs_kept
            plevs_kept(l)=plevs(l)
         end do
      end if

C Read desired quantities and levels from Phoenix dataset

      nvars_kept=0

C Load any surface fields

      do k=1,NUM_SFC_FIELDS

         found=.false.
         do ivar=1,nvars
            if (vars(ivar).eq.sfc_name(k)) then
               found=.true.
               kvar=ivar
               print 6001, k, sfc_name(k)
 6001          format(' will read sfc field ',i4,2x,a8)
            end if 
         end do

         if (found) then 
            nvars_kept=nvars_kept+1
            kept_var(nvars_kept)=vars(kvar)
            level_var(nvars_kept)=0
            descr_var(nvars_kept)=SFC_DESCR(k)
            CALL PHXREAD(lu,vars(kvar),0.,
     .                  q(1,1,nvars_kept),im,jnp,ipxrec,ierr)           
            print 6002, vars(kvar), nvars_kept
 6002       format(' read sfc field ',a8,' nvars_kept =',i5)
         end if 

      end do

      nvars_sfc=nvars_kept

C Load any upper air fields

      l=1

      do k=1,NUM_UA_FIELDS

         found=.false.
         do ivar=1,nvars
            if (vars(ivar).eq.UA_FIELD_NAME(k)) then
               found=.true.
               kvar=ivar
               print 6011, k, ua_field_name(k)
 6011          format(' will read upper air field ',i4,2x,a8)
            end if 
         end do

         if (found) then 
            nvars_kept=nvars_kept+1
            kept_var(nvars_kept)=vars(kvar)
            level_var(nvars_kept)=nplevs_kept
            descr_var(nvars_kept)=UA_FIELD_DESCR(k)
            ifield=nvars_sfc+1+nplevs_kept*(nvars_kept-nvars_sfc-1)
            CALL PHXREAD(lu,vars(kvar),plevs_kept(l),
     .                   q(1,1,ifield),im,jnp,ipxrec,ierr)           
            print 6012, vars(kvar), l, plevs_kept(l), nvars_kept
 6012       format(' read upper air field ',a8,' at level ',i3,
     .             ' (',f6.1,' mb)  nvars_kept =',i5)
         end if 

      end do 

      do l=2,nplevs_kept

         do k=nvars_sfc+1,nvars_kept
            ifield=nvars_sfc+l+nplevs_kept*(k-nvars_sfc-1)
            CALL PHXREAD(lu,kept_var(k),plevs_kept(l),
     .                   q(1,1,ifield),im,jnp,ipxrec,ierr)           
            print 6012, kept_var(k), l, plevs_kept(l), nvars_kept
         end do

      end do

C Load any diagnostic fields 

      ifield=nvars_sfc+nplevs_kept*(nvars_kept-nvars_sfc)

      do k=1,NUM_DIAG_FIELDS

         found=.false.
         do ivar=1,nvars
            if (vars(ivar).eq.DIAG_FIELD_NAME(k)) then
               found=.true.
               kvar=ivar
               print 6021, k, DIAG_FIELD_NAME(k)
 6021          format(' will read diagnostic field ',i4,2x,a8)
            end if 
         end do

         if (found) then 
            nvars_kept=nvars_kept+1
            kept_var(nvars_kept)=vars(kvar)
            descr_var(nvars_kept)=DIAG_FIELD_DESCR(k)
            if (NUM_LEV_DIAG(k).gt.1) then
               level_var(nvars_kept)=min(nplevs_kept,NUM_LEV_DIAG(k))
               do l=1,level_var(nvars_kept)
                  ifield=ifield+1
                  CALL PHXREAD(lu,vars(kvar),plevs_kept(l),
     .                         q(1,1,ifield),im,jnp,ipxrec,ierr)           
                  print 6022, vars(kvar), l, plevs_kept(l), nvars_kept
 6022             format(' read diagnostic field ',a8,' at level ',
     .                   i3,' (',f6.1,' mb)  nvars_kept =',i5)
               end do
            else
! Ignore level ID for any single level diagnostics.  PHXREAD with level ID
! set to 0 will return next field with that variable ID, regardless of level ID.
               level_var(nvars_kept)=1
               ifield=ifield+1
               CALL PHXREAD(lu,vars(kvar),0.0,
     .                      q(1,1,ifield),im,jnp,ipxrec,ierr)           
               print 6022, vars(kvar), 1, 0.0, nvars_kept
            end if
         end if 

      end do 


C POSITION INPUT PHOENIX FILE TO START OF NEXT HEADER                           
 
      CALL PHXSKIP(lu,IPXREC,NUM_SFC_FIELDS,
     .             NUM_UA_FIELDS, NUM_UA_LEVELS,
     .             NUM_DIAG_FIELDS,NUM_LEV_DIAG,ierr_skip)

      if (ierr_skip.eq.0.or.ierr_skip.eq.1) then
         error_flag=0
      else
         error_flag=ierr_skip
      end if

! EOF or error on input file

      else

         error_flag=ierr

      end if

      end
