      subroutine openif( file_type, input_path, file_name, lu, lu_list, 
     .                                                     error_flag )

! Error exit codes (error_flag):
!
!    0: Input file opened successfully
!    1: No more input files to open (essentially an EOF on input stream)
!    2: Unrecognized file_type
!    3: Unable to bring file from remote location

      integer error_flag
      integer read_status
      integer getif_error

      character*80 input_path
      character*80 input_path_use
      character*80 full_path
      character*80 remaddr
      character*20 remhost
      character*20 remid
      character*80 full_file_name
      character*80 list_file_name
      character*80 in_file_name
      character*80 last_file_name
      character*(*) file_type
      character*(*) file_name
      character*80 string_temp

      logical found
      logical first

      integer trulen

      save first
      save last_file_name

      data first /.true./

! file_type: 'name'  = use string in "file_name" as input file
!            'list'  = use string in "file_name" as name of file containing
!                      a list of input files
!            'oneday'= input file names determined from usual convention for
!                      scattered one day files:
!                         on ibm:     'f4jwp.e{expno}.{type}.t{date} 
!                         on unitree: 'e{expno}.{type}.t{date}
! input_path: Path leading to input files.  If any of the input files, either
!             the name of the input file, the name of the list of input files
!             or the names of input files in the list, do not begin with a "/"
!             or a ".", then the program will prepend the input_path to get a 
!             full path name.  If any of the input file names begin with "/"
!             or ".", then the name is assumed to already be a full or relative
!             path name.  

! Look for remote host information in input_path

      if ( index(input_path,':') .gt. 0 ) then
         call parse_str( input_path, remaddr, input_path_use, ':', 0 )
         call parse_str( remaddr, remid, remhost, '@', 0 )
      else
         remid=' '
         remhost=' '
         input_path_use=input_path
      end if

C Get number of characters in path name

      ncpath=trulen(input_path_use)

! ----------------------
! Open initial data file
! ----------------------
 
      if ( first ) then

! -------------------------------------
! File name given is the data file name
! -------------------------------------

         if (file_type.eq.'name') then

! Set full path of data file

            if ( file_name(1:1).eq.'/' .or. file_name(1:1).eq.'.' .or.
     .                                              ncpath.eq.0 ) then
               full_file_name=file_name
            else 
               full_file_name=input_path_use(1:ncpath)//'/'//file_name
            end if

! Get data file from remote location, if necessary

            if ( remhost .ne. ' ' ) then 

               call getif( remid, remhost, full_file_name, getif_error )

               if ( getif_error .ne. 0 ) then
                  print 7012, remid,remhost,full_file_name,getif_error
 7012             format('*** Error getting file: ',a20,a20,a80,
     .                                            ' error code = ', i10)
                  error_flag=3
                  return
               end if

! File has been put in current directory, so reset full_file_name to reflect that

               call parse_str( full_file_name, full_path, 
     .                                             string_temp, '/', 1 )
               full_file_name=string_temp

            end if

! Open data file

            open(iabs(lu),file=full_file_name,form='unformatted')
            print 7001, lu,full_file_name
 7001       format(' input file opened on unit ',i2,': ',a80)
            error_flag=0

            last_file_name=full_file_name

! ---------------------------------------------------------------
! File name given is the list file containing names of data files
! ---------------------------------------------------------------

         else if (file_type.eq.'list') then

! Set full path of file containing lists

            if ( file_name(1:1).eq.'/' .or. file_name(1:1).eq.'.' .or.
     .                                              ncpath.eq.0 ) then
               list_file_name=file_name
            else 
               list_file_name=input_path_use(1:ncpath)//'/'//file_name
            end if

! Open list file.

            open(lu_list,file=list_file_name,form='formatted')

! Read first data file name from list file. 

            read(lu_list,9001) in_file_name
 9001       format(a80)

! Set full path of data file.

            if ( in_file_name(1:1).eq.'/' .or.
     .           in_file_name(1:1).eq.'.' .or.
     .                             ncpath.eq.0 ) then
               full_file_name=in_file_name
            else 
               full_file_name=input_path_use(1:ncpath)//'/'//
     .                                                   in_file_name
            end if

! Get data file from remote location, if necessary

            if ( remhost .ne. ' ' ) then 

               call getif( remid, remhost, full_file_name, getif_error )

               if ( getif_error .ne. 0 ) then
                  print 7012, remid,remhost,full_file_name,getif_error
                  error_flag=3
                  return
               end if

! File has been put in current directory, so reset full_file_name to reflect that

               call parse_str( full_file_name, full_path, 
     .                                             string_temp, '/', 1 )
               full_file_name=string_temp

            end if

! Open data file

            open(iabs(lu),file=full_file_name,form='unformatted')
            print 7001, lu,full_file_name
            error_flag=0

            last_file_name=full_file_name

         else

            print 7002, file_type
 7002       format('*** Unrecognized file_type: ',a16,' ***')
            error_flag=2

         end if

         first=.false.

! --------------------------
! Else open subsequent files
! --------------------------

      else

! If only one input file, attempt to open a second file is an eof condition

         if (file_type.eq.'name') then

            error_flag=1

! If list of input files, open next file in list 

         else if (file_type.eq.'list') then

! Read next data file name from list file.

            read(lu_list,9001,iostat=read_status) in_file_name

            if (read_status.eq.0) then

! Close previous file and if from a remote location, delete it.

               close(lu)
               if ( remhost .ne. ' ' ) call rmif( last_file_name )

! Set full path of data file.

               if ( in_file_name(1:1).eq.'/' .or.
     .              in_file_name(1:1).eq.'.' .or. 
     .                                ncpath.eq.0 ) then
                  full_file_name=in_file_name
               else 
                  full_file_name=input_path_use(1:ncpath)//'/'//
     .                                               in_file_name
               end if

! Get data file from remote location, if necessary

               if ( remhost .ne. ' ' ) then 

                  call getif( remid, remhost, full_file_name, 
     .                                                     getif_error )

                  if ( getif_error .ne. 0 ) then
                     print 7012, remid,remhost,full_file_name,
     .                                                     getif_error
                     error_flag=3
                     return
                  end if

! File has been put in current directory, so reset full_file_name to reflect that

                  call parse_str( full_file_name, full_path, 
     .                                             string_temp, '/', 1 )
                  full_file_name=string_temp

               end if

! Open new data file

               open(iabs(lu),file=full_file_name,form='unformatted')
               print 7001, lu,full_file_name
               error_flag=0

               last_file_name=full_file_name

! EOF on list of input files.  Return as for eof on input file.
! If files have been obtained from a remote location, delete the last file.
! Also reset "first" to true and close list file unit so that subsequent 
! calls will reopen the list file and start at the beginning.

            else   

               error_flag=1
               close(lu)
               if ( remhost .ne. ' ' ) call rmif( last_file_name )
               close(lu_list)
               first=.true.

            end if
               

         else

            print 7002, file_type
            error_flag=2

         end if

      end if   ! first/not first test

      end
