! tograds: General conversion program from Phoenix format to "GrADS" format.
! 
! Will accept a list of input Phoenix files, and extract specified fields 
! and levels to be converted to a file readable by the GrADS visualization
! tool.  
!
! Version 3.3
!
! Modification log:
!
! v3.0 DJL 940519:
!    Removed code that would get next one day file.
!    Added option to write a separate grads file for each variable.
!    Included function trulen to get actual length of string.
!    Created subroutine openif to open phoenix files.
!    Created subroutine dd_write to write dd files (table files).  The
!       dd files are written to the same unit.
!    Increased maximum number of fields to 250.
! v3.1 DJL 940519:
!    Check for repeated phoenix record and skip (same date, time, log8r).
! v3.2 DJL 940708:
!    Convert variable names to lower case using "tolower".
! v3.3 DJL 941025:
!    Handle -999 record types as 0.

      program tograds

      parameter ( nx =  144, ny =  91, nz = 46 )
      parameter (max_vars = 20)
      parameter (max_fields = 250) 

      real q(nx,ny,max_fields) 
      real plevs(nz)
      real plevs_kept(nz)
      integer begdat, enddat, begtim, endtim
      integer level_var(max_vars)
      integer nvars_kept
      integer ntimes(-1:1,max_vars)
      integer grads_irec(-1:1,max_vars)
      integer grads_unit(-1:1,max_vars)
      integer dd_unit
      integer dd_unit_temp
      integer first_date
      integer first_time
      integer getvars_error
      integer openif_error
      integer trulen
      integer expected_nrtype(3), num_nrtypes, idx_nrtype
      character*8 exp_number
      character*8 vars(max_vars)
      character*8 kept_var(max_vars)
      character*40 title, title2
      character*80 grads_dsn(-1:1,max_vars)
      character*80 grads_dsn_in_dd(-1:1,max_vars)
      character*80 dd_dsn(-1:1,max_vars)
      character*80 grads_path, input_path, output_path
      character*80 grads_file_tag
      character*3 cmonth(12)
      character*2 chh, cuu
      character*6 cymd
      character*40 descr_var(max_vars)
      character*8 file_type
      character*80 file_name
      logical timelt, timele, timeeq
      logical grads_open(-1:1,max_vars)
      logical pending
      logical split_by_var
      logical split_by_day
      logical dd_exist

      data nplevs /0/
      data nvars /4/
      data vars /'SLP     ','UWND    ','VWND    ','HGHT    ',
     &           16*'        '/
      data inchms /060000/
      data exp_number /' '/
      data title /' '/
      data dsn /' '/
      data cmonth /'JAN','FEB','MAR','APR','MAY','JUN',
     .             'JUL','AUG','SEP','OCT','NOV','DEC'/
      data output_path /'.'/
      data grads_path /' '/
      data input_path /'.'/
      data grads_file_tag /' '/
      data dd_unit    /30/
      data dd_unit_temp /29/
      data first_date /-999/
      data first_time /-999/
      data split_by_var /.false./
      data split_by_day /.false./
      data expected_nrtype /-999999,-999999,-999999/
      data idx_nrtype /1/

      data lu /10/
      data lu_list /9/

! file_type: 'name'  = use string in "file_name" as input file
!            'list'  = use string in "file_name" as name of file containing
!                      a list of input files
!            'oneday'= input file names determined from usual convention for
!                      scattered one day files:
!                         on ibm:     'f4jwp.e{expno}.{type}.t{date}
!                         on unitree: 'e{expno}.{type}.t{date}
! input_path: Path leading to input files.  If any of the input files, either
!             the name of the input file, the name of the list of input files
!             or the names of input files in the list, do not begin with a "/"
!             or a ".", then the program will prepend the input_path to get a
!             full path name.  If any of the input file names begin with "/"
!             or ".", then the name is assumed to already be a full or relative
!             path name.
      
      namelist /user_variables/ exp_number, begdat, begtim, 
     &                          inchms, 
     &                          enddat, endtim, itype,
     &                          nplevs, plevs, nvars, vars, 
     &                          file_type, file_name, input_path,
     &                          output_path, grads_path, grads_file_tag,
     &                          split_by_var, expected_nrtype, 
     &                          split_by_day

#ifdef RS6000
      call setrteopts('namelist=old')
#endif
      
      open(15,file='tograds.n',form='formatted')
      read (15,user_variables)
      write(6,user_variables)

! Initialize grads output units

      kunit=31
      do k=1,max_vars
      do log8r=-1,1
         grads_unit(log8r,k)=kunit
         kunit=kunit+1
      end do
      end do

! Set default namelist values for quantities not specified

      if (grads_path.eq.' ') grads_path=output_path

      if (grads_file_tag.eq.' ') grads_file_tag=exp_number

      do k=1,max_vars
      do log8r=-1,1
         ntimes(log8r,k)=0
      end do
      end do

      do k=1,max_vars
      do log8r=-1,1
         grads_open(log8r,k)=.false.
      end do
      end do

      nrdate=-999999
      nrtime=-999999
      nrtype=-999999

! See if any user defined nrtypes are set

      i=0
      do while ( i.lt.3 .and. expected_nrtype(i+1).ne.-999999 )
         i=i+1
      end do
      num_nrtypes=i

C Get number of characters in Grads file tag and path names

      ncgtag =trulen(grads_file_tag)
      ncgpath=trulen(grads_path)
      ncopath=trulen(output_path)

! Open first Phoenix input file

      call openif( file_type, input_path, file_name, lu, lu_list,
     .                                                  openif_error )


! Check for unsuccessful open.  EOF or error here is a problem.

      if (openif_error.ne.0) then
         print 6501, openif_error
 6501    format('Error return from first openif: return code =',i3)
         goto 600
      end if 


! Begin looping over all desired times

      pending=.false.

      idate=begdat
      itime=begtim


      do while (timele(idate,itime,enddat,endtim))


   10 continue

      if (.not.pending) then

! Input all requested variables from next record.

         nrdate_prev=nrdate
         nrtime_prev=nrtime
         nrtype_prev=nrtype

         call getvars(10, nx, ny, nz, 
     .                max_vars, max_fields,
     .                nplevs, plevs, nvars, vars, 
     .                im, jnp, nplevs_kept, plevs_kept, 
     .                nvars_kept, kept_var, level_var, descr_var, q,
     .                nrdate,nrtime,nrtype,rmiss,getvars_error)

! If EOF from getvars, try opening next file

         if (getvars_error.eq.1) then

            call openif( file_type, input_path, file_name, lu, lu_list,
     .                                                   openif_error )

! If successful open, read next record

            if (openif_error.eq.0) then
               goto 10
            else 

! If EOF, then break while loop
               if (openif_error.eq.1) then
                  goto 500
! Else error
               else
                  print 6503, openif_error
 6503             format('Error return from openif: return code =',i3)
                  goto 600
               end if 

            end if

         end if

! Other error from getvars 

         if (getvars_error.ne.0) then
            print 6502, getvars_error
 6502       format(' Error return from getvars: return code =',i3)
            goto 500
         end if

! For record type of -999, set to 0 unless namelist indicates that there
! are multiple type records at each time.

         if (nrtype.eq.-999) then
            if (num_nrtypes.gt.0) then
               nrtype=expected_nrtype(idx_nrtype)
               idx_nrtype=idx_nrtype+1
               if (idx_nrtype.gt.num_nrtypes) idx_nrtype=1
            else
               nrtype=0
            endif
         endif

! Record with same date, time and nrtype record ... skip 

         if (nrdate.eq.nrdate_prev.and.nrtime.eq.nrtime_prev.and.
     .                                 nrtype.eq.nrtype_prev) then
            print 6510, nrdate,nrtime,nrtype
 6510       format(/,' Duplicate record at date:',i7,' time:',i7,
     .                          ' type:',i4,' - will be SKIPPED',/)
            goto 10
         end if

      end if

      pending=.false.

! Date/time earlier than desired ... read again
      if (timelt(nrdate,nrtime,idate,itime)) then 
         print 6511, nrdate,nrtime,nrtype
 6511    format(/,' Skipping record at date:',i7,' time:',i7,
     .                                           ' type:',i4,/)
         goto 10

! Date/time equal ... check nrtype flag
      else if (timeeq(nrdate,nrtime,idate,itime)) then 

! Proper nrtype flag ... process data
         if (itype.le.1.and.nrtype.eq.itype.or.itype.eq.2) then
            goto 20          

! Not nrtype flag we want ... read again
         else 
            print 6511, nrdate,nrtime,nrtype
            goto 10
         end if

! Date/time is now greater than desired ... update idate,itime
! and compare again
      else
         pending=.true.
         goto 400
      end if
  

   20 continue


      print 6521, nrdate,nrtime,nrtype
 6521 format(/,' Accepting record at date:',i7,' time:',i7,
     .                                         ' type:',i4,/)

! Change variable names to lower case.

      do k=1,nvars_kept
         call tolower(kept_var(k))
      end do

! Set up file names when first acceptable date encountered.  Also
! save first date and time for input into dd file.

      if (first_date.le.0) then 

         first_date=nrdate
         first_time=nrtime

         ihh=first_time/10000
         iuu=mod(first_time/100,100)
         write(chh,'(i2.2)') ihh
         write(cuu,'(i2.2)') iuu
         write(cymd,'(i6.6)') first_date

         if (.not.split_by_var) then

            if (split_by_day) then

               grads_dsn(-1,1)=output_path(1:ncopath)//'/f.'//
     .               grads_file_tag(1:ncgtag)
               grads_dsn( 0,1)=output_path(1:ncopath)//'/a.'//
     .               grads_file_tag(1:ncgtag)
               grads_dsn( 1,1)=output_path(1:ncopath)//'/i.'//
     .               grads_file_tag(1:ncgtag)

               if ( grads_path(1:1) .eq. '^' ) then

                  grads_dsn_in_dd(-1,1)=
     .                  '^f.'//grads_file_tag(1:ncgtag)//'.%y2%m2%d2'
                  grads_dsn_in_dd( 0,1)=
     .                  '^a.'//grads_file_tag(1:ncgtag)//'.%y2%m2%d2'
                  grads_dsn_in_dd( 1,1)=
     .                  '^i.'//grads_file_tag(1:ncgtag)//'.%y2%m2%d2'

               else

                  grads_dsn_in_dd(-1,1)=grads_path(1:ncgpath)//'/f.'//
     .                  grads_file_tag(1:ncgtag)//'.%y2%m2%d2'
                  grads_dsn_in_dd( 0,1)=grads_path(1:ncgpath)//'/a.'//
     .                  grads_file_tag(1:ncgtag)//'.%y2%m2%d2'
                  grads_dsn_in_dd( 1,1)=grads_path(1:ncgpath)//'/i.'//
     .                  grads_file_tag(1:ncgtag)//'.%y2%m2%d2'

               end if

               dd_dsn(-1,1)=output_path(1:ncopath)//'/tf.'//
     .               grads_file_tag(1:ncgtag)
               dd_dsn( 0,1)=output_path(1:ncopath)//'/ta.'//
     .               grads_file_tag(1:ncgtag)
               dd_dsn( 1,1)=output_path(1:ncopath)//'/ti.'//
     .               grads_file_tag(1:ncgtag)

            else

               grads_dsn(-1,1)=output_path(1:ncopath)//'/f.'//
     .               grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh
               grads_dsn( 0,1)=output_path(1:ncopath)//'/a.'//
     .               grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh
               grads_dsn( 1,1)=output_path(1:ncopath)//'/i.'//
     .               grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh

               if ( grads_path(1:1) .eq. '^' ) then

                  grads_dsn_in_dd(-1,1)=
     .              '^f.'//grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh
                  grads_dsn_in_dd( 0,1)=
     .              '^a.'//grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh
                  grads_dsn_in_dd( 1,1)=
     .              '^i.'//grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh

               else

                  grads_dsn_in_dd(-1,1)=grads_path(1:ncgpath)//'/f.'//
     .                  grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh
                  grads_dsn_in_dd( 0,1)=grads_path(1:ncgpath)//'/a.'//
     .                  grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh
                  grads_dsn_in_dd( 1,1)=grads_path(1:ncgpath)//'/i.'//
     .                  grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh

               end if

               dd_dsn(-1,1)=output_path(1:ncopath)//'/tf.'//
     .               grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh
               dd_dsn( 0,1)=output_path(1:ncopath)//'/ta.'//
     .               grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh
               dd_dsn( 1,1)=output_path(1:ncopath)//'/ti.'//
     .               grads_file_tag(1:ncgtag)//'.'//cymd//'.'//chh

            end if

         else

            do k=1,nvars_kept

               ncvar=trulen(kept_var(k))

               grads_dsn(-1,k)=output_path(1:ncopath)//'/f.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)
               grads_dsn( 0,k)=output_path(1:ncopath)//'/a.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)
               grads_dsn( 1,k)=output_path(1:ncopath)//'/i.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)

               if (split_by_day) then

               if ( grads_path(1:1) .eq. '^' ) then

                  grads_dsn_in_dd(-1,k)='^f.'//
     .             kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)//
     .             '.%y2%m2%d2'
                  grads_dsn_in_dd( 0,k)='^a.'//
     .             kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)//
     .             '.%y2%m2%d2'
                  grads_dsn_in_dd( 1,k)='^i.'//
     .             kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)//
     .             '.%y2%m2%d2'

               else

                  grads_dsn_in_dd(-1,k)=grads_path(1:ncgpath)//'/f.'//
     .             kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)//
     .             '.%y2%m2%d2'
                  grads_dsn_in_dd( 0,k)=grads_path(1:ncgpath)//'/a.'//
     .             kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)//
     .             '.%y2%m2%d2'
                  grads_dsn_in_dd( 1,k)=grads_path(1:ncgpath)//'/i.'//
     .             kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)//
     .             '.%y2%m2%d2'

               end if

               else

               if ( grads_path(1:1) .eq. '^' ) then

                  grads_dsn_in_dd(-1,k)='^f.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)
                  grads_dsn_in_dd( 0,k)='^a.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)
                  grads_dsn_in_dd( 1,k)='^i.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)

               else

                  grads_dsn_in_dd(-1,k)=grads_path(1:ncgpath)//'/f.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)
                  grads_dsn_in_dd( 0,k)=grads_path(1:ncgpath)//'/a.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)
                  grads_dsn_in_dd( 1,k)=grads_path(1:ncgpath)//'/i.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)

               end if

               end if
            
               dd_dsn(-1,k)=output_path(1:ncopath)//'/tf.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)
               dd_dsn( 0,k)=output_path(1:ncopath)//'/ta.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)
               dd_dsn( 1,k)=output_path(1:ncopath)//'/ti.'//
     .               kept_var(k)(1:ncvar)//'.'//grads_file_tag(1:ncgtag)

            end do

         end if

      end if

! Write fields in combined output form

      if (.not.split_by_var) then

      ntimes(nrtype,1)=ntimes(nrtype,1)+1

! Open GrADS data file

      if (.not.grads_open(nrtype,1)) then
         if ( split_by_day ) then 
            write(cymd,'(i6.6)') nrdate
            ncgdsn=trulen(grads_dsn(nrtype,1))
            call guopen(grads_unit(nrtype,1),
     .                  grads_dsn(nrtype,1)(1:ncgdsn)//'.'//cymd,im,jnp)
         else
            call guopen(grads_unit(nrtype,1),grads_dsn(nrtype,1),im,jnp)
         end if
         grads_open(nrtype,1)=.true.
         grads_irec(nrtype,1)=1
      end if 

! If splitting by day and new day encountered, close old file and open new one.
! Must reset record counter.

      if ( split_by_day .and. nrtime.eq.0 ) then
         close(grads_unit(nrtype,1))
         write(cymd,'(i6.6)') nrdate
         ncgdsn=trulen(grads_dsn(nrtype,1))
         call guopen(grads_unit(nrtype,1),
     .               grads_dsn(nrtype,1)(1:ncgdsn)//'.'//cymd,im,jnp)
         grads_irec(nrtype,1)=1
      end if

! Write fields in GrADS format.  These fields are written with no control
! words.  On Cray, the -F null option is used on the assign, on SGI, direct 
! access is used.

      nfields=0
      do k=1,nvars_kept
         if (level_var(k).eq.0) then
            nfields=nfields+1
         else
            nfields=nfields+level_var(k)
         end if
      end do
  
      do l=1,nfields
         call guwrite(grads_unit(nrtype,1),grads_irec(nrtype,1),
     .                                             q(1,1,l),im,jnp)
         grads_irec(nrtype,1)=grads_irec(nrtype,1)+1
      end do

! Write fields in split by variable form.  Each variable is written to
! a different output stream.

      else

      ifield=1

      do k=1,nvars_kept

      ntimes(nrtype,k)=ntimes(nrtype,k)+1

! Open GrADS data file

      if (.not.grads_open(nrtype,k)) then
         if ( split_by_day ) then 
            write(cymd,'(i6.6)') nrdate
            ncgdsn=trulen(grads_dsn(nrtype,k))
            call guopen(grads_unit(nrtype,k),
     .                  grads_dsn(nrtype,k)(1:ncgdsn)//'.'//cymd,im,jnp)
         else
            call guopen(grads_unit(nrtype,k),grads_dsn(nrtype,k),im,jnp)
         end if
         grads_open(nrtype,k)=.true.
         grads_irec(nrtype,k)=1
      end if 

! If splitting by day and new day encountered, close old file and open new one.
! Must reset record counter.

      if ( split_by_day .and. nrtime.eq.0 ) then
         close(grads_unit(nrtype,k))
         write(cymd,'(i6.6)') nrdate
         ncgdsn=trulen(grads_dsn(nrtype,k))
         call guopen(grads_unit(nrtype,k),
     .               grads_dsn(nrtype,k)(1:ncgdsn)//'.'//cymd,im,jnp)
         grads_irec(nrtype,k)=1
      end if

! Write fields in GrADS format.  These fields are written with no control
! words.  On Cray, the -F null option is used on the assign, on SGI, direct 
! access is used.

      if (level_var(k).eq.0) then
         nfields=1
      else
         nfields=level_var(k)
      end if

      do l=1,nfields
         call guwrite(grads_unit(nrtype,k),grads_irec(nrtype,k),
     .                                          q(1,1,ifield),im,jnp)
         grads_irec(nrtype,k)=grads_irec(nrtype,k)+1
         ifield=ifield+1
      end do

      end do
  
      end if

  400 continue

! Increment date and time if only one record type being read (itype < 2) 
! or if processing is pending on a record (in which case latest record is
! at a time greater than idate, itime).

      if (itype.lt.2.or.pending) then
         call inctim(idate,itime,inchms)
      end if


      end do ! End while loop on date/time


  500 continue


! Write data descriptor file 

      if (.not.split_by_var) then

         do log8r=-1,1
         if (ntimes(log8r,1).gt.0) then

            inquire(file=dd_dsn(log8r,1),exist=dd_exist)

            open(dd_unit,file=dd_dsn(log8r,1),form='formatted')

            if (title.eq.' ') then
               if (log8r.eq.-1) 
     .                  title2=grads_file_tag(1:ncgtag)//' first guess'
               if (log8r.eq. 0) 
     .                  title2=grads_file_tag(1:ncgtag)//' analysis'
               if (log8r.eq. 1) 
     .                  title2=grads_file_tag(1:ncgtag)//' initialized'
            else
               title2=title
            end if

            if ( split_by_day .and. dd_exist ) then

               call dd_write_cont(dd_unit_temp, dd_dsn(log8r,1), 
     .                    dd_unit, grads_dsn_in_dd(log8r,1), title2, 
     .                    rmiss, im, jnp, nplevs_kept, plevs_kept,
     .                    first_date, first_time, inchms, 
     .                    ntimes(log8r,1),
     .                    nvars_kept, kept_var, level_var, descr_var,
     .                    split_by_day)

            else

               call dd_write(dd_unit, grads_dsn_in_dd(log8r,1), title2,
     .                    rmiss, im, jnp, nplevs_kept, plevs_kept,
     .                    first_date, first_time, inchms, 
     .                    ntimes(log8r,1),
     .                    nvars_kept, kept_var, level_var, descr_var,
     .                    split_by_day)

            end if

            close(dd_unit)

         end if
         end do

      else

         do k=1,nvars_kept
         do log8r=-1,1
         if (ntimes(log8r,k).gt.0) then

            inquire(file=dd_dsn(log8r,k),exist=dd_exist)

            open(dd_unit,file=dd_dsn(log8r,k),form='formatted')

            if (title.eq.' ') then
               if (log8r.eq.-1) 
     .                  title2=grads_file_tag(1:ncgtag)//' first guess'
               if (log8r.eq. 0) 
     .                  title2=grads_file_tag(1:ncgtag)//' analysis'
               if (log8r.eq. 1) 
     .                  title2=grads_file_tag(1:ncgtag)//' initialized'
            else
               title2=title
            end if

            if ( split_by_day .and. dd_exist ) then

               call dd_write_cont(dd_unit_temp, dd_dsn(log8r,k),
     .                    dd_unit, grads_dsn_in_dd(log8r,k), title2,
     .                    rmiss, im, jnp, nplevs_kept, plevs_kept,
     .                    first_date, first_time, inchms, 
     .                    ntimes(log8r,k),
     .                    1, kept_var(k), level_var(k), descr_var(k),
     .                    split_by_day)

            else

               call dd_write(dd_unit, grads_dsn_in_dd(log8r,k), title2,
     .                    rmiss, im, jnp, nplevs_kept, plevs_kept,
     .                    first_date, first_time, inchms, 
     .                    ntimes(log8r,k),
     .                    1, kept_var(k), level_var(k), descr_var(k),
     .                    split_by_day)

            end if

            close(dd_unit)

         end if
         end do
         end do

      end if


  600 continue

! Close input data unit, clear out any files remaining in file list
! and close list file unit.  This will remove any remaining input 
! files from the current directory.

      close(lu)

      do while ( openif_error.lt.1 )
         call openif( file_type, input_path, file_name, lu, lu_list,
     .                                                   openif_error )
      end do

      close(lu_list)

      stop
      end
