#!/bin/csh

# file_type: 'name'  = use string in "file_name" as input file
#            'list'  = use string in "file_name" as name of file containing
#                      a list of input files
# input_path: Path leading to input files.  If any of the input files, either
#             the name of the input file, the name of the list of input files
#             or the names of input files in the list, do not begin with a "/"
#             or a ".", then the program will prepend the input_path to get a 
#             full path name.  If any of the input file names begin with "/"
#             or ".", then the name is assumed to already be a full or relative
#             path name.  
# exp_number: ID used to tag output data sets if split_by_var is false.
#             Output data sets are tagged as 
#                fNNNN.MMDD.HH if first guess (-1) fields
#                aNNNN.MMDD.HH if analysis (0) fields
#                iNNNN.MMDD.HH if initialized or IAU (1) fields
#             Where
#                NNNN is the exp_number ID
#                MMDD.HH is the month, day and hour of the actual starting 
#                time of the data
#             The table files are created as 
#                tfNNNN.MMDD.HH
#                taNNNN.MMDD.HH
#                tiNNNN.MMDD.HH
# output_path: Path in which grads data and table files are to be placed.
# grads_path:  Path to write in table file for DSET keyword.
# grads_file_tag: Tag to append to data set names if split_by_var is
#                 true.  Output data sets are tagged as 
#                    f.VVVV.TAG if first guess (-1) fields
#                    a.VVVV.TAG if analysis (0) fields
#                    i.VVVV.TAG if initialized or IAU (1) fields
#                 Where
#                    VVVV is the variable mnemonic (e.g., vwnd) 
#                    TAG is the string specified by grads_file_tag
#                 The table files are created as 
#                    tf.VVVV.TAG
#                    ta.VVVV.TAG
#                    ti.VVVV.TAG
# split_by_var:   .true. if data are to be written as one variable per file.
#                    (This is like the reanalysis time series files.)
#                 .false. if all variables are to be in one file.
# begdat:         Beginning date.  
# begtim:         Beginning time.
# enddat:         Ending date.
# endtim:         Ending time.
# inchms:         Increment in HHMMSS format.
# itype:          -1 = First guess only
#                  0 = Analysis only
#                  1 = initialized or IAU only
#                  2 = All fields 
# nvars:          Number of variables.
# vars:           List of variable mnemonics.
# nplevs:         Number of upper air pressure levels.
# plevs:          List of upper air pressure (or sigma) levels.  Default
#                 is to take all upper air levels.
#     
#

ln -s ../run/fort.95 e.prs

/bin/rm tograds.list
cat > tograds.list << E-O-F
e.prs
E-O-F

/bin/ls -lat

/bin/rm tograds.n
cat > tograds.n << E-O-F
 &user_variables
  file_type           = 'list'
  file_name           = './tograds.list'
  input_path          = '.'
  output_path         = '.'
  grads_path          = '^'
  grads_file_tag      = '54A'
  split_by_var        = .false.
  split_by_day        = .true.
  begdat              = 870902
  begtim              = 000000
  enddat              = 870902
  endtim              = 180000
  inchms              = 060000
  itype               = 2
  nvars               =  8,
  vars                =  'SLP     ',
                         'SLU     ',
                         'SLV     ',
                         'UWND    ',
                         'VWND    ',
                         'HGHT    ',
                         'TMPU    ',
                         'MIXR    '
 &end
E-O-F

###  input_path          = 'w3djl@unitree:/u3/wlcrr/emaesaR'

tograds.phx.x

