      SUBROUTINE GCMDRV (KBC,N,NYMD,NHMS,SCHEME,NDT,ALPHA,QBEG,QBUDGET)         
C***********************************************************************        
C  PURPOSE                                                                      
C     Driver for the C-Grid GEOS GCM ( Dynamics and Physics )          
C  USAGE                                                                        
C     Called by User's Application                                              
C  ARGUMENTS   DESCRIPTION                                                      
C     KBC ...... Unit to read Boundary Conditions                               
C     N ........ Current Time Index                                             
C     NYMD ..... Current YYMMDD                                                 
C     NHMS ..... Current HHMMSS                                                 
C     SCHEME ... Character Description of Time-Scheme (MATS or LEAP)            
C     NDT ...... Time-Step in seconds                                           
C     ALPHA .... Time Filter Coefficient                                        
C     QBEG ..... Logical Flag for BEGIN cases (Cold Starts)                     
C     QBUDGET .. Logical Flag to calculate BUDGETS                              
C  SUBPROGRAMS NEEDED                                                           
C     SETMET     Initializes metric terms                                       
C     SETDMP     Initializes damping coeffs for high-latitude filter            
C     GETBCS     Updates GCM Boundary Conditions                                
C     MOISTIO    Driver for Moist Processes                                     
C     SWRIO      Driver for Shortwave Radiation                                 
C     LWRIO      Driver for Longwave  Radiation                                 
C     TURBIO     Driver for Turbulence Processes                                
C     DYCORE     Driver for Hydro-Dynamics  ARIES-GEOS Dynamical Core           
C     STEP       Updates Prognostic Fields One Time Step                        
C     TICK       Updates time counters                                          
C     PCHECK     Checks for pressure diagnostics                                
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
                                                                                
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <vphyscom.com>                                                         
#include <vmetric.com>                                                          
#include <iau.com>                                                          
                                                                                
      PARAMETER ( IMJMR    = IMR*(JMR+1) )                                       
      PARAMETER ( IMJMM1R  = IMR* JMR    )                                       
      PARAMETER ( IMJMM2R  = IMR*(JMR-1) )                                       
      PARAMETER ( KM       = 1           )                                       
      PARAMETER ( ONE      = 1.0 )                                               
      PARAMETER ( ZERO     = 0.0 )                                               
      PARAMETER ( ZP01     = 0.01)                                               
      PARAMETER ( ROTATION = 86164.09 )                                          
                                                                                
      PARAMETER ( NDLWR   = 030000 )                                            
      PARAMETER ( NDSWR   = 030000 )                                            
      PARAMETER ( NDTURB  = 003000 )                                            
 
      PARAMETER ( SECDAY  = 86400. )                                            
                                                                                
      CHARACTER*4 SCHEME, ZLEAP, ZMATS                                          
      LOGICAL     FIRST,  QBUDGET, QBEG, QLWR, QSWR, QMOIST, QTURB              
      LOGICAL     QHYDRO, LOMEGA
      DATA        FIRST /.TRUE./                                                
      DATA        ZLEAP /'LEAP'/                                                
      DATA        ZMATS /'MATS'/                                                
                                                                                
                                                                                
      common /space/  dpdt(imr,jmr+1)
      common /space/  dudt(imr,jmr+1,nlayr)
      common /space/  dvdt(imr,jmr+1,nlayr)
      common /space/  dtdt(imr,jmr+1,nlayr)
      common /space/  dqdt(imr,jmr+1,nlayr)
      common /space/   pkz(imr,jmr+1,nlayr)                                          
      
c Local Dynamic Arrays
c --------------------
      dimension uztmp(imr,jmr+1,nlayr)                                          
      dimension vztmp(imr,jmr+1,nlayr)                                          
      dimension pkhat(imr,jmr+1,nlayr+1)                                          

      dimension  tmp1(imr,jmr+1)                                          
      dimension  tmp2(imr,jmr+1)                                          
      dimension  tmp3(imr,jmr+1)                                          
      dimension  tmp4(imr,jmr+1)                                          
      dimension  qbar(imr,jmr+1)                                          

      dimension vintuq(imr,jmr+1)
      dimension vintvq(imr,jmr+1)
      dimension vintut(imr,jmr+1)
      dimension vintvt(imr,jmr+1)
                                                                        
      equivalence ( tmp1(1,1), uztmp(1,1,1) )
      equivalence ( tmp2(1,1), uztmp(1,1,2) )
      equivalence ( tmp3(1,1), uztmp(1,1,3) )
      equivalence ( tmp4(1,1), uztmp(1,1,4) )
      equivalence ( qbar(1,1), uztmp(1,1,5) )

      equivalence ( vintuq(1,1), uztmp(1,1,6) )
      equivalence ( vintvq(1,1), uztmp(1,1,7) )
      equivalence ( vintut(1,1), uztmp(1,1,8) )
      equivalence ( vintvt(1,1), uztmp(1,1,9) )

      save itcpu, itclk, ltcpu, ltclk
                                                                          
      NXTMOD(N,M) = MOD(N,M) + 1                                                
      MPER  (N  ) = MODHMS(NHMS,N)                                              
      NHMSF (N  ) = N/3600*10000 + MOD(N,3600 )/ 60*100 + MOD(N, 60)            
                                                                                
C **********************************************************************        
C ****                     MODEL INITIALIZATION                     ****        
C **********************************************************************        
                                                                                
      IF (FIRST) THEN                                                           
      CALL    CLOCKS (ITCPU,ITCLK)                                              
      LTCPU = ITCPU                                                             
      LTCLK = ITCLK                                                             
      CALL    SETMET                                                            
      FIRST = .FALSE.                                                           
      ENDIF                                                                     

      GRAV  = GETCON('GRAVITY')                                             
      AKAP  = GETCON('KAPPA')                                             
      CP    = GETCON('CP')                                             
      AE    = GETCON('EARTH RADIUS')                                             
      RGAS  = GETCON('RGAS')                                             
       OM   = 2.*PI/ROTATION
       P0K  = 1000.**AKAP
      AP0K  = 1./P0K

      J0     = 1
      LOMEGA = IOMEGA.NE.0
                                                                                
C **********************************************************************        
C ****                UPDATE GCM BOUNDARY CONDITIONS                ****        
C **********************************************************************        
                                                                                
      IF(KBC.GT.0) THEN                                                         
      CALL GETBCS (KBC, NYMD, NHMS, TGZ, PHISZ, TVAZ, RADSWT,                   
     .             SSTZ, ALBZ, GWZ, THKZ, SRZ, ZLWI, IMJM)                      
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                   COMPUTE PHYSICS FLAGS                      ****        
C **********************************************************************        
     
      if( ndt.ne.0 ) then
      ndmoist = int( 600./iabs(ndt) + 0.5 ) * iabs(ndt)
      else
      ndmoist = 2*ndt
      endif                                     
      ndmoist = nhmsf( ndmoist )                                                 

      ndhydro = nhmsf( ndt )
                                                                                
      QMOIST = NDMOIST .NE.0 .AND. MPER(NDMOIST) .EQ.0                            
      QSWR   = NDSWR   .NE.0 .AND. MPER(NDSWR)   .EQ.0                            
      QLWR   = NDLWR   .NE.0 .AND. MPER(NDLWR)   .EQ.0                            
      QTURB  = NDTURB  .NE.0 .AND. MPER(NDTURB)  .EQ.0                            
      QHYDRO = NDHYDRO .NE.0 .AND. MPER(NDHYDRO) .EQ.0                            
                                                                                
C **********************************************************************        
C ****                  CALL PHYSICS MINI-DRIVERS                   ****        
C **********************************************************************        
                                                                                
      if( qmoist.or.qswr.or.qlwr.or.qturb ) then                                
      call ctoa ( pz(1,1,  n), pz(1,1,  n), imr,jmr+1,1    ,0 )                 
      call ctoa ( tz(1,1,1,n), tz(1,1,1,n), imr,jmr+1,nlayr,0 )                 
      call ctoa ( qz(1,1,1,n), qz(1,1,1,n), imr,jmr+1,nlayr,0 )                 
      call ctoa ( pkht(1,1,1), pkht(1,1,1), imr,jmr+1,nlayr,0 )                 
      endif                                                                     
                                                                                
C MOIST PROCESSES                                                               
C ===============                                                               
      IF ( QMOIST ) THEN                                                        
      CALL MOISTIO (N  ,NDMOIST,QBUDGET)                                        
      ENDIF                                                                     
                                                                                
C SHORT WAVE RADIATION                                                          
C ====================                                                          
      IF ( QSWR ) THEN                                                          
      CALL SWRIO (N,NYMD,NHMS,NDSWR,NDT,QBUDGET)                                    
      ENDIF                                                                     
                                                                                
C LONG WAVE RADIATION                                                           
C ===================                                                           
      IF ( QLWR ) THEN                                                          
      CALL LWRIO (N,NYMD,NHMS,NDLWR,QBUDGET)                                    
      ENDIF                                                                     
                                                                                
C TURBULENCE                                                                    
C ==========                                                                    
      IF ( QTURB ) THEN                                                         
      do i=1,imr*(jmr+1)*nlayr                                                  
      uztmp(i,1,1) = uz(i,1,1,n)                                                
      vztmp(i,1,1) = vz(i,1,1,n)                                                
      enddo                                                                     
      do l=1,nlayr                                                              
      call polewnd ( uz(1,1,L,n),vz(1,1,L,n),                                   
     .               uz(1,1,L,n),vz(1,1,L,n),imr,jmr+1)                         
      enddo                                                                     
      call ctoa ( uz(1,1,1,n), uz(1,1,1,n),imr,jmr+1,nlayr,1 )                  
      call ctoa ( vz(1,1,1,n), vz(1,1,1,n),imr,jmr+1,nlayr,2 )                  
                                                                                
      CALL TURBIO (N,NYMD,NHMS,NDTURB,QBEG,QBUDGET,IMR,JMR+1,NLAYR)             
                                                                                
      do L=1,nlayr                                                              
      do i=1,imr*(jmr+1)                                                        
      uz(i,1,L,n) = uztmp(i,1,L)                                                
      vz(i,1,L,n) = vztmp(i,1,L)                                                
      duturb(i,1,L) = duturb(i,1,L)/pz(i,1,n)                                   
      dvturb(i,1,L) = dvturb(i,1,L)/pz(i,1,n)                                   
      enddo                                                                     
      enddo                                                                     
      call atoc ( duturb (1,1,1)  ,duturb (1,1,1)  ,imr,jmr+1,nlayr,1 )         
      call atoc ( dvturb (1,1,1)  ,dvturb (1,1,1)  ,imr,jmr+1,nlayr,2 )         
      endif                                                                     
                                                                                
      if( qmoist.or.qswr.or.qlwr.or.qturb ) then                                
      call atoc ( pz(1,1,  n), pz(1,1,  n), imr,jmr+1,1    ,0 )                 
      call atoc ( tz(1,1,1,n), tz(1,1,1,n), imr,jmr+1,nlayr,0 )                 
      call atoc ( qz(1,1,1,n), qz(1,1,1,n), imr,jmr+1,nlayr,0 )                 
      call atoc ( pkht(1,1,1), pkht(1,1,1), imr,jmr+1,nlayr,0 )                 
      endif                                                                     
                                                                                
C **********************************************************************        
C ****                     COMPUTE DIAGNOSTICS                      ****        
C **********************************************************************        
                                                                                
      CALL pkappc ( pz(1,1,n),pkht,pkz,imr,jmr+1,nlayr )                        
                                                                                
c Total Diabatic U-Tendency
c -------------------------
      if( idiabu.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idiabu+L-1) =   qdiag  (i,2,idiabu+L-1) 
     .                      + ( duanal (i,2,L) 
     .                        + duturb (i,2,L) 
     .                        + dumoist(i,2,L) )*86400
      enddo
      enddo
      endif
                                                                  
c Total Diabatic V-Tendency
c -------------------------
      if( idiabv.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idiabv+L-1) =   qdiag  (i,2,idiabv+L-1)
     .                      + ( dvanal (i,2,L) 
     .                        + dvturb (i,2,L) 
     .                        + dvmoist(i,2,L) )*86400
      enddo
      enddo
      endif
                                                                  
      if( idiabt.ne.0 .or. idiabq.ne.0 ) then
      do i=1,imjmm2r                                                    
      tmp1(i,2) = 1.0 / pz(i,2,n)                                    
      enddo                                                             
      endif

c Total Diabatic T-Tendency
c -------------------------
      if( idiabt.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idiabt+L-1) =   qdiag  (i,2,idiabt+L-1) 
     .                      + ( dtanal (i,2,L) 
     .                        + dtturb (i,2,L) 
     .                        + dtradlw(i,2,L)
     .                        + dtradsw(i,2,L) * radswt(i,2) 
     .                        + dtmoist(i,2,L) )
     .                        *     pkz(i,2,L) * tmp1(i,2)*86400
      enddo
      enddo
      endif
                                                                  
c Total Diabatic Q-Tendency
c -------------------------
      if( idiabq.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idiabq+L-1) =   qdiag  (i,2,idiabq+L-1) 
     .                      + ( DQANAL (i,2,L) 
     .                        + DQTURB (i,2,L) 
     .                        + DQMOIST(i,2,L) )
     .                        *  tmp1(i,2)*86400*1000
      enddo
      enddo
      endif
    
c DPANAL Analysis Increment
c -------------------------
      if( ianalp.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ianalp) = qdiag(i,2,ianalp) + dpanal(i,2)
      enddo
      endif
                                                                  
c DUANAL Analysis Increment
c -------------------------
      if( ianalu.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ianalu+L-1) = qdiag(i,2,ianalu+L-1)+duanal(i,2,L)
      enddo
      enddo
      endif
                                                                  
c DVANAL Analysis Increment
c -------------------------
      if( ianalv.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ianalv+L-1) = qdiag(i,2,ianalv+L-1)+dvanal(i,2,L)
      enddo
      enddo
      endif
                                                                  
c DTANAL Analysis Increment
c -------------------------
      if( ianalt.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ianalt+L-1) = qdiag(i,2,ianalt+L-1)+dtanal(i,2,L)
      enddo
      enddo
      endif
                                                                  
c DQANAL Analysis Increment
c -------------------------
      if( ianalq.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ianalq+L-1) = qdiag(i,2,ianalq+L-1)+dqanal(i,2,L)
      enddo
      enddo
      endif
    
c Incident Solar Radiation
c ------------------------
      IF (IRADSWT.NE.0) THEN                                                    
      DO    I=1,IMJMM2R                                                         
      QDIAG(I,2,IRADSWT) = QDIAG(I,2,IRADSWT) + RADSWT(I,2)                     
      ENDDO                                                                     
      ENDIF                                                                     
                                                                                
c Net Solar Radiation at the Ground
c ---------------------------------
      IF (IRADSWG.NE.0) THEN                                                    
      DO    I=1,IMJMM2R                                                         
      QDIAG(I,2,IRADSWG) = QDIAG(I,2,IRADSWG) + RADSWG(I,2)*RADSWT(I,2)         
      ENDDO                                                                     
      ENDIF                                                                     
                                                                                
c Solar Radiative Heating
c -----------------------
      IF (IRADSW.NE.0) THEN                                                     
      DO    L=1,NLAYR                                                           
      DO    I=1,IMJMM2R                                                         
      QDIAG(I,2,IRADSW+L-1) =   QDIAG(I,2,IRADSW+L-1)                           
     .                      + DTRADSW(I,2,L)*RADSWT(I,2)                        
     .                          * PKZ(I,2,L)/PZ(I,2,N)*SECDAY                   
      ENDDO                                                                     
      ENDDO                                                                     
      ENDIF                                                                     
                                                                                
      NDIABU  = NDIABU  + 1
      NDIABV  = NDIABV  + 1
      NDIABT  = NDIABT  + 1
      NDIABQ  = NDIABQ  + 1
      NANALP  = NANALP  + 1
      NANALU  = NANALU  + 1
      NANALV  = NANALV  + 1
      NANALT  = NANALT  + 1
      NANALQ  = NANALQ  + 1
      NRADSWT = NRADSWT + 1                                                     
      NRADSWG = NRADSWG + 1                                                     
      NRADSW  = NRADSW  + 1                                                     
                                                                                
C **********************************************************************        
C ****                      FILL PKHAT FOR DYCORE                   ****        
C **********************************************************************        
                                                                                
      if( j0.eq.1 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      pkhat(i,2,L) = pkht(i,2,L)
      enddo
      enddo
      endif

      if( j0.eq.2 ) then
      do i=1,imjmm2r
      pkhat(i,2,1) = ptop**akap * ap0k
      enddo
      do L=1,nlayr
      do i=1,imjmm2r
      pkhat(i,2,L+1) = pkht(i,2,L)*ap0k
      enddo
      enddo
      endif

C **********************************************************************        
C ****                       MATSUNO TIME SCHEME                    ****        
C **********************************************************************        
                                                                                
      IF(SCHEME.EQ.ZMATS) THEN                                                  
      NM1   = NXTMOD(N,2)                                                       
      NP1   = NXTMOD(N,2)                                                       
    
      timestep = ndt
                                                                        
c Predictor
c ---------
      do I=1,IMJMM2R                                                    
      DPDT(I,2) =    DPANAL(I,2)                                          
      enddo                                                             
      do L=1,NLAYR
      do I=1,IMJMM2R
      DUDT(I,2,L) =  DUANAL(I,2,L) + DUTURB(I,2,L) + DUMOIST(I,2,L)
      DVDT(I,2,L) =  DVANAL(I,2,L) + DVTURB(I,2,L) + DVMOIST(I,2,L)
      DTDT(I,2,L) =  DTANAL(I,2,L) + DTTURB(I,2,L) + DTRADLW(I,2,L)
     .            + DTRADSW(I,2,L) * RADSWT(I,2  ) + DTMOIST(I,2,L)
      DQDT(I,2,L) =  DQANAL(I,2,L) + DQTURB(I,2,L) + DQMOIST(I,2,L)
      enddo
      enddo

      call shapij ( PZ(1,1,N),UZ(1,1,1,N),VZ(1,1,1,N),
     .                        TZ(1,1,1,N),QZ(1,1,1,N),
     .                        dudt,dvdt,dtdt,dqdt,
     .                        imr,jmr+1,nlayr )
    
c Ensure non-reversable effects are positive
c ------------------------------------------
      IF ( NDT.LT.0 ) THEN                                              
      do I=1,IMJMM2R                                                    
      DPDT(I,2)   = - DPDT(I,2)                                       
      enddo                                                             
      do L=1,NLAYR                                                      
      do I=1,IMJMM2R                                                    
      DUDT(I,2,L) = - DUDT(I,2,L)                                       
      DVDT(I,2,L) = - DVDT(I,2,L)                                       
      DTDT(I,2,L) = - DTDT(I,2,L)                                       
      DQDT(I,2,L) = - DQDT(I,2,L)                                       
      enddo                                                             
      enddo                                                             
      ENDIF                                                             
                                                                        
      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,n)   = tz(i,2,L,n)  *p0k
      dtdt(i,2,L)   = dtdt(i,2,L)  *p0k
      enddo
      enddo
      endif
      call dycore ( imr,jmr,jmr+1,nlayr,sige,ptop,km,timestep,
     .              om,cp,rgas,ae,
     .           phisz(1,j0),  pkhat(1,j0,1),
     .              pz(1,j0,n),  uz(1,j0,1,n), vz(1,1, 1,n),
     .                           tz(1,j0,1,n), qz(1,j0,1,n),
     .              pz(1,j0,n),  uz(1,j0,1,n), vz(1,1, 1,n),
     .                           tz(1,j0,1,n), qz(1,j0,1,n),
     .              dpdt(1,j0),dudt(1,j0,1), dvdt(1,1 ,1),
     .              dtdt(1,j0,1),dqdt(1,j0,1), 
     .              uztmp(1,j0,1),vztmp(1,1,1),Lomega )

c Add Analysis Increment to Dynamical Omega Diagnostics
c -----------------------------------------------------
      if( iomega.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iomega+L-1) = qdiag(i,2,iomega+L-1) 
     .                      + uztmp(i,2,L)
     .                      + sige(L+1)*dpanal(i,2)*86400
      enddo
      enddo
      endif
      if( ivort.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm1r
      qdiag(i,1,ivort+L-1) = qdiag(i,1,ivort+L-1) 
     .                     + vztmp(i,1,L)
      enddo
      enddo
      endif
      NVORT  = NVORT  + 1
      NOMEGA = NOMEGA + 1

      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,n)   = tz(i,2,L,n)  *ap0k
      dtdt(i,2,L)   = dtdt(i,2,L)  *ap0k
      enddo
      enddo
      endif
      call step ( n,n,np1,ndt,zero,imr,jmr+1,nlayr,
     .            dpdt,dudt,dvdt,dtdt,dqdt,pkz,qbudget )                  

c Update PKHAT
c ------------
      if( j0.eq.1 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      pkhat(i,2,L) = pkht(i,2,L)
      enddo
      enddo
      endif

      if( j0.eq.2 ) then
      do i=1,imjmm2r
      pkhat(i,2,1) = ptop**akap * ap0k
      enddo
      do L=1,nlayr
      do i=1,imjmm2r
      pkhat(i,2,L+1) = pkht(i,2,L)*ap0k
      enddo
      enddo
      endif

c Corrector
c ---------
      do I=1,IMJMM2R                                                    
      DPDT(I,2) =    DPANAL(I,2)                                          
      enddo                                                             
      do L=1,NLAYR
      do I=1,IMJMM2R
      DUDT(I,2,L) =  DUANAL(I,2,L) + DUTURB(I,2,L) + DUMOIST(I,2,L)
      DVDT(I,2,L) =  DVANAL(I,2,L) + DVTURB(I,2,L) + DVMOIST(I,2,L)
      DTDT(I,2,L) =  DTANAL(I,2,L) + DTTURB(I,2,L) + DTRADLW(I,2,L)
     .            + DTRADSW(I,2,L) * RADSWT(I,2  ) + DTMOIST(I,2,L)
      DQDT(I,2,L) =  DQANAL(I,2,L) + DQTURB(I,2,L) + DQMOIST(I,2,L)
      enddo
      enddo
    
      call shapij ( PZ(1,1,N),UZ(1,1,1,N),VZ(1,1,1,N),
     .                        TZ(1,1,1,N),QZ(1,1,1,N),
     .                        dudt,dvdt,dtdt,dqdt,
     .                        imr,jmr+1,nlayr )
    
c Ensure non-reversable effects are positive
c ------------------------------------------
      IF ( NDT.LT.0 ) THEN                                              
      do I=1,IMJMM2R                                                    
      DPDT(I,2)   = - DPDT(I,2)                                       
      enddo                                                             
      do L=1,NLAYR                                                      
      do I=1,IMJMM2R                                                    
      DUDT(I,2,L) = - DUDT(I,2,L)                                       
      DVDT(I,2,L) = - DVDT(I,2,L)                                       
      DTDT(I,2,L) = - DTDT(I,2,L)                                       
      DQDT(I,2,L) = - DQDT(I,2,L)                                       
      enddo                                                             
      enddo                                                             
      ENDIF                                                             
                                                                        
      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,np1) = tz(i,2,L,np1)*p0k
      dtdt(i,2,L  ) = dtdt(i,2,L  )*p0k
      enddo
      enddo
      endif
      call dycore ( imr,jmr,jmr+1,nlayr,sige,ptop,km,timestep,
     .              om,cp,rgas,ae,
     .           phisz(1,j0),    pkhat(1,j0,1),
     .              pz(1,j0,np1),  uz(1,j0,1,np1), vz(1,1, 1,np1),
     .                             tz(1,j0,1,np1), qz(1,j0,1,np1),
     .              pz(1,j0,np1),  uz(1,j0,1,np1), vz(1,1, 1,np1),
     .                             tz(1,j0,1,np1), qz(1,j0,1,np1),
     .              dpdt(1,j0),  dudt(1,j0,1),   dvdt(1,1 ,1),
     .              dtdt(1,j0,1),dqdt(1,j0,1),
     .              uztmp(1,j0,1),vztmp(1,1,1),Lomega )

c Add Analysis Increment to Dynamical Omega Diagnostics
c -----------------------------------------------------
      if( iomega.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iomega+L-1) = qdiag(i,2,iomega+L-1) 
     .                      + uztmp(i,2,L)
     .                      + sige(L+1)*dpanal(i,2)*86400
      enddo
      enddo
      endif
      if( ivort.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm1r
      qdiag(i,1,ivort+L-1) = qdiag(i,1,ivort+L-1) 
     .                     + vztmp(i,1,L)
      enddo
      enddo
      endif
      NVORT  = NVORT  + 1
      NOMEGA = NOMEGA + 1

      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,np1) = tz(i,2,L,np1)*ap0k
      dtdt(i,2,L  ) = dtdt(i,2,L  )*ap0k
      enddo
      enddo
      endif
      call step ( n,np1,np1,ndt,zero,imr,jmr+1,nlayr,
     .            dpdt,dudt,dvdt,dtdt,dqdt,pkz,qbudget )                  
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                   LEAPFROG TIME SCHEME                       ****        
C **********************************************************************        
                                                                                
      IF(SCHEME.EQ.ZLEAP) THEN                                                  
      NM1   = NXTMOD(N,2)                                                       
      NP1   = NM1                                                               

      timestep = ndt

      do I=1,IMJMM2R                                                    
      DPDT(I,2) =    DPANAL(I,2)                                          
      enddo                                                             
      do L=1,NLAYR
      do I=1,IMJMM2R
      DUDT(I,2,L) =  DUANAL(I,2,L) + DUTURB(I,2,L) + DUMOIST(I,2,L)
      DVDT(I,2,L) =  DVANAL(I,2,L) + DVTURB(I,2,L) + DVMOIST(I,2,L)
      DTDT(I,2,L) =  DTANAL(I,2,L) + DTTURB(I,2,L) + DTRADLW(I,2,L)
     .            + DTRADSW(I,2,L) * RADSWT(I,2  ) + DTMOIST(I,2,L)
      DQDT(I,2,L) =  DQANAL(I,2,L) + DQTURB(I,2,L) + DQMOIST(I,2,L)
      enddo
      enddo
    
      call shapij ( PZ(1,1,NM1),UZ(1,1,1,NM1),VZ(1,1,1,NM1),
     .                          TZ(1,1,1,NM1),QZ(1,1,1,NM1),
     .                          dudt,dvdt,dtdt,dqdt,
     .                          imr,jmr+1,nlayr )
    
c Ensure non-reversable effects are positive
c ------------------------------------------
      IF ( NDT.LT.0 ) THEN                                              
      do I=1,IMJMM2R                                                    
      DPDT(I,2)   = - DPDT(I,2)                                       
      enddo                                                             
      do L=1,NLAYR                                                      
      do I=1,IMJMM2R                                                    
      DUDT(I,2,L) = - DUDT(I,2,L)                                       
      DVDT(I,2,L) = - DVDT(I,2,L)                                       
      DTDT(I,2,L) = - DTDT(I,2,L)                                       
      DQDT(I,2,L) = - DQDT(I,2,L)                                       
      enddo                                                             
      enddo                                                             
      ENDIF                                                             
                                                                        
      if( j0.eq.2 ) then
      timestep = timestep*2
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,n)   = tz(i,2,L,n)  *p0k
      tz(i,2,L,nm1) = tz(i,2,L,nm1)*p0k
      dtdt(i,2,L)   = dtdt(i,2,L)  *p0k
      enddo
      enddo
      endif
      call dycore ( imr,jmr,jmr+1,nlayr,sige,ptop,km,timestep,
     .              om,cp,rgas,ae,
     .           phisz(1,j0),    pkhat(1,j0,1),
     .              pz(1,j0,n  ),  uz(1,j0,1,n  ), vz(1,1, 1,n  ),
     .                             tz(1,j0,1,n  ), qz(1,j0,1,n  ),
     .              pz(1,j0,nm1),  uz(1,j0,1,nm1), vz(1,1, 1,nm1),
     .                             tz(1,j0,1,nm1), qz(1,j0,1,nm1),
     .              dpdt(1,j0),  dudt(1,j0,1),   dvdt(1,1 ,1),
     .              dtdt(1,j0,1),dqdt(1,j0,1),
     .              uztmp(1,j0,1),vztmp(1,1,1),Lomega )

c Add Analysis Increment to Dynamical Omega Diagnostics
c -----------------------------------------------------
      if( iomega.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iomega+L-1) = qdiag(i,2,iomega+L-1) 
     .                      + uztmp(i,2,L)
     .                      + sige(L+1)*dpanal(i,2)*86400
      enddo
      enddo
      endif
      if( ivort.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm1r
      qdiag(i,1,ivort+L-1) = qdiag(i,1,ivort+L-1) 
     .                     + vztmp(i,1,L)
      enddo
      enddo
      endif
      NVORT  = NVORT  + 1
      NOMEGA = NOMEGA + 1

      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,n)   = tz(i,2,L,n)  *ap0k
      tz(i,2,L,nm1) = tz(i,2,L,nm1)*ap0k
      dtdt(i,2,L)   = dtdt(i,2,L)  *ap0k
      enddo
      enddo
      endif
      call step ( nm1,n,np1,2*ndt,alpha,imr,jmr+1,nlayr,
     .            dpdt,dudt,dvdt,dtdt,dqdt,pkz,qbudget )               
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                      TICK THE CLOCK                          ****        
C **********************************************************************        
                                                                                
      CALL TICK (NYMD,NHMS,NDT,N)                                               
                                                                                
      CALL CLOCKS     (NTCPU,NTCLK)                                             
      TCPU   = ZP01 * (ITCPU-NTCPU)                                             
      TCLK   = ZP01 * (ITCLK-NTCLK)                                             
      SCPU   = ZP01 * (LTCPU-NTCPU)                                             
      SCLK   = ZP01 * (LTCLK-NTCLK)                                             
      TRATIO = TCPU / TCLK                                                      
      SRATIO = SCPU / SCLK                                                      
      LTCPU  = NTCPU                                                            
      LTCLK  = NTCLK                                                            
      DELTAP = GMEAN( DPANAL,IMR,JMR+1 )                               

      WRITE (6,1000) NYMD, NHMS,                                                
     .               SCPU, TCPU, SCLK,TCLK,SRATIO,TRATIO, DELTAP                
                                                                                
C **********************************************************************        
C ****                  COMPUTE MORE DIAGNOSTICS                    ****        
C **********************************************************************        

      CALL pkappc ( pz(1,1,n),pkht,pkz,imr,jmr+1,nlayr )                        
                                                                                
c Averaged P-Field
c ----------------
      if( ipave.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ipave) = qdiag(i,2,ipave) + pz(i,2,n)
      enddo
      endif
                                                                                
c Averaged U-Field
c ----------------
      if( iuave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iuave+L-1) = qdiag(i,2,iuave+L-1) + uz(i,2,L,n)
      enddo
      enddo
      endif
                                                                                
c Averaged V-Field
c ----------------
      if( ivave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ivave+L-1) = qdiag(i,2,ivave+L-1) + vz(i,2,L,n)
      enddo
      enddo
      endif
                                                                                
c Averaged T-Field
c ----------------
      if( itave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,itave+L-1) = qdiag(i,2,itave+L-1) 
     .                     +    tz(i,2,L,n)*pkz(i,2,L)
      enddo
      enddo
      endif
                                                                                
c Averaged Q-Field
c ----------------
      if( iqave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iqave+L-1) = qdiag(i,2,iqave+L-1) + qz(i,2,L,n)
      enddo
      enddo
      endif
                                                                                
c Averaged QQ-Field
c -----------------
      if( iqqave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iqqave+L-1) = qdiag(i,2,iqqave+L-1) + qqz(i,2,L)
      enddo
      enddo
      endif
                                                                                
c Precipitable Water
c ------------------
      if( iqint.ne.0 ) then
      do i=1,imjmm2r
      qbar(i,2) = 0.0
      enddo
      do L=1,nlayr
      do i=1,imjmm2r
      qbar(i,2) = qbar(i,2) + qz(i,2,L,n)*dsig(L)
      enddo
      enddo
      do i=1,imjmm2r
      qbar(i,2) = qbar(i,2) * 10 * pz(i,1,n) / grav
      enddo
      do i=1,imjmm2r
      qdiag(i,2,iqint) = qdiag(i,2,iqint) + qbar(i,2)
      enddo
      endif
                                                                                
c Compute Temperature and Moisture Convergence Diagnostics
c --------------------------------------------------------
      if( ivintuq.ne.0 .or. ivintvq.ne.0 .or. 
     .    ivintut.ne.0 .or. ivintvt.ne.0 ) then
      do i=1,imjmm2r
      vintuq(i,2) = 0.0
      vintvq(i,2) = 0.0
      vintut(i,2) = 0.0
      vintvt(i,2) = 0.0
      enddo
 
      do L=1,nlayr
      call qbaru ( uz(1,1,L,n),vz(1,1,L,n),
     .             tmp1,tmp2,tmp3,tmp4,imr,jmr+1 )
      do i=1,imjmm2r
      vintuq(i,2) = vintuq(i,2) + tmp1(i,2)*qz(i,2,L,n)*dsig(L)
      vintvq(i,2) = vintvq(i,2) + tmp2(i,2)*qz(i,2,L,n)*dsig(L)
      vintut(i,2) = vintut(i,2) + tmp1(i,2)*tz(i,2,L,n)*dsig(L)
     .                          * pkz (i,2,L)
      vintvt(i,2) = vintvt(i,2) + tmp2(i,2)*tz(i,2,L,n)*dsig(L)
     .                          * pkz (i,2,L)
      enddo
      enddo

      if( ivintuq.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ivintuq) = qdiag(i,2,ivintuq) + vintuq(i,2)*1000
      enddo
      endif
      if( ivintvq.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ivintvq) = qdiag(i,2,ivintvq) + vintvq(i,2) *1000
      enddo
      endif
      if( ivintut.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ivintut) = qdiag(i,2,ivintut) + vintut(i,2)
      enddo
      endif
      if( ivintvt.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ivintvt) = qdiag(i,2,ivintvt) + vintvt(i,2)
      enddo
      endif
      endif
 

      NPAVE   = NPAVE   + 1                           
      NUAVE   = NUAVE   + 1                           
      NVAVE   = NVAVE   + 1                           
      NTAVE   = NTAVE   + 1                           
      NQAVE   = NQAVE   + 1                           
      NQQAVE  = NQQAVE  + 1                           
      NQINT   = NQINT   + 1                           
      NVINTUQ = NVINTUQ + 1
      NVINTVQ = NVINTVQ + 1
      NVINTUT = NVINTUT + 1
      NVINTVT = NVINTVT + 1
                                                                                
C **********************************************************************        
C ****                CHECK FOR PRESSURE DIAGNOSTIC                 ****        
C **********************************************************************        
                                                                                
      CALL PCHECK (N)                                                           
                                                                                
 1000 FORMAT(1X,'Date: ',I6,2X,                                                 
     .          'Time: ',I6,2X,                                                 
     .          ' CPU: ',F6.2,F8.2,2X,                                          
     .          ' CLK: ',F6.2,F8.2,2X,                                          
     .       'CPU/CLK: ',F6.3,F6.3,2X,'DELP: ',F8.5,' mb')                      
      RETURN                                                                    
      END                                                                       
      SUBROUTINE SWRIO (N,NYMD,NHMS,NDSWR,NDT,QBUDGET)
#include <vrslv.com>
#include <vmetric.com>
#include <vstate.com>
#include <vphyscom.com>
#include <vbudget.com>
#include <stripmine.com>
C
      PARAMETER ( IJALL   =  IMR * (JMR+1)  )
      PARAMETER ( ZERO    =    0.0 )
      PARAMETER ( ZP01    =    0.01)
      PARAMETER ( ONE     =    1.0 )
      PARAMETER ( AHALF   =    0.5 )
      PARAMETER ( TWOTHRD =   2./3.)
      PARAMETER ( TWO     =    2.0 )
      PARAMETER ( TEN     =   10.0 )
      PARAMETER ( ONEETY  =  180.0 )
C
      DIMENSION ALON(IMR), ALAT(JMR+1)
      LOGICAL QBUDGET
C
      COMMON /SPACE/ SPACE1(IMR,JMR+1,5*NLAYR+11)
      COMMON /SPACE/ WRK1  (IJALL,17     ,2)
      COMMON /SPACE/ WRK2  (IJALL,NLAYR+1,2)
C

      DIMENSION     PKZ(IMR, JMR+1, NLAYR)
      DIMENSION     PLZ(IMR*(JMR+1),NLAYR+1)
      DIMENSION      OZ(IMR*(JMR+1),NLAYR+1)
      DIMENSION    PLZE(IMR*(JMR+1),NLAYR+2)
      DIMENSION  CLRSKY(IMR,JMR+1,NLAYR)
      DIMENSION  ALLATS(IMR,JMR+1)
      DIMENSION    XLAT(IJALL)
      DIMENSION  TEMPIJ(IJALL)
      DIMENSION TEMPIJ1(IJALL)
      DIMENSION TEMPIJ2(IJALL)
      DIMENSION swgclr (IJALL)
      DIMENSION zenith (IJALL)

      EQUIVALENCE (   PKZ(1,1,1) , SPACE1(1,1,        1) )
      EQUIVALENCE (CLRSKY(1,1,1) , SPACE1(1,1,  NLAYR+1) )
      EQUIVALENCE (  ALLATS(1,1) , SPACE1(1,1,2*NLAYR+1) )
      EQUIVALENCE (   XLAT   (1) , SPACE1(1,1,2*NLAYR+2) )
      EQUIVALENCE (   TEMPIJ (1) , SPACE1(1,1,2*NLAYR+3) )
      EQUIVALENCE (   TEMPIJ1(1) , SPACE1(1,1,2*NLAYR+4) )
      EQUIVALENCE (   TEMPIJ2(1) , SPACE1(1,1,2*NLAYR+5) )
      EQUIVALENCE (   swgclr (1) , SPACE1(1,1,2*NLAYR+6) )
      EQUIVALENCE (    plz (1,1) , SPACE1(1,1,2*NLAYR+7) )
      EQUIVALENCE (    plze(1,1) , SPACE1(1,1,3*NLAYR+8) )
      EQUIVALENCE (     oz (1,1) , SPACE1(1,1,4*NLAYR+10))
C
      DIMENSION  SWHEAT(ISTRIP,NLAYR+1), PKSTR(ISTRIP,NLAYR)
      DIMENSION    SHZL(ISTRIP,NLAYR+1),  CLRO(ISTRIP,NLAYR+1)
      DIMENSION     TZL(ISTRIP,NLAYR+1),  CLMO(ISTRIP,NLAYR+1)
      DIMENSION     OZL(ISTRIP,NLAYR+1),  PLZSTR(ISTRIP,NLAYR+1)
      DIMENSION PLZESTR(ISTRIP,NLAYR+2)
      DIMENSION   TEMP1(ISTRIP)
      DIMENSION   TEMP2(ISTRIP)
      DIMENSION   TEMP3(ISTRIP)
      DIMENSION    COSZ(ISTRIP)
      DIMENSION   SCOSZ(ISTRIP)
      DIMENSION   RSURF(ISTRIP)

      DATA    KOZ /20/
C
C **********************************************************************
C ****                       INITIALIZATION                         ****
C **********************************************************************
C
      ssalb = getcon('SSALB')
C
      DO JJ=1,JMR+1
       ALAT(JJ) = - 90. + (JJ-1)*180./JMR
      ENDDO
      DO II=1,IMR
       ALON(II) = -180. + (II-1)*360./IMR
      ENDDO
C
      DO J = 1,JNP
      DO I = 1,IMR
       ALLATS(I,J) = J
      ENDDO
      ENDDO

      NTMSTP    = MODHMS(NDSWR,0)
      ISTNLP1   = ISTRIP * (NLAY+1)
      ISTNLAY   = ISTRIP * NLAY
      PI180     = PI / ONEETY
      FJEQ      = AHALF * (1 + JNP)

      DO I = 1,IJALL
       PLZE(I,1) = ZERO
       PLZE(I,2) = PTOP
        PLZ(I,1) = PTOP * AHALF
         XLAT(I) = DP * (ALLATS(I,1) - FJEQ) / PI180
      ENDDO
C
      DO L = 2,NLAYP1
       DO I = 1,IJALL
        PLZ (I,L  ) = SIG(L-1) * PZ(I,1,N) + PTOP
        PLZE(I,L+1) = SIGE(L)  * PZ(I,1,N) + PTOP
       ENDDO
      ENDDO
C
C **********************************************************************
C ****             CALCULATE COSINE OF THE ZENITH ANGLE             ****
C **********************************************************************

      NYMD2 = NYMD
      NHMS2 = NHMS
      do i=1,ijall
      zenith(i) = 0.0
      enddo
      ncmax = ntmstp/ndt
      do nc = 1,ncmax
      CALL ASTRO ( NYMD2,NHMS2, ALAT,ALON, IMR,JMR+1, TEMPIJ1,RA )
      do i=1,ijall
      zenith(i) = zenith(i) + tempij1(i)
      enddo
      call tick ( nymd2,nhms2,ndt,ndum )
      enddo
      do i=1,ijall
      zenith(i) = zenith(i) / ncmax
      enddo

      CALL ASTRO ( NYMD,   NHMS,  ALAT,ALON, IMR,JMR+1, TEMPIJ1,RA )
                   NYMD2 = NYMD
                   NHMS2 = NHMS
      CALL TICK  ( NYMD2,  NHMS2, NTMSTP,NB2 )
      CALL ASTRO ( NYMD2,  NHMS2, ALAT,ALON, IMR,JMR+1, TEMPIJ2,RA )

      DO I = 1,IJALL
             TEMPIJ(I) = TEMPIJ1(I) + TEMPIJ2(I)
       IF  ( TEMPIJ(I) .GT.ZERO ) THEN
             TEMPIJ(I) =   TWOTHRD   *  ( TEMPIJ(I) -
     1      TEMPIJ2(I) * TEMPIJ1(I)/TEMPIJ(I) )
       ENDIF
      ENDDO
 
C **********************************************************************
C ****                   COMPUTE P**KAPPA                           ****
C **********************************************************************
 
      CALL PKAPPA ( PZ(1,1,N),PKHT,PKZ,IM,JNP,NLAY )
 
C **********************************************************************
C *** GET OZONE MASS MIXING RATIOS
C **********************************************************************
 
      CALL GETO3 (KOZ,NYMD,NHMS,IJALL,XLAT,NLAYP1,PLZ,WRK1,WRK2,OZ)
 
C***********************************************************************
C ****                     LOOP OVER GLOBAL REGIONS                 ****
C **********************************************************************
 
#if macro
cmic$ do all shared (albz, clrsky, dtradsw, iclmosw, iclrosw, ijall)
cmic$*       shared (imjm, imr, iosrclr, istnlay, istnlp1, istrip)
cmic$*       shared (iswclr, iswgclr, jmr, n, nlay, nlayp1, nlayr)
cmic$*       shared (npcs, nprogu2, nswcld, one, oz, pkz, plz)
cmic$*       shared (plze, qbudget, qdiag, qz, radswg, ssalb, swgclr)
cmic$*       shared (tempij, tempij1, tempij2, tz, zero)
cmic$*       shared (cswmo, cswro)
cmic$*       private (clmo, clro, cosz, i, nn, ozl)
cmic$*       private (pkstr, plzestr, plzstr, rsurf, scosz, shzl)
cmic$*       private (swheat, temp1, temp2, tzl)
#endif

      DO 1000 NN = 1,NPCS
 
C **********************************************************************
C ****  CALCULATE COSINE OF ZENITH ANGLE AND INCOMING RADIATION     ****
C **********************************************************************
 
      CALL STRIP ( TEMPIJ,COSZ,IJALL,ISTRIP,1,NN )
 
      DO I = 1,ISTRIP
      SCOSZ(I)  = ONE
      CLRO(I,1) = ZERO
      CLMO(I,1) = ZERO
      ENDDO
 
C **********************************************************************
C ****                   VARIABLE INITIALIZATION                    ****
C **********************************************************************
 
      CALL STRIP (PLZE,PLZESTR          ,IJALL,ISTRIP,NLAYP1+1,NN)
      CALL STRIP (PLZ,PLZSTR            ,IJALL,ISTRIP,NLAYP1  ,NN)
      CALL STRIP ( OZ,OZL               ,IJALL,ISTRIP,NLAYP1  ,NN)
      CALL STRIP (PKZ,PKSTR             ,IJALL,ISTRIP,NLAY    ,NN)
      CALL STRIP (ALBZ,RSURF            ,IJALL,ISTRIP,1       ,NN)
      CALL STRIP (TZ(1,1,1,N),TZL(1,2)  ,IJALL,ISTRIP,NLAY    ,NN)
      CALL STRIP (QZ(1,1,1,N),SHZL(1,2) ,IJALL,ISTRIP,NLAY    ,NN)
C
      DO I = 1,ISTNLAY
       TZL(I,2) =  TZL(I,2) * PKSTR(I,1)
      ENDDO
C
C *** INSERT FICTICIOUS UPPER BOUNDARY
C
      DO I = 1,ISTRIP
        TZL(I,1)  =  TZL(I,2)
       SHZL(I,1)  = SHZL(I,2)
      ENDDO
C
C **********************************************************************
C ****  CLEAR SKY  SHORT WAVE RADIATION FOR BUDGETS                 ****
C **********************************************************************
C
      IF (QBUDGET .or. iswclr .ne.0
     .            .or. iswgclr.ne.0
     .            .or. iosrclr.ne.0 ) THEN
       DO I = 1,ISTNLP1
        CLRO(I,1) = ZERO
        CLMO(I,1) = ZERO
       ENDDO

      CALL RADSW ( TZL,SHZL,OZL,PLZESTR(1,2),CLRO,CLMO,SCOSZ,RSURF,
     .             COSZ,TEMP1,SWHEAT,TEMP2,istrip,nlayr+1,ssalb )

      CALL PASTE ( SWHEAT(1,2),CLRSKY,ISTRIP,IJALL,NLAY,NN )
      CALL PASTE ( TEMP1    ,  swgclr,ISTRIP,IJALL,1   ,NN )
      CALL PASTE ( TEMP2    , TEMPIJ1,ISTRIP,IJALL,1   ,NN )
      ENDIF
C
C **********************************************************************
C ****              SHORT WAVE RADIATION                            ****
C **********************************************************************
C
      CALL STRIP (CSWRO,CLRO(1,2),IJALL,ISTRIP,NLAY,NN)
      CALL STRIP (CSWMO,CLMO(1,2),IJALL,ISTRIP,NLAY,NN)

      IF(NSWCLD.NE.0)THEN
       DO I = 1,ISTNLP1
       CLRO(I,1) = CLRO(I,1)/NSWCLD
       CLMO(I,1) = CLMO(I,1)/NSWCLD
       ENDDO
      ENDIF

      CALL RADSW ( TZL,SHZL,OZL,PLZESTR(1,2),CLRO,CLMO,SCOSZ,RSURF,
     .             COSZ,TEMP1,SWHEAT,TEMP2,istrip,nlayr+1,ssalb )
 
C **********************************************************************
C ****                     TENDENCY UPDATES                         ****
C **********************************************************************
 
      IF(ICLMOSW.GT.0)
     .CALL PSTBMP (CLMO(1,2),QDIAG(1,1,ICLMOSW),ISTRIP,IMJM,NLAY,NN)
      IF(ICLROSW.GT.0)
     .CALL PSTBMP (CLRO(1,2),QDIAG(1,1,ICLROSW),ISTRIP,IMJM,NLAY,NN)

      CALL PASTE ( SWHEAT(1,2),DTRADSW ,ISTRIP,IJALL,NLAY,NN )
      CALL PASTE ( TEMP1      , RADSWG ,ISTRIP,IJALL,1   ,NN )
      CALL PASTE ( TEMP2      , TEMPIJ2,ISTRIP,IJALL,1   ,NN )
 
 1000 CONTINUE
 
C **********************************************************************
C ****      SET DIAGNOSTICS  AND  ZERO-OUT CLOUD COUNTERS           ****
C **********************************************************************
 
      S0 = GETCON('S0')
      DO I=1,ijall
      tempij(I) = S0 * (ONE/RA**2) * zenith(i)
      enddo

      if(iosr.gt.0) then
      do i=1,ijall
      qdiag(i,1,iosr) = qdiag(i,1,iosr) + (1.-tempij2(i))*tempij(i)
      enddo
      endif

      if(iosrclr.gt.0) then
      do i=1,ijall
      qdiag(i,1,iosrclr) = qdiag(i,1,iosrclr)
     .                   + (1.-tempij1(i))*tempij(i)
      enddo
      endif

      if(iswgclr.gt.0) then
      do i=1,ijall
      qdiag(i,1,iswgclr) =  qdiag(i,1,iswgclr) + swgclr(i)*tempij(i)
      enddo
      endif

      if(iswclr.gt.0) then
      do L=1,nlayr
      do i=1,ijall
      qdiag(i,1,iswclr+L-1) =  qdiag(i,1,iswclr+L-1)
     .                      + clrsky(i,1,L)*tempij(i)*86400
      enddo
      enddo
      endif

      if(icldfrc.gt.0) then
       do I =1,ijall
        tempij1(I) = 1.
        tempij2(I) = 1.
       enddo
       do L=1,nlay
       do I =1,ijall
        cswro(i,1,L) = cswro(i,1,L) / nswcld
        cswmo(i,1,L) = cswmo(i,1,L) / nswcld
        tempij1(I) = tempij1(I) * (1.-cswro(I,1,L))
        if(tempij2(I) .GT. (1.-cswmo(I,1,L)) )
     1                           tempij2(I) = (1.-cswmo(I,1,L))
       enddo
       enddo
       do I =1,ijall
        tempij1(I)  = 1. - tempij1(I) * tempij2(I)
       enddo
       do i=1,ijall
        qdiag(i,1,icldfrc) =  qdiag(i,1,icldfrc) + tempij1(i)
       enddo
      endif

      DO L = 1,NLAY
       DO I = 1,IJALL
        CSWMO(I,1,L) = ZERO
        CSWRO(I,1,L) = ZERO
       ENDDO
      ENDDO
      NSWCLD  = 0
      NCLMOSW = NCLMOSW + 1
      NCLROSW = NCLROSW + 1
      NOSR    = NOSR    + 1
      NOSRCLR = NOSRCLR + 1
      NSWCLR  = NSWCLR  + 1
      NSWGCLR = NSWGCLR + 1
      ncldfrc = ncldfrc + 1
C
C **********************************************************************
C ****                PI-WEIGHT THE TENDENCIES                      ****
C **********************************************************************
C
      DO L = 1,NLAY
       DO I = 1,IJALL
        DTRADSW(I,1,L) = DTRADSW(I,1,L)*PZ(I,1,N) / PKZ(I,1,L)
       ENDDO
      ENDDO

C **********************************************************************
C ****        CALCULATE AND FILL APPROXIMATE BUDGETS                ****
C **********************************************************************

      IF (QBUDGET) THEN
      DO L = 1,NLAY
       DO I = 1,IJALL
        CLRSKY (I,1,L) = CLRSKY (I,1,L)*PZ(I,1,N) / PKZ(I,1,L)
       ENDDO
      ENDDO
      TMSTP = NTMSTP
      DO L=1,NLAY
       DO I = 1,IJALL
        ZTEMP(I,1) = DTRADSW(I,1,L)*RADSWT(I,1)
       ENDDO
       CALL ZMEAN ( ZTEMP(1,1),ZSWAVE(1,L),IM,JNP,TMSTP)
       DO I = 1,IJALL
        ZTEMP(I,1) = ZTEMP(I,1) - CLRSKY(I,1,L)*RADSWT(I,1)
       ENDDO
       CALL ZMEAN ( ZTEMP(1,1),ZSWCLD(1,L),IM,JNP,TMSTP)
      ENDDO
      ENDIF

      RETURN
      END
