      SUBROUTINE CHGZ1000 (PSMOD,TSMOD,PSL,PHIS1,Z1000,pmand,
     .                     IM,JNP,MLEV)
C**********************************************************************
C
C  COMPUTE NEW HEIGHT FIELD AT 1000MB USING ANALYZED SEA LEVEL PRESSURE
C  AND FIRST GUESS SURFACE TEMPERATURE AND PRESSURE.
C
C   PSMOD  - FIRST GUESS SURFACE TEMPERATURE
C   TSMOD  - FIRST GUESS SURFACE PRESSURE
C   PHIS1  - GEOPOTENTIAL (HEIGHT*G)
C   Z1000  - NEW HEIGHT FIELD AT 1000 MB.
C
C**********************************************************************
C
C INPUT ARGUMENTS
      real           PSMOD(IM,JNP),TSMOD(IM,JNP),PSL(IM,JNP),
     .               PHIS1(IM,JNP),Z1000(IM,JNP),
     .               pmand(mlev) 
C
C PHYSICAL CONSTANTS
       real           BETA,CP,GRAV,OMEGA2,RADE,RGAS,ROCP,CPOG,
     .                GAMMA,ROCP1,GOCP
       COMMON /PHYSIC/ BETA, CP, GRAV, OMEGA2, RADE, RGAS, ROCP,
     .                 CPOG, GAMMA, ROCP1, GOCP
 
       REAL KAP, KAP1
C
C DEFINE CONSTANT KAPPA = ROCP
      KAP  = ROCP
      KAP1 = ROCP1
C
      BETAP = .0552
C
C LOOP OVER ALL THE GRID POINTS.
C ---------------------------------------------------------------------
 
      DO 200 J = 1,JNP
      DO 200 I = 1,IM
C
C Perform the computations looping over all grid points. 
C ---------------------------------------------------------------------
 
        PSFG = PSMOD(I,J)
        TSFG = TSMOD(I,J)
        ZSURF = PHIS1(I,J)/GRAV
        TMID = TSFG + 0.5*BETA*ZSURF
C
C COMPUTE NEW SURFACE PRESSURE FROM PSL , REVISE SURFACE TEMPERATURE.
C ---------------------------------------------------------------------
 
        PSURF = PSL(I,J)*EXP(-GRAV/RGAS*ZSURF/TMID)
        TSURF = TSFG + (PSURF - PSFG) * BETAP
 
        TSMOD(I,J) = TSURF
C
C FIND A MANDATORY PRESSURE LEVEL JUST BELOW SURFACE.
C ---------------------------------------------------------------------
 
        DO 100 L = 1,MLEV
          IF(PMAND(L) .GT. PSURF) GO TO 101
 100    CONTINUE
        l=mlev
 
C
C COMPUTE 1000MB HEIGHTS .
C ---------------------------------------------------------------------
 
 101    CONTINUE
 
        ZUP = ZSURF
        TUP = TSURF
        PUP = PSURF
        PDN   = PMAND(L)
 
        IF( ABS(PUP-PDN) .LT. .01 ) THEN
          ZDN = ZUP
          GO TO 199
        END IF
 
102     PMIDK = (PUP**KAP1 - PDN**KAP1)/(KAP1*(PUP - PDN))
        PMID  = PMIDK**(1./KAP)
        TMID  = TUP + (PMID - PUP)*BETAP
 
        ZDN   = ZUP + CPOG*TMID*(PUP**KAP - PDN**KAP)/PMIDK
 
199     IF(L.LT.mlev) THEN
          TUP = TUP + (PDN - PUP)*BETAP
          PUP = PDN
          ZUP = ZDN
          L  = L + 1
          PDN = PMAND(L)
          GO TO 102
        END IF
 
        Z1000(I,J) = ZDN
 
 200  CONTINUE
 
      RETURN
      END
