
 
      subroutine delmix ( nymd, nhms, rhmod, rmixmod, pmand, psfc,
     &                    data, nobs, im, jnp, mlev, mmix, sigdat,
     &                    bq, fg_t )
 
!
! Program Name: DELMIX
!         Date:   1984
!   Programmer: Y. Brin
!
! Program DELMIX reads rh/mixing ratio data, does gross and buddy
! checks, and returns mixing ratio deltas.
!
! Modifications:
!
!     6/91  M.Seablom   Adjustable dimensions for DATA array.
!     6/92  M.Seablom   RH and mixing ratio values checked for
!                       consistency by using emperical saturation
!                       vapour pressure formula.
!     2/93  D.Lamich    chkmix not called if nlobs = 0
c..................................................................c
!                                                                  !
! NASA Goddard Space Flight Center       Data Assimilation Office  !
!                                                                  !
! Goddard Earth Observing System                                   !
! Data Analysis System - Version 2.0                               !
! February, 1994                                                   !
!                                                                  !
!------------------------------------------------------------------c
 
      real  data(1500000), psfc(im,jnp), pmand(mlev), 
     &      rhmod(im,jnp,mlev), rmixmod(im,jnp,mlev), typesum,
     &      fg_t(im,jnp,mlev)
 
      common /scan/     maxobs, rscan, onestage, max_data
 
      logical           strat,   slp,     fdcos,   fexp,    flow,
     &                  sigdat,  onestage, repack

C  DATA POINTERS
      common /dataptrs/ ikx, ilat, ilon, ipmix, itype, imix, ifcer,
     &                  itime
 
      integer           bq(150000)
 
C  DATA AT A PRESSURE LEVEL .
      COMMON /LEVEL/    INDX(20000), KX(20000), LAT(20000), LON(20000),
     $                  PR(20000),   RH(20000), MIX(20000), DIFF(20000),
     $                  QDATA(20000)
      real              INDX, KX, LAT, LON, MIX
      DIMENSION         TYPE(20000)
      EQUIVALENCE     ( RH(1), TYPE(1) )
      INTEGER           QDATA
 
C  PRINT FLAG
      COMMON /PRINT/    MSGLEV
 
C  MANDATORY PRESSURE LEVELS
      COMMON /PRMAND18/ PMAND18(18),  ALOGP18(18), PKAP18(18),
     $                  PKAP118(18),  PM18(18)
 
      IF( NOBS.EQ.0 ) RETURN
 
C  INITIALIZE TYPE PART OF DATA ARRAY TO 0.0
      DO 100 I = 0, NOBS-1
         DATA(ITYPE+I) = 0.0
  100 CONTINUE
      IRH = IMIX + NOBS
 
C  LOOP OVER THE PRESSURE LEVELS, TOP TO BOTTOM
      TYPESUM = 0.0
      lev1 = mlev - mmix + 1
      DO 500 L = lev1,mlev
 

         CALL GETMIX( L, DATA(IKX), DATA(ILAT), DATA(ILON), DATA(IPMIX),
     $                DATA(IMIX), DATA(IRH), BQ, NOBS, NLOBS,
     &                pmand, mlev, mmix, sigdat )
 
         IF( MSGLEV.GT.3 ) WRITE( 6, 900 ) PMAND(L), NLOBS
 
C  COMPUTE DELTAS AND DO QUALITY CONTROL FOR MOISTURE DATA

         if (nlobs.gt.0) then
            call chkmix ( L, rhmod, rmixmod, nlobs, pmand, im, jnp,
     &                    mlev, mmix, sigdat, fg_t )
         end if
 
C SCATTER MIXING RATIO DELTAS AND TYPE INDICATOR INTO MAIN DATA ARRAY
         DO 200  I = 1, NLOBS
            KK = INDX(I) - 0.9
            DATA(IMIX+KK)  = MIX(I)
            DATA(ITYPE+KK) = TYPE(I)
            TYPESUM = TYPESUM + TYPE(I)
  200    CONTINUE
 
  500 CONTINUE
 
C  PRINT OUT TOTAL NUMBER OF DELTAS ACCEPTED .
      WRITE( 6, 910 ) TYPESUM
 
c  Write out to DEL dataset.
 
      write(91) nobs, nymd, nhms, 'MIX'
      do i = 1,nobs
       write(91) data(ilat+i-1), data(ilon+i-1), data(itype+i-1),
     &          data(ipmix+i-1), data(ikx+i-1), 0., 0., data(imix+i-1)
      end do
 
      call rptlly ( data(ikx), data(itype), nobs, 3 )
 
      RETURN
  900 FORMAT(' DELMIX: * * *  LEVEL = ',F10.2,'     NUMBER OF OBS ',I8)
  910 FORMAT('0DELMIX: * *  TOTAL NUMBER OF ACCEPTED DELTAS = ',F8.2,/)
      END
