      subroutine obscov ( ocv, zkx, gi, gj, obserr, kq, pi, pj,
     &                    d, xki, hco, vco, nz, ztime )
                                                                        
c..................................................................c
!                                                                  !
! NASA Goddard Space Flight Center       Data Assimilation Office  !
!                                                                  !
! Goddard Earth Observing System                                   !
! Data Analysis System - Version 2.0                               !
! April, 1994                                                      !
!                                                                  !
!------------------------------------------------------------------c

!     Programmer: S. Bloom
!     Date:       July, 1990

C     ROUTINE TO GENERATE THE OBSERVATIONAL ERROR COVARIANCES (BOTH     
C     HORIZONTAL AND VERTICAL)                                          
C                                                                       
C     INPUTS:                                                           
C                                                                       
C             D         OBS-OBS DISTANCES (FROM MMCOV)                  
C             GI        GETI                                            
C             GJ        GETJ                                            
C             NZ        NUMBER OF HEIGHT OBSEVATIONS                    
C             PI        PRSI (OBS PRESSURES AT 'I' POINTS)              
C             PJ        PRSJ (OBS PRESSURES AT 'J' POINTS)              
C                                                                       
C     OUTPUT:                                                           
C                                                                       
C             OCV       OBS-OBS HEIGHT ERROR COVARIANCES                
C                                                                       
C                                                                       
C     THIS CODE REPLACES A SECTION OF ORIGINAL ZUVCOV ROUTINE           
C                                                                       
C     S. BLOOM   7/90                                            
c     M. SEABLOM 9/91 Removed this subroutine; replaced with version
c                     using modified matrix setup to accomodate large
c                     volumes.
c     M. SEABLOM 5/92 Resurrected original subroutine for use in low
c                     and middle latitude minivolumes.
c     M. SEABLOM 8/93 Enlarged IARR and IHOLD to incorporate larger
c                     amounts of data in minivolumes.
!     M. SEABLOM 4/94 Decoupled satellite error correlations.
C----------------------------------------------------------------       
                                                                        
                                                                        

! ( Parameter statements will not be used in Fortran 90 version!
!   Temporary fix to get around dynamic allocation problems. )

      parameter ( max_nh=600, maxrank=75 )
      real    ocv(*), zkx(*), gi(*), gj(*), kq(*), d(*), pi(*),
     &        pj(*), xki(*), hco(*), vco(*), obserr(200,4,19),
     &        ztime(*)

      integer  iarr(99000), ihold(99000)
      task common /ztimes/ zti1((max_nh+maxrank)*maxrank),
     &                     zti2((max_nh+maxrank)*maxrank)
                                                                        
      COMMON /OBSCORR/ VCORLIST(200),HCORLIST(200),VCOROE(13,13,4),     
     .                 HCOROE(1200,5)                                    
                                                                        
      DATA ZERO/0.0/                                                    
      DATA ONE/1.0/                                                     
      DATA TENTH/0.10/                                                  
      DATA TENM3/.001/                                                  
                                                                        
C------------------------------------------------------------           
      I2(I) = I*(I+1)/2                                                 
                                                                        
C------------------------------------------------------------           
      NZS = (NZ*(NZ+1))/2                                               
                                                                        
      CALL A77HGATH(ZKX,GI,XKI,NZS)                                     
      CALL A77HGATH(ZKX,GJ,HCO,NZS)                                     
      call a77hgath(ztime,gi,zti1,nzs)
      call a77hgath(ztime,gj,zti2,nzs)
                                                                        
      CALL A77HGATH(VCORLIST,HCO,VCO,NZS)                               
      CALL A77HGATH(HCORLIST,XKI,OCV,NZS)                               
                                                                        
      NCOUNT = 0                                                        
C                                                                       
C     FIRST SECTION COUNTS THE NUMBER OF OBSERVATION PAIRS TO           
C     CORRELATE (NCOUNT). ALSO COMPRESSES THE ARRAYS, REMEMBERING       
C     THE CORRECT LOCATION IN IARR.                                     
C                                                                       
C     THE COMPRESSION STEP USES IHOLD AS A "PLACEHOLDER", WITH ALL      
C     ELEMENTS INITIALLY ONE.  THEN DIAGONAL ELEMENTS ARE SET TO ZERO.  
C     FINALLY, ALL APPROPRIATE LOCATIONS ARE MARKED WITH INTEGERS.      
C     INDEX ARRAY "IARR" IS OBTAINED BY USING WHENIGT FUNCTION, WHICH   
C     POINTS TO ALL NON-ZERO ELEMENTS OF IHOLD.                         
C                                                                       
                                                                        
      ihold(1:nzs) = 1                                                  
                                                                        
      do n = 1,nz                                                       
          IHOLD( I2(N) ) = 0                                            
      end do                                                            
                                                                        
      do k = 1,nzs

!        if ( (xki(k) .eq. hco(k)  .and.  vco(k) .ne.0.)
!    &        .and.  ihold(k) .ne. 0  .and.
!    &        (abs(zti1(k)-zti2(k)) .lt. 90.) ) then
         if ( (xki(k) .eq. hco(k)  .and.  vco(k) .ne.0.)
     &        .and.  ihold(k) .ne. 0  )then
                                                                        
              ncount   = ncount + 1                                     
              ihold(k) = ncount                                         
          else                                                          
              ihold(k) = 0                                              
          endif                                                         

      end do
                                                                        
      if ( ncount .eq. 0 ) then                                         
           ocv(1:nzs) = 0.
           RETURN                                                       
      endif                                                             
      CALL WHENIGT(NZS,IHOLD,1,0,IARR,NCNT)                             
      IF(NCNT.NE.NCOUNT) WRITE(6,401) NCNT,NCOUNT                       
 401  FORMAT('  WHENIGT PROBLEM, NCNT  NCOUNT ',2I8)                    
                                                                        
              D(1:NCOUNT)  =   D(IARR(1:NCOUNT))                        
             PI(1:NCOUNT)  =  PI(IARR(1:NCOUNT))                        
             PJ(1:NCOUNT)  =  PJ(IARR(1:NCOUNT))                        
            XKI(1:NCOUNT)  = XKI(IARR(1:NCOUNT))                        
            VCO(1:NCOUNT)  = VCO(IARR(1:NCOUNT))                        
            OCV(1:NCOUNT)  = OCV(IARR(1:NCOUNT))                        
C                                                                       
C     NEXT SECTION GENERATES HORIZONTAL CORRELATIONS,                   
C     CHECKING FOR RAWINSONDES (ITY=-1) WHICH ARE NOT                   
C     HORIZONTALLY CORRELATED (HENCE THE CHECK FOR D >0,                
C     SINCE RAWINSONDES HAVE VERTICAL CORRELATION...)                   
C                                                                       
                                                                        
      DO 30 L=1,NCOUNT                                                  
                                                                        
          ID  = INT( .1   *D(L) + .001 )                                
                                                                        
          R   = .1   *D(L) - ID                                         
                                                                        
          ID  = ID + 1                                                  
          ID1 = ID + 1                                                  
          ITY = INT( OCV(L) + .001 )                                    
                                                                        
          IF(ITY .GT. 0) THEN                                           
                                                                        
              T1     = HCOROE(ID ,ITY)                                  
              T2     = HCOROE(ID1,ITY)                                  
                                                                        
              OCV(L) = T1 + R * ( T2 - T1 )                             
                                                                        
          ELSE                                                          
                                                                        
              OCV(L) = 1.0                                              
                                                                        
              IF( D(L) .GT. 0.0  ) OCV(L) = 0.0                         
                                                                        
          ENDIF                                                         
 30   CONTINUE                                                          
C                                                                       
C     THIS SECTION OBTAINS THE OBSERVATION HEIGHT ERROR VARIANCES,      
C     AND ACCUMULATES THEM INTO OCV.                                    
C                                                                       
C     NOTE: REDEFINING VARIABLES TO SAVE SPACE,                         
C             D   = 'OBERI' (IN OLD ZUVCOV CODE)                        
C             HCO = 'OBERJ' ( " )                                       
C                                                                       
                                                                        
      CALL OBERTRP0(XKI,PI,D,OBSERR,KQ(1),NCOUNT)                       
      CALL OBERTRP0(XKI,PJ,HCO,OBSERR,KQ(1),NCOUNT)                     
                                                                        
      DO 40 L=1,NCOUNT                                                  
          OCV(L) = D(L) * HCO(L) * OCV(L)                               
 40   CONTINUE                                                          
C                                                                       
C     THIS SECTION OBTAINS THE VERTICAL CORRELATION TERMS, TAKING       
C     CARE TO USE THE CORRECT VERTICAL CORRELATION FUNCTION FOR EACH    
C     INSTRUMENT                                                        
C                                                                       
      DO 50 L=1,NCOUNT                                                  
          HCO(L) = 0.0                                                  
 50   CONTINUE                                                          
                                                                        
      CALL VCORROE(PI,PJ,HCO,VCO,NCOUNT)                      
                                                                        
C     FINALLY, COMBINE THE RESULTS.                                     
C             1. COMBINE VERTICAL CORRELATIONS WITH REST OF TERMS       
C                (SAVING COMPRESSED COVARIANCES IN VCO)                 
C                                                                       
C             2. ZERO THE OCV ARRAY                                     
C                                                                       
C             3. LOAD COVARIANCES INTO PROPER LOCATIONS IN OCV USING    
C                IARR                                                   
C                                                                       
      DO 75 L=1,NCOUNT                                                  
          VCO(L) = OCV(L) * HCO(L)                                      
 75   CONTINUE                                                          
                                                                        
      DO 80 K=1,NZS                                                     
          OCV(K) = 0.0                                                  
 80   CONTINUE                                                          
                                                                        
      DO 90 L=1,NCOUNT                                                  
          OCV( IARR(L) ) = VCO(L)                                       
 90   CONTINUE                                                          
      RETURN                                                            
      END                                                               
