      
      SUBROUTINE PUVCOV(NREP,IGP,IREG,LEV,LEVLINDX,OBSERR,IRANK,
     &                  data, nobs, symat, delta, force, gp, ztx,
     &                  psl_err, u_err, v_err )
 
! PUVCOV forms a covariance matrix for p-u-v sea level analysis using the
! damped cosine formulas.
!
! The order of the matrix is returned to SOLVEDRV.         
!
!
c..................................................................c
!                                                                  !
! NASA Goddard Space Flight Center       Data Assimilation Office  !
!                                                                  !
! Goddard Earth Observing System                                   !
! Data Analysis System - Version 2.0                               !
! February, 1994                                                   !
!                                                                  !
!------------------------------------------------------------------c
!
!  Record of modifications:
!        J.WOOLLEN 4/84
!        M.NESTLER 7/84
!        J.WOOLLEN 9/85
!        J.WOOLLEN 9/86
!        M.SEABLOM 6/89    * Tropical decoupling added.
!        S.Bloom   9/90    * Original Cray version.
!        M.Seablom 8/91    * Upgrade to 1.2; variable minivolumes.
!        M.Seablom 3/93    * Parallelization of matrix solves.
!        M.Seablom 8/93    * Optional removal of data selection.
!
 
 
      parameter ( max_nh=600, maxrank=75 )
      parameter  ( max_huge=((max_nh+maxrank+1)*(max_nh+maxrank))/2 )
      real levlindx(nrep), obserr(200,4,19), delta(maxrank),
     &     gp(max_nh), force(maxrank,max_nh,3), data(1500000),
     &     symat(maxrank*(maxrank+1)/2), ztx(150000),
     &     psl_err(im*jnp,mlev+1),u_err(im,jnp,mlev+1),
     &     v_err(im,jnp,mlev+1)
 
      common /dataptrs/ ikx,ilat,ilon,iprs,ityp,idel,istd,itime
      common /scan/     maxobs, rscan, onestage, max_data
      common /gridxx/  im,jnp,mlev
      common /sfcdrag/ drag(13104),iwater(13104)
 


      task common /zuvmat/
     &  plat(max_nh+maxrank),plon(max_nh+maxrank),pstd(max_nh+maxrank),
     &  pkx (max_nh+maxrank),pdel(max_nh+maxrank),
     &  wlat(max_nh+maxrank),wlon(max_nh+maxrank),wdrg(max_nh+maxrank),
     &  wstd(max_nh+maxrank),
     &  wkx(max_nh+maxrank),siny(max_nh+maxrank),
     &  cosy(max_nh+maxrank),plst(max_nh+maxrank),udel(max_nh+maxrank),
     &  vdel(max_nh+maxrank),
     &  zlst(max_nh+maxrank),wlst(max_nh+maxrank),type(max_nh+maxrank),
     &  bc(max_nh+maxrank),bf(max_nh+maxrank),
     &  gdec(max_nh+maxrank),vfac(max_nh+maxrank),twstd(max_nh+maxrank),
     &  glat(max_nh+maxrank),glon(max_nh+maxrank),gprs(max_nh+maxrank),
     &  gstd(max_nh+maxrank),
     &  cv1(max_huge), cv2((max_nh+maxrank)*maxrank),
     &  cv3((max_nh+maxrank)*maxrank),
     &  cv4((max_nh+maxrank)*maxrank),
     &  cv5((max_nh+maxrank)*maxrank),
     &  cv6((max_nh+maxrank)*maxrank),
     &  w1(max_huge), w2(max_huge),
     &  w3(max_huge), w4(max_huge),
     &  w5(max_huge), w6(max_huge),
     &  ztime(max_nh+maxrank)


	parameter (zero=0.0,one=1.0,two=2.0,onept1=1.10)
	parameter (ninety=90.0,one80=180.0,tenth=0.10,pt05=0.05)
	parameter (tenm4=0.0001)

 
      COMMON /PUV/ CRSCOV
 
      common /sparam/ lsolve, ns, nr, nv, nh(13104), nq,
     &                kq(3), volcentre_x(13104),
     &                ngv, volcentre_y(13104), glatup(13104),
     &                glonup(13104)
 
      character*1 qi,qj
      character*60 message
      integer     num(maxrank), nump(maxrank), numw(maxrank),
     &            ipoint(maxrank)
      logical     water, unislp, onestage
 
!  --- Statement functions -----------------------!
      I1(I) = I*(I-1)/2                           !
      I2(I) = I*(I+1)/2                           !
                                                  !
      JX(XLAT,DLAT) = (XLAT+NINETY)/DLAT + ONEPT1 !
      IX(XLON,DLON) = (XLON+ONE80)/DLON  + ONEPT1 !
!-------------------------------------------------!
      message(1:60) = ' '
 
      CALL GETCONH('RADE    ',RADE  )
      CALL GETCONH('RADEI   ',RADEI )
      CALL GETCONH('RADEI2  ',RADEI2)
      cALL GETCONH('OMEGA2  ',OMEGA2)
      CALL GETCONH('RHOBAR  ',RHOBAR)
      cALL GETCONH('PI180   ',PI180 )
      CALL GETCONI('MAXREP  ',MAXREP)
      CALL GETCONL('SLP     ',UNISLP)
 
         IF(     UNISLP) THEN
            CRSCOV = ZERO
         ELSE
            CRSCOV = ONE
         END IF
 
      if ( maxrep*3 .gt. maxrank ) then
            message = ' PUVCOV: Excessive amount of data.'
            call flash( 5,message,1 )
      end if

 
      call getconh('DLAT    ',dlat)
      call getconh('DLON    ',dlon)

      force( 1:maxrank, 1:max_nh, 1:3 ) = 0.0
      delta( 1:maxrank ) = 0.0
      symat( 1:(maxrank*(maxrank+1)/2) )= -1.

      do i = 1,maxrank
        num(i) = i
      end do
 
      irank = 0
      nhgp = nh( igp )
 
!  Locate grid points within a volume.

      nh_count = 0
      nhnv_count = 0

      do j = 1,2
         do i = 1,nhgp/2       

           nh_count = nh_count + 1
           glatx = glatup( igp ) + ( j-1 )*dlat
           if ( glatx .gt. 90. ) glatx = 90.
           glat( nh_count ) = glatx
           glonx = glonup( igp ) + ( i-1 )*dlon
           glon( nh_count ) = glonx
           iigp = ( jx(glatx,dlat)-1)*im + ix( glonx,dlon )
           gp( nh_count ) = iigp
           gstd( nh_count ) = psl_err(iigp,mlev+1)

         end do   !        <-- End looping over latitudes.
      end do      !        <-- End looping over longitudes.
                                                                                
 
C  MAKE LISTS OF PRESSURE AND WIND DATA
C  ------------------------------------
 
      NP = 0
      NW = 0
 
      CALL A77HGATH(DATA(ITYP),LEVLINDX,TYPE,NREP)
 
      ICOUNT = 0
 
      DO 5006 IPT=1,NREP
        IF(TYPE(IPT) .NE. TWO) THEN
              ICOUNT       = ICOUNT + 1
              PLST(ICOUNT) = LEVLINDX(IPT)
              NUMP(ICOUNT) = NUM(IPT)
        END IF
 5006 CONTINUE
 
      NP = ICOUNT
 
      ICOUNT = 0
 
      DO 5007 IPT=1,NREP
        IF(TYPE(IPT) .NE. ONE) THEN
              ICOUNT       = ICOUNT + 1
              WLST(ICOUNT) = LEVLINDX(IPT)
              NUMW(ICOUNT) = NUM(IPT)
        END IF
 5007 CONTINUE
 
      NW = ICOUNT
 
C.......................................................
C
C     IF THE MINI-VOLUME IS NOT A WATER POINT
C     THEN ONLY PERFORM PRESSURE ANALYSIS
C
C.......................................................
 
      KXX   = (JX(GLATUP(IGP),DLAT) - 1)*IM + IX(GLONUP(IGP),DLON)
 
      WATER = IWATER(KXX).EQ.1
 
      IF(.NOT.WATER) NW = 0
 
      IF(NP.EQ.0 .AND. NW.EQ.0) IRANK = 0
      IF(NP.EQ.0 .AND. NW.EQ.0) RETURN
 
C  GATHER STUFF FOR PRESSURE-PRESSURE COVARIANCE
C  ---------------------------------------------
 
      DO 5008 MGV=1,nhgp
 
         PLAT(NP+MGV) = GLAT(MGV)
         PLON(NP+MGV) = GLON(MGV)
         PSTD(NP+MGV) = GSTD(MGV)
 5008 CONTINUE
 
      IF(NP.NE.0) THEN
 
         CALL A77HGATH(DATA(ILAT),PLST,PLAT,NP)
         CALL A77HGATH(DATA(ILON),PLST,PLON,NP)
         CALL A77HGATH(DATA(ISTD),PLST,PSTD,NP)
         CALL A77HGATH(DATA(IDEL),PLST,PDEL,NP)
         CALL A77HGATH(DATA(IKX ),PLST,PKX ,NP)
 
C
C..... LOAD IN THE GRIDPOINT INFO FOR THE MINIVOLUME PTS
C
 
C  OBTAIN   PRES - PRES ERROR COVARIANCES
C  --------------------------------------
C
C     PRESSURE-PRESSURE COVARIANCE WORKING ARRAY
C
 
         NP2 = (NP+nhgp)*np
         NPS = NP*(NP+1)/2
         npf = np + nhgp
 
ccmic$ guard
         call mmcovh ( 'PUV', ireg, npf, np2, plon, plat,
     &                 pstd, cv1, w1, w2, w3, w4, w5, w6, np )
ccmic$ endguard
 
C  LOAD OBS-OBS COVARIANCES INTO SYMMETRIC MATRIX 'SYMAT'
C  ------------------------------------------------------
C  (NB..HERE OBS-OBS MEANS F.E.'S AT OBS LOCATIONS)
 
 
         do kz = 1,np

           izz = i1( kz )
           ioffset = ( kz-1 )*np
           
           do kzz = 1,kz
              symat( izz+kzz ) = cv1( ioffset+kzz )
           end do
         end do

C  LOAD GRID-OBS COVARIANCES INTO ARRAY 'FORCE'
C  --------------------------------------------
 
         IOS = 0
         nos = np*np
         do kg = 1,nhgp*nv
            ios = nos + ( kg-1 )*np
            do kz = 1,np
               force( kz,kg,1 ) = cv1( ios+kz )
            end do
         end do
 
      END IF

 
C  DECIDE IF A WIND ANALYSIS IS REQUIRED
C  -------------------------------------
 
      IF(.NOT.WATER) GOTO 100
 
C  GATHER STUFF FOR COVARIANCES WITH WIND TERMS
C  --------------------------------------------
 
      IF(NW.NE.0) THEN
         CALL A77HGATH(DATA(ILAT       ),WLST,WLAT,NW)
         CALL A77HGATH(DATA(ILON       ),WLST,WLON,NW)
         CALL A77HGATH(DATA(ISTD       ),WLST,WSTD,NW)
         CALL A77HGATH(DATA(IDEL+  NOBS),WLST,UDEL,NW)
         CALL A77HGATH(DATA(IDEL+2*NOBS),WLST,VDEL,NW)
         CALL A77HGATH(DATA(IKX        ),WLST,WKX ,NW)
      ENDIF
 
      DO 5016 MGV=1,nhgp
 
        WLAT(NW+MGV) = GLAT(MGV)
        WLON(NW+MGV) = GLON(MGV)
        WSTD(NW+MGV) = GSTD(MGV)
 
 5016 CONTINUE
 
      NWF = NW+nhgp
 
      DO 5017 L=1,NWF
 
          SINY(L) = SIN(WLAT(L)*PI180)
          COSY(L) = COS(WLAT(L)*PI180)
 
          IF(SINY(L).GT.-PT05 .AND. SINY(L).LT.ZERO) SINY(L) = -PT05
          IF(SINY(L).LT. PT05 .AND. SINY(L).GE.ZERO) SINY(L) =  PT05
          IF(COSY(L).EQ.ZERO) COSY(L) = TENM4
 
 5017 CONTINUE
 
C
C
C.... OBATAIN TROPICALLY DECOUPLED WIND ERROR VARIANCES (WITH
C     EKMAN BALANCE ASSUMPTION IN EXTRA-TROPICS)
C
C     IN ADDITION, COMPUTE THE EKMAN BALANCE COEFFICENTS BC & BF
C
 
      CALL WVARP(IREG,NWF,SINY,COSY,WLAT,WLON,WSTD,WDRG,BC,BF,SCALE)
 
C  COMPUTE THE WIND-PRESSURE COVARIANCES
C  -------------------------------------
C
C     WORKING ARRAYS
C
C     CV1 = UPCOV
C     CV2 = VPCOV
C
 
      IF(NP.NE.0) THEN
 
         NWP = NWF*NP
 
ccmic$ guard
         CALL WMCOV('PUV',IREG,NP,NWF,PLON,PLAT,PSTD,WLON,WLAT,WSTD,
     1              BC,BF,SCALE,CV1,CV2,W1,W2,W3,W4,W5,W6)
ccmic$ endguard
 
 
C
C.... STORE THE RESULTS IN APPROPRIATE LOCATIONS IN SYMAT AND FORCE
C
 
         IF(NW.NE.0) THEN
 
 
         DO 5025 KW=1,NW
 
            IUP = I1(NP+KW)
            IVP = I1(NP+NW+KW)
            IOS = (KW-1)*NP
 
            DO 5020 KP=1,NP
 
                   SYMAT(IUP+KP) = CV1(IOS+KP)
                   SYMAT(IVP+KP) = CV2(IOS+KP)
 
 5020       CONTINUE
 5025    CONTINUE
 
         END IF
 
         NOS = NW*NP
 
         DO 5035 KG=1,nhgp
 
            IOS = NOS + (KG-1)*NP
 
            DO 5030 KP=1,NP
 
               FORCE(KP,KG,2) = CV1(IOS+KP)
               FORCE(KP,KG,3) = CV2(IOS+KP)
 
 5030       CONTINUE
 5035    CONTINUE
 
      END IF
 
 
C  COMPUTE THE WIND-WIND COVARIANCES
C  ---------------------------------
C
C     WORKING ARRAYS
C
C     CV1 = PUCOV
C     CV2 = PVCOV
C
C     CV3 = UUCOV
C     CV4 = UVCOV
C     CV5 = VUCOV
C     CV6 = VVCOV
C
 
      IF(NW.NE.0) THEN
 
 
ccmic$ guard
      CALL WWCOV('PUV',IREG,NW,nhgp,NP,WLON,WLAT,WSTD,PSTD,BC,BF,CV1,
     1           cv2,CV3,CV4,CV5,CV6,W1,W2,W3,W4,W5,W6)
 
ccmic$ endguard
 
              DO 5050 KW=1,NW
 
                  IUU = I1(NP+KW)    + NP
                  IVU = I1(NP+NW+KW) + NP
                  IVV = I1(NP+NW+KW) + NP + NW
 
                  IOS = (KW-1)*NW
 
                  DO 5040 KWW=1,KW
 
                      SYMAT(IUU+KWW) = CV3(IOS+KWW)
                      SYMAT(IVV+KWW) = CV6(IOS+KWW)
 
 5040             CONTINUE
                  DO 5045 KWW=1,NW
                      SYMAT(IVU+KWW) = CV5(IOS+KWW)
 5045             CONTINUE
 5050         CONTINUE
 
 
              NOS = NW * NW
 
              DO 5070 KG=1,nhgp
 
                  IOS = NOS + (KG-1)*NW
 
                  DO 5060 KW=1,NW
 
                      FORCE(NP+KW   ,KG,1) = CV1(IOS+KW)
                      FORCE(NP+NW+KW,KG,1) = CV2(IOS+KW)
 
                      FORCE(NP+KW   ,KG,2) = CV3(IOS+KW)
                      FORCE(NP+NW+KW,KG,2) = CV4(IOS+KW)
                      FORCE(NP+KW   ,KG,3) = CV5(IOS+KW)
                      FORCE(NP+NW+KW,KG,3) = CV6(IOS+KW)
 
 5060             CONTINUE
 5070         CONTINUE
 
      END IF
 
C  ADD THE OBSEVATIONAL ERROR TO THE FORECAST ERROR
C  ------------------------------------------------
 
100   IF(NP.NE.0) THEN
         DO 110 I=1,NP
         IPP        = I2(I)
         KX         = PKX(I)+TENTH
         OBERR      = OBSERR(KX,2,19)
110      SYMAT(IPP) = SYMAT(IPP) + OBERR*OBERR
      ENDIF
 
      IF(NW.NE.0) THEN
         DO 120 I=1,NW
         KX         = WKX(I)+TENTH
         IUU        = I2(I+NP)
         OBERR      = OBSERR(KX,3,19)
         SYMAT(IUU) = SYMAT(IUU) + OBERR*OBERR
         IVV        = I2(I+NP+NW)
         OBERR      = OBSERR(KX,4,19)
120      SYMAT(IVV) = SYMAT(IVV) + OBERR*OBERR
      ENDIF
 
C  COUNT THE RANK AND STORE THE DELTAS
C  -----------------------------------
 
      IRANK = NP + NW + NW
 
      DO 5080 K=1,NP
 
         DELTA(K      ) = PDEL(K)
 
 5080 CONTINUE
 
      DO 5090 K=1,NW
 
         DELTA(NP+K   ) = UDEL(K)
         DELTA(NP+NW+K) = VDEL(K)
 
 5090 CONTINUE
 
C  FOR PRINTING INDIVIDUAL CASES
C  -----------------------------
 
      IF(IGP.eq.0000) THEN
 
         PRINT 912,IGP
912      FORMAT(' PUVCOV: Diagnostic printout for gridpoint ',i5,//)
         IF(NP.NE.0) THEN
            PRINT*, ' Values of I, PKX, PLAT, PLON, PDEL, PSTD:'
            DO 102 I=1,NP
102         PRINT914,I,PKX (I),
     .                 PLAT(I),
     .                 PLON(I),
     .                 PDEL(I),
     .                 PSTD(I)
         ENDIF
         IF(NW.NE.0) THEN
            WRITE(6,'(///)')
            PRINT*, ' Values of I,KX,LAT,LON,UDEL,VDEL,STD, AND DRAG:'
            DO 103 I=1,NW
103         PRINT914,I,WKX (I),
     .                 WLAT(I),
     .                 WLON(I),
     .                 UDEL(I),
     .                 VDEL(I),
     .                 WSTD(I),
     .                 WDRG(I)
            PRINT*, ' Values of BF and BC:', BF(I), BC(I)
         ENDIF
914      FORMAT(' ',I5,6F12.2,2X,E12.6)
C        PRINT*, ' VALUES OF OBSERVATIONAL ERROR --------- '
C        DO 1066 IMIKE = 2,4
C           PRINT*, ' VARIABLE TYPE = ', IMIKE
C           DO 1067 JMIKE = 1,200
C             PRINT*, OBSERR(JMIKE,IMIKE,19)
C1067       CONTINUE
C1066    CONTINUE
         IP = NP
         IU = NP+NW
 
         DO 101 I=1,IRANK
 
         IF(I.LE.IP) THEN
            QI   = 'P'
            NUMI = I
         ENDIF
 
         IF(I.GT.IP .AND. I.LE.IU) THEN
            QI   = 'U'
            NUMI = I-NP
         ENDIF
 
         IF(I.GT.IU) THEN
            QI   = 'V'
            NUMI = I-IU
         ENDIF
 
         DO 101 J=1,I
 
         IF(J.LE.IP) THEN
            QJ   = 'P'
            NUMJ = J
         ENDIF
 
         IF(J.GT.IP .AND. J.LE.IU) THEN
            QJ   = 'U'
            NUMJ = J-NP
         ENDIF
 
         IF(J.GT.IU) THEN
            QJ   = 'V'
            NUMJ = J-IU
         ENDIF
 
 
         IJ = I*(I-1)/2 + J
 
 101     PRINT 911,QI,NUMI,QJ,NUMJ,IJ,SYMAT(IJ)
 911     FORMAT(' ',A2,I2,A5,I2,I10,G15.4)
 
         PRINT*, ' Printout of forcing terms:'
 
         PRINT 913,(FORCE(I,1,1),I=1,IRANK)
         PRINT 913,(FORCE(I,1,2),I=1,IRANK)
         PRINT 913,(FORCE(I,1,3),I=1,IRANK)
913      FORMAT(10G12.4)
 
      ENDIF
 
      RETURN
      END
