c.....................................................................
c:                                                                   :
c:  Program:    RETSUB                                               :
c:                                                                   :
c:  Language:   Fortran V (ANSI77 with Fortran 90 enhancements)      :
c:                                                                   :
c:  Programmer: Mike Seablom              Date: May 6, 1991          :
c:              NASA/GSFC Code 911                                   :
c:              Greenbelt, Md. 20771                                 :
c:           (301) 286-7360  (FTS) 888-7360                          :
c:                                                                   :
c:  RETSUB positions the retrieval dataset to the proper date/time   :
c:  and passes the data back to RDHUV.                               :
c:                                                                   :
c.....................................................................
 
***********************************************************************
*                                                                     *
*   N A S A   O P T I M U M   I N T E R P O L A T I O N   -   1 . 1   *
*                                                                     *
*   NNN    NN     AA     SSSSSSSS      AA        Global Modeling &    *
*   NN N   NN    AAAA   SSS           AAAA       Simulation Branch    *
*   NN  N  NN   AA  AA   SSSSSSSS    AA  AA                           *
*   NN   N NN  AA    AA        SSS  AA    AA          G S F C         *
*   NN    NNN AA      AA SSSSSSSS  AA      AA                         *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
 
      subroutine RETSUB ( retrieval_count, nretymd, nrethms, rkx, rlat,
     &                    rlon, retv, rpref )
 
      real tau(200), latitude(200), longitude(200), scm_kx(200),
     &     OI_KX(200), REF_PRESSURE(200), HEIGHTS(200,15), PMAND(20)
     &    ,hour, month, rkx(12000), rlat(12000), rlon(12000),
     &     RETV(12000,15), RPREF(12000), ZENANG(12000), gravity,
     &     TOPOG(200), REF2SUR(200), RMSERR(200), PSR(200)
 
      common /CONTRL/ im, jnp, nb, ndalt, nhms, nlay, nlaym1, nlayp1,
     &                nymd, strat, idimen, slp, fdcos, fexp, saveae,
     &                sigdat, maxrep
 
      logical final_data_group, partial_record
      integer dummy, record_count, retrieval_count, err_test, time
      character * 6 ds_date
      character *60 flash_message
 
      data pmand / 1000., 850., 700., 500., 400., 300., 250.,
     &              200., 150., 100.,  70.,  50.,
     $              30.,  20.,  10., 7., 5., 3., 2., 1. /
 
      data partial_record /.false./, gravity/ 9.81 /
      data   final_data_group /.false./, record_count/0/, fill/1.e15/
      flash_message(1:60) = ' '
 
 
c  Compute tau.
 
      call deldate ( nymd, requested_tau )
 
 
      requested_tau = requested_tau - 720560.
      requested_tau = requested_tau * 24.
 
c  Add in "overlap" hours.
 
      requested_tau = requested_tau + float( nhms/10000 )
      begin_tau = requested_tau - 3.
      end_tau   = requested_tau + 3.
 
      write ( 6, '(//,'' RETSUB:Searching for retrievals between '',
     &       f12.0, '' and '',f12.0,/)' )   begin_tau, end_tau
 
 
c  Retrievals are stored in groups of 200. The record counter is
c  incremented based upon useable data only; the final record is
c  padded.
 
 
      retrieval_count  = 0
      final_data_group = .false.
      rewind (21)
 
      do while ( .not. final_data_group )
 
         read ( unit=21, iostat=err_test )
     &     dummy, record_count,
     &   ( tau(i),       i = 1,200          ),
     &   ( latitude(i),  i = 1,200          ),
     &   ( longitude(i), i = 1,200          ),
     &   ( scm_kx(i),    i = 1,200          ),
     & ( ( HEIGHTS(I,J), I = 1,200         ), J = 1,15 ),
     &   ( RMSERR      (I), I = 1,200          ),  ! quality indicator
     &   ( REF_PRESSURE(I), I = 1,200          ),  ! reference pressure
     &   ( TOPOG       (I), I = 1,200          ),  ! topography
     &   ( REF2SUR     (I), I = 1,200          ),  ! z surface to ref
     &   ( PSR         (I), I = 1,200          ),  ! surf. pres. for ret
     &   ( ZENANG      (I), I = 1,200          )   ! sat. scan angle
 
 
         if ( err_test .ne. 0 ) then
            final_data_group = .true.
         else
 
         do i = 1,record_count
 
           if ( tau(i) .gt. begin_tau .and. tau(i) .le. end_tau ) then
 
              retrieval_count = retrieval_count + 1
              if ( retrieval_count .gt. 12000 ) then
                write(flash_message,'("RETSUB: Number of retrievals",
     &                " exceeds 12000.")')
                call flash ( 5,flash_message,1 )
              end if
 
              rlat (retrieval_count) = latitude(i)
              rlon (retrieval_count) = longitude(i)
              rkx  (retrieval_count) = scm_kx(i)
              rpref(retrieval_count) = ref_pressure(i)
 
              do j = 1,15
               if ( rpref(retrieval_count) .gt. pmand(j) ) then
!!!             retv(retrieval_count,j) = heights(i,j) +
!!!  &              REF2SUR(i) + topog(i)
                retv(retrieval_count,j) = heights(i,j)
               else if ( rpref(retrieval_count) .eq. pmand(j) ) then
!!!             retv(retrieval_count,j) = REF2SUR(i) +  topog(i)
                retv(retrieval_count,j) = fill
               else if ( rpref(retrieval_count) .lt. pmand(j) ) then
                retv(retrieval_count,j) = fill
               end if
              end do
 
 
           else if ( tau(i) .gt. end_tau ) then
 
              final_data_group = .true.
              partial_record = .true.
 
           end if
 
         end do
         end if
 
      end do ! <-- End looping over groups of retrievals.
 
      write(6,'(" RETSUB: Number of retrievals encountered:",i9)')
     &     retrieval_count
 
!  Convert SCM KX's to OI KX's.
 
      if ( retrieval_count .eq. 0 ) then
 
        write(6,'(" RETSUB: No retrievals encountered; KXSCM2OI has",
     &        " been bypassed.")')
 
      else
 
        call kxscm2oi ( rkx, rkx, retrieval_count )
 
      end if
 
      return
      end
