 
      subroutine search( igp, lu, ld, datalist, nreps, data, nobs,
     &                   nrep )
 
C------------------------------------------------------------------C
!                                                                  !
!   Program Name: Search              Date:  May 16, 1992          !
!   Programmer  : J. Pfaendtner                                    !
!   Modifications: M. Seablom, D. Lamich                           !
!                                                                  !
!   NASA / DAO  Optimum Interpolation Version 1.6                  !
!   ----------------------------------------------                 !
!                                                                  !
!  Performs data selection for all volumes in a vertical column.   !
!  Selection algorithm is designed to search for mostly rawinsondes!
!  first, then other data types at mandatory levels, then left     !
!  over data at significant levels.                                !
!                                                                  !
!  Modifications:                                                  !
!                                                                  !
!   May 30, 1994: (Seablom) Forced selection of KX 88 (MTP)        !
!   Aug 28, 1994: (Seablom) Forced selection of KX 91 (UARS)       !
C------------------------------------------------------------------C
 
 
      parameter ( maxnrep=30000, mxreps=75, ntrig=4, percentA=.2,
     &            percentB=.2, percentS=.5 )

      real  alat(maxnrep), alon(maxnrep), data(1500000), 
     &      datalist(mxreps,50), dheap(maxnrep), dist(maxnrep),
     &      pres(maxnrep), trigtab(450*ntrig+1)

      integer  iflag(maxnrep,0:4), iheap(maxnrep), indexd(maxnrep),
     &         itype(maxnrep), ixkxs(maxnrep), kheap(maxnrep),
     &         knt(0:4), markoff(maxnrep), nreps(50), num(0:4)
 
      logical  first, onestage, qprint

      character *60  message

 
      common /sparam/ lsolve, ns, nr, nv, nh(13104), nq,
     &                kq(3), volcentre_x(13104),
     &                ngv, volcentre_y(13104), glatup(13104),
     &                glonup(13104)
 
      common /dataptrs/ ikx, ilat, ilon, iprs, ityp, idel, ifce,
     &                   itime
 
      COMMON /scan/     maxobs, xscan, onestage, max_data  
 
      COMMON /GRIDXX/   IM,      JNP,     MLEV
 
      COMMON /OFFDATA/  KOFFLV(200)
 
      COMMON /PMH/      PMAND(18)
 
      COMMON /TRIGCOM/  NTRIGS, TRIGCON, TRIGTAB

      data first/.true./
c      save rscan, rade, maxrep
 
      message(1:60) = ' '
 

         if ( ntrig .ne. ntrigs ) then
           message = 'SEARCH: NTRIG does not equal NTRIGS.'
           call flash ( 5,message,1 )
         end if

         call getconh( 'RSCAN   ', RSCAN )
         call getconh( 'RADE    ', RADE )
         call getconi( 'MAXREP  ', MAXREP )

 
      qprint = igp.eq.0000
 
!  Gather all data within RSCAN kilometres of this data volume.

      vollat = volcentre_y( igp )
      vollon = volcentre_x( igp )

c     call zeitbeg ('dgsort ')
      call dgsort ( data(ilat),  data(ilon),  data(ityp),  data(iprs),
     &             data(ikx),   nobs,        nrep,        maxnrep,
     &             indexd,      pres,        alat,        alon,
     &             ixkxs,       itype,       rscan,       rade,
     &             vollat,      vollon,
     &             ntrigs,      trigcon,     trigtab,     dist        )
c     call zeitend
 
      if ( nrep .eq. 0 ) then
         nreps ( lu : ld : nv ) = 0
         return
      end if
 
      if ( nrep .gt. maxnrep ) then
        write(6,'(" NREP:",i10,3x," MAXNREP:",i10,"  (Fatal)")')
     &       nrep, maxnrep
        message = 'SEARCH: NREP exceeds MAXNREP.'
        call flash ( 5,message,1 )
      end if

!  Set MARKOFF = 1 for off-level and other secondary data types,
!  otherwise zero.
 
      markoff ( 1:nrep ) = kofflv ( ixkxs( 1:nrep ) )
 
      if (qprint) then
 
         print 8801, igp,vollat,vollon,nrep
 8801    format(/,' SEARCH: Data selected by DGSORT for horizontal',
     .          ' volume:',i5,2x,'lat = ',f6.1,2x,'lon =',f6.1,//,
     .          ' Number of reports around horizontal volume at all',
     .          ' levels:',i8)
 
         if (nrep.gt.0) then
 
            print 8811
 8811       format(/,4x,'indexd',7x,'lat',7x,'lon',6x,'pres',8x,'kx',
     .             6x,'type',6x,'dist',3x,'markoff',/)
            do i=1,nrep
               print 8814, indexd(i),alat(i),alon(i),pres(i),ixkxs(i),
     .                     itype(i),dist(i),markoff(i)
 8814          format(' ',i10,3f10.2,2i10,f10.2,i10)
            end do
 
         end if
 
      end if
 
C  LOOP OVER THE GRID VOLUME LEVELS
      DO 300 LEV = LU, LD, NV
 
!        if (qprint) then
!           print 7701, lev
!7701       format(' begin loop for level ',i4)
!        end if
 
         NREPOK = 0
         NREPS(LEV) = 0
 
         IF( LEV.LE.MLEV ) THEN
 
C  START UPPER AIR DATA SELECTION
            NUMCAT = 4
            PABOVE = 0.0
            IF( LEV.GT.1 ) PABOVE = PMAND(LEV-1)
            PBELOW = 2000.0
            IF( (LEV+NV).LE.MLEV ) PBELOW = PMAND(LEV+NV)
 
!           if (qprint) then
!              print 7703, pabove,pbelow
!7703          format(' pabove =',f10.2,2x,'pbelow =',f10.2)
!           end if
 
!  Initialize flags for selecting upper air data.
!
!    0    for rawinsonde data
!    1    for data at pres = pabove with markoff = 0
!    2    for data at pres = pbelow with markoff = 0
!    3    for data between pres = pabove and  pres = pbelow   with
!         markoff = 0
!    4    for data between pres = pabove and  pres = pbelow   with
!         markoff = 1
 
 
            knt ( 0:4 ) = 0
            iflag ( 1:nrep, 0:4 ) = 0
 
 
!  Count and flag the available data.
 
!           if (qprint) then
!              print 7301, igp
!7301          format(/,' SEARCH: Begin categorizing of data',
!    $                ' (loop 240) for grid volume:',i5)
!              print 7302
!7302          format(/,3x,'irep',
!    .             1x,'indexd',5x,'lat',5x,'lon',4x,'pres',2x,'kx',
!    .             1x,'t',4x,'dist',1x,'m',3x,'rscan',2x,'pabove',
!    .             2x,'pbelow',5(1x,'i'),5(2x,'k'),/)
!           end if
 
            do i = 1,nrep
 
               if( dist(i) .le. rscan ) then
 
                if ( ixkxs(i) .eq. 7 ) then
 
                  if ( pres(i) .ge. pabove .and. pres(i) .le. pbelow
     &                 )then
 
                    iflag( i,0 ) = 1 - markoff(i)
 
                  end if

                else if ( ixkxs(i) .eq. 88 .or. ixkxs(i).eq.91) then
 
                  if ( pres(i) .ge. pabove .and. pres(i) .le. pbelow
     &                 )then
                    iflag(i,0) = 1
                  end if

                else ! for types other than rawinsondes...
 
                  if( pres(i).eq.pabove ) iflag(i,1) = 1 - markoff(i)
 
                  if( pres(i).eq.pbelow ) iflag(i,2) = 1 - markoff(i)
 
                  if( (pres(i).gt.pabove)
     &               .and. ( pres(i) .lt. pbelow ) ) then
 
                     iflag(i,3) = 1 - markoff(i)
                     iflag(i,4) = markoff(i)
 
                  end if
 
                endif
 
                knt(0) = knt(0) + iflag(i,0) * itype(i)
                knt(1) = knt(1) + iflag(i,1) * itype(i)
                knt(2) = knt(2) + iflag(i,2) * itype(i)
                knt(3) = knt(3) + iflag(i,3) * itype(i)
                knt(4) = knt(4) + iflag(i,4) * itype(i)
 
               end if
 
!              if (qprint) then
!                 print 7303, i,
!    .                  indexd(i),alat(i),alon(i),pres(i),ixkxs(i),
!    .                  itype(i),dist(i),markoff(i),rscan,pabove,
!    .                  pbelow,(iflag(i,icat),icat=0,4),knt
!7303             format(2(1x,i6),3(1x,f7.2),1x,i3,1x,i1,1x,f7.2,
!    .                  1x,i1,3(1x,f7.2),5(1x,i1),5(1x,i2))
!              end if
 
            end do
 
            maxdat  = maxrep * 3
 
! Avoid saturating volumes with rawinsondes, particularly along coastlines.
! We initially limit the number of available rawinsonde slots to 60% of "maxdat".

            max_rawinsondes = nint(float(maxdat) * .6)
            num(0)  = min0( knt(0), max_rawinsondes )
 
            maxpa   = (maxdat-num(0)) * percenta
            maxpb   = (maxdat-num(0)) * percentb
            maxps   = (maxdat-num(0)) * percents
            maxsn   = (maxdat-num(0)) - maxpa - maxpb - maxps
 
            num(1) = min0( knt(1), maxpa )
            num(2) = min0( knt(2), maxpb )
            num(3) = min0( knt(3), maxps )
            num(4) = min0( knt(4), maxsn )
 
            ngot   = num(1) + num(2) + num(3) + num(4) + num(0)
 
!           if (qprint) then
!              print 7705
!7705          format(' ',13x,'ngot',1x,'maxdat',
!    .                1x,'NUM(0)',1x,'KNT(0)',
!    .                1x,'NUM(1)',1x,'KNT(1)',2x,'MAXPA',
!    .                1x,'NUM(2)',1x,'KNT(2)',2x,'MAXPB',
!    .                1x,'NUM(3)',1x,'KNT(3)',2x,'MAXPS',
!    .                1x,'NUM(4)',1x,'KNT(4)',2x,'MAXSN')
!              message(1:10)='1st pass'
!              print 7707, message(1:10),ngot,maxdat,
!    .                     NUM(0),KNT(0),
!    .                     NUM(1),KNT(1),MAXPA,
!    .                     NUM(2),KNT(2),MAXPB,
!    .                     NUM(3),KNT(3),MAXPS,
!    .                     NUM(4),KNT(4),MAXSN
!7707          format(' ',a10,16i7)
!           end if
 

!  If the slots remain unfilled, look first for rawinsondes....

            if ( ngot .lt. maxdat ) then

                need = maxdat - ngot
                ngot = ngot - num(0)
                num(0) = min0( knt(0), need+num(0) )
                ngot = ngot + num(0)

            end if

! ... and then add other types.

            IF( NGOT.LT.MAXDAT ) THEN
C  ADD SOME MORE DATA WITH MARKOFF=0 BETWEEN PABOVE AND PBELOW
               NEED   = MAXDAT - NGOT
               NGOT   = NGOT - NUM(3)
               NUM(3) = MIN0( KNT(3), NEED+NUM(3) )
               NGOT   = NGOT + NUM(3)
            ENDIF
 
!           if (qprint) then
!              message(1:10)='after 3'
!              print 7707, message(1:10),ngot,maxdat,
!    .                     NUM(0),KNT(0),
!    .                     NUM(1),KNT(1),MAXPA,
!    .                     NUM(2),KNT(2),MAXPB,
!    .                     NUM(3),KNT(3),MAXPS,
!    .                     NUM(4),KNT(4),MAXSN
!           end if
 
            IF( NGOT.LT.MAXDAT ) THEN
C  ADD SOME MORE DATE WITH MARKOFF=0 AT PABOVE
               NEED   = MAXDAT - NGOT
               NGOT   = NGOT - NUM(1)
               NUM(1) = MIN0( KNT(1), NEED+NUM(1) )
               NGOT   = NGOT + NUM(1)
            ENDIF
 
!           if (qprint) then
!              message(1:10)='after 1'
!              print 7707, message(1:10),ngot,maxdat,
!    .                     NUM(0),KNT(0),
!    .                     NUM(1),KNT(1),MAXPA,
!    .                     NUM(2),KNT(2),MAXPB,
!    .                     NUM(3),KNT(3),MAXPS,
!    .                     NUM(4),KNT(4),MAXSN
!           end if
 
            IF( NGOT.LT.MAXDAT ) THEN
C  ADD SOME MORE DATA WITH MARKOFF=0 AT BBELOW
               NEED   = MAXDAT - NGOT
               NGOT   = NGOT - NUM(2)
               NUM(2) = MIN0( KNT(2), NEED+NUM(2) )
               NGOT   = NGOT + NUM(2)
            ENDIF
 
!           if (qprint) then
!              message(1:10)='after 2'
!              print 7707, message(1:10),ngot,maxdat,
!    .                     NUM(0),KNT(0),
!    .                     NUM(1),KNT(1),MAXPA,
!    .                     NUM(2),KNT(2),MAXPB,
!    .                     NUM(3),KNT(3),MAXPS,
!    .                     NUM(4),KNT(4),MAXSN
!           end if
 
            IF( NGOT.LT.MAXDAT ) THEN
C  ADD SOME MORE DATA WITH MARKOFF=1 BETWEEN PABOVE AND PBELOW
               NEED   = MAXDAT - NGOT
               NUM(4) = MIN0( KNT(4), NEED+NUM(4) )
            ENDIF
 
!           if (qprint) then
!              message(1:10)='after 4'
!              print 7707, message(1:10),ngot,maxdat,
!    .                     NUM(0),KNT(0),
!    .                     NUM(1),KNT(1),MAXPA,
!    .                     NUM(2),KNT(2),MAXPB,
!    .                     NUM(3),KNT(3),MAXPS,
!    .                     NUM(4),KNT(4),MAXSN
!           end if
 
         ELSE
 
C  SEA LEVEL DATA SELECTION
            NUMCAT = 1
            iflag ( 1:nrep, 0:4 ) = 0
            maxdat  = maxrep * 3
            NUM(1) = 0
            DO 250 I = 1, NREP
               IFLAG(I,1) = 0
               IF( DIST(I).LE.RSCAN ) IFLAG(I,1) = 1
               NUM(1) = NUM(1) + ITYPE(I) * IFLAG(I,1)
  250       CONTINUE
            NUM(1) = MIN0( NUM(1), MAXDAT )
         ENDIF
 
C  LOOP TO CHOOSE THE DATA FROM EACH CATEGORY
 
      do 280 icat = 0, numcat    
 
!        if (qprint) then
!           print 7721, icat
!7721       format(' begin loop for category',i4)
!        end if
 
C  CHECK TO SEE IF THERE ARE ANY USABLE OBSERVATIONS IN CATEGORY ICAT
         NFLAG = 0
         DO 265 I = 1, NREP
            IF( IFLAG(I,ICAT).NE.0 ) THEN
               NFLAG = NFLAG + 1
               DHEAP(NFLAG) = DIST(I)
               KHEAP(NFLAG) = I
            ENDIF
  265    CONTINUE
 
!        if (qprint) then
!           print 7723, nflag
!7723       format(' (nflag) number of obs in category:',i10)
!           print 7724
!7724       format(/,5x,'indexd',7x,'lat',7x,'lon',6x,'pres',8x,'kx',
!    .             6x,'type',5x,'itype',
!    .             6x,'ncat',1x,'num(icat)',4x,'nrepok')
!        end if
 
         IF( NFLAG.GT.0 ) THEN
 
C  CREATE A MIN-HEAP ON DISTANCE
            LHEAP = NFLAG
            CALL MIRHEAPY( DHEAP, IHEAP, LHEAP )
 
C  LOOP TO FIND THE NEXT REPORT TO ADD
            NCAT = 0
            DO 275 J = 1, NFLAG
 
C  EXTRACT CLOSEST DATA AND REPAIR THE HEAP
               MIND     = KHEAP(IHEAP(1))
               IHEAP(1) = IHEAP(LHEAP)
               LHEAP    = LHEAP - 1
               IF( LHEAP.GT.1 ) CALL MIRHEAPJ( DHEAP, IHEAP, 1, LHEAP )
 
C  ACCEPT THE DATA PENDING FURTHER QUALITY CONTROL
               NCAT                 = NCAT + ITYPE(MIND)
               IF( NCAT.GT.NUM(ICAT) ) GOTO 280
               NREPOK               = NREPOK + 1
               DATALIST(NREPOK,LEV) = INDEXD(MIND)
 
 
!              if (qprint) then
!                 indexd_p=datalist(nrepok,lev)+0.001
!                 alat_p  = data(ilat-1+indexd_p)
!                 alon_p  = data(ilon-1+indexd_p)
!                 pres_p  = data(iprs-1+indexd_p)
!                 ixkxs_p = data(ikx -1+indexd_p)+0.001
!                 itype_p = data(ityp-1+indexd_p)+0.001
!                 print 7725, indexd_p,alat_p,alon_p,pres_p,ixkxs_p,
!    .                     itype_p,itype(mind),ncat,num(icat),nrepok
!7725             format(' ',i10,3f10.2,6i10)
!              end if
 
  275       CONTINUE
 
         ENDIF
 
  280 CONTINUE
 
      NREPS(LEV) = NREPOK
      IF( NREPS(LEV).GT.MXREPS ) THEN
         WRITE(6,*) ' SEARCH: ',LEV, NREPS(LEV), MXREPS
         message = ' SEARCH: MXREPS has been exceeded - see message.'
         call flash( 5,message,1 )
      ENDIF
 
  300 CONTINUE
 
      if (qprint.and.nrep.gt.0) then
 
         print 8901, igp,vollat,vollon
 8901    format(/,' SEARCH: Data selected by SEARCH for horizontal',
     .          ' volume:',i5,' at each level',
     .          2x,'lat = ',f6.1,2x,'lon =',f6.1)
 
         do lev = lu, ld, nv
 
            print 8911, lev, nreps(lev)
 8911       format(/,' SEARCH: Data selection at level:',i4,2x,
     .             'Number of reports at this level:',i8)
 
            print 8912
 8912       format(/,4x,'indexd',7x,'lat',7x,'lon',6x,'pres',8x,'kx',
     .             6x,'type',/)
            do i=1,nreps(lev)
               indexd_p=datalist(i,lev)+0.001
               alat_p  = data(ilat-1+indexd_p)
               alon_p  = data(ilon-1+indexd_p)
               pres_p  = data(iprs-1+indexd_p)
               ixkxs_p = data(ikx -1+indexd_p)+0.001
               itype_p = data(ityp-1+indexd_p)+0.001
               print 8914, indexd_p,alat_p,alon_p,pres_p,ixkxs_p,
     .                     itype_p
 8914          format(' ',i10,3f10.2,2i10)
            end do
 
         end do
 
      end if
 
      RETURN
      END
