 
      subroutine growth( per, ter, rer, her )
 
c Purpose:   Increase the analysis errors by some amount defined
c            by an empirical formula which represents an error in
c            the forecast.
c            Result is the forecast error which is used in the oi
c            covariance calculations.
c
c Programmer:J. Pfaendtner
c
c Modification(s): 1990/04/10 Replacement of NMC obs errors with NASA
c                  obs errors; corrected computation of alpha and beta.
c                  ...M. Seablom
c
c                  1990/03/08 Allows for increasing and generalizing the 
c                  number of levels. 
c                  ...D. Lamich 
 
      parameter ( max_levels = 18 )
 
      dimension         per(im,jnp), rer(im*jnp,mlev), her(im,jnp,mlev)
      dimension         ter(im,jnp)
 
      common /gridxx/ im,jnp,mlev
 
      dimension         rgro(50),  rlim(50)
      dimension         hgrot(50), hlimt(50)
      dimension         hgrox(50), hlimx(50)
      real              rgro_table(max_levels)
      real              hgrot_table(max_levels)
      real              hlimt_table(max_levels)
      real              hgrox_table(max_levels)
      real              hlimx_table(max_levels)
      real              pi, phi, deglat
      logical           first
 
      data  rgro_table /    0.,    0.,    0.,    0.,    0.,    0.,
     $                      0.,    0.,    0.,    0.,    0.,    0.,
     $                     .05,   .15,   .20,   .65,    .9,   1.2 /
      data hgrot_table / 125.0, 125.0, 125.0, 125.0, 125.0, 110.0,
     $                    95.0,  80.0,  60.0,  25.0,  21.0,  17.0,
     $                    14.0,  13.0,  12.0,  10.5,   9.0,   9.0 /
      data hlimt_table / 147.0, 147.0, 147.0, 147.0, 147.0, 132.0,
     $                   117.0, 105.0,  96.0,  76.0,  56.0,  37.0,
     $                    29.0,  26.9,  24.5,  20.0,  20.0,  20.0 /
      data hgrox_table /  70.0,  70.0,  70.0,  70.0,  70.0,  59.0, 
     $                    48.0,  37.0,  35.0,  32.0,  30.0,  28.0,
     $                    26.5,  23.0,  20.0,  15.0,  12.5,  10.5 /
      data hlimx_table / 112.5, 112.5, 112.5, 112.5, 112.5, 109.5,
     $                   106.5, 103.0,  98.5,  95.0,  91.5,  88.5,
     $                    85.0,  78.0,  71.0,  58.0,  56.5,  55.0 /
      data  first/.true./
 
      blend(a,b,phi) = 0.5*(a+b) +
     $                 0.5*(a-b)*cos((abs(phi)-philo)/(phihi-philo)*pi)
 
 !    if(first) then
 
         level_start = max_levels - mlev + 1
 
         do l=1,mlev
            rgro(l) = rgro_table(level_start+l-1)
         end do 
 
         do l=1, mlev
            rlim(l) = rgro(l) * 8.0
         end do
 
         do l=1,mlev
            hgrot(l) = hgrot_table(level_start+l-1)
         end do 
 
         do l=1,mlev
            hlimt(l) = hlimt_table(level_start+l-1)
         end do 
 
         do l=1,mlev
            hgrox(l) = hgrox_table(level_start+l-1)
         end do 
 
         do l=1,mlev
            hlimx(l) = hlimx_table(level_start+l-1)
         end do 
 
         pgrot = hgrot(mlev)/8.0
         pgrox = hgrox(mlev)/8.0
         plimt = hlimt(mlev)/8.0
         plimx = hlimx(mlev)/8.0
 
         pi = 4.0*atan(1.0)
         dphi = 180.0 / ( jnp - 1 )
         philo = 15.
         phihi = 45.
         imjnp = im * jnp
         first = .false.
 
!     endif
 
      print 900
 
c  Augment analysis errors with empirical growth formula
 
      deglat = -90.0
      do j = 1, jnp
         if ( abs(deglat).le.philo ) then
            pgro = pgrot
            plim = plimt
         else if ( abs(deglat).ge.phihi ) then
            pgro = pgrox
            plim = plimx
         else
            pgro = blend(pgrot,pgrox,deglat)
            plim = blend(plimt,plimx,deglat)
         endif
         const = (plim-pgro)/plim
         do i = 1, im
            per(i,j) = const * per(i,j) + pgro
         end do
         deglat = deglat + dphi
      end do
 
      do l = 1, mlev
         if( rlim(l).ne.0.0 ) then
            const = (rlim(l)-rgro(l)) / rlim(l)
            do i = 1, imjnp
               rer(i,l) = const * rer(i,l) + rgro(l)
            end do
         endif
         deglat = -90.0
         do j = 1, jnp
            if ( abs(deglat).le.philo ) then
               hgro = hgrot(l)
               hlim = hlimt(l)
            else if ( abs(deglat).ge.phihi ) then
               hgro = hgrox(l)
               hlim = hlimx(l)
            else
               hgro = blend(hgrot(l),hgrox(l),deglat)
               hlim = blend(hlimt(l),hlimx(l),deglat)
            endif
            const = (hlim-hgro)/hlim
            do i = 1, im
               her(i,j,l) = const * her(i,j,l) + hgro
            end do
            deglat = deglat + dphi
         end do
      end do
 
      return
  900 format(' GROWTH: Adding 6hr growth to analysis errors')
      end
