C......................................................................
C                                                                     .
C  Program:    MIXCOV                                                 .
C  Language:   Cray cft77 fortran version 6.0                         .
C  Computer:   Cray C98 6/256                                         .
C                                                                     .
C  Programmer: M. Nestler, D. Lamich, M. Seablom                      .
C  Date:       August, 1991                                           .
C                                                                     .
C  MIXCOV computes moisture covariances.                              .
C                                                                     .
C  * MODIFICATIONS:                                                   .
C                                                                     .
C    FEBRUARY 1991: (M. SEABLOM)  Capability to process > 100k obs.   .
C    AUGUST   1991: (M. SEABLOM)  Variable minivolumes incorporated.  .
C    February 1994: (M. Seablom)  Larger minivolumes incorporated.    .
C                                                                     .
C......................................................................
!
!  Gregor von Laszewski, 12/95 Changed to compile on RS6000 using xlf
!                      made observations variable in max_observations

 
c..................................................................c
!                                                                  !
! NASA Goddard Space Flight Center       Data Assimilation Office  !
!                                                                  !
! Goddard Earth Observing System                                   !
! Data Analysis System - Version 2.0                               !
! February, 1994                                                   !
!                                                                  !
!------------------------------------------------------------------c

 
      SUBROUTINE MIXCOV( NREP,     IGP,      IREG,     LEV,
     &                   RINDX,    OBSERR,   ISIZE,
     &                   data,     nobs, symat, delta, force, gp, 
     &                   ztx ,  mix_err, u_err, v_err )
 
#include <obs.com>

C  COMPUTES COVARIANCE MATRIX FOR MIXING RATIO ANALYSIS
 
 
      parameter (max_nh=600, maxrank=75)
      INTEGER           MXGRID
      PARAMETER       ( MXGRID = 600)
 
      INTEGER           MXREPS
      PARAMETER       ( MXREPS = 150 )
 
      REAL              COSDX
      REAL              COSDY
      REAL              COSY1
      REAL              COSY2
      REAL              D
      real              data( max_obsspace )
      real              DLAT
c      SAVE              DLAT
      real              DLON
c      SAVE              DLON
      real              mix_err(IM*JNP,MLEV)
!gvl      LOGICAL           FIRST
!gvl      data              first /.true./
c      SAVE              FIRST
      real              GLAT(MXGRID)
      REAL              GLATX
      real              GLON(MXGRID)
      REAL              GLONX
      real              GPRS(MXGRID)
      real              GSTD(MXGRID)
      REAL              HCOR
      INTEGER           I
      INTEGER           IGP
      INTEGER           IIGP
      INTEGER           INDX
      INTEGER           ISIZE
      INTEGER           IX
!gvl      INTEGER           I4X5
      INTEGER           J
      INTEGER           JX
!gvl      INTEGER           J4X5
      INTEGER           K
      INTEGER           LEV
      real              OBER(MXREPS)
      real              PI180
c      SAVE              PI180
      DIMENSION         RINDX(NREP)
      INTEGER           N
!gvl      INTEGER           NHROOT
c      SAVE              NHROOT
      INTEGER           NL
!gvl      INTEGER           NNH
!gvl      INTEGER           NNI
!gvl      INTEGER           NNJ
!gvl      INTEGER           NNV
!gvl      INTEGER           NREG
      INTEGER           NX
      DIMENSION         OBSERR(200,4,19)
      real              PRSI((MXREPS+MXGRID)*(MXREPS+MXGRID+1)/2)
      real              PRSJ((MXREPS+MXGRID)*(MXREPS+MXGRID+1)/2)
      real              RADE
c      SAVE              RADE
      REAL              S
      REAL              VAR
      real              VCOR((MXREPS+MXGRID)*(MXREPS+MXGRID+1)/2)
      real              XDEL(MXREPS)
      real              XKX(MXREPS)
      real              XLAT(MXREPS+MXGRID)
      real              XLON(MXREPS+MXGRID)
      real              XPRS(MXREPS+MXGRID)
      real              XSTD(MXREPS+MXGRID)
      real              XXCOV((MXREPS+MXGRID)*(MXREPS+MXGRID+1)/2)
      REAL              X1
      REAL              X2
      REAL              Y1
      REAL              Y2
      real              ztx(max_observations)
 
      real u_err(im*jnp,mlev+1),v_err(im*jnp,mlev+1)
      common /dataptrs/ ikx, ilat, ilon, iprs, ityp, idel, istd,
     &                  itime
 
      COMMON /scan/     maxobs, rscan, onestage, max_data
 
      COMMON /GRIDXX/   IM, JNP, MLEV
 
      COMMON /PMH/      PMAND(18)
 
      common /sparam/ lsolve, ns, nr, nv, nh(13104), nq,
     &                kq(3), volcentre_x(13104),
     &                ngv, volcentre_y(13104), glatup(13104),
     &                glonup(13104)
 
      real symat(maxrank*(maxrank+1)/2), delta(maxrank),
     &     force(maxrank,max_nh,3), gp(max_nh)
 
      COMMON /VCORS/    VCORFE(13,13,20,2)

      ix( xxlon,dlon ) = ( xxlon + 180. ) / dlon + 1.1
      jx( xxlat,dlat ) = ( xxlat + 90.  ) / dlat + 1.1
 
         CALL GETCONH( 'DLAT    ', DLAT )
         CALL GETCONH( 'DLON    ', DLON )
         CALL GETCONH( 'RADE    ', RADE )
         CALL GETCONH( 'PI180   ', PI180 )
 
      force( 1:maxrank, 1:max_nh, 1:3 ) = 0.0
      delta( 1:maxrank ) = 0.0
      symat( 1:(maxrank*(maxrank+1)/2))= -1.


!  Set "nhgp" to be the number of horizontal grid points for current volume.

      nhgp = nh(igp)
      n = 0
 
!  Locate grid points within a volume.      

      nh_count = 0
      nhnv_count = 0

      do j = 1,2
         do i = 1,nhgp/2

           nh_count = nh_count + 1
           glatx = glatup( igp ) + ( j-1 ) * dlat
           if ( glatx .gt. 90. ) glatx = 90.
           glonx = glonup( igp ) + ( i-1 ) * dlon
           iigp = ( jx(glatx,dlat) -1 ) * im + ix( glonx,dlon )
           gp( nh_count ) = iigp

           do k = 1,nv

              nhnv_count = nhnv_count + 1
              nl = lev - 1 + k
              glat(nhnv_count) = glatx
              glon(nhnv_count) = glonx
              gprs(nhnv_count) = pmand(nl)
              gstd(nhnv_count) = mix_err(iigp,nl)

           end do

         end do
      end do
      
C  GATHER STUFF FOR MAKING THE COVARIANCES
      NX    = NREP
      ISIZE = NREP
      DO 160 K = 1, NX
         INDX     = RINDX(K) + 0.1
         XLAT(K)  = DATA(ILAT+INDX-1)
         XLON(K)  = DATA(ILON+INDX-1)
         XPRS(K)  = DATA(IPRS+INDX-1)
         XSTD(K)  = DATA(ISTD+INDX-1)
         XDEL(K)  = DATA(IDEL+INDX-1)
         XKX(K)   = DATA(IKX+INDX-1)
  160 CONTINUE
      DO 170 K = 1, NH(igp)*NV
         XLAT(NX+K) = GLAT(K)
         XLON(NX+K) = GLON(K)
         XPRS(NX+K) = GPRS(K)
         XSTD(NX+K) = GSTD(K)
  170 CONTINUE
 
C  COMPUTE THE COVARIANCES
      N = 0
      DO 200 I = 1, NX+NH(igp)*NV
         X1    = XLON(I)
         Y1    = XLAT(I)
         COSY1 = COS( Y1 * PI180 )
         DO 190 J = 1, MIN0( I, NX )
            N        = N + 1
            PRSI(N)  = xprs(i)
            PRSJ(N)  = xprs(j)
            X2       = XLON(J)
            Y2       = XLAT(J)
            COSY2    = COS(Y2 * PI180 )
            COSDY    = COS( (Y1-Y2) * PI180 )
            COSDX    = COS( (X1-X2) * PI180 )
            S        = 1.0 - COSDY + COSY1 * COSY2 * ( 1.0 - COSDX )
            S        = SQRT( 2.0 * S )
            IF( S.LE.0.002 ) S = 0.0
            D        = S * RADE
            HCOR     = EXP( -0.002 * D )
            VAR      = XSTD(I) * XSTD(J)
            XXCOV(N) = VAR * HCOR
  190    CONTINUE
  200 CONTINUE
 
      call vcorrfe ( 'MIX', prsi, prsj, vcor, n )
 
      CALL OBERTRP0( XKX, XPRS, OBER, OBSERR, KQ(1), NX )
 
      N = 0
      DO 220 I = 1, NX+NH(igp)*NV
         DO 210 J = 1, MIN0( I, NX )
            N = N + 1
            XXCOV(N) = XXCOV(N) * VCOR(N)
            IF( I.LE.NX ) THEN
               SYMAT(N) = XXCOV(N)
               IF( I.EQ.J ) THEN
                  SYMAT(N) = SYMAT(N) + OBER(I)**2
                  DELTA(I) = XDEL(I)
               ENDIF
            ELSE
               FORCE(J,I-NX,1) = XXCOV(N)
            ENDIF
  210    CONTINUE
  220 CONTINUE
 
C  PRINTING INTERESTING CASES
      IF( IGP.EQ. 0000) THEN
         WRITE( 6, 900 ) ( I, XKX(I), XLAT(I), XLON(I), XPRS(I),
     $                                  XDEL(I), XSTD(I), I = 1, NX )
         WRITE( 6, 910 ) IGP
         DO 400 I = 1, NX
            WRITE( 6, 920 ) I, ( SYMAT(K), K=I*(I+1)/2-I+1, I*(I+1)/2 )
  400    CONTINUE
         WRITE( 6, 930 ) IGP
         DO 410 I = 1, NH(igp)
            DO 405 J = 1, NV
               WRITE( 6, 940 ) I, J, (FORCE(N,J+(I-1)*NV,1), N = 1, NX)
  405       CONTINUE
  410    CONTINUE
      ENDIF
 
      RETURN
  900 FORMAT(' MIXCOV: ',I5,6F12.2)
  910 FORMAT(//' MIXCOV: COV. MATRIX FOR MOISTURE ANALYSIS IGP =',I6 )
  920 FORMAT('0MIXCOV:  I = ',I4 / (' MIXCOV: ',10F7.3 ) )
  930 FORMAT(//' MIXCOV: FORCING TERMS FOR MOISTURE ANALYSIS IGP =',I6 )
  940 FORMAT('0MIXCOV: IH =',I2,' IV =',I2 / (' MIXCOV: ',10F7.3 ) )
      END
