!disabled deloutput because of performance reasons 
c......................................................................
c                                                                     .
c  PROGRAM:    DELHUV                                                 .
c  LANGUAGE:   Cray Fortran 77 Version 6                              .
c  COMPUTER:   Cray C90                                               .
c                                                                     .
c  PROGRAMMER: M. SEABLOM                                             .
c  DATE:       November 14, 1994                                       .
c                                                                     .
c  Program DELHUV performs version 2.0 quality control and computes   .
c  deltas for upper air height and wind data.                         .
c                                                                     .
c  Program modifications:                                             .
c    - 11/90 (Seablom)  Corrected error in OBERTRP3                   .
c    -  1/91 (Lamich)   Rejecting observations below ground           .
c                                                                     .
c  Modifications for version 1.1:                                     .
c    -  3/91 (Seablom)  Expanded DATA array to allow >100K obs.       .
c    -  3/91 (Seablom)  Reports tallied from DELHUV rather than RDHUV..
c    -  4/91 (Seablom)  Added date/time stamp to DATA array.          .
c    -  4/91 (Seablom)  Level-dependent tolerance values included.    .
c                       Wind tolerances increased significantly over  .
c                       OI version 1.0.                               .
c                                                                     .
c  Modifications for version 1.2:                                     .
c    - 11/91 (Seablom)  Removal of data ingest.                       .
c                                                                     .
c  Modifications for version 1.3:                                     .
c    -  7/92 (Seablom)  Corrected bug in wind quality control.        .
c    -  7/92 (Seablom)  Code modifications to allow parallelization.  .
c    -  7/92 (Seablom)  Wind standard deviations passed back to driver.
c                                                                     .
c  Modifications for version 1.4:                                     .
c    - 11/93 (Seablom)  Explicitly designated variables for parallel- .
c                       ization.                                      .
c    - 11/93 (Lamich)   Properly passed "suspect" array.              .
c                                                                     .
c  Modifications for version 1.5:                                     .
c    - 11/93 (Seablom)  Implemented "bit vector" syntax for wind      .
c                       quality control following reproducibility     .
c                       problems in parallel environment.             .
!
!  Gregor von Laszewski, 12/95 Changed to compile on RS6000 using xlf
!                      made observations variable in max_observations
c......................................................................
!                                                                     !
! NASA Goddard Space Flight Center       Data Assimilation Office     !
!                                                                     !
! Goddard Earth Observing System                                      !
! Data Analysis System - Version 1.5                                  !
! October, 1993                                                       !
!                                                                     !
c---------------------------------------------------------------------c
! #ifndef DEC_ALPHA
! #include "pindx2.f" 
! #endif
 
      subroutine par_delhuv (zmod,umod,vmod,tmod,psmod,
     &        z1000,z_err,
     &        u_err,v_err,data,nobs,ze,zelat,zelon,nze,nymd,nhms,
     &        suspect,ztime)
 
#include <obs.com>
#ifdef PARALLEL
#include <parenv.h>      
#include <mpif.h>

	integer recv_count(0:MAX_PROCESSORS)
        integer displ     (0:MAX_PROCESSORS)
#endif
      real zmod(im*jnp,mlev),umod(im*jnp*mlev),
     &     w_error(im,jnp,mlev),
     &     psmod(im*jnp), ztime(max_observations), 
     &     z_err(im,jnp,mlev+1),
     &     z1000(im*jnp),  u_err(im,jnp,mlev+1),
     &     tmod(im*jnp,mlev), vmod(im*jnp*mlev), lat, lon, kx,
     &     latmax, latmin, lonmax, lonmin, londif, ze(20000),
     &     zelat(20000), zelon(20000),srchlat, srchlon, dist,
     &     data (max_obsspace    ), 
     &     fg1  (max_observations  ), 
     &     fg2  (max_observations  ),
     &     fg3  (max_observations ),
     &     psobs(max_observations  ), 
     &     ztol(mlev), 
     &     v_err(im,jnp,mlev+1)

#ifdef PARALLEL
      integer
     &     fg1_int  (max_observations  ), 
     &     fg2_int  (max_observations  )
#endif
      real before, after
!gvl  real prslev(12)

      character*8 dataname
      character*24 obsname
      character*60 message
      logical polar, suspect(max_observations  ,2), qobs, 
     &        offlev, onestage,
     &        dateline_case
      integer gcfpoint(30000)
      integer undergnd_count
 
      common /dataptrs/ ikx,ilat,ilon,iprs,ityp,idel,ife,itime
      common /gridxx/ im,jnp,mlev
      common /scan/  maxobs, rscan, onestage, max_data
      common /print/ msglev
      common / obs / maxdex, nfgge, lun, dataname(5), hh(13,13,20),
     &               vcoroe(18,18,4), obserr(200,4,19), ww(13,13,20),
     &               obsname(200), qobs(200,5), offlev(200)

      integer nreacc
#ifdef PARALLEL
 	integer home(nobs)
#endif 
 
c   Initialize data.
 
      data fill/1.e10/, alpha/9.e10/, rade/6.37e6/
 
      pi = 4.*atan(1.)
      drc = pi / 180.
      rdc = 180. / pi
      message(1:60) = ' '
      undergnd_count = 0
      nreacc = 0
 
c   Initialize tolerance arrays.
 
      ztol( mlev ) = 2.8
 
      do itol = mlev-1, 1, -1
 
        ztol( itol ) = ztol( itol+1 ) + .04
 
      end do
 
c  Compute the geostrophic wind errors.
 
      call werrinit (z_err,im,jnp,mlev,u_err,v_err)

      write(6,'(//," DELHUV: Starting upper air quality ",
     &  "control...")')
 
      write(6,*)
      write(6,*) ' Total number of observations=', nobs
      if(nobs.eq.0) return
      fg2( 1:nobs ) = 0.
 
c  Set pointers for winds, observation and forecast errors.
 
      iz  = idel
      iu  = iz  + nobs
      iv  = iu  + nobs
      ife = iv  + nobs
      ioe = ife + nobs

c  Save observation times from data array.

 
      do izt = 1,nobs

        ztime(izt) = data(ioe+izt-1)

      end do

c  Establish major loop to perform the following:
c   1. Retrieve forecast and obs. errors and store in array "data".
c   2. Compute deltas.
c   3. Pass data through gross check.
 
c  Obtain first guess height values at observation locations.
 
      call vterpv ( data(ilat), data(ilon), data(iprs), fg1,
     &              nobs, zmod )
 
c  Interpolate the forecast errors to observation locations.
 
      call vterpv ( data(ilat), data(ilon), data(iprs), data(ife),
     &              nobs, z_err )
 
c  Interpolate observation error to observation location.
 
      call obertrp0 ( data(ikx), data(iprs), data(ioe), obserr, 2,
     &                nobs )
 
c  Interpolate the model surface pressure to the observation
c  locations.
 
      call terpv ( data(ilat), data(ilon), psobs, nobs, psmod )
 
c  QC gross check / delta computation for height data ----------------
 
      igcfail = 0
      itest = 0
      message = 'DELHUV: Impossible location; rejected point.'
 
cmic$ do all vector shared (data, drc, fg1, fg2, fill, ife, ikx, ilat)
cmic$*       shared (ilon, ioe, iprs, ityp, iz, kx, max_data, message)
cmic$*       shared (mlev, nobs, psobs, suspect, undergnd_count)
cmic$*       shared (ztol, itime)
cmic$*       private (delta, fe, i, lat, lon, oe, pointer, prs)
cmic$*       private (tolerance, trwt) autoscope

	call timerstart (52)
      do 1 i = 1,nobs
 
        if ( data(iz+i-1) .lt. fill ) then
           data(ityp+i-1) = 1.
        else
           data(ityp+i-1) = 0.4
        end if
 
        lat = data(ilat+i-1)
        lon = data(ilon+i-1)
        prs = data(iprs+i-1)
        kx  = data(ikx+i-1)
        if ( lat .gt. 90. .or. lat .lt. -90. .or. lon .gt. 180.
     &       .or. lon .lt. -180. ) then
           data(ityp+i-1) = 0.4
celp       call flash( 3,message,1 )
        end if
 
c  The interpolated surface pressure is used to eliminate observations
c  that are underground as far as the model is concerned.  This check
c  is quite crude in that it will sometimes keep and sometimes remove
c  observations in areas of rapidly varying topography.
 
        if ( prs .gt. psobs(i) ) then
           data(ityp+i-1) = 0.4
c           undergnd_count = undergnd_count + 1
        end if
 
 
c  Now compute the delta if valid data exists. Generated 1000mb height
c  reports are already stored as deltas.
 
        if ( data(ityp+i-1) .ge. 1. .and. data(ikx+i-1) .ne. 87. )
     &     data(iz+i-1) = data(iz+i-1) - fg1(i)
 
c  Gross check the height data.
 
        delta  = data(iz +i-1)
        oe     = data(ioe+i-1)
        fe     = data(ife+i-1)
 
c  Tighten tolerance in the tropics.
 
        if ( abs( lat ) .le. 30.0 ) then
           trwt = ( exp( sin( abs(lat)*drc ) ) / 1.65 )
        else
           trwt = 1.
        end if
 
        pointer = pindx2( prs )
        tolerance = ztol( int(pointer) )**2
 
        if ( (delta**2 .gt. ( oe**2 + fe**2 )*tolerance*trwt )
     &     .and. data(iz+i-1) .lt. fill ) then
           data(ityp+i-1) = 0.1
           fg2( i ) = 1.
        end if
 
c  Flag suspicious observations and subject them to the gross check.
 
        if ( suspect(i,1).and.data(iz+i-1) .lt. fill ) then
           data( ityp+i-1 ) = 0.3
           fg2( i ) = 1.
        end if
 
1     continue
 
      if (undergnd_count.gt.0) then
         write( 6,1001) undergnd_count
 1001    format(i5,' observations below model surface')
      end if
 
      call wheneq ( nobs, fg2, 1, 1., gcfpoint, igcfail )
 
      call timerdump (52,"delhuv - grosscheck\0")	
      write(6,'(" Total z-reports failing gross check:",i6)')
     &     igcfail
 
      if ( igcfail .gt. 30000 ) then
       message=' DELHUV: Excessive gross check failures.'
       call flash( 5,message,1 )
      end if
 
c  Find neighbouring observations to those flagged by the gross check.
c  Perform an interpolation of the good quality neighbours (within a
c  1000km radius) to the location of a suspicious report.
 
cmic$ do all vector shared (alpha, data, drc, gcfpoint, ife, igcfail)
cmic$*       shared (ilat, ilon, ioe, iprs, ityp, iz, max_data)
cmic$*       shared (mlev, nneigh, nobs, nreacc, rade, rdc, ztol)
cmic$*       private (cdif1, clat1, clat2, clond, dateline_case)
cmic$*       private (dist, fg1, fg2, gcflat, gpv, i, ifail, isum)
cmic$*       private (j, latmax, latmin, londif, lonmax, lonmin)
cmic$*       private (pointer, polar, srchiz, srchlat, srchlon)
cmic$*       private (srchprs, srchtyp, swts, swtsv, temp, tolerance)
cmic$*       private (trwt) autoscope

      if (rank .eq. 0) then
         do i=1,igcfail
            ifail = gcfpoint(i)
            write (77,*) data(ifail+ilat-1), data(ifail+ilon-1)
         
         end do
      end if

      call timerstart (53)
#ifdef PARALLEL

      call block_distribution (rank, processors,igcfail,
     $    displacement(0),length(0))


!!      do 2 i = displacement(rank),displacement(rank)+length(rank)-1
!!         if (.true.) then


      fg1(1:nobs) = 0.
      fg2(1:nobs) = 0.
#endif

      do 2 i = 1,igcfail
#ifdef PARALLEL
! block decomposition
!         if ((i .ge. displacement(rank)) .and.
!     $        (i .le. (displacement(rank)+length(rank)-1))) then
! cyclic decomposition
         if ( MODULO (i,processors) .eq. rank) then
#endif 
        ifail = gcfpoint(i)
        polar = .false.
 	gcflat = data(ifail+ilat-1)
c     Define the search region.
        
        if ( gcflat .gt. 84.9 ) then
           latmax = 90.
           latmin = 70.
           polar  = .true.
        else if ( gcflat .lt. -84.9 ) then
           latmax = -70.
           latmin = -90.
           polar  = .true.
        else
           latmax = gcflat + 10.
           latmin = gcflat - 10.
        end if
 
        if ( .not. polar ) then

           londif =  atan2(.3139e0, cos(gcflat*drc ) ) * rdc / 2.
           lonmax = data(ifail+ilon-1) + londif
           lonmin = data(ifail+ilon-1) - londif
 
c  Case where region overlaps the international date line.
 
           if ( lonmax .gt. 180. ) then
              lonmax = lonmax - 360.
              dateline_case = .true.
           else if ( lonmin .lt. -180. ) then
              lonmin = lonmin + 360.
              dateline_case = .true.
           else
              dateline_case = .false.
           end if
        end if
 
c     print*,' box (latx,latn,lonx,lonn):', latmax,latmin,lonmax,
c     &           lonmin
 
c  Find the neighbours.
 
        nneigh = 0
        swts   = 0.
        swtsv  = 0.
!        fg1(1:nobs) = 0.
!        fg2(1:nobs) = 0.

!        call timerstart (80)
        do j = 1,nobs
 
           srchlat = data( ilat+j-1 )
           srchlon = data( ilon+j-1 )
           srchiz  = data( iz  +j-1 )
           srchtyp = data( ityp+j-1 )
           srchprs = data( iprs+j-1 )
 
           if ( srchlat .le. latmax .and. srchlat .ge. latmin
     &          .and.  srchtyp .eq. 1. .and.
     &          (abs(srchprs - data(ifail+iprs-1)) .lt.15.) ) 
     $          then
 
              if (( .not.(dateline_case).and. srchlon.gt.lonmin
     &             .and. srchlon.lt.lonmax)
     &             .or.  (dateline_case  .and. srchlon.gt.lonmin
     &             .or.  srchlon.lt.lonmax)
     &             .or.   polar ) then
 
 
c   Compute distance between neighbour and suspect point.
 
                 clat1 = cos ( srchlat  *drc )
                 clat2 = cos ( gcflat*drc )
                 clond = cos ((srchlon-data(ifail+ilon-1))*drc)
                 cdif1 = cos ((srchlat-gcflat)*drc )
                 
                 temp  = 1.-cdif1 + clat1*clat2*(1.-clond)
                 dist  = sqrt (2.0*temp) * rade
 
c               print*,' neighbour:', srchlat, srchlon, srchiz
 
c  Compute the weight (alpha is the spatial convergence parameter).
                 weight = ( alpha - dist ) / ( alpha + dist )
                 swts = swts + weight
                 swtsv = swtsv + weight*srchiz
 
!gvl            fg1(j) = ( alpha - dist ) / ( alpha + dist )
!gvl            fg2(j) =(( alpha - dist )/( alpha + dist ))*srchiz
!gvl            fg2(j) = fg1(j)*srchiz
 
              end if
           end if
 
        end do
       !     call timerdump (80,"delhuv get neighbors\0")
#ifdef PARALLEL
!not necessary since the sum is only usefull for the region
!no lobal comm
! does not work if icgfail is 0 tah allreduce should be called too.
!      call MPI_ALLREDUCE (swtsv,swtsv_a,1,MPI_REAL,MPI_SUM,
!     $    MPI_COMM_WORLD,ierr)
!      call MPI_ALLREDUCE (swts ,swts_a ,1,MPI_REAL,MPI_SUM,
!     $     MPI_COMM_WORLD,ierr)
!      swts  = swts_a
!      swtsv = swtsv_a
#endif

!        do isum = 1,nobs
!          swtsv = swtsv + fg2(isum)
!          swts  = swts  + fg1(isum)
!        end do
 
        if ( swts .gt. 0.) then	
           gpv = swtsv/swts	
 
 
           if ( abs( data( ifail+ilat-1 )) .le. 30. ) then
              trwt = (exp(sin(abs(gcflat)*drc ) ) / 1.65 )
           else
              trwt = 1.
           end if
 
           pointer = pindx2( data( ifail+iprs-1 ) )
           tolerance = ztol( int(pointer) ) * trwt
 
           if ((( data(ifail+iz-1) - gpv ) **2  .lt.
     &        (data(ifail+ioe-1)**2 + data(ifail+ife-1)**2)*tolerance)
     &       .and. data(ifail+ioe-1) .le. 900. ) then
 
c               data(ifail+ityp-1) = 1.2
                if(data(ifail+ityp-1) .eq. 0.1) then
                   data(ifail+ityp-1) = 1.2
                else if(data(ifail+ityp-1) .eq. 0.3) then
                   data(ifail+ityp-1) = 1.3
                end if

                nreacc = nreacc + 1
#ifdef PARALLEL
                fg1(nreacc) = data(ifail+ityp-1)
                fg1_int(nreacc) = ifail
#endif                

                write (0,*) 'R',rank,ifail,
     $               data(ifail+ilat-1),data(ifail+ilon-1)
           end if
 
        end if
#ifdef PARALLEL
      endif
#endif

 2    continue


#ifdef PARALLEL

#ifdef XXX
      write (0,*) rank, 'nreac=', nreacc,processors,
     $    (recv_count(ii), ii=0,processors-1)

      call MPI_BARRIER (MPI_COMM_WORLD,mpi_err)

      call MPI_ALLGATHER (nreacc,1,MPI_INTEGER,
     $    recv_count,1,MPI_INTEGER,
     $    MPI_COMM_WORLD,mpi_err)

      call MPI_BARRIER (MPI_COMM_WORLD,mpi_err)
      write (0,*) rank, 'nreac=', nreacc,processors,
     $     (recv_count(ii), ii=0,processors-1)
      call MPI_BARRIER (MPI_COMM_WORLD,mpi_err)




      displ(0) = 0
      do ii=1,processors-1
         displ(ii) = displ(ii-1) + recv_count(ii-1) + 1
      end do
      call MPI_BARRIER (MPI_COMM_WORLD,mpi_err)
      write (0,*) rank, 'displ=', nreacc,processors,
     $     (displ(ii), ii=0,processors-1)
      call MPI_BARRIER (MPI_COMM_WORLD,mpi_err)



! first send all the nreacc to each processor and store in an array
! use the array for receiving the data

      call MPI_ALLGATHERV (fg1,nreacc,MPI_REAL,
     $     fg2,recv_count,displ,MPI_REAL,
     $     MPI_COMM_WORLD,ierr)
      call MPI_ALLGATHERV (fg1_int,nreacc,MPI_INTEGER,
     $     fg2_int,recv_count,displ,MPI_INTEGER,
     $     MPI_COMM_WORLD,ierr)



! gsum nreacc

!      call MPI_ALLREDUCE (snreacc,nreacc_a,1,MPI_REAL,MPI_SUM,
!     $    MPI_COMM_WORLD,ierr)
!      nreacc = nreacc_a

      do i=0,processors
         nreacc = nreacc + recv_count(i)
      end do

      do i=1,nreacc
         ifail = fg2_int(i)
         data(ifail+ityp-1) = fg2(i)
!         write (0,*)  "S", rank, ifail, fg2(i)
      end do

#endif

#endif

      close (77)
      close (76)


	 call timerdump (53,"delhuv - buddy check height\0")
 
      write(6,'(" Height reports reaccepted by buddy check:",i6)')
     &         nreacc

      write(6,'(/," ... height quality control complete.",/)')
 
      if ( msglev .gt. 3 ) then
         before = SECOND()
         call qcreport ( data, nobs, 1, mlev, nymd, nhms )
         after = second()
         cputime = after -before
         write (6,*) "CPUTIME, qcreport: ", cputime

      end if
 
      do i = 1,nobs
        data( ityp+i-1 ) = int( data( ityp+i-1 ) )
      end do
 
      call rptlly ( data(ikx), data(ityp), nobs, 4 )
 
	call timerstart(59)
 
c  QC gross check / delta computation for wind data ----------------
 
      igcfail = 0
      fg3(1:max_observations) = 0.  !  Initialize working array.
 
!  Compute the forecast wind speed errors.

      w_error(:,:,1:mlev) = sqrt( u_err(:,:,1:mlev)**2 + 
     &                       v_err(:,:,1:mlev)**2 )
 
c  Interpolate first guess winds to locations of observations.
 
      call vterpv ( data(ilat), data(ilon), data(iprs), fg1,
     &              nobs, umod )
 
      call vterpv ( data(ilat), data(ilon), data(iprs), fg2,
     &              nobs, vmod )
 
      nreacc = 0
 
c  Interpolate the forecast errors to observation locations.
 
      call vterpv ( data(ilat), data(ilon), data(iprs), data(ife),
     &              nobs, w_error )
 
c  Interpolate observation error to observation location.
 
      call obertrp0 ( data(ikx), data(iprs), data(ioe), obserr, 3,
     &                nobs )
 
 	 call timerdump (59,"delhuv - interpolations, vterp ober\0")	  

cmic$ do all vector shared(data,fg1,fg2,fill,gcfpoint,ife,igcfail)
cmic$*       shared (ikx, ilat, ilon, ioe, iprs, ityp, iu, iv)
cmic$*       private ( lat,lon, kx )
cmic$*       shared (max_data, nobs, psobs, suspect, fg3)
cmic$*       private (delta, fe, i, oe, prs, tolerance) autoscope

	call timerstart (55)

      do 6 i = 1,nobs
 
        if ( data(iu+i-1) .lt. fill  .and.          ! Perform a check
     &      data(iv+i-1) .lt. fill   .and.          ! for fill values
     &      data(iprs+i-1) .le. psobs(i) ) then     ! and underground
                                                    ! data points.
           data(ityp+i-1) = data(ityp+i-1) + 2.     !
 
        else
 
           data(ityp+i-1) = data(ityp+i-1) + 0.04
 
        end if
 
        lat = data(ilat+i-1)
        lon = data(ilon+i-1)
        prs = data(iprs+i-1)
        kx  = data(ikx+i-1)
 
 
 
c  Now compute the delta if valid data exists.
 
        if ( data(ityp+i-1) .ge. 2. ) then
 
          data(iu+i-1) = data(iu+i-1) - fg1(i)
          data(iv+i-1) = data(iv+i-1) - fg2(i)
 
        end if
 
c  Gross check the wind data.
 
        delta  = sqrt( data(iu +i-1)**2 + data(iv +i-1)**2 )
        oe     = data(ioe+i-1)
        fe     = data(ife+i-1)
        tolerance = 2.
        if ( prs .le. 500. .and. prs .gt. 70. ) tolerance = 2.5
 
 
        if ( (delta**2 .gt. ( oe**2 + fe**2 )* tolerance)
     &    .and. data(ityp+i-1) .ge. 2. .and. (.not.suspect(i,2))) then
          data( ityp+i-1) = data(ityp+i-1) - 1.99
          fg3( i ) = 1.
        end if
 
        if ( suspect(i,2) .and. data(ityp+i-1) .ge. 2. ) then
          data(ityp+i-1) = data(ityp+i-1) - 1.97
          fg3( i ) = 1.
        end if
 
 
6     continue

      call wheneq( nobs, fg3, 1, 1., gcfpoint, igcfail )

	 call timerdump (55,"delhuv - buddy check wind\0")	 

      write(6,'(//,"DELHUV: Starting wind quality control.",/,
     &  i6," wind observations failed gross check.")') igcfail
 
 
c  Find neighbouring observations to those flagged by the gross check.
c  Perform an interpolation of the good quality neighbours (within a
c  1000km radius) to the location of a suspicious report.
 
cmic$ do all vector shared (alpha, data, drc, gcfpoint, ife, igcfail)
cmic$*    shared (ilat, ilon, ioe, iprs, ityp, iu, iv, max_data)
cmic$*    shared (nobs, nreacc, rade, rdc, srchiz, srchws)
cmic$*    private (cdif1, clat1, clat2, clond, dateline_case)
cmic$*    private (delta, dist, fg1, fg2, gcflat, gpv, i, ifail)
cmic$*    private (isum, j, latmax, latmin, londif, lonmax)
cmic$*    private (lonmin, polar, prs, srchlat, srchlon, srchprs)
cmic$*    private (srchtyp, swts, swtsv, temp, testval, tolerance)
cmic$*    autoscope

	call timerstart(56)
      do 7 i = 1,igcfail
 
        ifail = gcfpoint(i)
        polar = .false.
 
c  Define the search region.
 
        if ( data(ifail+ilat-1) .gt. 84.9 ) then
           latmax = 90.
           latmin = 70.
           polar  = .true.
        else if ( data(ifail+ilat-1) .lt. -84.9 ) then
           latmax = -70.
           latmin = -90.
           polar  = .true.
        else
           latmax = data(ifail+ilat-1) + 10.
           latmin = data(ifail+ilat-1) - 10.
        end if
 
        if ( .not. polar ) then
           gcflat = data(ifail+ilat-1)
           londif = atan2(.31390,cos(gcflat*drc ) ) * rdc / 2.
           lonmax = data(ifail+ilon-1) + londif
           lonmin = data(ifail+ilon-1) - londif
 
c  Case where box overlaps the international date line.
 
           if ( lonmax .gt. 180. ) then
              lonmax = lonmax - 360.
              dateline_case = .true.
           else if ( lonmin .lt. -180. ) then
              lonmin = lonmin + 360.
              dateline_case = .true.
           else
              dateline_case = .false.
           end if
        end if
 
 
c  Find the neighbours.
 
        fg1(1:nobs) = 0.
        fg2(1:nobs) = 0.
        swts   = 0.
        swtsv  = 0.
!        call timerstart (81)
        do j = 1,nobs
 
          srchlat = data( ilat+j-1 )
          srchlon = data( ilon+j-1 )
          srchws  = sqrt( data(iu+j-1)**2 + data(iv+j-1)**2 )
          srchtyp = data( ityp+j-1 )
          srchprs = data( iprs+j-1 )
 
          if ( srchlat .le. latmax .and. srchlat .ge. latmin
     &        .and.  srchtyp .ge. 2. .and.
     &        (abs(srchprs - data(ifail+iprs-1)) .lt.15.) )then
 
 
             if (( .not.(dateline_case).and. srchlon.gt.lonmin
     &                                 .and. srchlon.lt.lonmax)
     &           .or.  (dateline_case  .and. srchlon.gt.lonmin
     &                                 .or.  srchlon.lt.lonmax)
     &           .or.   polar ) then
 
 
 
c   Compute distance between neighbour and suspect point.
 
                clat1 = cos ( srchlat  *drc )
                clat2 = cos ( data(ifail+ilat-1)*drc )
                clond = cos ((srchlon - data(ifail+ilon-1))*drc )
                cdif1 = cos ((srchlat - data(ifail+ilat-1))*drc )
                temp  = 1.-cdif1 + clat1*clat2*(1.-clond)
                dist  = sqrt (2.0*temp) * rade
 
c               print*,' neighbour:', srchlat, srchlon, srchws
 
c  Compute the weight (alpha is the spatial convergence parameter).
 
                fg1(j) = ( alpha - dist ) / ( alpha + dist )
                fg2(j) =(( alpha - dist )/( alpha + dist ))*srchiz
c               weight = ( alpha - dist ) / ( alpha + dist )
c               swts = swts + weight
c               swtsv = swtsv + weight*srchws
             end if
          end if
 
        end do
!        call timerdump (81,"delhuv get neighbors\0")
        do isum = 1,nobs
          swtsv = swtsv + fg2(isum)
          swts  = swts  + fg1(isum)
        end do
 
        if ( swts .gt. 0.) then
 
           gpv = swtsv/swts
           prs = data ( ifail+iprs-1 )
           if ( prs .le. 70. ) tolerance = 2.
           if ( prs .gt. 500.) tolerance = 2.
           if ( prs .le. 500. .and. prs .gt. 70. ) tolerance = 2.5
 
           delta = sqrt( data(ifail+iu-1)**2 + data(ifail+iv-1)**2 )
           if ( ( delta - gpv ) **2  .lt.
     &        (data(ifail+ioe-1)**2 + data(ifail+ife-1)**2)*tolerance
     &       .and. data(ifail+ioe-1) .le. 900. ) then
 
                testval = data(ifail+ityp-1)
                if ((int(testval*100.)-int(testval*10.)*10.).eq.3.)then
                  data(ifail+ityp-1) = data(ifail+ityp-1) + 2.
                else
                  data(ifail+ityp-1) = data(ifail+ityp-1) + 2.01
                end if
                nreacc = nreacc + 1
 
           end if
         end if
 7    continue
 	 call timerdump (56,"delhuv - buddy check wind\0")	 

      write(6,'(" Wind reports reaccepted by buddy check:",i6)')
     &         nreacc
      write(6,'(/," ... wind quality control complete.",/)')
    
      if ( msglev .gt. 3 ) then
      call qcreport ( data, nobs, 2, mlev, nymd, nhms )
      end if
!gvl
#ifdef WITHDELOUTPUT 
      write(91) nobs, nymd, nhms, 'ZUV'
      do 10 i = 1,nobs
        write(91) data(ilat+i-1), data(ilon+i-1),
     &       data(ityp+i-1), data(iprs+i-1), data(ikx+i-1),
     &       data(iu+i-1), data(iv+i-1), data(iz+i-1)
 10   continue
#endif 

      do i = 1,nobs
       data(ityp+i-1) = int(data(ityp+i-1))
      end do

	call timerstart(57)
      call rptlly ( data(ikx), data(ityp), nobs, 5 )
 	 call timerdump (57,"delhuv - rptlly\0")	  
 
c  Call offlvl to alter the type array to allow/disallow off-level
c  data.
 
	call timerstart(58)
      call offlvl ( data, nobs )

 	 call timerdump (58,"delhuv - offlvl\0")	  
      write(6,'(//,"DELHUV: Exiting quality control.")')
 
      return
      end


