      subroutine par_solvedrv ( lu, ld, cov, aquan, data,
     &                      nobs, ztime, mass_err, u_err, v_err )

#include <obs.com>
#ifdef PARALLEL
#include <parenv.h>
      integer  valid_volume
      EXTERNAL valid_volume
#endif
!.....................................................................o
!                                                                     :
!  Program name:  SOLVEDRV                                            :
!                                                                     :
!  Programmer: M. Seablom                                             :
!  Date:       November 15, 1994                                      :
!  Computer:   Cray YMP C90                                           :
!  Language:   Cray cft 6.0 fortran                                   :
!                                                                     :
! Program SOLVEDRV is the driver for the matrix solver. Looping is    :
! performed over all grid volumes. The Choleski algorithm is called   :
! for each matrix, while the looping is spread over the number of     :
! available CPUs.                                                     :
!                                                                     :
! Modifications:                                                      :
!  11/1994 (Seablom) Added NQ quantities to analysis error array.     :
!
!  Gregor von Laszewski, 12/95 Changed to compile on RS6000 using xlf
! 
!.....................................................................o
!     NASA/DAO OPTIMUM INTERPOLATION ANALYSIS  -  VERSION 2.0         :
!.....................................................................o


! Parameter statements will be removed in Fortran 90 version! (temporary
! fix to bypass dynamic space allocation problems).

      parameter( max_nh=600, maxrank=75 )

      integer nreps(50), augment, total_reports, max_reports

      logical first, final, onestage, qobs, offlev, solved

      external  cov

      real  aquan(im, jnp, mlev+1, nq), 
     &      clam(144), datalist(maxrank,50), data(max_obsspace), 
     &      slam(144), min_element, u_err(im,jnp,mlev+1),
     &      v_err(im,jnp,mlev+1), mass_err(im,jnp,mlev+1),
     &      delta(maxrank), symat(maxrank*(maxrank+1)/2),
     &      force(maxrank,max_nh,3), gp(max_nh),
     &      force2(maxrank,max_nh*3), npd_mark, aug(3), 
     &      symat_temp(maxrank*(maxrank+1)/2),
     &      ztime(max_observations),
     &      force3(maxrank,max_nh*3)

      character *8 dataname
      character*24 obsname
      character*60 message
 

      integer gp_no
      integer gp_list(250000)
 
      common /gridxx/ im, jnp, mlev
 
      common /obs/      maxdex,  nfgge,   lun,     dataname(5),
     &                  vhc(13,13,20),    vcorox(18,18,4),
     &                  obserr(200,4,19), ww(13,13,20),
     &                  obsname(200),     qobs(200,5),  offlev(200)
      common /region/   reg(13104)
      common / scan /   maxobs, rscan, onestage, max_data
 
 
      common /sparam/ lsolve, ns, nr, nv, nh(13104), nq,
     &                kq(3), volcentre_x(13104),
     &                ngv, volcentre_y(13104), glatup(13104),
     &                glonup(13104)



      data aug /1.01, 1.1, 10./
!  Function to calculate diagonal entry index.

       ml(i) = i * (i+1) / 2

       tolerance = 1.e-3
       ns = 1

       call getconh( 'DLON    ', dlon )
       call getconh( 'PI180   ', pi180 )

       do i = 1,im
            alam    = dlon * (i-1) * pi180
            clam(i) = cos ( alam )
            slam(i) = sin ( alam )
       end do  

 
c  Set some counters
       gp_no = 0

      matrices = 0
      isizes   = 0
      inreps   = 0
      total_reports = 0
      max_reports = 0
 
c 
      write(6,'(" Now beginning matrix solves.....")')

!  Loop over grid volumes ( horizontal and vertical ) forming and
!  solving matrices.

cmic$ do all shared (aquan, data, im, inreps, isizes)
cmic$*       shared (jnp, ld, lu, matrices, max_data, mlev, ngv, aug)
cmic$*       shared (nh, nq, nv, obserr, reg, nobs, cov, ns, nr )
cmic$*       shared (tolerance,total_reports,max_reports,ztime)
cmic$*       shared (mass_err,u_err,v_err)
cmic$*       private ( force3,nrep,message )
cmic$*       private (iptx,isymc,min_element)
cmic$*       private (datalist, final, ireg, isize, levels,oldval)
cmic$*       private ( npd_mark, force2, solved, weights, icount )
cmic$*       private ( augment, idiag, itry, symat_temp )
cmic$*       private ( indx,isum,info,i,j,k,igp )
cmic$*       private (nreps,symat,delta,force,gp,iq,jq) autoscope

#ifdef PARALLEL
      call block_distribution (rank, processors,ngv,
     $     displacement(0),length(0))
#endif

      do 1 igp = 1, ngv    ! <-- BEGIN looping over volumes.
#ifdef PARALLEL
! block decomposition
!         if ((igp .ge. displacement(rank)) .and.
!     $        (igp .le. (displacement(rank)+length(rank)-1))) then
! cyclic decomposition
        if ( MODULO (igp,processors) .eq. rank) then
#endif 

!         if (1.eq.valid_volume (rank, processors, decomposition,
!     $        igp, ngv)) then

!            write (6,*) "valid volume", igp
         ireg = reg(igp) + 0.1
         force2(1:maxrank,1:max_nh*3) = 0.
         force3(1:maxrank,1:max_nh*3) = 0.

!	 call timerstart (20)
         call search ( igp, lu, ld, datalist, nreps, data, nobs,
     &                 nrep )
!         call timerdump (20,"search\0")

         total_reports = total_reports + nrep
         if ( nrep .gt. max_reports ) max_reports = nrep
 
!  Loop over the levels at a gridpoint.

!	 call timerstart (24)
         do levels = lu, ld, nv

            isize = 0
 
!  Calculate covariance matrix.

            if( nreps(levels).ne.0 ) then

               call cov( nreps(levels), igp,   ireg, levels,
     &                   datalist(1,levels), obserr, isize,
     &                   data, nobs, symat, delta, force, gp,ztime,
     &                   mass_err, u_err, v_err )

               matrices = matrices + 1

            endif
 
            solved = .false.
            itry = 0

            icount = 1
            do i = 1,nq
              do j = 1,nh(igp)*nv
                do k = 1,isize
                  force2(k,icount) = force(k,j,i)
                  force3(k,icount) = force(k,j,i)
                end do
              icount = icount + 1    
              end do
            end do

            symat_temp(1:maxrank*(maxrank+1)/2) =
     &         symat(1:maxrank*(maxrank+1)/2)

!  Solve for the weights.
!	     call timerstart (21)

           do while ((.not. solved).and. (isize .gt. 0) ) 

            info = 0

! Perform Choleski factorization.

            call spptrf ('u', isize,symat,info )

! Return code of info > 0 indicates non-positive definite matrix.

            if ( info .gt. 0 ) then
             
               write(6,'(" NPD matrix:",i6,"  Augment #",i2,2i8)')
     &             igp,itry+1,info,isize
               itry = itry + 1
               symat(1:maxrank*(maxrank+1)/2) = 
     &          symat_temp(1:maxrank*(maxrank+1)/2)
CFPP$ SKIP L
               do 2 augment = 1,isize
                 idiag = augment*(augment+1)*.5
                 symat(idiag) = symat(idiag)*aug(itry)   
 2             continue

               if ( itry .gt. 3 ) then
                  message = "SOLVEDRV: Excessive resolving"
                  call flash(5,message,1)
               end if

               solved = .false.

            else ! the solver has identified a PD matrix...

!  Check for nearly singular matrices....

               solved = .true.
               iptx = 1
               do isymc = 1,isize
                 if ( isymc .eq. 1 ) then
                   if (symat(1)**2.le.1.e-3*symat_temp(1) ) then
                      solved = .false.
                   end if
                 else
                   idiag = isymc*(isymc+1)*.5
                   if (symat(idiag)**2 .le. 1.e-3*symat_temp(idiag) ) 
     &               solved = .false.
                 end if
               end do

               if ( .not. solved ) then

                write(6,*) rank, ':', 'ILL matrix:',igp,'Augment #',
     &             ,itry+1,isize
                itry = itry + 1
                symat(1:maxrank*(maxrank+1)/2) =
     &           symat_temp(1:maxrank*(maxrank+1)/2)
CFPP$ SKIP L
                do 6 augment = 1,isize
                   idiag = augment*(augment+1)*.5
                   symat(idiag) = symat(idiag)*aug(itry)
 6              continue
                if ( itry .gt. 3 ) then
                   message = "SOLVEDRV: EXCESSIVE RESOLVING."
                   call flash(5,message,1)
                end if

               end if

            end if

           end do

!
!  Perform Choleski solution.
!
           call spptrs('u', isize,nh(igp)*nv*nq,symat,force2,         
     &                 maxrank, info )         

!           call timerdump (21,"solve\0")

           if ( isize .gt. 0 ) then
!              call timerstart(23)

             call par_store ( float(isize), float(igp),
     &                  float(levels), gp, delta, force3,
     &                  force2,aquan, nh(igp), force2,
     &                  force3, mass_err, u_err, v_err)
!           call timerdump (23,"store\0")
           end if
                
         end do  ! <--  END looping over levels.
!         call timerdump (23,"levelsolve")
#ifdef PARALLEL
         end if
#endif
  1   continue   ! <--  END looping over grid volumes.


!     call par_store_exchange (...)
  
      write(6,*) '...matrix solutions complete.'
      write(6,*)
      write(6,*) 'Average size of data within minivolumes:',
     &     total_reports / ngv
      write(6,*) ' Largest amount of data encountered:',max_reports

! this is a problem in the parallel code
 
!  Average the wind increments at the poles.

      if( nq.eq.3 ) then

        do levels = lu, ld

         call avguvinc ( im, clam, slam,
     &        aquan(1,jnp,levels,2), aquan(1,jnp,levels,3),
     &        aquan(1,  1,levels,2), aquan(1,  1,levels,3)    )

        end do   

      endif
 
 
      return
      end
