!     stack
!     Gregor von Laszewski

      subroutine push(msg,value,pos)
      real msg(*)
      real value
      integer pos
      pos = pos + 1
      msg(pos) = value
      end ! subroutine push

      subroutine pop(msg,value,pos)
      real msg(*)
      real value
      integer pos
      pos = pos + 1
      value = msg(pos)
      end ! subroutine pop

! 

      subroutine par_store_exchange ( rank, 
     $     no_grid_volumes, grid_volume, aquan, mass_err, u_err, v_err )

      implicit none
 
      integer rank
      integer no_grid_volumes
      real    grid_volume(*) ! array of igp's
      real    aquan( im*jnp*(mlev+1), nq ), 
      real    mass_err(im*jnp*(mlev+1))
      real    u_err(im*jnp*(mlev+1))
      real    v_err(im*jnp*(mlev+1))

      
      integer i,ii,j igp

      real var_tmp


#ifdef PARALLEL
!   #include <par_store.h>

      
      integer, parameter :: MSG_SHIFT = 1567
      integer, parameter :: MAX_MSG   = 1000000

      real msg(MAX_MSG)
      real in_msg(MAX_MSG)
      integer no_of_igp, msg_pos, igp_nq
      integer in_length, msg_length
      integer next_proc, prev_proc
      real, parameter :: empty = 0
#endif

      common /gridxx/ im, jnp, mlev
 

      common /sparam/ lsolve, ns, nr, nv, nh(13104), nq,
     &                kq(3), volcentre_x(13104),
     &                ngv, volcentre_y(13104), glatup(13104),
     &                glonup(13104)


 
! Store analysis increment and analysis error if matrix is      
! positive definite.

      next_proc = MOD((processors + (rank + 1)), processors)
      prev_proc = MOD((processors + (rank - 1)), processors)

      msg_length = 0
      call push(msg,float(0),msg_length)
      call push(msg,float(nq),msg_length)

      do ii = 1,no_of_grid_volumes
         igp = grid_volume(ii)
         
         call  push(msg, float(igp), msg_length)
         call push(msg, mass_err(igp), msg_length)

         if ( nq .eq. 3 ) then
            call push(msg, u_err(igp), msg_length)
            call push(msg, v_err(igp), msg_length)
         end if

         do i=1,nq 
            call push (msg, aquan(igp,i), msg_length)
         end do

      end do

      msg(1) = no_of_grid_volumes  ! number of ipg's

      if (msg_length .gt. MAX_MSG) then
         write (6,*) rank,
     $        'msg(',MAX_MSG,') to short, should be ', msg_length
!      else
!         write (6,*) rank, 
!     $        'msg(',MAX_MSG,') but used only ', msg_length
!          length is most likely only 58  (8*7+2)
      end if   
!     note the msgs could be of different length


      msg_pos = 0

      do p=1, processors-1

         call mpi_sendrecv(msg,  msg_length,MPI_REAL,next_proc,MSG_SHIFT,
     $        in_msg,in_length,MPI_REAL,prev_proc,MSG_SHIFT,
     $        MPI_COMM_WORLD, mpi_err)

         ! could be improved if communic. and calculation are 
         ! overlapped using assyncrounous comm.

         msg(1:in_length) = in_msg(1:in_length)


         call pop(msg,no_of_igp, msg_pos)
         call pop(msg,igp_nq,msg_pos)

         do j=1,no_of_igp
            call pop(msg, igp, msg_pos)
            call pop(msg, mass_err(igp), msg_pos)
            if ( igp_nq .eq. 3 ) then
               call pop (msg, u_err(igp), msg_pos)
               call pop (msg, v_err(igp), msg_pos)
            end if
            do i=1,nq 
               call pop (msg, aquan(igp,i), msg_pos)
            end do
         end do
      end do


#endif


      end
