!     stack
!     Gregor von Laszewski

      subroutine ipush(msg,value,pos)
      integer msg(*)
      integer value
      integer pos
      pos = pos + 1
      msg(pos) = value
      end ! subroutine push

      subroutine ipop(msg,value,pos)
      integer msg(*)
      integer value
      integer pos
      pos = pos + 1
      value = msg(pos)
      end ! subroutine pop

! 


#ifdef SEQUENTIAL
      subroutine store_p(rank,grid_volume, level, gp, del, b, x, aquan,
     &                    mv_size, weights, force, mass_err, u_err,
     &                    v_err )
#endif
#ifdef PARALLEL
      subroutine store_p(rank,grid_volume, level, gp, del, b, x, aquan,
     &                    mv_size, weights, force, mass_err, u_err,
     &                    v_err , gp_no, gp_list)
#endif
!.....................................................................o
!                                                                     :
!  Program name:  STORE                                               :
!                                                                     :
!  Programmer: M. Seablom, S. Bloom, D. Lamich                        :
!  Date:       November, 1994                                         :
!  Computer:   Cray YMP C90                                           :
!  Language:   Cray cft77 6.0 fortran                                 :
!                                                                     :
!  Computes and stores analysis errors and analysis increments from   :
!  the computed weights.                                              :
!                                                                     :
! Modifications:                                                      :
!                                                                     :
!   10-1992: (Lamich)  Included diagnostic prints.                    :
!   09-1993: (Seablom) Removed matrix stacks; cleaned up code.        :
!   11-1994: (Seablom) Added wind quantities to analysis errors.      :
!                                                                     :
!.....................................................................o
!                                                                     !
! NASA Goddard Space Flight Center       Data Assimilation Office     !
!                                                                     !
! Goddard Earth Observing System                                      !
! Data Analysis System - Version 2.0                                  !
! November, 1994                                                      !
!                                                                     !
!---------------------------------------------------------------------o

!implicit NONE
 
! Parameter statements will be removed in Fortran 90 version.

      parameter(max_nh=600,maxrank=75)

      integer mv_size, i, ie, igp, igv, imjnp,
     &        nnr, nnv, horiz_gp, vert_gp, quan, gridpoint,
     &        vert_loc, horiz_loc, lev_dump

      real    grid_volume, aquan( im*jnp*(mlev+1), nq ), rank, 
     &        b( nr,nv,mv_size,nq), mass_err(im*jnp*(mlev+1)),
     &        u_err(im*jnp*(mlev+1)), v_err(im*jnp*(mlev+1)),
     &        del(nr), gp(mv_size), x( nr,nv,mv_size,nq),
     &        lon_dump, lat_dump, level, anal_inc(nq), rhs_save(nr,nq),
     &        inc_save(nr,nq), wgt_save(nr,nq), del_save(nr) , dlat,
     &        dlon, weights(maxrank,max_nh*3), force(maxrank,max_nh*3)

      integer gp_no
      integer gp_list(*)

      logical mv_dump
      character*60  message

      common /gridxx/ im, jnp, mlev
 
      common/mvdumper/ num_dump, lat_dump(20),
     &                 lon_dump(20), lev_dump(20)

      common /sparam/ lsolve, ns, nr, nv, nh(13104), nq,
     &                kq(3), volcentre_x(13104),
     &                ngv, volcentre_y(13104), glatup(13104),
     &                glonup(13104)


      jx( xlat,dlat ) = ( xlat + 90. ) / dlat + 1.1 ! Functions to lo-  
      ix( xlon,dlon ) = ( xlon +180. ) / dlon + 1.1 ! cate gridpoints.

      call getconh('DLAT    ',dlat  )
      call getconh('DLON    ',dlon  )

      mv_dump = .false.
      message(1:60) = ' '
      vert_loc = 0
      horiz_loc = 0
 
 
      irank = nint(rank)

      icount = 0
      do k = 1,nq
       do j = 1,mv_size*nv
        icount = icount + 1
        do i = 1,irank
          x(i,j,1,k)=weights(i,icount)
          b(i,j,1,k)=force(i,icount)
        end do
       end do
      end do

!  Determine if this grid volume is flagged for a diagnostic print.

      if ( num_dump .ne. 0 ) then

       do i = 1,num_dump


         gridpoint = ( (jx(lat_dump(i),dlat)-1)*im +
     &                  ix(lon_dump(i),dlon) )

         do j = 1,mv_size

           if ( gridpoint .eq. nint( gp(j) ) ) then
             horiz_loc = j
             vert_loc  = 1
             if ( mod(lev_dump(i),2) .eq. 0 ) vert_loc = 2
             idump = i
           end if
           

         end do

       end do
      end if
         
!  Save forcing terms and weights for diagnostic print.

      if ( mv_dump ) then

        rhs_save(1:irank,1:nq) = b(1:irank,vert_loc,horiz_loc,1:nq)
        wgt_save(1:irank,1:nq) = x(1:irank,vert_loc,horiz_loc,1:nq)
        del_save(1:irank) = del(1:irank)

      end if

!  Calculate the OI estimate of the height and wind analysis errors.

      do quan = 1,nq
       do horiz_gp = 1,mv_size
        do vert_gp = 1,nv
            b( 1:irank, vert_gp, horiz_gp, quan ) =
     &        b( 1:irank,vert_gp, horiz_gp, quan ) *
     &        x( 1:irank, vert_gp, horiz_gp, quan )
          ie = nint( level ) + ( vert_gp - 1 )
          if ( level .gt. mlev ) ie = 1+MLEV
          ie = nint( gp( horiz_gp ) ) + im * jnp * ( ie - 1 )
        if ( quan .eq. 1 ) then
          b( 1, vert_gp, horiz_gp, quan) = mass_err( ie )**2 -
     &                       b( 1, vert_gp, horiz_gp, quan )
        else if ( quan .eq. 2 ) then
          b( 1, vert_gp, horiz_gp, quan) = u_err( ie )**2 -
     &                       b( 1, vert_gp, horiz_gp, quan )
        else if ( quan .eq. 3 ) then
          b( 1, vert_gp, horiz_gp, quan) = v_err( ie )**2 -
     &                       b( 1, vert_gp, horiz_gp, quan )
        end if
          if ( irank .gt. 1 ) then
            do nnr = 2,irank
             b( 1, vert_gp, horiz_gp, quan ) =
     &                     b( 1, vert_gp, horiz_gp, quan ) -
     &                     b( nnr, vert_gp, horiz_gp, quan )
            end do
          endif

          if ( b(1,vert_gp,horiz_gp,quan) .ge. 0.0 ) then
            b( 1,vert_gp,horiz_gp,quan ) =
     &           sqrt(b( 1,vert_gp,horiz_gp,quan ))
          end if

        end do
       end do
      end do
 
! Now compute the increments for gridpoints and variables within the 
! minivolume.

      do quan = 1,nq
       do horiz_gp = 1,mv_size
        do vert_gp = 1,nv
          x(1:nr,vert_gp,horiz_gp,quan) = x(1:nr,vert_gp,horiz_gp,quan)
     &                             * del ( 1:nr )

! Save individual increments for diagnostic print.

          if (horiz_gp .eq. horiz_loc .and. vert_gp .eq. vert_loc) 
     &     inc_save(1:irank,quan) = x(1:irank,vert_loc,horiz_loc,quan)
 
! Sum individual increments to form analysis increments.

         if ( irank .gt. 1 ) then

          do nnr = 2,irank
           x(1,vert_gp,horiz_gp,quan) = x(1,vert_gp,horiz_gp,quan) +
     &                                  x(nnr,vert_gp,horiz_gp,quan)
          end do

         end if

        end do
       end do
      end do
         
! Save total increments for diagnostic print.

      if ( mv_dump ) 
     &   anal_inc(1:nq) = x(1,vert_loc,horiz_loc,1:nq)

! Check for negative analysis errors. 

      do horiz_gp = 1,mv_size
       do vert_gp = 1,nv

! where ( b(1,vert_gp,horiz_gp,1) < 0.0 )

         if ( b(1,vert_gp,horiz_gp,1) .lt. 0. ) then

           write (6,'(//,"CATASTROPHIC CONDITION - NEGATIVE",
     &            " ANALYSIS ERRORS",/," Cannot continue -",
     &            " program abort.")')
           print*,' Value of B at time of error:',
     &           b(1,vert_gp,horiz_gp,1)

           message = 'STORE: Negative analysis errors.'
           write(6,'(1x," Error occurred at grid volume:",
     &      i9)') nint(grid_volume)
           call flash( 6,message,1 )

         else if ((b(1,vert_gp,horiz_gp,2) .lt. 0. .or.
     &             b(1,vert_gp,horiz_gp,3) .lt. 0.).and. 
     &             ( level .ne. mlev+1 .and. nq .gt. 1 ))then

           continue
!          write (6,'(//,"PROBLEM DETECTED: - NEGATIVE",
!    &            " WIND ANALYSIS ERRORS",/," Ambiguous ",
!    &            "response.")')
!          message = 'STORE: Negative wind analysis errors.'
!          call flash(3,message,1)

         end if

 
! endwhere

        end do
      end do
 
! Store analysis increment and analysis error if matrix is      
! positive definite.

      imjnp = im*jnp

      igv = nint( level )
      igv = imjnp * ( igv - 2 )
        
      do horiz_gp = 1,mv_size
          igp = nint( gp(horiz_gp)  + igv )
          do vert_gp = 1,nv
            igp = igp + imjnp
#ifdef PARALLEL
            call ipush (gp_list, igp, gp_no)
#endif
            mass_err(igp) = b(1,vert_gp,horiz_gp,1)
            if ( nq .eq. 3 ) then
            u_err(igp) = b(1,vert_gp,horiz_gp,2)
            v_err(igp) = b(1,vert_gp,horiz_gp,3)
            end if
            aquan(igp,1:nq) = x(1,vert_gp,horiz_gp,1:nq)
          end do
      end do

! #ifdef PARALLEL
! mark the gidpoints which are modified
! propagete modifications to other processors
! #endif


! Call DIAGMVP to perform diagnostic prints.

      if ( mv_dump ) then

        call diagmvp ( anal_inc, rhs_save, inc_save, wgt_save,
     &        del_save, irank, nq, nr, lat_dump(idump), lon_dump(idump),
     &        lev_dump(idump) )
        
      end if

      end
