
 
      SUBROUTINE TERPV ( DEGLAT, DEGLON, FINT, LEN, F )
 
C  BILINEAR INTERPOLATION OF A FIELD, F, TO A SET OF LAT/LON LOCATIONS
C  DEGLAT(I) IS ASSUMED TO BE BETWEEN  -90.0 AND  +90.0
C  DEGLON(I) IS ASSUMED TO BE BETWEEN -180.0 AND +180.0
 
 
      DIMENSION         DEGLAT(LEN)
      DIMENSION         DEGLON(LEN)
      DIMENSION         F(IM,JNP)
      real              fill
      DIMENSION         FINT(LEN)
      INTEGER           II
      INTEGER           I0
      INTEGER           I1
      INTEGER           J0
      INTEGER           J1
      REAL              QDEGDX
      REAL              QDEGDY
      REAL              WTI1
      REAL              WTJ1
      REAL              XI
      REAL              YJ
 
C  IM AND JNP ARE NEEDED FROM COMMON BLOCK /CONTRL/
      LOGICAL           STRAT,   SLP,     FDCOS,   FEXP,    SAVEAE,
     $                  SIGDAT
      COMMON /CONTRL/   IM,      JNP,     NB,      NDALT,   NHMS,
     $                  NLAY,    NLAYM1,  NLAYP1,  NYMD,    STRAT,
     $                  IDIMEN,  SLP,     FDCOS,   FEXP,    SAVEAE,
     $                  SIGDAT,  MAXREP
 
      data fill /1.e10/
 
      QDEGDX = IM / 360.0
      QDEGDY = (JNP-1) / 180.0
 
      DO 100 II = 1, LEN
 
         YJ    = ( DEGLAT(II) +  90.0 ) * QDEGDY + 1.0
         J0    = YJ
         J0    = MAX0( J0, 1 )
         J0    = MIN0( J0, JNP-1 )
         J1    = J0 + 1
 
         XI    = ( DEGLON(II) + 180.0 ) * QDEGDX + 1.0
         IF( XI.GE.(IM+1) ) XI = XI - IM
         IF( XI.LT.1 ) XI = XI + IM
         I0    = XI
         I0    = MAX0( I0, 1 )
         I0    = MIN0( I0, IM )
         I1    = I0 + 1
         IF( I0.EQ.IM ) I1 = 1
 
         WTJ1  = YJ - J0
         WTI1  = XI - I0
 
c Screen for fill values on level - replace fill values according 
c to predetermined criteria. 
 
C
C For brevity, the four corner points will be identified as follows:
C
C         X X   8               X X   4
C         X X                   X X
C         C  (D3)               D  (D4)
C
C
C
C
C         X X   1               X X   2
C         X X                   X X
C         A   (D1)              B   (D2)
C
C
C
C Assign corner values
C
      A = F(I0,J0)
      B = F(I1,J0)
      C = F(I0,J1)
      D = F(I1,J1)
 
C Identify pattern of filled points by using a 4 bit binary representation,
C e.g., 2 (decimal) = 0010 (binary) = corner B only is filled. 
 
      IFL  = 0
 
      IF( A .EQ. FILL) THEN
              IFL = 1
      ENDIF
      IF( B .EQ. FILL) THEN
              IFL = IFL + 2
      ENDIF
      IF( D .EQ. FILL) THEN
              IFL = IFL + 4
      ENDIF
      IF( C .EQ. FILL) THEN
              IFL = IFL + 8
      ENDIF
 
C
C Filled values are repalced with values obtained from the other corners
C so that the horizontal interpolation can be done.  Note that there are 
C 16 special cases of filled patterns. 
C
      IF( IFL .EQ. 0 ) THEN
              D1 = A
              D2 = B
              D3 = C
              D4 = D
      ELSE IF( IFL .EQ. 1 ) THEN
              D1 = .5 * ( B + C )
              D2 = B
              D3 = C
              D4 = D
      ELSE IF( IFL .EQ. 2 ) THEN
              D1 = A
              D2 = .5 * ( A + D )
              D3 = C
              D4 = D
      ELSE IF( IFL .EQ. 3 ) THEN
              D1 = .5 * ( C + D )
              D2 = D1
              D3 = C
              D4 = D
      ELSE IF( IFL .EQ. 4 ) THEN
              D1 = A
              D2 = B
              D3 = C
              D4 = .5 * ( B + C )
      ELSE IF( IFL .EQ. 5 ) THEN
              D1 = .5 * ( B + C )
              D2 = B
              D3 = C
              D4 = D1
      ELSE IF( IFL .EQ. 6 ) THEN
              D1 = A
              D2 = .5 * ( A + C )
              D3 = C
              D4 = D2
      ELSE IF( IFL .EQ. 7 ) THEN
              D1 = C
              D2 = C
              D3 = C
              D4 = C
      ELSE IF( IFL .EQ. 8 ) THEN
              D1 = A
              D2 = B
              D3 = .5 * ( A + D )
              D4 = D
      ELSE IF( IFL .EQ. 9 ) THEN
              D1 = .5 * ( B + D )
              D2 = B
              D3 = D1
              D4 = D
      ELSE IF( IFL .EQ. 10) THEN
              D1 = A
              D2 = .5 * ( A + D )
              D3 = D2
              D4 = D
      ELSE IF( IFL .EQ. 11) THEN
              D1 = D
              D2 = D
              D3 = D
              D4 = D
      ELSE IF( IFL .EQ. 12) THEN
              D1 = A
              D2 = B
              D3 = .5 * ( A + B )
              D4 = D3
      ELSE IF( IFL .EQ. 13) THEN
              D1 = B
              D2 = B
              D3 = B
              D4 = B
      ELSE IF( IFL .EQ. 14) THEN
              D1 = A
              D2 = A
              D3 = A
              D4 = A
      ELSE IF( IFL .EQ. 15) THEN
              D1 = FILL
              D2 = FILL
              D3 = FILL
              D4 = FILL
      ENDIF
 
C  HORIZONTAL INTERPOLATION
 
      if (d1.eq.fill) then 
         fint(ii) =  fill 
      else 
         A = D1 
         B = D2 
         C = D3 
         D = D4 
         fint(ii) =  WTJ1  * ( WTI1*D+(1.0-WTI1)*C )
     $        + (1.0-WTJ1) * ( WTI1*B+(1.0-WTI1)*A )
      end if 
 
  100 CONTINUE
 
      RETURN
      END
