      SUBROUTINE PTSDRV ( PHIS, TSURFA, psla, USLFG, VSLFG,
     .                    UFG, VFG, TFG, RMIXFG,
     .                    PMAND,PMIDM,
     .                    PS, U, V, T, SH,dsig,
     .                    SIGE,pkht,IWATER,PTOP,IM,JNP,NLAY,MLEV)
C
C Modified: 901215 DJL - Compute surface pressure (PSFC) and pass to
C                        PTOSIGW so that test can be made for underground
C                        analysis increments on mandatory levels.
C           910929 DJL - Argument list changed to accept previously
C                        computed analysis increments on pressure levels.
C                        Subroutine MANDELT no longer needed and removed.
c
c           920204 rcg - Iterates once to update the virtual
c                        surface temperature using after analysis
c                        sigma level temperatures.  Virtual 
c                        surface temperature is used to compute
c                        surface pressure which in turn sets the
c                        sigma level pressures.
c
c           930106 djl - Interpolates from madatory levels to first guess
c                        sigma level pressures.  Surface pressure and pkht
c                        are updated to analysis values only after the 
c                        upper air interpolations.  The analysis temperatures on 
c                        sigma levels are dried out using the analyzed specific
c                        humidity before being returned to to the model.
c
 
 
C LOCAL VARIABLES
C ---------------
 
      LOGICAL OCEAN(13104)
      LOGICAL STRAT,SLP,FDCOS,FEXP,SAVEAE,SIGDAT
 
C SUBROUTINE ARGUMENT DIMENSIONS
C ------------------------------
 
      DIMENSION      PHIS(IM,JNP), TSURFA(IM,JNP),SIGE(nlay+1),
     .               USLFG(IM,JNP), VSLFG(IM,JNP), UFG(IM,JNP,MLEV),
     .               VFG(IM,JNP,MLEV), TFG(IM,JNP,MLEV),
     .               HTFG(IM,JNP,MLEV), RMIXFG(IM,JNP,MLEV),
     .               RELHFG(IM,JNP,MLEV),PMAND(mlev),PMIDM(mlev-1)
 
      DIMENSION PSLA(IM,JNP), USLA(IM,JNP), VSLA(IM,JNP),
     .               UA(IM,JNP,MLEV), VA(IM,JNP,MLEV), TA(IM,JNP,MLEV),
     .               HTA(IM,JNP,MLEV), RMIXA(IM,JNP,MLEV)
 
      DIMENSION  PS(IM,JNP),
     .               U(IM,JNP,NLAY), V(IM,JNP,NLAY),
     .               T(IM,JNP,NLAY), SH(IM,JNP,NLAY)
C
      dimension dsig(nlay)
      DIMENSION PKHT(IM,JNP,NLAY)
      INTEGER IWATER(13104)
 
 
      COMMON /CONST/    SIGE0(20),DLAT0,  DLON0,   PI0,
     $                  PI0180,  PTOP0,
     $                  PINT0,   GI,     OMG2RE0,
     $                  RADEI0, RADI2, GRADI,
     $                  GRADI2,G02,     POLEFC,RTD0,    POLELT
 
 
 
C PHYSICAL CONSTANTS
C -------------------
 
      COMMON /PHYSIC/ BETA,  CP, GRAV,OMEGA2,RADE,RGAS,
     .                ROCP,CPOG,GAMMA, ROCP1,GOCP
 
 
C INDEXES OF PROFILES TO PRINT
C ----------------------------
 
      COMMON /PRNTPROF/ NPROF,ILOC(10),JLOC(10)
 
C PRODUCTS OF GRID LIMITS
C -----------------------
 
      COMMON /PRODLIM/ IJMLEV, IJMLEVM1, ILBELOW, IMJNP, IMLSM1,
     .                 IMMLEV, IMMLEVM1, IMMLEVP1, IMNLAY, IMNLAYP1
 
 
C Define work space arrays. Any dimension may be used.
 
      dimension
     .   PEDGESIG(im,nlay+1,jnp), PMANDK(mlev),
     .   PMANDK1(mlev),
     .   PMIDSIG(im,nlay,jnp),
     .       VDPMANDK(im,mlev-1), VPMAND(im,mlev),
     .       VPMIDM(im,mlev-1), VPMIDMK(im,mlev-1),
     .    HWIM1(im,mlev), HWIMM11(im,mlev-1),
     .       HWIMP11(im,mlev+1),
     .       HWIN1(im,nlay), HWIN2(im,nlay), HWIN3(im,nlay),
     .       HPROFILE(7,18,10), TPROFILE(8,19,10),
     .       BUFFR(im,nlay),
     .       psfc(im,jnp)
      INTEGER ILOC0(9),JLOC0(9)
c
      dimension   vtheta_sig(im,jnp,nlay)
      DIMENSION PKZ (IM,JNP,NLAY)
 
      DATA   NPROF0 /0/
      DATA   ILOC0    /51,53,55,57,59,61,63,65,67/
      DATA   JLOC0    /32,32,31,31,31,31,31,31,31/
 
      PRINT 1
    1 FORMAT(/,' *** IN PTSDRV ***')
 
C SET PROFILES AT WHICH DIAGNOSTIC PRINTS ARE TO BE MADE
C  NPROF = NUMBER OF PROFILES OR 0 IF NO PRINTS ARE TO BE MADE
C  ILOC  = ARRAY OF LONGITUDE INDEXES FOR PROFILES
C  JLOC  = ARRAY OF LATITUDE INDEXES FOR PROFILES
C ------------------------------------------------------------
 
      NPROF=NPROF0
      DO 2 IP=1,NPROF
      ILOC(IP)=ILOC0(IP)
    2 JLOC(IP)=JLOC0(IP)
 
C SET UP OCEAN FLAG
C -----------------
 
 
      DO 33 KK=1,13104
C
         IF(IWATER(KK) .EQ. 0) THEN
             OCEAN(KK) = .FALSE.
         ELSE
             IF(IWATER(KK) .EQ. 1) THEN
                  OCEAN(KK) = .TRUE.
             ENDIF
         ENDIF
 
   33 CONTINUE
C
C            CALL A ROUTINE TO PRINT OCEAN FLAGS
C
c      CALL PRNTOCN(OCEAN,IM,JNP)
 
C SET MANDATORY LEVEL PRESSURES DEPENDING ON TYPE OF ANALYSIS
C -----------------------------------------------------------
 
      CALL PMANDLEV ( PMAND, VPMAND, PMANDK, VDPMANDK, PMANDK1,
     .                PMIDM, VPMIDM, VPMIDMK, IM,MLEV )
 
 
C SET UP FOR REFERENCE PRESSURE COMPUTATION
C -----------------------------------------
 
      GOVR=GRAV/RGAS
      BETAHALF=BETA*0.50
C
C Convert potential temperatures (T/p**kappa) on model levels
C to virtual temperatures, using first guess model level pressures

      CALL PKAPPA(PS,PKHT,PKZ,IM,JNP,NLAY)
C
      DO  K = 1,NLAY
         DO I = 1,IM*JNP
            T(I,1,K) = T(I,1,K) * PKZ(I,1,K) *
     .                        (1. + .61 * sh(i,1,k))
         end do   
      end do   

C Compute pressures at model levels

      CALL PSIGLEV ( PS,SIGE,PTOP,PMIDSIG,PEDGESIG,IM,NLAY,JNP)
 
C COMPUTE ACTUAL SURFACE PRESSURE
 
      do i=1,im*jnp
         PSFC(I,1)=PS(I,1)+PTOP
      end do
 
C Interpolate pressure level virtual temperature analysis increments to 
C model levels and update model level virtual temperatures
 
      do   lat = 1,jnp 
         CALL BUFFER(BUFFR,T,IM,LAT,JNP,NLAY)
         CALL PTOSIGT ( LAT, TFG, BUFFR,
     .                  PMIDSIG(1,1,LAT), PEDGESIG(1,1,LAT), HWIN1,
     .                  TPROFILE, pmand, PMIDM, PTOP)
         CALL TRANSFR(T,BUFFR,IM,LAT,JNP,NLAY)
      end do 
 
C Interpolate pressure level mixing ratio analysis increments to 
C model levels and update first guess.  Mixing ratios are converted
C to specific humidities.

      do   lat = 1,jnp 
         CALL BUFFER(BUFFR,SH,IM,LAT,JNP,NLAY)
         call PTOSIGM(lat,rmixfg,buffr,pmand,pmidsig(1,1,lat),
     .                psfc(1,lat),im,jnp,mlev,nlay)
         CALL TRANSFR(sh,BUFFR,IM,LAT,JNP,NLAY)
      end do 
 
c Compute model level dry bulb potential temperatures (using analysis
c specific humidity)
 
c Compute model level virtual potential temperatures 
 
      do  l = 1,nlay
         do  j = 1,jnp
            do  i = 1,im
               vtheta_sig(i,j,l) = t(i,j,l) / PKZ(i,j,l)
            end do
         end do
      end do

c Compute model level dry bulb potential temperature

      do  l = 1,nlay
         do  j = 1,jnp
            do  i = 1,im
               t(i,j,l) = vtheta_sig(i,j,l) / 
     .                        (1. + .61 * sh(i,j,l))
            end do
         end do
      end do
 
C INTERPOLATE MANDATORY LEVEL U AND V WIND DELTAS TO SIGMA LEVELS AND
C ADD THEM TO FIRST GUESS U AND V
 
      DO LAT=1,JNP
         CALL BUFFER(BUFFR,U,IM,LAT,JNP,NLAY)
         CALL PTOSIGW ( LAT, UFG, USLFG, BUFFR, PMAND, PSLA, 
     .        PMIDSIG(1,1,LAT), OCEAN, PTOP, TPROFILE, psfc(1,lat) )
         CALL TRANSFR(U,BUFFR,IM,LAT,JNP,NLAY)
      end do
 
      DO LAT=1,JNP
         CALL BUFFER(BUFFR,V,IM,LAT,JNP,NLAY)
         CALL PTOSIGW ( LAT, VFG, VSLFG, BUFFR, PMAND, PSLA, 
     .        PMIDSIG(1,1,LAT), OCEAN, PTOP, TPROFILE, psfc(1,lat) )
         CALL TRANSFR(V,BUFFR,IM,LAT,JNP,NLAY)
      end do
 
c Compute virtual surface temperature
 
      call tspbl(ps,ptop,vtheta_sig,dsig,im,jnp,nlay,tsurfa)

C Using this estimate of the after analysis virtual surface 
C temperature, compute the after analysis reference pressure
 
      DO LAT=1,JNP
       DO I = 1,IM
         Z=PHIS(I,LAT)*GI
         TMID=TSURFA(I,LAT)+BETAHALF*Z
         PART=EXP(-GOVR*Z/TMID)
         PS(I,LAT)=PSLA(I,LAT)*PART-PTOP
       end do  
      end do

c Recompute pkht using analysis surface pressure
 
      AKAP = GETCON('KAPPA')
      DO  L = 1,NLAY
      DO  I = 1,IM*JNP
         PKHT(I,1,L)   = SIGE(L+1)*PS(I,1) + PTOP
         PKHT(I,1,L)   = PKHT(I,1,L) * * AKAP
      end do
      end do
 
      RETURN
      END
