      SUBROUTINE TRIGSET                                                
                                                                        
C  SETS CONSTANTS AND TABLE NEEDED FOR INLINE APPROXIMATIONS TO SINE    
C  AND COSINE                                                           
                                                                        
      INTEGER           NTRIG                                           
      PARAMETER       ( NTRIG = 4 )                                     
                                                                        
      COMMON /TRIGCOM/  NTRIGS, TRIGCON, TRIGTAB                        
                                                                        
      REAL              ANGDEG                                          
      REAL              CCOS                                            
      REAL              COSONE                                          
      REAL              COSTAB                                          
      INTEGER           N                                               
      INTEGER           NSUBSC                                          
      INTEGER           NTRIGS                                          
      REAL              SINDEL                                          
      REAL              SINONE                                          
      REAL              SINTAB                                          
      REAL              SSIN                                            
      REAL              SUBSCR                                          
      REAL              TRIGCON                                         
      REAL              TRIGTAB(450*NTRIG+1)                            
                                                                        
C  TO USE THE IN-LINE SINE AND COSINE APPROXIMATIONS, PASS NTRIGS,      
C  TRIGCON, AND TRIGTAB ( DIMENSIONED TRIGTAB(450*NTRIGS+1) ) TO        
C  YOUR ROUTINE AND INCLUDE THE FOLLOWING BLOCK OF STATEMENT FUNCTIONS. 
                                                                        
      SUBSCR(ANGDEG) = NTRIGS * ( ANGDEG + 180.0 ) + 1.5                
      NSUBSC(ANGDEG) = IFIX ( SUBSCR( ANGDEG ) )                        
      SINDEL(ANGDEG) = ( SUBSCR(ANGDEG)-NSUBSC(ANGDEG)-0.5 ) * TRIGCON  
      SINTAB(ANGDEG) = TRIGTAB( NSUBSC(ANGDEG) )                        
      COSTAB(ANGDEG) = TRIGTAB( NSUBSC(ANGDEG) + 90*NTRIGS )            
      SSIN(ANGDEG)   = SINTAB(ANGDEG) + COSTAB(ANGDEG) * SINDEL(ANGDEG) 
      CCOS(ANGDEG)   = COSTAB(ANGDEG) - SINTAB(ANGDEG) * SINDEL(ANGDEG) 
                                                                        
C     ANGDEG MUST BE BETWEEN -180.0 AND +180.0, OVER THIS ENTIRE RANGE  
C     THE APPROXIMATIONS HAVE THE FOLLOWING ABSOLUTE ACCURACY: THEY ARE 
C     EXACT FOR ANGLES WHICH ARE MULTIPLES OF ( 1.0 / NTRIGS ) DEGREES. 
                                                                        
C     NTRIGS     COSTAB       CCOS          SINTAB       SSIN           
C                                                                       
C       1       0.872E-02  0.381E-04       0.872E-02  0.381E-04         
C       2       0.436E-02  0.109E-04       0.436E-02  0.113E-04         
C       3       0.291E-02  0.924E-05       0.291E-02  0.900E-05         
C       4       0.218E-02  0.787E-05       0.217E-02  0.721E-05         
C       5       0.174E-02  0.555E-05       0.174E-02  0.507E-05         
C       6       0.145E-02  0.596E-05       0.145E-02  0.560E-05         
                                                                        
C  SET ENTRIES IN COMMON BLOCK /TRIGCOM/                                
      NTRIGS  = NTRIG                                                   
      TRIGCON = 4.0 * ATAN( 1.0 ) / ( 180.0 * NTRIGS )                  
      DO 100 N = 1, 450 * NTRIGS + 1                                    
         TRIGTAB(N) = - SIN( ( N - 1 ) * TRIGCON )                      
  100 CONTINUE                                                          
                                                                        
C  EXERCISE THE STATEMENT FUNCTIONS                                     
      COSONE = CCOS(1.0)                                                
      SINONE = SSIN(1.0)                                                
                                                                        
      RETURN                                                            
      END                                                               
