 
      subroutine zuvanl ( z_err,u_err,v_err,z,u,v,t,psfc,z1000,
     &         cov,aquan, ze,zelat,zelon,nze,data,nobs,ztime,
     &         nymd, nhms)

!.....................................................................o
!  Program:    ZUVANL                                                 :
!  Date:       November 15, 1994                                      :
!  Computer:   Cray YMP C90                                           :
!  Language:   Cray cft 6.0 fortran                                   :
!                                                                     :
! Program ZUVANL is the driver for the upper height/wind analysis.    :
!                                                                     :
! Input parameters                                                    :
! ----------------                                                    :
! z_err, u_err, v_err                  Arrays containing forecast     :
!                                      std dev estimates.             :
!                                                                     :
! z,u,v,t,psfc                         Model first guess quantities.  :
!                                                                     :
! ze,zelat,zelon,nze       Generated 1000mb heights / parameters;     :
!                          ze contains heights, nze contains number of:
!                          data points of this type.                  :
!                                                                     :
! data                     Observation data in parallel vector format.:
!                                                                     :
! nobs                     Number of observations in data array.      :
!                                                                     :
! ztime                    Array containing offset of observation     :
!                          times.                                     :
!                                                                     :
!                                                                     :
! Output parameters                                                   :
! -----------------                                                   :
!                                                                     :
! aquan                    Analysis solution at gridpoint locations.  :
!                                                                     :
! z_err, u_err, v_err      Arrays containing analysis error estimate. :
!.....................................................................o
!     NASA/DAO OPTIMUM INTERPOLATION ANALYSIS  -  VERSION 2.0         :
!.....................................................................o

#include <obs.com>

      integer   nze, i, imjm, lev, nnq
      logical   first
      data      first/.true./
      save      first
      external  cov

      real      aquan(im,jnp,mlev+1,nq), z_err(im,jnp,mlev),
     &          u_err(im,jnp,mlev), v_err(im,jnp,mlev),
     &          data (max_obsspace), 
     &          ztime(max_observations), fill, psfc(im,jnp),
     &          t(im,jnp,mlev), u(im,jnp,mlev), v(im,jnp,mlev),
     &          z(im,jnp,mlev), ze(20000), zelat(20000),
     &          zelon(20000), z1000(im,jnp)
 
      character*60      message 
 
! Statistical arrays.

      real zquan_mean( mlev ), uquan_mean( mlev ), vquan_mean( mlev ),
     &   zinc_mean( mlev ), uinc_mean( mlev ), vinc_mean( mlev ),
     &   zquan_stdev( mlev ), uquan_stdev( mlev ), vquan_stdev( mlev ),
     &   zinc_stdev( mlev ), uinc_stdev( mlev ), vinc_stdev( mlev )
      integer zcount( mlev ), ucount( mlev ), vcount( mlev )
 
      common /scan/     maxobs, rscan, onestage, max_data
      common /pmh/      pmand(18)
 
 
      common /dataptrs/ ikx, ilat, ilon, ipres, itype, idelt, ifcer,
     &                  itime
 
      common /gridxx/   im,jnp,mlev
 
      common /sparam/ lsolve, ns, nr, nv, nh(13104), nq,
     &                kq(3), volcentre_x(13104),
     &                ngv, volcentre_y(13104), glatup(13104),
     &                glonup(13104)
 
      data fill /1.e10/
 
!  Initialize arrays.

      imjm = im*jnp
      write(6,'(//," ZUVANL: Starting upper air height/wind analysis"
     &     )')
      message(1:60) = ' '
      zcount = 0
      ucount = 0
      vcount = 0
      zquan_stdev = 0.
      uquan_stdev = 0.
      vquan_stdev = 0.
      zinc_stdev = 0.
      uinc_stdev = 0.
      vinc_stdev = 0.
      zquan_mean = 0.
      uquan_mean = 0.
      vquan_mean = 0.
      zinc_mean = 0.
      uinc_mean = 0.
      vinc_mean = 0.
      aquan = 0.
 
      if( nobs.eq.0 ) then
        message = ' ZUVANL: No observations present. '
        call flash ( 3,message,1 )                             
        return
      end if

      if ( first ) then     
        call obscorrs(pmand,mlev)
        first = .false.
      endif 
 
! Set indicies for ZUV analysis.
      kq(1) = 2
      kq(2) = 3
      kq(3) = 4

! Restore the height forecast errors to the DATA array.

      call vterpv ( data(ilat), data(ilon), data(ipres),
     &              data(ifcer), nobs, z_err )

! Search for upper air data; form covariance matrices, and solve.

      call solvedrv( 1, mlev, cov, aquan, data,
     &               nobs, ztime, z_err, u_err, v_err )
 
!  Add analysis increments and form analysis solution array, AQUAN.

      where ( z .lt. fill ) z = aquan(:,:,1:mlev,1) + z
      where ( u .lt. fill ) u = aquan(:,:,1:mlev,2) + u
      where ( v .lt. fill ) v = aquan(:,:,1:mlev,3) + v

      do lev = 1,mlev

       zquan_mean(lev) = ssum( im*jnp,z(1,1,lev),1 ) / float(im*jnp)
       zinc_mean(lev) = ssum( im*jnp,aquan(1,1,lev,1),1 ) /
     &                    float(im*jnp)
       uquan_mean(lev) = ssum( im*jnp,u(1,1,lev),1 ) / float(im*jnp)
       uinc_mean(lev) = ssum( im*jnp,aquan(1,1,lev,2),1 ) /
     &                    float(im*jnp)
       vquan_mean(lev) = ssum( im*jnp,v(1,1,lev),1 ) / float(im*jnp)
       vinc_mean(lev) = ssum( im*jnp,aquan(1,1,lev,3),1 ) /
     &                    float(im*jnp)

      end do
         

! Report the results.

      write(6,'(//," HEIGHT ANALYSIS REPORT ",2(2x,i8),//,14x,"Incr",
     &     "ements",12x,"Quantities",/,2x,"Level",3x,"Mean",7x,"Std",
     &     " Dev",3x,"Mean",9x,"Std Dev",t59,"Comments",/,2x,"-----",
     &     3x,18("-"),3x,20("-"),t59,"--------",/)') nymd, nhms

      do i = 1,mlev

        write(6,'(2x,f5.0,3x,f8.2,2x,f8.2,3x,f9.2,2x,f9.2)')
     &        pmand(i), zinc_mean(i), zinc_stdev(i), zquan_mean(i),
     &        zquan_stdev(i)

      end do

      write(6,'(//," U-WIND ANALYSIS REPORT ",2(2x,i8),//,14x,"Incr",
     &     "ements",12x,"Quantities",/,2x,"Level",3x,"Mean",7x,"Std",
     &     " Dev",3x,"Mean",9x,"Std Dev",t59,"Comments",/,2x,"-----",
     &     3x,18("-"),3x,20("-"),t59,"--------",/)') nymd, nhms

      do i = 1,mlev

        write(6,'(2x,f5.0,3x,f8.2,2x,f8.2,3x,f9.2,2x,f9.2)')
     &        pmand(i), uinc_mean(i), uinc_stdev(i), uquan_mean(i),
     &        uquan_stdev(i)

      end do

      write(6,'(//," V-WIND ANALYSIS REPORT ",2(2x,i8),//,14x,"Incr",
     &     "ements",12x,"Quantities",/,2x,"Level",3x,"Mean",7x,"Std",
     &     " Dev",3x,"Mean",9x,"Std Dev",t59,"Comments",/,2x,"-----",
     &     3x,18("-"),3x,20("-"),t59,"--------",/)') nymd, nhms

      do i = 1,mlev

        write(6,'(2x,f5.0,3x,f8.2,2x,f8.2,3x,f9.2,2x,f9.2)')
     &        pmand(i), vinc_mean(i), vinc_stdev(i), vquan_mean(i),
     &        vquan_stdev(i)

      end do
      end
