C...................................................................... 
C                                                                     . 
C  Program:    CHKMIX                                                 . 
C  Language:   FORTRAN V (ANSI-77 STANDARD)                           . 
C  Computer:   Cray YMP 4/832                                         . 
C                                                                     . 
C  Programmer: J. Pfaendtner                                          . 
C  Date:       July, 1990                                             . 
C                                                                     . 
C  Modifications:                                                     .
C                                                                     .
!     6/92  M.Seablom   RH and mixing ratio values checked for        .
!                       consistency by using emperical saturation     .
!                       vapour pressure formula.                      .
C...................................................................... 
                                                                        
      SUBROUTINE CHKMIX( LL, RHMOD, RMIXMOD, NLOBS,
     .                   pmand, im, jnp, mlev, mmix, sigdat, fg_t )
                                                                        
C  COMPUTE DELTAS AND DO QUALITY CONTROL FOR MOISTURE DATA              
                                                                        
                                                                        
      COMMON            CUTVAL(20000)                                   
      INTEGER           I                                               
      INTEGER           KK                                              
      INTEGER           LL                                              
      COMMON            LQU(20000)                                      
      real              LQU                                             
      INTEGER           NLOBS                                           
      INTEGER           NPASS                                           
      real              pmand(mlev)
      COMMON            PWT(20000)                                      
      DIMENSION         RHMOD(IM,JNP,MLEV)                              
      DIMENSION         RMIXMOD(IM,JNP,MLEV)                            
      real              fg_t( im,jnp,mlev ), obs_t( nlobs ),
     &                  sat_vap_prs( nlobs ), test_RH( nlobs )
      character*60      message
                                                                        
      COMMON /BUDBITS/  BGRS(20000), BACC(20000), BBUD(20000),          
     $                  BCHK(20000)                                     
      INTEGER           BGRS, BACC, BBUD, BCHK                          
                                                                        
      LOGICAL           SIGDAT
                                                                        
                                                                        
C  DATA AT A PRESSURE LEVEL.                                            
      COMMON /LEVEL/    INDX(20000), KX(20000), LAT(20000), LON(20000), 
     .                  PR(20000),   RH(20000), MIX(20000), DIFF(20000),
     .                  QDATA(20000)                                    
      DIMENSION         TYPE(20000)                                     
      EQUIVALENCE     ( RH(1), TYPE(1) )                                
      real              INDX, KX, LAT, LON, MIX                         
      INTEGER           QDATA                                           
                                                                        
C  PRINT FLAG                                                           
      COMMON /PRINT/    MSGLEV                                          
                                                                        
      message(1:60) = ' '

C  Use 50% as the cutoff value for all reports.
                                 
      DO 100 I = 1, NLOBS                                               
         KK = KX(I) + 0.5                                               
         CUTVAL(I) = 50.
  100 CONTINUE                                                          
                                                                        
c  Check for consistency in computation of mixing ratio. Interpolate
c  first guess temperature to observation locations. Compute saturation
c  vapour pressure and mixing ratio based upon pressure, temperature,
c  and relative humidity.

      call vterpv( lat, lon, pr, obs_t, nlobs,
     &             fg_t )

      do i = 1,nlobs

        if ( obs_t(i) .ge. 273.16 ) then
         sat_vap_prs(i) = 6.1078 * exp( 17.269*(obs_t(i)-273.16)/
     &                    (obs_t(i)-35.86) )
        else 
         sat_vap_prs(i) = 6.1078 * exp( 21.8747*(obs_t(i)-273.16)/
     &                    (obs_t(i)- 7.66) )
        end if

      end do

      do i = 1,nlobs

       sat_mixing_ratio = (.622*sat_vap_prs(i)/pr(i)-sat_vap_prs(i))
     &             * 1000.
       test_RH(i) = 100.*(mix(i) / sat_mixing_ratio )

      end do

        
C  INTERPOLATE FIRST GUESS TO OBS LOCATIONS AND COMPUTE DELTAS          
                                                                        
C  CASE 1:  NO SIGNIFICANT LEVEL DATA IS BEING USED                     
      IF( .NOT.SIGDAT ) THEN                                            
         CALL TERPV( LAT, LON, DIFF, NLOBS, RHMOD(1,1,LL) )             
         DO 110 I = 1, NLOBS                                            
            RH(I)   = RH(I) - DIFF(I)                                   
            test_RH(i) = test_RH(i) - diff(i)
  110    CONTINUE                                                       
         CALL TERPV( LAT, LON, DIFF, NLOBS, RMIXMOD(1,1,LL) )           
         DO 120 I = 1, NLOBS                                            
            MIX(I)  = MIX(I) - DIFF(I)                                  
  120    CONTINUE                                                       
                                                                        
C  CASE 2:  SIGNIFICANT LEVEL DATA IS BEING USED                        
      ELSE                                                              
         CALL PREFAC( LL, PR, NLOBS, LQU, PWT )
         CALL TERPV3( LL, NLOBS, LAT, LON, RH,  RHMOD,
     $                                          PWT, LQU, DIFF )        
         DO 130 I = 1, NLOBS                                            
            RH(I)  = DIFF(I)                                            
  130    CONTINUE                                                       
         CALL TERPV3( LL, NLOBS, LAT, LON, MIX, RMIXMOD,
     $                                          PWT, LQU, DIFF )        
         DO 140 I = 1, NLOBS                                            
            MIX(I) = DIFF(I)                                            
  140    CONTINUE                                                       
      ENDIF                                                             
                                                                        
c  Perform the gross check. 

      do i = 1,nlobs

        if ( (abs( rh(i) ) .le. cutval(i)) .and.
     &       (abs( test_RH(i) ) .le. cutval(i)) ) then

            bgrs(i) = 1

        else

            bgrs(i) = 0

        end if
      
        bacc(i) = bgrs(i) * qdata(i)

      end do
                                                                        
C  DETAIL PRINT OF DATA NOT PASSING THE GROSS CHECK                     
      IF( MSGLEV.GT.5 ) THEN                                            
        WRITE( 6, 900 )                                                 
        DO 160 I = 1, NLOBS                                             
          IF( BACC(I).EQ.0 )                                            
     $       WRITE( 6, 910 ) I, KX(I), LAT(I), LON(I), PR(I), RH(I)     
  160   CONTINUE                                                        
      ENDIF                                                             
                                                                        
C  PERFORM BUDDY CHECK OF MOISTURE DATA                                 
      CALL BUDCHK( KX, LAT, LON, RH, NLOBS )                            
                                                                        
C  MARK REPORTS OF ORIGINALLY SUSPICIOUS QUALITY THAT PASSED THE GROSS  
C  CHECK BUT FAILED THE BUDDY CHECK                                     
      DO 170 I = 1, NLOBS                                               
         BCHK(I) = (1-QDATA(I)) * BGRS(I) * (1-BCHK(I))                 
C  NEXT LINE DOES LOGICAL   BBUD = BBUD .OR. BCHK    WITH INTEGERS      
         BBUD(I) = BBUD(I) + BCHK(I) - BBUD(I) * BCHK(I)                
  170 CONTINUE                                                          
                                                                        
      IF( SIGDAT.AND.(MSGLEV.GT.2) ) THEN
         WRITE( 6, 920 ) PMAND(LL)
         CALL SIGRMS( KX, PR, MIX, MIX, 1, NLOBS )
      END IF 
                                                                        
      IF( MSGLEV.GT.2 ) THEN                                            
         WRITE( 6, 920 ) PMAND(LL)                                      
         CALL PRTRMS( KX, MIX, BGRS, BACC, QDATA, BBUD, BCHK, NLOBS )   
      ENDIF                                                             
                                                                        
C  SET 'TYPE' ARRAY AND REPORT THE NUMBER OF GOOD OBSERVATIONS.         
      NPASS = 0                                                         
      DO 180 I = 1, NLOBS                                               
         NPASS   = NPASS + BBUD(I)                                      
         TYPE(I) = BBUD(I)                                              
  180 CONTINUE                                                          

  
      write(6,'(1x," No. of observations passing full quality cont",
     &      "rol:",i8)') npass
                                                                        
      RETURN                                                            
  900 FORMAT('0CHKMIX: MOISTURE DATA WHICH DID NOT PASS GROSS CHECK'/)  
  910 FORMAT(1x,I8,4F10.2,' ',E14.7)                            
  920 FORMAT(25X,' Mixing ratio quality control for level',F7.2,     
     $                                                     'mb.')  
      END                                                               
