      SUBROUTINE CHOLESKI ( IGP, TOL, ISIZE, SYMAT, FORCE, WEIGHTS)
C.....................................................................o
C                                                                     :
C  Program name:  CHOLESKI (Cray solver version)                      :
C                                                                     :
C  Programmer: S. Bloom                                               :
C  Date:       July 14, 1995                                          :
C  Computer:   Cray YMP C90                                           :
C  Language:   Cray cft 6.0 fortran                                   :
C                                                                     :
C Program CHOLESKI is the matrix solver plug.  Symmetric matrix and   :
C forcing function input, analysis weights are output.  Choleski      :
C factorization method (here using Cray assembly routines) used to    :
C solve system directly.                                              :
C                                                                     :
C      INPUTS:       IGP           mini-volume number                 :
C                    ISIZE         matrix rank                        :
C                    SYMAT         symmetric stored covariance matrix :
C                    FORCE         rhs forcing covariances            :
C                    TOL           tolerance used for smoothness      :
C                                                                     :
C      OUTPUT        WEIGHTS       analysis weights                   :
C                                                                     :
C Modifications:                                                      :
C.....................................................................o
C     NASA/DAO OPTIMUM INTERPOLATION ANALYSIS  -  VERSION 2.0         :
C.....................................................................o

      PARAMETER( max_nh=600, maxrank=75 )

      INTEGER  IAUG

      LOGICAL  SOLVED

      REAL SYMAT(maxrank*(maxrank+1)/2),
     &     FORCE(maxrank,max_nh,3),WEIGHTS(maxrank,max_nh*3),
     &     AUG(3), SYMAT_TEMP(maxrank*(maxrank+1)/2)

      CHARACTER*60 MESSAGE
 
      DATA AUG /1.01, 1.1, 10./
 
      COMMON /SPARAM/ LSOLVE, NS, NR, NV, NH(13104), NQ,
     &     KQ(3), VOLCENTRE_X(13104),
     &     NGV, VOLCENTRE_Y(13104), GLATUP(13104),
     &     GLONUP(13104)

cmic$ do all shared (AUG, NH, NQ, NV, NR, TOL)
cmic$*       private ( SOLVED, WEIGHTS, FORCE, SYMAT )
cmic$*       private ( IAUG, IDIAG, ITRY, SYMAT_TEMP )
cmic$*       private ( INFO, MESSAGE ) AUTOSCOPE

      WEIGHTS(1:maxrank,1:max_nh*3) = 0.

      SOLVED = .FALSE.
      ITRY   = 0

      SYMAT_TEMP(1:maxrank*(maxrank+1)/2) =
     &     SYMAT(1:maxrank*(maxrank+1)/2)

!  Solve for the weights.

      DO WHILE ((.NOT. SOLVED).AND. (ISIZE .GT. 0) ) 

         INFO = 0

! Perform Choleski factorization.

         CALL SPPTRF ('u', ISIZE,SYMAT,INFO )

! Return code of info > 0 indicates non-positive definite matrix.

         IF ( INFO .GT. 0 ) THEN
            WRITE(6,'(" NPD matrix:",i6,"  Augment #",i2,2i8)')
     &           IGP,ITRY+1,INFO,ISIZE
            ITRY = ITRY + 1
            SYMAT(1:maxrank*(maxrank+1)/2) = 
     &           SYMAT_TEMP(1:maxrank*(maxrank+1)/2)
CFPP$ SKIP L
            DO 20 IAUG = 1,ISIZE
               IDIAG = IAUG*(IAUG+1)*.5
               SYMAT(IDIAG) = SYMAT(IDIAG)*AUG(ITRY)   
 20         CONTINUE

            IF ( ITRY .GT. 3 ) THEN
               MESSAGE = "SOLVEDRV: Excessive resolving"
               CALL FLASH(5,MESSAGE,1)
            END IF

            SOLVED = .FALSE.

         ELSE  ! the solver has identified a PD matrix...

C  Check for nearly singular matrices....

            SOLVED = .TRUE.
            
            DO ISYMC = 1,ISIZE
               IF ( ISYMC .EQ. 1 ) THEN
                  IF (SYMAT(1)**2 .LE. TOL*SYMAT_TEMP(1) ) THEN
                     SOLVED = .FALSE.
                  END IF
               ELSE
                  IDIAG = ISYMC*(ISYMC+1)*.5
                  IF (SYMAT(IDIAG)**2 .LE. TOL*SYMAT_TEMP(IDIAG) ) 
     &                 SOLVED = .FALSE.
               END IF
            END DO

            IF ( .NOT. SOLVED ) THEN

               WRITE(6,'(" ILL matrix:",i6,"  Augment #",i2,i8)')
     &              IGP,ITRY+1,ISIZE
               ITRY = ITRY + 1
               SYMAT(1:maxrank*(maxrank+1)/2) =
     &              SYMAT_TEMP(1:maxrank*(maxrank+1)/2)
CFPP$ SKIP L
               DO 60 IAUG = 1,ISIZE
                  IDIAG = IAUG*(IAUG+1)*.5
                  SYMAT(IDIAG) = SYMAT(IDIAG)*AUG(ITRY)
 60            CONTINUE
               IF ( ITRY .GT. 3 ) THEN
                  MESSAGE = "SOLVEDRV: EXCESSIVE RESOLVING."
                  CALL FLASH(5,MESSAGE,1)
               END IF

            END IF

         END IF
      END DO
C
C  Perform Choleski solution.
C
      ICOUNT = 1
      DO I = 1,NQ
         DO J = 1,NH(IGP)*NV
            DO K = 1,ISIZE
               WEIGHTS(K,ICOUNT) = FORCE(K,J,I)
            END DO
            ICOUNT = ICOUNT + 1
         END DO
      END DO

      CALL SPPTRS('u', ISIZE,NH(IGP)*NV*NQ,SYMAT,WEIGHTS,         
     &                 MAXRANK, INFO )         

      RETURN
      END
