                                                                        
C***************************************************************
C                                                                       
       SUBROUTINE  HYDRO ( Z, T, N, kl)

                                                                        
C*********************************   GODDARD LABORATORY FOR ATMOSPHERES 
C                                                                       
C                                                                       
C                                                                       
C  PURPOSE: GIVEN A TIME WINDOW, THIS SUBROUTINE  READS  UPPER AIR      
C         CONVENTIONAL DATA(LEVEL 2A) AND WRITE THEM TO THE ARRAY DATA  
C                                                                       
C      DATE :   02/05/92                                                
C                                                                       
C      INPUT:                                                           
C                                                                       
C         Z(N)    - heights                
C         T(N)    - temperature                  
C         N  -   number of levels in the report                         
C                                                                       
C                                                                       
C  OUTPUT:                                                              
C         Z(N)    - heights         !  if pass hydrostatic check        
C         T(N)    - temperature     !  miss- if not               
C         kl - number of levels,which do not pass hydrostatic check 
                                                                        
C  LOCAL VARIABLE:                                                      
                                                                        
C  PROGRAMMER:  YELENA KONDRATYEVA                                      
C                                                                       
C***********************************************************************
                                                                        
C ------------                                                          
      REAL            PMAND (20)                  
      
      REAL           cut ,g ,r                                 
      REAL           eps, mis                                 
c     REAL           misT                                
      REAL           Z ( N), T (N)                                 
      REAL           BOUNDHT (20,2)                             
                                                                        
      DATA  cut  /0.2/ g /9.81/, r /287.04/                             
      DATA  mis  /99999.0/ , eps / 1.E-10/                              
c     DATA  misT /999.0/                            
                                                                        
                                                                        
                                                                        
C    MANDATORY CONSTANT-PRESSURE LEVEL (1 - 20)                         
                                                                        
      DATA PMAND /1000.0, 850.0, 700.0, 500.0, 400.0, 300.0,            
     &               250.0, 200.0, 150.0, 100.0, 70.0, 50.0,            
     &               30.0, 20.0, 10.0, 7.0, 5.0, 3.0, 2.0, 1.0/         
                                                                        
C    LEFT BOUND FOR HEIGHT ON MANDATORY LEVEL (1 - 20)                  
                                                                        
       DATA (BOUNDHT(I,1),I=1,20)  /-2000.0, -1000.0, 0.0, 1000.0,      
     &       3000.0, 4000.0, 6000.0, 7000.0, 8000.0, 10000.0, 13000.0,  
     &       14000.0, 17000.0, 20000.0, 21000.0, 23000.0, 28000.0,      
     &       30000.0, 34000.0, 39000.0/                                 
                                                                        
C    RIGHT BOUND FOR HEIGHT ON MANDATORY LEVEL (1 - 20)                 
                                                                        
       DATA (BOUNDHT(I,2),I=1,20)  /3000.0, 5000.0, 6000.0, 8000.0,     
     &       10000.0, 13000.0, 15000.0, 16000.0, 18000.0, 21000.0,      
     &       23000.0, 25000.0, 29000.0, 34000.0, 37000.0, 40000.0,      
     &       42000.0, 46000.0, 49000.0, 55000.0/                        
                                                                        
                                                                        
                                                                        
                kl          =0 
           if ( N.eq. 1) return

c                  start loop by levels
c            if  two neibourgh levels have not "missing"
c            heihts and temperature, we check them for hydrostatic
	  
	  do  20   i = 1,N -1
		   L=i+1
            if  ( T(i).le.150.0 .or. T(i).ge. 320.0.or.
     .	   Z(i).lt.boundht(i,1).or.Z(i).ge.boundht(i,2).or.
     .            T(L).le.150.0 .or. T(L).ge. 320.0.or. Z(L).lt.
     .		   boundht(L,1).or.Z(L).ge.boundht(L,2)) go to 20  

                 T1 =T(i)
		    Z1 =Z(i)
		    AP1 = Alog(pmand(i))
                 T0 =T(L)
		    Z0 =Z(L)
		    AP0 = Alog(pmand(L))

c       hydrostatic balance:   must be    A=B ,where 
c       A =delta (log P),  B= -g/rT * delta Z.
c       Check  if  abs ( A/B -1) <=  cut   , (cut=0.2)

                  tbar = 0.5 * ( T1 + T0 )
		    thik = Z0 -Z1
		    A = AP1 - AP0 
		    B = g/ (r * tbar) * thik 

c       check 

        if (thik.le. eps.and. A .le. eps)  go to 20 
          if (thik .le. eps ) go to 30   
            deltaAB = abs ( A/B -1) 
		if (deltaAB .le.cut ) go to 20


c     Level do not pass hydrostatic  check:
c     we put "misT"in temrerature and "mis"in heights at this level.
c     If next level is the last one,we do the same for last level
  30     continue 
	  kl=kl+1 

c	    T (i) = misT 
	    Z (i) = mis 
	    if  ( L.eq.N ) then  
	      kl=kl+1 
c	        T (L) = misT
	        Z (L) = mis 
	    endif 
	    
  20     continue 
	    
	    return 
         end
