                                                                        
         SUBROUTINE  PRTSTAT( IDATE, INTIME, IWINDOW,TYP_DATA,          
     &                       KNT,NOBS)                                  
                                                                        
C*********************************   GODDARD LABORATORY FOR ATMOSPHERES 
C                                                                       
C                                                                       
C                                                                       
C  PURPOSE: GIVEN A TIME WINDOW,TYPEOF DATASET AND KOUNTES SUBROUTINE   
C         PRINTS STATISTICAL REPORT                                     
C                                                                       
C      DATE :   08/29/91                                                
C                                                                       
C      INPUT:                                                           
C                                                                       
C         IDATE    - CENTRAL DATE OF WINDOW (YYMMHH)                    
C         INTIME    - CENTRAL TIME OF WINDOW (HHMMSS)                   
C         IWINDOW  - SIZE OF WINDOW  ( HHMMSS)                          
C          NOBS     - NUMBER OF OBSERVATIONS ACTUALLY FOUND ( <=MAX_DATA
C         TYP_DATA  - NAME OF DATA SET (RAWINSONDE,AIRCRAFT, ...)       
C          KNT(30)  - ARRAY OF KOUNTERS                                 
C                                                                       
C                                                                       
C     KNT(I)  COUNT THE NUMBER OF LEVELS OR REPORTS                     
C                                                                       
                                                                        
C  KNT(1)=KOUNTLL  - COUNT REPORTS WITH "BAD" LATTITUDE OR LONGITUDE    
C  KNT(2)=KOUNTALT - COUNT LEVELS WITH "BAD" ALTITUDE                   
C  KNT(3)=KOUNTPRES- COUNT LEVELS WITH "BAD" PRESSURE ( CAT.            
C  KNT(4)=KOUNTW  - COUNT LEVELS WITH "BAD" WIND                        
C  KNT(5)= KOUNTBLEV- COUNT LEVELS WITH "BAD" HEIGHT AND WIND (BOTH)    
                                                                        
C  KNT(6)=KOUNT_PA- COUNT LEVELS WITH "BAD" PRESSURE OR PRESSURE        
C                   ALTITUDE (SATELLITE DATA)                           
C  KNT(7)=KOUNT_BCASE  - COUNT REPORTS FOR "BAD" CASE(UPPERAIR)         
C  KNT(8)=KOUNT_KX- COUNT REPORTS WITH "BAD" INDEX                      
C                                                                       
C  KNT(9)= KOUNT1_0 - COUNT REPORTS WITH NO DATA FOR CATEGORY 1         
C  KNT(10)=KOUNT1_21- COUNT REPORTS WITH NUMBER 0F LEVELS >20 FOR CAT.1 
C  KNT(11)= KOUNT6_0 - COUNT REPORTS WITH NO DATA FOR CATEGORY 6        
C  KNT(12)=KOUNT6_2- COUNT REPORTS WITH NUMBER 0F LEVELS >1 FOR CAT.6   
C  KNT(13)=KOUNT8_2- COUNT REPORTS WITH NUMBER 0F LEVELS >1 FOR CAT.8   
C  KNT(14)= KOUNT_HYDR- COUNT LEVELS DO NOT PASS HYDROSTATIC CHECK 
C                                                                       
C  KNT(20)=KOUNTREP - COUNT REPORTS                                     
C  KNT(21)=KOUNTH_SUSP  -  COUNT "SUSPICIOUS"                   "FOR    
C  KNT(24)=KOUNTW_SUSP  -  DATA(QFLAG =D OR L)                  "HEIGHT 
C  KNT(23)=KOUNTH_BQUAL -  COUNT "BAD QUALITY" DATA             "AND    
C  KNT(26)=KOUNTW_BQUAL-   (QFLAG = EITHER B,J,C,K,E,M,N,G,O)   "WIND   
C  KNT(22)=KOUNTH_FAIL -   COUNT MISSING OR FAILED              "       
C  KNT(25)=KOUNTW_FAILL -  LIMIT CONTROL                        "       
                                                                        
C  PROGRAMMER:  YELENA KONDRATYEVA                                      
C                                                                       
C                                                                       
C***********************************************************************
                                                                        
C ------------                                                          
      CHARACTER*60 MESSAGE                                              
      CHARACTER*60 MES (30)                                             
                                                                        
       CHARACTER*10  TYP_DATA                                           
       INTEGER         KNT (30)                                         
        MES(1)='   REPORTS DO NOT PASS LIMIT CHECK FOR LAT / LONG'      
        MES(2)='   LEVELS DO NOT PASS LIMIT CHECK FOR PRES.ALTITUDE'    
        MES(3)='   LEVELS DO NOT PASS LIMIT CHECK FOR PRESSURE '        
        MES(4)='   LEVELS DO NOT PASS LIMIT CHECK FOR WIND '            
        MES(5)='   LEVELS DO NOT PASS LIMIT CHECK FOR HEIGHT&WIND'      
        MES(6)='   LEVELS DO NOT PASS LIMIT CHECK FOR PRES.OR.ALTIT.'   
        MES(7)='   REPORTS DO NOT FIT APPROPRIATE CASES '               
        MES(8)='   REPORTS DO NOT FIND APPROPRIATE KX(INDEX)'           
        MES(9)='   REPORTS HAVE NO DATA(LEVELS) FOR CAT.1'              
        MES(10)='   REPORTS HAVE MORE THAH 20 LEVELS FOR CAT.1'         
        MES(11)='   REPORTS HAVE NO DATA(LEVELS) FOR CAT.6'             
        MES(12)='   REPORTS HAVE MORE THAH 1 LEVELS FOR CAT.6'          
        MES(13)='   REPORTS HAVE MORE THAH 1 LEVELS FOR CAT.8'          
        MES(14)='   LEVELS DO NOT PASS HYDROSTATIC CHECK '      
                                                                        
       MESSAGE (1:60) = ' '                                             
C      -----------------------------------                              
C      !   PRINT  RESULTS                !                              
C      -----------------------------------                              
                                                                        
 7000   PRINT *, ' '                                                    
        PRINT *,'         ',TYP_DATA,' DATA SUMMARY FOR : '             
        PRINT *,'        ------------------------------'                
        PRINT *,'                            DATE   - ',IDATE           
        PRINT *,'                            TIME   - ',INTIME          
        PRINT *,'                       TIME WINDOW - ', IWINDOW        
        PRINT *,'    '                                                  
        PRINT *,' NUMBER OF ',TYP_DATA,' REPORTS = ',KNT(20)            
        PRINT *,'     '                                                 
                                                                        
            DO I=1,15                                                   
                  IF (KNT(I).GT.0)   THEN                               
        PRINT *,KNT( I), MES(I)                                         
                  ENDIF                                                 
             ENDDO                                                      
         IF (NOBS.EQ.0) GO TO 333                                       
                                                                        
            KH_GOOD = NOBS -KNT(21)-KNT(22)-KNT(23)                     
            KW_GOOD = NOBS -KNT(24)-KNT(25)-KNT(26)                     
                                                                        
           WRITE (6,2022)                                               
           WRITE  (6,2100)  KH_GOOD,KW_GOOD                             
        WRITE  (6,2200)  KNT(21), KNT(24)                               
                                                                        
        WRITE (6,2300) KNT(22) ,KNT(25)                                 
                                                                        
        WRITE (6,2400) KNT(23) ,KNT(26)                                 
 333    PRINT *,' RETURNED NUMBER OF OBSERVATIONS (NOBS) =',NOBS        
                                                                        
2022    FORMAT (//,1X,75('-'),/41X,'HEIGHT ',8X,' WIND ',/,1X,75('-'),/)
2100    FORMAT (1X,'GOOD',36X,I6,8X,I6,/)                               
2200    FORMAT (1X,'SUSPICIOUS',30X,I6,8X,I6,/)                         
2300    FORMAT (1X,'MISSING OR FAILED LIMIT CHECK',11X,I6,8X,I6,/)      
2400    FORMAT (1X,'PASSED LIMIT CHECK BUT BAD QUALITY',6X,I6,8X,I6,    
     &    /,1X,75('-'),/)                                               
                                                                        
                                                                        
                                                                        
                                                                        
                                                                        
C       STOP                                                            
        RETURN                                                          
2000    FORMAT  (2X, 6(1X,F6.1))                                        
        END                                                             
