c.....................................................................c
!                                                                     .
!  Program:    PUVANL                                                 .
!  Language:   Fortran 77 (with Fortran 90 enhancements)              .
!  Computer:   Cray C98                                               .
!                                                                     .
!  Programmer: J. Pfaendtner, M. Seablom                              .
!  Date:       December, 1991                                         .
!                                                                     .
!  Program PUVANL is the driver for the surface analysis.             .
!                                                                     .
!  Modifications:                                                     .
!   11/1994 (Seablom)  Added NQ quantities to analysis error array.   .
!
!  Gregor von Laszewski, 12/95 Changed to compile on RS6000 using xlf
! 
!......................................................................
 
 
      subroutine puvanl( psl_err,u_err,v_err,phis,psl,usl,vsl,
     &                   tsfc, cov,    aquan,  t,     ze,
     &                   zelat,  zelon,  nze,    fgslp, data,
     &                   nobs, suspect, ztx )
 
 
      real  aquan( im*jnp, mlev+1, nq ), psl_err(im,jnp,mlev+1),
     &      fgslp( im,jnp ), phis( im*jnp ), u_err(im,jnp,mlev+1),
     &      psl( im*jnp ), t( im*jnp*mlev ), tsfc( im*jnp ),
     &      usl( im*jnp ), vsl( im*jnp ), ze(20000), zelat(20000),
     &      zelon(20000), data( 1500000     ), max_value, min_value,
     &      ztx(150000), v_err(im,jnp,mlev+1)
 
      logical
     &      onestage, seasat, usesas, dalflg, suspect( 150000,2 )
!gvl  gregor repack
 
      external
     &      cov
 
      character * 60
     &      message
 
 
      common /dataptrs/ ikx, ilat, ilon, ipres, itype, idelslp, ifcer,
     &                  itime
 
      COMMON /DALCTL/   SEASAT, USESAS, DALFLG
 
      common /scan/     maxobs, rscan, onestage, max_data
 
      COMMON /GRIDXX/   IM, JNP, MLEV
 
      common /sparam/ lsolve, ns, nr, nv, nh(13104), nq,
     &                kq(3), volcentre_x(13104),
     &                ngv, volcentre_y(13104), glatup(13104),
     &                glonup(13104)
 
      data fill /1.e10/
 
!  Initialize parameters.
 
      IMJM   = IM * JNP
      LEV    = MLEV + 1
      KQ(1)  = 2
      KQ(2)  = 3
      KQ(3)  = 4
 
      aquan( 1:imjm, lev, 1:3 ) = 0.0
      suspect( 1:150000, 1:2 ) = .false.
      message(1:60) = ' '
 
 
      CALL GETCONI( 'NYMD    ', NYMD )
      CALL GETCONI( 'NHMS    ', NHMS )
 
      write( 6,'(/,1x," Initiate sea level pressure analysis",
     &       " for ",i7,1x,i7)') nymd, nhms
 
 
!  Read sea level pressure and surface wind data.
 
      if ( nobs .eq. 0 ) then
         message = 'PUVANL: No data!'
         call flash( 2,message,1 )
         return
      end if
 
C  INTERPOLATE FORECAST ERROR TO DATA POINTS
 
      IFCER = IDELSLP + 3 * NOBS
      CALL TERPV( DATA(ILAT), DATA(ILON), DATA(IFCER), NOBS, 
     &            psl_err(1,1,mlev+1) )
      write(6,'(/,1x," PUVANL: Number of observations ",i10)') nobs
 
C  FORM COVARIANCE MATRICES AND SOLVE FOR ANALYSIS INCREMENTS
 
      call solvedrv ( lev, lev, cov, aquan, 
     &                data, nobs, ztx, psl_err, u_err, v_err )
 
C  ADD ANALYSIS INCREMENTS TO FIRST GUESS AND SET ANALYSIS ERRORS
 
      do i = 1,imjm
         if (psl(i).lt.fill) PSL(I) = AQUAN(I,LEV,1) + PSL(I)
         if (usl(i).lt.fill) USL(I) = AQUAN(I,LEV,2) + USL(I)
         if (vsl(i).lt.fill) VSL(I) = AQUAN(I,LEV,3) + VSL(I)
      end do
 
      max_value = -9.99e9
      min_value = 9.99e9
      do i = 1,imjm
        if ( psl(i) .gt. max_value .and. psl(i) .ne. fill ) then
           max_value = psl(i)
        end if
        if ( psl(i) .lt. min_value .and. psl(i) .ne. fill ) then
           min_value = psl(i)
        end if
      end do

      write(6,'(/," PUVANL: Max slp value: ",e13.4)') max_value
      write(6,'(/," PUVANL: Min slp value: ",e13.4)') min_value

      max_value = -9.99e9
      min_value = 9.99e9
      do i = 1,imjm
        if ( usl(i) .gt. max_value .and. usl(i) .ne. fill ) then
           max_value = usl(i)
        end if
        if ( usl(i) .lt. min_value .and. usl(i) .ne. fill ) then
           min_value = usl(i)
        end if
      end do
      write(6,'(/," PUVANL: Max usl value: ",e13.4)') max_value
      write(6,'(/," PUVANL: Min usl value: ",e13.4)') min_value

      max_value = -9.99e9
      min_value = 9.99e9
      do i = 1,imjm
        if ( vsl(i) .gt. max_value .and. vsl(i) .ne. fill ) then
           max_value = vsl(i)
        end if
        if ( vsl(i) .lt. min_value .and. vsl(i) .ne. fill ) then
           min_value = vsl(i)
        end if
      end do

      write(6,'(/," PUVANL: Max vsl value: ",e13.4)') max_value
      write(6,'(/," PUVANL: Min vsl value: ",e13.4)') min_value

      if ( seasat ) then
        CALL SLP2HGT(AQUAN,PSL,T,
     .             NOBS,DATA(IKX),DATA(ILAT),DATA(ILON),DATA(ITYPE),
     .             NZE, ZE, ZELAT, ZELON )
      end if
 
      RETURN
 
      END
