c********************************************************************** 
c                                                                     * 
c    Program Name: QCREPORT                                           * 
c    Programmer  : Michael Seablom, NASA/GSFC Code 910.3              * 
c                  Greenbelt, MD 20771  (301)286-7360 (FTS)888-7360   * 
c                                                                     * 
c    Date        : April 23, 1991                                     * 
c    Computer    : Cray YMP                                           * 
c    Language    : Cray Fortran (CFT77 Version 4.0)                   * 
c                                                                     * 
c  Program QCREPORT prints out upper air quality control statistics.  * 
c                                                                     * 
c  Modifications:                                                     *
c                                                                     *
c  Dec 1992 (Seablom) Corrected logic to allow report of 0.4mb level  *
c  Dec 1992 (Seablom) Modified code to print out off-level cases.     *
!
!  Gregor von Laszewski, 12/95 Changed to compile on RS6000 using xlf
!                                                                     *
c********************************************************************** 
                                                                        
                                                                        
      subroutine qcreport ( data, nobs, type, mlev, nymd, nhms )        
                                                                        
      real        data(1500000), testlevel
      integer     nobs, index(max_data), statistics(200,10), testval,   
     &            type, level_type, index2_count, index2(max_data)
      character*6 analysis_time, time_temp
      character*8 dataname, date_temp           
      character*10 analysis_date
      character*24 obsname
      logical     qobs, offlev, onestage, found, mandatory
                                                                        
      common /scan/ maxobs, rscan, onestage, max_data                   
      common /dataptrs/ ikx,ilat,ilon,iprs,ityp,idel,ifcer,itime
                                                                        
      common /obs/ maxdex, nfgge, lun, dataname(5), hh(13,13,20),       
     &             vcoroe(18,18,4), obserr(200,4,19), ww(13,13,20),     
     &             obsname(200), qobs(200,5), offlev(200)               
                                                                        
      common / pmh / pmand(18)                                          
                                                                        
      data statistics / 2000*0 /                                        
                                                                        
                                                                        

 1000 format(//,5x,"Observation Type",t28,"Eligible Reports",t48,
     &       "RMS-Delta",t60,"Mean Fcst Error",/)
                                                                        
!
!  Disable parallelization of this subroutine.
!
CFPP$ NOCONCUR
!gvl comma missing
      write(6,'("1",//,1x,80("-"),//,19x,"Quality Control, Version",
     &      " 2.1    April, 1991",//, 80("-"),//)')                     
                                                                        
      if ( nymd .gt. 500000 ) write (date_temp,'("19",i6)') nymd                                     
      if ( nymd .le. 500000 ) write (date_temp,'("20",i6)') nymd                                     
      analysis_date = date_temp(5:6) // '-' // date_temp(7:8) // '-'    
     &                // date_temp(1:4)
      write (time_temp,'(bz,i6)') nhms                                  
      analysis_time = time_temp(1:2) // ':' // time_temp(3:4) // 'Z'    
      if ( nhms .eq. 0 )  analysis_time = ' 0:00Z'                      
      iuwnd = idel + nobs                                               
      ivwnd = iuwnd + nobs                                              
      ife   = ivwnd + nobs                                              

                                                                        
! First loop: group observations by mandatory level using INDEX as point
! Second loop: find off level observations nearby.
                                                                        
      do 1 levels = 1,mlev                                              

        mandatory = .true.

        do 2 level_type = 1,2

        if ( mandatory ) then
        write(6,'(///)')                                                
        if ( type .eq. 1 ) write(6,'(/," Geopotential height statis",   
     &    "tics for ",f7.1,"mb. Tolerance variable.",/," Analysis",     
     &    " date: ",a10,3x,"Analysis time: ",bz,a6)')                    
     &    pmand( levels ), analysis_date, analysis_time                 
                                                                        
                                                                        
        if ( type .ne. 1 ) write(6,'(/," Wind speed statis",            
     &    "tics for ",f7.1,"mb. Tolerance variable.",/," Analysis",     
     &    " date: ",a10,3x,"Analysis time: ",bz,a6)')                    
     &    pmand( levels ), analysis_date, analysis_time                 
                                                                        
        write(6,1000)
        end if

                                                                        
        statistics( 1:200, 1:10 ) = 0                                   
                                                                        
        if ( mandatory ) then

         index1_count = 0

         do ix = 1,nobs

           if ( abs(data(iprs+ix-1)-pmand(levels)) .lt. 1.e-3 ) then

              index1_count = index1_count + 1
              index(index1_count) = ix

           end if

         end do

         nval = index1_count

        else

         index2(1:nobs) = 0
         index2_count = 0
         do ix = 1,nobs

            testlevel = log(data(iprs+ix-1))-log(pmand(levels))
            if ( abs(testlevel) .le. .1 ) then
               found = .false.
               do jx = 1,nobs
                  if ( index(jx) .eq. ix ) found = .true.
               end do
               if ( .not. found ) then
                  index2_count = index2_count + 1
                  index2(index2_count) = ix
               end if
            end if

         end do

         nval = index2_count
         index(1:nval) = index2(1:nval)

         if ( nval .gt. 0 ) then
        write(6,'(///)')                                                
        if ( type .eq. 1 ) write(6,'(/," Geopotential height statis",   
     &    "tics for nearby ",f7.1,"mb. Tolerance variable.",/,          
     &    " Analysis date: ",a10,3x,"Analysis time: ",bz,a6,
     &    "  ( off-level or significant level data )"
     &    )')
     &    pmand( levels ), analysis_date, analysis_time                 
                                                                        
                                                                        
        if ( type .ne. 1 ) write(6,'(/," Wind speed statis",            
     &    "tics for nearby ",f7.1,"mb. Tolerance variable.",/,
     &    " date: ",a10,3x,"Analysis time: ",bz,a6,
     &    "  ( off-level or significant level data )"
     &    )')
     &    pmand( levels ), analysis_date, analysis_time                 
                                                                        
        write(6,1000)

        end if
        end if

c Tally the total number of observations for each data type.            
                                                                        
        do 3 j = 1,200                                                  
          do  i = 1,nval                                                
                                                                        
            if ( data(ikx+index(i)-1) .eq. float(j) )                   
     &        statistics(j,1) = statistics(j,1) + 1                     
                                                                        
          end do                                                        
                                                                        
c Total the number of accepted reports.                                 
                                                                        
        do i = 1,nval                                                   
                                                                        
          if ((int(data(ityp+index(i)-1)) .eq. type .or.                
     &         int(data(ityp+index(i)-1)) .eq. 3) .and.                 
     &         data(ikx+index(i)-1) .eq. float(j) )                     
     &                  statistics(j,2) = statistics(j,2) + 1           
                                                                        
        end do                                                          
                                                                        
c Number of suspect observations. If type = 1, height observations      
c are being tallied, otherwise tally wind observations.                 
                                                                        
        if ( type .eq. 1 ) then                                         
                                                                        
c Suspicious height observations.                                       
                                                                        
          do i = 1,nval                                                 
           if ( data(ikx+index(i)-1) .eq. float(j) ) then               
            testval = int( data( ityp+index(i)-1 ) * 10. )              
                                                                        
            if ( testval .eq. 13 .or. testval .eq. 3 )                  
     &        statistics(j,3) = statistics(j,3) + 1                     
                                                                        
            if ( testval .eq. 13 )                                      
     &        statistics(j,4) = statistics(j,4) + 1                     
                                                                        
c Gross check failures.                                                 
                                                                        
            if ( testval .eq. 12 .or. testval .eq. 1 )                  
     &        statistics(j,5) = statistics(j,5) + 1                     
                                                                        
c Number reaccepted by buddy check.                                     
                                                                        
            if ( testval .eq. 12 ) statistics(j,6) = statistics(j,6) + 1
           end if                                                       
          end do                                                        
                                                                        
       else                                                             
                                                                        
c Suspicious wind observations.                                         
                                                                        
          do i = 1,nval                                                 
           if ( data(ikx+index(i)-1) .eq. float(j) ) then               
            testval = int( data( ityp+index(i)-1) * 100. ) -            
     &                int( data( ityp+index(i)-1 ) * 10.)* 10           
                                                                        
            if ( testval .eq. 3 ) then                                  
              statistics(j,3) = statistics(j,3) + 1                     
                                                                        
              if ( int(data(ityp+index(i)-1)) .eq. 2 .or.               
     &           int(data(ityp+index(i)-1)) .eq. 3 )                    
     &           statistics(j,4) = statistics(j,4) + 1                  
                                                                        
            end if                                                      
                                                                        
c Gross check failures.                                                 
                                                                        
            if ( testval .eq. 2 .or. testval .eq. 1 )                   
     &        statistics(j,5) = statistics(j,5) + 1                     
                                                                        
c Number reaccepted by buddy check.                                     
                                                                        
            if ( testval .eq. 2 )                                       
     &         statistics(j,6)= statistics(j,6) + 1                     
           end if                                                       
          end do                                                        
                                                                        
       end if                                                           
                                                                        
                                                                        
c Average forecast errors and deltas for observations which passed and  
c which failed the quality control.
                                                                        
       fe_accept = 0.                                                   
       fe_fail   = 0.                                                   
       delta_accept = 0.                                                
       delta_fail   = 0.                                                
       number_accept = 0                                                
       number_fail   = 0                                                
                                                                        
       if ( type .eq. 1 ) then                                          
                                                                        
         do i = 1,nval                                                  
          if ( data(ikx+index(i)-1) .eq. float(j) ) then                
           itype = int( data( ityp+index(i)-1 ) )                       
                                                                        
           if ( itype .eq. 1 .or. itype .eq. 3 ) then                   
                                                                        
              number_accept = number_accept + 1                         
              fe_accept = fe_accept + data( ife+index(i)-1 )            
              delta_accept = delta_accept + (data( idel+index(i)-1))**2 
                                                                        
           else if ( int( data(ityp+index(i)-1)*10.) .ne. 4 ) then      
                                                                        
              number_fail = number_fail + 1                             
              fe_fail = fe_fail + data( ife+index(i)-1 )                
              delta_fail = delta_fail + (data( idel+index(i)-1))**2     
                                                                        
           end if                                                       
          end if                                                        
         end do                                                         
                                                                        
       else                                                             
                                                                        
         do i = 1,nval                                                  
          if ( data(ikx+index(i)-1) .eq. float(j) ) then                
           itype = int( data( ityp+index(i)-1 ) )                       
                                                                        
           if ( itype .eq. 2 .or. itype .eq. 3 ) then                   
                                                                        
              number_accept = number_accept + 1                         
              fe_accept = fe_accept + data( ife+index(i)-1 )            
              delta_accept = delta_accept +                             
     &         (sqrt(data(iuwnd+index(i)-1)**2  +                       
     &               data( ivwnd+index(i)-1 ) **2 ))**2                 
                                                                        
           else                                                         
            testval = int( data( ityp+index(i)-1) * 100. ) -            
     &                int( data( ityp+index(i)-1 ) * 10.)* 10.          
            if ( testval .ne. 4 ) then                                  
                                                                        
              number_fail = number_fail + 1                             
              fe_fail = fe_fail + data( ife+index(i)-1 )                
              delta_fail = delta_fail +                                 
     &                      (sqrt( data(iuwnd+index(i)-1) **2 +         
     &                       data( ivwnd+index(i)-1 ) ** 2 ))**2        
                                                                        
            end if                                                      
           end if                                                       
          end if                                                        
         end do                                                         
                                                                        
         end if                                                         
                                                                        
         if ( number_fail .ne. 0 ) then                                 
           fe_fail = fe_fail / float( number_fail )                     
           delta_fail = sqrt(delta_fail / float( number_fail ))         
         end if                                                         
         if ( number_accept .ne. 0 ) then                               
           fe_accept = fe_accept / float( number_accept )               
           delta_accept = sqrt(delta_accept / float( number_accept ))   
         end if                                                         
         if ( statistics(j,1) .ne. 0 ) write(6,'(1x,a24,4x,i5,"[",i4,
     &      "%]",t48,e9.3,t61,e9.3)')
     &      obsname(j), statistics(j,1),nint(float(statistics(j,2))/
     &      float(statistics(j,1))*100), delta_accept, fe_accept
                                                                        
 3      end do                                                          
                                                                        
       mandatory = .false.
 2     end do
 1    end do                                                            
!
CFPP $ CONCUR
      return                                                            
      end                                                               
                                                                        
