      subroutine rdhuv (zmod, tmod, z1000, data, nobs, suspect,
     &                  ze, zelat, zelon, nze )
 
c***********************************************************************
c
c  Program Name: RDHUV
c  Programmer:   M. Nestler ( modifications by M. Seablom )
c  Date:         May 31, 1994 
c  Computer:     Cray C90
c
c  Program RDHUV reads upper air observational data.
c
c  Modifications:
c
c   - 03/1991    DATA array expanded to allow >100K observations.
c   - 03/1991    Satellites used over land.
c   - 04/1991    Add date/time stamp to DATA array.
c   - 05/1991    Modifications to incorporate data above 50mb.
c   - 08/1992    Modifications to allow data above 1mb.
!
!  Gregor von Laszewski, 12/95 Changed to compile on RS6000 using xlf
! 
!-----------------------------------------------------------------------
!    NASA / DAO  OPTIMUM  INTERPOLATION  VERSION  1.3  SUMMER, 1992
!-----------------------------------------------------------------------
c
c Upper air data is stored in the DATA array, initial data quality
c information is contained in array SUSPECT.
c Include model fields to compute satellite heights.
c
c Special version to incorporate MTP (ER2 aircraft) and UARS temperatures.
c 


      real    data(1500000), zmod(1,1,1), tmod(1,1,1), z1000(1,1),
     &        ze(20000),zelat(20000),zelon(20000), ibmbuf, zwtim,
     &        zwkx, zwlat, zwlon, pzw, z, wu, wv, pzr, tw, qz, qw,
     &        zwdum, sat_wind(150000), ref_pressure(150000),
     &        mtp_lat(6000), mtp_lon(6000), mtp_pressure(6000),
     &        mtp_temp(6000), uars_lat(10000), uars_lon(10000),
     &        uars_pressure(10000), uars_temp(10000)
 
      logical suspect(150000,2), next, end_of_file, file_exists
 
      integer        datasets, uars_time
!gvl  integer end_address  ! never used

      COMMON /LEVEL/ IBMBUF(85,12), ZWTIM(85), ZWKX(85),
     .               ZWLAT(85), ZWLON(85), PZW(85),
     .               Z(85), WU(85), WV(85), PZR(85), TW(85),
     .               QZ(85),QW(85), ZWDUM(4),
     .               PAD4(157956),BPAD(20000)
C
C OBSERVATIONS INFORMATION
C ----------------------------------------------------------------------
      real HH,VCOROE,OBSERR,WW
      character*24 obsname
      character* 8 dataname , cnymdh
      character*6  cnymd, cnhms
      LOGICAL QOBS, OFFLEV
      COMMON /OBS/ MAXDEX,NFGGE,LUN,DATANAME(5),HH(13,13,20),
     .              VCOROE(18,18,4),OBSERR(200,4,19),WW(13,13,20),
     .              OBSNAME(200),QOBS(200,5),OFFLEV(200)
      common /scan/ maxobs, rscan, onestage, max_data
 
! Set vertical levels.
 
       common /mlay/ mlev, mlevm1, mmix, mmixm1, ls, lsm1
       common/ pmh / pmand(18)
C
C ANALYSIS CONTROL PARAMETERS
C ----------------------------------------------------------------------
      LOGICAL STRAT,SLP,FDCOS,FEXP,SAVEAE,SIGDAT
      COMMON /CONTRL/ IM, JNP, NB, NDALT, NHMS, NLAY, NLAYM1, NLAYP1,
     .                 NYMD, STRAT, IDIMEN, SLP, FDCOS, FEXP, SAVEAE,
     .                 SIGDAT,MAXREP
C
C OBSERVATIONAL DATA INFO.
C ----------------------------------------------------------------------
      COMMON /RECORD/  INDAT, INTIM, INCTIM, NFOR(5), NBUF(5)
C
C DATA POINTERS
C ----------------------------------------------------------------------
      COMMON /DATAPTRS/ IKX,ILAT,ILON,IPZW,ITYPE,IZ,IFCER,itime
C
C CLOUD TRACK WIND REASSIGNMENT FLAG.
C ----------------------------------------------------------------------
      COMMON /SATWIND/ CWIND
      LOGICAL CWIND
C
C LOCAL VARIABLES.
C ----------------------------------------------------------------------
       character*60 message
       real  FORM, BUFF, XFORM, XBUFF, FILL

 
      real  rkx(12000), rlat(12000), rlon(12000), retv(12000,15), ! <v1.
     &      rpref(12000)                                          !

!gvl  no data declaration before not all variables declared

      data fill /1.e15/

      message(1:60) = ' '
      suspect(1:150000,1:2) = .false.
 
 
! Define highest pressure level for upper air analysis.
 
      top_level = pmand(1)
 
! Starting address of data array.
 
      IKX = 1
      ILAT  = 150000   + 1
      ILON  = 2*150000   + 1
      IPZW  = 3*150000   + 1
      ITYPE = 4*150000   + 1
      IZ    = 5*150000   + 1
      IWU   = 6*150000   + 1
      IWV   = 7*150000   + 1
      itime = 9*150000   + 1
 
! Double check on data positioned to right date and time.
 
      IF (INDAT.NE.NYMD .OR. INTIM.NE.NHMS) THEN
         WRITE (6,1000)NYMD,NHMS,INDAT,INTIM
         message = ' RDHUV: Dataset is positioned to wrong date/time.'
         call flash( 5,message,1 )
      ENDIF
1000  FORMAT(' FATAL: RDHUV>> Observation dataset positioned to  ',
     . 'the wrong date/time. '
     .     ,/' ',5X,'Expected to perform analysis at :', 2I8
     .     ,/' ',5X,'Data positioned at date / time=',2I8)
 
      NOBS = 0
 
! Read file(s) of upper air height / wind data.
 
      DO 15 NF=1,NFGGE
 
        LIN = LUN + NF - 1
        NFORM = NFOR(NF)
        IF(NFORM.NE.4) then
          print*,' FATAL: RDHUV>> Dataset positioned to wrong form, ',nf
          message = ' RDHUV: Dataset positioned to wrong form. '
          call flash( 5,message,1 )
        end if
        NBUFF = NBUF(NF)
 
5       CONTINUE
        IF (NBUFF .EQ. 0) GO TO  60
 
        DO 10 J = 1, NBUFF
 
C READ AND CONVERT A BUFFER OF REPORTS
C ------------------------------------
 
          READ  (LIN,END=999) IBMBUF
 
          zwtim(1:85) = ibmbuf(1:85,1)
          zwkx(1:85) = ibmbuf(1:85,2)
          zwlat(1:85) = ibmbuf(1:85,3)
          zwlon(1:85) = ibmbuf(1:85,4)
          pzw(1:85) = ibmbuf(1:85,5)
          z(1:85) = ibmbuf(1:85,6)
          wu(1:85) = ibmbuf(1:85,7)
          wv(1:85) = ibmbuf(1:85,8)
          pzr(1:85) = ibmbuf(1:85,9)
          tw(1:85) = ibmbuf(1:85,10)
          qz(1:85) = ibmbuf(1:85,11)
          qw(1:85) = ibmbuf(1:85,12)
 
C STORE ONLY REPORTS THAT PASSED PROXIMITY CHECK(Q<2), HAVE DATA TYPES
C MARKED FOR THE ANALYSIS, AND REPORTED AT PRESSURE LEVEL AT/OR
C BELOW HIGHEST ANALYSIS LEVEL.
C ---------------------------------------------------------------------
 
          DO 20 I = 1,85
 
C
C PART OF THE BUFFER IS FILLED.
            IF (ZWKX(I) .GT. 10.e6) GO TO 20
            KK=ZWKX(I)+.1
 
            IF (QOBS(KK,NF).AND.PZW(I).GE. .99*top_level .and.
     .                     (QZ(I).LT.1.99 .OR.QW(I).LT.1.99) )
     .                                                           THEN
              NOBS = NOBS + 1
              IF(NOBS.GT.max_data) then
                message = ' RDHUV: Excessive number of observations.  '
                call flash( 5,message,1 )
              end if
              DATA( IKX  +  NOBS-1) =   ZWKX(I)
              DATA(ILAT  +  NOBS-1) =  ZWLAT(I)
              DATA(ILON  +  NOBS-1) =  ZWLON(I)
              DATA(IPZW  +  NOBS-1) =    PZW(I)
              DATA(IZ    +  NOBS-1) =      Z(I)
              DATA(IWU   +  NOBS-1) =     WU(I)
              DATA(IWV   +  NOBS-1) =     WV(I)
              data(itime +  nobs-1) =  zwtim(i)
              ref_pressure( nobs ) =     pzr(i)
              sat_wind( nobs ) =          tw(i)
 
!  For suspicious data set SUSPECT to true.
 
              if ( qz(i) .gt. .5 ) then
                 suspect( nobs,1 ) = .true.
              else
                 suspect( nobs,1 ) = .false.
              end if
 
              if ( qw(i) .gt. .5 ) then
                 suspect( nobs,2 ) = .true.
              else
                 suspect( nobs,2 ) = .false.
              end if
 
!  Fill data that did not pass initial quality control.
 
              IF( QZ(I) .GT. 1.5) DATA(IZ+NOBS-1) = FILL
              IF( QW(I) .GT. 1.5) THEN
                  DATA(IWU+NOBS-1) = FILL
                  DATA(IWV+NOBS-1) = FILL
              ENDIF
 
            ENDIF
 
20        CONTINUE
10      CONTINUE
C
C  READ NEXT SUBHEADER
C  -------------------
60       READ (LIN,END=999) FORM, BUFF
         xform = form
         xbuff = buff
         NFORM = int ( xform )
         NFOR(NF) = NFORM
         NBUFF = int ( xbuff )
         NBUF(NF) = NBUFF
         IF (NFORM .EQ. 4) GO TO 5
         write(6,'(/," RDHUV: Successfully read ",a8,".")')dataname(nf)
15    CONTINUE
 
 
!  Add satellite retrievals to the data array.
 
      NRET = 0
      NOBRET = 0
      next = .true.
 
      i = 71
      do while ( (next) .and. i .lt. 87 )
 
         if ( qobs(i,1) ) then
           call retsub ( nret, nretymd, nrethms, rkx, rlat, rlon,
     &                 retv, rpref )
           next = .false.
         else
           i = i + 1
         end if
 
      end do
 
      do i = 1,nret
 
         kk = nint( rkx(i) )
 
         if ( rkx(i) .le. 10.e6  .and.  qobs( kk,1 ) ) then
 
          do l = 1,mlev
 
           if ( retv( i,(mlev-l+1) ) .le. 10.e6 ) then
 
                   nobs = nobs + 1
                   nobret = nobret + 1
                   if ( nobs .gt. max_data ) then
                    message = ' RDHUV: Excessive number of GLA '//
     &               'retrievals.'
                    call flash( 5,message,1 )
                   end if
 
                   DATA(IKX   +  NOBS-1) =  RKX(I)
                   DATA(ILAT  +  NOBS-1) =  RLAT(I)
                   DATA(ILON  +  NOBS-1) =  RLON(I)
                   DATA(IPZW  +  NOBS-1) =  pmand (l)
                   DATA(IZ    +  NOBS-1) =  retv( i,(mlev-l+1) )
                   DATA(IWU   +  NOBS-1) =  FILL
                   DATA(IWV   +  NOBS-1) =  FILL
                   ref_pressure( nobs )  =  rpref(i)
                   sat_wind( nobs )      =  fill
 
           end if
          end do
         end if
      end do
 
      write (6,'(1x,i5," retrievals were produced.",/,1x,i5,
     &       " retrievals will be used in the analysis.",/)')nret,nobret
 
 
c  Include 1000mb heights generated from sea level pressure reports
c  over oceanic regions.
 
      DO 30 I = 1,NZE
 
         NOBS = NOBS + 1
         IF ( NOBS .GT. max_data ) THEN
            WRITE(6,'(///)')
            message = ' RDHUV: Excessive number of GEN1000 heights. '
            call flash( 5,message,1 )
         END IF
 
         DATA ( IKX  + NOBS - 1 )  =  87.
         DATA ( ILAT + NOBS - 1 )  =  ZELAT ( I )
         DATA ( ILON + NOBS - 1 )  =  ZELON ( I )
         DATA ( IPZW + NOBS - 1 )  =  1000.
         DATA ( IZ   + NOBS - 1 )  =  ZE ( I )
         DATA ( IWU  + NOBS - 1 )  =  FILL
         DATA ( IWV  + NOBS - 1 )  =  FILL
         ref_pressure( nobs )      =  FILL
         sat_wind( nobs )          =  FILL
 
 30   CONTINUE
 
      WRITE (6,'(" RDHUV:",i6, " 1000mb heights obtained from ",
     &            "sea-level pressure reports.")') nze
 
!  Add MTP data (if available).

      if ( qobs(88,1) ) then

        end_of_file = .false.
        imtp = 1

        write(cnymd,'(i6)') nymd
        if ( nhms .eq. 0 ) cnhms = '000000'
        if ( nhms .eq. 60000 ) cnhms = '060000'
        if ( nhms .eq. 120000 ) cnhms = '120000'
        if ( nhms .eq. 180000 ) cnhms = '180000'

        cnymdh = cnymd // cnhms(1:2)
        inquire(file='mtp.t'//cnymdh,exist=file_exists)

        if ( file_exists ) then
         open (84,iostat=ierr,form='formatted',file=
     &       'mtp.t'//cnymdh)

         do while ( .not. end_of_file )

          read(84,*,iostat=ierr) mtp_lat(imtp), mtp_lon(imtp),
     &                           mtp_pressure(imtp),mtp_temp(imtp)
          if ( ierr .ne. 0 ) then
            end_of_file = .true.
          else
            imtp = imtp + 1
          end if

         end do

         call mtpthick( mtp_lat, mtp_lon, mtp_pressure, mtp_temp,
     &                 imtp, zmod, tmod )

         do i = 1,imtp
           if ( mtp_lat(i) .ge. -90. .and. mtp_lat(i) .le. 90..and.
     &          mtp_lon(i) .ge. -180. .and. mtp_lon(i).le.180..and.
     &          mtp_pressure(i) .gt. 5..and. mtp_pressure(i)
     &          .lt. 400. ) then
           nobs = nobs + 1
           data( ikx + nobs - 1 )   = 88.
           data( ilat+ nobs - 1 )   = mtp_lat(i)
           data( ilon+ nobs - 1 )   = mtp_lon(i)
           data( ipzw+ nobs - 1 )   = mtp_pressure(i)
           data( iz  + nobs - 1 )   = mtp_temp(i)
           data( iwu + nobs - 1 )   = fill
           data( iwv + nobs - 1 )   = fill
           data(itime+ nobs - 1 )   = 0.
           ref_pressure(nobs)       = fill
           sat_wind    (nobs)       = fill
         end if
 
         end do
        end if
      end if

!  Add in the UARS retrievals (if available).

      if ( qobs(91,1) ) then

       end_of_file = .false.
       iuars = 1
       write(cnymd,'(i6)') nymd
       inquire(file='mls_oiinput.'//cnymd,exist=file_exists)
       
       if ( file_exists ) then
         open(85,iostat=ierr,form='formatted',file=
     &        'mls_oiinput.'//cnymd)

         do while ( .not. end_of_file )

          read(85,*,iostat=ierr) uars_time, uars_lat(iuars),
     &         uars_lon(iuars), uars_pressure(iuars),
     &         uars_temp(iuars)
             
          if ( ierr .ne. 0 ) then
            end_of_file = .true.
          else 
            utime = float( uars_time/1000 ) / 3600.
            if ( abs(utime-float(nhms/10000) ) .le. 3. ) then
                iuars = iuars + 1
            end if
          end if

         end do

         call mtpthick( uars_lat, uars_lon, uars_pressure,
     &                  uars_temp, iuars, zmod, tmod )

         do i = 1,iuars

         if (uars_lat (i) .ge. -90. .and.uars_lat(i) .le. 90..and.
     &         uars_lon(i) .ge. -180. .and.uars_lon(i).le.180..and.
     &         uars_pressure(i) .gt. 5..and.uars_pressure(i)
     &          .lt. 400. ) then
           nobs = nobs + 1
           data( ikx + nobs - 1 )   = 91.
           data( ilat+ nobs - 1 )   = uars_lat(i)
           data( ilon+ nobs - 1 )   = uars_lon(i)
           data( ipzw+ nobs - 1 )   = uars_pressure(i)
           data( iz  + nobs - 1 )   = uars_temp(i)
           data( iwu + nobs - 1 )   = fill
           data( iwv + nobs - 1 )   = fill
           data(itime+ nobs - 1 )   = 0.
           ref_pressure(nobs)       = fill
           sat_wind    (nobs)       = fill
         end if

         end do 
        end if

!  Case where data from 21Z on the previous day are needed.

        if ( nhms .eq. 0 ) then

         close(85)
         nymdm1 = nymd - 1
         call defdate ( nymdm1 )
         write(cnymd,'(i6)') nymdm1
         inquire(file='mls_oiinput.'//cnymd,exist=file_exists)

         if ( file_exists ) then
          open(85,iostat=ierr,form='formatted',file=
     &         'mls_oiinput.'//cnymd)

         do while ( .not. end_of_file )

          read(85,*,iostat=ierr) uars_time, uars_lat(iuars),
     &         uars_lon(iuars), uars_pressure(iuars),
     &         uars_temp(iuars)

          if ( ierr .ne. 0 ) then
            end_of_file = .true.
          else
            utime = float( uars_time/1000 ) / 3600.
            if ( utime .ge. 21. ) then
                iuars = iuars + 1
            end if
          end if

         end do

         call mtpthick( uars_lat, uars_lon, uars_pressure,
     &                  uars_temp, iuars, zmod, tmod )

         do i = 1,iuars
 
         if (uars_lat (i) .ge. -90. .and.uars_lat(i) .le. 90..and.
     &         uars_lon(i) .ge. -180. .and.uars_lon(i).le.180..and.
     &         uars_pressure(i) .gt. 5..and.uars_pressure(i)
     &          .lt. 400. ) then
           nobs = nobs + 1
           data( ikx + nobs - 1 )   = 91.
           data( ilat+ nobs - 1 )   = uars_lat(i)
           data( ilon+ nobs - 1 )   = uars_lon(i)
           data( ipzw+ nobs - 1 )   = uars_pressure(i)
           data( iz  + nobs - 1 )   = uars_temp(i)
           data( iwu + nobs - 1 )   = fill
           data( iwv + nobs - 1 )   = fill
           data(itime+ nobs - 1 )   = 0.
           ref_pressure(nobs)       = fill
           sat_wind    (nobs)       = fill
         end if
 
         end do
        end if


      end if
      end if

         
      if ( nobs .eq. 0 ) return
 
! Add in the reference heights.
 
      call refhgt ( data, nobs, ikx, ilat, ilon, ipzw, itype, iz,
     &              iwu, iwv, itime, ref_pressure, max_data )
 
 
! Compress the DATA array.
 
      do i = 1,nobs
          data( i+nobs ) =data( ilat+i-1 )
      end do
 
      do i = 1,nobs
          data( i+2*nobs ) = data( ilon+i-1 )
      end do
 
      do i = 1,nobs
          data( i+3*nobs ) = data( ipzw+i-1 )
      end do
 
      do i = 1,nobs
          data( i+4*nobs ) = data( itype+i-1 )
      end do
 
      do i = 1,nobs
          data( i+5*nobs ) = data( iz  +i-1 )
      end do
 
      do i = 1,nobs
          data( i+6*nobs ) = data( iwu  +i-1 )
      end do
 
      do i = 1,nobs
          data( i+7*nobs ) = data( iwv +i-1 )
      end do
 
      do i = 1,nobs
          data( i+9*nobs ) = data( itime +i-1 )
      end do
 
 
 
C
C CHANGE DATA POINTERS.
C---------------------------------------------------------------------
      ILAT  =   NOBS + 1
      ILON  = 2*NOBS + 1
      IPZW  = 3*NOBS + 1
      ITYPE = 4*NOBS + 1
      IZ    = 5*NOBS + 1
      IWU   = 6*NOBS + 1
      IWV   = 7*NOBS + 1
      ifcer = 8*nobs + 1
      itime = 9*nobs + 1
 
! Alter the satellite observations.
 
      next = .true.
      do datasets = 1,nfgge
 
        i = 31
 
        do while ( (next) .and. i .le. 86 )
 
          if ( qobs(i,datasets) ) then
            call sathgt ( data(ikx), data(ilat), data(ilon), data(iz),
     &                  ref_pressure, nobs, zmod, z1000 )
            next = .false.
          else
            i = i + 1
          end if
 
         end do
 
      end do
 
!gvl      if ( cwind )  call satwnd ( data(ikx), data(ilat), data(ilon),
!gvl     &                            data(ipzw), data(iwu), data(iwv),
!gvl     &                            sat_wind, nobs, tmod, zmod )


      RETURN
 
C  ERROR EXIT
C  ----------
 
999   WRITE (6,1010)LIN,NYMD,NHMS
1010  FORMAT(' ****** ERROR IN RDZUV: END OF FILE ON UNIT ',I5,
     .      /' THE OBSERVATION DATA FOR DATE/TIME=',2I10,' IS ',
     .'MISSING OR INCOMPLETE ')
      STOP 'RDZUV 2'
      END
