                                                                        
c------------------------------------------------------------------c
c                                                                  c
c  Program Name: RDSLP              Last modification: 11-21-1991  c
c  Programmers: Y. Brin, D. Lamich, M. Seablom                     c
c                                                                  c
c  Program RDSLP reads in sea level pressure and wind data from    c
c  the repack dataset and performs initial quality control (v2.1)  c
c  functions. Datasets are assumed to be postioned ready to read   c
c  (via subroutine PTNDAT).                                        c
c                                                                  c
c------------------------------------------------------------------c
c                                                                  c
c   N A S A / D A O     O P T I M U M   I N T E R P O L A T I O N  c
c                     Version 2.0  Winter, 1994                    c
c                                                                  c
c------------------------------------------------------------------c
 
 
      subroutine rdslp ( data, nobs, suspect )
 
      parameter ( mxqobs= 200 )
 
      real              data( 1500000 ), ibmbuf
      logical           suspect(150000,2), strat, sslp, fdcos,
     &                  fexp, saveae, sigdat, qobs, offlev, onestage
      character  *8     dataname
      character  *24    obsname
      character  *60    message
 
      data fill /1.e15/
      common /scan/     maxobs, rscan, onestage, max_data
      common /contrl/   im, jnp, nb, ndalt, nhms, nlay, nlaym1,
     &                  nlayp1, nymd, strat, idimen, sslp, fdcos,
     &                  fexp, saveae, sigdat, maxrep
 
      common /dataptrs/ ikx, ilat, ilon, ipr, itype, islp, ifcer
 
      common /level/    ibmbuf(113,9), slptim(113), slpkx(113),
     &                  slplat(113), slplon(113), slp(113),
     &                  uwnd(113), vwnd(113), qslp(113), qwnd(113),
     &                  spdum(7), pad2(157959), bpad(20000)
 
      common /obs/      maxdex, nfgge, lun, dataname(5), hh(13,13,20),
     &                  vcoroe(18,18,4), obserr(mxqobs,4,19),
     &                  ww(13,13,20),
     &                  obsname(mxqobs), qobs(mxqobs,5), offlev(mxqobs)
 
      common /record/   indat, intim, inctim, nfor(5), nbuf(5)
 
!  Initialize data quality flags ( array 'suspect' is true if an ob-
!  servation fails first-pass quality control ).
 
      message(1:60) = ' '
      suspect(1:150000,1) = .false.
      suspect(1:150000,2) = .false.
 
C  Initialize the starting addresses for array "DATA".
 
      ikx   =              1
      ilat  = 1*150000 + 1
      ilon  = 2*150000 + 1
      ipr   = 3*150000 + 1
      itype = 4*150000 + 1
      islp  = 5*150000 + 1
      iuwnd = 6*150000 + 1
      ivwnd = 7*150000 + 1
 
C  Check for date and time match and dataset positioning.
 
      if( indat .ne. nymd .or. intim .ne. nhms ) then
         write(6,'(/," Analysis time: ",2i9,/," Time on dataset: ",
     &         2i9)') nymd, nhms, indat, intim
         message = ' RDSLP: Dataset positioned at wrong ' //
     &            'date/time.'
         call flash ( 5,
     &    'ABORT in RDSLP. Dataset positioned to wrong date/time.      '
     &    ,1)
 
      end if
 
      nobs = 0
 
!  Loop over NFGGE datasets.
 
      do nf = 1,nfgge
 
        lin = lun + nf - 1
        nform = nfor( nf )
        nbuff = nbuf( nf )
 
C  Read sea level pressure and wind data. "NOBS" is the total number of
C  observations.
 
 
        if ( nbuff .ne. 0 .and. nform .eq. 2 ) then
 
         do j = 1,nbuff
 
            read( lin, iostat=ierr ) ibmbuf
            slptim(1:113) = ibmbuf(1:113,1)
            slpkx(1:113)  = ibmbuf(1:113,2)
            slplat(1:113) = ibmbuf(1:113,3)
            slplon(1:113) = ibmbuf(1:113,4)
            slp(1:113)    = ibmbuf(1:113,5)
            uwnd(1:113)   = ibmbuf(1:113,6)
            vwnd(1:113)   = ibmbuf(1:113,7)
            qslp(1:113)   = ibmbuf(1:113,8)
            qwnd(1:113)   = ibmbuf(1:113,9)
 
            if ( ierr .ne. 0 ) then
              call flash ( 2,
     &    'I/O error encountered in RDSLP. Data ignored.               '
     &        ,1)
            end if
 
!  Store only those reports which pass the proximity check.
 
            do i = 1,113
 
               kk = slpkx( i ) + .1
               if( slpkx(i) .lt. 1.e7 .and. qobs(kk,nf) .and.
     &             qslp(i) .lt. 1.99 .and. qwnd(i) .lt. 1.99 ) then
                  nobs = nobs + 1
 
                  if( nobs .GT. max_data ) then
                    message = ' RDSLP: NOBS exceeds MAX_DATA.    '
                    call flash ( 5,message,1 )
                  end if
 
                  data(  ikx+nobs-1) =  slpkx(i)
                  data( ilat+nobs-1) = slplat(i)
                  data( ilon+nobs-1) = slplon(i)
                  data( islp+nobs-1) =    slp(i)
                  data(iuwnd+nobs-1) =   uwnd(i)
                  data(ivwnd+nobs-1) =   vwnd(i)
 
!  Identify suspect data.
 
                  if( qslp(i) .gt. 0.5 ) suspect( nobs,1 ) = .true.
                  IF( qwnd(i) .gt. 0.5 ) suspect( nobs,2 ) = .true.
 
!  Identify and elimilate very bad data.
 
                  if( qslp(i).gt.1.5) data( islp +nobs-1 ) = fill
                  if( qwnd(i).gt.1.5) data( iuwnd+nobs-1 ) = fill
                  if( qwnd(i).gt.1.5) data( ivwnd+nobs-1 ) = fill
               endif
            end do    ! <- End storing this buffer of reports.
         end do       ! <- End reading this buffer of reports.
      end if          ! <- NBUFF <> 0
 
!  Read the next sub-header.
 
      read( lin, iostat=ierr ) form, buff
 
      if( ierr .ne. 0 ) then
         message = ' RDSLP: I/O error reading data. '
         call flash( 5,message,1 )
      end if
 
      nfor(nf) = int( form )
      nbuf(nf) = int( buff )
 
      end do  ! <- End looping over NFGGE datasets.
 
      write ( 6,'(/," Total number of surface observations read:",
     &        i9)') nobs

!  Compress the data array in the surface  analysis.

      do  i = 1,nobs
        data( i+nobs ) = data( ilat+i-1 )
      end do  

      do i = 1,nobs  
        DATA( I+ 2*NOBS ) = DATA( ILON+I-1 )
      end do  

      do  i = 1,nobs
        data( i+ 3*nobs ) = data( ipr+i-1 )
      end do  

      do  i = 1,nobs
        data( i+ 4*nobs ) = data( itype+i-1 )
      end do  

      do  i = 1,nobs
        data( i+ 5*nobs ) = data( islp+i-1 )
      end do  
   
      do  i = 1,nobs
        data( i+ 6*nobs ) = data( iuwnd+i-1 )
      end do  

      do  i = 1,nobs
        data( i+ 7*nobs ) = data( ivwnd+i-1 )
      end do  

      IKX    =          1
      ILAT   = IKX  + NOBS
      ILON   = ILAT + NOBS
      IPR    = ILON + NOBS
      ITYPe   = IPR  + NOBS
      ISLP   = ITYPe + NOBS

 
 
      return
      end
