c---------------------------------------------------------------------c
c                                                                     c
c                                                                     c
c    Program Name: SQCRPT ( Surface Quality Control Report )          c
c    Programmer  : Michael Seablom, NASA/GSFC Code 910.3              c
c                  Greenbelt, MD 20771  (301)286-7360 (FTS)888-7360   c
!
!  Gregor von Laszewski, 12/95 Changed to compile on RS6000 using xlf
! 
c                                                                     c
c    Date        : December 7, 1994                                   c
c    Computer    : Cray C90                                           c
c    Language    : Cray Fortran (CFT77 Version 6.0)                   c
c                                                                     c
c  Program SQCRPT prints out surface quality control statistics.      c
c  This subroutine is patterned after QCREPORT which performs similar c
c  duties in the upper air analysis.                                  c
c                                                                     c
c---------------------------------------------------------------------c
 
 
      subroutine sqcrpt ( data, nobs, type, nymd, nhms )
 
      real        data(1500000)
      integer     nobs, statistics(200,10), testval,
     &            type
      character*6 analysis_time, date_temp, time_temp
      character*8 dataname
      character*10 analysis_date
      character*24 obsname
      logical     qobs, offlev, onestage
 
      common /scan/ maxobs, rscan, onestage, max_data
      common /dataptrs/ ikx,ilat,ilon,iprs,ityp,idel,ifcer,itime
 
      common /obs/ maxdex, nfgge, lun, dataname(5), hh(13,13,20),
     &             vcoroe(18,18,4), obserr(200,4,19), ww(13,13,20),
     &             obsname(200), qobs(200,5), offlev(200)
 
 
      data statistics / 2000*0 /
 
 1000 format(//,
     &     5x,"Observation Type",t28,"Reports [Passed]",t48,"RMS-",
     &       "Delta",t60,"Mean Fcst Err",/,5x,16("-"),t28,18("-"),
     &      t48, 9("-"),t60,13("-"),//)
!gvl comma missing 
      write(6,'("1",//,80("-"),//,16x,"Quality Control, Version",
     &      " 2.1   Autumn, 1991",//,80("-"),//)')
 
      write (date_temp,'(i6)') nymd
      analysis_date = date_temp(3:4) // '-' // date_temp(5:6) // '-'
     &    //'19'// date_temp(1:2)
      if ( nymd .le. 500000 ) then
      analysis_date = date_temp(3:4) // '-' // date_temp(5:6) // '-'
     &    //'20'// date_temp(1:2)
      end if
      write (time_temp,'(bz,i6)') nhms
      analysis_time = time_temp(1:2) // ':' // time_temp(3:4) // 'Z'
      if ( nhms .eq. 0 )  analysis_time = ' 0:00Z'
      iuwnd = idel + nobs
      ivwnd = iuwnd + nobs
      ife   = ivwnd + nobs
 
 
      write(6,'(///)')
      if ( type .eq. 1 ) write(6,'(/," SEA LEVEL PRESSURE STATIS",
     &  "TICS    VARIABLE TOLERANCES",/," Analysis",
     &  " date: ",a10,3x,"Analysis time: ",bz,a6)')
     &   analysis_date, analysis_time
 
 
      if ( type .ne. 1 ) write(6,'(/," OCEAN SURFACE WIND STA",
     &  "TISTICS   VARIABLE TOLERANCES",/," Analysis",
     &  " date: ",a10,3x,"Analysis time: ",bz,a6)')
     &   analysis_date, analysis_time
 
      write(6,1000)
 
      statistics( 1:200, 1:10 ) = 0
 
 
c Tally the total number of observations for each data type.
 
        do 2 j = 1,200
          do i = 1,nobs
 
            if ( data(ikx+i-1) .eq. float(j) )
     &        statistics(j,1) = statistics(j,1) + 1
 
          end do
 
c Total the number of accepted reports.
 
        do i = 1,nobs
 
          if ((int(data(ityp+i-1)) .eq. type .or.
     &         int(data(ityp+i-1)) .eq. 3) .and.
     &         data(ikx+i-1) .eq. float(j) )
     &                  statistics(j,2) = statistics(j,2) + 1
 
        end do
 
c Number of suspect observations. If type = 1, pressure observations
c are being tallied, otherwise tally wind observations.
 
        if ( type .eq. 1 ) then
 
c Suspicious pressure observations.
 
          do i = 1,nobs
           if ( data(ikx+i-1) .eq. float(j) ) then
            testval = int( data( ityp+i-1 ) * 10. )
 
            if ( testval .eq. 13 .or. testval .eq. 3 )
     &        statistics(j,3) = statistics(j,3) + 1
 
            if ( testval .eq. 13 )
     &        statistics(j,4) = statistics(j,4) + 1
 
c Gross check failures.
 
            if ( testval .eq. 12 .or. testval .eq. 1 )
     &        statistics(j,5) = statistics(j,5) + 1
 
c Number reaccepted by buddy check.
 
            if ( testval .eq. 12 ) statistics(j,6) = statistics(j,6) + 1
           end if
          end do
 
       else
 
c Suspicious wind observations.
 
          do i = 1,nobs
           if ( data(ikx+i-1) .eq. float(j) ) then
            testval = int( data( ityp+i-1) * 100. ) -
     &                int( data( ityp+i-1 ) * 10.)* 10
 
            if ( testval .eq. 3 ) then
              statistics(j,3) = statistics(j,3) + 1
 
              if ( int(data(ityp+i-1)) .eq. 2 .or.
     &           int(data(ityp+i-1)) .eq. 3 )
     &           statistics(j,4) = statistics(j,4) + 1
 
            end if
 
c Gross check failures.
 
            if ( testval .eq. 2 .or. testval .eq. 1 )
     &        statistics(j,5) = statistics(j,5) + 1
 
c Number reaccepted by buddy check.
 
            if ( testval .eq. 2 )
     &         statistics(j,6)= statistics(j,6) + 1
           end if
          end do
 
       end if
 
 
c Average forecast errors and deltas for observations which passed and
c failed the quality control.
 
       fe_accept = 0.
       fe_fail   = 0.
       delta_accept = 0.
       delta_fail   = 0.
       number_accept = 0
       number_fail   = 0
 
       if ( type .eq. 1 ) then
 
         do i = 1,nobs
          if ( data(ikx+i-1) .eq. float(j) ) then
           itype = int( data( ityp+i-1 ) )
 
           if ( itype .eq. 1 .or. itype .eq. 3 ) then
 
              number_accept = number_accept + 1
              fe_accept = fe_accept + data( ife+i-1 )
              delta_accept = delta_accept + (data( idel+i-1))**2
 
           else if ( int( data(ityp+i-1)*10.) .ne. 4 ) then
 
              number_fail = number_fail + 1
              fe_fail = fe_fail + data( ife+i-1 )
              delta_fail = delta_fail + (data( idel+i-1))**2
 
           end if
          end if
         end do
 
       else
 
         do i = 1,nobs
          if ( data(ikx+i-1) .eq. float(j) ) then
           itype = int( data( ityp+i-1 ) )
 
           if ( itype .eq. 2 .or. itype .eq. 3 ) then
 
              number_accept = number_accept + 1
              fe_accept = fe_accept + data( ife+i-1 )
              delta_accept = delta_accept +
     &         (sqrt(data(iuwnd+i-1)**2  +
     &               data( ivwnd+i-1 ) **2 ))**2
 
           else
            testval = int( data( ityp+i-1) * 100. ) -
     &                int( data( ityp+i-1 ) * 10.)* 10.
            if ( testval .ne. 4 ) then
 
              number_fail = number_fail + 1
              fe_fail = fe_fail + data( ife+i-1 )
              delta_fail = delta_fail +
     &                      (sqrt( data(iuwnd+i-1) **2 +
     &                       data( ivwnd+i-1 ) ** 2 ))**2
 
            end if
           end if
          end if
         end do
 
         end if
 
         if ( number_fail .ne. 0 ) then
           fe_fail = fe_fail / float( number_fail )
           delta_fail = sqrt(delta_fail / float( number_fail ))
         end if
         if ( number_accept .ne. 0 ) then
           fe_accept = fe_accept / float( number_accept )
           delta_accept = sqrt(delta_accept / float( number_accept ))
         end if
         if ( statistics(j,1) .ne. 0 ) write(6,'(a24,4x,i5,"[",i4,
     &      "%]",t48,e9.3,t61,e9.3)')
     &      obsname(j), statistics(j,1),nint(float(statistics(j,2))/
     &      float(statistics(j,1))*100), delta_accept, fe_accept
 
 2      continue
 
 1    continue
      return
      end
