      subroutine terp ( im, jm, lat, lon, f, fint )

c     Program Name: TERP
c     Date:         August 29, 1990
c     Language:     Fortran 77
c     Computer:     Cray YMP
c
c     TERP interpolates horizontally from earth coordinates (gridded).
c     Input values "lat" and "lon" are the latitude and longitude of 
c     a point, "f" is the field to be interpolated. The output value,
c     "fint", contains the value at the point.

      real lat, lon, f(im,jm), fint, yx(2)
          
      yx(1) = ( lat + 90. ) / ( 180./( float(jm-1) ) ) + 1.
      yx(2) = ( 180. + lon ) / ( 360./(float(im)) ) + 1.

      if ( yx(2) .ge. (im+1) ) yx(2) = yx(2) - im

      j0 = yx(1)

c   Protect j1 from being equal to jm+1, which can occur at the north pole.

      if ( j0 .lt. jm ) then
         j1 = j0 + 1
      else
         j1 = j0
      end if

      wt_j1 = yx(1) - j0
      wt_j0 = 1. - wt_j1

      i0 = yx(2)
      i1 = i0 + 1

      wt_i1 = yx(2) - i0
      wt_i0 = 1 - wt_i1

      if ( i1 .gt. im ) i1 = i1 - im

      fint = wt_j1 * wt_i1 * f( i1,j1 ) + wt_j1 * wt_i0 * f( i0,j1 ) +
     &       wt_j0 * wt_i1 * f( i1,j0 ) + wt_j0 * wt_i0 * f( i0,j0 )

      return
      end
