      SUBROUTINE TESTMVP( nin, ns, gv_nh, nv, im, jnp, mlev,
     $                    rlat_dump, rlon_dump, rlev_dump, n_dump,
     $                    gp, al,
     $                    ns_dump, nh_dump, nv_dump, 
     $                    rlat_save, rlon_save, rlev_save,
     $                    n_dump_this_call )

! nin      - Number of volume matrix systems in stack
! ns       - Maximum number of matricies allowed in stack
! gv_nh    - Number of horizontal grid points in volumes in this stack
! nv        - Number of vertical grid points in volumes in this stack
! im        - Number of longitudinal points
! jnp       - Number of latitudinal points 
! mlev      - Number of vertical analysis levels 
! rlat_dump - latitudes of grid points to dump
! rlon_dump - longitudes of grid points to dump
! rlev_dump - pressure levels of grid points to dump
! n_dump    - number of grid points to dump
! gp        - Horizontal grid point indicies of all grid points of 
!             all volumes in this stack
! al        - Level indicies of top levels of volumes in this stack
! ns_dump   - Pointer to volumes to dump (if any)
! nh_dump   - Pointer to horizontal grid point indicies to dump (if any)
! nv_dump   - Pointer to vertical grid point indicies to dump (if any)
! rlat_save - Latitudes of grid points that will be dumped (if any)
! rlon_save - Longitudes of grid points that will be dumped (if any)
! rlev_save - Levels of grid points that will be dumped (if any)
! n_dump_this_call - Number of grid points that will be dumped from this
!                    stack
 
C  SETS POINTERS TO THOSE grid points IN THE CURRENT BATCH WHICH ARE
C  TO BE PRINTED.
 
 
      integer           max_nh, grid_volume, gv_nh
      parameter       ( max_nh = 288 )
      real              AL(NS)
      real              GP(NS,gv_nh)
      real              rlat_dump(n_dump)
      real              rlon_dump(n_dump)
      real              rlev_dump(n_dump)
      integer           ns_dump(n_dump)
      integer           nh_dump(n_dump)
      integer           nv_dump(n_dump)
      real              rlat_save(n_dump)
      real              rlon_save(n_dump)
      real              rlev_save(n_dump)

      integer           i3d_volgp(nin,gv_nh,nv)
      integer           i3d_dmpgp(n_dump)

      COMMON /PMH/ PMAND(18)                                                    

      JX(XLAT,DLAT) = (XLAT+90.0)/DLAT + 1.1                                  
      IX(XLON,DLON) = (XLON+180.0 )/DLON + 1.1                                  

      IMJNP = IM * JNP
      CALL GETCONH('DLAT    ',DLAT  )                                        
      CALL GETCONH('DLON    ',DLON  )                                        
 
! Find 3D index of each grid point in each volume in this stack

      DO I = 1, NIN
         IGV = nint( AL(I) )
         IGV = IMJNP * ( IGV - 2 )
         DO NNH = 1, gv_nh
            IGP = nint( GP(I,NNH) + IGV )
            DO NNV = 1, NV
               IGP = IGP + IMJNP
               i3d_volgp(i,nnh,nnv)=igp
            end do    
         end do    
      end do

! Find 3D index of each grid point to dump

      do i=1,n_dump
         do l=1,mlev
            if (rlev_dump(i).eq.pmand(l)) goto 100
         end do 
         igv=99
         goto 110
  100    igv=l
  110    continue
         
         i3d_dmpgp(i)=(igv-1)*imjnp + 
     .         (jx(rlat_dump(i),dlat)-1)*im+ix(rlon_dump(i),dlon)
      end do
 
! Check if any 3D grid point indicies of the grid points to be saved 
! match any of the 3D grid point indicies of the grid points in each
! of the volumes.  If so, save pointer to that grid point in that volume.

      n_dump_this_call=0
 
      do idump=1,n_dump

         DO I = 1, NIN
         DO NNH = 1, gv_nh
         DO NNV = 1, NV
            if (i3d_volgp(i,nnh,nnv).eq.i3d_dmpgp(idump)) then
               n_dump_this_call = n_dump_this_call + 1
               ns_dump(n_dump_this_call)=i
               nh_dump(n_dump_this_call)=nnh
               nv_dump(n_dump_this_call)=nnv
               rlat_save(n_dump_this_call)=rlat_dump(idump)
               rlon_save(n_dump_this_call)=rlon_dump(idump)
               rlev_save(n_dump_this_call)=rlev_dump(idump)
            end if
         end do
         end do
         end do
         
      end do
               
      RETURN
      END
