      subroutine zuvcov ( nrep, igp, ireg, lev, levlindx, obserr,
     &                 irank, data, nobs, symat, delta, force,
     &                 gp, ztx, z_err, u_err, v_err )

c..................................................................c
!                                                                  !
! NASA Goddard Space Flight Center       Data Assimilation Office  !
!                                                                  !
! Goddard Earth Observing System                                   !
! Data Analysis System - Version 2.0                               !
! February, 1994                                                   !
!                                                                  !
!------------------------------------------------------------------c
! 
!
!  Programmers: S. Bloom, J. Woollen, M. Seablom
!  Language   : Cray cft77 Fortran version 6.0
!  Computer   : Cray C90 6/256
!
! ZUVCOV forms a covariance matrix for low and middle latitude volumes
! in the upper air z-u-v analysis.
C
C  Passed input arguments:
C
C     NREP     - NUMBER OF OBSERVATIONS TO USE IN GRIDPOINT ANALYSIS
C     IGP      - GRID INDEX
C     IREG     - REGION OF CORRELATIONS TO USE
C     LEV      - LEVEL OF ANALYSIS
C     LEVLINDX - ARRAY WITH DATA INDEXES FOR THIS MATRIX
C     OBSERR   - OBSERVATIONAL ERROR ARRAY
C     Z_ERR    - FORECAST ERROR ARRAY FOR HEIGHTS
C     U_ERR    - FORECAST ERROR ARRAY FOR U_WINDS (FUTURE USE)
C     V_ERR    - FORECAST ERROR ARRAY FOR V-WINDS (FUTURE USE)
C
C  OUTPUT ARGUMENTS:
C
C     IRANK    - Order of matrix ( passed back to SOLVEDRV )
C
C
!  Program modification history:
!
C              J.WOOLLEN  4/84
C              M.NESTLER  7/84
C              J.WOOLLEN  9/85
C              J.WOOLLEN  9/86
C              M.SEABLOM  5/89 TROPICAL DECOUPLING ADDED
C              D.LAMICH  11/89 DIAGNOSTIC PRINTS AT MINI-VOLUMES
C              D.LAMICH  11/89 CORRECTED TROPICAL WIND ANALYSIS
C              D.LAMICH   4/90 USING ZSTD*WSTD FOR ZUCOV & ZVCOV INSTEAD
C                              OF WSTD*WSTD. FIXES GP-OB COV IN TROPICS.
C              S.BLOOM    7/90 DECYBERIZATION; MODULARIZATION OF
C                              COVARIANCE COMPUTATIONS
!              M.Seablom  8/91 Incorporation of variable-sized
!                              minivolumes.
!              M.Seablom  5/92 Implementation of hybrid matrix setup.
!              M.Seablom  3/93 Parallelization of matrix solution.
!              M.Seablom  8/93 Optional removal of data selection;
!                              minivolumes use all available data.
C
!--------------------------------------------------------------------
                                                                        
! ( Parameter statements will not be used in Fortran 90 version!
!   Temporary fix to get around dynamic allocation problems. )

      parameter  ( max_nh=600, maxrank=75 )
#ifdef RS6000
      parameter  ( max_huge=228150 )
#endif
#ifdef CRAY
      parameter  ( max_huge=((max_nh+maxrank+1)*(max_nh+maxrank))/2 )
#endif
      real
     &   bc(max_nh+maxrank),bf(max_nh+maxrank),zoberr(600),uoberr(600),
     &   ustdp(600), vstdp(600), levlindx( nrep ),z_err(im*jnp,mlev),
     &   obserr(200,4,19),data(1500000),voberr(600), ztx(150000),
     &   u_err(im*jnp,mlev), v_err(im*jnp,mlev)

                                                                        
      common /mvdumper/ mv_dumper, num_dump, igpmvp(20),
     &                  levmvp(20)

#ifdef CRAY
      task common /zuvmat/
     &  zlat(max_nh+maxrank),zlon(max_nh+maxrank),zprs(max_nh+maxrank),
     &  zstd(max_nh+maxrank),zkx(max_nh+maxrank),
     &  wlat(max_nh+maxrank),wlon(max_nh+maxrank),wprs(max_nh+maxrank),
     &  wstd(max_nh+maxrank),wkx(max_nh+maxrank),
     &  siny(max_nh+maxrank),cosy(max_nh+maxrank),zdel(max_nh+maxrank),
     &  udel(max_nh+maxrank),vdel(max_nh+maxrank),
     &  zlst(max_nh+maxrank),wlst(max_nh+maxrank),type(max_nh+maxrank),
     &  ober(max_nh+maxrank),fcor(max_nh+maxrank),
     &  ufac(max_nh+maxrank),vfac(max_nh+maxrank),twstd(max_nh+maxrank),
     &  glat(max_nh+maxrank),glon(max_nh+maxrank),gprs(max_nh+maxrank),
     &  gstd(max_nh+maxrank), 
     &  cv1(max_huge),cv2((max_nh+maxrank)*maxrank),
     &  cv3((max_nh+maxrank)*maxrank), 
     &  cv4((max_nh+maxrank)*maxrank),   
     &  cv5((max_nh+maxrank)*maxrank),
     &  cv6((max_nh+maxrank)*maxrank),
     &  w1(max_huge), w2(max_huge),
     &  w3(max_huge), w4(max_huge), 
     &  w5(max_huge), w6(max_huge),
     &  ztime(max_nh+maxrank)
#endif
#ifdef RS6000
      common /zuvmat/
     &  zlat(max_nh+maxrank),zlon(max_nh+maxrank),zprs(max_nh+maxrank),
     &  zstd(max_nh+maxrank),zkx(max_nh+maxrank),
     &  wlat(max_nh+maxrank),wlon(max_nh+maxrank),wprs(max_nh+maxrank),
     &  wstd(max_nh+maxrank),wkx(max_nh+maxrank),
     &  siny(max_nh+maxrank),cosy(max_nh+maxrank),zdel(max_nh+maxrank),
     &  udel(max_nh+maxrank),vdel(max_nh+maxrank),
     &  zlst(max_nh+maxrank),wlst(max_nh+maxrank),type(max_nh+maxrank),
     &  ober(max_nh+maxrank),fcor(max_nh+maxrank),
     &  ufac(max_nh+maxrank),vfac(max_nh+maxrank),twstd(max_nh+maxrank),
     &  glat(max_nh+maxrank),glon(max_nh+maxrank),gprs(max_nh+maxrank),
     &  gstd(max_nh+maxrank), 
     &  cv1(max_huge),cv2((max_nh+maxrank)*maxrank),
     &  cv3((max_nh+maxrank)*maxrank), 
     &  cv4((max_nh+maxrank)*maxrank),   
     &  cv5((max_nh+maxrank)*maxrank),
     &  cv6((max_nh+maxrank)*maxrank),
     &  w1(max_huge), w2(max_huge),
     &  w3(max_huge), w4(max_huge), 
     &  w5(max_huge), w6(max_huge),
     &  ztime(max_nh+maxrank)
#endif

      real symat(maxrank*(maxrank+1)/2), delta(maxrank), gp(max_nh),
     &     force(maxrank,max_nh,3)

      logical onestage, mv_dumper, dump_flag, qmvprint
                                                                        
      common /dataptrs/ ikx,ilat,ilon,iprs,ityp,idel,istd,itime         
      COMMON /scan/     maxobs, rscan, onestage, max_data               
                                                                        
      COMMON /GETIJ/ GETI(250000),GETJ(250000)
                                                                        
      COMMON /GRIDXX/ IM,JNP,MLEV                                       
                                                                        
      COMMON /OBSCORR/ VCORLIST(200),HCORLIST(200),VCOROE(13,13,4),     
     .                 HCOROE(1200,5)                                    
                                                                        
      COMMON /PMH/ PMAND(18)                                            
                                                                        
                                                                        
      common /sparam/ lsolve, ns, nr, nv, nh(13104), nq,
     &                kq(3), volcentre_x(13104),
     &                ngv, volcentre_y(13104), glatup(13104),
     &                glonup(13104)                  


                                                                        
      COMMON /VCORS/ VCORFE(13,13,20,2)                                 
                                                                        
      character *1 qi, qj
      character *8 obstag(200)
      character*60 message
                                                                        
      parameter (zero=0.0,one=1.0,two=2.0,onept1=1.10)
      parameter (ninety=90.0,one80=180.0,tenth=0.1,pt05=0.05)
      parameter (tenm4=0.0001)
      data qmvprint/.false./
C                                                                       
! Function statements.

      i1(i) = i*(i-1)/2
      i2(i) = i*(i+1)/2

      jx( xlat,dlat ) = ( xlat + 90. ) / dlat + 1.1
      ix( xlon,dlon ) = ( xlon +180. ) / dlon + 1.1

                                                                        
! Variable initialization.

      message(1:60) = ' '                                               
                                                                        
      call getconh('GRADEI  ',gradei)                                
      call getconh('OMEGA2  ',omega2)                                
      call getconh('PI180   ',pi180 )                                
      call getconh('DLAT    ',dlat  )                                
      call getconh('DLON    ',dlon  )                                
      call getconi('MAXREP  ',maxrep)                                
                                                                        
      if ( maxrep*3 .gt. maxrank ) then
            message = ' ZUVCOV: Excessive amount of data.'       
            call flash( 5,message,1 )                                   
      end if                                                         



!  Set "nhgp" to be the number of horizontal grid points for current
!  volume.

      nhgp = nh( igp )
      n = 0

! Initialize arrays.

      force( 1:maxrank, 1:max_nh, 1:3 ) = 0.0
      delta( 1:maxrank ) = 0.0
      symat( 1:(maxrank*(maxrank+1)/2)) = -1.

      irank = 0
      n = 0
      do ii = 1,maxrep*3 + nv*nhgp
        do jj = 1,ii

           n = n + 1

           geti( n ) = float( ii )
           getj( n ) = float( jj )

        end do
      end do

c      if ( n .gt. 99999 ) then
c         message = ' ZUVCOV: Pointers have gone out of bounds.'
c         call flash( 5,message,1 )
c      end if
   

!  Locate grid points within a volume.

      nh_count = 0
      nhnv_count = 0

      do j = 1,2
         do i = 1,nhgp/2

           nh_count = nh_count + 1
           glatx = glatup( igp ) + ( j-1 )*dlat
           if ( glatx .gt. 90. ) glatx = 90.
           glonx = glonup( igp ) + ( i-1 )*dlon
           iigp = ( jx(glatx,dlat)-1)*im + ix( glonx,dlon )
           gp( nh_count ) = iigp

           do k = 1,nv

             nhnv_count = nhnv_count + 1
             nl = lev - 1 + k
             glat(nhnv_count) = glatx
             glon(nhnv_count) = glonx
             gprs(nhnv_count) = pmand(nl)
             gstd(nhnv_count) = z_err(iigp,nl)
 
           end do !        <-- End looping over levels.
 
         end do   !        <-- End looping over latitudes.
      end do      !        <-- End looping over longitudes.


                                                                        
                                                                        
! Set flag for diagnostic print.                            
 
      dump_flag = .false.
      if ( mv_dumper ) then
         do imvp = 1,num_dump
 
            if ( igp.eq.igpmvp(imvp) .and. lev.eq.levmvp(imvp) ) then
               dump_flag = .true.
#ifdef RS6000
               write(6,'(/," ZUVCOV: Grid point information at vmv ",
     &              i6," at level ",i4,//," ",5x,"Index",7x,"Lat",7x,
     &              "Lon",6x,"Pres",2x,"FG H Err",/)') igp, lev
#endif
#ifdef CRAY
               write(6,'(/," ZUVCOV: Grid point information at vmv ",
     &              i6," at level ",i4,//," ",5x,"Index",7x,"Lat",7x,
     &              "Lon",6x,"Pres",2x,"FG H Err",/)') igp, lev

#endif
               do idxgp = 1,nv*nhgp                          
                  write(6,'(" ",i10,f10.2)') idxgp, glat(idxgp),
     &                 glon(idxgp), gprs(idxgp), gstd(idxgp)
              end do
 
            end if
 
         end do
      end if

                                                                        
C  MAKE LISTS OF HEIGHT AND WIND DATA                                   
C  ----------------------------------                                   
                                                                        
      NZ = 0                                                            
      NW = 0                                                            
                                                                        
      CALL A77HGATH(DATA(ITYP),LEVLINDX,TYPE,NREP)                      
                                                                        
      ICOUNT = 0                                                        
                                                                        
      DO 5006 IPT=1,NREP                                                
        IF(TYPE(IPT) .NE. TWO) THEN                                     
              ICOUNT       = ICOUNT + 1                                 
              ZLST(ICOUNT) = LEVLINDX(IPT)                              
        END IF                                                          
 5006 CONTINUE                                                          
                                                                        
      NZ = ICOUNT                                                       
                                                                        
      ICOUNT = 0                                                        
                                                                        
      DO 5007 IPT=1,NREP                                                
        IF(TYPE(IPT) .NE. ONE) THEN                                     
              ICOUNT       = ICOUNT + 1                                 
              WLST(ICOUNT) = LEVLINDX(IPT)                              
        END IF                                                          
 5007 CONTINUE                                                          
                                                                        
      NW = ICOUNT                                                       
                                                                        
      IF(NZ.EQ.0 .AND. NW.EQ.0) IRANK = 0                               
      IF(NZ.EQ.0 .AND. NW.EQ.0) RETURN                                  
                                                                        
      IF(NZ+2*NW .GT. MAXREP*NQ) then                                   
       print*,' ZUVCOV: FATAL>> Too much data. '                        
       print*,' NZ:',nz,' NW:',nw,' MAXREP:',maxrep,' NQ:',nq           
       print*, ' Printout of TYPE array: '                              
       print*, type                                                     
       message = ' ZUVCOV: Excessive amount of data. See prev. msg.'    
       call flash ( 5,message,1 )                                       
      end if                                                            
                                                                        
C  GATHER STUFF FOR HEIGHT-HEIGHT COVARIANCE                            
                                                                        
                                                                        
!  Load gridpoint information for minivolumes.
 
      do mgv = 1,nhgp*nv
 
        zlat( nz+mgv ) = glat( mgv )
        zlon( nz+mgv ) = glon( mgv )
        zprs( nz+mgv ) = gprs( mgv )
        zstd( nz+mgv ) = gstd( mgv )
 
      end do

      IF(NZ.NE.0) THEN                                                  
                                                                        
         CALL A77HGATH(DATA(ILAT),ZLST,ZLAT,NZ)                         
         CALL A77HGATH(DATA(ILON),ZLST,ZLON,NZ)                         
         CALL A77HGATH(DATA(IPRS),ZLST,ZPRS,NZ)                         
         CALL A77HGATH(DATA(ISTD),ZLST,ZSTD,NZ)                         
         CALL A77HGATH(DATA(IDEL),ZLST,ZDEL,NZ)                         
         CALL A77HGATH(DATA(IKX ),ZLST,ZKX ,NZ)                         
         call a77hgath( ztx, zlst, ztime, nz )
                                                                        
C  OBTAIN   HEIGHT-HEIGHT ERROR COVARIANCES                             
C  ----------------------------------------                             
C                                                                       
C     HEIGHT - HEIGHT COVARIANCE WORKING ARRAY                          
C                                                                       
C     CV1 = ZZCOV                                                       
C                                                                       
                                                                        
         nz2 = ( nz+nhgp*nv ) * ( nz+nhgp*nv+1 )/2
         NZS = NZ*(NZ+1)/2                                              
         nzf = nz + nhgp*nv

C         IF (nhgp .GT. 20) then
C
C            CALL MMCOVH2('ZUV',IREG,nzf,NZ2,ZLON,ZLAT,ZSTD,GETI,GETJ,
C     1           CV1,W1,W2,W3,W4,W5,W6)
C
C         ELSE

         CALL MMCOV('ZUV',IREG,nzf,NZ2,ZLON,ZLAT,ZSTD,GETI,GETJ,CV1,
     1        W1,W2,W3,W4,W5,W6)

C         ENDIF
                                                                        
                                                                        
         CALL A77HGATH(ZPRS,GETI,W1,NZ2)                                
         CALL A77HGATH(ZPRS,GETJ,W2,NZ2)                                
                                                                        
         call vcorrfe( 'ZUV', w1, w2, w4, nz2 )
                                                                        
C  LOAD OBS-OBS COVARIANCES INTO SYMMETRIC MATRIX 'SYMAT'               
C  ------------------------------------------------------               
C  (NB..HERE OBS-OBS MEANS F.E.'S AT OBS LOCATIONS)                     
C                                                                       
C     ALSO - MULTIPLY BY VERTICAL CORRELATION 'W3'                      
                                                                        
                                                                        
         DO 5010 K=1,NZS                                                
                                                                        
           SYMAT(K) = CV1(K) * W4(K)                                    
                                                                        
 5010    CONTINUE                                                       
                                                                        
C  LOAD GRID-OBS COVARIANCES INTO ARRAY 'FORCE'                         
C  --------------------------------------------                         
                                                                        
         IOS = 0                                                        
                                                                        
         DO 5014 I=1,nhgp*NV 
           IOS = IOS + I - 1                                            
           KOS = NZS + (I-1)*NZ + IOS                                   
                                                                        
           DO 5012 K=1,NZ                                               
                                                                        
             FORCE(K,I,1) = CV1(K+KOS) * W4(K+KOS)                      
                                                                        
 5012      CONTINUE                                                     
 5014    CONTINUE                                                       
                                                                        
C                                                                       
C  GENERATE OBSERVATION HEIGHT-HEIGHT COVARIANCES, AND ADD THEM TO      
C  THE COVARIANCE MATRIX                                                
C                                                                       
                                                                        
         call obscov ( cv1, zkx, geti, getj, obserr, kq, w1, w2,
     &                 w3, w4, w5, w6, nz, ztime )
                                                                        
         DO 5015 K=1,NZS                                                
                                                                        
           SYMAT(K) = SYMAT(K) + CV1(K)                                 
                                                                        
 5015    CONTINUE                                                       
                                                                        
                                                                        
      END IF                                                            
                                                                        
C  GATHER STUFF FOR COVARIANCES WITH WIND TERMS                         
C  --------------------------------------------                         
                                                                        
      IF(NW.NE.0) THEN                                                  
         CALL A77HGATH(DATA(ILAT       ),WLST,WLAT,NW)                  
         CALL A77HGATH(DATA(ILON       ),WLST,WLON,NW)                  
         CALL A77HGATH(DATA(IPRS       ),WLST,WPRS,NW)                  
         CALL A77HGATH(DATA(ISTD       ),WLST,WSTD,NW)                  
         CALL A77HGATH(DATA(IDEL+  NOBS),WLST,UDEL,NW)                  
         CALL A77HGATH(DATA(IDEL+2*NOBS),WLST,VDEL,NW)                  
         CALL A77HGATH(DATA(IKX        ),WLST,WKX ,NW)                  
      ENDIF                                                             
                                                                        
      DO 5016 MGV=1,nhgp*nv 
                                                                        
        WLAT(NW+MGV) = GLAT(MGV)                                        
        WLON(NW+MGV) = GLON(MGV)                                        
        WPRS(NW+MGV) = GPRS(MGV)                                        
        WSTD(NW+MGV) = GSTD(MGV)                                        
                                                                        
 5016 CONTINUE                                                          
                                                                        
      nwf = nw + nhgp*nv
                                                                        
      DO 5017 L=1,NWF                                                   
                                                                        
          SINY(L) = SIN(WLAT(L)*PI180)                                  
          COSY(L) = COS(WLAT(L)*PI180)                                  
                                                                        
          IF(SINY(L).GT.-.05 .AND. SINY(L).LT.0.0) SINY(L) = -0.05
          IF(SINY(L).LT. .05 .AND. SINY(L).GE.0.0) SINY(L) =  0.05
          IF(COSY(L).EQ.0.0) COSY(L) = 1.e-4

                                                                        
 5017 CONTINUE                                                          
                                                                        
C                                                                       
C                                                                       
C.... OBATAIN TROPICALLY DECOUPLED WIND ERROR VARIANCES (WITH           
C     GEOSTROPHIC BALANCE ASSUMED IN EXTRA-TROPICS)                     
C                                                                       

      CALL WVARZ(IREG,NWF,SINY,COSY,WLAT,WSTD,WPRS,BC,BF,SCALE)         
                                                                        
C  COMPUTE THE WIND-HEIGHT COVARIANCES                                  
C  -----------------------------------                                  
C                                                                       
C     WORKING ARRAYS                                                    
C                                                                       
C     CV1 = UPCOV                                                       
C     CV2 = VPCOV                                                       
C                                                                       
                                                                        
      IF(NZ.NE.0) THEN                                                  
                                                                        
         NWZ = NWF*NZ                                                   
                                                                        
         CALL WMCOV('ZUV',IREG,NZ,NWF,ZLON,ZLAT,ZSTD,WLON,WLAT,WSTD,    
     1              BC,BF,SCALE,CV1,CV2,W1,W2,W3,W4,W5,W6)              
                                                                        
C                                                                       
C.... STORE THE RESULTS IN APPROPRIATE LOCATIONS IN SYMAT AND FORCE     
C                                                                       
                                                                        
c        IF(NW.NE.0) THEN                                               
                                                                        
         DO 5021 K=1,NWF                                                
            IOS = (K-1)*NZ                                              
                                                                        
            DO 5019 L=1,NZ                                              
                   W1(L+IOS) = float(K)                                 
                   W3(L+IOS) = float(L)                                 
 5019       CONTINUE                                                    
 5021    CONTINUE                                                       
                                                                        
         CALL A77HGATH(WPRS,W1,W2,NWZ)                                  
         CALL A77HGATH(ZPRS,W3,W4,NWZ)                                  
                                                                        
         call vcorrfe( 'ZUV', w2, w4, w5, nwz )
                                                                        
         DO 5025 KW=1,NW                                                
                                                                        
            IUP = I1(NZ+KW)                                             
            IVP = I1(NZ+NW+KW)                                          
            IOS = (KW-1)*NZ                                             
                                                                        
            DO 5023 KP=1,NZ                                             
                                                                        
                   SYMAT(IUP+KP) = CV1(IOS+KP) * W5(IOS+KP)             
                   SYMAT(IVP+KP) = CV2(IOS+KP) * W5(IOS+KP)             
                                                                        
 5023       CONTINUE                                                    
 5025    CONTINUE                                                       
                                                                        
c        END IF                                                         
                                                                        
         NOS = NW*NZ                                                    
                                                                        
         do kg = 1,nhgp*nv

            ios = nos + (kg-1)*nz

            do kp=1,nz

               force(kp,kg,2) = cv1(ios+kp) * w5(ios+kp)
               force(kp,kg,3) = cv2(ios+kp) * w5(ios+kp)

            end do
         end do

      END IF                                                            
                                                                        
C  COMPUTE THE WIND-WIND COVARIANCES                                    
C  ---------------------------------                                    
C                                                                       
C     WORKING ARRAYS                                                    
C                                                                       
C     CV1 = ZUCOV                                                       
C     CV2 = ZVCOV                                                       
C                                                                       
C     CV3 = UUCOV                                                       
C     CV4 = UVCOV                                                       
C     CV5 = VUCOV                                                       
C     CV6 = VVCOV                                                       
C                                                                       
                                                                        
      IF(NW.NE.0) THEN                                                  
                                                                        
      nhv = nhgp * nv                                                   
                                                                        
      NW2 = NW * NWF                                                    
                                                                        
c     call zeitbeg('wwcov')
      CALL WWCOV('ZUV',IREG,NW,NHV,NZ,WLON,WLAT,WSTD,ZSTD,BC,BF,        
     1           CV1,CV2,CV3,CV4,CV5,CV6,W1,W2,W3,W4,W5,W6)             
c      call zeitend
                                                                        
         DO 5040 K=1,NWF                                                
            IOS = (K-1)*NW                                              
                                                                        
            DO 5037 L=1,NW                                              
                   W1(L+IOS) = float(K)                                 
                   W3(L+IOS) = float(L)                                 
 5037       CONTINUE                                                    
 5040    CONTINUE                                                       
                                                                        
         CALL A77HGATH(WPRS,W1,W2,NW2)                                  
         CALL A77HGATH(WPRS,W3,W4,NW2)                                  
                                                                        
         call vcorrfe('ZUV', w2, w4, w5, nw2 )
                                                                        
              DO 5050 KW=1,NW                                           
                                                                        
                  IUU = I1(NZ+KW)    + NZ                               
                  IVU = I1(NZ+NW+KW) + NZ                               
                  IVV = I1(NZ+NW+KW) + NZ + NW                          
                                                                        
                  IOS = (KW-1)*NW                                       
                                                                        
                  DO 5042 KWW=1,KW                                      
                                                                        
                      SYMAT(IUU+KWW) = CV3(IOS+KWW) * W5(IOS+KWW)       
                      SYMAT(IVV+KWW) = CV6(IOS+KWW) * W5(IOS+KWW)       
                                                                        
 5042             CONTINUE                                              
                                                                        
                  DO 5045 KWW=1,NW                                      
                                                                        
                      SYMAT(IVU+KWW) = CV5(IOS+KWW) * W5(IOS+KWW)       
                                                                        
 5045             CONTINUE                                              
 5050         CONTINUE                                                  
                                                                        
              NOS = NW * NW                                             

              do kg=1,nhgp*nv

                  ios = nos + (kg-1)*nw

                  do kw=1,nw

                      force(nz+kw   ,kg,1) = cv1(ios+kw) * w5(ios+kw)
                      force(nz+nw+kw,kg,1) = cv2(ios+kw) * w5(ios+kw)

                      force(nz+kw   ,kg,2) = cv3(ios+kw) * w5(ios+kw)
                      force(nz+nw+kw,kg,2) = cv4(ios+kw) * w5(ios+kw)
                      force(nz+kw   ,kg,3) = cv5(ios+kw) * w5(ios+kw)
                      force(nz+nw+kw,kg,3) = cv6(ios+kw) * w5(ios+kw)

                  end do
              end do

                                                                        
      END IF                                                            
                                                                        
C  ADD THE OBSERVATIONAL ERROR TO THE FORECAST ERROR                    
C  -------------------------------------------------                    
                                                                        
100   IF(NZ.NE.0) THEN                                                  
                                                                        
         CALL obertrp0(ZKX,ZPRS,OBER,OBSERR,KQ(1),NZ)                   
                                                                        
         DO 110 I=1,NZ                                                  
             ZOBERR(I)    = OBER(I)                                     
             SYMAT(I2(I)) = SYMAT(I2(I)) + OBER(I)*OBER(I)              
 110     CONTINUE                                                       
      ENDIF                                                             
                                                                        
      IF(NW.NE.0) THEN                                                  
                                                                        
         CALL obertrp0(WKX,WPRS,OBER,OBSERR,KQ(2),NW)                   
                                                                        
         DO 120 I=1,NW                                                  
             UOBERR(I)       = OBER(I)                                  
             SYMAT(I2(NZ+I)) = SYMAT(I2(NZ+I)) + OBER(I)*OBER(I)        
 120     CONTINUE                                                       
                                                                        
c        CALL obertrp0(INDX,WPRS,OBER,OBSERR,KQ(3),NW)                  
         CALL obertrp0(wkx, WPRS,OBER,OBSERR,KQ(3),NW)                  
                                                                        
         DO 130 I=1,NW                                                  
             VOBERR(I)          = OBER(I)                               
             SYMAT(I2(NZ+NW+I)) = SYMAT(I2(NZ+NW+I)) + OBER(I)*OBER(I)  
 130     CONTINUE                                                       
      ENDIF                                                             
                                                                        
C  COUNT THE RANK AND STORE THE DELTAS                                  
C  -----------------------------------                                  
                                                                        
      IRANK = NZ + NW + NW                                              
                                                                        
      DO 140 I=1,NZ                                                     
          DELTA(I      ) = ZDEL(I)                                      
 140  CONTINUE                                                          
                                                                        
      DO 150 I=1,NW                                                     
          DELTA(NZ+I   ) = UDEL(I)                                      
          DELTA(NZ+NW+I) = VDEL(I)                                      
 150  CONTINUE                                                          
                                                                        
C DIAGNOSTIC PRINTS                                                     
C -----------------                                                     
                                                                        
c     IF (num_dump .gt. 0 .and. mv_dumper ) then                        
      if ( igp .eq. 000 ) then
                                                                        
C DIAGNOSTIC PRINTS OF HEIGHT DATA                                      
C --------------------------------                                      
                                                                        
      IF(NZ.GT.0) THEN                                                  
         PRINT 6701, IGP, LEV                                           
 6701    FORMAT(' ZUVCOV: HEIGHT DATA INFLUENCING MINI-VOLUME',I6,      
     .          ' AT LEVEL',I4,//,' ',5X,'INDEX',7X,'LAT',7X,'LON',     
     .          6X,'PRES',2X,'FG H ERR',3X,'OBS ERR',                   
     .          3X,'H DELTA',8X,'KX',6X,'TYPE',/)                       
         DO 6711 IDXDATA=1,NZ                                           
            IOBSKX=ZKX(IDXDATA)+0.1                                     
            PRINT 6703, IDXDATA,ZLAT(IDXDATA),ZLON(IDXDATA),            
     .                  ZPRS(IDXDATA),ZSTD(IDXDATA),ZOBERR(IDXDATA),    
     .                  ZDEL(IDXDATA),IOBSKX,OBSTAG(IOBSKX)             
 6703       FORMAT(' ',I10,6F10.2,I10,2X,A8)                            
 6711    CONTINUE                                                       
      END IF                                                            
                                                                        
C DIAGNOSTIC PRINTS OF WIND DATA                                        
C ------------------------------                                        
                                                                        
      IF (NW.GT.0) THEN                                                 
         PRINT 6805, IGP, LEV                                           
 6805    FORMAT(' ZUVCOV: WIND DATA INFLUENCING MINI-VOLUME',I6,        
     .          ' AT LEVEL',I4,//,' ',5X,'INDEX',7X,'LAT',7X,'LON',     
     .          6X,'PRES',                                              
     .          2X,'FG U ERR',2X,'U OB ERR',3X,'U DELTA',               
     .          2X,'FG V ERR',2X,'V OB ERR',3X,'V DELTA',               
     .          8X,'KX',6X,'TYPE',/)                                    
         DO 6809 IDXDATA=1,NW                                           
            IOBSKX=WKX(IDXDATA)+0.1                                     
            PRINT 6807, IDXDATA,WLAT(IDXDATA),WLON(IDXDATA),            
     .                  WPRS(IDXDATA),                                  
     .                  USTDP(IDXDATA),UOBERR(IDXDATA),UDEL(IDXDATA),   
     .                  VSTDP(IDXDATA),VOBERR(IDXDATA),VDEL(IDXDATA),   
     .                  IOBSKX,OBSTAG(IOBSKX)                           
 6807       FORMAT(' ',I10,9F10.2,I10,2X,A8)                            
 6809    CONTINUE                                                       
      END IF                                                            
                                                                        
C  FOR PRINTING INDIVIDUAL CASES                                        
C  -----------------------------                                        
                                                                        
                                                                        
         PRINT 912,IGP                                                  
912      FORMAT(' GP=',I5,' L = 9  (500 MB)                ',/,         
     .          ' TEST OF DEVECTORIZED UPPER AIR CODE  ',/,             
     .          '                                          ',///,       
     .          '                         ' )                           
         IF(NZ.NE.0) THEN                                               
            PRINT*, ' VALUES OF I, ZKX, ZLAT, ZLON, ZPRS, ZDEL, ZSTD:'  
            DO 102 I=1,NZ                                               
102         PRINT914,I,ZKX (I),                                         
     .                 ZLAT(I),                                         
     .                 ZLON(I),                                         
     .                 ZPRS(I),                                         
     .                 ZDEL(I),                                         
     .                 ZSTD(I)                                          
         ENDIF                                                          
         IF(NW.NE.0) THEN                                               
            WRITE(6,'(///)')                                            
            PRINT*, ' VALUES OF I,KX,LAT,LON,WPRS,UDEL,VDEL,WSTD,HSD:'  
            DO 103 I=1,NW                                               
103         PRINT914,I,WKX (I),                                         
     .                 WLAT(I),                                         
     .                 WLON(I),                                         
     .                 WPRS(I),                                         
     .                 UDEL(I),                                         
     .                 VDEL(I),                                         
     .                 WSTD(I)                                          
         ENDIF                                                          
914      FORMAT(' ',I5,6F12.2,2X,2F7.2)                                 
         IP = NZ                                                        
         IU = NZ+NW                                                     
                                                                        
         DO 101 I=1,IRANK                                               
                                                                        
         IF(I.LE.IP) THEN                                               
            QI   = 'Z'                                                  
            NUMI = I                                                    
         ENDIF                                                          
                                                                        
         IF(I.GT.IP .AND. I.LE.IU) THEN                                 
            QI   = 'U'                                                  
            NUMI = I-NZ                                                 
         ENDIF                                                          
                                                                        
         IF(I.GT.IU) THEN                                               
            QI   = 'V'                                                  
            NUMI = I-IU                                                 
         ENDIF                                                          
                                                                        
         DO 101 J=1,I                                                   
                                                                        
         IF(J.LE.IP) THEN                                               
            QJ   = 'Z'                                                  
            NUMJ = J                                                    
         ENDIF                                                          
                                                                        
         IF(J.GT.IP .AND. J.LE.IU) THEN                                 
            QJ   = 'U'                                                  
            NUMJ = J-NZ                                                 
         ENDIF                                                          
                                                                        
         IF(J.GT.IU) THEN                                               
            QJ   = 'V'                                                  
            NUMJ = J-IU                                                 
         ENDIF                                                          
                                                                        
         IJ = I*(I-1)/2 + J                                             
                                                                        
         PRINT 911,QI,NUMI,QJ,NUMJ,IJ,SYMAT(IJ)                         
 911     FORMAT(' ',A2,I4,A5,I4,I10,G18.7,G18.7)                        
 101  CONTINUE                                                          
                                                                        
         PRINT*, ' PRINTOUT OF FORCING TERMS '                          
                                                                        
      DO 112 j = 1,1
         print*,' Mini volume:', j                                      
         PRINT 913,(FORCE(I,j,1),I=1,IRANK)                             
 112  continue                                                          
                                                                        
      DO 113 j = 1,1
         print*,' Mini volume:', j                                      
         PRINT 913,(FORCE(I,j,2),I=1,IRANK)                             
 113  CONTINUE                                                          
      DO 114 j = 1,1
         print*,' Mini volume:', j                                      
         PRINT 913,(FORCE(I,j,3),I=1,IRANK)                             
 114  CONTINUE                                                          
913      FORMAT(10G12.4)                                                
                                                                        
      END IF                                                            
                                                                        
      RETURN                                                            
      END                                                               
