      SUBROUTINE GCMDRV (KBC,N,NYMD,NHMS,SCHEME,NDT,ALPHA,QBEG,QBUDGET)         
C***********************************************************************        
C  PURPOSE                                                                      
C     Driver for the C-Grid GEOS GCM ( Dynamics and Physics )          
C  USAGE                                                                        
C     Called by User's Application                                              
C  ARGUMENTS   DESCRIPTION                                                      
C     KBC ...... Unit to read Boundary Conditions                               
C     N ........ Current Time Index                                             
C     NYMD ..... Current YYMMDD                                                 
C     NHMS ..... Current HHMMSS                                                 
C     SCHEME ... Character Description of Time-Scheme (MATS or LEAP)            
C     NDT ...... Time-Step in seconds                                           
C     ALPHA .... Time Filter Coefficient                                        
C     QBEG ..... Logical Flag for BEGIN cases (Cold Starts)                     
C     QBUDGET .. Logical Flag to calculate BUDGETS                              
C  SUBPROGRAMS NEEDED                                                           
C     SETMET     Initializes metric terms                                       
C     SETDMP     Initializes damping coeffs for high-latitude filter            
C     GETBCS     Updates GCM Boundary Conditions                                
C     MOISTIO    Driver for Moist Processes                                     
C     SWRIO      Driver for Shortwave Radiation                                 
C     LWRIO      Driver for Longwave  Radiation                                 
C     TURBIO     Driver for Turbulence Processes                                
C     DYCORE     Driver for Hydro-Dynamics  ARIES-GEOS Dynamical Core           
C     STEP       Updates Prognostic Fields One Time Step                        
C     TICK       Updates time counters                                          
C     PCHECK     Checks for pressure diagnostics                                
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
                                                                                
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <vphyscom.com>                                                         
#include <vmetric.com>                                                          
#include <iau.com>                                                          
                                                                                
      PARAMETER ( IMJMR    = IMR*(JMR+1) )                                       
      PARAMETER ( IMJMM1R  = IMR* JMR    )                                       
      PARAMETER ( IMJMM2R  = IMR*(JMR-1) )                                       
      PARAMETER ( KM       = 1           )                                       
      PARAMETER ( ONE      = 1.0 )                                               
      PARAMETER ( ZERO     = 0.0 )                                               
      PARAMETER ( ZP01     = 0.01)                                               
      PARAMETER ( ROTATION = 86164.09 )                                          
                                                                                
      PARAMETER ( NDLWR   = 030000 )                                            
      PARAMETER ( NDSWR   = 030000 )                                            
      PARAMETER ( NDTURB  = 003000 )                                            
 
      PARAMETER ( SECDAY  = 86400. )                                            
                                                                                
      CHARACTER*4 SCHEME, ZLEAP, ZMATS                                          
      LOGICAL     FIRST,  QBUDGET, QBEG, QLWR, QSWR, QMOIST, QTURB              
      LOGICAL     QHYDRO, LOMEGA
      DATA        FIRST /.TRUE./                                                
      DATA        ZLEAP /'LEAP'/                                                
      DATA        ZMATS /'MATS'/                                                
                                                                                
                                                                                
      common /space/  dpdt(imr,jmr+1)
      common /space/  dudt(imr,jmr+1,nlayr)
      common /space/  dvdt(imr,jmr+1,nlayr)
      common /space/  dtdt(imr,jmr+1,nlayr)
      common /space/  dqdt(imr,jmr+1,nlayr)
      common /space/   pkz(imr,jmr+1,nlayr)                                          
      
c Local Dynamic Arrays
c --------------------
      dimension uztmp(imr,jmr+1,nlayr)                                          
      dimension vztmp(imr,jmr+1,nlayr)                                          
      dimension pkhat(imr,jmr+1,nlayr+1)                                          

      dimension  tmp1(imr,jmr+1)                                          
      dimension  tmp2(imr,jmr+1)                                          
      dimension  tmp3(imr,jmr+1)                                          
      dimension  tmp4(imr,jmr+1)                                          
      dimension  qbar(imr,jmr+1)                                          

      dimension vintuq(imr,jmr+1)
      dimension vintvq(imr,jmr+1)
      dimension vintut(imr,jmr+1)
      dimension vintvt(imr,jmr+1)
                                                                        
      equivalence ( tmp1(1,1), uztmp(1,1,1) )
      equivalence ( tmp2(1,1), uztmp(1,1,2) )
      equivalence ( tmp3(1,1), uztmp(1,1,3) )
      equivalence ( tmp4(1,1), uztmp(1,1,4) )
      equivalence ( qbar(1,1), uztmp(1,1,5) )

      equivalence ( vintuq(1,1), uztmp(1,1,6) )
      equivalence ( vintvq(1,1), uztmp(1,1,7) )
      equivalence ( vintut(1,1), uztmp(1,1,8) )
      equivalence ( vintvt(1,1), uztmp(1,1,9) )

      save itcpu, itclk, ltcpu, ltclk
                                                                          
      NXTMOD(N,M) = MOD(N,M) + 1                                                
      MPER  (N  ) = MODHMS(NHMS,N)                                              
      NHMSF (N  ) = N/3600*10000 + MOD(N,3600 )/ 60*100 + MOD(N, 60)            
                                                                                
C **********************************************************************        
C ****                     MODEL INITIALIZATION                     ****        
C **********************************************************************        
                                                                                
      IF (FIRST) THEN                                                           
      CALL    CLOCKS (ITCPU,ITCLK)                                              
      LTCPU = ITCPU                                                             
      LTCLK = ITCLK                                                             
      CALL    SETMET                                                            
      FIRST = .FALSE.                                                           
      ENDIF                                                                     

      GRAV  = GETCON('GRAVITY')                                             
      AKAP  = GETCON('KAPPA')                                             
      CP    = GETCON('CP')                                             
      AE    = GETCON('EARTH RADIUS')                                             
      RGAS  = GETCON('RGAS')                                             
       OM   = 2.*PI/ROTATION
       P0K  = 1000.**AKAP
      AP0K  = 1./P0K

      J0     = 1
      LOMEGA = IOMEGA.NE.0
                                                                                
C **********************************************************************        
C ****                UPDATE GCM BOUNDARY CONDITIONS                ****        
C **********************************************************************        
                                                                                
      IF(KBC.GT.0) THEN                                                         
      CALL GETBCS (KBC, NYMD, NHMS, TGZ, PHISZ, TVAZ, RADSWT,                   
     .             SSTZ, ALBZ, GWZ, THKZ, SRZ, ZLWI, IMJM)                      
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                   COMPUTE PHYSICS FLAGS                      ****        
C **********************************************************************        
     
      if( ndt.ne.0 ) then
      ndmoist = int( 600./iabs(ndt) + 0.5 ) * iabs(ndt)
      else
      ndmoist = 2*ndt
      endif                                     
      ndmoist = nhmsf( ndmoist )                                                 

      ndhydro = nhmsf( ndt )
                                                                                
      QMOIST = NDMOIST .NE.0 .AND. MPER(NDMOIST) .EQ.0                            
      QSWR   = NDSWR   .NE.0 .AND. MPER(NDSWR)   .EQ.0                            
      QLWR   = NDLWR   .NE.0 .AND. MPER(NDLWR)   .EQ.0                            
      QTURB  = NDTURB  .NE.0 .AND. MPER(NDTURB)  .EQ.0                            
      QHYDRO = NDHYDRO .NE.0 .AND. MPER(NDHYDRO) .EQ.0                            
                                                                                
C **********************************************************************        
C ****                  CALL PHYSICS MINI-DRIVERS                   ****        
C **********************************************************************        
                                                                                
      if( qmoist.or.qswr.or.qlwr.or.qturb ) then                                
      call ctoa ( pz(1,1,  n), pz(1,1,  n), imr,jmr+1,1    ,0 )                 
      call ctoa ( tz(1,1,1,n), tz(1,1,1,n), imr,jmr+1,nlayr,0 )                 
      call ctoa ( qz(1,1,1,n), qz(1,1,1,n), imr,jmr+1,nlayr,0 )                 
      call ctoa ( pkht(1,1,1), pkht(1,1,1), imr,jmr+1,nlayr,0 )                 
      endif                                                                     
                                                                                
C MOIST PROCESSES                                                               
C ===============                                                               
      IF ( QMOIST ) THEN                                                        
      CALL MOISTIO (N  ,NDMOIST,QBUDGET)                                        
      ENDIF                                                                     
                                                                                
C SHORT WAVE RADIATION                                                          
C ====================                                                          
      IF ( QSWR ) THEN                                                          
      CALL SWRIO (N,NYMD,NHMS,NDSWR,QBUDGET)                                    
      ENDIF                                                                     
                                                                                
C LONG WAVE RADIATION                                                           
C ===================                                                           
      IF ( QLWR ) THEN                                                          
      CALL LWRIO (N,NYMD,NHMS,NDLWR,QBUDGET)                                    
      ENDIF                                                                     
                                                                                
C TURBULENCE                                                                    
C ==========                                                                    
      IF ( QTURB ) THEN                                                         
      do i=1,imr*(jmr+1)*nlayr                                                  
      uztmp(i,1,1) = uz(i,1,1,n)                                                
      vztmp(i,1,1) = vz(i,1,1,n)                                                
      enddo                                                                     
      do l=1,nlayr                                                              
      call polewnd ( uz(1,1,L,n),vz(1,1,L,n),                                   
     .               uz(1,1,L,n),vz(1,1,L,n),imr,jmr+1)                         
      enddo                                                                     
      call ctoa ( uz(1,1,1,n), uz(1,1,1,n),imr,jmr+1,nlayr,1 )                  
      call ctoa ( vz(1,1,1,n), vz(1,1,1,n),imr,jmr+1,nlayr,2 )                  
                                                                                
      CALL TURBIO (N,NYMD,NHMS,NDTURB,QBEG,QBUDGET,IMR,JMR+1,NLAYR)             
                                                                                
      do L=1,nlayr                                                              
      do i=1,imr*(jmr+1)                                                        
      uz(i,1,L,n) = uztmp(i,1,L)                                                
      vz(i,1,L,n) = vztmp(i,1,L)                                                
      duturb(i,1,L) = duturb(i,1,L)/pz(i,1,n)                                   
      dvturb(i,1,L) = dvturb(i,1,L)/pz(i,1,n)                                   
      enddo                                                                     
      enddo                                                                     
      call atoc ( duturb (1,1,1)  ,duturb (1,1,1)  ,imr,jmr+1,nlayr,1 )         
      call atoc ( dvturb (1,1,1)  ,dvturb (1,1,1)  ,imr,jmr+1,nlayr,2 )         
      endif                                                                     
                                                                                
      if( qmoist.or.qswr.or.qlwr.or.qturb ) then                                
      call atoc ( pz(1,1,  n), pz(1,1,  n), imr,jmr+1,1    ,0 )                 
      call atoc ( tz(1,1,1,n), tz(1,1,1,n), imr,jmr+1,nlayr,0 )                 
      call atoc ( qz(1,1,1,n), qz(1,1,1,n), imr,jmr+1,nlayr,0 )                 
      call atoc ( pkht(1,1,1), pkht(1,1,1), imr,jmr+1,nlayr,0 )                 
      endif                                                                     
                                                                                
C **********************************************************************        
C ****                     COMPUTE DIAGNOSTICS                      ****        
C **********************************************************************        
                                                                                
      CALL pkappc ( pz(1,1,n),pkht,pkz,imr,jmr+1,nlayr )                        
                                                                                
c Total Diabatic U-Tendency
c -------------------------
      if( idiabu.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idiabu+L-1) =   qdiag  (i,2,idiabu+L-1) 
     .                      + ( duanal (i,2,L) 
     .                        + duturb (i,2,L) 
     .                        + dumoist(i,2,L) )*86400
      enddo
      enddo
      endif
                                                                  
c Total Diabatic V-Tendency
c -------------------------
      if( idiabv.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idiabv+L-1) =   qdiag  (i,2,idiabv+L-1)
     .                      + ( dvanal (i,2,L) 
     .                        + dvturb (i,2,L) 
     .                        + dvmoist(i,2,L) )*86400
      enddo
      enddo
      endif
                                                                  
      if( idiabt.ne.0 .or. idiabq.ne.0 ) then
      do i=1,imjmm2r                                                    
      tmp1(i,2) = 1.0 / pz(i,2,n)                                    
      enddo                                                             
      endif

c Total Diabatic T-Tendency
c -------------------------
      if( idiabt.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idiabt+L-1) =   qdiag  (i,2,idiabt+L-1) 
     .                      + ( dtanal (i,2,L) 
     .                        + dtturb (i,2,L) 
     .                        + dtradlw(i,2,L)
     .                        + dtradsw(i,2,L) * radswt(i,2) 
     .                        + dtmoist(i,2,L) )
     .                        *     pkz(i,2,L) * tmp1(i,2)*86400
      enddo
      enddo
      endif
                                                                  
c Total Diabatic Q-Tendency
c -------------------------
      if( idiabq.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idiabq+L-1) =   qdiag  (i,2,idiabq+L-1) 
     .                      + ( DQANAL (i,2,L) 
     .                        + DQTURB (i,2,L) 
     .                        + DQMOIST(i,2,L) )
     .                        *  tmp1(i,2)*86400*1000
      enddo
      enddo
      endif
    
c DPANAL Analysis Increment
c -------------------------
      if( ianalp.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ianalp) = qdiag(i,2,ianalp) + dpanal(i,2)
      enddo
      endif
                                                                  
c DUANAL Analysis Increment
c -------------------------
      if( ianalu.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ianalu+L-1) = qdiag(i,2,ianalu+L-1)+duanal(i,2,L)
      enddo
      enddo
      endif
                                                                  
c DVANAL Analysis Increment
c -------------------------
      if( ianalv.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ianalv+L-1) = qdiag(i,2,ianalv+L-1)+dvanal(i,2,L)
      enddo
      enddo
      endif
                                                                  
c DTANAL Analysis Increment
c -------------------------
      if( ianalt.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ianalt+L-1) = qdiag(i,2,ianalt+L-1)+dtanal(i,2,L)
      enddo
      enddo
      endif
                                                                  
c DQANAL Analysis Increment
c -------------------------
      if( ianalq.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ianalq+L-1) = qdiag(i,2,ianalq+L-1)+dqanal(i,2,L)
      enddo
      enddo
      endif
    
c Incident Solar Radiation
c ------------------------
      IF (IRADSWT.NE.0) THEN                                                    
      DO    I=1,IMJMM2R                                                         
      QDIAG(I,2,IRADSWT) = QDIAG(I,2,IRADSWT) + RADSWT(I,2)                     
      ENDDO                                                                     
      ENDIF                                                                     
                                                                                
c Net Solar Radiation at the Ground
c ---------------------------------
      IF (IRADSWG.NE.0) THEN                                                    
      DO    I=1,IMJMM2R                                                         
      QDIAG(I,2,IRADSWG) = QDIAG(I,2,IRADSWG) + RADSWG(I,2)*RADSWT(I,2)         
      ENDDO                                                                     
      ENDIF                                                                     
                                                                                
c Solar Radiative Heating
c -----------------------
      IF (IRADSW.NE.0) THEN                                                     
      DO    L=1,NLAYR                                                           
      DO    I=1,IMJMM2R                                                         
      QDIAG(I,2,IRADSW+L-1) =   QDIAG(I,2,IRADSW+L-1)                           
     .                      + DTRADSW(I,2,L)*RADSWT(I,2)                        
     .                          * PKZ(I,2,L)/PZ(I,2,N)*SECDAY                   
      ENDDO                                                                     
      ENDDO                                                                     
      ENDIF                                                                     
                                                                                
      NDIABU  = NDIABU  + 1
      NDIABV  = NDIABV  + 1
      NDIABT  = NDIABT  + 1
      NDIABQ  = NDIABQ  + 1
      NANALP  = NANALP  + 1
      NANALU  = NANALU  + 1
      NANALV  = NANALV  + 1
      NANALT  = NANALT  + 1
      NANALQ  = NANALQ  + 1
      NRADSWT = NRADSWT + 1                                                     
      NRADSWG = NRADSWG + 1                                                     
      NRADSW  = NRADSW  + 1                                                     
                                                                                
C **********************************************************************        
C ****                      FILL PKHAT FOR DYCORE                   ****        
C **********************************************************************        
                                                                                
      if( j0.eq.1 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      pkhat(i,2,L) = pkht(i,2,L)
      enddo
      enddo
      endif

      if( j0.eq.2 ) then
      do i=1,imjmm2r
      pkhat(i,2,1) = ptop**akap * ap0k
      enddo
      do L=1,nlayr
      do i=1,imjmm2r
      pkhat(i,2,L+1) = pkht(i,2,L)*ap0k
      enddo
      enddo
      endif

C **********************************************************************        
C ****                       MATSUNO TIME SCHEME                    ****        
C **********************************************************************        
                                                                                
      IF(SCHEME.EQ.ZMATS) THEN                                                  
      NM1   = NXTMOD(N,2)                                                       
      NP1   = NXTMOD(N,2)                                                       
    
      timestep = ndt
                                                                        
c Predictor
c ---------
      do I=1,IMJMM2R                                                    
      DPDT(I,2) =    DPANAL(I,2)                                          
      enddo                                                             
      do L=1,NLAYR
      do I=1,IMJMM2R
      DUDT(I,2,L) =  DUANAL(I,2,L) + DUTURB(I,2,L) + DUMOIST(I,2,L)
      DVDT(I,2,L) =  DVANAL(I,2,L) + DVTURB(I,2,L) + DVMOIST(I,2,L)
      DTDT(I,2,L) =  DTANAL(I,2,L) + DTTURB(I,2,L) + DTRADLW(I,2,L)
     .            + DTRADSW(I,2,L) * RADSWT(I,2  ) + DTMOIST(I,2,L)
      DQDT(I,2,L) =  DQANAL(I,2,L) + DQTURB(I,2,L) + DQMOIST(I,2,L)
      enddo
      enddo

      call shapij ( PZ(1,1,N),UZ(1,1,1,N),VZ(1,1,1,N),
     .                        TZ(1,1,1,N),QZ(1,1,1,N),
     .                        dudt,dvdt,dtdt,dqdt,
     .                        imr,jmr+1,nlayr )
    
c Ensure non-reversable effects are positive
c ------------------------------------------
      IF ( NDT.LT.0 ) THEN                                              
      do I=1,IMJMM2R                                                    
      DPDT(I,2)   = - DPDT(I,2)                                       
      enddo                                                             
      do L=1,NLAYR                                                      
      do I=1,IMJMM2R                                                    
      DUDT(I,2,L) = - DUDT(I,2,L)                                       
      DVDT(I,2,L) = - DVDT(I,2,L)                                       
      DTDT(I,2,L) = - DTDT(I,2,L)                                       
      DQDT(I,2,L) = - DQDT(I,2,L)                                       
      enddo                                                             
      enddo                                                             
      ENDIF                                                             
                                                                        
      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,n)   = tz(i,2,L,n)  *p0k
      dtdt(i,2,L)   = dtdt(i,2,L)  *p0k
      enddo
      enddo
      endif
      call dycore ( imr,jmr,jmr+1,nlayr,sige,ptop,km,timestep,
     .              om,cp,rgas,ae,
     .           phisz(1,j0),  pkhat(1,j0,1),
     .              pz(1,j0,n),  uz(1,j0,1,n), vz(1,1, 1,n),
     .                           tz(1,j0,1,n), qz(1,j0,1,n),
     .              pz(1,j0,n),  uz(1,j0,1,n), vz(1,1, 1,n),
     .                           tz(1,j0,1,n), qz(1,j0,1,n),
     .              dpdt(1,j0),dudt(1,j0,1), dvdt(1,1 ,1),
     .              dtdt(1,j0,1),dqdt(1,j0,1), 
     .              uztmp(1,j0,1),vztmp(1,1,1),Lomega )

c Add Analysis Increment to Dynamical Omega Diagnostics
c -----------------------------------------------------
      if( iomega.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iomega+L-1) = qdiag(i,2,iomega+L-1) 
     .                      + uztmp(i,2,L)
     .                      + sige(L+1)*dpanal(i,2)*86400
      enddo
      enddo
      endif
      if( ivort.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm1r
      qdiag(i,1,ivort+L-1) = qdiag(i,1,ivort+L-1) 
     .                     + vztmp(i,1,L)
      enddo
      enddo
      endif
      NVORT  = NVORT  + 1
      NOMEGA = NOMEGA + 1

      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,n)   = tz(i,2,L,n)  *ap0k
      dtdt(i,2,L)   = dtdt(i,2,L)  *ap0k
      enddo
      enddo
      endif
      call step ( n,n,np1,ndt,zero,imr,jmr+1,nlayr,
     .            dpdt,dudt,dvdt,dtdt,dqdt,pkz,qbudget )                  

c Update PKHAT
c ------------
      if( j0.eq.1 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      pkhat(i,2,L) = pkht(i,2,L)
      enddo
      enddo
      endif

      if( j0.eq.2 ) then
      do i=1,imjmm2r
      pkhat(i,2,1) = ptop**akap * ap0k
      enddo
      do L=1,nlayr
      do i=1,imjmm2r
      pkhat(i,2,L+1) = pkht(i,2,L)*ap0k
      enddo
      enddo
      endif

c Corrector
c ---------
      do I=1,IMJMM2R                                                    
      DPDT(I,2) =    DPANAL(I,2)                                          
      enddo                                                             
      do L=1,NLAYR
      do I=1,IMJMM2R
      DUDT(I,2,L) =  DUANAL(I,2,L) + DUTURB(I,2,L) + DUMOIST(I,2,L)
      DVDT(I,2,L) =  DVANAL(I,2,L) + DVTURB(I,2,L) + DVMOIST(I,2,L)
      DTDT(I,2,L) =  DTANAL(I,2,L) + DTTURB(I,2,L) + DTRADLW(I,2,L)
     .            + DTRADSW(I,2,L) * RADSWT(I,2  ) + DTMOIST(I,2,L)
      DQDT(I,2,L) =  DQANAL(I,2,L) + DQTURB(I,2,L) + DQMOIST(I,2,L)
      enddo
      enddo
    
      call shapij ( PZ(1,1,N),UZ(1,1,1,N),VZ(1,1,1,N),
     .                        TZ(1,1,1,N),QZ(1,1,1,N),
     .                        dudt,dvdt,dtdt,dqdt,
     .                        imr,jmr+1,nlayr )
    
c Ensure non-reversable effects are positive
c ------------------------------------------
      IF ( NDT.LT.0 ) THEN                                              
      do I=1,IMJMM2R                                                    
      DPDT(I,2)   = - DPDT(I,2)                                       
      enddo                                                             
      do L=1,NLAYR                                                      
      do I=1,IMJMM2R                                                    
      DUDT(I,2,L) = - DUDT(I,2,L)                                       
      DVDT(I,2,L) = - DVDT(I,2,L)                                       
      DTDT(I,2,L) = - DTDT(I,2,L)                                       
      DQDT(I,2,L) = - DQDT(I,2,L)                                       
      enddo                                                             
      enddo                                                             
      ENDIF                                                             
                                                                        
      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,np1) = tz(i,2,L,np1)*p0k
      dtdt(i,2,L  ) = dtdt(i,2,L  )*p0k
      enddo
      enddo
      endif
      call dycore ( imr,jmr,jmr+1,nlayr,sige,ptop,km,timestep,
     .              om,cp,rgas,ae,
     .           phisz(1,j0),    pkhat(1,j0,1),
     .              pz(1,j0,np1),  uz(1,j0,1,np1), vz(1,1, 1,np1),
     .                             tz(1,j0,1,np1), qz(1,j0,1,np1),
     .              pz(1,j0,np1),  uz(1,j0,1,np1), vz(1,1, 1,np1),
     .                             tz(1,j0,1,np1), qz(1,j0,1,np1),
     .              dpdt(1,j0),  dudt(1,j0,1),   dvdt(1,1 ,1),
     .              dtdt(1,j0,1),dqdt(1,j0,1),
     .              uztmp(1,j0,1),vztmp(1,1,1),Lomega )

c Add Analysis Increment to Dynamical Omega Diagnostics
c -----------------------------------------------------
      if( iomega.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iomega+L-1) = qdiag(i,2,iomega+L-1) 
     .                      + uztmp(i,2,L)
     .                      + sige(L+1)*dpanal(i,2)*86400
      enddo
      enddo
      endif
      if( ivort.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm1r
      qdiag(i,1,ivort+L-1) = qdiag(i,1,ivort+L-1) 
     .                     + vztmp(i,1,L)
      enddo
      enddo
      endif
      NVORT  = NVORT  + 1
      NOMEGA = NOMEGA + 1

      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,np1) = tz(i,2,L,np1)*ap0k
      dtdt(i,2,L  ) = dtdt(i,2,L  )*ap0k
      enddo
      enddo
      endif
      call step ( n,np1,np1,ndt,zero,imr,jmr+1,nlayr,
     .            dpdt,dudt,dvdt,dtdt,dqdt,pkz,qbudget )                  
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                   LEAPFROG TIME SCHEME                       ****        
C **********************************************************************        
                                                                                
      IF(SCHEME.EQ.ZLEAP) THEN                                                  
      NM1   = NXTMOD(N,2)                                                       
      NP1   = NM1                                                               

      timestep = ndt

      do I=1,IMJMM2R                                                    
      DPDT(I,2) =    DPANAL(I,2)                                          
      enddo                                                             
      do L=1,NLAYR
      do I=1,IMJMM2R
      DUDT(I,2,L) =  DUANAL(I,2,L) + DUTURB(I,2,L) + DUMOIST(I,2,L)
      DVDT(I,2,L) =  DVANAL(I,2,L) + DVTURB(I,2,L) + DVMOIST(I,2,L)
      DTDT(I,2,L) =  DTANAL(I,2,L) + DTTURB(I,2,L) + DTRADLW(I,2,L)
     .            + DTRADSW(I,2,L) * RADSWT(I,2  ) + DTMOIST(I,2,L)
      DQDT(I,2,L) =  DQANAL(I,2,L) + DQTURB(I,2,L) + DQMOIST(I,2,L)
      enddo
      enddo
    
      call shapij ( PZ(1,1,NM1),UZ(1,1,1,NM1),VZ(1,1,1,NM1),
     .                          TZ(1,1,1,NM1),QZ(1,1,1,NM1),
     .                          dudt,dvdt,dtdt,dqdt,
     .                          imr,jmr+1,nlayr )
    
c Ensure non-reversable effects are positive
c ------------------------------------------
      IF ( NDT.LT.0 ) THEN                                              
      do I=1,IMJMM2R                                                    
      DPDT(I,2)   = - DPDT(I,2)                                       
      enddo                                                             
      do L=1,NLAYR                                                      
      do I=1,IMJMM2R                                                    
      DUDT(I,2,L) = - DUDT(I,2,L)                                       
      DVDT(I,2,L) = - DVDT(I,2,L)                                       
      DTDT(I,2,L) = - DTDT(I,2,L)                                       
      DQDT(I,2,L) = - DQDT(I,2,L)                                       
      enddo                                                             
      enddo                                                             
      ENDIF                                                             
                                                                        
      if( j0.eq.2 ) then
      timestep = timestep*2
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,n)   = tz(i,2,L,n)  *p0k
      tz(i,2,L,nm1) = tz(i,2,L,nm1)*p0k
      dtdt(i,2,L)   = dtdt(i,2,L)  *p0k
      enddo
      enddo
      endif
      call dycore ( imr,jmr,jmr+1,nlayr,sige,ptop,km,timestep,
     .              om,cp,rgas,ae,
     .           phisz(1,j0),    pkhat(1,j0,1),
     .              pz(1,j0,n  ),  uz(1,j0,1,n  ), vz(1,1, 1,n  ),
     .                             tz(1,j0,1,n  ), qz(1,j0,1,n  ),
     .              pz(1,j0,nm1),  uz(1,j0,1,nm1), vz(1,1, 1,nm1),
     .                             tz(1,j0,1,nm1), qz(1,j0,1,nm1),
     .              dpdt(1,j0),  dudt(1,j0,1),   dvdt(1,1 ,1),
     .              dtdt(1,j0,1),dqdt(1,j0,1),
     .              uztmp(1,j0,1),vztmp(1,1,1),Lomega )

c Add Analysis Increment to Dynamical Omega Diagnostics
c -----------------------------------------------------
      if( iomega.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iomega+L-1) = qdiag(i,2,iomega+L-1) 
     .                      + uztmp(i,2,L)
     .                      + sige(L+1)*dpanal(i,2)*86400
      enddo
      enddo
      endif
      if( ivort.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm1r
      qdiag(i,1,ivort+L-1) = qdiag(i,1,ivort+L-1) 
     .                     + vztmp(i,1,L)
      enddo
      enddo
      endif
      NVORT  = NVORT  + 1
      NOMEGA = NOMEGA + 1

      if( j0.eq.2 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      tz(i,2,L,n)   = tz(i,2,L,n)  *ap0k
      tz(i,2,L,nm1) = tz(i,2,L,nm1)*ap0k
      dtdt(i,2,L)   = dtdt(i,2,L)  *ap0k
      enddo
      enddo
      endif
      call step ( nm1,n,np1,2*ndt,alpha,imr,jmr+1,nlayr,
     .            dpdt,dudt,dvdt,dtdt,dqdt,pkz,qbudget )               
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                      TICK THE CLOCK                          ****        
C **********************************************************************        
                                                                                
      CALL TICK (NYMD,NHMS,NDT,N)                                               
                                                                                
      CALL CLOCKS     (NTCPU,NTCLK)                                             
      TCPU   = ZP01 * (ITCPU-NTCPU)                                             
      TCLK   = ZP01 * (ITCLK-NTCLK)                                             
      SCPU   = ZP01 * (LTCPU-NTCPU)                                             
      SCLK   = ZP01 * (LTCLK-NTCLK)                                             
      TRATIO = TCPU / TCLK                                                      
      SRATIO = SCPU / SCLK                                                      
      LTCPU  = NTCPU                                                            
      LTCLK  = NTCLK                                                            
      DELTAP = GMEAN( DPANAL,IMR,JMR+1 )                               

      WRITE (6,1000) NYMD, NHMS,                                                
     .               SCPU, TCPU, SCLK,TCLK,SRATIO,TRATIO, DELTAP                
                                                                                
C **********************************************************************        
C ****                  COMPUTE MORE DIAGNOSTICS                    ****        
C **********************************************************************        

      CALL pkappc ( pz(1,1,n),pkht,pkz,imr,jmr+1,nlayr )                        
                                                                                
c Averaged P-Field
c ----------------
      if( ipave.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ipave) = qdiag(i,2,ipave) + pz(i,2,n)
      enddo
      endif
                                                                                
c Averaged U-Field
c ----------------
      if( iuave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iuave+L-1) = qdiag(i,2,iuave+L-1) + uz(i,2,L,n)
      enddo
      enddo
      endif
                                                                                
c Averaged V-Field
c ----------------
      if( ivave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,ivave+L-1) = qdiag(i,2,ivave+L-1) + vz(i,2,L,n)
      enddo
      enddo
      endif
                                                                                
c Averaged T-Field
c ----------------
      if( itave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,itave+L-1) = qdiag(i,2,itave+L-1) 
     .                     +    tz(i,2,L,n)*pkz(i,2,L)
      enddo
      enddo
      endif
                                                                                
c Averaged Q-Field
c ----------------
      if( iqave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iqave+L-1) = qdiag(i,2,iqave+L-1) + qz(i,2,L,n)
      enddo
      enddo
      endif
                                                                                
c Averaged QQ-Field
c -----------------
      if( iqqave.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iqqave+L-1) = qdiag(i,2,iqqave+L-1) + qqz(i,2,L)
      enddo
      enddo
      endif
                                                                                
c Precipitable Water
c ------------------
      if( iqint.ne.0 ) then
      do i=1,imjmm2r
      qbar(i,2) = 0.0
      enddo
      do L=1,nlayr
      do i=1,imjmm2r
      qbar(i,2) = qbar(i,2) + qz(i,2,L,n)*dsig(L)
      enddo
      enddo
      do i=1,imjmm2r
      qbar(i,2) = qbar(i,2) * 10 * pz(i,1,n) / grav
      enddo
      do i=1,imjmm2r
      qdiag(i,2,iqint) = qdiag(i,2,iqint) + qbar(i,2)
      enddo
      endif
                                                                                
c Compute Temperature and Moisture Convergence Diagnostics
c --------------------------------------------------------
      if( ivintuq.ne.0 .or. ivintvq.ne.0 .or. 
     .    ivintut.ne.0 .or. ivintvt.ne.0 ) then
      do i=1,imjmm2r
      vintuq(i,2) = 0.0
      vintvq(i,2) = 0.0
      vintut(i,2) = 0.0
      vintvt(i,2) = 0.0
      enddo
 
      do L=1,nlayr
      call qbaru ( uz(1,1,L,n),vz(1,1,L,n),
     .             tmp1,tmp2,tmp3,tmp4,imr,jmr+1 )
      do i=1,imjmm2r
      vintuq(i,2) = vintuq(i,2) + tmp1(i,2)*qz(i,2,L,n)*dsig(L)
      vintvq(i,2) = vintvq(i,2) + tmp2(i,2)*qz(i,2,L,n)*dsig(L)
      vintut(i,2) = vintut(i,2) + tmp1(i,2)*tz(i,2,L,n)*dsig(L)
     .                          * pkz (i,2,L)
      vintvt(i,2) = vintvt(i,2) + tmp2(i,2)*tz(i,2,L,n)*dsig(L)
     .                          * pkz (i,2,L)
      enddo
      enddo

      if( ivintuq.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ivintuq) = qdiag(i,2,ivintuq) + vintuq(i,2)*1000
      enddo
      endif
      if( ivintvq.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ivintvq) = qdiag(i,2,ivintvq) + vintvq(i,2) *1000
      enddo
      endif
      if( ivintut.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ivintut) = qdiag(i,2,ivintut) + vintut(i,2)
      enddo
      endif
      if( ivintvt.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,ivintvt) = qdiag(i,2,ivintvt) + vintvt(i,2)
      enddo
      endif
      endif
 

      NPAVE   = NPAVE   + 1                           
      NUAVE   = NUAVE   + 1                           
      NVAVE   = NVAVE   + 1                           
      NTAVE   = NTAVE   + 1                           
      NQAVE   = NQAVE   + 1                           
      NQQAVE  = NQQAVE  + 1                           
      NQINT   = NQINT   + 1                           
      NVINTUQ = NVINTUQ + 1
      NVINTVQ = NVINTVQ + 1
      NVINTUT = NVINTUT + 1
      NVINTVT = NVINTVT + 1
                                                                                
C **********************************************************************        
C ****                CHECK FOR PRESSURE DIAGNOSTIC                 ****        
C **********************************************************************        
                                                                                
      CALL PCHECK (N)                                                           
                                                                                
 1000 FORMAT(1X,'Date: ',I6,2X,                                                 
     .          'Time: ',I6,2X,                                                 
     .          ' CPU: ',F6.2,F8.2,2X,                                          
     .          ' CLK: ',F6.2,F8.2,2X,                                          
     .       'CPU/CLK: ',F6.3,F6.3,2X,'DELP: ',F8.5,' mb')                      
      RETURN                                                                    
      END                                                                       
      SUBROUTINE PCHECK (N)                                                     
C***********************************************************************        
C  PURPOSE                                                                      
C     CHECK FOR PRESSURE DIAGNOSTIC                                             
C  ARGUMENTS   DESCRIPTION                                                      
C     N ...... CURRENT TIME INDEX                                               
C  SUBPROGRAMS NEEDED                                                           
C     HARM     PRODUCES HARMONIC ANALYSIS OF PROGNOSTIC FIELDS                  
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
                                                                                
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <vmetric.com>                                                          
                                                                                
      PARAMETER ( IMJMR   = IMR*(JMR+1) )                                       
      PARAMETER ( IMJMM1R = IMR* JMR    )                                       
      PARAMETER ( IMJMM2R = IMR*(JMR-1) )                                       
      PARAMETER ( ONE     =    1.0 )                                            
      PARAMETER ( ZERO    =    0.0 )                                            
      PARAMETER ( PSMIN   =  300.0 )                                            
      PARAMETER ( PSMAX   = 1200.0 )                                            
                                                                                
C **********************************************************************        
C ****                CHECK FOR PRESSURE DIAGNOSTIC                 ****        
C **********************************************************************        
                                                                                
      NOBLOW = 0                                                                
      DO 10 I=1,IMJMM2R                                                         
      IF( PZ(I,2,N).GT.PSMAX-PTOP .OR.                                          
     .    PZ(I,2,N).LT.PSMIN-PTOP ) NOBLOW = NOBLOW + 1                         
   10 CONTINUE                                                                  
      IF( NOBLOW.EQ.0 ) RETURN                                                  
                                                                                
C **********************************************************************        
C ****                 PRINT OUT UNSTABLE FIELDS                    ****        
C **********************************************************************        
                                                                                
      DO 20 J=2,JNP-1                                                           
      DO 20 I=1,IM                                                              
      IF( ( PZ(I,J,N).LE.PSMAX-PTOP )   .AND.                                   
     .    ( PZ(I,J,N).GE.PSMIN-PTOP ) )  GOTO 20                                
      WRITE(6,5000) I,J,PZ(I,J,N)                                               
                                                                                
      CALL HARM ( PZ(1,J,  N),'P   ',0,IM)                                      
      DO 30 L=1,NLAY                                                            
      do ii=1,im                                                                
      qz(ii,j,L,n) = qz(ii,j,L,n)*1000                                          
      enddo                                                                     
      CALL HARM ( UZ(1,J,L,N),'U   ',L,IM)                                      
      CALL HARM ( VZ(1,J,L,N),'V   ',L,IM)                                      
      CALL HARM ( TZ(1,J,L,N),'T   ',L,IM)                                      
      CALL HARM ( QZ(1,J,L,N),'SH  ',L,IM)                                      
   30 CONTINUE                                                                  
      STOP                                                                      
   20 CONTINUE                                                                  
                                                                                
 5000 FORMAT(///1X,'PRESSURE DIAGNOSTIC FOR P(',I3,',',I3,') = ',               
     .          F10.3,//)                                                       
      RETURN                                                                    
      END                                                                       
      SUBROUTINE SETMET                                                         
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     INITIALIZE RESOLUTION DEPENDENT TERMS IN MODEL                            
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
                                                                                
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <vmetric.com>                                                          
                                                                                
      PARAMETER ( IMJMR   = IMR*(JMR+1) )                                       
      PARAMETER ( IMJMM1R = IMR* JMR    )                                       
      PARAMETER ( IMJMM2R = IMR*(JMR-1) )                                       
      PARAMETER ( IMD2R   = IMR/2       )                                       
                                                                                
      PARAMETER ( ZERO   = 0.0 )                                                
      PARAMETER ( AHALF  = 0.5 )                                                
      PARAMETER ( ONE    = 1.0 )                                                
      PARAMETER ( TWO    = 2.0 )                                                
      PARAMETER ( FOUR   = 4.0 )                                                
      PARAMETER ( ROTSEC =   86164.09 )                                         
                                                                                
C *********************************************************                     
C ****            COMPUTE RESOLUTION FACTORS           ****                     
C *********************************************************                     
                                                                                
      IM     = IMR                                                              
      JNP    = JMR+1                                                            
      NLAY   = NLAYR                                                            
                                                                                
      IMH    = IM/2                                                             
      IMM1   = IM-1                                                             
      IMM2   = IM-2                                                             
      IMM3   = IM-3                                                             
      JMM1   = JNP-1                                                            
      JMM2   = JNP-2                                                            
      JMM3   = JNP-3                                                            
      JMM5   = JNP-5                                                            
      IMJM   = IM*JNP                                                           
      IMJMM1 = IM*JMM1                                                          
      IMJMM2 = IM*JMM2                                                          
      IMJMM3 = IM*JMM3                                                          
      NLAYP1 = NLAY+1                                                           
      NLAYM1 = NLAY-1                                                           
                                                                                
C *********************************************************                     
C ****            COMPUTE GRID INCREMENTS              ****                     
C *********************************************************                     
                                                                                
      A    = GETCON('EARTH RADIUS')                                             
      PI   = FOUR * ATAN(ONE)                                                   
      DL   = TWO*PI/IM                                                          
      DP   = PI/JMM1                                                            
      ACAP = TWO*PI *A**2 * (ONE-COS(AHALF*DP))                                 
                                                                                
C *********************************************************                     
C ****        COMPUTE TRIGONOMETRIC FACTORS            ****                     
C *********************************************************                     
                                                                                
      DO 10 I=1,IMD2R                                                           
      COSLON(I)     = -COS((I-1)*DL)                                            
      COSLON(I+IMH) = -COSLON(I)                                                
      SINLON(I)     = -SIN((I-1)*DL)                                            
      SINLON(I+IMH) = -SINLON(I)                                                
   10 CONTINUE                                                                  
                                                                                
      FJEQ = ( JNP+1 )*AHALF                                                    
      DO 20 J=2,JMM1                                                            
           PHI    = DP * (J - FJEQ)                                             
       COSP(J)    =     COS(PHI)                                                
       SINP(J)    =     SIN(PHI)                                                
      ACOSP(J)    = ONE/COSP(J)                                                 
      DO 20 I=1,IMR                                                             
       COSIJ(I,J) =     COSP(J)                                                 
       SINIJ(I,J) =     SINP(J)                                                 
      SINLIJ(I,J) =   SINLON(I)                                                 
      COSLIJ(I,J) =   COSLON(I)                                                 
   20 CONTINUE                                                                  
      DO 30 I=1,IMR                                                             
      SINLIJ(I,1  ) = SINLON(I)                                                 
      COSLIJ(I,1  ) = COSLON(I)                                                 
      SINLIJ(I,JNP) = SINLON(I)                                                 
      COSLIJ(I,JNP) = COSLON(I)                                                 
   30 CONTINUE                                                                  
                                                                                
       SGNP(1)      = -ONE                                                      
       SGNP(2)      =  ONE                                                      
       COSP(1)      = ZERO                                                      
       SINP(1)      = -ONE                                                      
       COSP(JNP)    = ZERO                                                      
       SINP(JNP)    =  ONE                                                      
      DO 40 I=1,IMR                                                             
       SINIJ(I,  1) = -ONE                                                      
       SINIJ(I,JNP) =  ONE                                                      
       COSIJ(I,  1) = ZERO                                                      
       COSIJ(I,JNP) = ZERO                                                      
      ACOSIJ(I,  1) = ZERO                                                      
      ACOSIJ(I,JNP) = ZERO                                                      
   40 CONTINUE                                                                  
                                                                                
                 OMEGA   = TWO*PI / ROTSEC                                      
      DO 50 I=1,IMJMR                                                           
         FIJ(I,1)   = TWO*OMEGA*SINIJ(I,1)                                      
   50 CONTINUE                                                                  
      DO 60 I=1,IMJMM2R                                                         
      ACOSIJ(I,2) = ONE/COSIJ(I,2)                                              
      ATANIJ(I,2) = SINIJ(I,2)*ACOSIJ(I,2)/A                                    
   60 CONTINUE                                                                  
                                                                                
C *********************************************************                     
C ****       COMPUTE VERTICAL GRID INCREMENTS          ****                     
C ****         (NOTE: SIGE IS FROM RESTART)            ****                     
C *********************************************************                     
                                                                                
      DO 70 L=1,NLAYR                                                           
      DSIG(L) =  SIGE(L+1) - SIGE(L)                                            
       SIG(L) = (SIGE(L+1) + SIGE(L))*AHALF                                     
   70 CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE STEP (N1,N2,N3,NDT,ALPHA,IMAX,JMAX,KMAX,
     .                 DPDT,DUDT,DVDT,DTDT,DQDT,PKZ,BUDFLG)       
C***********************************************************************
C  PURPOSE                                                              
C     UPDATES PROGNOSTIC FIELDS ONE TIME-STEP                           
C  USAGE                                                                
C     CALLED BY GCMDRV                                                  
C                                                                       
C  ARGUMENTS   DESCRIPTION                                              
C     N1 ..... Time Index you are coming from                           
C     N2 ..... Time Index you are using  in R.H.S.                      
C     N3 ..... Time Index you are going  to                             
C     NDT .... Total Time Step (Seconds) used ( MATS:DT   LEAP:2DT )    
C     ALPHA .. Time Filter Coefficient                                  
C     IMAX ... Zonal      Dimension
C     JMAX ... Meridional Dimension
C     KMAX ... Vertical   Dimension
C     DPDT ... Total Surface Pressure  Tendency
C     DUDT ... Total Zonal      U-Wind Tendency
C     DVDT ... Total Meridional V-Wind Tendency
C     DTDT ... Total PI*THETA          Tendency
C     DQDT ... Total PI*SPHU           Tendency
C     PKZ .... Work Space for PKZ, etc.
C     BUDFLG . Logical for compubing budgets                            
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
                                                                        
#include <vrslv.com>                                                    
#include <vstate.com>                                                   
#include <vmetric.com>                                                  
#include <vbudget.com>                                                  

      PARAMETER ( ZERO    = 0.0      )                                  
      PARAMETER ( AHALF   = 0.5      )                                  
      PARAMETER ( ONE     = 1.0      )                                  
      PARAMETER ( DPMAX   = 0.003    )                                  
      PARAMETER ( SECDAY  = 86400.   )                                  
      PARAMETER ( CNSTP   = SECDAY       )                              
      PARAMETER ( CNSTU   = SECDAY/1000. )                              
      PARAMETER ( CNSTV   = SECDAY/1000. )                              
      PARAMETER ( CNSTT   = SECDAY/1000.*7.1968 )                       
      PARAMETER ( CNSTQ   = SECDAY       )                              
                                                                        
      PARAMETER ( IMJMR   = IMR*(JMR+1) )                               
      PARAMETER ( IMJMM1R = IMR* JMR    )                               
      PARAMETER ( IMJMM2R = IMR*(JMR-1) )                               
      PARAMETER ( IMJMKMR = IMR*(JMR+1)*NLAYR )                         
                                                                        
      LOGICAL    BUDFLG                                                 
                                                                        
      real dpdt (imax,jmax)                               
      real dudt (imax,jmax,kmax)                         
      real dvdt (imax,jmax,kmax)                         
      real dtdt (imax,jmax,kmax)                         
      real dqdt (imax,jmax,kmax)                         
      real pkz  (imax,jmax,kmax)                         

c Local Dynamic Space
c -------------------
      real tmp1 (imax,jmax)
                                                                        
C *****************************************************************     
C ****               INITIALIZATION                            ****     
C *****************************************************************     
                                                                        
      CP    = GETCON('CP')                                              
      PSTD  = GETCON('PSTD')                                            
      TSTD  = GETCON('TSTD')                                            
      PKSTD = EXPBYK (PSTD)                                             
      THSTD = TSTD/PKSTD                                                
      TMSTP = NDT*AHALF                                                         
      CP    = GETCON('CP')                                               
      AKAP  = GETCON('KAPPA')                                            

      CALL PKAPPC ( PZ(1,1,N2),PKHT,PKZ,IMR,JMR+1,NLAYR )               
                                                                        
C **********************************************************************
C ****           COMPUTE TOTAL TENDENCY DIAGNOSTICS                 ****
C **********************************************************************

      if( idpdt.ne.0 ) then
      do i=1,imjmm2r
      qdiag(i,2,idpdt) = qdiag(i,2,idpdt) + dpdt(i,2)*86400
      enddo
      endif

      if( idudt.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idudt+L-1) = qdiag(i,2,idudt+L-1) + dudt(i,2,L)*86400
      enddo
      enddo
      endif
                                                                  
      if( idvdt.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idvdt+L-1) = qdiag(i,2,idvdt+L-1) + dvdt(i,2,L)*86400
      enddo
      enddo
      endif
                                                                  
      if( idtdt.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
       tmp1(i,2) = sig(L)*pz(i,2,n2) + ptop
      qdiag(i,2,idtdt+L-1) = qdiag(i,2,idtdt+L-1) 
     .                     + ( pkz(i,2,L)*dtdt(i,2,L)/pz(i,2,n2)
     .                     +  tz(i,2,L,n2)*pkz(i,2,L)*dpdt(i,2)
     .         *(sig(L)*akap/tmp1(i,2) - 1./pz(i,2,n2)) )*86400
      enddo
      enddo
      endif
                                                                  
      if( idqdt.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,idqdt+L-1) = qdiag(i,2,idqdt+L-1) 
     .                     + ( dqdt(i,2,L)-qz(i,2,L,n2)*dpdt(i,2) )
     .                     /     pz(i,2,n2)*86400*1000
      enddo
      enddo
      endif
    
C **********************************************************************
C ****                   UPDATE PROGNOSTIC FIELDS                   ****
C ****  Note:                                                       ****
C ****           DPDT gets overwritten and becomes P(NP1)           ****
C ****           DUDT gets overwritten and becomes U(NP1)           ****
C ****           DVDT gets overwritten and becomes V(NP1)           ****
C ****           DTDT gets overwritten and becomes T(NP1)           ****
C ****           DQDT gets overwritten and becomes Q(NP1)           ****
C ****                                                              ****
C **********************************************************************
                                                                        
      do I=1,IMJMM2R                                                    
      DPDT(I,2) = PZ(I,2,N1) + NDT*DPDT(I,2)                             
      enddo                                                             
                                                                        
      do L=1,NLAYR                                                      
      do I=1,IMJMM2R                                                    
      DUDT(I,2,L) =            UZ(I,2,L,N1) + NDT*DUDT(I,2,L)            
      DVDT(I,2,L) =            VZ(I,2,L,N1) + NDT*DVDT(I,2,L)            
      DTDT(I,2,L) = PZ(I,2,N1)*TZ(I,2,L,N1) + NDT*DTDT(I,2,L)            
      DQDT(I,2,L) = PZ(I,2,N1)*QZ(I,2,L,N1) + NDT*DQDT(I,2,L)            
      enddo                                                             
      enddo                                                             
                                                                        
C **********************************************************************
C ****            CHECK GLOBAL MEAN SURFACE PRESSURE                ****
C **********************************************************************
                                                                        
      DELP   = PIMEAN - PMEAN( DPDT,IMR,jmr+1 )                               
      PIMEAN = PIMEAN - DELP
                                                                        
C **********************************************************************
C ****                CHECK FOR NEGATIVE HUMIDITIES                 ****
C **********************************************************************
                                                                        
      if( iqfill.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iqfill+L-1) = qdiag(i,2,iqfill+L-1)
     .                      -  dqdt(i,2,L)/dpdt(i,2)*86400/ndt * 1000
      enddo
      enddo
      endif

      CALL QCHECK ( DQDT,DSIG,IMR,JMR+1,NLAYR,BUDFLG,PKZ )

      if( iqfill.ne.0 ) then
      do L=1,nlayr
      do i=1,imjmm2r
      qdiag(i,2,iqfill+L-1) = qdiag(i,2,iqfill+L-1)
     .                       +  dqdt(i,2,L)/dpdt(i,2)*86400/ndt * 1000
      enddo
      enddo
      endif
                                                                        
C **********************************************************************
C ****                 APPLY ASSELIN TIME FILTER                    ****
C **********************************************************************
                                                                        
      IF(NDT*ALPHA.NE.ZERO) THEN                                        
      CALL TMFILT ( DPDT,DUDT,DVDT,DTDT,DQDT,N1,N2,
     .              IMR,JMR+1,NLAYR,ALPHA )                                 
      ENDIF                                                             
                                                                        
C **********************************************************************
C ****                  UNSCALE FLUX VARIABLES                      ****
C **********************************************************************
                                                                        
      do I=1,IMJMM2R                                                    
      tmp1(I,2)    = ONE / DPDT(I,2)                                      
      PZ  (I,2,N3) =       DPDT(I,2)                                      
      enddo                                                             
                                                                        
      do L=1,NLAYR                                                       
      do I=1,IMJMM2R                                                    
      DTDT(I,2,L) = DTDT(I,2,L) * tmp1(I,2)                             
      DQDT(I,2,L) = DQDT(I,2,L) * tmp1(I,2)                             
      enddo                                                             
      enddo                                                             
                                                                        
C **********************************************************************
C ****                        COPY FIELDS                           ****
C **********************************************************************
                                                                        
      do L=1,NLAYR                                                       
      do I=1,IMJMM2R                                                    
      UZ(I,2,L,N3) =   DUDT(I,2,L)                                       
      VZ(I,2,L,N3) =   DVDT(I,2,L)                                       
      TZ(I,2,L,N3) =   DTDT(I,2,L)                                       
      QZ(I,2,L,N3) =   DQDT(I,2,L)                                       
                                                                        
      PKHT(I,2,L)  = SIGE(L+1)*PZ(I,2,N3) + PTOP                        
      PKHT(I,2,L)  = PKHT(I,2,L) ** AKAP                                
      enddo                                                             
      enddo                                                             
                                                                        
C **********************************************************************
C ****                   BUMP DIAGNOSTIC COUNTERS                   ****
C **********************************************************************
                                                                        
      NDPDT    = NDPDT    + 1                                              
      NDUDT    = NDUDT    + 1                                              
      NDVDT    = NDVDT    + 1                                              
      NDTDT    = NDTDT    + 1                                              
      NDQDT    = NDQDT    + 1                                              
      NQFILL   = NQFILL   + 1                                              
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE TMFILT ( PN3,UN3,VN3,TN3,QN3,N1,N2,                 
     .                    IMAX,JMAX,KMAX,ALPHA )                        
C***********************************************************************
C  PURPOSE                                                              
C     PERFORM ASSELIN TIME FILTER                                       
C  ARGUMENTS                                                            
C     PN3 .... P AT TIME-LEVEL N3                                       
C     UN3 .... U AT TIME-LEVEL N3                                       
C     VN3 .... V AT TIME-LEVEL N3                                       
C     TN3 .... T AT TIME-LEVEL N3                                       
C     QN3 .... Q AT TIME-LEVEL N3                                       
C     N1 ..... TIME INDEX N1                                            
C     N2 ..... TIME INDEX N2                                            
C     ALPHA .. TIME FILTER COEFFICIENT                                  
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
                                                                        
#include <vrslv.com>                                                    
#include <vstate.com>                                                   
                                                                        
      PARAMETER ( ONE    = 1.0 )                                        
      PARAMETER ( AHALF  = 0.5 )                                        
      PARAMETER ( IMJM   = IMR*(JMR+1) )                                
      PARAMETER ( IMJMM1 = IMR* JMR    )                                
      PARAMETER ( IMJMM2 = IMR*(JMR-1) )                                
                                                                        
      DIMENSION  PN3 (IMAX,JMAX)                                        
      DIMENSION  UN3 (IMAX,JMAX,KMAX)                                   
      DIMENSION  VN3 (IMAX,JMAX,KMAX)                                   
      DIMENSION  TN3 (IMAX,JMAX,KMAX)                                   
      DIMENSION  QN3 (IMAX,JMAX,KMAX)                                   
                                                                        
c Local Dynamic Space
c -------------------
      DIMENSION   PH (IMAX,JMAX)                                        
                                                                        
      A1   = ONE   - ALPHA                                              
      A2   = AHALF * ALPHA                                              
                                                                        
C **********************************************************************
C ****           PI-WEIGHT TIME-LEVEL N2 FOR MASS FIELDS            ****
C **********************************************************************
                                                                        
      DO 20 L=1,NLAYR                                                   
                                                                        
      DO 30 I=1,IMJMM2                                                  
      TZ(I,2,L,N2) = PZ(I,2,N2) * TZ(I,2,L,N2)                          
      QZ(I,2,L,N2) = PZ(I,2,N2) * QZ(I,2,L,N2)                          
   30 CONTINUE                                                          
                                                                        
C **********************************************************************
C ****                 APPLY ASSELIN TIME FILTER                    ****
C **********************************************************************
                                                                        
      DO 50 I=1,IMJMM2                                                  
      UZ(I,2,L,N2) = A1 *   UZ(I,2,L,N2)                                
     .             + A2 * ( UZ(I,2,L,N1)            + UN3(I,2,L) )      
      VZ(I,2,L,N2) = A1 *   VZ(I,2,L,N2)                                
     .             + A2 * ( VZ(I,2,L,N1)            + VN3(I,2,L) )      
      TZ(I,2,L,N2) = A1 *   TZ(I,2,L,N2)                                
     .             + A2 * ( TZ(I,2,L,N1)*PZ(I,2,N1) + TN3(I,2,L) )      
      QZ(I,2,L,N2) = A1 *   QZ(I,2,L,N2)                                
     .             + A2 * ( QZ(I,2,L,N1)*PZ(I,2,N1) + QN3(I,2,L) )      
   50 CONTINUE                                                          
                                                                        
   20 CONTINUE                                                          
                                                                        
C **********************************************************************
C ****             APPLY ASSELIN TIME FILTER ON PRESSURE            ****
C **********************************************************************
                                                                        
      DO 100 I=1,IMJMM2                                                 
      PZ(I,2,N2) = A1*PZ(I,2,N2) + A2*( PZ(I,2,N1)+PN3(I,2) )           
  100 CONTINUE                                                          
                                                                        
C **********************************************************************
C ****                  UNSCALE TIME-LEVEL N2                       ****
C **********************************************************************
                                                                        
      DO 150 I=1,IMJMM2                                                 
      PH(I,2) = ONE / PZ(I,2,N2)                                        
  150 CONTINUE                                                          
      DO 160 L=1,NLAYR                                                  
      DO 160 I=1,IMJMM2                                                 
      TZ(I,2,L,N2) = TZ(I,2,L,N2) * PH(I,2)                             
      QZ(I,2,L,N2) = QZ(I,2,L,N2) * PH(I,2)                             
  160 CONTINUE                                                          
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE ATOC ( qa,qc,im,jm,km,itype )                          
C                                                                       
C ******************************************************************    
C ****                                                          ****    
C ****  This program converts 'A' gridded data                  ****    
C ****                     to 'C' gridded data.                 ****    
C ****                                                          ****    
C ****  A direct transfer           is made      for itype = 0  ****    
C ****  An FFT shift transformation is made in x for itype = 1  ****    
C ****  An FFT shift transformation is made in y for itype = 2  ****    
C ****                                                          ****    
C ******************************************************************    
                                                                        
#include <vrslv.com>                                                    
                                                                        
      DIMENSION qa   (im,jm,km)                                         
      DIMENSION qc   (im,jm,km)                                         
      DIMENSION qz   (im,jm,km)                                         
                                                                        
      DIMENSION qax  (   im+2 ,km)                                          
      DIMENSION  cx  (2*(im+2),km)                                          
      DIMENSION qay  (   2*jm ,km)                                          
      DIMENSION  cy  (2*(2*jm),km)                                          
                                                                        
      DIMENSION cosx (imr/2), sinx(imr/2)                               
      DIMENSION cosy (jmr)  , siny(jmr)                                 
      DIMENSION IFX  (100) , IFY (100)                                    
      REAL      TRIGX(3*(IMR+1))                                        
      REAL      TRIGY(3*(2*JMR+1))                                          
      
      save IFX  , IFY
      save TRIGX
      save TRIGY
      save cosx , sinx
      save cosy , siny

      logical first                                                     
      data    first /.true./                                            
                                                                        
      parameter ( zero  = 0.0 )                                         
      parameter ( ahalf = 0.5 )                                         
                                                                        
      JMM1  = JM-1                                                      
      JMM2  = JM-2                                                      
      JMM3  = JM-3                                                      
      JP    = 2*jmm1                                                    
                                                                        
      IMD2  = IM/2                                                      
      PI    = 4.*ATAN(1.)                                               
      DX    = 2*PI/IM                                                   
      DY    = PI/JMM1                                                   
                                                                        
C *********************************************************             
C ****     define sinx, cosx and trigs coefficients    ****             
C *********************************************************             
                                                                        
      if (first) then                                                   
                                                                        
      call fftfax (im,ifx,trigx)                                        
      do k=1,im/2                                                       
      thx = k*dx*ahalf                                                  
      cosx(k) = cos(thx)                                                
      sinx(k) = sin(thx)                                                
      enddo                                                             
                                                                        
      call fftfax (jp,ify,trigy)                                        
      do l=1,jmm1                                                       
      thy = l*dy*ahalf                                                  
      cosy(l) = cos(thy)                                                
      siny(l) = sin(thy)                                                
      enddo                                                             
                                                                        
      first = .false.                                                   
      endif                                                             
                                                                        
C *********************************************************             
C ****       load input array into temporary array     ****             
C *********************************************************             
                                                                        
      do i = 1,im*jm*km                                                 
      qz(i,1,1) = qa(i,1,1)                                             
      enddo                                                             
                                                                        
C *********************************************************             
C ****                mass points                      ****             
C *********************************************************             
                                                                        
      if (itype.eq.0) then                                              
      do i=1,im*jm*km                                                   
      qc(i,1,1) = qz(i,1,1)                                             
      enddo                                                             
      endif                                                             
                                                                        
C *********************************************************             
C ****           stagger in x-direction                ****             
C *********************************************************             
                                                                        
      if (itype.eq.1) then                                              
      do j=1,jm                                                         
                                                                        
      do l=1,km                                                         
      do i=1,im                                                         
      qax(i,l) = qz(i,j,l)                                              
      enddo                                                             
      enddo                                                             
      call rfftmlt (qax,cx,trigx,ifx,1 ,im+2,im,km,-1)                   
                                                                        
      do l=1,km                                                         
      do k=1,IMD2                                                       
      kr = 2*k+1                                                        
      ki = 2*k+2                                                        
      crprime = qax(kr,l)*cosx(k) - qax(ki,l)*sinx(k)                   
      ciprime = qax(ki,l)*cosx(k) + qax(kr,l)*sinx(k)                   
      qax(kr,l) = crprime                                               
      qax(ki,l) = ciprime                                               
      enddo                                                             
      enddo                                                             
                                                                        
      call rfftmlt (qax,cx,trigx,ifx,1 ,im+2,im,km, 1)                   
      do l=1,km                                                         
      do i=1,im                                                         
      qc(i,j,l) = qax(i,l)                                              
      enddo                                                             
      enddo                                                             
      enddo                                                             
                                                                        
      endif                                                             
                                                                        
C *********************************************************             
C ****           stagger in y-direction                ****             
C *********************************************************             
                                                                        
      if (itype.eq.2) then                                              
                                                                        
      do i=1,im/2                                                       
                                                                        
      do l=1,km                                                         
      do j=1,jmm1                                                       
      qay(j,l)      =  qz(i,j,l)                                        
      qay(j+jmm1,l) = -qz(i+imd2,jm-j+1,l)                              
      enddo                                                             
      enddo                                                             
                                                                        
      call rfftmlt (qay,cy,trigy,ify,1 ,jp+2,jp,km,-1)                   
                                                                        
      do l=1,km                                                         
      do k=1,jmm1                                                       
      kr = 2*k+1                                                        
      ki = 2*k+2                                                        
      crprime = qay(kr,l)*cosy(k) - qay(ki,l)*siny(k)                   
      ciprime = qay(ki,l)*cosy(k) + qay(kr,l)*siny(k)                   
      qay(kr,l) = crprime                                               
      qay(ki,l) = ciprime                                               
      enddo                                                             
      enddo                                                             
                                                                        
      call rfftmlt (qay,cy,trigy,ify,1 ,jp+2,jp,km, 1)                   
                                                                        
      do l=1,km                                                         
      do j=1,jmm1                                                       
      qc(i,j,l)             =  qay(j,l)                                 
      qc(i+imd2,jmm1-j+1,l) = -qay(j+jmm1,l)                            
      enddo                                                             
      enddo                                                             
                                                                        
      enddo                                                             
      endif                                                             
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE CTOA ( qc,qa,im,jm,km,itype )                          
C                                                                       
C ******************************************************************    
C ****                                                          ****    
C ****  This program converts 'C' gridded data                  ****    
C ****                     to 'A' gridded data.                 ****    
C ****                                                          ****    
C ****  A direct transfer           is made      for itype = 0  ****    
C ****  An FFT shift transformation is made in x for itype = 1  ****    
C ****  An FFT shift transformation is made in y for itype = 2  ****    
C ****                                                          ****    
C ******************************************************************    
                                                                        
#include <vrslv.com>                                                    
                                                                        
      DIMENSION qa   (im,jm,km)                                         
      DIMENSION qc   (im,jm,km)                                         
      DIMENSION qz   (im,jm,km)                                         
                                                                        
      DIMENSION qcx  (   im+2 ,km)                                          
      DIMENSION  cx  (2*(im+2),km)                                          
      DIMENSION qcy  (   2*jm ,km)                                          
      DIMENSION  cy  (2*(2*jm),km)                                          
                                                                        
      DIMENSION cosx (imr/2), sinx(imr/2)                               
      DIMENSION cosy (jmr)  , siny(jmr)                                 
      DIMENSION IFX  (100) , IFY (100)                                    
      REAL      TRIGX(3*(IMR+1))                                        
      REAL      TRIGY(3*(2*JMR+1))                                          
                                                                        
      save IFX  , IFY
      save TRIGX
      save TRIGY
      save cosx , sinx
      save cosy , siny

      logical first                                                     
      data    first /.true./                                            
                                                                        
      parameter ( zero  = 0.0 )                                         
      parameter ( ahalf = 0.5 )                                         
                                                                        
      imh   = IM/2                                                      
      JMM1  = JM-1                                                      
      JMM2  = JM-2                                                      
      JMM3  = JM-3                                                      
      JP    = 2*jmm1                                                    
                                                                        
      PI    = 4.*ATAN(1.)                                               
      DX    = 2*PI/IM                                                   
      DY    = PI/JMM1                                                   
                                                                        
C *********************************************************             
C ****     define sinx, cosx and trigs coefficients    ****             
C *********************************************************             
                                                                        
      if (first) then                                                   
                                                                        
      call fftfax (im,ifx,trigx)                                        
      do k=1,imh                                                        
      thx = k*dx*ahalf                                                  
      cosx(k) = cos(thx)                                                
      sinx(k) = sin(thx)                                                
      enddo                                                             
                                                                        
      call fftfax (jp,ify,trigy)                                        
      do l=1,jmm1                                                       
      thy = l*dy*ahalf                                                  
      cosy(l) = cos(thy)                                                
      siny(l) = sin(thy)                                                
      enddo                                                             
                                                                        
      first = .false.                                                   
      endif                                                             
                                                                        
C *********************************************************             
C ****       load input array into temporary array     ****             
C *********************************************************             
                                                                        
      do i = 1,im*jm*km                                                 
      qz(i,1,1) = qc(i,1,1)                                             
      enddo                                                             
                                                                        
C *********************************************************             
C ****                mass points                      ****             
C *********************************************************             
                                                                        
      if (itype.eq.0) then                     
      c1  =  4./3.
      c2  = -1./3.                         
      do I=1,im*jm*km                                                   
      qa(I,1,1) = qz(I,1,1)                                             
      enddo                                                             
      do L=1,km                                                         

      do i=1,im
      qa(i,1 ,L) = c1*qz(i,2   ,L) + c2*qz(i,3   ,L)
      qa(i,jm,L) = c1*qz(i,jm-1,L) + c2*qz(i,jm-2,L)
      enddo 

      qnp = 0.                                                          
      qsp = 0.                                                          
      do i=1,im                                                         
      qnp = qnp + qa(i,jm,L)                                          
      qsp = qsp + qa(i,1 ,L)                                          
      enddo                                                             
      qnp = qnp / im                                                    
      qsp = qsp / im                                                    
      do i=1,im                                                         
      qa(I,jm,L) = qnp                                                  
      qa(I,1 ,L) = qsp                                                  
      enddo                                                             
      enddo                                                             
      endif                                                             
                                                                        
C *********************************************************             
C ****           stagger in x-direction                ****             
C *********************************************************             
                                                                        
      if (itype.eq.1) then                                              
      do j=1,jm                                                         
                                                                        
      do l=1,km                                                         
      do i=1,im                                                         
      qcx(i,l) = qz(i,j,l)                                              
      enddo                                                             
      enddo                                                             
      call rfftmlt (qcx,cx,trigx,ifx,1 ,im+2,im,km,-1)                   
                                                                        
      do l=1,km                                                         
      do k=1,imh                                                        
      kr = 2*k+1                                                        
      ki = 2*k+2                                                        
      crprime = qcx(kr,l)*cosx(k) + qcx(ki,l)*sinx(k)                   
      ciprime = qcx(ki,l)*cosx(k) - qcx(kr,l)*sinx(k)                   
      qcx(kr,l) = crprime                                               
      qcx(ki,l) = ciprime                                               
      enddo                                                             
      enddo                                                             
                                                                        
      call rfftmlt (qcx,cx,trigx,ifx,1 ,im+2,im,km, 1)                   
      do l=1,km                                                         
      do i=1,im                                                         
      qa(i,j,l) = qcx(i,l)                                              
      enddo                                                             
      enddo                                                             
      enddo                                                             
                                                                        
      endif                                                             
                                                                        
C *********************************************************             
C ****           stagger in y-direction                ****             
C *********************************************************             
                                                                        
      if (itype.eq.2) then                                              
                                                                        
      do i=1,imh                                                        
                                                                        
      do l=1,km                                                         
      do j=1,jmm1                                                       
      qcy(j,l)      =  qz(i,j,l)                                        
      qcy(j+jmm1,l) = -qz(i+imh,jmm1-j+1,l)                            
      enddo                                                             
      enddo                                                             
                                                                        
      call rfftmlt (qcy,cy,trigy,ify,1 ,jp+2,jp,km,-1)                   
                                                                        
      do l=1,km                                                         
      do k=1,jmm1                                                       
      kr = 2*k+1                                                        
      ki = 2*k+2                                                        
      crprime = qcy(kr,l)*cosy(k) + qcy(ki,l)*siny(k)                   
      ciprime = qcy(ki,l)*cosy(k) - qcy(kr,l)*siny(k)                   
      qcy(kr,l) = crprime                                               
      qcy(ki,l) = ciprime                                               
      enddo                                                             
      enddo                                                             
                                                                        
      call rfftmlt (qcy,cy,trigy,ify,1 ,jp+2,jp,km, 1)                   
                                                                        
      do l=1,km                                                         
      do j=1,jmm1                                                       
      qa(i,j,l)          =  qcy(j,l)                                   
      qa(i+imh,jm-j+1,l) = -qcy(j+jmm1,l)                              
      enddo                                                             
      enddo                                                             
      enddo                                                             
                                                                        
      do l=1,km                                                         
      do i=1,imh                                                       
      qa(i+imh,1,l) = -qa(i,1,l)                                       
      qa(i,jm,l)    = -qa(i+imh,jm,l)                                 
      enddo                                                             
      enddo                                                             
      endif                                                             
                                                                        
      return                                                            
      END                                                               
      SUBROUTINE QCHECK (QZ,DSIG,IMAX,JMAX,KMAX,BUDFLG,QOLD)            
C***********************************************************************
C  PURPOSE                                                              
C     CHECK SPECIFIC HUMIDITY FIELD FOR NEGATIVE VALUES.                
C  USAGE                                                                
C     CALLED BY HYDRO                                                   
C  RECORD OF MODIFICATIONS                                              
C  REMARKS:                                                             
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
                                                                        
#include <vrslv.com>                                                    
#include <vbudget.com>                                                  
                                                                        
      DIMENSION   QZ(IMAX,JMAX,KMAX), DSIG(KMAX)                        
      DIMENSION QOLD(IMAX,JMAX,KMAX)                                    
      PARAMETER ( ZERO    = 0.0 )                                       
      PARAMETER ( AHALF   = 0.5 )                                       
      PARAMETER ( IMJMR   = IMR*(JMR+1) )                               
      PARAMETER ( IMJMM1R = IMR* JMR    )                               
      PARAMETER ( IMJMM2R = IMR*(JMR-1) )                               
      LOGICAL     BUDFLG                                                
                                                                        
C **********************************************************************
C ****               CORRECT SPECIFIC HUMIDITY                      ****
C **********************************************************************
                                                                        
      IF(BUDFLG) THEN                                                   
      DO 10 L=1,KMAX                                                    
      DO 10 I=1,IMJMM2R                                                 
      QOLD(I,2,L) = QZ(I,2,L)                                           
   10 CONTINUE                                                          
      ENDIF                                                             
                                                                        
      DO 20 L = 2,KMAX                                                  
          LM1 = L-1                                                     
          NCHECK = 0                                                    
          DO 30 I=1,IMJMM2R                                             
          IF( QZ(I,2,LM1).LT.ZERO ) NCHECK = NCHECK + 1                 
   30     CONTINUE                                                      
      IF (NCHECK.EQ.0) GOTO 20                                          
          DDSIG = DSIG(LM1)/DSIG(L)                                     
          DO 40 I=1,IMJMM2R                                             
          IF( QZ(I,2,LM1).LT.ZERO ) THEN                                
          QZ(I,2,L  ) = QZ(I,2,L) + QZ(I,2,LM1)*DDSIG                   
          QZ(I,2,LM1) = ZERO                                            
          ENDIF                                                         
   40     CONTINUE                                                      
   20 CONTINUE                                                          
          DO 50 I=1,IMJMM2R                                             
          IF( QZ(I,2,KMAX).LT.ZERO ) QZ(I,2,KMAX) = ZERO                
   50     CONTINUE                                                      
C                                                                       
      IF(BUDFLG) THEN                                                   
      DO 60 L=1,KMAX                                                    
      DO 70 I=1,IMJMM2R                                                 
      ZTEMP(I,2) = QZ(I,2,L)-QOLD(I,2,L)                                
   70 CONTINUE                                                          
      CALL ZMEAN ( ZTEMP,ZDPQDT(1,L),IMAX,JMAX,AHALF )                  
      CALL ZMEAN ( ZTEMP,ZQFILL(1,L),IMAX,JMAX,AHALF )                  
   60 CONTINUE                                                          
      ENDIF                                                             
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE SHAPIJ ( PZ,UZ,VZ,TZ,QZ,DUDT,DVDT,DTDT,DQDT,            
     .                    IMAX,JMAX,KMAX )
C***********************************************************************
C  PURPOSE                                                              
C     Used to globally filter U-wind, V-wind, Temperature and Moisture  
C     to damp small scale dispersive waves.                             
C  USAGE                                                                
C     CALLED BY STEP EVERY TIMESTEP (ZDT).                              
C  ARGUMENTS   DESCRIPTION                                              
C     PZ ..... SURFACE PRESSURE FIELD (PS-PTOP)                         
C     UZ ..... U-WIND                                                   
C     VZ ..... V-WIND                                                   
C     TZ ..... POTENTIAL TEMPERATURE                                    
C     QZ ..... SPECIFIC  HUMIDITY                                       
C     IMAX ... X-DIMENSION OF FIELDS                                    
C     JMAX ... Y-DIMENSION OF FIELDS                                    
C     KMAX ... Z-DIMENSION OF FIELDS                                    
C
C  REMARKS:                                                             
C     FULL  FILTER IS ASSUMED TO HAVE A TIME-SCALE OF TAU.              
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
                                                                        
      PARAMETER ( ZERO  = 0.0     )                                     
      PARAMETER ( AHALF = 0.5     )                                     
      PARAMETER ( ONE   = 1.0     )                                     
      PARAMETER ( TAU   = 5400.0  )                                     
      PARAMETER (  DF   = ONE/TAU )                                     
                                                                        
      DIMENSION PZ(IMAX,JMAX)                                           
      DIMENSION UZ(IMAX,JMAX,KMAX)                                      
      DIMENSION VZ(IMAX,JMAX,KMAX)                                      
      DIMENSION TZ(IMAX,JMAX,KMAX)                                      
      DIMENSION QZ(IMAX,JMAX,KMAX)                                      
                                                                        
      DIMENSION DUDT(IMAX,JMAX,KMAX)                                    
      DIMENSION DVDT(IMAX,JMAX,KMAX)                                    
      DIMENSION DTDT(IMAX,JMAX,KMAX)                                    
      DIMENSION DQDT(IMAX,JMAX,KMAX)                                    
                                                                        
c Local Dynamic Space
c -------------------
      dimension  SHAPU (imax,jmax)                                       
      dimension  SHAPV (imax,jmax)                                       
      dimension  SHAPT (imax,jmax)                                       
      dimension  SHAPQ (imax,jmax)                                       
      dimension  tmpt  (imax,jmax)                                       
      dimension  tmpq  (imax,jmax)                                       
      dimension  TMP1  (imax,jmax)                                       
      dimension  TMP2  (imax,jmax)                                       
      dimension  ustr  (imax,jmax)                                       
      dimension  vstr  (imax,jmax)                                       
                                                                        
C **********************************************************************
C ****               DETERMINE ORDER OF SHAPIRO FILTER              ****
C **********************************************************************
                                                                        
      if( imax.le.72 ) nshap = 16                                         
      if( imax.gt.72 ) nshap =  8                                         
                                                                        
      IMJMM2R = IMAX*(JMAX-2)                               
                                                                        
C **********************************************************************
C ****        FRACTIONAL FILTER ON FIELDS U, V, T AND Q             ****
C **********************************************************************
                                                                        
#if macro
cmic$ do all shared (df, dqdt, dtdt, dudt, dvdt, imax, imjmm2r)
cmic$*       shared (jmax, kmax, nshap, one, pz, qz, tau, tz, uz)
cmic$*       shared (vz)
cmic$*       private (i, l, shapq, shapt, shapu, shapv, tmp1, tmp2)
cmic$*       private (tmpq, tmpt, ustr, vstr)
#endif
      DO 1000 L=1,KMAX                                                  
                                                                        
      call polewnd ( uz(1,1,L),vz(1,1,L),ustr,vstr,imax,jmax )
      call ctoa    ( tz(1,1,L),tmpt,imax,jmax,1,0 )
      call ctoa    ( qz(1,1,L),tmpq,imax,jmax,1,0 )

      CALL SHAP ( ustr,SHAPU,imax,jmax,-1,nshap,TMP1,TMP2 )                
      CALL SHAP ( vstr,SHAPV,imax,jmax,-2,nshap,TMP1,TMP2 )                
      CALL SHAP ( tmpt,SHAPT,imax,jmax, 1,nshap,TMP1,TMP2 )                
      CALL SHAP ( tmpq,SHAPQ,imax,jmax, 1,nshap,TMP1,TMP2 )                
                                                                        
C **********************************************************************
C ****             MASS-WEIGHT SHAPIRO ADJUSTMENTS                  ****
C **********************************************************************
                                                                        
      do I=1,imjmm2r                                                    
      SHAPU(I,2) =           SHAPU(I,2) * DF                            
      SHAPV(I,2) =           SHAPV(I,2) * DF                            
      SHAPT(I,2) = PZ(I,2) * SHAPT(I,2) * DF                            
      SHAPQ(I,2) = PZ(I,2) * SHAPQ(I,2) * DF                            
      enddo                                                             
                                                                        
C **********************************************************************
C ****              UPDATE MASS-WEIGHTED TENDENCIES                 ****
C **********************************************************************
   
      do I=1,IMJMM2R                                                    
      DUDT(I,2,L) = DUDT(I,2,L) - SHAPU(I,2)
      DVDT(I,2,L) = DVDT(I,2,L) - SHAPV(I,2)
      DTDT(I,2,L) = DTDT(I,2,L) - SHAPT(I,2)
      DQDT(I,2,L) = DQDT(I,2,L) - SHAPQ(I,2)
      enddo                                                             
                                                                        
 1000 CONTINUE                                                          

      RETURN                                                            
      END                                                               
      SUBROUTINE SHAP (Q,DQ,IM,JM,M,NORDER,D1,D2)                       
C***********************************************************************
C  PURPOSE                                                              
C     APPLY SHAPIRO FILTER TO A GENERAL FIELD                           
C  USAGE                                                                
C     CALLED FROM SHAPIJ                                                
C  DESCRIPTION OF PARAMETERS                                            
C     Q ....... FIELD TO BE FILTERED - DIMENSIONED IM X JM              
C     DQ ...... FILTER ADJUSTMENTS:  Q - FILTER(Q)                      
C     IM ...... FIRST (EAST-WEST) DIMENSION OF Q                        
C     JM ...... SECOND (NORTH-SOUTH) DIMENSION OF Q                     
C     M ....... SET M = +1 FOR SCALAR FIELDS                            
C                   M = -1 FOR U-Wind FIELD                             
C                   M = -2 FOR V-Wind FIELD                             
C     NORDER .. THE ORDER OF THE SHAPIRO FILTER                         
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
                                                                        
#include <vrslv.com>                                                    
                                                                        
      PARAMETER ( ZERO  = 0.0 )                                         
      PARAMETER ( ONE   = 1.0 )                                         
      PARAMETER ( FOUR  = 4.0 )                                         

      PARAMETER ( IMH    = IMR/2       )                                
      PARAMETER ( IMJMR  = IMR*(JMR+1) )                                
      PARAMETER ( IMJM1R = IMR* JMR    )                                
      PARAMETER ( IMJM2R = IMR*(JMR-1) )                                
      PARAMETER ( IMJM3R = IMR*(JMR-2) )                                
      PARAMETER ( IMJM4R = IMR*(JMR-3) )                                
                                                                        
C  TEMPORARY VARIABLES FOR SHAPIRO FILTER                               
C  ======================================                               
      DIMENSION  D1   (imr,jmr+1)                                       
      DIMENSION  D2   (imr,jmr+1)                                       
                                                                        
      DIMENSION  Q(imr,jmr+1), dq(imr,jmr+1)                        
                                                                        
C **********************************************************************
C ****           IORDER = 8  IMPLIES 16TH ORDER SHAPIRO             ****
C ****           IORDER = 4  IMPLIES  8TH ORDER SHAPIRO             ****
C ****                A = 1.0 / 4.0**IORDER                         ****
C **********************************************************************
                                                                        
      IORDER = NORDER / 2                                               
      A = ONE / ( FOUR ** IORDER )                                      
                                                                        
C **********************************************************************
C ****                   FILTER MASS POINTS                         ****
C **********************************************************************

      if ( m.eq. 1 ) n =  1
      if ( m.eq.-1 ) n = -1

      if ( m.eq. 1 .or. m.eq.-1 ) then                                  
                                                                        
c Zonal Direction
c ---------------

      DO I=1,imjmr                                                   
      D1(I,1) = Q(I,1)                                                  
      enddo                                                             
                                                                        
      DO num=1,IORDER                                              
      DO I=1,imjmr                                                   
      D2(I,1) = D1(I,1) - D1(I-1,1)                                   
      enddo                                                             
      DO J=1,jmr+1                                                  
      D2(1,J)   = D1(1,J)   - D1(IMR,J)                                 
      enddo                                                             
      DO I=1,imjmr                                                   
      D1(I,1)   = D2(I+1,1) - D2(I,1)                                   
      enddo                                                             
      DO J=1,jmr+1                                                   
      D1(IMR,J) = D2(1,J)   - D2(IMR,J)                                 
      enddo                                                             
      enddo                                                             
                                                                        
      DO I=1,imjmr                                                   
      DQ(I,1) = Q(I,1) - D1(I,1)*A                                      
      enddo                                                             
                                                                        
c Meridional Direction
c --------------------
                                                                        
      DO I=1,imjmr                                                   
      D1(I,1) = DQ(I,1)                                                 
      enddo                                                             
                                                                        
      DO num=1,IORDER
      DO I=1,imjmr
      D2(I,1) = D1(I,1)
      enddo

      j = 2
      do i=1,imjm2r
      d1(i,j) = d2(i,j+1)-2.*d2(i,j)+d2(i,j-1)
      enddo
      do i=1,imh
      j = 1 
      d1(i,j)     = ( d2(i,j+1)     -   d2(i,j)       )
     .            - ( d2(i,j  )     - n*d2(i+imh,j+1) )
      d1(i+imh,j) = ( d2(i+imh,j+1) -   d2(i+imh,j)   )
     .            - ( d2(i+imh,j  ) - n*d2(i,j+1)     )
      j = jm 
      d1(i,j)     = ( d2(i,j-1)     -   d2(i,j)       )
     .            - ( d2(i,j  )     - n*d2(i+imh,j-1) )
      d1(i+imh,j) = ( d2(i+imh,j-1) -   d2(i+imh,j)   )
     .            - ( d2(i+imh,j  ) - n*d2(i,j-1)     )
      enddo

      enddo

c Zero-out Shapiro Effect near poles
c ----------------------------------
      do i=1,im
      d1(i,2   ) = 0.
      d1(i,jm-1) = 0.
      enddo

      endif

C **********************************************************************
C ****                     FILTER V-WIND POINTS                     ****
C **********************************************************************

      if ( m.eq.-2 ) then                                               
                                                                       
c Zonal Direction
c ---------------

      DO I=1,imjm1r                                                    
      D1(I,1) = Q(I,1)                                                  
      enddo                                                             
                                                                        
      DO num=1,IORDER                                              
      DO I=1,imjm1r                                                    
      D2(I,1) = D1(I,1) - D1(I-1,1)                                   
      enddo                                                             
      DO J=1,jmr                                                    
      D2(1,J)   = D1(1,J)   - D1(IMR,J)                                 
      enddo                                                             
      DO I=1,imjm1r                                                    
      D1(I,1)   = D2(I+1,1) - D2(I,1)                                   
      enddo                                                             
      DO J=1,jmr                                                    
      D1(IMR,J) = D2(1,J)   - D2(IMR,J)                                 
      enddo                                                             
      enddo                                                             
                                                                        
      DO I=1,imjm1r                                                    
      DQ(I,1) = Q(I,1) - D1(I,1)*A                                      
      enddo                                                             
                                                                        
c Meridional Direction
c --------------------
                                                                        
      DO I=1,imjm1r                                                    
      D1(I,1) = DQ(I,1)                                                 
      enddo                                                             
                                                                        
      DO num=1,IORDER                                              
      DO I=1,imjmr
      D2(I,1) = D1(I,1)
      enddo

      j = 2
      do i=1,imjm3r
      d1(i,j) = d2(i,j+1)-2.*d2(i,j)+d2(i,j-1)
      enddo
      do i=1,imh
      j = 1 
      d1(i,j)     = ( d2(i,j+1)     - d2(i,j)       )
     .            - ( d2(i,j  )     + d2(i+imh,j) )
      d1(i+imh,j) = ( d2(i+imh,j+1) - d2(i+imh,j)   )
     .            - ( d2(i+imh,j  ) + d2(i,j)     )
      j = jm-1
      d1(i,j)     = ( d2(i,j-1)     - d2(i,j)       )
     .            - ( d2(i,j  )     + d2(i+imh,j) )
      d1(i+imh,j) = ( d2(i+imh,j-1) - d2(i+imh,j)   )
     .            - ( d2(i+imh,j  ) + d2(i,j)     )
      enddo
                                                                        
      enddo

c Zero-out Shapiro Effect near poles
c ----------------------------------
      do i=1,im
      d1(i,2   ) = 0.
      d1(i,jm-2) = 0.
      enddo

      endif                                                             

C **********************************************************************
C ****              CALCULATE FULL FILTERED FIELD                   ****
C **********************************************************************
                                                                        
      DO 100 I=1,imjmr                                                  
      DQ(I,1) = DQ(I,1) - D1(I,1)*A                                     
  100 CONTINUE                                                          
                                                                        
C **********************************************************************
C ****                CALCULATE FILTER ADJUSTMENT                   ****
C **********************************************************************
                                                                        
      DO 130 I=1,imjmr                                                  
      DQ(I,1) = Q(I,1) - DQ(I,1)                                        
  130 CONTINUE                                                          
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE ctoaset (n)                                            
C***********************************************************************
C  PURPOSE                                                              
C     Converts C-Grid data to A-Grid data                               
C  ARGUMENTS   DESCRIPTION                                              
C     N ........ Current Time Index                                     
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
                                                                        
#include <vrslv.com>                                                    
#include <vstate.com>                                                   
                                                                        
      dimension utmp(imr,jmr+1,nlayr)                                   
      dimension vtmp(imr,jmr+1,nlayr)                                   
                                                                        
C **********************************************************************
C ****                      CONVERT TO A-GRID                       ****
C **********************************************************************
                                                                        
      do l=1,nlayr                                                      
      call polewnd ( uz(1,1,L,n),vz(1,1,L,n),                           
     .               utmp(1,1,L),vtmp(1,1,L),imr,jmr+1)                 
      enddo                                                             
                                                                        
      call ctoa ( phisz      , phisz      , imr,jmr+1,1    ,0 )         
      call ctoa ( pz(1,1,  n), pz(1,1,  n), imr,jmr+1,1    ,0 )         
      call ctoa ( utmp(1,1,1), uz(1,1,1,n), imr,jmr+1,nlayr,1 )         
      call ctoa ( vtmp(1,1,1), vz(1,1,1,n), imr,jmr+1,nlayr,2 )         
      call ctoa ( tz(1,1,1,n), tz(1,1,1,n), imr,jmr+1,nlayr,0 )         
      call ctoa ( qz(1,1,1,n), qz(1,1,1,n), imr,jmr+1,nlayr,0 )         
      call ctoa ( pkht(1,1,1), pkht(1,1,1), imr,jmr+1,nlayr,0 )         
                                                                        
      return                                                            
      end                                                               
      SUBROUTINE atocset (n)                                            
C***********************************************************************
C  PURPOSE                                                              
C     Converts A-Grid data to C-Grid data                               
C  ARGUMENTS   DESCRIPTION                                              
C     N ........ Current Time Index                                     
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
                                                                        
#include <vrslv.com>                                                    
#include <vstate.com>                                                   
                                                                        
C **********************************************************************
C ****                      CONVERT TO C-GRID                       ****
C **********************************************************************
                                                                        
      call atoc ( pz(1,1,  n), pz(1,1,  n), imr,jmr+1,1    ,0 )         
      call atoc ( uz(1,1,1,n), uz(1,1,1,n), imr,jmr+1,nlayr,1 )         
      call atoc ( vz(1,1,1,n), vz(1,1,1,n), imr,jmr+1,nlayr,2 )         
      call atoc ( tz(1,1,1,n), tz(1,1,1,n), imr,jmr+1,nlayr,0 )         
      call atoc ( qz(1,1,1,n), qz(1,1,1,n), imr,jmr+1,nlayr,0 )         
      call atoc ( pkht(1,1,1), pkht(1,1,1), imr,jmr+1,nlayr,0 )         
                                                                        
      return                                                            
      end                                                               
      SUBROUTINE PKAPPC (PZ,PKHT,PKZ,IMAX,JMAX,KMAX)
C***********************************************************************
C  PURPOSE
C     CALCULATE PHILLIPS P**KAPPA ON C-GRID
C  ARGUMENTS
C     PZ ..... SURFACE PRESSURE - PTOP
C     PKHT ... SIGMA EDGE-LEVEL PRESSURE**KAPPA
C     PKZ .... SIGMA      LEVEL PRESSURE**KAPPA
C     IMAX ... LONGITUDE  DIMENSION
C     JMAX ... LATITUDE   DIMENSION
C     KMAX ... VERTICAL   DIMENSION
C  REMARKS:
C     NOTE:  PERTURBATION GEOPOTENTIALS ARE USED WITH
C             TSTD = 280 DEG. K.    ,  PSTD = 1000 MB.
C            PKSTD = PSTD ** KAPPA  , THSTD = TSTD / PKSTD
C
C***********************************************************************
C*                GODDARD LABORATORY FOR ATMOSPHERES
C***********************************************************************

#include <vreal.com>
#include <vrslv.com>
#include <vmetric.com>

      PARAMETER (  ONE    = 1.0 )
      PARAMETER ( IMJMR   = IMR*(JMR+1) )
      PARAMETER ( IMJMM1R = IMR* JMR    )
      PARAMETER ( IMJMM2R = IMR*(JMR-1) )
                  
      DIMENSION PZ  (IMAX,JMAX)
      DIMENSION PKHT(IMAX,JMAX,KMAX)
      DIMENSION PKZ (IMAX,JMAX,KMAX)
                  
      DIMENSION PINV(IMR,JMR+1)
      DIMENSION PLE (IMR,JMR+1,2)
      DIMENSION PKLE(IMR,JMR+1,2)
 
C **********************************************************************
C ****                      INITIALIZATION                          ****
C **********************************************************************
                  
      akap1 = one + getcon('KAPPA')
                  
      DO 10 I=1,IMJMM2R
      PINV(I,2) = ONE / ( akap1*PZ(I,2) )
   10 CONTINUE   
                  
C **********************************************************************
C ****                  COMPUTE PHILLIPS P**KAPPA                   ****
C **********************************************************************
                  
      LUP = 1    
      LDN = 2    
                  
      PKTOP = EXPBYK(PTOP)
      DO 20 I=1,IMJMM2R
      PLE (I,2,LUP) = PTOP
      PKLE(I,2,LUP) = PKTOP
   20 CONTINUE   
                  
      DO 1000 L = 1,NLAYR
      DSIGINV = ONE/DSIG(L)
      DO 30 I=1,IMJMM2R
      PLE (I,2,LDN) = SIGE(L+1)*PZ(I,2) + PTOP
      PKLE(I,2,LDN) = PKHT(I,2,L)
                  
      PKZ(I,2,L) = ( PLE (I,2,LDN)*PKLE(I,2,LDN)
     .             - PLE (I,2,LUP)*PKLE(I,2,LUP) )
     .           *   PINV(I,2)*DSIGINV
   30 CONTINUE   
                  
      LSV = LUP  
      LUP = LDN  
      LDN = LSV  
 1000 CONTINUE   
                  
      RETURN     
      END   
      FUNCTION GMEAN (Q,IMAX,JMAX)
#include <vrslv.com>
#include <vmetric.com>
      DIMENSION Q(IMAX,JMAX)

      PARAMETER ( ZERO = 0.0 )
      PARAMETER ( FOUR = 4.0 )
      PARAMETER ( IMJMM2R = IMR*(JMR-1) )

C *********************************************************
C ****             CALCULATE GLOBAL MEAN               ****
C *********************************************************

      QSMJ  = ZERO
      DO 10 I=1,IMJMM2R
      QSMJ  = QSMJ + Q(I,2)*COSIJ(I,2)
   10 CONTINUE
      QSMJ  = QSMJ*DL*DP
      GMEAN = QSMJ/(FOUR*PI)

      RETURN
      END
      FUNCTION PMEAN (P,IMAX,JMAX)
#include <vrslv.com>
#include <vmetric.com>
      DIMENSION P(IMAX,JMAX)

      PARAMETER ( ZERO = 0.0 )
      PARAMETER ( FOUR = 4.0 )
      PARAMETER ( IMJMM2R = IMR*(JMR-1) )

C *********************************************************
C ****      CALCULATE GLOBAL MEAN SURFACE PRESSURE     ****
C *********************************************************

      QSMJ  = ZERO
      DO 10 I=1,IMJMM2R
      QSMJ  = QSMJ + P(I,2)*COSIJ(I,2)
   10 CONTINUE
      QSMJ  = QSMJ*DL*DP
      PMEAN = QSMJ/(FOUR*PI) + PTOP

      RETURN
      END
      SUBROUTINE QBARU ( u,v,ubar,vbar,utmp,vtmp,im,jm )
      Dimension      u(im,jm)
      Dimension      v(im,jm)
      Dimension   ubar(im,jm)
      Dimension   vbar(im,jm)
      Dimension   utmp(im,jm)
      Dimension   vtmp(im,jm)
      Parameter (ahalf = 0.5  )

      imh    = im/2
      imjmm2 = im*(jm-2)

      call polewnd ( u,v,utmp,vtmp,im,jm )
      j =  2
      do i=1,imjmm2
      ubar(i,j) = ( utmp(i,j) + utmp(i-1,j) ) * ahalf
      vbar(i,j) = ( vtmp(i,j) + vtmp(i,j-1) ) * ahalf
      enddo

c fix longitudinal boundaries of ubar
c -----------------------------------
      im1 = im
      i   = 1
      do J=2,JM-1
      ubar(i,j) = ( utmp(i,j) + utmp(im1,j) ) * ahalf
      enddo
                  
      Return     
      End 
      SUBROUTINE DYCORE ( IM,JM,JDIM,LM,SIGE,PTOP,KM,DT,
     .                    OM,CP,RGAS,AE,PHIS,PKHT,
     .                    PN  ,UN  ,VN  ,TN  ,QN  ,
     .                    PNM1,UNM1,VNM1,TNM1,QNM1,
     .                    DPDT,DUDT,DVDT,DTDT,DQDT,
     .                    OMEGA,VORT,QDIAG )

C **********************************************************************
C  PURPOSE
C     Update Time-Tendencies of Prognostic Fields 
C     due to Hydro-Dynamical Processes
C
C  INPUT ARGUMENT DESCRIPTION
C     IM ...... Number of Grid Increments in Zonal      Direction
C     JM ...... Number of Grid Increments in Meridional Direction
C     JDIM .... Meridional (Second) Dimension of Input  Fields (>=JM+1)
C     LM ...... Number of Vertical Levels
C     SIGE .... SIGE(LM+1): Sigma Edges
C     PTOP .... Model Top Pressure at Sigma = 0.0 
C     KM ...... Number of Scalars to be Advected
C     DT ...... Time-Step (Seconds)
C     OM ...... Rotational Frequency ( 2*PI / ROTATION(Sec) ) 
C     CP ...... Heat Capacity
C     RGAS .... Universal Gas Constant
C     AE . .... Earth Radius
C     PHIS .... PHIS (IM,JDIM): Surface Geopotential Heights (m * m/sec**2)
C     PKHT .... PKHT (IM,JDIM,LM): P**Kappa at model edges starting at SIGE(2)
C
C     PN ...... PN   (IM,JDIM):    Mass (Psurf-Ptop)     at Current  Time-Level (N)
C     UN ...... UN   (IM,JDIM,LM): Zonal      Wind       at Current  Time-Level (N)
C     VN ...... VN   (IM,JDIM,LM): Meridional Wind       at Current  Time-Level (N)
C     TN ...... TN   (IM,JDIM,LM): Potential Temperature at Current  Time-Level (N)
C     QN ...... QN   (IM,JDIM,LM,KM): Scaler Fields      at Current  Time-Level (N)
C
C     PNM1 .... PNM1 (IM,JDIM):    Mass (Psurf-Ptop)     at Previous Time-Level (NM1)
C     UNM1 .... UNM1 (IM,JDIM,LM): Zonal      Wind       at Previous Time-Level (NM1)
C     VNM1 .... VNM1 (IM,JDIM,LM): Meridional Wind       at Previous Time-Level (NM1)
C     TNM1 .... TNM1 (IM,JDIM,LM): Potential Temperature at Previous Time-Level (NM1)
C     QNM1 .... QNM1 (IM,JDIM,LM,KM): Scaler Fields      at Previous Time-Level (NM1)
C
C  OUTPUT ARGUMENT DESCRIPTION
C     DPDT .... DPDT (IM,JDIM):       Updated Surface Pressure   Time-Tendency
C     DUDT .... DUDT (IM,JDIM,LM):    Updated Zonal Wind         Time-Tendency
C     DVDT .... DVDT (IM,JDIM,LM):    Updated Meridional Wind    Time-Tendency
C     DTDT .... DTDT (IM,JDIM,LM):    Updated PI-Weighted Theta  Time-Tendency
C     DQDT .... DQDT (IM,JDIM,LM,KM): Updated PI-Weighted Scalar Time-Tendency
C
C     OMEGA ... OMEGA (IM,JDIM,LM): Vertical Omega Velocity Diagnostic
C     VORT .... VORT  (IM,JDIM,LM): Vorticity Diagnostic
C     QDIAG ... Logical (On/Off) Flag for Diagnostics
C
C  NOTES:
C     (1) JDIM is to be used for DIMENSION Purposes ONLY!
C         Vectorization will be performed using IM and JM.
C     (2) The Vertical Levels are numbered from TOP(1) to BOTTOM(LM).
C     (3) All Time-Tendencies are INCREMENTED (bumped).
C         The Momentum Time-Tendencies ARE NOT mass-weighted.
C         The Potential Temperature and Scalar Time-Tendencies ARE mass-weighted.
C
C **********************************************************************

      real dsig (lm)
      real phis (im,jdim)

      real pn   (im,jdim)
      real pnm1 (im,jdim)
      real un   (im,jdim,lm), vn   (im,jdim,lm)
      real unm1 (im,jdim,lm), vnm1 (im,jdim,lm)
      real tn   (im,jdim,lm), qn   (im,jdim,lm,km)
      real tnm1 (im,jdim,lm), qnm1 (im,jdim,lm,km)

      real   dpdt (im,jdim)
      real   dudt (im,jdim,lm), dvdt (im,jdim,lm)
      real   dtdt (im,jdim,lm), dqdt (im,jdim,lm,km)
      real   pkht (im,jdim,lm)

      logical qdiag
      real    omega (im,jdim,1)
      real    vort  (im,jdim,1)

c Local Dynamic Work Space
c ------------------------
      real pkz  (im,jdim,lm)
      real phiz (im,jdim,lm)
      real conv (im,jdim,lm)

      real tp   (im,jdim)
      real zke  (im,jdim)
      real pvor (im,jdim)
      real a1   (im,jdim)
      real b1   (im,jdim)
      real c1   (im,jdim)
      real d1   (im,jdim)
      real ustar(im,jdim)
      real vstar(im,jdim)
      real tbaru(im,jdim)
      real tbarv(im,jdim)
      real hbarq(im,jdim)
      real vcon (im,jdim)
      real tmp1 (im,jdim)
      real tmp2 (im,jdim)
      real tmp3 (im,jdim)
      real tmp4 (im,jdim)
      real tmp5 (im,jdim)

      real sig  (lm)
      real sige (lm+1)

c Local Static Arrays
c -------------------
#include <vrslv.com>
      real cosij (imr,jmr+1)
      real coshij(imr,jmr+1)
      real fij   (imr,jmr+1)
      real facth (imr,jmr+1)
      real factu (imr,jmr+1)
      real factv (imr,jmr+1)
      real damp  (imr,jmr+1,2)
      integer nj (2)
      integer jf (jmr+1,2)

      save cosij, coshij, fij, facth, factu, factv
      save damp,  nj, jf

      logical first
      data    first /.true./

C **********************************************************************
C ****                     INITIALIZATION                           ****
C **********************************************************************
                                                                        
      imjm   = im* jdim
      imjmm1 = im*(jdim-1)
      imjmm2 = im*(jdim-2)
      imjmm3 = im*(jdim-3)

      PSTD  = GETCON ('PSTD')                                            
      TSTD  = GETCON ('TSTD')                                            
      AKAP  = GETCON ('KAPPA')                                            

      PKSTD = EXPBYK (PSTD)                                             
      THSTD = TSTD/PKSTD                                                

      if (first) then
      call setgrid (IM,JM,JDIM,COSIJ,COSHIJ,FIJ,FACTH,FACTU,FACTV,OM,AE)
      first = .false.
      endif

      call setdmp ( DAMP,NJ,JF,IM,JDIM,DT,AE )

C **********************************************************************
C ****               Compute PKZ at Model Levels                    ****
C **********************************************************************
                                                                        
c     sige(1) = 0.0
      do L=2,lm+1
c     sige(L)   =  sige(L-1)+dsig(L-1)
      dsig(L-1) =  sige(L)-sige(L-1)
       sig(L-1) = (sige(L)+sige(L-1))*.5
      enddo

c     do L=1,LM
c     do I=1,IMJMM2                                                    
c     pkht(I,2,L)  = sige(L+1)*pn(I,2) + ptop                        
c     pkht(I,2,L)  = pkht(I,2,L) ** AKAP                                
c     enddo                                                             
c     enddo                                                             
                                                                        
      call pkap (pn,pkht,pkz,ptop,sige,dsig,im,jm,jdim,lm)

C **********************************************************************
C ****                       COMPUTE D(PHI)                         ****
C **********************************************************************
                                                                        
      do L=1,LM-1                                                    
      do I=1,IMJMM2                                                    
      PHIZ(I,2,L) = (PKHT(I,2,L)-PKZ(I,2, L ))*TN(I,2, L )           
     .            - (PKHT(I,2,L)-PKZ(I,2,L+1))*TN(I,2,L+1)           
      enddo                                                             
      enddo                                                             
                                                                        
C **********************************************************************
C ****                COMPUTE GEOPOTENTIAL HEIGHTS                  ****
C **********************************************************************
                                                                        
      do I=1,IMJMM2                                                    
      PHIZ(I,2,LM) =   PHIS(I,2) + CP*TN(I,2,LM)             
     .             * ( PKHT(I,2,LM) -PKZ(I,2,LM) )              
      enddo                                                             
                                                                        
      do L=LM-1,1,-1                                                 
      do I=1,IMJMM2                                                    
      PHIZ(I,2,L) = PHIZ(I,2,L+1) + CP * PHIZ(I,2,L)                    
      enddo                                                             
      enddo                                                             
                                                                        
C **********************************************************************
C ****           COMPUTE PERTURBATION GEOPOTENTIAL HEIGHTS          ****
C **********************************************************************
                                                                        
      do L=1,LM                                                      
      do I=1,IMJMM2                                                    
      PHIZ(I,2,L) = PHIZ(I,2,L) - (CP*THSTD) * (PKSTD-PKZ(I,2,L))       
      enddo                                                             
      enddo                                                             
                                                                        
C **********************************************************************
C ****              AVERAGE MASS TO VORTICITY POINT                 ****
C **********************************************************************
                                                                        
      j = 2                                                             
      do I=1,IMJMM3                                                     
      hbarq(i,j) = ( (pn(I+1,j+1) + pn(I,j+1) )*COSIJ(I,j+1)                   
     .             + (pn(I+1,j  ) + pn(I,j  ) )*COSIJ(I,j  ) )                 
     .             / (cosij(i,j+1) + cosij(i,j))*.5                          
      enddo                                                             
                                                                        
      do J=2,JDIM-2                                                       
      hbarq(im,j) = ( (pn(1 ,j+1) + pn(im,j+1) )*COSIJ(im,j+1)                  
     .              + (pn(1 ,j  ) + pn(im,j  ) )*COSIJ(im,j) )                  
     .              / (cosij(im,j+1) + cosij(im,j))*.5                        
      enddo                                                             
                                                                        
      hnp = 0.                                                          
      hsp = 0.                                                          
      do I=1,IM                                                         
      hnp = hnp + pn(I,jdim-1)                                             
      hsp = hsp + pn(I,2)                                                
      enddo                                                             
      hnp = hnp/im                                                      
      hsp = hsp/im                                                      
      do i=1,im
      hbarq(i,1)      = hsp
      hbarq(i,jdim-1) = hnp
      enddo
                                                                        
C **********************************************************************
C ****                    HORIZONTAL ADVECTION                      ****
C **********************************************************************
                                                                        
#if macro
cmic$ do all shared (ae, conv, coshij, cosij, cp, dqdt, dtdt)
cmic$*       shared (dudt, dvdt, facth, factu, factv, fij, hbarq)
cmic$*       shared (im, imjmm1, imjmm2, imr, jdim, jm, km, lm)
cmic$*       shared (phiz, pkz, pn, qdiag, qn, thstd, tn, un, vn)
cmic$*       shared (vort)
cmic$*       private (a1, b1, c1, d1, i, k, l, pvor, tbaru, tbarv)
cmic$*       private (tmp1, tmp2, tmp3, tmp4, tmp5, tp, ustar)
cmic$*       private (vstar, zke)
#endif

      do L=1,LM                                                      
                                                                        
c compute perturbation temperatures                                     
c ---------------------------------                                     
      do i=1,IMJMM2                                                      
      tp(i,2) = TN(I,2,L) - THSTD                                    
      enddo                                                             
                                                                        
      call sub1 ( UN(1,1,L),VN(1,1,L),PN(1,1),hbarq,tp,              
     .            cosij,coshij,fij,im,jm,jdim,
     .            zke,pvor,ustar,vstar,tbaru,tbarv, 
     .            tmp1,tmp2,tmp3,tmp4,tmp5,ae )                        
                                                                        
C compute vorticity diagnostic                                           
C ----------------------------                           
      if( qdiag ) then
      do i=1,imjmm1
      vort(i,1,L) = tmp5(i,1)
      enddo
      endif
                                                                  
C compute alpha,beta,gamma,delta and advection                           
C --------------------------------------------                           
      call sub2 ( pvor, a1,b1,c1,d1, im,jdim )            
                                                                        
      call hadvect ( dudt(1,1,L), dvdt(1,1,L), conv(1,1,L),               
     .               factu, factv, facth, a1,b1,c1,d1,
     .               zke, pvor, phiz(1,1,L), pkz(1,1,L),               
     .               ustar,vstar,tbaru,tbarv, im,jdim,                
     .               tmp1,tmp2,cp )                                   
                                                                        
      CALL HADVCTT ( tn(1,1,L),dtdt(1,1,L),ustar,vstar,
     .               facth, im,jdim, tmp1,tmp2,tmp3 )                                   
      do k=1,km
      CALL HADVCTT ( qn(1,1,L,k),dqdt(1,1,L,k),ustar,vstar,
     .               facth, im,jdim, tmp1,tmp2,tmp3 )                                   
      enddo
                                                                        
      enddo                                                             

C construct adiabatic pressure tendency and load pi*sigdot => phiz
C ----------------------------------------------------------------
      do i=1,imjm
      vcon(i,1) = 0.0
      enddo
      do L=1,LM                                                      
      do I=1,IMJMM2                                                    
      vcon(I,2  ) = vcon(I,2) + conv(I,2,L)*dsig(L)                     
      phiz(I,2,L) = vcon(I,2)                                           
      enddo                                                             
      enddo                                                             

C calculate total pressure tendency                                           
C ---------------------------------                                           
      do I=1,IMJMM2                                                    
      dpdt(I,2) = dpdt(I,2) + vcon(I,2)
      enddo                                                             
                                                                  
C compute omega diagnostic                                           
C ------------------------                           
      if(qdiag) then
      call getomega ( pn,un,vn,
     .                im,jdim,lm,ae,cosij,sige,dsig,
     .                phiz(1,1,1),omega(1,1,1) )
      endif

C compute pi*sigdot
C -----------------                                                                 

      do L=1,LM                                                      
      do I=1,IMJMM2                                                    
      phiz(I,2,L) = phiz(I,2,L) - vcon(I,2)*SIGE(L+1)                       
      enddo                                                             
      enddo                                                             

C **********************************************************************
C ****                    VERTICAL ADVECTION                        ****
C **********************************************************************
       
      call vadvct ( pn,un,vn,tn,qn,im,jdim,lm,km,
     .              pkz,pkht,phiz,dsig,dudt,dvdt,dtdt,dqdt )                   
                                                                        
C **********************************************************************
C ****            HIGH-LATITUDE FILTER TIME TENDENCIES              ****
C **********************************************************************

      if( nj(1).ne.0 ) then
      CALL FFTDDT ( DPDT,JF(1,1),DAMP(1,1,1),NJ(1),IM,JDIM,1  )
      CALL FFTDDT ( DUDT,JF(1,1),DAMP(1,1,1),NJ(1),IM,JDIM,LM )
      CALL FFTDDT ( DTDT,JF(1,1),DAMP(1,1,1),NJ(1),IM,JDIM,LM )
      do k=1,km
      CALL FFTDDT ( DQDT(1,1,1,k),JF(1,1),DAMP(1,1,1),NJ(1),IM,JDIM,LM )
      enddo
      endif
      if( nj(2).ne.0 ) then
      CALL FFTDDT ( DVDT,JF(1,2),DAMP(1,1,2),NJ(2),IM,JDIM,LM )
      endif

      return
      end
      SUBROUTINE SETGRID (IM,JM,JDIM,COSIJ,COSHIJ,FIJ,
     .                    FACTH,FACTU,FACTV,OMEGA,A)
                                                                        
      PARAMETER ( ZERO   = 0.00 )                                       
      PARAMETER ( AHALF  = 0.50 )                                       
      PARAMETER ( ONE    = 1.00 )                                       
      PARAMETER ( TWO    = 2.00 )                                       
      PARAMETER ( FOUR   = 4.00 )                                       

      real facth (im,jdim)
      real factu (im,jdim)
      real factv (im,jdim)
      real cosij (im,jdim)
      real coshij(im,jdim)
      real fij   (im,jdim)
                                                                        
      IMJM   = IM* JDIM                                                 
      IMJMM1 = IM*(JDIM-1)                                                
      IMJMM2 = IM*(JDIM-2)                                                
                                                                        
      PI    = FOUR*ATAN(ONE)                                            
      DL    = TWO*PI/ IM                                                
      DP    =     PI/(JDIM-1)                                             
      ADL   = A*DL                                                      
      ADP   = A*DP                                                      
      DLI   = ONE/DL                                                    
      DPI   = ONE/DP                                                    
                                                                        
c define analytic cosines at mass points                                
c --------------------------------------                                
      do j=1,jdim                                                         
            ANG  = (J-1)*DP-PI/2.
            ANGH = ang + DP/2.
      do i=1,im                                                         
      COSIJ(I,J) = COS(ANG)                                            
      enddo                                                             
      enddo                                                             
      do i=1,im                                                         
      COSIJ(I,1)    = zero                                            
      COSIJ(I,jdim) = zero                                            
      enddo                                                             

                                                                        
c define analytic coriolis at v-points                                
c ------------------------------------                                
      do j=1,jdim-1                                                         
          ANG  = (J-1)*DP-PI/2.
          ANGH = ang + DP/2.
      do i=1,im                                                         
      FIJ(I,J) = TWO*OMEGA*SIN(angh)                                 
      enddo                                                             
      enddo                                                             

      do I=1,IM                                                         
      FIJ (I,1 )     = -TWO*OMEGA                                       
      FIJ (I,jdim-1) =  TWO*OMEGA                                       
      enddo                                                             
                                                                        

c average analytic cosines to v-wind points                             
c -----------------------------------------                             
      DO I=1,IM                                                         
      DO J=2,JDIM-2                                                       
      coshIJ (I,J) = AHALF*( cosij(i,j)+cosij(i,j+1) )                    
      enddo                                                             
      coshIJ (I,1)      = ZERO                                               
      coshIJ (I,jdim-1) = ZERO                                               
      enddo                                                             
                                                                        
                                                                        
c compute scale factors                                                 
c ---------------------                                                 
      do i=1,imjmm2                                                     
      FACTH(i,2)  = ONE/(ADL*COSIJ(i,2)*ADP)                             
      FACTU(i,2)  = ONE/(ADL*cosij(i,2))                                 
      enddo                                                             
      do i=1,imjm                                                      
      FACTV(i,1) = ONE/(ADP)                                            
      enddo                                                             
                                                                        
c print map factors
c -----------------
      write(6,1000) 
      do j=jdim,1,-1
      ang = -90 + (j-1.)/(jdim-1.)*180.
      write(6,2000) ang,cosij(1,j),coshij(1,j),fij(1,j),
     .              facth(1,j),factu(1,j),factv(1,j)
      enddo
 1000 format(/,2x,'Lat',4x,'COSIJ',9x,'COSHIJ',11x,'FIJ',
     .        11x,'FACTH',10x,'FACTU',9x,'FACTV',/)
 2000 format(1x,f5.1,2x,6(e12.6,3x))

      RETURN                                                            
      END                                                               
      SUBROUTINE SUB1 ( U,V,H,HBARQ,T, 
     .                  cosij,coshij,fij,IM,JM,JDIM,  
     .                  ZKE,Q,USTAR,VSTAR,TBARU,TBARV,
     .                  ustr2,vstr2,hbaru,hbarv,vor,a )                   
                                                                        
      real     U(IM,JDIM),     V(IM,JDIM)                              
      real     H(IM,JDIM),     T(IM,JDIM)                              
      real   ZKE(IM,JDIM),     Q(IM,JDIM)                              
      real USTAR(IM,JDIM), VSTAR(IM,JDIM)                              
      real TBARU(IM,JDIM), TBARV(IM,JDIM)                              
      real HBARU(IM,JDIM), HBARV(IM,JDIM)                              
      real   VOR(IM,JDIM), HBARQ(IM,JDIM)                            

      real cosij(im,jdim), coshij(im,jdim), fij(im,jdim)
                                                                        
      real ustr2(IM,JDIM), vstr2(IM,JDIM)                              
                                                                        
      PARAMETER ( ZERO   = 0.00 )                                       
      PARAMETER ( AHALF  = 0.50 )                                       
      PARAMETER ( ONE    = 1.00 )                                       
      PARAMETER ( TWO    = 2.00 )                                       
      PARAMETER ( FOUR   = 4.00 )                                       
                                                                        
      IMJM   = IM* JDIM                                                 
      IMJMM1 = IM*(JDIM-1)                                           
      IMJMM2 = IM*(JDIM-2)                                              
      IMJMM3 = IM*(JDIM-3)                                              
      IMJMM4 = IM*(JDIM-4)                                              
                                                                        
      PI    = FOUR*ATAN(ONE)                                            
      DL    = TWO*PI/ IM                                                
      DP    =     PI/(JDIM-1)                                        
      ADL   = A*DL                                                      
      ADP   = A*DP                                                      
      DLI   = ONE/DL                                                    
      DPI   = ONE/DP                                                    
                                                                        
C **********************************************************************
C ****          COMPUTE WINDS FOR SICK-PROOF KINETIC ENERGY         ****
C ***************************)*******************************************

      call polewnd ( u,v,ustar,vstar,im,jdim )
      s1 = 1./3.

      j = 3
      do i=1,imjmm4
      ustr2(i,j) = s1 * u(i,j) * (u(i,j+1)+u(i,j)+u(i,j-1))
      enddo
      do i=1,im
      j = 2
      ustr2(i,j) = s1 * u(i,j) * (u(i,j+1)+u(i,j)+ustar(i,j-1))
      j = jdim-1
      ustr2(i,j) = s1 * u(i,j) * (ustar(i,j+1)+u(i,j)+u(i,j-1))
      enddo

      j = 2
      do i=1,imjmm3
      vstr2(i,j)  = s1 * v(i,j)* (v(i+1,j)+v(i,j)+v(i-1,j))
      enddo
      do j=2,jdim-2
      vstr2(1 ,j) = s1 * v(1 ,j)*(v(2,j)+v(1 ,j)+v(im  ,j))
      vstr2(im,j) = s1 * v(im,j)*(v(1,j)+v(im,j)+v(im-1,j))
      enddo

      do i=1,im
      j = 2
      vstr2(i,j) = s1 * v(i,j)*( v(i+1,j)+v(i-1,j) )
      j = jdim-2
      vstr2(i,j) = s1 * v(i,j)*( v(i+1,j)+v(i-1,j) )
      enddo
      i = 1
      im1 = im
      ip1 = 2
      j = 2
      vstr2(i,j) = s1 * v(i,j)*( v(ip1,j)+v(im1,j) )
      j = jdim-2
      vstr2(i,j) = s1 * v(i,j)*( v(ip1,j)+v(im1,j) )
      i = im
      im1 = im-1
      ip1 = 1
      j = 2
      vstr2(i,j) = s1 * v(i,j)*( v(ip1,j)+v(im1,j) )
      j = jdim-2
      vstr2(i,j) = s1 * v(i,j)*( v(ip1,j)+v(im1,j) )
 

C **********************************************************************
C ****                   COMPUTE KINETIC ENERGY                     ****
C **********************************************************************
       

      j = 3
      do  I=1,IMJMM4
      ZKE(I,j) = 0.25 * ( Ustr2(I  ,j) + Ustr2(I-1,j)
     .                + ( Vstr2(I,j  ) *  coshIJ(I,j)
     .                  + Vstr2(I,j-1) *  coshIJ(I,j-1) )
     .                  / cosij(i,j) )
      enddo

      do  I=1,IM
      j = 2
      ZKE(I,j) = 0.25 * ( ( Ustr2(I,j)+Ustr2(I-1,j) )*1.5
     .                  + ( Vstr2(I,j)* coshIJ(I,j) ) / cosij(i,j) )

      j = jdim-1
      ZKE(I,j) = 0.25 * ( ( Ustr2(I,j)+Ustr2(I-1,j) )*1.5
     .                  + ( Vstr2(I,j-1)*coshIJ(I,j-1) )
     .                    / cosij(i,j) )
      enddo

c fix longitudinal values
c -----------------------
      im1 = im
      i   = 1
      do J=3,JDIM-2
      ZKE(I,j) = 0.25 * ( Ustr2(I  ,j) + Ustr2(Im1,j)
     .                + ( Vstr2(I,j  ) *  coshIJ(I,j)
     .                  + Vstr2(I,j-1) *  coshIJ(I,j-1) )
     .                  / cosij(i,j) )
      enddo
      j = 2
      ZKE(I,j) = 0.25 * ( ( Ustr2(I,j)+Ustr2(Im1,j) )*1.5
     .                  + ( Vstr2(I,j)* coshIJ(I,j) ) / cosij(i,j) )

      j = jdim-1
      ZKE(I,j) = 0.25 * ( ( Ustr2(I,j)+Ustr2(Im1,j) )*1.5
     .                  + ( Vstr2(I,j-1)*coshIJ(I,j-1) )
     .                    / cosij(i,j) )


C **********************************************************************
C ****        COMPUTE POTENTIAL VORTICITY, USTAR, AND VSTAR         ****
C **********************************************************************
                                                                        
      do i=1,imjm
      hbaru(i,1) = (h(i+1,1)+h(i,1))*.5
      tbaru(i,1) = (t(i+1,1)+t(i,1))*.5
      hbarv(i,1) = (h(i  ,2)+h(i,1))*.5
      tbarv(i,1) = (t(i  ,2)+t(i,1))*.5
      enddo

      do j=2,jdim-1
      i   = im
      ip1 = 1
      hbaru(i,j) = (h(ip1,j)+h(i,j))*.5
      tbaru(i,j) = (t(ip1,j)+t(i,j))*.5
      enddo

      call vort ( u,v,vor,cosij,coshij, im,jdim,a )                                       
                                                                        
      do I=1,IMJMM1                                                       
      USTAR(I,2) = HBARU(I,2) * U(I,2) * ADP                            
      VSTAR(I,1) = HBARV(I,1) * V(I,1) * ADL * COShiJ(I,1)              
          Q(I,1) = ( FIJ(I,1)+VOR(I,1) ) / HBARQ(I,1)                       
      enddo                                                             
      do I=1,IM                                                         
      USTAR(I,2)      = HBARU(I,2)      * U(I,2)      * ADP * 1.5             
      USTAR(I,jdim-1) = HBARU(I,jdim-1) * U(I,jdim-1) * ADP * 1.5             
      enddo                                                             
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE SUB2 ( Q, A,B,C,D, IM,JM )                     
      PARAMETER ( ONE = 1.0 )                                           
      PARAMETER ( T2  = ONE/12.0 )                                      
                                                                        
      DIMENSION  Q(IM,JM), A(IM,JM), B(IM,JM), C(IM,JM), D(IM,JM)       
                                                                        
      IMJM   = IM*JM                                                    
                                                                        
C **********************************************************************
C ****          COMPUTE A-F, W, X FOR POTENTIAL VORTICITY           ****
C **********************************************************************
                                                                        
         j = 1                                                          
      do I = 1,IMJM                                                     
      A(I,j) = ( Q(I+1,j  ) + Q(I,j) + Q(I,j-1) )*T2
      B(I,j) = ( Q(I-1,j  ) + Q(I,j) + Q(I,j-1) )*T2
      C(I,j) = ( Q(I-1,j-1) + Q(I,j) + Q(I,j-1) )*T2
      D(I,j) = ( Q(I+1,j-1) + Q(I,j) + Q(I,j-1) )*T2
      enddo                                                             
                                                                        
c fix longitudinal boundaries                                           
c ---------------------------                                           
      do j = 1,jm                                                       
       im1 = im                                                         
       i   = 1                                                          
       ip1 = 2                                                          
      A(I,j) = ( Q(Ip1,j  ) + Q(I,j) + Q(I,j-1) )*T2
      B(I,j) = ( Q(Im1,j  ) + Q(I,j) + Q(I,j-1) )*T2
      C(I,j) = ( Q(Im1,j-1) + Q(I,j) + Q(I,j-1) )*T2
      D(I,j) = ( Q(Ip1,j-1) + Q(I,j) + Q(I,j-1) )*T2
                                                                        
      im1 = im-1                                                        
      i   = im                                                          
      ip1 = 1                                                           
      A(I,j) = ( Q(Ip1,j  ) + Q(I,j) + Q(I,j-1) )*T2
      B(I,j) = ( Q(Im1,j  ) + Q(I,j) + Q(I,j-1) )*T2
      C(I,j) = ( Q(Im1,j-1) + Q(I,j) + Q(I,j-1) )*T2
      D(I,j) = ( Q(Ip1,j-1) + Q(I,j) + Q(I,j-1) )*T2
      enddo                                                             
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE VORT ( U,V,VOR,COSIJ,COSHIJ, IM,JM,A )
      DIMENSION       U(IM,JM), V(IM,JM), VOR(IM,JM)
      DIMENSION  COSIJ (IM,JM)
      DIMENSION  COSHIJ(IM,JM)

      IMJM   = IM* JM
      IMJMM1 = IM*(JM-1)
      IMJMM2 = IM*(JM-2)
      IMJMM3 = IM*(JM-3)
      IMJMM4 = IM*(JM-4)
      IMJMM5 = IM*(JM-5)

      PI    = 4.*ATAN(1.)
      DL    = 2.*PI/ IM
      DP    =    PI/(JM-1)
      ADL   = A*DL
      ADP   = A*DP
      DLI   = 1./DL
      DPI   = 1./DP

C **********************************************************************
C ****                      COMPUTE VORTICITY                       ****
C **********************************************************************

         j=2     
      do I=1,IMJMM3
      vor(i,j) = ( ( V(I+1,j)-V(I,j) )*DLI
     .           - ( U(I,j+1)*COSIJ(I,j+1)
     .              -U(I,j  )*COSIJ(I,j  ) )*DPI ) / ( A*COShIJ(I,j) )
      enddo      
                  
c fix longitudinal boundaries
c ---------------------------
      i   = im   
      ip1 = 1    
      do J=2,JM-2
      vor(i,j) = ( ( V(Ip1,j)-V(I,j) )*DLI
     .           - ( U(I,j+1)*COSIJ(I,j+1)
     .              -U(I,j  )*COSIJ(I,j  ) )*DPI ) / ( A*COShIJ(I,j) )
      enddo      

c compute pole values
c -------------------
      con1 = 2. / (A*IM*cosij(1,2))
       
      sum = 0.
      do I=1,IM
      sum =  sum + U(I,2)
      enddo
      sum = -sum*con1
      do I=1,IM
      vor(I,1) = sum
      enddo

      sum = 0.
      do I=1,IM
      sum =  sum + U(I,jm-1)
      enddo
      sum = sum*con1
      do I=1,IM
      vor(I,jm-1) = sum
      vor(I,jm)   = -999.9
      enddo

      RETURN
      END
      SUBROUTINE HADVECT ( dudt, dvdt, conv,
     .                     factu, factv, facth, A,B,C,D,
     .                     ZKE, Q, PHI, PKZ,
     .                     USTAR,VSTAR,TBARU,TBARV, IM,JM,
     .                     UDOT,VDOT,CP )
 
      DIMENSION  dudt(IM,JM),  dvdt(IM,JM), conv(IM,JM)
      DIMENSION     A(IM,JM),     B(IM,JM),    C(IM,JM),   D(IM,JM)
      DIMENSION   ZKE(IM,JM),     Q(IM,JM),  PHI(IM,JM), PKZ(IM,JM)
      DIMENSION USTAR(IM,JM), VSTAR(IM,JM)
      DIMENSION TBARU(IM,JM), TBARV(IM,JM)
      DIMENSION  UDOT(IM,JM),  VDOT(IM,JM)
      DIMENSION factu(IM,JM)
      DIMENSION factv(IM,JM)
      DIMENSION facth(IM,JM)

      PARAMETER ( ZERO   = 0.00 )
      PARAMETER ( AHALF  = 0.50 )
      PARAMETER ( ONE    = 1.00 )
      PARAMETER ( TWO    = 2.00 )
      PARAMETER ( THREE  = 3.00 )
      PARAMETER ( FOUR   = 4.00 )

      IMJM   = IM* JM
      IMJMM1 = IM*(JM-1)
      IMJMM2 = IM*(JM-2)
      IMJMM3 = IM*(JM-3)
      IMJMM4 = IM*(JM-4)
       
C **********************************************************************
C ****                COMPUTE HEIGHT TENDENCIES                     ****
C **********************************************************************
       
      j = 2
      do I=1,IMJMM2
      conv(I,j) = - ( USTAR(I,j) - USTAR(I-1,j)
     .              + VSTAR(I,j) - VSTAR(I,j-1) )
      enddo
       
c fix longitudinal boundaries
c ---------------------------
      im1 = im
      i   = 1
      do J=2,JM-1
      conv(I,j) = - ( USTAR(I,j) - USTAR(Im1,j)
     .              + VSTAR(I,j) - VSTAR(I,j-1) )
      enddo

C **********************************************************************
C ****                COMPUTE U-WIND TENDENCIES                     ****
C **********************************************************************

      j = 2
      DO  I=1,IMJMM2
            UDOT(I,j)     =  A (I  ,j  ) * VSTAR (I+1,j  )
     .                     + B (I  ,j  ) * VSTAR (I  ,j  )
     .                     + C (I  ,j  ) * VSTAR (I  ,j-1)
     .                     + D (I  ,j  ) * VSTAR (I+1,j-1)
     .                   - (ZKE(I+1,j  ) + PHI   (I+1,j  ) )
     .                   + (ZKE(I  ,j  ) + PHI   (I  ,j  ) )
     .     -cp*tbaru(i,j)* (PKZ(I+1,j  ) - PKZ   (I  ,j  ) )
      ENDDO

c fix longitudinal boundaries
c ---------------------------
      do j=2,jm-1
      im1 = im-1
      i   = im
      ip1 = 1
            UDOT(I,j)     =  A (I  ,j  ) * VSTAR (Ip1,j  )
     .                     + B (I  ,j  ) * VSTAR (I  ,j  )
     .                     + C (I  ,j  ) * VSTAR (I  ,j-1)
     .                     + D (I  ,j  ) * VSTAR (Ip1,j-1)
     .                   - (ZKE(Ip1,j  ) + PHI   (Ip1,j  ) )
     .                   + (ZKE(I  ,j  ) + PHI   (I  ,j  ) )
     .     -cp*tbaru(i,j)* (PKZ(Ip1,j  ) - PKZ   (I  ,j  ) )
       
      im1 = im
      i   = 1
      ip1 = 2
            UDOT(I,j)     =  A (I  ,j  ) * VSTAR (Ip1,j  )
     .                     + B (I  ,j  ) * VSTAR (I  ,j  )
     .                     + C (I  ,j  ) * VSTAR (I  ,j-1)
     .                     + D (I  ,j  ) * VSTAR (Ip1,j-1)
     .                   - (ZKE(Ip1,j  ) + PHI   (Ip1,j  ) )
     .                   + (ZKE(I  ,j  ) + PHI   (I  ,j  ) )
     .     -cp*tbaru(i,j)* (PKZ(Ip1,j  ) - PKZ   (I  ,j  ) )
      enddo
       
C **********************************************************************
C ****                COMPUTE V-WIND TENDENCIES                     ****
C **********************************************************************
       
        j = 2
        DO  I=1,IMJMM3
            VDOT(I,j)     = - C (I  ,j+1) * USTAR (I  ,j+1)
     .                      - D (I-1,j+1) * USTAR (I-1,j+1)
     .                      - A (I-1,j  ) * USTAR (I-1,j  )
     .                      - B (I  ,j  ) * USTAR (I  ,j  )
     .                   - ( ZKE(I  ,j+1) + PHI   (I  ,j+1) )
     .                   + ( ZKE(I  ,j  ) + PHI   (I  ,j  ) )
     .      -cp*tbarv(i,j)*( PKZ(I  ,j+1) - PKZ   (I  ,j  ) )
      ENDDO
       
      DO  J=2,JM-2
      IM1 = IM
      I   = 1
      IP1 = 2
            VDOT(I,j)     = - C (I  ,j+1) * USTAR (I  ,j+1)
     .                      - D (Im1,j+1) * USTAR (Im1,j+1)
     .                      - A (Im1,j  ) * USTAR (Im1,j  )
     .                      - B (I  ,j  ) * USTAR (I  ,j  )
     .                   - ( ZKE(I  ,j+1) + PHI   (I  ,j+1) )
     .                   + ( ZKE(I  ,j  ) + PHI   (I  ,j  ) )
     .      -cp*tbarv(i,j)*( PKZ(I  ,j+1) - PKZ   (I  ,j  ) )
       
      IM1 = IM-1
      I   = IM
      IP1 = 1
            VDOT(I,j)     = - C (I  ,j+1) * USTAR (I  ,j+1)
     .                      - D (Im1,j+1) * USTAR (Im1,j+1)
     .                      - A (Im1,j  ) * USTAR (Im1,j  )
     .                      - B (I  ,j  ) * USTAR (I  ,j  )
     .                   - ( ZKE(I  ,j+1) + PHI   (I  ,j+1) )
     .                   + ( ZKE(I  ,j  ) + PHI   (I  ,j  ) )
     .      -cp*tbarv(i,j)*( PKZ(I  ,j+1) - PKZ   (I  ,j  ) )
      ENDDO
       
C **********************************************************************
C ****                    APPLY SCALE FACTORS                       ****
C **********************************************************************
       
      do i=1,imjmm2
      conv(i,2) =             conv(i,2) * facth(i,2)
      dudt(i,2) = dudt(i,2) + udot(i,2) * factu(i,2)
      dvdt(i,2) = dvdt(i,2) + vdot(i,2) * factv(i,2)
      enddo
       
      RETURN
      END
      SUBROUTINE HADVCTT ( t,dtdt,ustar,vstar,facth, IM,JM,
     .                     tbaru,tbarv,tdot )
      
      DIMENSION     t(IM,JM),  dtdt(IM,JM)
      DIMENSION USTAR(IM,JM), VSTAR(IM,JM)
      DIMENSION TBARU(IM,JM), TBARV(IM,JM)
      DIMENSION  TDOT(IM,JM)
      DIMENSION facth(IM,JM)
       
      PARAMETER ( ZERO  = 0.00  )
      PARAMETER ( AHALF = 0.50  )
       
      IMJM   = IM* JM
      IMJMM1 = IM*(JM-1)
      IMJMM2 = IM*(JM-2)
      IMJMM3 = IM*(JM-3)
      IMJMM4 = IM*(JM-4)
       
C **********************************************************************
C ****                    COMPUTE AVERAGED VALUES                   ****
C **********************************************************************
                                                                        
      do i=1,imjm
      tbaru(i,1) = (t(i+1,1)+t(i,1))*.5
      tbarv(i,1) = (t(i  ,2)+t(i,1))*.5
      enddo

      do j=2,jm-1
      i   = im
      ip1 = 1
      tbaru(i,j) = (t(ip1,j)+t(i,j))*.5
      enddo
       
C **********************************************************************
C ****                 COMPUTE TRACER TENDENCIES                    ****
C **********************************************************************
       
      j = 2
      do I=1,IMJMM2
      tdot(I,j) = - ( USTAR(I  ,j)*tbaru(i  ,j)
     .              - USTAR(I-1,j)*tbaru(i-1,j)
     .              + VSTAR(I  ,j)*tbarv(i  ,j)
     .              - VSTAR(I,j-1)*tbarv(i,j-1) )
      enddo
       
c fix longitudinal boundaries
c ---------------------------
      im1 = im
      i   = 1
      do J=2,JM-1
      tdot(I,j) = - ( USTAR(I  ,j)*tbaru(i  ,j)
     .              - USTAR(Im1,j)*tbaru(im1,j)
     .              + VSTAR(I  ,j)*tbarv(i  ,j)
     .              - VSTAR(I,j-1)*tbarv(i,j-1) )
      enddo
       
C **********************************************************************
C ****                     APPLY SCALE FACTORS                      ****
C **********************************************************************
       
      do i=1,imjmm2
      dtdt(i,2) = dtdt(i,2) + tdot(i,2)*facth(i,2)
      enddo
       
      RETURN
      END
      SUBROUTINE VADVCT ( PZ,UZ,VZ,TZ,QZ,IM,JM,LM,KM,
     .                    PKZ,PKHT,WZ,DSIG,
     .                    DUDT,DVDT,DTDT,DQDT )

C *********************************************************
C ****                                                 ****
C ****  THIS PROGRAM CALCULATES CENTERED 2ND-ORDER     ****
C ****  VERTICAL ADVECTION                             ****
C ****                                                 ****
C ****  ARGUMENTS:                                     ****
C ****    PZ ...... SURFACE PRESSURE - PTOP            ****
C ****    UZ ...... ZONAL WIND                         ****
C ****    VZ ...... MERIDIONAL WIND                    ****
C ****    TZ ...... POTENTIAL TEMPERATURE              ****
C ****    QZ ...... SCALARS (SPECIFIC HUMIDITY)        ****
C ****    IM ...... LONGITUDE DIMENSION                ****
C ****    JM ...... LATITUDE  DIMENSION                ****
C ****    LM ...... VERTICAL  DIMENSION                ****
C ****    KM ...... NUMBER OF SCALARS                  ****
C ****    PKZ ..... P**KAPPA AT MID-LAYERS (PHILLIPS)  ****
C ****    PKHT .... P**KAPPA AT EDGES                  ****
C ****    WZ ...... PI*SIGDOT                          ****
C ****    DSIG .... SIGMA THICKNESS                    ****
C ****                                                 ****
C *********************************************************
C
      DIMENSION   PZ(IM,JM)
      DIMENSION   UZ(IM,JM,LM)
      DIMENSION   VZ(IM,JM,LM)
      DIMENSION   TZ(IM,JM,LM)
      DIMENSION   QZ(IM,JM,LM,KM)
      DIMENSION  PKZ(IM,JM,LM)
      DIMENSION PKHT(IM,JM,LM)
      DIMENSION   WZ(IM,JM,LM)
      DIMENSION DSIG(LM)
                  
      DIMENSION  DUDT(IM,JM,LM)
      DIMENSION  DVDT(IM,JM,LM)
      DIMENSION  DTDT(IM,JM,LM)
      DIMENSION  DQDT(IM,JM,LM,KM)
                  
c Local Dynamic Arrays
c --------------------
      DIMENSION     TMPQ(IM,JM)
      DIMENSION      WZ1(IM,JM)
      DIMENSION      WZ2(IM,JM)
      DIMENSION     DWDZ(IM,JM)
      DIMENSION    pzbar(IM,JM)
      DIMENSION    wzbar(IM,JM)
C       
      PARAMETER ( ZERO   = 0.0    )
      PARAMETER ( AHALF  = 0.5    )
                  
      imjm   = im* jm
      imjmm1 = im*(jm-1)
      imjmm2 = im*(jm-2)
      imjmm3 = im*(jm-3)

C *********************************************************
C ****       VERTICAL ADVECTION OF U MOMENTUM          ****
C *********************************************************
                  
      do I=1,IMJM
      WZ1(I,1) = ZERO
      enddo      
         j=2     
      do I=1,IMJMM2
      pzbar(i,j) = ( pz(i+1,j)  +pz(i,j) )*ahalf
      enddo      
      do J=2,JM-1
      pzbar(im,j) = ( pz(1,j)  +pz(im,j) )*ahalf
      enddo      
                  
      DO 1000 L=1,LM-1
                  
         j=2     
      do I=1,IMJMM2
      wzbar(i,j) = ( wz(i+1,j,l)+wz(i,j,l) )*ahalf
      enddo      
      do J=2,JM-1 
      wzbar(im,j) = ( wz(1,j,l)+wz(im,j,l) )*ahalf
      enddo      
                  
         j=2     
      do I=1,IMJMM2
      TMPQ(I,j)   = (  uz(i,j,l+1)-uz(i,j,l) )
       WZ2(I,j)   = wzbar(I,j)   * TMPQ(I,j)
      DWDZ(I,j)   = ( WZ2(I,j  ) +  WZ1(I,j) )/(2.*dsig(l)*pzbar(i,j))
       WZ1(I,j)   =   WZ2(I,j  )
      DUDT(I,j,L) =  DUDT(I,j,L) - DWDZ(I,j)
      enddo      
 1000 CONTINUE   
                  
         j=2     
      do I=1,IMJMM2
      DUDT(I,j,LM) = DUDT(I,j,LM)
     .               -  WZ1(I,j) / (2.*dsig(LM)*pzbar(i,j))
      enddo      

C *********************************************************
C ****       VERTICAL ADVECTION OF V MOMENTUM          ****
C *********************************************************
                  
      do I=1,IMJM
      WZ1(I,1) = ZERO
      enddo      
         j=2     
      do I=1,IMJMM3
      pzbar(i,j) = ( pz(i,j+1) + pz(i,j) )*ahalf
      enddo      
                  
      DO 2000 L=1,LM-1
                  
         j=2     
      do I=1,IMJMM3
      wzbar(i,j) = ( wz(i,j+1,L) + wz(i,j,L) )*ahalf
      enddo      
                  
         j=2     
      do I=1,IMJMM3
      TMPQ(I,j)   = (  vz(i,j,l+1)-  vz(i,j,l) )
       WZ2(I,j)   = wzbar(I,j)   * TMPQ(I,j)
      DWDZ(I,j)   = ( WZ2(I,j  ) +  WZ1(I,j) )/(2.*dsig(L)*pzbar(i,j))
       WZ1(I,j)   =   WZ2(I,j  )
      DVDT(I,j,L) =  DVDT(I,j,L) - DWDZ(I,j)
      enddo      
 2000 CONTINUE   
                  
         j=2     
      do I=1,IMJMM3
      DVDT(I,j,LM) = DVDT(I,j,LM)
     .               -  WZ1(I,j) / (2.*dsig(LM)*pzbar(i,j))
      enddo      
                  
C *********************************************************
C ****   VERTICAL ADVECTION OF POTENTIAL TEMPERATURE   ****
C *********************************************************
                  
      do I=1,IMJM
      WZ1(I,1) = ZERO
      enddo      
                  
      DO 3000 L=1,LM-1
         j=2     
      do I=1,IMJMM2
      TMPQ(I,j) = ( (PKHT(I,j,L  )- PKZ(I,j,L  ))*TZ(I,j,L  )
     .            - (PKHT(I,j,L  )- PKZ(I,j,L+1))*TZ(I,j,L+1) )
     .          /   ( PKZ(I,j,L+1)- PKZ(I,j,L  ))
       WZ2(I,j  ) =    WZ(I,j,L) * TMPQ(I,j)
      DWDZ(I,j  ) = ( WZ2(I,j  ) -  WZ1(I,j) )/DSIG(L)
       WZ1(I,j  ) =   WZ2(I,j  )
      DTDT(I,j,L) =  DTDT(I,j,L) - DWDZ(I,j)
      enddo      
 3000 CONTINUE   
                  
         j=2     
      do I=1,IMJMM2
      DTDT(I,j,LM) = DTDT(I,j,LM) + WZ1(I,j) / DSIG(LM)
      enddo      
                  
C *********************************************************
C ****        VERTICAL ADVECTION OF MOISTURE           ****
C *********************************************************
                  
      do k=1,KM
      do I=1,IMJM
      WZ1(I,1) = ZERO
      enddo      
                  
      DO 4000 L=1,LM-1
         j=2     
      do I=1,IMJMM2
          TMPQ(I,j) = ( QZ(I,j,L,k) + QZ(I,j,L+1,k) )*AHALF
      IF( TMPQ(I,j).NE.ZERO )
     .    TMPQ(I,j) =   QZ(I,j,L,k) * QZ(I,j,L+1,k) / TMPQ(I,j)
                  
       WZ2(I,j) =    WZ(I,j,L) * TMPQ(I,j)
      DWDZ(I,j) = ( WZ2(I,j  ) -  WZ1(I,j) )/DSIG(L)
       WZ1(I,j) =   WZ2(I,j  )
      DQDT(I,j,L,k) =  DQDT(I,j,L,k) - DWDZ(I,j)
      enddo      
 4000 CONTINUE   
                  
         j=2     
      do I=1,IMJMM2
      DQDT(I,j,LM,k) = DQDT(I,j,LM,k) + WZ1(I,j) / DSIG(LM)
      enddo      
      enddo      
                  
      RETURN     
      END         
      SUBROUTINE PKAP (PN,PKHT,PKZ,PTOP,SIGE,DSIG,IM,JM,JDIM,LM)                    
C***********************************************************************
C  PURPOSE                                                              
C     CALCULATE PHILLIPS P**KAPPA ON C-GRID                             
C  ARGUMENTS                                                            
C     PN ..... SURFACE PRESSURE - PTOP                                  
C     PKHT ... SIGMA EDGE-LEVEL PRESSURE**KAPPA                         
C     PKZ .... SIGMA      LEVEL PRESSURE**KAPPA                         
C     IM . ... LONGITUDE  GRID INCREMENTS                               
C     JM . ... LATITUDE   GRID INCREMENTS                               
C     JDIM ... LATITUDE   DIMENSION OF PZ                               
C     LM . ... VERTICAL   DIMENSION                                     
C  REMARKS:                                                             
C     NOTE:  PERTURBATION GEOPOTENTIALS ARE USED WITH                   
C             TSTD = 280 DEG. K.    ,  PSTD = 1000 MB.                  
C            PKSTD = PSTD ** KAPPA  , THSTD = TSTD / PKSTD              
C                                                                       
C***********************************************************************
C*                GODDARD LABORATORY FOR ATMOSPHERES                    
C***********************************************************************
                                                                        
                                                                        
      PARAMETER (  ONE    = 1.0 )                                       
                                                                        
      REAL PN   (IM,JDIM)                                         
      REAL PKHT (IM,JDIM,LM)                                    
      REAL PKZ  (IM,JDIM,LM)                                    
      REAL DSIG (LM)                                    
      REAL SIGE (LM+1)                                    
                                                                        
c Local Dynamic Storage
c ---------------------
      REAL PINV (IM,JDIM)                                         
      REAL PLE  (IM,JDIM,2)                                       
      REAL PKLE (IM,JDIM,2)                                       
                                                                        
C **********************************************************************
C ****                      INITIALIZATION                          ****
C **********************************************************************
                                                                        
      imjm   = im* jdim
      imjmm1 = im*(jdim-1)
      imjmm2 = im*(jdim-2)

      akap  = getcon('KAPPA')                                     
      akap1 = one + akap
                                                                        
      DO   I=1,IMJMM2                                                 
      PINV(I,2) = ONE / ( akap1*PN(I,2) )                               
      ENDDO
                                                                        
C **********************************************************************
C ****                  COMPUTE PHILLIPS P**KAPPA                   ****
C **********************************************************************
                                                                        
      LUP = 1                                                           
      LDN = 2                                                           
                                                                        
      PKTOP = PTOP**akap                                         

      DO   I=1,IMJMM2                                                 
      PLE (I,2,LUP) = PTOP                                              
      PKLE(I,2,LUP) = PKTOP                                             
      ENDDO                                                          
                                                                        
      DO 1000 L = 1,LM                                               
      DSIGINV = ONE/DSIG(L)                                             

      DO   I=1,IMJMM2                                                 
      PLE (I,2,LDN) = SIGE(L+1)*PN(I,2) + PTOP                          
      PKLE(I,2,LDN) = PKHT(I,2,L)                                       
                                                                        
      PKZ(I,2,L) = ( PLE (I,2,LDN)*PKLE(I,2,LDN)                        
     .             - PLE (I,2,LUP)*PKLE(I,2,LUP) )                      
     .           *   PINV(I,2)*DSIGINV                                  
      ENDDO                                                          
                                                                        
      LSV = LUP                                                         
      LUP = LDN                                                         
      LDN = LSV                                                         
 1000 CONTINUE                                                          
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE SETDMP ( DAMP,NJ,JF,IM,JM,DT,A )
C***********************************************************************
C  PURPOSE         
C     CALCULATES DAMPING COEFFICIENTS FOR HIGH-LATITUDE FILTER
C  USAGE           
C     CALLED BY GCMDRV
C                  
C  ARGUMENTS   DESCRIPTION
C     DAMP ... DAMPING COEFFICIENTS
C     NJ ..... TOTAL NUMBER OF LATITUDES TO FILTER
C     JF ..... LATITUDE INDICES TO FILTER
C     IM ..... ZONAL      DIMENSION
C     JM ..... MERIDIONAL DIMENSION
C     NDT .... TIME-STEP
C                  
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
                   
      PARAMETER ( ZERO    = 0.0      )
      PARAMETER ( AHALF   = 0.5      )
      PARAMETER ( ONE     = 1.0      )
      PARAMETER ( TWO     = 2.0      )
      PARAMETER ( FOUR    = 4.0      )
      PARAMETER ( SIX     = 6.0      )
      PARAMETER ( EIGHT   = 8.0      )
      PARAMETER ( UNDEF   = 999999.0 )
      PARAMETER ( CG      = 400.0    )
      PARAMETER ( ubar    =  80.0    )
      PARAMETER ( alpha   = ubar/cg  )
      PARAMETER ( npow    = 2        )

      DIMENSION    JF (JM,2), NJ(2)
      DIMENSION  DAMP (IM,JM,2)

c Dynamic Local Array
c -------------------
      DIMENSION  W(IM,2)

      DATA DT0   / 9999 /

C *********************************************************************
C ****         CALCULATE HIGH-LATITUDE FILTER COEFFICIENTS         ****
C *********************************************************************

      IF( ABS(DT) .NE. ABS(DT0) ) THEN

      imd2 = im/2
      nlat = 2*(jm-1)/6

      PI  = FOUR*ATAN(ONE)                                            
      DL  = TWO*PI/ IM                                                
      DP  =     PI/(JM-1)                                           
      ADL = A*DL                                                      
      ADP = A*DP                                                      
      DLI = ONE/DL                                                    
      DPI = ONE/DP                                                    
                                                                        
         a1 =      1.5
         a2 = sqrt(1.5)
      thmax = acos( (1.-sqrt(1.+8.*alpha**2))/(4.*alpha**2) )
       wmax = alpha*SIN(thmax) + 2.*sin(thmax/2.)
      DO  K = 1,imd2
      THETA = K * DL
      W(K,1) = one / (ubar*SIN(THETA)    + 2.*cg*sin(theta/2.)   )
      W(K,2) = one / (ubar*SIN(THETA)*a1 + 2.*cg*sin(theta/2.)*a2)
      ENDDO

      print *
      print *, 'Gravity Wave Speed for filter: ',cg
      print *, 'Zonal   Wind Speed for filter: ',ubar
      print *, 'Zonal   Wind / Gravity Speed : ',alpha
      print *, 'Power   Used       for filter: ',npow
      print *, 'Theta   Max        for filter: ',thmax*180./pi
      print *, 'Omega   Max        for filter: ',wmax
      print *

      IF(DT.NE.ZERO) THEN
      ZCRIT = ABS( A*DL/DT )
      ELSE
      ZCRIT = UNDEF
      ENDIF

      FJEQ = (JM+1)*AHALF

c filter coefficients for mass and u-momentum
c -------------------------------------------
      NJ(1) = 0
      DO 30 J = 2,JM-1

          PHI = DP * (J - FJEQ)
       COSPHI = COS(PHI)

      if( j.eq.2 .or. j.eq.jm-1 ) then
      jk = 2
      else
      jk = 1
      endif

      DMIN = ONE
      DO 40  K  = 1,imd2
      DAMP(2*K-1,J,1) = MIN ( ZCRIT*COSPHI*W(K,jk), ONE )
      DAMP(2*K-1,J,1) = damp(2*k-1,j,1)**npow
      DAMP(2*K  ,J,1) = DAMP(2*K-1,J,1)
      DMIN            = MIN ( DMIN, DAMP(2*K,J,1) )
   40 CONTINUE
 
      IF( DMIN.LT.ONE ) THEN
         NJ(1)  = NJ(1) + 1
      JF(NJ(1),1) = J
      PRINT *
      PRINT *,'FFT Filter Coefficients for Mass Points:  J = ',J
      PRINT *
      WRITE(6,100) (DAMP(2*K,J,1),K=1,imd2)
      ENDIF
   30 CONTINUE
 
c filter coefficients for v-momentum
c ----------------------------------
      NJ(2) = 0
      DO 70 J = 2,JM-2
 
         PHI  = -pi/2. + (j-1)*DP + DP/2.
      COSPHI  =  COS(PHI)

      if( j.eq.2 .or. j.eq.jm-2 ) then
      jk = 2
      else
      jk = 1
      endif
 
      DMIN = ONE
      DO 80  K  = 1,imd2
      DAMP(2*K-1,J,2) = MIN ( ZCRIT*COSPHI*W(K,jk), ONE )
      DAMP(2*K-1,J,2) = damp(2*k-1,j,2)**npow
      DAMP(2*K  ,J,2) = DAMP(2*K-1,J,2)
      DMIN            = MIN ( DMIN, DAMP(2*K,J,2) )
   80 CONTINUE
 
      IF( DMIN.LT.ONE ) THEN
         NJ(2)  = NJ(2) + 1
      JF(NJ(2),2) = J
      PRINT *
      PRINT *,'FFT Filter Coefficients for V-momentum Points:  J = ',J
      PRINT *
      WRITE(6,100) (DAMP(2*K,J,2),K=1,imd2)
      ENDIF
   70 CONTINUE
 
      PRINT *
      PRINT *, 'Total no. of MASS   point latitudes to filter: ',NJ(1)
      PRINT *, 'Total no. of V-WIND point latitudes to filter: ',NJ(2)
      PRINT *
 
      DT0 = DT
      ENDIF
  100 FORMAT(5(1X,16(F6.4,2X)/))
      RETURN
      END
      SUBROUTINE FFTDDT ( DQDT,JF,DAMP,NJ,IM,JM,LM )

      DIMENSION   DQDT(IM,JM,LM)
      DIMENSION     JF(IM)
      DIMENSION   DAMP(IM,JM)

c Local Dynamic Arrays
c --------------------
      DIMENSION     PQ(   IM+2, NJ*LM)
      DIMENSION     QP(2*(IM+2),NJ*LM)
                  
c Local Static Arrays
c -------------------
#include <vrslv.com>
      REAL       TRIGS  (3*(IMR+1))
      DIMENSION  IFAX   (100)

      LOGICAL  BEGIN
      DATA     BEGIN /.TRUE./
                  
      SAVE TRIGS, IFAX
                  
C *********************************************************************
C ****                    INITIALIZATION                           ****
C *********************************************************************
                  
      IF (BEGIN)   THEN
      CALL FFTFAX (IM,IFAX,TRIGS)
      BEGIN = .FALSE.
      ENDIF      
                  
C *********************************************************************
C ****                  FILTER TIME TENDENCY                       ****
C *********************************************************************
                  
      IF(NJ.NE.0) THEN
      DO 10 N=1,NJ
       J = JF(N) 
      DO 20 L=1,LM
      L1 = L + (N-1)*LM
      DO 30 I=1,IM
      PQ(I,L1) = DQDT(I,J,L)
   30 CONTINUE   
   20 CONTINUE   
   10 CONTINUE   
                  
C *********************************************************************
C ****             FORWARD AND BACKWARD TRANSFORMS                 ****
C ****                 WITH COEFFICIENT DAMPING.                   ****
C *********************************************************************
                  
      CALL RFFTMLT (PQ,QP,TRIGS,IFAX,1,IM+2,IM,NJ*LM,-1)
                  
      DO 40 N=1,NJ
       J = JF(N) 
      DO 50 L=1,LM
      L1 = L + (N-1)*LM
      DO 60 I=1,IM
      PQ(I+2,L1) = PQ(I+2,L1) * DAMP(I,J)
   60 CONTINUE   
   50 CONTINUE   
   40 CONTINUE   
                  
      CALL RFFTMLT (PQ,QP,TRIGS,IFAX,1,IM+2,IM,NJ*LM, 1)
                  
C *********************************************************************
C ****               RECOMPUTE  TIME TENDENCY                      ****
C *********************************************************************
                  
      DO 70 N=1,NJ
       J = JF(N) 
      DO 80 L=1,LM
      L1 = L + (N-1)*LM
      DO 90 I=1,IM
      DQDT(I,J,L) = PQ(I,L1)
   90 CONTINUE   
   80 CONTINUE   
   70 CONTINUE   
      ENDIF      
                  
      RETURN     
      END         
      SUBROUTINE getomega ( pz,uz,vz,
     .                      im,jm,km,a,cosij,sige,dsig,
     .                      wz,omega )
      DIMENSION     pz(im,jm)
      DIMENSION     uz(im,jm,km)
      DIMENSION     vz(im,jm,km)
      DIMENSION     wz(im,jm,km)
      DIMENSION  omega(im,jm,km)
      DIMENSION cosij (im,jm)
      DIMENSION  sige (km+1)
      DIMENSION  dsig (km)

      parameter ( one   = 1.0 )
      parameter ( two   = 2.0 )
      parameter ( four  = 4.0 )

c Local Dynamic Space
c -------------------
      dimension    pa(im,jm)
      dimension  dpdx(im,jm)
      dimension  dpdy(im,jm)
      dimension  ulm1(im,jm)
      dimension  vlm1(im,jm)
      dimension    ul(im,jm)
      dimension    vl(im,jm)
      dimension ulm1x(im,jm)
      dimension vlm1y(im,jm)
      dimension   ulx(im,jm)
      dimension   vly(im,jm)

      imjmm1 = im*(jm-1)
      imjmm2 = im*(jm-2)
       
      PI    = FOUR*ATAN(ONE)                                            
      DL    = TWO*PI/ IM                                                
      DP    =     PI/(JM-1)                                             

      ADL   = A*DL
      ADP   = A*DP
      DLI   = 1./(2.*adl)
      DPI   = 1./(2.*adp)

C **********************************************************************
C ****                      COMPUTE GRADIENTS                       ****
C **********************************************************************
       
      c1  =  4./3.
      c2  = -1./3.
      do I=1,im*jm
      pa(I,1) = pz(I,1)
      enddo

      do i=1,im
      pa(i,1 ) = c1*pz(i,2   ) + c2*pz(i,3   )
      pa(i,jm) = c1*pz(i,jm-1) + c2*pz(i,jm-2)
      enddo

      pnp = 0.
      psp = 0.
      do i=1,im
      pnp = pnp + pa(i,jm)
      psp = psp + pa(i,1 )
      enddo
      pnp = pnp / im
      psp = psp / im
      do i=1,im
      pa(I,jm) = pnp
      pa(I,1 ) = psp
      enddo
 
         j=2
      do I=1,IMJMM2
      dpdx(i,j) = ( pa(I+1,j)-pa(i-1,j) )*DLI / cosij(i,j)
      dpdy(i,j) = ( pa(I,j+1)-pa(i,j-1) )*DPI
      enddo
       
c fix longitudinal boundaries
c ---------------------------
      do J=2,JM-1
      im1 = im-1
      i   = im
      ip1 = 1
      dpdx(i,j) = ( pa(ip1,j)-pa(im1,j) )*DLI / cosij(i,j)
 
      im1 = im-1
      i   = im
      ip1 = 1
      dpdx(i,j) = ( pa(ip1,j)-pa(im1,j) )*DLI / cosij(i,j)
      enddo

C **********************************************************************
C ****                   COMPUTE VERTICAL VELOCITY                  ****
C **********************************************************************
       
      call polewnd ( uz(1,1,1),vz(1,1,1),ulm1,vlm1,im,jm )
 
      do I=1,imjmm2
      ulm1x(i,2) = ( ulm1(i,2)+ulm1(i-1,2) )*0.5
      vlm1y(i,2) = ( vlm1(i,2)+vlm1(i  ,1) )*0.5
      enddo
      im1 = im
      i   = 1
      do j=2,jm-1
      ulm1x(i,j) = ( ulm1(i,j)+ulm1(im1,j) )*0.5
      enddo
       
      do L = 2,km
      factor = 1./( dsig(l)+dsig(l-1) )
 
      call polewnd ( uz(1,1,L),vz(1,1,L),ul,vl,im,jm )
 
      do I=1,imjmm2
      ulx(i,2) = ( ul(i,2)+ul(i-1,2) )*0.5
      vly(i,2) = ( vl(i,2)+vl(i  ,1) )*0.5
      enddo
      im1 = im
      i   = 1
      do j=2,jm-1
      ulx(i,j) = ( ul(i,j)+ul(im1,j) )*0.5
      enddo
 
      do I=1,imjmm2
      ubar      = ( dsig(l-1)*ulx(i,2) + dsig(l)*ulm1x(i,2) )*factor
      vbar      = ( dsig(l-1)*vly(i,2) + dsig(l)*vlm1y(i,2) )*factor
       
      omega(i,2,L-1) = ( wz(i,2,L-1)
     .               + sige(L)*( ubar*dpdx(i,2)
     .                         + vbar*dpdy(i,2) ) )*86400
      ulm1x(i,2) = ulx(i,2)
      vlm1y(i,2) = vly(i,2)
      enddo
      enddo
 
      L = km
      do I=1,imjmm2
      ubar = ulx(i,2)
      vbar = vly(i,2)
       
      omega(i,2,L) = ( wz(i,2,L)
     .             +         ( ubar*dpdx(i,2)
     .                       + vbar*dpdy(i,2) ) )*86400
      enddo
       
      RETURN
      END
      SUBROUTINE polewnd (uz,vz,ustr,vstr,im,jm)
C***********************************************************************
C  PURPOSE     To compute fictitious winds at and near the pole
C  ARGUMENTS   DESCRIPTION
C     Uz ..... U-WIND on C-Grid
C     Vz ..... V-WIND on C-Grid
C     Ustr ... U-WIND on C-Grid at   pole
C     Vstr ... V-WIND on C-Grid near pole
C
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************


      PARAMETER ( ZERO  = 0.0     )
      PARAMETER ( AHALF = 0.5     )
      PARAMETER ( ONE   = 1.0     )
      PARAMETER ( TWO   = 2.0     )
      PARAMETER ( FOUR  = 4.0     )

      DIMENSION Uz   (im,jm)
      DIMENSION Vz   (im,jm)
      dimension ustr (im,jm)
      dimension vstr (im,jm)
 
c Local Dynamic Space
c -------------------
      dimension cosu (im)
      dimension cosv (im)
      dimension sinu (im)
      dimension sinv (im)
      dimension   vp (im)
       
      imh    = im/2
      imjmm2 = im*(jm-2)

      PI  = FOUR*ATAN(ONE)                                            
      DL  = TWO*PI/IM                                               

      do i=1,im
      cosu(i) = -cos( (i-1+.5)*dl )
      sinu(i) = -sin( (i-1+.5)*dl )
      enddo

      DO I=1,IMH
      COSV(I)     = -COS((I-1)*DL)
      COSV(I+IMH) = -COSV(I)
      SINV(I)     = -SIN((I-1)*DL)
      SINV(I+IMH) = -SINV(I)
      ENDDO

C **********************************************************************
C ****                  compute winds at the pole                   ****
C **********************************************************************
       
      a1 =  2./3.
      a2 = -1./6.
      a3 =  1./9.
       
      do i=1,imjmm2
      ustr(i,2) = uz(i,2)
      vstr(i,2) = vz(i,2)
      enddo
       
      do m=1,2
      ju1 = 1 + (m-1)*(jm-1)
      ju2 = 2 + (m-1)*(jm-3)
      ju3 = 3 + (m-1)*(jm-5)
      jv1 = 1 + (m-1)*(jm-2)
      jv2 = 2 + (m-1)*(jm-4)
       
      uc = 0.0
      us = 0.0
      do i=1,imh
      uc = uc + ( a1*(ustr(i,ju2)-ustr(i+imh,ju2))
     .          + a2*(ustr(i,ju3)-ustr(i+imh,ju3)) )*cosu(i)
      us = us + ( a1*(ustr(i,ju2)-ustr(i+imh,ju2))
     .          + a2*(ustr(i,ju3)-ustr(i+imh,ju3)) )*sinu(i)
      enddo
      do i=1,imh
      uc = uc + ( a1*(ustr(i+imh,ju2)-ustr(i,ju2))
     .          + a2*(ustr(i+imh,ju3)-ustr(i,ju3)) )*cosu(i+imh)
      us = us + ( a1*(ustr(i+imh,ju2)-ustr(i,ju2))
     .          + a2*(ustr(i+imh,ju3)-ustr(i,ju3)) )*sinu(i+imh)
      enddo
       
      uc = uc * 2./im
      us = us * 2./im
      vc =   (-1)**m * us
      vs = - (-1)**m * uc
      do i=1,im
      ustr(i,ju1) = uc*cosu(i) + us*sinu(i)
        vp(i)     = vc*cosv(i) + vs*sinv(i)
      enddo
      do i=1,imh
      vstr(i,jv1) = ( 8.*vp(i) + 2.*vstr(i,jv2) + vstr(i+imh,jv2) )*a3
      enddo
      do i=imh+1,im
      vstr(i,jv1) = ( 8.*vp(i) + 2.*vstr(i,jv2) + vstr(i-imh,jv2) )*a3
      enddo
      enddo
       
      RETURN
      END
      SUBROUTINE MOISTIO (N,NDMOIST,QBUDGET)
C
#include <vrslv.com>
#include <vstate.com>
#include <vphyscom.com>
#include <vmetric.com>
#include <vbudget.com>
#include <stripmine.com>

      PARAMETER ( ZERO     =     0.0 )
      PARAMETER ( ONE      =     1.0 )
      PARAMETER ( THOUSAND =  1000.0 )

      PARAMETER ( IMJMR   = IMR*(JMR+1) )
      parameter (ncrnd = 10)
C
      dimension save1(istrip,nlayr),save2(istrip,nlayr)
      dimension save3(istrip,nlayr)
C
      DIMENSION   PZINV(IMR,JMR+1)  , TMPDIAG(IMR,JMR+1)
      DIMENSION   PKZ(IMR,JMR+1,NLAYR)
      DIMENSION   CLSM(IMR,JMR+1,NLAYR)  , CPENM(IMR,JMR+1,NLAYR)
      DIMENSION   CLSB(IMR,JMR+1,NLAYR)  , CPENB(IMR,JMR+1,NLAYR)


      DIMENSION     PLE(ISTRIP,NLAYR+1), GAM(ISTRIP,NLAYR)
      DIMENSION      TL(ISTRIP,NLAYR)  , SHL(ISTRIP,NLAYR)
      DIMENSION      PL(ISTRIP,NLAYR)  , PLK(ISTRIP,NLAYR)
      DIMENSION    PLKE(ISTRIP,NLAYR+1)
      DIMENSION      TH(ISTRIP,NLAYR)  ,CVTH(ISTRIP,NLAYR)
      DIMENSION   SHSAT(ISTRIP,NLAYR)  , CVQ(ISTRIP,NLAYR)
      DIMENSION  CLMAXO(ISTRIP,NLAYR),CLBOTH(ISTRIP,NLAYR)
      DIMENSION  CLSMAX(ISTRIP,NLAYR),CLSBTH(ISTRIP,NLAYR)
      DIMENSION    TMP1(ISTRIP,NLAYR),  TMP2(ISTRIP,NLAYR)
      DIMENSION    TMP3(ISTRIP,NLAYR),  TMP4(ISTRIP,NLAYR+1)
      DIMENSION   ITMP1(ISTRIP,NLAYR), ITMP2(ISTRIP,NLAYR)


      DIMENSION   PRECIP(ISTRIP), PCMID(ISTRIP), PCNET(ISTRIP)
      DIMENSION   PCLOW(ISTRIP), SP(ISTRIP),  PREP(ISTRIP)
      DIMENSION   PCPEN(ISTRIP,NLAYR)


      DIMENSION   RASIL1(ISTRIP,NLAYR), RASIL2(ISTRIP,NLAYR)
      DIMENSION   RASIL3(ISTRIP,NLAYR), RASIL4(ISTRIP,NLAYR)
      DIMENSION   RASIL5(ISTRIP,NLAYR), RASIL6(ISTRIP,NLAYR)
      DIMENSION   RASIL7(ISTRIP,NLAYR), RASIL8(ISTRIP,NLAYR)
      DIMENSION   RASIL9(ISTRIP,NLAYR), RASIL10(ISTRIP,NLAYR)
      DIMENSION   RASIL11(ISTRIP,NLAYR), RASIL12(ISTRIP,NLAYR)
      DIMENSION   RASIL13(ISTRIP,NLAYR)


      DIMENSION   RASI1(ISTRIP), RASI2(ISTRIP), RASI3(ISTRIP)
      DIMENSION   RASI4(ISTRIP), RASI5(ISTRIP), RASI6(ISTRIP)
      DIMENSION   RASI7(ISTRIP), RASI8(ISTRIP), RASI9(ISTRIP)
      DIMENSION   RASI10(ISTRIP), RASI11(ISTRIP), RASI12(ISTRIP)
      DIMENSION   RASI13(ISTRIP), RASI14(ISTRIP), RASI15(ISTRIP)
      DIMENSION   RASI16(ISTRIP)

      LOGICAL      QBUDGET
      LOGICAL      FIRST
      DIMENSION   QSAVE(ISTRIP,NLAYR)
      DIMENSION   PHISIM(ISTRIP),CLFRAC(ISTRIP,NLAYR)
      dimension rnd(ncrnd)
      DATA      FIRST /.TRUE./
C
C **********************************************************************
C ****                     INITIALIZATION                           ****
C **********************************************************************

      IMSTP  = MODHMS(NDMOIST,0)
      TMSTP  = FLOAT(IMSTP)
      TMINV  = ONE / TMSTP
      SDAY   = GETCON('SDAY')
      GRAV   = GETCON('GRAVITY')
      ALHL   = GETCON('LATENT HEAT COND')
      CP     = GETCON('CP')
      ELOCP  = GETCON('LATENT HEAT COND') / GETCON('CP')
      GAMFAC = GETCON('LATENT HEAT COND') * GETCON('EPS') * ELOCP
     .       / GETCON('RGAS')
      RKAPPA = GETCON('KAPPA')
      P0KAPPA  =  1000.0**RKAPPA
      P0KINV   =  ONE / P0KAPPA
C
      NLTOP = 1
      DO L=1,NLAY
       PCHECK = (1000.-ptop)*SIG(L) + PTOP
       IF (PCHECK.GE.10.) THEN
        NLTOP = L
        GO TO 2
       ENDIF
      ENDDO
 2    CONTINUE

      if(first)then
       PRINT *,'Top Layer allowed for Convection: ',NLTOP
       first = .false.
      endif

c bump the counter for number of calls to convection
      iras = iras + 1

c select the 'random' cloud detrainment levels for RAS
      call rndcloud(iras,ncrnd,rnd)
C
      DO 815 L=1,NLAYR
       DO 816 I=1,IMJMR
        DTMOIST(I,1,L) = ZERO
        DQMOIST(I,1,L) = ZERO
  816  CONTINUE
       DO 676 I=1,ISTRIP
        cvth(I,L) = ZERO
        cvq(I,L) = ZERO
  676  CONTINUE
  815 CONTINUE

C **********************************************************************
C ****                    COMPUTE P**KAPPA                          ****
C **********************************************************************

      CALL PKAPPA ( PZ(1,1,N),PKHT,PKZ,IM,JNP,NLAY )

C***********************************************************************
C ****                     LOOP OVER NPCS PEICES                    ****
C **********************************************************************

#if macro
cmic$ do all shared (clsb, clsm, cpenb, cpenm, dqmoist, dsig, dtmoist)
cmic$*       shared (icldmas, idtrain, im, imjmr, imoistq, imoistt)
cmic$*       shared (imr, ipreacc, iprecon, istrip, jmr, jnp, n)
cmic$*       shared (ncrnd, nlay, nlayp1, nlayr, npcs, nprogs2)
cmic$*       shared (nprogu2, one, phisz, pkht, pkz, ptop, pz)
cmic$*       shared (qdiag, qz, rnd, sig, sige, thousand, tz, zcconq)
cmic$*       shared (zccont, zcpenq, zcpent, zero)
cmic$*       shared (alhl, cp, gamfac, grav, nltop, p0kappa, p0kinv)
cmic$*       shared (qbudget, sday, rkappa, tmstp)
cmic$*       private (clboth, clfrac, clmaxo, clsbth, clsmax)
cmic$*       private (cvq, cvth, i, ii, itmp1)
cmic$*       private (itmp2, l, ll, nlras, nn, nsubcl)
cmic$*       private (pcnet, pcpen, phisim, pl, ple, plk)
cmic$*       private (plke, precip, prep, qsave, rasi1)
cmic$*       private (rasi10, rasi11, rasi12, rasi13, rasi14, rasi15)
cmic$*       private (rasi16, rasi2, rasi3, rasi4, rasi5, rasi6)
cmic$*       private (rasi7, rasi8, rasi9, rasil1, rasil2, rasil3)
cmic$*       private (rasil4, rasil5, rasil6, rasil7, rasil8)
cmic$*       private (save1, save2, save3, shl, sp, th, tl)
cmic$*       private (tmp1, tmp2, tmp3, tmp4, tommpd)
#endif

      DO 1000 NN = 1,NPCS

C **********************************************************************
C ****                   VARIABLE INITIALIZATION                    ****
C **********************************************************************

       CALL STRIP ( PZ(1,1,N),SP   ,IMJMR,ISTRIP,1   ,NN)
       CALL STRIP ( PKZ,PLK        ,IMJMR,ISTRIP,NLAY,NN)
       CALL STRIP ( PHISZ(1,1),PHISIM,IMJMR,ISTRIP,1 ,NN)
       CALL STRIP ( PKHT,PLKE      ,IMJMR,ISTRIP,NLAY,NN)
       CALL STRIP ( TZ(1,1,1,N),TH ,IMJMR,ISTRIP,NLAY,NN)
       CALL STRIP ( QZ(1,1,1,N),SHL,IMJMR,ISTRIP,NLAY,NN)

      DO 10 L=1,NLAY
      DO 11 I=1,ISTRIP
         PL(I,L) =  SIG(L)*SP(I) + PTOP
        PLE(I,L) = SIGE(L)*SP(I) + PTOP
   11 CONTINUE
   10 CONTINUE
      DO 19 I = 1,ISTRIP
       PLE(I,NLAYP1) =    SP(I) + PTOP
  19  CONTINUE

C **********************************************************************
C ****                  CUMULUS PARAMETERIZATION                    ****
C **********************************************************************

       DO 901 L = 1,NLAY
        DO 900 I = 1,ISTRIP
         TH(I,L) =   TH(I,L) * P0KAPPA
  900   CONTINUE
  901  CONTINUE
      DO 15 L=1,NLAY
       DO 16 I=1,ISTRIP
        CVTH(I,L) = ZERO
        CVQ(I,L) = ZERO
        CLMAXO(I,L) = ZERO
        CLBOTH(I,L) = ZERO
   16  CONTINUE
   15 CONTINUE

       DO 600 I=1,ISTRIP
        TMP1(I,1) = ONE / SP(I)
        TMP1(I,2) = ZERO
        TMP1(I,3) = ZERO
 600   CONTINUE

       NSUBCL = NLAY-1
       NLRAS = NSUBCL - NLTOP + 1
       DO 601 L=NSUBCL,NLAY
       DO 602 I=1,ISTRIP
        TMP1(I,2) = TMP1(I,2) + (PLE(I,L+1)-PLE(I,L))*TMP1(I,1)*TH(I,L)
        TMP1(I,3) = TMP1(I,3) + (PLE(I,L+1)-PLE(I,L))*TMP1(I,1)*SHL(I,L)
 602   CONTINUE
 601   CONTINUE
       DO 603 I=1,ISTRIP
        TMP1(I,4) = ONE / ( (PLE(I,NLAYP1)-PLE(I,NSUBCL))*TMP1(I,1) )
        TH(I,NSUBCL) = TMP1(I,2) *TMP1(I,4)
        SHL(I,NSUBCL) = TMP1(I,3) * TMP1(I,4)
 603   CONTINUE
       DO 111 L=1,NLAY
       DO 112 I=1,ISTRIP
        TMP4(I,L) = PLE(I,L)
 112   CONTINUE
 111   CONTINUE
       DO  I=1,ISTRIP
        TMP4(I,NLAY+1) = PLE(I,NLAY+1)
       ENDDO
       DO 113 I=1,ISTRIP
        TMP4(I,NSUBCL+1) = PLE(I,NLAY+1)
 113   CONTINUE

       do 989 ll = 1,nlay
       do 988 ii = 1,istrip
        save1(ii,ll)=th(ii,ll)
        save2(ii,ll)=shl(ii,ll)
        save3(ii,ll)=ple(ii,ll)
 988   CONTINUE
 989   CONTINUE

      DO 115 I=1,ISTRIP*NLAYR
       PCPEN(I,1)  = ZERO
       CLFRAC(I,1)  = ZERO
  115 CONTINUE

      CALL RASG(nn,ISTRIP,ISTRIP,NLRAS,TMSTP,TH(1,NLTOP),SHL(1,NLTOP)
     1, TMP4(1,NLTOP), rnd, ncrnd, PCPEN(1,NLTOP), CLBOTH(1,NLTOP)
     2, CLMAXO(1,NLTOP), CVTH(1,NLTOP), CVQ(1,NLTOP), CLFRAC(1,NLTOP)
     3, RASIL1,RASIL2,RASIL3,RASIL4,RASIL5,RASIL6,RASIL7,RASIL8
     4, RASI1,RASI2,RASI3,RASI4,RASI5,RASI6,RASI7,RASI8,RASI9,RASI10
     5, RASI11,RASI12,RASI13,RASI14,RASI15,RASI16
     6, ITMP1,ITMP2,TMP1,TMP2,TMP3,qdiag,icldmas,idtrain
     7, cp,grav,rkappa,alhl)

       DO 607 L=1,NLAY
       DO 608 I=1,ISTRIP
        CVTH(I,L) =  TH(I,L) - save1(i,l)
        CVQ (I,L) = SHL(I,L) - save2(i,l)
 608   CONTINUE
 607   CONTINUE
       DO 517 L=nsubcl+1,nlay
       DO 518 I=1,ISTRIP
        CVTH(I,L) = cvth(i,nsubcl)
        CVQ (I,L) = cvq(i,nsubcl)
 518   CONTINUE
 517   CONTINUE
      DO 905 L=1,NLAY
       DO 906 I=1,ISTRIP
        CVTH(I,L) = CVTH(I,L)*P0KINV*SP(I)/TMSTP
        CVQ (I,L) = CVQ (I,L)*SP(I)/TMSTP
  906  CONTINUE
  905 CONTINUE

C **********************************************************************
C ****                     TENDENCY UPDATES                         ****
C **********************************************************************

      DO 904 L=1,NLAY
       CALL PSTBMP(CVTH(1,L),DTMOIST(1,1,L),ISTRIP,IMJMR,1,NN)
       CALL PSTBMP(CVQ(1,L),DQMOIST(1,1,L),ISTRIP,IMJMR,1,NN)
  904 CONTINUE

C **********************************************************************
C ****                     BUDGETS (FILL CONVECTIVE AND PEN)        ****
C **********************************************************************

      DO 505 L=1,NLAY
       DO 506 I=1,ISTRIP
        CVTH(I,L) = CVTH(I,L) * TMSTP
        CVQ (I,L) = CVQ (I,L) * TMSTP
  506  CONTINUE
  505 CONTINUE
      IF(QBUDGET)THEN
       DO 4001 L = 1,NLAY
        CALL BDGBMP(CVTH(1,L),ZCCONT(1,L),ISTRIP,IM,JNP,1,NN)
        CALL BDGBMP(CVTH(1,L),ZCPENT(1,L),ISTRIP,IM,JNP,1,NN)
        CALL BDGBMP(CVQ(1,L),ZCCONQ(1,L),ISTRIP,IM,JNP,1,NN)
        CALL BDGBMP(CVQ(1,L),ZCPENQ(1,L),ISTRIP,IM,JNP,1,NN)
 4001  CONTINUE
      ENDIF

C **********************************************************************
C                          BUMP DIAGNOSTICS
C **********************************************************************

        IF((IMOISTT.GT.0).OR.(IMOISTQ.GT.0))THEN
         DO 702 L=1,NLAY
         DO 700 I = 1,ISTRIP
          TMP1(I,L) = CVTH(I,L) * SDAY * PLK(I,L) / (SP(I)*TMSTP)
          TMP2(I,L) = CVQ (I,L) * SDAY * THOUSAND / (SP(I)*TMSTP)
 700     CONTINUE
        IF(IMOISTT.GT.0)CALL PSTBMP(TMP1(1,L),QDIAG(1,1,IMOISTT+L-1),
     1        ISTRIP,IMJMR,1,NN)
        IF(IMOISTQ.GT.0)CALL PSTBMP(TMP2(1,L),QDIAG(1,1,IMOISTQ+L-1),
     1        ISTRIP,IMJMR,1,NN)
 702     CONTINUE
       ENDIF

C *********************************************************************
C ****         RE-EVAPORATION OF PENETRATING CONVECTIVE RAIN       ****
C *********************************************************************

       CALL STRIP ( TZ(1,1,1,N),TH ,IMJMR,ISTRIP,NLAY,NN)
       CALL STRIP ( QZ(1,1,1,N),SHL,IMJMR,ISTRIP,NLAY,NN)

       DO  L = 1,NLAY
        DO I = 1,ISTRIP
          TH(I,L) =  TH(I,L) + CVTH(I,L)/SP(I)
         SHL(I,L) = SHL(I,L) +  CVQ(I,L)/SP(I)
        enddo
       enddo
      DO 902 L=1,NLAY
       DO 903 I=1,ISTRIP
         TL(I,L) = TH(I,L)*PLK(I,L)
        QSAVE(I,L) = SHL(I,L)
        CVTH(I,L) = ZERO
        CVQ(I,L) = ZERO
  903  CONTINUE
  902 CONTINUE
C
       CALL RNEVP (NN,ISTRIP,NLAY,TL,SHL,PCPEN,PL,CLFRAC,SP,DSIG,PLKE,
     .  PLK,TH,PHISIM,TMP1,TMP2,TMP3,ITMP1,ITMP2,PCNET,PRECIP,QBUDGET,
     .  CLSMAX,CLSBTH,TMSTP,ONE,cp,grav,alhl,gamfac)
C
C **********************************************************************
C ****                     TENDENCY UPDATES                         ****
C **********************************************************************
C
      DO 804 L=1,NLAY
      DO 9054 I =1,ISTRIP
      TMP1(I,L) =   TL(I,L) /  PLK(I,L)
      TMP1(I,L) = TMP1(I,L) - TH(I,L)
      TMP1(I,L) = SP(I) * TMP1(I,L) / TMSTP
9054  CONTINUE
       CALL PSTBMP(TMP1(1,L),DTMOIST(1,1,L),ISTRIP,IMJMR,1,NN)
      DO 9056 I =1,ISTRIP
      TMP1(I,L) = SHL(I,L)-QSAVE(I,L)
      TMP1(I,L) = SP(I) * TMP1(I,L) / TMSTP
9056  CONTINUE
       CALL PSTBMP(TMP1(1,L),DQMOIST(1,1,L),ISTRIP,IMJMR,1,NN)
  804 CONTINUE
C
C
C **********************************************************************
C                          BUMP DIAGNOSTICS
C **********************************************************************

        IF((IMOISTT.GT.0).OR.(IMOISTQ.GT.0))THEN
         DO L=1,NLAY
      DO I =1,ISTRIP
      TMP1(I,L) =   TL(I,L) /  PLK(I,L)
      TMP1(I,L) = TMP1(I,L) - TH(I,L)
      TMP1(I,L) = SP(I) * TMP1(I,L) / TMSTP
      TMP1(I,L) = TMP1(I,L) / sp(i) * plk(i,l) * sday
      ENDDO
        IF(IMOISTT.GT.0)CALL PSTBMP(TMP1(1,L),QDIAG(1,1,IMOISTT+L-1),
     1        ISTRIP,IMJMR,1,NN)
      DO 9057 I =1,ISTRIP
      TMP2(I,L) = SHL(I,L)-QSAVE(I,L)
      TMP2(I,L) = SP(I) * TMP2(I,L) / TMSTP
      TMP2(I,L) = TMP2(I,L) / sp(i) * sday * THOUSAND
9057  CONTINUE
        IF(IMOISTQ.GT.0)CALL PSTBMP(TMP2(1,L),QDIAG(1,1,IMOISTQ+L-1),
     1        ISTRIP,IMJMR,1,NN)
         enddo
       ENDIF
C
C *********************************************************************
C ****                PASTE CLOUD AMOUNTS                          ****
C *********************************************************************

      CALL PASTE(CLSMAX,CLSM,ISTRIP,IMJMR,NLAY,NN)
      CALL PASTE(CLSBTH,CLSB,ISTRIP,IMJMR,NLAY,NN)
      CALL PASTE(CLMAXO,CPENM,ISTRIP,IMJMR,NLAY,NN)
      CALL PASTE(CLBOTH,CPENB,ISTRIP,IMJMR,NLAY,NN)
 
C *********************************************************************
C ****                  PRECIPITATION DIAGNOSTICS                  ****
C ****    PREACC(I,J) = TOTAL ACCUMULATED PRECIP FROM LARGE AND    ****
C ****                  SMALL SCALE TYPES (MM/DAY)                 ****
C ****    PRECON(I,J) = TOTAL ACCUMULATED PRECIP FROM CUMULUS      ****
C ****                  CONVECTION TYPES (MM/DAY)                  ****
C *********************************************************************
C
      TOMMPD = SDAY / TMSTP
      DO 21 I = 1,ISTRIP
       PREP(I) = ( PRECIP(I) + PCNET(I) ) * TOMMPD
  21  CONTINUE
C
      IF(IPREACC.GT.0)CALL PSTBMP(PREP,QDIAG(1,1,IPREACC),ISTRIP,
     1        IMJMR,1,NN)
      DO I = 1,ISTRIP
       PREP(I) = PCNET(I) * TOMMPD
      enddo
      IF(IPRECON.GT.0)CALL PSTBMP(PREP,QDIAG(1,1,IPRECON),ISTRIP,
     1        IMJMR,1,NN)
C
 1000 CONTINUE
C
C **********************************************************************
C ****      SET MEAN CLOUD ARRAYS BASED ON CONV AND LS CLOUDS       ****
C **********************************************************************
C
      DO 20 L=1,NLAY
       DO 24 I = 1,IMJMR
        IF( CLSM(I,1,L) .GT. CPENM(I,1,L) ) THEN
         CSWMO(I,1,L) = CSWMO(I,1,L) + CLSM(I,1,L)
         CLWMO(I,1,L) = CLWMO(I,1,L) + CLSM(I,1,L)
        ELSE
         CSWMO(I,1,L) = CSWMO(I,1,L) + CPENM(I,1,L)
         CLWMO(I,1,L) = CLWMO(I,1,L) + CPENM(I,1,L)
        ENDIF
        IF ( (CLSM(I,1,L).EQ.ZERO).AND.(CPENM(I,1,L).EQ.ZERO) ) THEN
         IF ( CLSB(I,1,L) .GT. CPENB(I,1,L) ) THEN
          CSWRO(I,1,L) = CSWRO(I,1,L) + CLSB(I,1,L)
          CLWRO(I,1,L) = CLWRO(I,1,L) + CLSB(I,1,L)
         ELSE
          CSWRO(I,1,L) = CSWRO(I,1,L) + CPENB(I,1,L)
          CLWRO(I,1,L) = CLWRO(I,1,L) + CPENB(I,1,L)
         ENDIF
        ENDIF
   24  CONTINUE
   20 CONTINUE
C
C **********************************************************************
C ****                      INCREMENT COUNTERS                      ****
C **********************************************************************
C
       NLWCLD   = NLWCLD   + 1
       NSWCLD   = NSWCLD   + 1

       NMOISTT  = NMOISTT  + 1
       NMOISTQ  = NMOISTQ  + 1
       NPREACC  = NPREACC  + 1
       NPRECON  = NPRECON  + 1

       NFREQMD  = NFREQMD  + 1
       NFREQLW  = NFREQLW  + 1
       NFREQPN  = NFREQPN  + 1

       NCLDMAS  = NCLDMAS  + 1
       Ndtrain  = Ndtrain  + 1
C
      RETURN
      END
      subroutine rndcloud (iras,nrnd,rnd)
      parameter (irm = 1000)
      dimension rnd(nrnd)
      dimension random(irm)
      logical first
      data first /.true./
      data iras0 /0/
      save random, iras0
C
      if(nrnd.eq.0.)then
       do i = 1,nrnd
        rnd(i) = 0
       enddo
       if(first) print *,' NO RANDOM CLOUDS IN RAS '
       go to 100
      endif

      mcheck = mod(iras-1,irm/nrnd)

c First Time In From a Continuing RESTART (IRAS.GT.1) or Reading a New RESTART
c ----------------------------------------------------------------------------
      if( first.and.(iras.gt.1) .or. iras.ne.iras0+1 )then
      print *, 'Recreating Random Number Array in RNDCLOUD'
      print *, 'IRAS: ',iras,'  IRAS0: ',iras0
       numrand = mod(iras,irm/nrnd) * nrnd
       iseed   = iras * nrnd - numrand
       call ranset(iseed)
       do i = 1,irm
        random(i) = ranf()
       enddo
       index = (iras-1)*nrnd

c Multiple Time In But have Used Up all 1000 numbers (MCHECK.EQ.0)
c ----------------------------------------------------------------
      else if (mcheck.eq.0) then
          iseed = (iras-1)*nrnd
          call ranset(iseed)
          do i = 1,irm
           random(i) = ranf()
          enddo
          index = iseed

c Multiple Time In But have NOT Used Up all 1000 numbers (MCHECK.NE.0)
c --------------------------------------------------------------------
      else
          index = (iras-1)*nrnd
      endif

      index = mod(index,irm)
C
      do n = 1,nrnd
       rnd(n) = random(index+n)
      enddo
C
 100  continue
      first = .false.
      iras0 = iras
      return
      end
      SUBROUTINE RASG(nn,LEN, LENC,   K,   DT
     *,               POI, QOI, PRS, rnd, ncrnd
     *,               PCU, CLN, CLNMAX, TCU, QCU, CLF
     *,               ALF, BET, GAM, PRH, PRJ, PRI, HOI, ETA
     *,               TX1, TX2,    TX3, TX4, TX5, TX6, TX7, TX8, TX9
     *,               TXA, TXB, TXC, TXD, TXE
     *,               WFN, AKM,ITMP1,ITMP2,TMP1,TMP2,TMP3
     *,               qdiag,imf,idet,cp,grav,rkappa,alhl)
C
C*********************************************************************
C*********************** ARIES   MODEL *******************************
C********************* SUBROUTINE  RAS   *****************************
C********************** 16 MARCH   1988 ******************************
C*********************************************************************
C
#include <vrslv.com>
#include <vbudget.com>
#include <stripmine.com>
      PARAMETER ( IMJMR   = IMR*(JMR+1) )
      PARAMETER ( lnz     =  (IMJMR-1) / NPCS + 1  )
      PARAMETER ( lnzc    =  lnz )
      PARAMETER ( kz      =  nlayr )
C
      PARAMETER (KRMIN=01, KRMAX=NLAYR-3)
      PARAMETER (ICM=1000)
      PARAMETER (ONE=1.0, THOUSAND=1000.)
C
      DIMENSION PII(lnz),  POI(lnz,Kz), QOI(lnz,Kz), PRS(lnz,Kz+1)
      dimension rnd(ncrnd)
C
      DIMENSION CLN(lnz,Kz)
     *,         TCU(lnz,Kz), QCU(lnz,Kz), PCU(lnz,Kz), CLF(lnz,Kz)
     *,         CLNMAX(lnz,Kz)
C
      DIMENSION ALF(lnz,Kz), BET(lnz,Kz), GAM(lnz,Kz)
     *,         ETA(lnz,Kz), HOI(lnzc,Kz)
     *,         PRH(lnz,Kz), PRJ(lnzc,Kz), PRI(lnz,Kz)
C
      DIMENSION TX1(lnzc), TX2(lnzc), TX3(lnzc), TX4(lnzc), TX5(lnzc)
     *,         TX6(lnzc), TX7(lnzc), TX8(lnzc), TX9(lnzc), TXA(lnzc)
     *,         TXB(lnzc), TXC(lnzc), TXD(lnzc), TXE(lnzc)
     *,         WFN(lnzc), AKM(lnzc), ITMP1(lnzc), ITMP2(lnzc)
     *,         TMP1(lnzc), TMP2(lnzc), TMP3(lnzc)
C
      DIMENSION SGE(nlayr+1), IC(ICM),   IRND(icm)
      dimension qdiag(imr,jmr+1,1), cmass(lnz,kz)
      dimension save4(lnz,kz),save5(lnz,kz)
      dimension rains(lnz,kz)
      LOGICAL SETRAS
C
         DO 1150 I=1,LENC
         do 1151 L = 1,k
         save4(i,L)=poi(i,L)
         save5(i,L)=qoi(i,L)
         rains(i,l) = 0.
 1151    continue
 1150    CONTINUE
C
      P0KINV   =  ONE / (THOUSAND**RKAPPA)

      KM1    = K  - 1
      KP1    = K  + 1
C we want the ras adjustment time scale to be one hour (indep of dt)
      RASBLF = 1./3600.
C
       KPRV  = KM1
       KCR   = MIN(KM1,KRMAX)
       KFX   = KM1 - KCR
       NCMX  = KFX + NCRND
C
       IF (KFX .GT. 0) THEN
        DO 20 NC=1,KFX
         IC(NC) = K - NC
   20   CONTINUE
       ENDIF
C
      IF (NCRND .GT. 0) THEN
       DO 25 I=1,ncrnd
        IRND(I) = (RND(I)-0.0005)*(KCR-KRMIN+1)
        IRND(I) = IRND(I) + KRMIN
   25  CONTINUE
C
       DO 30 NC=1,NCRND
        IC(KFX+NC) = IRND(NC)
   30  CONTINUE
      ENDIF
C
      DO 100 NC=1,NCMX
C
       IF (NC .EQ. 1 ) THEN
        SETRAS = .TRUE.
       ELSE
        SETRAS = .FALSE.
       ENDIF
       IB = IC(NC)
C

       CALL CLOUDE(nn,LEN, LENC, K, IB, RASBLF, SETRAS
     *,           POI, QOI, PRS
     *,           PCU, CLN, CLNMAX, TCU, QCU, CLF
     *,           ALF, BET, GAM, PRH, PRJ, PRI, HOI, ETA
     *,           TX1, TX2, TX3, TX4, TX5, TX6, TX7, TX8, TX9
     *,           TXA, TXB, TXC, TXD, TXE
     *,           WFN, AKM
     *,           ITMP1,ITMP2,TMP1,TMP2,TMP3,dt
     *,           cp,grav,rkappa,alhl)
C
c Compute Total Cloud Mass Flux
c *****************************
         if((idet.ne.0).or.(imf.ne.0)) then
         do L=ib,k-1
         do i=1,lenc
         cmass(i,L) = wfn(i) * (eta(i,L+1)+one) * dt/grav
         enddo
         if(imf.ne.0)
     1 call pstbmp(cmass(1,L),qdiag(1,1,imf+L-1),lenc,imjmr,1,nn)
         enddo
         do i=1,lenc
         cmass(i,k) = wfn(i) * dt/grav
         enddo
         if(imf.ne.0)
     1 call pstbmp(cmass(1,k),qdiag(1,1,imf+k-1),lenc,imjmr,1,nn)
         if(idet.ne.0)
     1 call pstbmp(cmass(1,ib),qdiag(1,1,idet+ib-1),lenc,imjmr,1,nn)
         endif

       DO 50 L=IB,K
        DO 50 I=1,LENC
         POI(I,L) = POI(I,L) + TCU(I,L) * DT
         QOI(I,L) = QOI(I,L) + QCU(I,L) * DT
   50  CONTINUE

  100 CONTINUE
C
      RETURN
      END
      SUBROUTINE RNF(LEN, PL, RNO)
      PARAMETER (P5=500.0,  P8=800.0, PT8=0.8, PT2=0.2)
      PARAMETER (PFAC=PT2/(P8-P5))
      DIMENSION PL(LEN), RNO(LEN)
C
      DO 10 I=1,LEN
      RNO(I) = 1.0
   10 CONTINUE
C
      RETURN
      END
      SUBROUTINE CLDFAC(LEN, PL, CLF)
      PARAMETER (P4=400.0,ONE=1.,ZERO=0.,HALF=0.5,FOURTH=0.25)
      DIMENSION PL(LEN), CLF(LEN)
      DO I=1,LEN
      CLF(I) = FOURTH
      ENDDO
      DO I=1,LEN
      IF (PL(I) .GT. P4 ) CLF(I) = ZERO
      ENDDO
      RETURN
      END
      SUBROUTINE ACRIT(LEN, PL, ACR)
C
      PARAMETER (ACTP=2.0,   FACM=1.0)
      DIMENSION PL(LEN), ACR(LEN), IWK(LEN)
      DIMENSION PH(15), A(15), AD(15), AU(15), AC(15)
      LOGICAL FIRST
      DATA    FIRST/.TRUE./
C
      DATA PH/150.0, 200.0, 250.0, 300.0, 350.0, 400.0, 450.0, 500.0
     *,       550.0, 600.0, 650.0, 700.0, 750.0, 800.0, 850.0/
C
      DATA A/ 1.6851, 1.1686, 0.7663, 0.5255, 0.4100, 0.3677
     *,       0.3151, 0.2216, 0.1521, 0.1082, 0.0750, 0.0664
     *,       0.0553, 0.0445, 0.0633/
C
      DATA AD/15*0.0/, AC/15*0.0/, ACTOP/0.0/
C
      IF (FIRST) THEN
         FIRST   = .FALSE.
         ACTOP   = ACTP*FACM
         DO 5 L=1,15
         A(L) = A(L)*FACM
    5    CONTINUE
         DO 10 L=2,15
         TEM   = PH(L) - PH(L-1)
         AU(L) = A(L-1) / TEM
         AD(L) = A(L)   / TEM
         AC(L) = PH(L)*AU(L) - PH(L-1)*AD(L)
         AD(L) = AD(L) - AU(L)
   10    CONTINUE
      ENDIF
C
      DO 15 I=1,LEN
      IWK(I) = PL(I) * 0.02 - 0.999999999
      IF (IWK(I) .GT. 1) THEN
         IF (IWK(I) .LE. 15) THEN
            ACR(I) = AC(IWK(I)) + PL(I) * AD(IWK(I))
         ELSE
            ACR(I) = A(15)
         ENDIF
      ELSE
         ACR(I) = ACTOP
      ENDIF
   15 CONTINUE
C
      RETURN
      END
      SUBROUTINE CLOUDE(nn,LEN, LENC, K, IC, RASBLF, SETRAS
     *,                 POI, SHI, PRS
     *,                 PCU, CLN, CLNMAX, TCU, QCU, CLF
     *,                 ALF, BET, GAM, PRH, PRJ, PRI, HOI, ETA
     *,                 TX1, TX2, TX3, TX4, TX5, TX6, TX7, TX8, TX9
     *,                 TXA, TXB, TXC, TXD, TXE
     *,                 WFN, AKM
     *,                 ITMP1,ITMP2,TMP1,TMP2,TMP3,dt
     *,                 cp,grav,rkap,alhl)
C
C
C*********************************************************************
C*********************** EXPORT  MODEL *******************************
C********************* SUBROUTINE CLOUD  *****************************
C********************** 28 AUGUST  1990 ******************************
C*********************************************************************
C
#include <vrslv.com>
#include <stripmine.com>
      PARAMETER ( IMJMR   = IMR*(JMR+1) )
      PARAMETER ( lnz     =  (IMJMR-1) / NPCS + 1  )
      PARAMETER ( lnzc    =  lnz )
      PARAMETER ( kz      =  nlayr )

      PARAMETER (P00=1000.0)
      PARAMETER (DAYLEN=86400.0,  HALF=0.5,  ONE=1.0)
      PARAMETER (ZERO=0.0,     CRTP=400.0)
      PARAMETER (CMB2PA=100.0)
C
      PARAMETER (CRTLM1=0.0E-7, CRTLM2=0.5E-6, CRTMSF=0.0, DELH=1.0E3)
      PARAMETER (CRTFAC=ONE/(CRTLM2-CRTLM1))
      PARAMETER (rknob = 10.)
C
      DIMENSION POI(lnz,Kz),  SHI(lnz,Kz),  PRS(lnz,Kz+1), CLF(lnz,Kz)
     *,         PCU(lnz,Kz),  TCU(lnz,Kz),  QCU(lnz,Kz), CLN(lnz,Kz)
     *,         CLNMAX(lnz,Kz)
      DIMENSION ALF(lnzc,Kz), BET(lnzc,Kz),  GAM(lnzc,Kz), QOI(lnzc,Kz)
C     DIMENSION ALF(lnz ,Kz), BET(lnz ,Kz),  GAM(lnz ,Kz)
     *,         PRH(lnzc,Kz), PRJ(lnzc,Kz),  PRI(lnzc,Kz)
      DIMENSION AKM(lnzc),   WFN(lnzc)
      DIMENSION HOI(lnzc,Kz), ETA(lnzc,Kz)
     *,         TX1(lnzc),   TX2(lnzc), TX3(lnzc),   TX4(lnzc)
     *,         TX5(lnzc),   TX6(lnzc), TX7(lnzc),   TX8(lnzc)
     *,         TX9(lnzc),   TXA(lnzc), TXB(lnzc),   TXC(lnzc)
     *,         TXD(lnzc),   TXE(lnzc), TX10(lnzc)
     *,         ITMP1(lnzc), ITMP2(lnzc)
     *,         TMP1(lnzc), TMP2(lnzc), TMP3(lnzc)

      dimension gammah(lnzc,kz)
      LOGICAL SETRAS
C
      DATA IP1/220/, NSTX/1000006950/, NST/00/
CCC   DATA IP1/220/, NSTX/0/, NST/00/
C     DATA IP1/1607/, NSTX/0/, NST/00/
C
      CPBG=CP/GRAV
      ONBRKP=ONE/RKAP
      RKAPP1=1.0+RKAP
      ALBCP=ALHL/CP
C
      NST = NST + 1
      KM1 = K  - 1
      IC1 = IC + 1
C
C INITIALIZE LOCAL COPY OF SPECIFIC HUMIDITY

      do L=1,k
      do i=1,lenc
      qoi(i,L) = shi(i,L)
      enddo
      enddo

C      SETTIING ALF, BET, PRH, PRJ, AND PRI : DONE ONLY WHEN SETRAS=.T.
C
      IF (SETRAS) THEN
         DO 2010 I=1,LENC
         TX1(I) = PRS(I,1)
         TX2(I) = (TX1(I)*0.001) ** RKAP
 2010    CONTINUE
         DO 2050 L=1,K
         DO 2030 I=1,LENC
         PRJ(I,L) = (PRS(I,L+1)*0.001)  ** RKAP
         PRH(I,L) = (PRJ(I,L)*PRS(I,L+1) - TX2(I)*PRS(I,L))
     *                           / ((PRS(I,L+1)-PRS(I,L)) * RKAPP1)
         TX2(I) = PRJ(I,L)
 2030    CONTINUE
 2050    CONTINUE
         DO 2070 L=1,K
         DO 2060 I=1,LENC
         TX5(I) = POI(I,L) * PRH(I,L)
         TX1(I) = (PRS(I,L) + PRS(I,L+1)) * 0.5
         TX3(I) = TX5(I)
 2060    CONTINUE
C
c safety check on moisture - use minimum of q and saturation q
         CALL VQSAT(TX3, TX1, TX2, TX4, LENC, .TRUE.)
         do i = 1,lenc
          qoi(i,l) = amin1( shi(i,l),tx2(i) )
         enddo
C
         DO 2065 I=1,LENC
         ALF(I,L) = TX2(I) - TX4(I) * TX5(I)
         BET(I,L) = TX4(I) * PRH(I,L)
         GAM(I,L) = 1.0 / ((1.0 + TX4(I)*ALBCP) * PRH(I,L))
CARIES   GAM(I,L) = GAM(I,L) / PRH(I,L)
         PRI(I,L) = (CP/CMB2PA) / (PRS(I,L+1) - PRS(I,L))
 2065    CONTINUE
 2070    CONTINUE
      ENDIF
C
      DO 20 I=1,LENC
      TX2(I) = ZERO
   20 CONTINUE
C
      DO 30 I=1,LENC
      TX1(I)   = PRJ(I,K) * POI(I,K)
      HOI(I,K) = TX2(I) + TX1(I)*CP + QOI(I,K)*ALHL
      ETA(I,K) = ZERO
      TX2(I)   = TX2(I) + (PRJ(I,K) - PRJ(I,KM1)) * POI(I,K) * CP
   30 CONTINUE

      IF (IC .LT. KM1) THEN
         DO 3703 L=KM1,IC1,-1
         DO 50 I=1,LENC
         TX4(I) = ALF(I,L) + BET(I,L)*POI(I,L)
         IF (TX4(I) .GT. QOI(I,L)) THEN
            TX4(I) = QOI(I,L)
         ENDIF
   50    CONTINUE
C
         DO 60 I=1,LENC
         HOI(I,L) = TX2(I) + PRJ(I,L) * POI(I,L) * CP + TX4(I)*ALHL
         TX1(I)   = (PRJ(I,L) - PRJ(I,L-1)) * POI(I,L)
         ETA(I,L) = ETA(I,L+1) + TX1(I)*CPBG
         TX2(I)   = TX2(I)     + TX1(I)*CP
   60    CONTINUE
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' HOI=',HOI(IP1,L),' PRJ=',PRJ(IP1,L),PRJ(IP1,L-1)
     *,       ' TX2=', TX2(IP1), ' TX1=',TX1(IP1),' POI=',POI(IP1,L)
      ENDIF
 3703    CONTINUE
      ENDIF


      DO 70 I=1,LENC
      HOI(I,IC) = TX2(I)
      TX4(I)    = ALF(I,IC) + BET(I,IC)*POI(I,IC)
      TX2(I)    = TX4(I)
      IF (TX4(I) .GT. QOI(I,IC)) THEN
         TX4(I) = QOI(I,IC)
      ENDIF
   70 CONTINUE
C
      DO 80 I=1,LENC
      HOI(I,IC) = HOI(I,IC) + PRJ(I,IC) * POI(I,IC) * CP + TX4(I)*ALHL
      TX3(I   ) = (PRJ(I,IC) - PRH(I,IC)) * POI(I,IC)
      ETA(I,IC) = ETA(I,IC1) + CPBG * TX3(I)
      TX8(I)    = TX2(I)    - QOI(I,IC)
   80 CONTINUE
C
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' TX8=',TX8(IP1),' TX2=',TX2(IP1),' QOI=',QOI(IP1,IC)
      PRINT *,' ALF=',ALF(IP1,IC),' BET=',BET(IP1,IC),'POI=',POI(IP1,IC)
      endif
      DO 120 I=1,LENC
      IF (TX8(I) .LT. ZERO) THEN
         TX8(I) = ZERO
      ENDIF
  120 CONTINUE
      DO 130 I=1,LENC
      TX8(I) = HOI(I,IC) + TX8(I)*ALHL
      TX9(I) = HOI(I,K)  - TX8(I)
      TXC(I) = TX8(I)
      TX4(I) = HOI(I,K)
  130 CONTINUE
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' TX8=',TX8(IP1),' HOI=',HOI(IP1,IC)
      endif
C
C     ENTRAINMENT PARAMETER
C
      DO 140 I=1,LENC
      TX1(I) = (TX8(I) - HOI(I,IC)) * TX3(I)
C     ADDED ON 06/23/89
      TX3(I) = HOI(I,IC) * TX3(I)
  140 CONTINUE
      IF (IC .LT. KM1) THEN
         DO 3707 L=IC1,KM1
         DO 160 I=1,LENC
         TX5(I) = POI(I,L) * (PRJ(I,L) - PRJ(I,L-1))
         TX1(I) = TX1(I) + (TX8(I) - HOI(I,L)) * TX5(I)
C     ADDED ON 06/23/89
         TX3(I) = TX3(I) + HOI(I,L) * TX5(I)
  160    CONTINUE
 3707    CONTINUE
      ENDIF
C
      DO 195 I=1,LENC
      TX1(I) = TX1(I) * CPBG
      TX3(I) = TX3(I) * CPBG
      IF (TX1(I) .NE. ZERO .AND. TX9(I) .GT. ZERO) THEN
         TX9(I) = TX9(I) / TX1(I)
      ELSE
         TX9(I) = - ONE
C     TXA(1;LENC) =  ZERO
      ENDIF
  195 CONTINUE
      DO 210 I=1,LENC
      IF ((TX9(I) .LT. CRTLM1) .AND. (HOI(I,K) .LT. TX8(I))) THEN
         TX9(I) = CRTLM1
         TXC(I) = (HOI(I,K) + TX3(I)*CRTLM1) / (ONE + ETA(I,IC)*CRTLM1)
      ENDIF
  210 CONTINUE
C
C     UNCOMMENT THE LINE BELOW TO DEACTIVATE DLAMDA/DT TERM
C
      DO 230 I=1,LENC
      TXA(I) = ZERO
      TXB(I) = ZERO
  230 CONTINUE
C
      if(nn.eq.5)then
      IF (NST .GE. NSTX) WRITE(6,7001) TX9(IP1), TX8(IP1)
      endif
 7001 FORMAT(' TX9=',E15.7,' TX8=',E15.8)

C
C     NORMALIZED MASSFLUX
C
      DO 250 I=1,LENC
      ETA(I,K) = ZERO
  250 CONTINUE
      DO 3709 L=KM1,IC,-1
      DO 260 I=1,LENC
      ETA(I,L) = TX9(I) * ETA(I,L)
  260 CONTINUE
 3709 CONTINUE
C
      if(nn.eq.5)then
      IF (NST .GE. NSTX) THEN
         DO 7030 L=IC,K
         WRITE (6,7031) POI(IP1,L), QOI(IP1,L), HOI(IP1,L), ETA(IP1,L)
 7031    FORMAT(' POI=',E14.6,' QOI=',E14.6,' HOI=',E15.7,' ETA=',E12.4)
 7030    CONTINUE
      ENDIF
      ENDIF

C
C     CLOUD WORKFUNCTION
C
      DO 270 I=1,LENC
      TX1(I) = GAM(I,IC) * (PRJ(I,IC) - PRH(I,IC))
  270 CONTINUE
C**
      DO 280 I=1,LENC
      IF (TX9(I) .EQ. CRTLM1) TX1(I) = ZERO
  280 CONTINUE
C
      DO 290 I=1,LENC
      WFN(I) = -TX1(I) * TX8(I) * (ETA(I,IC1)+ONE)
C
      TX1(I) = HOI(I,K)
C
      TXD(I) = ONE
  290 CONTINUE
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' AFTER 290:, WFN=',WFN(IP1)
      endif
      IF (IC1 .LE. KM1) THEN
      DO 3710 L=KM1,IC1,-1
C
      DO 330 I=1,LENC
      TX4(I) = ALF(I,L) + BET(I,L)*POI(I,L) - QOI(I,L)
  330 CONTINUE
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' AFT 330 TX4=',TX4(IP1),' QOI=',QOI(I,L)
      endif
      DO 360 I=1,LENC
      IF (TX4(I) .LT. ZERO) TX4(I) = ZERO
  360 CONTINUE
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' AFT 360 TX4=',TX4(IP1)
      endif
CC*
      DO 370 I=1,LENC
      TX4(I) = HOI(I,L) + TX4(I)*ALHL
  370 CONTINUE
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' AFT 370 TX4=',TX4(IP1),' HOI=',HOI(IP1,L)
      endif
C
      DO 380 I=1,LENC
      TX5(I) = (ETA(I,L) - ETA(I,L+1)) * HOI(I,L)
      TX5(I) = TX1(I) + TX5(I)
C
      TX10(I) = PRJ(I,L) - PRH(I,L)
      TX3(I) = (ETA(I,L+1) + 1.0) * TX10(I)
      TX1(I) = TX1(I)*TX10(I)
C
      TX10(I) = PRH(I,L) - PRJ(I,L-1)
      TX3(I) = (TX3(I) + (ETA(I,L) + 1.0) * TX10(I)) * TX4(I)
      TX1(I) = TX1(I) + TX5(I)*TX10(I)
C
      WFN(I) = WFN(I) + (TX1(I) - TX3(I)) * GAM(I,L)
      TX1(I) = TX5(I)
  380 CONTINUE
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' AFTER 380:, WFN=',WFN(IP1),' GAM=',GAM(IP1,L)
     *,       ' TX1=',TX1(IP1),' TX3=',TX3(IP1), ' TX4=',TX4(IP1)
       endif
 3710 CONTINUE
      ENDIF
      if(nn.eq.5)then
      IF (NST .GE. NSTX) THEN
         WRITE (6,4031) WFN(IP1), TX1(IP1), GAM(IP1,IC)
 4031    FORMAT(' AFT3710 WFN=',E14.6,' TX1=',E14.6,' GAM=',E14.6)
      ENDIF
      ENDIF
C
      DO 510 I=1,LENC
      TX1(I) = TX1(I) * GAM(I,IC)
  510 CONTINUE
C**
      DO 520 I=1,LENC
      IF (TX9(I) .EQ. CRTLM1) TX1(I) = ZERO
  520 CONTINUE
      DO 530 I=1,LENC
      WFN(I) = WFN(I) + TX1(I) * (PRJ(I,IC) - PRH(I,IC))
C
      TXC(I) = TX4(I) - TXC(I)
      TX4(I) = TX4(I) - TX8(I)
  530 CONTINUE
      if(nn.eq.5)then
      IF (NST .GE. NSTX) THEN
         WRITE (6,4032) WFN(IP1), TX1(IP1), PRJ(IP1,IC), PRH(IP1,IC)
 4032    FORMAT(' AFT530 WFN=',E14.6,' TX1=',3E14.6)
      ENDIF
      ENDIF
      DO 560 I=1,LENC
      IF (TX4(I) .NE. ZERO) THEN
         TXC(I) = TXC(I) / TX4(I)
      ELSE
         TXC(I) = ZERO
      ENDIF
  560 CONTINUE
      if(nn.eq.5)then
      IF (NST .GE. NSTX) THEN
         WRITE (6,4033) TXC(IP1)
 4033    FORMAT(' AFT560 TXC=',E14.6)
      ENDIF
      ENDIF
      DO 580 I=1,LENC
      IF (TX9(I) .GT. CRTLM1) TXC(I) = ONE
  580 CONTINUE
      DO 590 I=1,LENC
      IF (TXC(I) .GT. ONE .OR. TXC(I) .LT. ZERO) THEN
         TX9(I) = - ONE
         TXC(I) = ZERO
      ENDIF
  590 CONTINUE
      DO 600 I=1,LENC
      TX8(I) = TX8(I) - HOI(I,IC)
  600 CONTINUE
C*
      DO 610 I=1,LENC
      IF (TX9(I) .EQ. CRTLM1) THEN
         TX1(I) = TXC(I) * (HOI(I,IC)-HOI(I,IC1)) + HOI(I,IC1)
         TX8(I) = HOI(I,K) - TX1(I)
C
         TXE(I) = TXC(I) * (QOI(I,IC)-QOI(I,IC1)) + QOI(I,IC1)
         TX5(I) = TX1(I) - TXE(I)*ALHL
         TX2(I) = TXE(I) + TX8(I)*(ONE/ALHL)
      ENDIF
  610 CONTINUE
      DO 620 I=1,LENC
      TX4(I) = QOI(I,K) - TX2(I) * (ETA(I,IC) + ONE)
  620 CONTINUE
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' TX1=',TX1(IP1),' TX2=',TX2(IP1), ' TX4=', TX4(IP1)
     *,       ' TX8=',TX8(IP1),' TXE=',TXE(IP1)
      endif
      DO 3711 L=KM1,IC,-1
      DO 630 I=1,LENC
      TX4(I) = TX4(I) + (ETA(I,L)-ETA(I,L+1)) * QOI(I,L)
  630 CONTINUE
 3711 CONTINUE
C
C     CALCULATE GS AND PART OF AKM (THAT REQUIRES ETA)
C
      DO 690 I=1,LENC
      TXE(I)   = HOI(I,K)
      TX7(I)   = (POI(I,KM1) - POI(I,K)) / (PRH(I,K) - PRH(I,KM1))
      HOI(I,K) = TX7(I) * (PRJ(I,KM1)-PRH(I,KM1)) * PRH(I,K) * PRI(I,K)
      AKM(I)   = ZERO
      TX2(I)   = 0.5 * (PRS(I,IC) + PRS(I,IC1))
  690 CONTINUE
cccc  PRINT *,' AFTER 690 CONTINUE'

      IF (IC1 .LE. KM1) THEN
      DO 3715 L=KM1,IC1,-1
      DO 750 I=1,LENC
      TX1(I)   = TX7(I) * PRH(I,L) * (PRH(I,L+1)-PRJ(I,L))
      TX7(I)   = (POI(I,L-1) - POI(I,L)) * (ONE + ETA(I,L))
     *         / (PRH(I,L) - PRH(I,L-1))
C
      HOI(I,L) = (TX1(I) +
     *           TX7(I) * (PRJ(I,L-1)-PRH(I,L-1)) * PRH(I,L)) * PRI(I,L)
C
      AKM(I)   = AKM(I) - HOI(I,L)
     *         * ((ETA(I,L)   + 1.0) * (PRH(I,L)-PRJ(I,L-1)) +
     *            (ETA(I,L+1) + 1.0) * (PRJ(I,L)-PRH(I,L))) / PRH(I,L)
  750 CONTINUE
C
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' IN 3715 LOOP L=',L,' AKM=',AKM(IP1)
      endif
 3715 CONTINUE
C
      ENDIF
cccc  PRINT *,' AFTER 3715 CONTINUE'
C
      CALL RNF(LENC, TX2, TX1)
      DO 770 I=1,LENC
      TX2(I) = (ONE - TX1(I)) * TX4(I)
      TX4(I) = TX1(I) * TX4(I)
C
      TX7(I) = TX7(I) * PRH(I,IC) * (PRH(I,IC1)-PRJ(I,IC))
  770 CONTINUE
C
cccc  PRINT *,' AFTER 770 CONTINUE'
      DO 790 I=1,LENC
      IF (TX9(I) .EQ. CRTLM1) THEN
         TX7(I) = TX7(I) + (TX5(I)-HOI(I,IC)+QOI(I,IC)*ALHL) * (1.0/CP)
      ENDIF
  790 CONTINUE

      DO 800 I=1,LENC
      HOI(I,IC) = (TX7(I) - TX2(I) * ALBCP) * PRI(I,IC)
      TX3(I)    = ETA(I,IC) * TXB(I)
  800 CONTINUE
C
      DO 810 I=1,LENC
      IF (TX9(I) .NE. CRTLM1) THEN
         TX3(I) = TX3(I) + TXB(I) + (ETA(I,IC1) + 1.0)
     *                            * (PRJ(I,IC) - PRH(I,IC)) / PRH(I,IC)
      ENDIF
  810 CONTINUE
C
      DO 820 I=1,LENC
      AKM(I) = AKM(I) - TX3(I) * TX7(I) * PRI(I,IC)
  820 CONTINUE
cccc  PRINT *,' AFTER 820 CONTINUE, AKM=',AKM(IP1)
C
C     CALCULATE GH
C
      DO 830 I=1,LENC
      TX3(I) = ETA(I,K)
C     ETA(1,K ;LENC) = HOI(1,K  ;LENC) + TX6(1;LENC)
C    *                                * (ALHL*HALF*ONBDSG(K))
CCCC  ETA(I,K) = HOI(I,K) + ALBCP * PRI(I,K) * (QOI(I,KM1) - QOI(I,K))
      gammah(I,K) = HOI(I,K) + ALBCP*PRI(I,K) * (QOI(I,KM1)-QOI(I,K))
      TX6(I)   = ZERO
  830 CONTINUE

      IF (IC1 .LE. KM1) THEN
      DO 3720 L=KM1,IC1,-1
      DO 840 I=1,LENC
      TX2(I) = TX6(I)
      TX6(I) = (QOI(I,L-1) - QOI(I,L)) * (ETA(I,L) + ONE)
      TX2(I) = TX2(I) + TX6(I)
      TX1(I) = ETA(I,L) - TX3(I)
      TX3(I) = ETA(I,L)
C
CCCC  ETA(I,L) = HOI(I,L) + TX2(I)   * PRI(I,L) * (ALBCP*HALF)
      gammah(I,L) = HOI(I,L) + TX2(I)   * PRI(I,L) * (ALBCP*HALF)
      AKM(I)   = AKM(I)   + gammah(I,L) * TX1(I) * TXA(I)
  840 CONTINUE
      if((nn.eq.5).and.(nst.ge.nstx))then
      PRINT *,' IN 3720 LOOP L=',L,' AKM=',AKM(IP1)
      endif
C
 3720 CONTINUE
      ENDIF
C
cccc  PRINT *,' BEFOR 850 CONTINUE'
      DO 850 I=1,LENC
      IF (TX9(I) .EQ. CRTLM1) THEN
         TX6(I) = TX6(I) + (2.0/ALHL)
     *          * (TXE(I) - TX8(I) - TX5(I) - QOI(I,IC)*ALHL)
      ENDIF
  850 CONTINUE
      DO 860 I=1,LENC
      TX1(I) = ETA(I,IC) - TX3(I)
      gammah(I,IC) = PRI(I,IC) * (TX7(I) + (1.0/CP)
     *          * (TX6(I)*(ALHL*HALF) + (ETA(I,IC)+ONE) * TX8(I)))
      AKM(I) = AKM(I) + gammah(I,IC) * TXA(I) * TX1(I)
  860 CONTINUE
      if(nn.eq.5)then
      IF (NST .GE. NSTX) THEN
         WRITE (6,4034) AKM(IP1)
 4034    FORMAT(' AFT860 AKM=',E14.6)
      ENDIF
      ENDIF
C
C     CALCULATE HC PART OF AKM
C
cccc  PRINT *,' BEFORE 870 CONTINUE'
      IF (IC1 .LE. KM1) THEN
         DO 870 I=1,LENC
         TX5(I) = ZERO
  870    CONTINUE
         DO 3725 L=KM1,IC1,-1
         DO 880 I=1,LENC
         TX5(I) = TX5(I) + CPBG * gammah(I,L) * POI(I,L)
     *                                     * (PRJ(I,L) - PRJ(I,L-1))
         TX1(I) = TX9(I)*TX5(I) + gammah(I,K)
         TX2(I) = GAM(I,L-1) * (PRJ(I,L-1) - PRH(I,L-1))
  880    CONTINUE
C
         IF (L .EQ. IC1) THEN
             DO 890 I=1,LENC
             IF (TX9(I) .EQ. CRTLM1) TX2(I) = ZERO
  890       CONTINUE
         ENDIF
         DO 900 I=1,LENC
         AKM(I) = AKM(I) + (TX2(I) + GAM(I,L) * (PRH(I,L)-PRJ(I,L-1)))
     *                                        * TX1(I)
  900    CONTINUE
 3725    CONTINUE
      ENDIF
      if(nn.eq.5)then
      IF (NST .GE. NSTX) THEN
         WRITE (6,4035) AKM(IP1)
 4035    FORMAT(' AFT3725 AKM=',E14.6)
      ENDIF
      ENDIF
C
      DO 910 I=1,LENC
      AKM(I) = AKM(I) + (TXA(I) + GAM(I,KM1) * (PRJ(I,KM1)-PRH(I,KM1)))
     *                                       * gammah(I,K)
C
      TX2(I) = (PRS(I,IC) + PRS(I,IC1)) * 0.5
      TX3(I) = TX2(I)
  910 CONTINUE
C
      DO 920 I=1,LENC
      IF (TX9(I) .EQ. CRTLM1) THEN
         TX3(I) = TX2(I) + 0.5*(PRS(I,IC+2) - PRS(I,IC)) * (ONE-TXC(I))
         TX2(I) = TX3(I)
      ENDIF
  920 CONTINUE

      DO 930 I=1,LENC
      TXC(I) = ZERO
      TX1(I) = PRS(I,IC1)
      TX5(I) = 0.5 * (PRS(I,IC1) + PRS(I,IC+2))
  930 CONTINUE

      DO 935 I=1,LENC
      IF (TX2(I) .LT. TX1(I))  TXC(I) = ONE
  935 CONTINUE
      if(nn.eq.5)then
      IF (NST .GE. NSTX) THEN
         WRITE (6,4036) AKM(IP1), TX5(IP1), TX1(IP1), TXC(IP1)
 4036    FORMAT(' AFT935 AKM=',E14.6,' TX5=',E14.6,' TX1=',E14.6
     *,                  ' TXC=',E14.6)
      ENDIF
      ENDIF
C
      DO 940 I=1,LENC
      IF ((TX2(I) .GE. TX1(I)) .AND. (TX2(I) .LT. TX5(I))) THEN
         TXC(I)     = ONE - (TX2(I) - TX1(I)) / (TX5(I) - TX1(I))
C
         TX6(I)     = PRI(I,IC1) / PRI(I,IC)
         HOI(I,IC1) = HOI(I,IC1) + HOI(I,IC) * TX6(I)
         HOI(I,IC)  = ZERO
C
         gammah(I,IC1) = gammah(I,IC1) + gammah(I,IC) * TX6(I)
         gammah(I,IC)  = ZERO
      ENDIF
  940 CONTINUE
C
      CALL ACRIT(LENC, TX3, TX1)
      DO 950 I=1,LENC
      TX3(I) = TX1(I) * (PRS(I,K) - TX2(I))
      TXC(I) = TXC(I)*TXD(I)
  950 CONTINUE
CC
      if(nn.eq.5)then
      IF (NST .GE. NSTX)
     *WRITE(6,7002) WFN(IP1),  TX3(IP1), AKM(IP1), TXC(IP1)
 7002 FORMAT(' WFN=',E15.5,' ACRIT=',E12.4,' AKM=',E15.5,' TXC=',E15.6)
      endif
C
      DO 970 I=1,LENC
      IF (TX9(I) .GE. CRTLM1 .AND. AKM(I) .LT. ZERO
     *   .and.tx4(i).gt.zero .AND. WFN(I).GT.TX3(I)) THEN
         WFN(I) = TXC(I) * (TX3(I) - WFN(I)) * RASBLF / AKM(I)
      ELSE
         WFN(I)=ZERO
      ENDIF
CCCCCCIF (TX9(I) .LE. CRTLM1 ) WFN(I) = ZERO
  970 CONTINUE
      if(nn.eq.5)then
      IF (NST .GE. NSTX) WRITE(6,7005) WFN(IP1)
      endif
 7005 FORMAT(' MB=',2E15.5)
C
      CALL CLDFAC(LENC, TX2, TX1)
C
      do i = 1,lenc
       tx1(i) = cmb2pa *  (prs(i,k+1) - prs(i,k))*(1./dt)
       if(wfn(i).gt.tx1(i))wfn(i) = tx1(i)
      enddo
C
      do i = 1,lenc
c mass in detrainment layer
       tx1(i) = cmb2pa *  (prs(i,ic+1) - prs(i,ic))*(1./dt)
c ratio of detraining cloud mass to mass in detrainment layer
       tx1(i) = rknob * wfn(i) * (eta(i,ic)+ONE) / tx1(i)
        if(wfn(i).gt.crtmsf) clf(i,ic) = clf(i,ic) + tx1(i)
        if(clf(i,ic).gt.one) clf(i,ic)  = one
        if((prs(i,ic+1).le.400.).and.(wfn(i).gt.0.))clnmax(i,ic) = one
        if((ic.ge.k-2).and.(wfn(i).gt.0.))cln(i,ic) = 0.5
      enddo
C
      DO 980 I=1,LENC
      TX1(I) = WFN(I) * (1.0/CP)
  980 CONTINUE
C
C     THETA AND Q CHANGE DUE TO CLOUD TYPE IC
C

      TEM1 = 0.0
      TEM2 = 0.0
      DO 3730 L=IC,K
      DO 985 I=1,LENC
      TX2(I)   = TX1(I)   * HOI(I,L)
      TX3(I)   = (gammah(I,L) - HOI(I,L)) * WFN(I)
C
      TCU(I,L) = TX2(I) / PRH(I,L)
      QCU(I,L) = TX3(I)*(1.0/ALHL)
  985 CONTINUE
C
 3730 CONTINUE
C
      DO 990 I=1,LENC
COMMM PCU(I) = (DAYLEN/GRAV) * TX4(I) * WFN(I)
      PCU(I,IC) = PCU(I,IC) + (DT/GRAV) * TX4(I) * WFN(I)
  990 CONTINUE
C
ccc   IF (NST .GE. NSTX) THEN
      PMX = 0.0
      DO 995 I=1,LENC
      PMX = AMAX1(PMX,PCU(I,IC))
  995 CONTINUE

      if(nn.eq.5)then
      DO 996 I=1,LENC
      IF (PCU(I,IC) .EQ. PMX .AND. PMX .NE. 0.0) THEN
cccc     WRITE(6,997) PMX, I
 997     FORMAT(' MAXIMUM PRECIP PCU = ',E14.6,' AT I=',I5)
      ENDIF
  996 CONTINUE
      endif
ccc   ENDIF
C
C  PENETRATIVE CALCULATION OVER
ccccc IF (NST .GE. NSTX) WRITE(6,8003) TEM1, TEM2, TEM3
 8003 FORMAT(' PCUQ1=', E13.6,'PCUQ2=',E13.6,' PCU=',E13.6)
      IF ((nn.eq.5).and.(NST .GE. NSTX)) WRITE(6,7004) PCU(IP1,IC)
 7004 FORMAT(' PCU=',2E15.5)
      RETURN
      END
      SUBROUTINE VQSAT(T,P,Q,DQDT,LEN,LDQDT)
C
      LOGICAL LDQDT
C
      PARAMETER (ESFAC=0.622)
      PARAMETER (B6= 6.136820929E-11*ESFAC)
      PARAMETER (B5= 2.034080948E-8 *ESFAC)
      PARAMETER (B4= 3.031240396E-6 *ESFAC)
      PARAMETER (B3= 2.650648471E-4 *ESFAC)
      PARAMETER (B2= 1.428945805E-2 *ESFAC)
      PARAMETER (B1= 4.436518521E-1 *ESFAC)
      PARAMETER (B0= 6.107799961E+0 *ESFAC)
      PARAMETER (C1= B1   )
      PARAMETER (C2= B2*2.)
      PARAMETER (C3= B3*3.)
      PARAMETER (C4= B4*4.)
      PARAMETER (C5= B5*5.)
      PARAMETER (C6= B6*6.)
      PARAMETER (TMIN=223.15,TMAX=323.15, TICE=273.16, ONE=1.0)
C
      DIMENSION T(LEN), P(LEN), Q(LEN), DQDT(LEN),TT(LEN),PP(LEN)
      DIMENSION B(7), C(6)
      DATA B/B0,B1,B2,B3,B4,B5,B6/
      DATA C/   C1,C2,C3,C4,C5,C6/
C
      DO 10 II = 1,LEN
      TT(II)  = T(II)
      IF (TT(II) .LT. TMIN) TT(II) = TMIN
      IF (TT(II) .GT. TMAX) TT(II) = TMAX
      PP(II) = ONE / P(II)
      TT(II) = TT(II) - TICE
  10  CONTINUE

C
      CALL POLY(Q,TT,B,7,LEN)
      DO 11 II = 1,LEN
      Q(II) = Q(II) * PP(II)
  11  CONTINUE
C
      IF (LDQDT)  THEN
         CALL POLY(DQDT,TT,C,6,LEN)
      DO 12 II = 1,LEN
         DQDT(II) = DQDT(II) * PP(II)
  12  CONTINUE
      ENDIF
C
      RETURN
      END
      SUBROUTINE POLY(P,X,B,N,LEN)
      DIMENSION P(LEN), X(LEN), B(N)
C
      DO 10 II = 1,LEN
      P(II)  = B(N)
  10  CONTINUE
      IF (N .GT. 1) THEN
         DO 100 L=N,2,-1
      DO 11 II = 1,LEN
         P(II)  = P(II) * X(II) + B(L-1)
  11  CONTINUE
100      CONTINUE
      ENDIF
C
      RETURN
      END
       SUBROUTINE RNEVP(NN,IRUN,NLAY,TL,QL,RAIN,PL,CLFRAC,SP,DSIG,PLKE,
     1   PLK,TH,PHIS,TEMP1,TEMP2,TEMP3,ITMP1,ITMP2,RCON,RLAR,QBUDGET,
     2   CLSMAX,CLSBTH,tmscl,tmfrc,cp,gravity,alhl,gamfac)
#include <vrslv.com>
#include <vbudget.com>
#include <stripmine.com>
C
      PARAMETER ( IMJMR   = IMR*(JMR+1) )
      PARAMETER ( JNP     = (JMR+1) )
      PARAMETER (ZM1P04 = -1.04E-4 )
      PARAMETER (ZERO = 0.)
      PARAMETER (TWO89= 2.89E-5)
      PARAMETER ( ZP44= 0.44)
      PARAMETER ( ZP01= 0.01)
      PARAMETER ( ZP1 = 0.1 )
      PARAMETER ( ZP001= 0.001)
      PARAMETER ( HALF= 0.5)
      PARAMETER ( ZP578 = 0.578 )
      PARAMETER ( ONE = 1.)
      PARAMETER ( THOUSAND = 1000.)
      PARAMETER ( Z3600 = 3600.)
C
       DIMENSION TL(IRUN,NLAY),QL(IRUN,NLAY),RAIN(IRUN,NLAY),
     $ PL(IRUN,NLAY),CLFRAC(IRUN,NLAY),SP(IRUN),TEMP1(IRUN,NLAY),
     $ TEMP2(IRUN,NLAY),PLKE(IRUN,NLAY),
     $ RCON(IRUN),RLAR(IRUN),DSIG(NLAY),PLK(IRUN,NLAY),TH(IRUN,NLAY),
     $ PHIS(IRUN),TEMP3(IRUN,NLAY),ITMP1(IRUN,NLAY),
     $ ITMP2(IRUN,NLAY),CLSMAX(IRUN,NLAY),CLSBTH(IRUN,NLAY)
       LOGICAL QBUDGET
C
       DIMENSION EVP9(ISTRIP,NLAYR)
C
       NLAYM1 = NLAY - 1
       IRNLAY = IRUN*NLAY
       IRNLM1 = IRUN*(NLAY-1)
C
       RPHF = Z3600/tmscl
C
       ELOCP   = alhl/cp
       CPOG    = cp/gravity

       DO 9000 I = 1,IRUN
       RCON(I) = ZERO
       RLAR(I) = ZERO
 9000  CONTINUE
       DO 10 I = 1,IRNLAY
         TEMP1(I,1) = ZERO
         TEMP2(I,1) = ZERO
         TEMP3(I,1) = ZERO
         ITMP1(I,1) = ZERO
         ITMP2(I,1) = ZERO
        CLSMAX(I,1) = ZERO
        CLSBTH(I,1) = ZERO
 10    CONTINUE
C
C RHO(ZERO) / RHO FOR TERMINAL VELOCITY APPROX.
      DO 4 L=1,NLAY
       DO 9008 I = 1,IRUN
        TEMP2(I,L) = PL(I,L)*ZP001
        TEMP2(I,L) = SQRT(TEMP2(I,L))
 9008  CONTINUE
4     CONTINUE
C INVERSE OF MASS IN EACH LAYER
      DO 5 L=1,NLAY
       DO 9010 I = 1,IRUN
      TEMP3(I,L) = SP(I) * DSIG(L)
      TEMP3(I,L) = GRAVITY*ZP01 / TEMP3(I,L)
 9010  CONTINUE
5     CONTINUE
C
       DO 9014 I = 1,IRNLAY
       EVP9(I,1)=ZERO
       TEMP1(I,1)=ZERO
 9014  CONTINUE
C
C      DO LOOP FOR MOISTURE EVAPORATION ABILITY AND CONVEC EVAPORATION.
C
       DO 100 L=2,NLAY
       DO 9016 I = 1,IRUN
        IF ( TEMP1(I,8) . LT . CLFRAC(I,L) )
     1                        TEMP1(I,8) = CLFRAC(I,L)
        TEMP1(I,3)=TL(I,L)
        TEMP1(I,4)=QL(I,L)
 9016  CONTINUE
C
C
       DO 50 N=1,2
       IF(N.EQ.1)RELAX=HALF
       IF(N.GT.1)RELAX=ONE
       CALL QSTARS (TEMP1(1,3),PL(1,L),TEMP1(1,15),TEMP1(1,16),
     1              TEMP1(1,17),ITMP2(1,1),ITMP2(1,2),IRUN,TEMP1(1,2))
       DO 9018 I = 1,IRUN
       TEMP1(I,5)=TEMP1(I,2)-TEMP1(I,4)
       TEMP1(I,6)=TEMP1(I,2)*GAMFAC/(TEMP1(I,3)
     1           *TEMP1(I,3))
       TEMP1(I,5)=TEMP1(I,5)/(ONE+TEMP1(I,6))
       TEMP1(I,4)=TEMP1(I,4)+TEMP1(I,5)*RELAX
       TEMP1(I,3)=TEMP1(I,3)-TEMP1(I,5)*ELOCP*RELAX
 9018  CONTINUE
  50   CONTINUE
C
       DO 9020 I = 1,IRUN
       EVP9(I,L)=(TEMP1(I,4)-QL(I,L))/TEMP3(I,L)
C
       RCON(I)=RCON(I)+RAIN(I,L)
 9020  CONTINUE
C
C**********************************************************************
C  FOR CONVECTIVE PRECIP, FIND THE "EVAPORATION EFFICIENCY" USING     *
C                   KESSLERS PARAMETERIZATION                         *
C**********************************************************************
C
       DO 20 I=1,IRUN
       IF ((RCON(I).GT.ZERO).AND.(EVP9(I,L).GT.ZERO)) THEN
       RNFRAC=TEMP1(I,8)
COMMM  RNFRAC=0.4
       IF(RNFRAC.GT.ONE)RNFRAC=ONE
       RINMM   =  RCON(I)
       RPOW    = (RINMM*RPHF*TEMP2(I,L))**ZP578
       EXPARG  = ZM1P04*tmscl*RPOW
       AREARAT = ONE-(EXP(EXPARG))
                               TEMP1(I,7) = EVP9(I,L)*AREARAT*RNFRAC
       IF(TEMP1(I,7).GE.RINMM) TEMP1(I,7) = RINMM
       RCON(I)=RCON(I)-TEMP1(I,7)
C
       EVP9(I,L)=EVP9(I,L)-TEMP1(I,7)
C
       QL(I,L) = QL(I,L)+TEMP1(I,7)*TEMP3(I,L)
       TL(I,L) = TL(I,L)-TEMP1(I,7)*TEMP3(I,L)*ELOCP
C
       ENDIF
  20   CONTINUE
C
C **********************************************************************
C ****   FILL CONVECTIVE BUDGET ARRAYS FOR DIAGNOSTIC OUTPUT        ****
C **********************************************************************
C
      IF(QBUDGET)THEN
        DO 17 I = 1,IRUN
         ZTEMP(I,1) = -TEMP1(I,7)*TEMP3(I,L)*SP(I)*ELOCP/PLK(I,L)
     .              *  tmfrc
  17    CONTINUE
        CALL BDGBMP(ZTEMP(1,1),ZCCONT(1,L),IRUN,IMR,JNP,1,NN)
        CALL BDGBMP(ZTEMP(1,1),ZCLOWT(1,L),IRUN,IMR,JNP,1,NN)
        DO 18 I = 1,IRUN
         ZTEMP(I,1) =  TEMP1(I,7)*TEMP3(I,L)*SP(I) * tmfrc
  18    CONTINUE
        CALL BDGBMP(ZTEMP(1,1),ZCCONQ(1,L),IRUN,IMR,JNP,1,NN)
        CALL BDGBMP(ZTEMP(1,1),ZCLOWQ(1,L),IRUN,IMR,JNP,1,NN)
      ENDIF
C **********************************************************************
C
  100  CONTINUE
c      DO 987 L = 1,nlay
c      DO 986 I = 1,IRUN
c       RIN = PCINP(I)
c       ROUT = RCON(I)
c       IF(RIN.NE.ZERO)POUT = ROUT /  RIN
c       IF(PCINP(I).NE.ZERO)PRINT *,' i ',i,' L ',L
c       IF(PCINP(I).NE.ZERO)PRINT *,' IN ',RIN,' OUT ',ROUT,' % ',POUT
c       IF(PCINP(I).NE.ZERO)PRINT *,' CLOUD ',(CLFRAC(I,LL),LL=1,NLAY)
c986   continue
c987   continue
       DO 200 L=2,NLAY
        DO 30 I = 1,IRUN
         IF (EVP9(I,L).LE.ZERO) THEN
          CLSBTH(I,L)=ONE
          RLAR(I) = RLAR(I)-EVP9(I,L)
          QL(I,L) = QL(I,L)+EVP9(I,L)*TEMP3(I,L)
          TL(I,L) = TL(I,L)-EVP9(I,L)*TEMP3(I,L)*ELOCP
         ENDIF
  30    CONTINUE
C **********************************************************************
C ****   FILL LARGE SCALE BUDGET ARRAYS FOR GENERATING LS RAIN      ****
C **********************************************************************
C
      IF(QBUDGET)THEN
        DO 14 I = 1,IRUN
         IF(EVP9(I,L).LE.ZERO)THEN
         ZTEMP(I,1) = -EVP9(I,L)*TEMP3(I,L)*SP(I)*ELOCP/PLK(I,L)
     .              *  tmfrc
         ELSE
         ZTEMP(I,1) = ZERO
         ENDIF
  14    CONTINUE
        CALL BDGBMP(ZTEMP(1,1),ZLCONT(1,L),IRUN,IMR,JNP,1,NN)
        DO 13 I = 1,IRUN
         IF(EVP9(I,L).LE.ZERO)THEN
         ZTEMP(I,1) =  EVP9(I,L)*TEMP3(I,L)*SP(I) * tmfrc
         ELSE
         ZTEMP(I,1) = ZERO
         ENDIF
  13    CONTINUE
        CALL BDGBMP(ZTEMP(1,1),ZLCONQ(1,L),IRUN,IMR,JNP,1,NN)
      ENDIF
C **********************************************************************
        DO 31 I=1,IRUN
         IF((RLAR(I).GT.ZERO).AND.(EVP9(I,L).GT.ZERO))THEN
          RPOW=(RLAR(I)*RPHF*TEMP2(I,L))**ZP578
          EXPARG  = ZM1P04*tmscl*RPOW
          AREARAT = ONE-(EXP(EXPARG))
                                    TEMP1(I,7) = EVP9(I,L)*AREARAT
          IF(TEMP1(I,7).GE.RLAR(I)) TEMP1(I,7) = RLAR(I)
          RLAR(I) = RLAR(I)-TEMP1(I,7)
          QL(I,L) = QL(I,L)+TEMP1(I,7)*TEMP3(I,L)
          TL(I,L) = TL(I,L)-TEMP1(I,7)*TEMP3(I,L)*ELOCP
         ENDIF
  31    CONTINUE
C
C **********************************************************************
C ****   FILL LARGE SCALE BUDGET ARRAYS FOR RE-EVAP OF LS RAIN      ****
C **********************************************************************
C
      IF(QBUDGET)THEN
        DO 15 I = 1,IRUN
         IF((RLAR(I).GT.ZERO).AND.(EVP9(I,L).GT.ZERO))THEN
         ZTEMP(I,1) = -TEMP1(I,7)*TEMP3(I,L)*SP(I)*ELOCP/PLK(I,L)
     .              *  tmfrc
         ELSE
         ZTEMP(I,1) = ZERO
         ENDIF
  15    CONTINUE
        CALL BDGBMP(ZTEMP(1,1),ZLCONT(1,L),IRUN,IMR,JNP,1,NN)
        DO 16 I = 1,IRUN
         IF((RLAR(I).GT.ZERO).AND.(EVP9(I,L).GT.ZERO))THEN
         ZTEMP(I,1) =  TEMP1(I,7)*TEMP3(I,L)*SP(I) * tmfrc
         ELSE
         ZTEMP(I,1) = ZERO
         ENDIF
  16    CONTINUE
        CALL BDGBMP(ZTEMP(1,1),ZLCONQ(1,L),IRUN,IMR,JNP,1,NN)
      ENDIF
C **********************************************************************
C
  200  CONTINUE
c      DO 32 I = 1,IRUN
c       RLAR(I) = RLAR(I) / THOUSAND
c32    CONTINUE
C
       RETURN
       END
      SUBROUTINE SWRIO (N,NYMD,NHMS,NDSWR,QBUDGET)
#include <vrslv.com>
#include <vmetric.com>
#include <vstate.com>
#include <vphyscom.com>
#include <vbudget.com>
#include <stripmine.com>
C
      PARAMETER ( IJALL   =  IMR * (JMR+1)  )
      PARAMETER ( ZERO    =    0.0 )
      PARAMETER ( ZP01    =    0.01)
      PARAMETER ( ONE     =    1.0 )
      PARAMETER ( AHALF   =    0.5 )
      PARAMETER ( TWOTHRD =   2./3.)
      PARAMETER ( TWO     =    2.0 )
      PARAMETER ( TEN     =   10.0 )
      PARAMETER ( ONEETY  =  180.0 )
C
      DIMENSION ALON(IMR), ALAT(JMR+1)
      LOGICAL QBUDGET
C
      COMMON /SPACE/ SPACE1(IMR,JMR+1,5*NLAYR+11)
      COMMON /SPACE/ WRK1  (IJALL,17     ,2)
      COMMON /SPACE/ WRK2  (IJALL,NLAYR+1,2)
C

      DIMENSION     PKZ(IMR, JMR+1, NLAYR)
      DIMENSION     PLZ(IMR*(JMR+1),NLAYR+1)
      DIMENSION      OZ(IMR*(JMR+1),NLAYR+1)
      DIMENSION    PLZE(IMR*(JMR+1),NLAYR+2)
      DIMENSION  CLRSKY(IMR,JMR+1,NLAYR)
      DIMENSION  ALLATS(IMR,JMR+1)
      DIMENSION    XLAT(IJALL)
      DIMENSION  TEMPIJ(IJALL)
      DIMENSION TEMPIJ1(IJALL)
      DIMENSION TEMPIJ2(IJALL)
      DIMENSION swgclr (IJALL)

      EQUIVALENCE (   PKZ(1,1,1) , SPACE1(1,1,        1) )
      EQUIVALENCE (CLRSKY(1,1,1) , SPACE1(1,1,  NLAYR+1) )
      EQUIVALENCE (  ALLATS(1,1) , SPACE1(1,1,2*NLAYR+1) )
      EQUIVALENCE (   XLAT   (1) , SPACE1(1,1,2*NLAYR+2) )
      EQUIVALENCE (   TEMPIJ (1) , SPACE1(1,1,2*NLAYR+3) )
      EQUIVALENCE (   TEMPIJ1(1) , SPACE1(1,1,2*NLAYR+4) )
      EQUIVALENCE (   TEMPIJ2(1) , SPACE1(1,1,2*NLAYR+5) )
      EQUIVALENCE (   swgclr (1) , SPACE1(1,1,2*NLAYR+6) )
      EQUIVALENCE (    plz (1,1) , SPACE1(1,1,2*NLAYR+7) )
      EQUIVALENCE (    plze(1,1) , SPACE1(1,1,3*NLAYR+8) )
      EQUIVALENCE (     oz (1,1) , SPACE1(1,1,4*NLAYR+10))
C
      DIMENSION  SWHEAT(ISTRIP,NLAYR+1), PKSTR(ISTRIP,NLAYR)
      DIMENSION    SHZL(ISTRIP,NLAYR+1),  CLRO(ISTRIP,NLAYR+1)
      DIMENSION     TZL(ISTRIP,NLAYR+1),  CLMO(ISTRIP,NLAYR+1)
      DIMENSION     OZL(ISTRIP,NLAYR+1),  PLZSTR(ISTRIP,NLAYR+1)
      DIMENSION PLZESTR(ISTRIP,NLAYR+2)
      DIMENSION   TEMP1(ISTRIP)
      DIMENSION   TEMP2(ISTRIP)
      DIMENSION   TEMP3(ISTRIP)
      DIMENSION    COSZ(ISTRIP)
      DIMENSION   SCOSZ(ISTRIP)
      DIMENSION   RSURF(ISTRIP)

      DATA    KOZ /20/
C
C **********************************************************************
C ****                       INITIALIZATION                         ****
C **********************************************************************
C
      ssalb = getcon('SSALB')
C
      DO JJ=1,JMR+1
       ALAT(JJ) = - 90. + (JJ-1)*180./JMR
      ENDDO
      DO II=1,IMR
       ALON(II) = -180. + (II-1)*360./IMR
      ENDDO
C
      DO J = 1,JNP
      DO I = 1,IMR
       ALLATS(I,J) = J
      ENDDO
      ENDDO

      NTMSTP    = MODHMS(NDSWR,0)
      ISTNLP1   = ISTRIP * (NLAY+1)
      ISTNLAY   = ISTRIP * NLAY
      PI180     = PI / ONEETY
      FJEQ      = AHALF * (1 + JNP)

      DO I = 1,IJALL
       PLZE(I,1) = ZERO
       PLZE(I,2) = PTOP
        PLZ(I,1) = PTOP * AHALF
         XLAT(I) = DP * (ALLATS(I,1) - FJEQ) / PI180
      ENDDO
C
      DO L = 2,NLAYP1
       DO I = 1,IJALL
        PLZ (I,L  ) = SIG(L-1) * PZ(I,1,N) + PTOP
        PLZE(I,L+1) = SIGE(L)  * PZ(I,1,N) + PTOP
       ENDDO
      ENDDO
C
C **********************************************************************
C ****             CALCULATE COSINE OF THE ZENITH ANGLE             ****
C **********************************************************************
 
      CALL ASTRO ( NYMD,   NHMS,  ALAT,ALON, IMR,JMR+1, TEMPIJ1,RA )
                   NYMD2 = NYMD
                   NHMS2 = NHMS
      CALL TICK  ( NYMD2,  NHMS2, NTMSTP,NB2 )
      CALL ASTRO ( NYMD2,  NHMS2, ALAT,ALON, IMR,JMR+1, TEMPIJ2,RA )

      DO I = 1,IJALL
             TEMPIJ(I) = TEMPIJ1(I) + TEMPIJ2(I)
       IF  ( TEMPIJ(I) .GT.ZERO ) THEN
             TEMPIJ(I) =   TWOTHRD   *  ( TEMPIJ(I) -
     1      TEMPIJ2(I) * TEMPIJ1(I)/TEMPIJ(I) )
       ENDIF
      ENDDO
 
C **********************************************************************
C ****                   COMPUTE P**KAPPA                           ****
C **********************************************************************
 
      CALL PKAPPA ( PZ(1,1,N),PKHT,PKZ,IM,JNP,NLAY )
 
C **********************************************************************
C *** GET OZONE MASS MIXING RATIOS
C **********************************************************************
 
      CALL GETO3 (KOZ,NYMD,NHMS,IJALL,XLAT,NLAYP1,PLZ,WRK1,WRK2,OZ)
 
C***********************************************************************
C ****                     LOOP OVER GLOBAL REGIONS                 ****
C **********************************************************************
 
#if macro
cmic$ do all shared (albz, clrsky, dtradsw, iclmosw, iclrosw, ijall)
cmic$*       shared (imjm, imr, iosrclr, istnlay, istnlp1, istrip)
cmic$*       shared (iswclr, iswgclr, jmr, n, nlay, nlayp1, nlayr)
cmic$*       shared (npcs, nprogu2, nswcld, one, oz, pkz, plz)
cmic$*       shared (plze, qbudget, qdiag, qz, radswg, ssalb, swgclr)
cmic$*       shared (tempij, tempij1, tempij2, tz, zero)
cmic$*       shared (cswmo, cswro)
cmic$*       private (clmo, clro, cosz, i, nn, ozl)
cmic$*       private (pkstr, plzestr, plzstr, rsurf, scosz, shzl)
cmic$*       private (swheat, temp1, temp2, tzl)
#endif

      DO 1000 NN = 1,NPCS
 
C **********************************************************************
C ****  CALCULATE COSINE OF ZENITH ANGLE AND INCOMING RADIATION     ****
C **********************************************************************
 
      CALL STRIP ( TEMPIJ,COSZ,IJALL,ISTRIP,1,NN )
 
      DO I = 1,ISTRIP
      SCOSZ(I)  = ONE
      CLRO(I,1) = ZERO
      CLMO(I,1) = ZERO
      ENDDO
 
C **********************************************************************
C ****                   VARIABLE INITIALIZATION                    ****
C **********************************************************************
 
      CALL STRIP (PLZE,PLZESTR          ,IJALL,ISTRIP,NLAYP1+1,NN)
      CALL STRIP (PLZ,PLZSTR            ,IJALL,ISTRIP,NLAYP1  ,NN)
      CALL STRIP ( OZ,OZL               ,IJALL,ISTRIP,NLAYP1  ,NN)
      CALL STRIP (PKZ,PKSTR             ,IJALL,ISTRIP,NLAY    ,NN)
      CALL STRIP (ALBZ,RSURF            ,IJALL,ISTRIP,1       ,NN)
      CALL STRIP (TZ(1,1,1,N),TZL(1,2)  ,IJALL,ISTRIP,NLAY    ,NN)
      CALL STRIP (QZ(1,1,1,N),SHZL(1,2) ,IJALL,ISTRIP,NLAY    ,NN)
C
      DO I = 1,ISTNLAY
       TZL(I,2) =  TZL(I,2) * PKSTR(I,1)
      ENDDO
C
C *** INSERT FICTICIOUS UPPER BOUNDARY
C
      DO I = 1,ISTRIP
        TZL(I,1)  =  TZL(I,2)
       SHZL(I,1)  = SHZL(I,2)
      ENDDO
C
C **********************************************************************
C ****  CLEAR SKY  SHORT WAVE RADIATION FOR BUDGETS                 ****
C **********************************************************************
C
      IF (QBUDGET .or. iswclr .ne.0
     .            .or. iswgclr.ne.0
     .            .or. iosrclr.ne.0 ) THEN
       DO I = 1,ISTNLP1
        CLRO(I,1) = ZERO
        CLMO(I,1) = ZERO
       ENDDO

      CALL RADSW ( TZL,SHZL,OZL,PLZESTR(1,2),CLRO,CLMO,SCOSZ,RSURF,
     .             COSZ,TEMP1,SWHEAT,TEMP2,istrip,nlayr+1,ssalb )

      CALL PASTE ( SWHEAT(1,2),CLRSKY,ISTRIP,IJALL,NLAY,NN )
      CALL PASTE ( TEMP1    ,  swgclr,ISTRIP,IJALL,1   ,NN )
      CALL PASTE ( TEMP2    , TEMPIJ1,ISTRIP,IJALL,1   ,NN )
      ENDIF
C
C **********************************************************************
C ****              SHORT WAVE RADIATION                            ****
C **********************************************************************
C
      CALL STRIP (CSWRO,CLRO(1,2),IJALL,ISTRIP,NLAY,NN)
      CALL STRIP (CSWMO,CLMO(1,2),IJALL,ISTRIP,NLAY,NN)

      IF(NSWCLD.NE.0)THEN
       DO I = 1,ISTNLP1
       CLRO(I,1) = CLRO(I,1)/NSWCLD
       CLMO(I,1) = CLMO(I,1)/NSWCLD
       ENDDO
      ENDIF

      CALL RADSW ( TZL,SHZL,OZL,PLZESTR(1,2),CLRO,CLMO,SCOSZ,RSURF,
     .             COSZ,TEMP1,SWHEAT,TEMP2,istrip,nlayr+1,ssalb )
 
C **********************************************************************
C ****                     TENDENCY UPDATES                         ****
C **********************************************************************
 
      IF(ICLMOSW.GT.0)
     .CALL PSTBMP (CLMO(1,2),QDIAG(1,1,ICLMOSW),ISTRIP,IMJM,NLAY,NN)
      IF(ICLROSW.GT.0)
     .CALL PSTBMP (CLRO(1,2),QDIAG(1,1,ICLROSW),ISTRIP,IMJM,NLAY,NN)

      CALL PASTE ( SWHEAT(1,2),DTRADSW ,ISTRIP,IJALL,NLAY,NN )
      CALL PASTE ( TEMP1      , RADSWG ,ISTRIP,IJALL,1   ,NN )
      CALL PASTE ( TEMP2      , TEMPIJ2,ISTRIP,IJALL,1   ,NN )
 
 1000 CONTINUE
 
C **********************************************************************
C ****      SET DIAGNOSTICS  AND  ZERO-OUT CLOUD COUNTERS           ****
C **********************************************************************
 
      if(iosr.gt.0) then
      do i=1,ijall
      qdiag(i,1,iosr) = qdiag(i,1,iosr) + (1.-tempij2(i))*radswt(i,1)
      enddo
      endif

      if(iosrclr.gt.0) then
      do i=1,ijall
      qdiag(i,1,iosrclr) = qdiag(i,1,iosrclr)
     .                   + (1.-tempij1(i))*radswt(i,1)
      enddo
      endif

      if(iswgclr.gt.0) then
      do i=1,ijall
      qdiag(i,1,iswgclr) =  qdiag(i,1,iswgclr) + swgclr(i)*radswt(i,1)
      enddo
      endif

      if(iswclr.gt.0) then
      do L=1,nlayr
      do i=1,ijall
      qdiag(i,1,iswclr+L-1) =  qdiag(i,1,iswclr+L-1)
     .                      + clrsky(i,1,L)*radswt(i,1)*86400
      enddo
      enddo
      endif

      if(icldfrc.gt.0) then
       do I =1,ijall
        tempij1(I) = 1.
        tempij2(I) = 1.
       enddo
       do L=1,nlay
       do I =1,ijall
        cswro(i,1,L) = cswro(i,1,L) / nswcld
        cswmo(i,1,L) = cswmo(i,1,L) / nswcld
        tempij1(I) = tempij1(I) * (1.-cswro(I,1,L))
        if(tempij2(I) .GT. (1.-cswmo(I,1,L)) )
     1                           tempij2(I) = (1.-cswmo(I,1,L))
       enddo
       enddo
       do I =1,ijall
        tempij1(I)  = 1. - tempij1(I) * tempij2(I)
       enddo
       do i=1,ijall
        qdiag(i,1,icldfrc) =  qdiag(i,1,icldfrc) + tempij1(i)
       enddo
      endif

      DO L = 1,NLAY
       DO I = 1,IJALL
        CSWMO(I,1,L) = ZERO
        CSWRO(I,1,L) = ZERO
       ENDDO
      ENDDO
      NSWCLD  = 0
      NCLMOSW = NCLMOSW + 1
      NCLROSW = NCLROSW + 1
      NOSR    = NOSR    + 1
      NOSRCLR = NOSRCLR + 1
      NSWCLR  = NSWCLR  + 1
      NSWGCLR = NSWGCLR + 1
      ncldfrc = ncldfrc + 1
C
C **********************************************************************
C ****                PI-WEIGHT THE TENDENCIES                      ****
C **********************************************************************
C
      DO L = 1,NLAY
       DO I = 1,IJALL
        DTRADSW(I,1,L) = DTRADSW(I,1,L)*PZ(I,1,N) / PKZ(I,1,L)
       ENDDO
      ENDDO

C **********************************************************************
C ****        CALCULATE AND FILL APPROXIMATE BUDGETS                ****
C **********************************************************************

      IF (QBUDGET) THEN
      DO L = 1,NLAY
       DO I = 1,IJALL
        CLRSKY (I,1,L) = CLRSKY (I,1,L)*PZ(I,1,N) / PKZ(I,1,L)
       ENDDO
      ENDDO
      TMSTP = NTMSTP
      DO L=1,NLAY
       DO I = 1,IJALL
        ZTEMP(I,1) = DTRADSW(I,1,L)*RADSWT(I,1)
       ENDDO
       CALL ZMEAN ( ZTEMP(1,1),ZSWAVE(1,L),IM,JNP,TMSTP)
       DO I = 1,IJALL
        ZTEMP(I,1) = ZTEMP(I,1) - CLRSKY(I,1,L)*RADSWT(I,1)
       ENDDO
       CALL ZMEAN ( ZTEMP(1,1),ZSWCLD(1,L),IM,JNP,TMSTP)
      ENDDO
      ENDIF

      RETURN
      END
      SUBROUTINE GETO3 (KOZ,NYMD,NHMS,IRUN,XLAT,NLEVS,PLEVS,
     .                  O3INT1,O3INT2, OZONE)
C***********************************************************************
C  PURPOSE
C     TO READ AND INTERPOLATE OZONE VALUES
C
C  ARGUMENTS   DESCRIPTION
C     KOZ .... UNIT TO READ OZONE DATA
C     NYMD ... YYMMDD TO INTERPOLATE TO
C     NHMS ... HHMMSS TO INTERPOLATE TO
C     IRUN ... NUMBER OF COLUMNS TO BE FILLED
C     XLAT ... LATITUDE IN DEGREES
C     NLEVS... NUMBER OF PRESSURE (MODEL SIGMA) LEVELS TO BE FILLED
C     PLEVS... PRESSURES IN MB
C     O3INT1.. WORK SPACE
C     O3INT2.. WORK SPACE
C     OZONE... OZONE RETURNED IN KG/KG (MASS MIXING RATIO)
C
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
C

      PARAMETER ( ZERO     = 0.0 )
      PARAMETER ( ONE      = 1.0 )
      PARAMETER ( O3MIN    = 1.0E-10  )
      PARAMETER ( VOLTOMAS = 1.655E-6 )

      DIMENSION XLAT(IRUN), PLEVS(IRUN,NLEVS), OZONE(IRUN,NLEVS)
      DIMENSION O3INT1(IRUN,17   ,2)
      DIMENSION O3INT2(IRUN,NLEVS,2)

      DIMENSION PRDAT(17), O3LAT(18), OZDAT(18,17,12)
      DIMENSION QPR1(IRUN), QPR2(IRUN), SLOPE(IRUN)
      DIMENSION  PR1(IRUN),  PR2(IRUN)

      INTEGER   MONTH, DAY, SEC
      INTEGER   DAYS(13)
      DATA      DAYS /31,28,31,30,31,30,31,31,30,31,30,31,30/

      LOGICAL FIRST
      DATA    FIRST /.TRUE./
      SAVE    prdat, ozdat

      NSECF(N) = N/10000*3600 + MOD(N,10000)/100* 60 + MOD(N,100)
      NMONF(N) = MOD(N,10000)/100
      NDAYF(N) = MOD(N,100)

C  SET OZONE DATA LATITUDES
C==========================
      NPO3  = 17
      NOLAT = 18
      DO 10 IL=1,NOLAT
      O3LAT(IL) = -85. + (IL-1)*10.
   10 CONTINUE

C===========================
C  READ DATA (ONCE PER RUN )
C===========================
      IF(FIRST) THEN
      REWIND KOZ
C  READ OZONE PRESSURE LEVELS
C============================
      READ(KOZ,1000) (PRDAT(N),N=1,NPO3)
 1000 FORMAT(1X,F9.3)

C  READ OZONE AMOUNTS BY MONTH AND LEVEL
C=======================================
      DO 20 MON=1,12
      DO 20 IL=1,NOLAT
      READ(KOZ,2000) (OZDAT(IL,IP,mon),IP=1,10)
      READ(KOZ,2000) (OZDAT(IL,IP,mon),IP=11,NPO3)
 2000 FORMAT(6X,10F6.2)
   20 CONTINUE

      FIRST = .FALSE.
      ENDIF

C*********************************************************************
C****             FIND PROPER MONTH BOUNDARIES                    ****
C*********************************************************************

      SEC    = NSECF(NHMS)
      MONTH  = NMONF(NYMD)
      DAY    = NDAYF(NYMD)

      MIDMON = DAYS(MONTH)/2 + 1

      IF(DAY.LT.MIDMON) THEN
         IMNM = MONTH - 1
         IMNP = MONTH
      ELSE
         IMNM = MONTH
         IMNP = MONTH + 1
      ENDIF

      IF(IMNM.LT.1 ) IMNM = 12
      IF(IMNP.GT.12) IMNP = 1

C **********************************************************************
C ****           INTERPOLATE ozone data to model latitudes           ***
C **********************************************************************
C
      DO 30 M = 1,2
      if(m.eq.1) mon = imnm
      if(m.eq.2) mon = imnp

      DO 32 K=1,NPO3
      DO 34 I=1,IRUN

      DO 36 jlat = 1,Nolat
      IF( o3lat(jlat).gt.xlat(i) ) THEN
      IF( jlat.EQ.1 ) THEN
      jlatm    = 1
      jlatp    = 1
      slope(i) = zero
        ELSE
      jlatm    = jlat-1
      jlatp    = jlat
      slope(i) = (  XLAT(I)   -O3LAT(jlat-1) )
     .         / ( O3LAT(jlat)-O3LAT(jlat-1) )
      ENDIF
      GOTO 37
      ENDIF
   36 CONTINUE
      jlatm    = nolat
      jlatp    = nolat
      slope(i) = one
   37 CONTINUE
      QPR1(I) = ozdat(jlatm,k,mon)
      QPR2(I) = ozdat(jlatp,k,mon)
   34 CONTINUE

      DO 38 I=1,IRUN
      o3int1(i,k,m) = qpr1(i) + slope(i)*( qpr2(i)-qpr1(i) )
   38 CONTINUE

   32 CONTINUE
   30 CONTINUE
C
C **********************************************************************
C ****     INTERPOLATE latitude ozone data to model pressures        ***
C **********************************************************************
C
      DO 40 M  = 1,2
      DO 40 L2 = 1,NLEVS

      DO 44 I  = 1,IRUN
      DO 46 L1 = 1,NPO3
      IF( PRDAT(L1).GT.PLEVS(I,L2) ) THEN
      IF( L1.EQ.1 ) THEN
          LM = 1
          LP = 2
        ELSE
          LM = L1-1
          LP = L1
      ENDIF
      GOTO 47
      ENDIF
   46 CONTINUE
            LM = NPO3-1
            LP = NPO3
   47 CONTINUE
       PR1(I) = PRDAT (LM)
       PR2(I) = PRDAT (LP)
      QPR1(I) = O3INT1(I,LM,M)
      QPR2(I) = O3INT1(I,LP,M)
   44 CONTINUE

      DO 48 I=1,IRUN
            SLOPE(I) = ( QPR1(I)-QPR2(I) )
     .               / (  PR1(I)- PR2(I) )
      O3INT2(I,L2,M) =   QPR2(I) + ( PLEVS(I,L2)-PR2(I) )*SLOPE(I)

      if( o3int2(i,l2,m).lt.o3min ) then
          o3int2(i,l2,m) =  o3min
      endif

   48 CONTINUE
   40 CONTINUE

C*********************************************************************
C**** INTERPOLATE TO TIME FOR ALL MODEL LEVELS AND STRIP POINTS   ****
C*********************************************************************

      IF(DAY.LT.MIDMON) THEN
               ISC  = (DAY+DAYS(IMNM))*86400 + SEC
      ELSE
               ISC  = (DAY           )*86400 + SEC
      ENDIF

      ISCM = (DAYS(IMNM)/2+1              )*86400
      ISCP = (DAYS(IMNP)/2+1 + DAYS(IMNM) )*86400

      FACP = (ISC-ISCM) / REAL(ISCP-ISCM)
      FACM = ONE-FACP

      DO 50 I=1,IRUN
      DO 50 L=1,NLEVS
      OZONE(I,L) = O3INT2(I,L,1)*FACM + O3INT2(I,L,2)*FACP

   50 CONTINUE
C
C **********************************************************************
C ****     CONVERT FROM VOLUME MIXING RATIO TO MASS MIXING RATIO     ***
C **********************************************************************
C
      DO 60 I=1,IRUN*NLEVS
      OZONE (I,1) = OZONE(I,1) * VOLTOMAS
  60  CONTINUE
C
      RETURN
      END
      SUBROUTINE RADSW ( TLO,QLO,O3O,PL2,CLU,CLC,SLR,ALB,ZTH,
     *                   SWD,SWF,SWN,m,lm,ssalb )

      LOGICAL     OZONE, FIXCLD, FIXOZE
C
      PARAMETER  (  OZONE = .TRUE.  )
      PARAMETER  ( FIXOZE = .FALSE. )
      PARAMETER  ( FIXCLD = .FALSE. )
C
      PARAMETER
     *          (RKAP=.286,    CP=1003.5,       ALHL=2.5E6, RGAS=CP*RKAP
     *,          GRAV=9.81,    GCGS=GRAV*100.0, TICE=273.16
     *,          P00=1000.,    SRFPRS=984.7,    STFBOL=5.6695E-8
     *,          VKRM=0.41,    ALBYCP=ALHL/CP)
      PARAMETER
     *          (CMB2PA=100.0, PATOCGS=10.,     CMB2CGS=CMB2PA*PATOCGS)
C
      PARAMETER
     *          (ZERO=0.0, ONE=1.0,         TWO=2.0,     THREE=3.0
     *,          FOUR=4.0, FIVE=5.0,        SIX=6.0,     ONE80=180.0
     *,          AHALF=0.5, THIRD=ONE/THREE, FOURTH=0.25, EIGHTH=0.125
     *,          UNDEF=ZERO)
C
      PARAMETER  (BSF=FIVE,  VSMIN=AHALF,  V2MIN=VSMIN*VSMIN, VIR=.61
     *,           PBLZET=125., BSF15=BSF*15.)
C
      PARAMETER (G0=.85, GP=G0*THREE/(ONE+G0))
      PARAMETER (PNOOZ=500.,WASTR=3.E-6,WAMIN=1.E-10,O3MIN=1.E-10)
      PARAMETER (TUMIN=1.E-6)
      PARAMETER (EXPMIN=-10.)
      PARAMETER (A1=35.,   A2=1224., A3=120.1612, A4=476.)
      PARAMETER (B1=.1299, B2=.2775, B3=.45,      B4=.866, B5=.118)
      PARAMETER (          C2=1.9,   C3=.647)
      PARAMETER (SEVEN=7.)
      PARAMETER (CIRTHK=.16 )
C
      DIMENSION TLO(M,LM), QLO(M,LM), O3O(M,LM), PL2(M,LM), CLU(M,LM)
     *,         ALB(M)   , SLR(M)   , ZTH(M)   , CLC(M,LM)
     *,         SWD(M)   , SWF(M,LM), SWN(M)
C
      DIMENSION TX1 (M,lm+1), TX2 (M,lm+1), TX3 (M,lm+1),
     .          TX4 (M,lm+1), TX5 (M,lm+1), TX6 (M,lm+1),
     .          TX7 (M,lm+1), TX8 (M,lm+1), TX9 (M,lm+1),
     .          DPO (M,lm+1), SWI (M,lm+1), OZL (M,lm+1),
     .          DNN (M,lm+1), UPP (M,lm+1), ACC (M,lm+1), SSC (M,lm+1),
     .          RLL (M,lm+1), TLL (M,lm+1), TAU (M,lm+1), SOL (M,lm+1),
     .          GM1 (M,lm+1), GM2 (M,lm+1), AL1 (M,lm+1), AL2 (M,lm+1),
     .          ALL (M,lm+1), AAL (M,lm+1), BBL (M,lm+1), CLL (M,lm+1),
     .          DLL (M,lm+1), ELL (M,lm+1), FLL (M,lm+1), ELO (M,lm+1),
     .          ULL (M,lm+1), TAC (M,lm+1)

      DIMENSION CSM (M), GLL (M), ECN (M), ACN (M)
      DIMENSION RC1 (M), RCO (M), RCG (M), TUT (M)
      DIMENSION TZ1 (M), TZ2 (M), TZ3 (M), TZ4 (M)
      DIMENSION TZ5 (M), TZ6 (M)

      DIMENSION FK(5), XK(5)
      DATA FK/0.107,0.104,0.073,0.044,0.025/
      DATA XK/0.005,0.041,0.416,4.752,72.459/
C
c     SAVE

       LMP1 = LM+1
       LMM1 = LM-1
       LMP2 = LM+2
       LMP3 = LM+3
       LMM2 = LM-2
      MLMM0 = M*LM
      MLMM1 = M*LMM1
      MLMP1 = M*LMP1
C
C Get Single Scattering Albedo Coefficient
C ----------------------------------------
      C1 = SSALB
C
      DO 100 I = 1,MLMP1
      ACC(I,1) = ZERO
 100  CONTINUE
      DO 101 I = 1,M
      TX2(I,1    ) = ZERO
 101  CONTINUE
      DO 102 I = 1,MLMM0
      TX2(I,2) = PL2(I,1)
      DPO(I,1) = TX2(I,2) - TX2(I,1)
 102  CONTINUE
      DO 103 I = 1,MLMP1
      TX1(I,1) = TX2(I,1) * TX2(I,1)
 103  CONTINUE
      DO 104 I = 1,MLMM0
      SWI(I,1) = TX1(I,2) - TX1(I,1)
 104  CONTINUE
C
      DO 105 I = 1,MLMM0
       TX2(I,1) = QLO(I,1)
       IF(TX2(I,1).LT.WAMIN) THEN
        TX2(I,1) = WAMIN
       ENDIF
       SWI(I,1) = SWI(I,1) * TX2(I,1)
       TX1(I,1) = A3             * SQRT(TLO(I,1))
       SWI(I,1) = SWI(I,1) / TX1(I,1)
C
       TX1(I,1) = O3O(I,1)
       IF(TX1(I,1).LT.O3MIN)THEN
        TX1(I,1) = O3MIN
       ENDIF
       IF(PL2(I,1).GT.PNOOZ) TX1(I,1) = ZERO
       TX1(I,1) = A4             * TX1(I,1)
       TX1(I,1) = TX1(I,1) * DPO(I,1)
 105  CONTINUE
C
      CALL OPTTHK (TLO,MLMM0,TX2)
C
      DO 55 I=1,MLMM0
      IF (CLC(I,1).GT.ZERO) TX2(I,1) = CIRTHK
55    CONTINUE
C
      DO 106 I = 1,M
       TUT(I)=ZERO
       OZL(I,1)=ZERO
 106  CONTINUE
      DO 10 L=1,LM
CB
      DO 9000 I =1,M
      TX2(I,L) = TX2(I,L) * DPO(I,L)
      TX3(I,L) = ONE - (ONE - CLC(I,L)) * (ONE - CLU(I,L))
      TAC(I,L) = TX2(I,L) * TX3(I,L)
CCC   TAC(I,L) = CLU(I,L) * TX2(I,L)
CCC   TAC(I,L) = TAC(I,L) + CLC(I,L) * CIRTHK
      TUT(I) = TUT(I) + TAC(I,L)
      OZL(I,L+1) = OZL(I,L) + TX1(I,L)
9000  CONTINUE
CE
 10   CONTINUE
C
CB
      DO 9002 I =1,M
      CSM(I) = A2*ZTH(I)*ZTH(I) + ONE
CI
      CSM(I) = A1 / SQRT(CSM(I))
CI
C
      TZ4(I) = TWO + THREE*ZTH(I)
      TZ5(I) = TWO - THREE*ZTH(I)
C
      TZ2(I) = ONE      + TUT(I)*B1
      TZ2(I) = ALB(I) / TZ2(I)
      ECN(I) = -B2      * TUT(I)
      ECN(I) = ECN(I) * CSM(I)
      IF(ECN(I).LE.EXPMIN) ECN(I) = EXPMIN
CI
      ECN(I) = EXP(ECN(I))
CI
      ACN(I) = TZ5(I) * ECN(I)
      ACN(I) = TZ4(I) + ACN(I)
      TZ1(I) = FOUR     + TUT(I)*B3
      ACN(I) = ACN(I) / TZ1(I)
      RC1(I) = ACN(I) * TZ2(I)
      TZ1(I) = TZ2(I) - ALB(I)
      TZ1(I) = TZ1(I) + ONE
      RC1(I) = RC1(I) / TZ1(I)
      RC1(I) = RC1(I) + ONE
      RC1(I) = RC1(I) - ACN(I)
C
      TZ3(I) = TUT(I) + G0
      GLL(I) = G0       * TUT(I)
      GLL(I) = GLL(I) / TZ3(I)
      TZ3(I) = (ONE-GLL(I)) * TZ3(I)
      TZ2(I) = ONE      + TZ3(I) * B4
      TZ2(I) = ALB(I) / TZ2(I)
      ECN(I) =-TZ3(I) * (ONE+GLL(I))
      ECN(I) = ECN(I) * CSM(I)
      IF(ECN(I).LE.EXPMIN) ECN(I) = EXPMIN
CI
      ECN(I) = EXP(ECN(I))
CI
      ACN(I) = TZ5(I) * ECN(I)
      ACN(I) = TZ4(I) + ACN(I)
      TZ1(I) = FOUR     + TZ3(I)*THREE
      ACN(I) = ACN(I) / TZ1(I)
      RCO(I) = ACN(I) * TZ2(I)
      TZ1(I) = TZ2(I) - ALB(I)
      TZ1(I) = TZ1(I) + ONE
      RCO(I) = RCO(I) / TZ1(I)
      RCO(I) = RCO(I) + ONE
      RCO(I) = RCO(I) - ACN(I)
      RCO(I) = RCO(I) + RC1(I)*TWO
      RCO(I) = RCO(I) * (ONE/THREE)
C
      TZ3(I) = TUT(I) + B5
      GLL(I) = G0 * TUT(I) / TZ3(I)
      TZ3(I) = (ONE-GLL(I)) * TZ3(I)
      TZ2(I) = ONE      + TZ3(I) * B4
      TZ2(I) = ALB(I) / TZ2(I)
      ECN(I) =-TZ3(I) * (ONE+GLL(I))
      ECN(I) = ECN(I) * CSM(I)
      IF(ECN(I).LE.EXPMIN) ECN(I) = EXPMIN
CI
      ECN(I) = EXP(ECN(I))
CI
      ACN(I) = TZ5(I) * ECN(I)
      ACN(I) = TZ4(I) + ACN(I)
      TZ1(I) = FOUR     + TZ3(I)*THREE
      ACN(I) = ACN(I) / TZ1(I)
C
      RCG(I) = ACN(I) * TZ2(I)
      TZ1(I) = TZ2(I) - ALB(I)
      TZ1(I) = TZ1(I) + ONE
      RCG(I) = RCG(I) / TZ1(I)
      RCG(I) = RCG(I) + ONE
      RCG(I) = RCG(I) - ACN(I)
C
      TZ1(I) = RCO(I) * SLR(I)
      TZ2(I) = OZL(I,LMP1) * (C2+CSM(I))
9002  CONTINUE
CE
      DO 20 L=1,LM
CB
      DO 9004 I =1,M
      TX5(I,L) = SLR(I)
      TX6(I,L) = CSM(I)
      TX7(I,L) = TZ2(I)
      TX8(I,L) = TZ1(I)
      TX9(I,L) = ZTH(I)
9004  CONTINUE
CE
20    CONTINUE
C
      DO 9006 I =1,M
      DNN(I,1   ) = ZERO
9006  CONTINUE
      DO 9008 I =1,MLMM0
      TX2(I,1) = OZL(I,2) * TX6(I,1)
9008  CONTINUE
      CALL ZUM(TX2,TX1,DNN(1,2),MLMM0)
C
      DO 9010 I =1,M
      UPP(I,LMP1) = DNN(I,LMP1)
9010  CONTINUE
      DO 9012 I =1,MLMM0
      TX2(I,1) = TX7(I,1) - OZL(I,1)*C2
9012  CONTINUE
      CALL ZUM(TX2,TX1,UPP(1,1),MLMM0)
C
CB
      DO 9014 I =1,MLMM0
      ACC(I,1) = DNN(I,2) - DNN(I,1)
      ACC(I,1) = ACC(I,1) * TX5(I,1)
9014  CONTINUE
CE
C
CB
      DO 9016 I =1,M
      SWD(I) = ONE      - RCG(I)
      SWD(I) = SWD(I) * SLR(I)
      TZ1(I) = C3       - DNN(I,LMP1)
      SWD(I) = SWD(I) * TZ1(I)
9016  CONTINUE
CE
C
CB
      DO 9018 I =1,MLMM0
      TX1(I,1) = UPP(I,1) - UPP(I,2)
      TX1(I,1) = TX1(I,1) * TX8(I,1)
      ACC(I,1) = ACC(I,1) + TX1(I,1)
      IF(ACC(I,1).LT.ZERO) ACC(I,1) = ZERO
9018  CONTINUE
CE
C
CB
      DO 9020 I =1,M
      TZ1(I) = TWO      - ZTH(I)*GP
      TZ1(I) = FOURTH   * TZ1(I)
      TZ2(I) = ONE      - TZ1(I)
9020  CONTINUE
CE
C
      DO 1000 IK=1,5
C
CB
      DO 9022 I =1,MLMM0
      TAU(I,1) = TAC(I,1)    + SWI(I,1)*XK(IK)
      SSC(I,1) = C1*TAC(I,1) / TAU(I,1)
C
      TX1(I,1) = ONE-SSC(I,1)*G0
      TX2(I,1) = ONE-SSC(I,1)
CI
      TX1(I,1) = SQRT(TX1(I,1))
      TX2(I,1) = SQRT(TX2(I,1))
CI
C
      IF(TX2(I,1).NE.ZERO) THEN
       TX3(I,1) = TX1(I,1) / TX2(I,1)
      ELSE
       TX3(I,1) = ZERO
      ENDIF
      TX1(I,1) = TX1(I,1) * TX2(I,1)
      TX1(I,1) = TX1(I,1) * TAU(I,1)
      TX1(I,1) =-SQRT(THREE)    * TX1(I,1)
C
      IF(TX1(I,1).LE.EXPMIN  ) TX1(I,1) = EXPMIN
9022  CONTINUE
CI
      CALL EXPSUB(TX1,MLMM0,TX4)
CI
C
      DO 9034 I =1,MLMM0
      TX1(I,1) = TX3(I,1) + ONE
      TX2(I,1) = TX3(I,1) - ONE
      TX1(I,1) = TX1(I,1) * TX1(I,1)
      TX2(I,1) = TX2(I,1) * TX2(I,1)
      TX2(I,1) = TX2(I,1) * TX4(I,1)
      TX2(I,1) = TX2(I,1) * TX4(I,1)
      TX1(I,1) = TX1(I,1) - TX2(I,1)
      TX1(I,1) = ONE            / TX1(I,1)
C
      RLL(I,1) = TX3(I,1) * TX3(I,1)-ONE
      TX2(I,1) = TX4(I,1) * TX4(I,1)-ONE
      RLL(I,1) = RLL(I,1) * TX2(I,1)
      RLL(I,1) =-RLL(I,1) * TX1(I,1)
C
      TLL(I,1) = TX3(I,1) * TX4(I,1)
      TLL(I,1) = TLL(I,1) * TX1(I,1)
      TLL(I,1) = TLL(I,1) * FOUR
C
      TX1(I,1) = ONE -  (G0*G0) * SSC(I,1)
      TAU(I,1) = TAU(I,1) * TX1(I,1)
      SSC(I,1) = SSC(I,1) * (ONE-G0*G0)
      SSC(I,1) = SSC(I,1) / TX1(I,1)
      TX1(I,1) =-TAU(I,1) * TX6(I,1)
C
      IF(TX1(I,1).LE.EXPMIN  ) TX1(I,1) = EXPMIN
9034  CONTINUE
CI
      CALL EXPSUB(TX1,MLMM0,ELO)
CI
C
      DO 9076 I =1,MLMM0
      GM1(I,1) = SEVEN  - SSC(I,1)*(FOUR+GP)
      GM1(I,1) = FOURTH * GM1(I,1)
      GM2(I,1) = ONE    - SSC(I,1)*(FOUR-GP)
      GM2(I,1) =-FOURTH * GM2(I,1)
C
      ALL(I,1) = GM2(I,1) * GM2(I,1)
      TX1(I,1) = GM1(I,1) * GM1(I,1)
      ALL(I,1) = TX1(I,1) - ALL(I,1)
CI
      ALL(I,1) = SQRT(ALL(I,1))
CI
9076  CONTINUE
CE
C
      DO 9090 I =1,M
      SOL(I,1) = SLR(I) * FK(IK)
9090  CONTINUE
C
      DO 30 L=1,LM
CB
      DO 9092 I =1,M
      SOL(I,L+1) = SOL(I,L) * ELO(I,L)
      TX1(I,L  ) = TZ1(I)
      TX2(I,L  ) = TZ2(I)
9092  CONTINUE
CE
30    CONTINUE
C
CB
      DO 9094 I =1,MLMM0
      TX3(I,1) = GM1(I,1) * TX2(I,1)
      AL1(I,1) = GM2(I,1) * TX1(I,1)
      AL1(I,1) = AL1(I,1) + TX3(I,1)
C
      TX3(I,1) = GM1(I,1) * TX1(I,1)
      AL2(I,1) = GM2(I,1) * TX2(I,1)
      AL2(I,1) = AL2(I,1) + TX3(I,1)
C
      TX3(I,1) = AL2(I,1) * TX9(I,1)
      AAL(I,1) = TX1(I,1) - TX3(I,1)
      AAL(I,1) = AAL(I,1) * ALL(I,1)
      AAL(I,1) = AAL(I,1) * ELO(I,1)
      AAL(I,1) = AAL(I,1) * TWO
C
      TX3(I,1) = AL1(I,1) * TX9(I,1)
      BBL(I,1) = TX2(I,1) + TX3(I,1)
      BBL(I,1) = BBL(I,1) * ALL(I,1)
      BBL(I,1) = BBL(I,1) * TWO
C
      TX3(I,1) = ALL(I,1) * TX9(I,1)
      TX7(I,1) = ONE            - TX3(I,1)
      TX8(I,1) = ONE            + TX3(I,1)
C
      TX3(I,1) = ALL(I,1) * TX1(I,1)
      CLL(I,1) = AL2(I,1) + TX3(I,1)
      DLL(I,1) = AL2(I,1) - TX3(I,1)
      CLL(I,1) = CLL(I,1) * TX7(I,1)
      DLL(I,1) = DLL(I,1) * TX8(I,1)
C
      TX3(I,1) = ALL(I,1) * TX2(I,1)
      FLL(I,1) = AL1(I,1) + TX3(I,1)
      ELL(I,1) = AL1(I,1) - TX3(I,1)
      FLL(I,1) = FLL(I,1) * TX8(I,1)
      ELL(I,1) = ELL(I,1) * TX7(I,1)
C
      TX3(I,1) = ALL(I,1) * TX9(I,1)
      TX3(I,1) = TX3(I,1) * TX3(I,1)
      TX3(I,1) = ONE            - TX3(I,1)
CI
      IF(ABS(TX3(I,1)).LT.TUMIN) TX3(I,1)=TUMIN
CI
C
      TX1(I,1) =-ALL(I,1) * TAU(I,1)
      IF(TX1(I,1).LE.EXPMIN) TX1(I,1) = EXPMIN
9094  CONTINUE
CI
      CALL EXPSUB(TX1,MLMM0,TX2)
CI
C
      DO 9024 I = 1,MLMM0
      TX4(I,1) = TX2(I,1) * TX2(I,1)
      TX1(I,1) = ALL(I,1) - GM1(I,1)
      TX1(I,1) = TX1(I,1) * TX4(I,1)
      TX7(I,1) = ALL(I,1) + GM1(I,1)
      TX1(I,1) = TX1(I,1) + TX7(I,1)
      TX1(I,1) = TX1(I,1) * TX3(I,1)
      TX1(I,1) = SSC(I,1) / TX1(I,1)
      TX1(I,1) = SOL(I,1) * TX1(I,1)
C
      TX7(I,1) = DLL(I,1) * TX4(I,1)
      TX7(I,1) = CLL(I,1) - TX7(I,1)
      TX3(I,1) = AAL(I,1) * TX2(I,1)
      TX7(I,1) =-TX3(I,1) + TX7(I,1)
      TX7(I,1) = TX1(I,1) * TX7(I,1)
C
      TX8(I,2) = ELL(I,1) * TX4(I,1)
      TX8(I,2) = FLL(I,1) - TX8(I,2)
      TX8(I,2) = ELO(I,1) * TX8(I,2)
      TX3(I,1) = BBL(I,1) * TX2(I,1)
      TX8(I,2) =-TX3(I,1) + TX8(I,2)
      TX8(I,2) =-TX1(I,1) * TX8(I,2)
9024  CONTINUE
CE
C
CB
      DO 9096 I =1,M
      TX7(I,LMP1) = ALB(I) * SOL(I,LMP1)
      RLL(I,LMP1) = ALB(I)
      TX8(I,1   ) = ZERO
9096  CONTINUE
CE
C
      DO 9098 I =1,MLMP1
      IF(RLL(I,1).LT.ZERO)   RLL(I,1) = ZERO
9098  CONTINUE
      DO 9100 I =1,MLMM0
      IF(TLL(I,1).LT.ZERO)   TLL(I,1) = ZERO
9100  CONTINUE
CB
      DO 9102 I =1,MLMP1
      IF(TX7(I,1).LT.ZERO)   TX7(I,1) = ZERO
      IF(TX8(I,1).LT.ZERO)   TX8(I,1) = ZERO
      IF(SOL(I,1).LT.ZERO)   SOL(I,1) = ZERO
9102  CONTINUE
CE
C
CB
      DO 9104 I =1,M
      AL1(I,1) = ZERO
      AL2(I,1) = ONE
      BBL(I,1) = ZERO
      AAL(I,1) = TX7(I,1)
9104  CONTINUE
CE
      DO 9106 I =1,MLMM0
      TX1(I,1) = TLL(I,1) * TLL(I,1)
9106  CONTINUE
C
      DO 300 L=1,LM
C
CB
      DO 9108 I =1,M
      AL1(I,L+1) = AL1(I,L  ) * AL2(I,L)
      AL1(I,L+1) = AL1(I,L+1) * TX1(I,L)
      AL1(I,L+1) = AL1(I,L+1) + RLL(I,L)
C
      AL2(I,L+1) = AL1(I,L+1) * RLL(I,L+1)
      AL2(I,L+1) = ONE          - AL2(I,L+1)
      AL2(I,L+1) = ONE          / AL2(I,L+1)
9108  CONTINUE
CE
C
300   CONTINUE
C
CB
      DO 9110 I =1,MLMM0
      TX2(I,1) = AL1(I,1) * TX7(I,1)
      TX3(I,1) = AL2(I,1) * TLL(I,1)
9110  CONTINUE
CE
C
      DO 400 L=1,LM
CB
      DO 9112 I =1,M
      BBL(I,L+1) = BBL(I,L  ) + TX2(I,L  )
      BBL(I,L+1) = BBL(I,L+1) * TX3(I,L  )
      BBL(I,L+1) = BBL(I,L+1) + TX8(I,L+1)
9112  CONTINUE
CE
400   CONTINUE
C
CB
      DO 9114 I =1,MLMM0
      TX2(I,1) = BBL(I,2) * RLL(I,2)
      TX2(I,1) = TX2(I,1) + TX7(I,2)
      AAL(I,2) = AL2(I,2) * TX2(I,1)
9114  CONTINUE
CE
C
CB
      DO 9116 I =1,M
      ULL(I,LMP1) = AAL(I,LMP1)
      DLL(I,LMP1) = AL1(I,LMP1) * ULL(I,LMP1)
      DLL(I,LMP1) = BBL(I,LMP1) + DLL(I,LMP1)
      DLL(I,LMP1) = DLL(I,LMP1) + SOL(I,LMP1)
      TZ3(I) = DLL(I,LMP1) - ULL(I,LMP1)
      ACC(I,LMP1) = ACC(I,LMP1) + TZ3(I)
9116  CONTINUE
CE
C
      DO 9118 I =1,MLMM0
      TX1(I,1) = TLL(I,1) * AL2(I,1)
9118  CONTINUE
      DO 500 L=LM,1,-1
CB
      DO 9120 I =1,M
      ULL(I,L) = ULL(I,L+1) * TX1(I,L)
      ULL(I,L) = AAL(I,L  ) + ULL(I,L)
9120  CONTINUE
CE
 500  CONTINUE
C
CB
      DO 9122 I =1,MLMM0
      TX1(I,1) = AL1(I,1) * ULL(I,1)
      DLL(I,1) = SOL(I,1) + TX1(I,1)
      DLL(I,1) = BBL(I,1) + DLL(I,1)
9122  CONTINUE
CE
C
      DO 9124 I =1,MLMP1
      TX1(I,1) = DLL(I,1) - ULL(I,1)
9124  CONTINUE
CB
      DO 9126 I =1,MLMM0
      TX2(I,1) = TX1(I,1) - TX1(I,2)
      ACC(I,1) = ACC(I,1) + TX2(I,1)
9126  CONTINUE
CE
C
1000  CONTINUE
C
CB
      DO 9128 I =1  ,M
      SWD(I) = SWD(I) + ACC(I,LMP1)
      SWN(I) = SWD(I)
9128  CONTINUE
CE
      DO 1010 L=1,LM
      DO 9130 I =1  ,M
      SWN(I) = SWN(I) + ACC(I,L)
9130  CONTINUE
1010  CONTINUE
CB
      DO 9132 I =1,MLMM0
      SWF(I,1) = ACC(I,1) / DPO(I,1)
      SWF(I,1) = SWF(I,1) * (GRAV/(CMB2PA*CP))
9132  CONTINUE
CE
C
      RETURN
      END
      SUBROUTINE EXPSUB(ARG,LEN,OUT)
      PARAMETER (ONE=1.0)
      DIMENSION ARG(LEN), OUT(LEN)
      DIMENSION EXPCOEF(4)
      DATA EXPCOEF/1., -2.507213E-1, 2.92732E-2, -3.8278E-3/

      DO 10 I = 1,LEN
       OUT(I) = EXPCOEF(4)
       OUT(I) = OUT(I) * ARG(I) + EXPCOEF(3)
       OUT(I) = OUT(I) * ARG(I) + EXPCOEF(2)
       OUT(I) = OUT(I) * ARG(I) + EXPCOEF(1)
       OUT(I) = OUT(I) * OUT(I)
       OUT(I) = OUT(I) * OUT(I)
       OUT(I) = ONE /(OUT(I))
 10   CONTINUE

      RETURN
      END
      SUBROUTINE ZUM(W,T,D,M)
C
      PARAMETER (A1=.042,    A2=.000323, A3=.02118, A4=138.57)
      PARAMETER (A5=1.08173, A6=103.63,  A7=.0658,  E1=-.805, ONE=1.0)
      DIMENSION W(M), T(M), D(M)
C
      DO 10 I = 1,M
      T(I) = A1     + A2     * W(I)
      T(I) = ONE    + T(I) * W(I)
      D(I) = A3     / T(I)
C
      T(I) = ONE    + A4     * W(I)
      T(I) = T(I) ** E1
      D(I) = D(I) + A5     * T(I)
C
      T(I) = A6     * W(I)
      T(I) = ONE    + T(I) * (T(I)*T(I))
      T(I) = A7     / T(I)
      D(I) = D(I) + T(I)
C
      D(I) = D(I) * W(I)
  10  CONTINUE
C
      RETURN
      END
       SUBROUTINE LWRIO (N,NYMD,NHMS,NDLWR,QBUDGET)
#include <vrslv.com>
#include <vstate.com>
#include <vmetric.com>
#include <vphyscom.com>
#include <vbudget.com>
#include <stripmine.com>
C
      PARAMETER ( IJALL   =  IMR * (JMR+1)  )
      PARAMETER ( ZERO   = 0.0      )
      PARAMETER ( AHALF  = 0.5      )
      PARAMETER ( ONE    = 1.0      )
      PARAMETER ( HUNDRD = 100.     )
      PARAMETER ( ONE80  = 180.0    )
C
C
      COMMON /SPACE/ SPACE1 (IMR,JMR+1,4*NLAYR+5)
      COMMON /SPACE/ WRK1   (IJALL,17     ,2)
      COMMON /SPACE/ WRK2   (IJALL,NLAYR+1,2)

      DIMENSION  WKSPCE (ISTRIP*(NLAYR+3)*25)
C
      DIMENSION       XLAT(IJALL)
      DIMENSION     ALLATS(IMR,JMR+1)
      DIMENSION   DTRADLWC(IMR,JMR+1,NLAYR)
      DIMENSION         OZ(IJALL,NLAYR+1)
      DIMENSION        PLZ(IJALL,NLAYR+1)
      DIMENSION       PLZE(IJALL,NLAYR+1)

      EQUIVALENCE (     XLAT(1)     , SPACE1(1,1,        1) )
      EQUIVALENCE (   ALLATS(1,1)   , SPACE1(1,1,        2) )
      EQUIVALENCE ( DTRADLWC(1,1,1) , SPACE1(1,1,        3) )
      EQUIVALENCE (      PLZ(1,  1) , SPACE1(1,1,  NLAYR+3) )
      EQUIVALENCE (       OZ(1,  1) , SPACE1(1,1,2*NLAYR+4) )
      EQUIVALENCE (     PLZE(1,  1) , SPACE1(1,1,3*NLAYR+5) )

      DIMENSION     PLK(ISTRIP,NLAYR)   , ADELPL(ISTRIP,NLAYR)
      DIMENSION   DTRAD(ISTRIP,NLAYR)   , DTRADC(ISTRIP,NLAYR)
      DIMENSION     OZL(ISTRIP,NLAYR+1) ,    TZL(ISTRIP,NLAYR+1)
      DIMENSION    SHZL(ISTRIP,NLAYR+1) ,   CLRO(ISTRIP,NLAYR+1)
      DIMENSION PLZESTR(ISTRIP,NLAYR+1) ,   CLMO(ISTRIP,NLAYR+1)
      DIMENSION PLZSTR (ISTRIP,NLAYR+1) ,   PLKE(ISTRIP,NLAYR+1)
      DIMENSION   FLXIR(ISTRIP,NLAYR+2) , CLRFLX(ISTRIP,NLAYR+2)

      DIMENSION  SGMT4(ISTRIP)
      DIMENSION  TGSTR(ISTRIP)    ,  TSURF(ISTRIP)

      LOGICAL OZONFL , QBUDGET
      DATA    KOZ    /20/

C **********************************************************************
C ****                     INITIALIZATION                           ****
C **********************************************************************
C
      OZONFL = .TRUE.
      FJEQ   = AHALF * (1+JNP)
      ZPI0   = ACOS    (-ONE )
      SECDAY = GETCON('SDAY')
      CONVRT = GETCON('GRAVITY') / ( HUNDRD * GETCON('CP') )
      ZKAPPA = GETCON('KAPPA')
      PTOPKP = PTOP**ZKAPPA
      co2    = getcon('CO2')
C
      DO J = 1,JNP
       DO I = 1,IMR
        ALLATS(I,J) = FLOAT(J)
       ENDDO
      ENDDO

      PI180     = PI / ONE80

      DO I = 1,IJALL
        PLZ(I,1) = PTOP * AHALF
       PLZE(I,1) = PTOP
         XLAT(I) = DP * (ALLATS(I,1) - FJEQ) / PI180
      ENDDO
C
       DO L = 2,NLAYP1
        DO I = 1,IJALL
         PLZ (I,L) = SIG(L-1) * PZ(I,1,N) + PTOP
         PLZE(I,L) = SIGE(L)  * PZ(I,1,N) + PTOP
        ENDDO
       ENDDO

C***********************************************************************
C *** GET OZONE MASS MIXING RATIOS
C***********************************************************************

      CALL GETO3 (KOZ,NYMD,NHMS,IJALL,XLAT,NLAYP1,PLZ,WRK1,WRK2,OZ)

C***********************************************************************
C ****                     LOOP OVER LATITUDE                       ****
C **********************************************************************

#if macro
cmic$ do all shared (clwmo, clwro, co2, convrt, dtradlw, dtradlwc)
cmic$*       shared (iclmolw, iclrolw, ijall, ilwclr, ilwgclr)
cmic$*       shared (imjm, imr, iolr, iolrclr, iozlw, iradlw, ist4)
cmic$*       shared (istrip, jmr, n, nlay, nlayp1, nlayr, nlwcld)
cmic$*       shared (npcs, nprogu2, oz, ozonfl, pkht, plz, plze, ptop)
cmic$*       shared (ptopkp, qdiag, qz, radlwg, secday, tgz, tz)
cmic$*       private (adelpl, clmo, clrflx, clro, dtrad, dtradc)
cmic$*       private (flxir, i, l, nn, ozl, plk, plke)
cmic$*       private (plzestr, plzstr, sgmt4, shzl, tgstr, tsurf)
cmic$*       private (tzl, wkspce)
#endif

      DO 1000 NN=1,NPCS

C **********************************************************************
C ****                   VARIABLE INITIALIZATION                    ****
C **********************************************************************

       CALL STRIP (TGZ,TGSTR            ,IJALL,ISTRIP,1       ,NN)
       CALL STRIP (OZ,OZL               ,IJALL,ISTRIP,NLAYP1  ,NN)
       CALL STRIP (PLZ,PLZSTR           ,IJALL,ISTRIP,NLAYP1  ,NN)
       CALL STRIP (PLZE,PLZESTR         ,IJALL,ISTRIP,NLAYP1  ,NN)
       CALL STRIP (PKHT,PLKE(1,2)       ,IJALL,ISTRIP,NLAY    ,NN)
       CALL STRIP (TZ(1,1,1,N),TZL(1,2) ,IJALL,ISTRIP,NLAY    ,NN)
       CALL STRIP (QZ(1,1,1,N),SHZL(1,2),IJALL,ISTRIP,NLAY    ,NN)
       CALL STRIP (CLWRO,CLRO(1,2)      ,IJALL,ISTRIP,NLAY    ,NN)
       CALL STRIP (CLWMO,CLMO(1,2)      ,IJALL,ISTRIP,NLAY    ,NN)

C **********************************************************************
C ****                   COMPUTE P**KAPPA                           ****
C **********************************************************************
 
      DO I = 1,ISTRIP
       PLKE(I,1) = PTOPKP
       CLRO(I,1) = ZERO
       CLMO(I,1) = ZERO
      ENDDO

       CALL P2KAP ( PLKE,PLZESTR,ISTRIP,NLAY,PLK,WKSPCE )

       DO I = 1,ISTRIP*NLAYR
        ADELPL(I,1) = PLZESTR(I,2) - PLZESTR(I,1)
        ADELPL(I,1) = CONVRT / ADELPL(I,1)
       ENDDO
C
       IF(NLWCLD.NE.0)THEN
       DO I = 1,ISTRIP*(NLAYR+1)
        CLRO(I,1) = CLRO(I,1)/NLWCLD
        CLMO(I,1) = CLMO(I,1)/NLWCLD
       ENDDO
       ENDIF
C
       DO I = 1,ISTRIP*NLAYR
        TZL(I,2) =  TZL(I,2) * PLK(I,1)
       ENDDO
       DO I = 1,ISTRIP
         TZL(I,1) =  TZL(I,2)
        SHZL(I,1) = SHZL(I,2)
         TSURF(I) = PLKE(I,NLAY+1) * TZL(I,NLAY+1)
         TSURF(I) = TSURF(I) / PLK(I,NLAY)
       ENDDO
 
C **********************************************************************
C ****                CALL HARSHVADARHN RADIATION                   ****
C **********************************************************************
 
       CALL RADLW ( ISTRIP,NLAYP1,ISTRIP,TZL,SHZL,OZL,PLZESTR,CLRO,
     .         CLMO,TSURF,TGSTR,OZONFL,FLXIR,CLRFLX,SGMT4,WKSPCE,co2 )
 
C **********************************************************************
C ****                   HEATING RATES                              ****
C **********************************************************************
 
       DO I = 1,ISTRIP*NLAYR
        DTRAD (I,1) = FLXIR (I,2) - FLXIR (I,3)
        DTRAD (I,1) = DTRAD (I,1) * ADELPL(I,1)
        DTRADC(I,1) = CLRFLX(I,2) - CLRFLX(I,3)
        DTRADC(I,1) = DTRADC(I,1) * ADELPL(I,1)
       ENDDO

C **********************************************************************
C ****              NET UPWARD LONGWAVE FLUX  (W/M**2)              ****
C **********************************************************************

       DO I = 1,ISTRIP
        FLXIR (I,2    )  = - FLXIR(I,2     )
        CLRFLX(I,2    )  = -CLRFLX(I,2     )
        FLXIR (I,NLAY+2) = - FLXIR(I,NLAY+2)
        CLRFLX(I,NLAY+2) = -CLRFLX(I,NLAY+2)
       ENDDO

       CALL PASTE ( FLXIR(1,NLAY+2),RADLWG,ISTRIP,IJALL,1,NN )

C **********************************************************************
C ****            PASTE AND BUMP SOME DIAGNOSTICS                   ****
C **********************************************************************

      IF(IST4.GT.0)    CALL PSTBMP (SGMT4,QDIAG(1,1,IST4),
     .                              ISTRIP,IMJM,1,NN)
      IF(IOLR.GT.0)    CALL PSTBMP (FLXIR(1,2),QDIAG(1,1,IOLR),
     .                              ISTRIP,IMJM,1,NN)
      IF(IOLRCLR.GT.0) CALL PSTBMP (CLRFLX(1,2),QDIAG(1,1,IOLRCLR),
     .                              ISTRIP,IMJM,1,NN)
      IF(ILWGCLR.GT.0) CALL PSTBMP (CLRFLX(1,NLAY+2),QDIAG(1,1,ILWGCLR),
     .                              ISTRIP,IMJM,1,NN)

      IF(ILWCLR.GT.0)  THEN
      DO I = 1,ISTRIP*NLAYR
       WKSPCE(I) = DTRADC(I,1) * SECDAY
      ENDDO
      CALL PSTBMP (WKSPCE,QDIAG(1,1,ILWCLR),ISTRIP,IMJM,NLAY,NN)
      ENDIF

      IF(IRADLW.GT.0)  THEN
      DO I = 1,ISTRIP*NLAYR
       WKSPCE(I) = DTRAD(I,1) * SECDAY
      ENDDO
      CALL PSTBMP (WKSPCE,QDIAG(1,1,IRADLW),ISTRIP,IMJM,NLAY,NN)
      ENDIF

      IF(IOZLW.GT.0)   CALL PSTBMP (OZL(1,2),QDIAG(1,1,IOZLW),
     .                              ISTRIP,IMJM,NLAY,NN)
      IF(ICLMOLW.GT.0) CALL PSTBMP (CLMO(1,2),QDIAG(1,1,ICLMOLW),
     .                              ISTRIP,IMJM,NLAY,NN)
      IF(ICLROLW.GT.0) CALL PSTBMP (CLRO(1,2),QDIAG(1,1,ICLROLW),
     .                              ISTRIP,IMJM,NLAY,NN)

C **********************************************************************
C ****                 TENDENCY UPDATES                             ****
C **********************************************************************

      DO L = 1,NLAY
       DO I = 1,ISTRIP
       DTRAD (I,L) = ( PLZESTR(I,NLAYP1)-PTOP ) * DTRAD (I,L) / PLK(I,L)
       DTRADC(I,L) = ( PLZESTR(I,NLAYP1)-PTOP ) * DTRADC(I,L) / PLK(I,L)
       ENDDO
      ENDDO
      CALL PASTE ( DTRAD ,DTRADLW ,ISTRIP,IJALL,NLAY,NN )
      CALL PASTE ( DTRADC,DTRADLWC,ISTRIP,IJALL,NLAY,NN )

 1000 CONTINUE

C **********************************************************************
C ****                     BUMP DIAGNOSTICS                         ****
C **********************************************************************

      IF(QBUDGET) THEN
      TMSTP = MODHMS (NDLWR,0)
      DO L=1,NLAY
       DO I=1,IJALL
        ZTEMP(I,1) = DTRADLW(I,1,L) - DTRADLWC(I,1,L)
       ENDDO
       CALL ZMEAN ( DTRADLW(1,1,L),ZLWRAD(1,L),IM,JNP,TMSTP)
       CALL ZMEAN ( ZTEMP         ,ZLWCLD(1,L),IM,JNP,TMSTP)
      ENDDO
      ENDIF

C **********************************************************************
C ****                 ZERO-OUT LONGWAVE CLOUDS                     ****
C **********************************************************************

      DO L = 1,NLAY
       DO I = 1,IMR*(JMR+1)
        CLWRO(I,1,L) = ZERO
        CLWMO(I,1,L) = ZERO
       ENDDO
      ENDDO
      NLWCLD = 0

C **********************************************************************
C ****                     BUMP DIAGNOSTICS                         ****
C **********************************************************************

      IF(IRADLWG.NE.0)THEN
      DO I = 1,IMR*(JMR+1)
       QDIAG(I,1,IRADLWG) = QDIAG (I,1,IRADLWG) + RADLWG(I,1)
      ENDDO
      ENDIF

      IF(ITGRLW.NE.0) THEN
      DO I = 1,IMR*(JMR+1)
       QDIAG(I,1,ITGRLW) = QDIAG (I,1,ITGRLW) + TGZ(I,1)
      ENDDO
      ENDIF

      IF (ITLW.NE.0) THEN
      DO L=1,NLAY
       DO I = 1,IMR*(JMR+1)
        QDIAG(I,1,ITLW+L-1) = QDIAG(I,1,ITLW+L-1) + TZ(I,1,L,N)
       ENDDO
      ENDDO
      ENDIF

      IF (ISHLW.NE.0) THEN
       DO L=1,NLAY
       DO I = 1,IMR*(JMR+1)
        QDIAG(I,1,ISHLW+L-1) = QDIAG(I,1,ISHLW+L-1) + TZ(I,1,L,N)
       ENDDO
       ENDDO
       ENDIF

C **********************************************************************
C ****              INCREMENT DIAGNOSTICS COUNTERS                  ****
C **********************************************************************

      NRADLWG  = NRADLWG  + 1
      NRADLW   = NRADLW   + 1
      NCLMOLW  = NCLMOLW  + 1
      NCLROLW  = NCLROLW  + 1
      NTLW     = NTLW     + 1
      NSHLW    = NSHLW    + 1
      NOZLW    = NOZLW    + 1
      NTGRLW   = NTGRLW   + 1
      NST4     = NST4     + 1
      NOLR     = NOLR     + 1
      NOLRCLR  = NOLRCLR  + 1
      NLWGCLR  = NLWGCLR  + 1
      NLWCLR   = NLWCLR   + 1
C
      RETURN
      END
      SUBROUTINE RADLW ( M, LL, IDIM, TLO, QLO, O3O, PLE, CLR, CLL,
     .                   SAT, SFT, OZNE, FLX, FLC, ST4, WORK ,co2)
C
C  INPUTS:
C    M ..... RUN
C    LL .... RISE
C    IDIM .. INNER DIMENSION
C            TLO, QLO, O3O, PLE, CLR, CLL
C            SAT,SFT,OZNE
C  OUTPUTS
C             FLX, FLC, ST4
C  SCRATCH
C            WORK
C **********************************************************************
C    THIS COVER FOR LONGIN FACILITATES THE PASSING OF SCRATCH SPACE.
C **********************************************************************
C
      DIMENSION TLO(IDIM,LL), QLO(IDIM,LL), O3O(IDIM,LL), SAT(M)
      DIMENSION PLE(IDIM,LL), CLR(IDIM,LL), CLL(IDIM,LL), SFT(M)
      LOGICAL   OZNE

      DIMENSION FLX(IDIM,LL+1), FLC(IDIM,LL+1), ST4(M)

      DIMENSION  WORK(M,LL+2,25)

      CALL LONGIN(M,LL,IDIM
     *,      TLO,QLO,O3O,PLE,CLR,CLL
     *,      SAT,SFT,OZNE
     *,      FLX,FLC,ST4
     *,      WORK(1,1, 1), WORK(1,1, 2), WORK(1,1, 3), WORK(1,1, 4)
     *,      WORK(1,1, 5), WORK(1,1, 6), WORK(1,1, 7), WORK(1,1, 8)
     *,      WORK(1,1, 9), WORK(1,1,10), WORK(1,1,11), WORK(1,1,12)
     *,      WORK(1,1,13), WORK(1,1,14), WORK(1,1,15), WORK(1,1,16)
     *,      WORK(1,1,17), WORK(1,1,18), WORK(1,1,19), WORK(1,1,20)
     *,      WORK(1,1,21), WORK(1,1,22), WORK(1,1,23), WORK(1,1,24)
     *,      WORK(1,1,25)  ,co2)

      RETURN
      END

      SUBROUTINE LONGIN (  M, LL,IDIM
     *,      TLO,QLO,O3O,PLE,CLR,CLL
     *,      SAT,SFT,OZNE
     *,      FLX,FLC,ST4
     *,      TLE,TV1,TV2,TV3,TV4,TY1,TY2,TY3,TY4,TY5
     *,      TY6, X1, X2, X3, X4,UBR,VBR,WBR,PCV,PCW
     *,      WVO,OUR,OVR,CLU,CLO,co2                 )

C*********************************************************************
C******************* SUBROUTINE   LONGWAVE ***************************
C********************** 13  MARCH  1989 ******************************
C*********************************************************************

C  INPUTS--THESE ARE NOT MODIFIED BY THE PACKAGE.

C    TLO(IDIM,LL)   :  LAYER TEMPERATURES (K)
C    QLO(IDIM,LL)   :  LAYER SPECIFIC HUMIDITIES
C    O3O(IDIM,LL)   :  LAYER OZONE MIXING RATIOS. IGNORED IF OZNE=FALSE.
C    OZNE           :  LOGICAL FLAG.
C    PLE(IDIM,LL)   :  PRESSURE AT THE BASE OF EACH LAYER. (MB)
C                         THE CODE ASSUMES THE TOP PRESSURE IS ZERO.
C                         PLE(*,LL) IS THE SURFACE PRESSURE.
C    CLR(IDIM,LL)   :  RANDOM OVERLAP CLOUD FRACTION AT EACH LAYER.
C    CLL(IDIM,LL)   :  MAXIMUM OVERLAP CLOUD FRACTION AT EACH LAYER.
C    SAT(M)         :  SURFACE-AIR TEMPERATURE (K).
C    SFT(M)         :  SURFACE TEMPERATURE (K).

C    M              :  NUMBER OF SOUNDINGS DONE ON EACH CALL. (RUN)
C    LL             :  NUMBER OF LAYERS IN EACH SOUNDING.     (RISE)
C    IDIM           :  INNER DIMENSION OF ARGUMENTS.          (>=M)


C SCRATCH SPACE--

C   TLE - CLO    : 25 WORK ARRAYS DIMENSIONED (M,LL+1) OR (M,LL+2)


C  OUTPUT--THESE ARE FULLY DEFINED BY LONGWAVE.


C    FLX(IDIM,LL+1) :  NET DOWNWARD FLUX. CLOUDY. (1=TOP)  (W/M**2)
C    FLC(IDIM,LL+1) :  NET DOWNWARD FLUX. CLEAR.  (1=TOP)  (W/M**2)
C    ST4(M)         :  UPWARD FLUX AT THE SURFACE          (W/M**2)

C
      PARAMETER (ZERO=0., ONE=1., HALF=.5)

      PARAMETER (CIRTHK=.16             )
      PARAMETER (CNV=10.0/9.81,    CMB2PA=100.            )
C     PARAMETER (CNV=1.02,         CMB2PA=100.            )
      PARAMETER (QMIN=1.E-7,       O3MIN=1.E-10           )
      PARAMETER (TREF=250.,        TREFCO2=240.           )
      PARAMETER (TRCNT=225.,       RCNT=.005,  PRCNT=275. )
      PARAMETER (TRWNG=256.,       RWNG=.016,  PRWNG=550. )
      PARAMETER (D1=1800.,         D2=6.08,     D3=642.   )
      PARAMETER (Z2P3EM4 = 2.32143E-4                     )
      PARAMETER (R1=.0089,         R2=.025                )
      PARAMETER (P1=ONE/30.,       P2=ONE/300., PC=2.0    )
      PARAMETER (PCH2C=ONE,        PCH2W=15.              )
      PARAMETER (CLBET=.75,        ATM=1013.25            )

C  INPUTS

      DIMENSION TLO(IDIM,LL), QLO(IDIM,LL), O3O(IDIM,LL), SAT(M)
      DIMENSION PLE(IDIM,LL), CLR(IDIM,LL), CLL(IDIM,LL), SFT(M)
      LOGICAL   OZNE

C  OUTPUTS

      DIMENSION FLX(IDIM,LL+1), FLC(IDIM,LL+1), ST4(M)

C  SCARTCH SIZE = (25*(LL+1)+2)*M

      DIMENSION
     *  TLE(M,LL+2), TV1(M,LL+2), TV2(M,LL+2), TV3(M,LL+2), TV4(M,LL+2)
     *, TY1(M,LL+1), TY2(M,LL+1), TY3(M,LL+1), TY4(M,LL+1), TY5(M,LL+1)
     *, TY6(M,LL+1),  X1(M,LL+1),  X2(M,LL+1),  X3(M,LL+1),  X4(M,LL+1)
     *, UBR(M,LL+1), VBR(M,LL+1), WBR(M,LL+1), PCV(M,LL+1), PCW(M,LL+1)
     *, WVO(M,LL+1), OUR(M,LL+1), OVR(M,LL+1), CLU(M,LL  ), CLO(M,LL  )


      DIMENSION B1(5),B2(5),B3(5),B4(5)

      DATA B1 /4.6255E1, 5.7048E-1, 4.2093E-3, 2.4271E-5, 5.708E-8/
      DATA B2 /9.9988E1, 1.6930E-0, 1.0623E-2, 2.9373E-5, 3.046E-8/
      DATA B3 /1.2791E1, 3.0606E-1, 2.4403E-3, 4.9731E-6,-1.475E-8/
      DATA B4 /6.2523E1, 9.7524E-1, 3.9791E-3,-2.4060E-6,-1.996E-8/

c     SAVE

      FCO2 = .26*CO2/330.

      D4 = (Z2P3EM4 * TREF**HALF) *HALF

      MLLM0 = M *  LL
      MLLM1 = M * (LL-1)
      MLLP1 = M * (LL+1)
      MLLP2 = M * (LL+2)

C********************************************************************
C***** PRELIMINARIES  ***********************************************
C********************************************************************

      DO 12 L=1,LL
CB
      DO 9000 I =1,M
      TY5(I,L) = QLO(I,L)
      TY6(I,L) = O3O(I,L)
       X1(I,L) = TLO(I,L)
9000  CONTINUE
CE
12    CONTINUE

CB
      DO 9002 I =1,MLLM0
      IF(TY5(I,1).LT. QMIN) TY5(I,1) =  QMIN
      IF(TY6(I,1).LT.O3MIN) TY6(I,1) = O3MIN
9002  CONTINUE
CE

C  PLO==> X2    AND   DPO==> X3,  TEMPORARILY.

CB
      DO 9004 I =1,M
      X2(I,1)  =  PLE(I,1) * HALF
      X3(I,1)  =  PLE(I,1) * CNV
9004  CONTINUE
CE

      DO 14 L=1,LL-1
CB
      DO 9006 I =1,M
      X2(I,L+1) = (PLE(I,L+1) + PLE(I,L)) * HALF
      X3(I,L+1) = (PLE(I,L+1) - PLE(I,L)) * CNV
9006  CONTINUE
CE
C     TY1(1,L  ;M) = X2(1,L;M) / PLE(1,L;M)
14    CONTINUE

C
C
C     TEMPERATURE INTERPOLATION
C
CB
      DO 9008 I =1,MLLM1
      TY1(I,1) =  X2(I,1) - PLE(I,1)
      TLE(I,2) = TY1(I,1) * TLO(I,2)
      TY1(I,1) = PLE(I,1) -  X2(I,2)
      TY1(I,1) = TY1(I,1) * TLO(I,1)
      TLE(I,2) = TLE(I,2) + TY1(I,1)
      TY1(I,1) =  X2(I,1) -  X2(I,2)
      TLE(I,2) = TLE(I,2) / TY1(I,1)
9008  CONTINUE
CE
C
C**** TEMPERATURE INTERPOLATION  *************

      DO 9010 I =1,M
      TLE(I,   1 ) =  X1(I, 1)
      TLE(I,LL+1 ) = SAT(I)
      TLE(I,LL+2 ) = SFT(I)
9010  CONTINUE

      DO 9012 I =1,MLLP2
      TLE(I,1) = TLE(I,1 ) - TREF
9012  CONTINUE

C****** CLOUD EMISSIVITIES *******************

      CALL OPTTHK(X1,MLLM0,TY1)
      DO 9166 I = 1,MLLM0
      TY1(I,1) = TY1(I,1) * CLBET
9166  CONTINUE

      DO 16 L=1,LL
      DO 9014 I =1,M
      IF(CLL(I,L).GT.ZERO) TY1(I,L) = CIRTHK*CLBET
9014  CONTINUE
16    CONTINUE

      DO 9016 I =1,MLLM0
      TY1(I,1) =-TY1(I,1) * X3(I,1)
      TY2(I,1) = EXP(TY1(I,1))

      TY1(I,1) = ONE - TY2(I,1)
9016  CONTINUE

      DO 18 L=1,LL
      DO 9020 I =1,M
      CLU(I,L) = ONE - CLR(I,L) * TY1(I,L)
      CLO(I,L) = ONE - CLL(I,L) * TY1(I,L)
9020  CONTINUE
18    CONTINUE

C***** SCALED ABSORBER AMOUNTS *****************

C  CO2 CENTER

      DO 9022 I =1,MLLM0
      TY4(I,1) = X3(I,1) * FCO2

      TY1(I,1) = ( X1(I,1) - TREFCO2) * R1
      PCV(I,2) = EXP(TY1(I,1))
      TY1(I,1) = X2(I,1) * P1
      IF(X2(I,1).LT.PC) TY1(I,1) = (PC*P1)
9022  CONTINUE

C  TY1 ** .84375
CI
      DO 9018 I = 1,MLLM0
      TY2(I,1) = SQRT(TY1(I,1))
      TY3(I,1) = SQRT(TY2(I,1))
      TY2(I,1) = TY2(I,1) *  TY3(I,1)
      TY3(I,1) = SQRT(TY3(I,1))
      TY3(I,1) = SQRT(TY3(I,1))
      TY2(I,1) = TY2(I,1) *  TY3(I,1)
      TY3(I,1) = SQRT(TY3(I,1))
      TY1(I,1) = TY2(I,1) *  TY3(I,1)

      PCV(I,2) = TY1(I,1) * PCV(I,2) * TY4(I,1)

C  CO2 WINGS

      TY1(I,1) = ( X1(I,1) - TREFCO2) * R2
      PCW(I,2) = EXP(TY1(I,1))

      TY1(I,1) = X2(I,1) * P2
      IF(X2(I,1).LT.PC) TY1(I,1) = (PC*P2)

      TY1(I,1) = SQRT(TY1(I,1))

      PCW(I,2) = TY1(I,1) * PCW(I,2) * TY4(I,1)
9018  CONTINUE
C   H20 CENTER
      DO 9024 I = 1,MLLM0
      TY4(I,1) = TY5(I,1) * X3(I,1)

      TY1(I,1) = (  X1(I,1) - TRCNT ) * RCNT
      VBR(I,2) = EXP(TY1(I,1))

      TY1(I,1) = X2(I,1)
      IF(X2(I,1).LT.PCH2C) TY1(I,1) = PCH2C

      VBR(I,2) = VBR(I,2) * TY4(I,1)
      VBR(I,2) = VBR(I,2) * TY1(I,1) * (ONE/PRCNT)

C   H20 WINGS

      TY1(I,1) = (  X1(I,1) - TRWNG ) * RWNG
      WBR(I,2) = EXP(TY1(I,1))

      TY1(I,1) = X2(I,1)
      IF(X2(I,1).LT.PCH2W) TY1(I,1) = PCH2W

      WBR(I,2) = WBR(I,2) * TY4(I,1)
      WBR(I,2) = WBR(I,2) * TY1(I,1) * (ONE/PRWNG)

C   H20 E-TYPE

      TY1(I,1) = D1 /  X1(I,1)  - D2
      UBR(I,2) = EXP(TY1(I,1))
      UBR(I,2) = UBR(I,2) * TY4(I,1) * TY5(I,1)
      UBR(I,2) = UBR(I,2) * X2(I,1) * (ONE/D3)
9024  CONTINUE
C   OZONE

      IF (OZNE) THEN
CB
      DO 9026 I =1,MLLM0
      TY1(I,1) = D4             +  X1(I,1) * (D4/TREF)
      TY1(I,1) = TY1(I,1) + X2(I,1) * (ONE/ATM)
      OVR(I,2) = TY6(I,1) * X3(I,1)
      OUR(I,2) = OVR(I,2) * TY1(I,1)
9026  CONTINUE
CE
      ELSE
CB
      DO 9028 I =1,MLLM0
      OUR(I,2) = O3MIN
      OVR(I,2) = O3MIN
9028  CONTINUE
CE
      ENDIF


C***** INTEGRATE SCALED ABSORBER AMOUNTS FROM T.O.A. TO LAYER BASES ***

CB
      DO 9030 I =1,M
      UBR(I,1) = ZERO
      VBR(I,1) = ZERO
      WBR(I,1) = ZERO
      PCV(I,1) = ZERO
      PCW(I,1) = ZERO
      OUR(I,1) = ZERO
      OVR(I,1) = ZERO
9030  CONTINUE
CE

      DO 10 L=3,LL+1
CB
      DO 9032 I =1,M
      UBR(I,L) = UBR(I,L-1) + UBR(I,L)
      VBR(I,L) = VBR(I,L-1) + VBR(I,L)
      WBR(I,L) = WBR(I,L-1) + WBR(I,L)
      PCV(I,L) = PCV(I,L-1) + PCV(I,L)
      PCW(I,L) = PCW(I,L-1) + PCW(I,L)
      OUR(I,L) = OUR(I,L-1) + OUR(I,L)
      OVR(I,L) = OVR(I,L-1) + OVR(I,L)
9032  CONTINUE
CE
10    CONTINUE

C***** PLANCK FUNCTIONS ********************************************

      CALL POLY(TV1,TLE,B1,5,MLLP2)
      CALL POLY(TV2,TLE,B2,5,MLLP2)
      CALL POLY(TV3,TLE,B3,5,MLLP2)
      CALL POLY(TV4,TLE,B4,5,MLLP2)


C********************************************************************
C***** START TRANSFER CALCULATION  **********************************
C********************************************************************

      DO 170 L=1,LL+1
CB
      DO 9034 I =1,M
      FLX(I,L) = ZERO
      FLC(I,L) = ZERO
9034  CONTINUE
CE
170   CONTINUE

C****** UPPER BOUNDARY ***************

C   H2O CENTER

      DO 9036 I =1,MLLM0
      X2(I,1) = VBR(I,2)
9036  CONTINUE

      CALL GFUN2(X2,WVO, X1, X3,TY1,TY3,MLLM0)

      DO 22 L=1,LL
CB
      DO 9038 I =1,M
      TY2(I,L) = TLE(I,1)
      TY3(I,L) = TV1(I,1)
9038  CONTINUE
CE
22    CONTINUE

CB
      DO 9040 I =1,MLLM0
      TY1(I,1) =  X3(I,1) * TY2(I,1) +  X1(I,1)
      TY1(I,1) = TY1(I,1) * TY2(I,1) + ONE
      TY5(I,1) = TY1(I,1) * TY3(I,1) * WVO(I,1)
9040  CONTINUE
CE

C    H2O WINGS
CB
      DO 9042 I =1,MLLM0
      X1(I,1) = UBR(I,2)
      X3(I,1) = WBR(I,2)
9042  CONTINUE
CE
      CALL GFUN1(X1,X3,WVO, X2, X4,TY1,TY2,TY3,MLLM0)

      DO 24 L=1,LL
CB
      DO 9044 I =1,M
      TY2(I,L) = TLE(I,1)
      TY3(I,L) = TV2(I,1)
9044  CONTINUE
CE
24    CONTINUE
CB
      DO 9046 I =1,MLLM0
      TY6(I,1) =  X4(I,1) * TY2(I,1) +  X2(I,1)
      TY6(I,1) = TY6(I,1) * TY2(I,1) + ONE
      TY6(I,1) = TY6(I,1) * TY3(I,1) * WVO(I,1)

      TY5(I,1) = TY5(I,1) + TY6(I,1)

C   9.6 MICRON

      X2(I,1) = OUR(I,2)
      X4(I,1) = OVR(I,2)
9046  CONTINUE
CE
      CALL TRO3(X1,X2,X4,WVO,TY1,OZNE,MLLM0)

      DO 30 L=1,LL
      DO 9048 I =1,M
      TY2(I,L) = TV3(I,1) * WVO(I,L)
9048  CONTINUE
30    CONTINUE

C   15 MICRON
CB
      DO 9050 I =1,MLLM0
      X2(I,1) = PCV(I,2)
      X4(I,1) = PCW(I,2)
9050  CONTINUE
CE
      CALL TRCO2(X2,X4,X3,X1,WVO,TY1,MLLM0)

      DO 32 L=1,LL
      DO 9052 I =1,M
      TY2(I,L) = TY2(I,L) + TV4(I,1) * WVO(I,L)
9052  CONTINUE
32    CONTINUE

C  GATHER ALL FOUR CONTRIBUTIONS

      DO 9054 I =1,MLLM0
      TY2(I,1) = TY2(I,1) + TY5(I,1)
9054  CONTINUE

C  PROBABILITY OF A CLEAR LINE OF SIGHT (TY5)

CB
      DO 9056 I =1,M
      TY5(I,1) = ONE
      TY1(I,1) = ONE
9056  CONTINUE
CE
      DO 20 L=1,LL
CB
      DO 9058 I =1,M
      TY5(I,L+1) = TY5(I,L) * CLU(I,L)
      TY1(I,L+1) = TY1(I,L)
      IF(TY1(I,L+1) .GT. CLO(I,L) ) TY1(I,L+1) = CLO(I,L)
9058  CONTINUE
CE
20    CONTINUE
      DO 9060 I =1,MLLM0
      TY5(I,2) = TY5(I,2) * TY1(I,2)
9060  CONTINUE

C   ADD TO FLUXES

      DO 172 L=1,LL
CB
      DO 9062 I =1,M
      FLC(I,L+1) = FLC(I,L+1) - TY2(I,L  )
      TY2(I,L  ) = TY2(I,L  ) * TY5(I,L+1)
      FLX(I,L+1) = FLX(I,L+1) - TY2(I,L  )
9062  CONTINUE
CE
172   CONTINUE

C****** SURFACE *******************

C  H20 CENTER

      DO 42 L=1,LL
      DO 9064 I =1,M
      X2(I,L) = VBR(I,LL+1) - VBR(I,L)
9064  CONTINUE
42    CONTINUE

      CALL GFUN2(X2,WVO, X1, X3,TY1,TY5,MLLM0)

C  SKIN TEMPTERATURE CONTRIBUTION

      DO 52 L=1,LL
CB
      DO 9066 I =1,M
      TY2(I,L) = TLE(I,LL+2)
      TY3(I,L) = TV1(I,LL+2)
9066  CONTINUE
CE
52    CONTINUE
CB
      DO 9068 I =1,MLLM0
      TY1(I,1) =  X3(I,1) * TY2(I,1) +  X1(I,1)
      TY1(I,1) = TY1(I,1) * TY2(I,1) + ONE
      TY1(I,1) = TY1(I,1) * TY3(I,1) * WVO(I,1)

      TY6(I,1) = - TY1(I,1)
9068  CONTINUE
CE
C  SURFACE AIR TEMPTERATURE CONTRIBUTION

      DO 60 L=1,LL
CB
      DO 9070 I =1,M
      TY4(I,L) = TLE(I,LL+1)
      TY3(I,L) = TV1(I,LL+1)
9070  CONTINUE
CE
60    CONTINUE
CB
      DO 9072 I =1,MLLM0
      TY1(I,1) =  X3(I,1) * TY4(I,1) +  X1(I,1)
      TY1(I,1) = TY1(I,1) * TY4(I,1) + ONE
      TY1(I,1) = TY1(I,1) * TY3(I,1) * WVO(I,1)

      TY6(I,1) = TY6(I,1) + TY1(I,1)
9072  CONTINUE
CE
C  H2O WINGS

      DO 40 L=1,LL
CB
      DO 9074 I =1,M
      X1(I,L) = UBR(I,LL+1) - UBR(I,L)
      X3(I,L) = WBR(I,LL+1) - WBR(I,L)
9074  CONTINUE
CE
40    CONTINUE

      CALL GFUN1(X1,X3,WVO, X2, X4,TY1,TY5,TY2,MLLM0)

C  SKIN TEMPTERATURE CONTRIBUTION

      DO 54 L=1,LL
CB
      DO 9076 I =1,M
      TY2(I,L) = TLE(I,LL+2)
      TY3(I,L) = TV2(I,LL+2)
9076  CONTINUE
CE
54    CONTINUE
CB
      DO 9078 I =1,MLLM0
      TY1(I,1) =  X4(I,1) * TY2(I,1) +  X2(I,1)
      TY1(I,1) = TY1(I,1) * TY2(I,1) + ONE
      TY1(I,1) = TY1(I,1) * TY3(I,1) * WVO(I,1)

      TY6(I,1) = TY6(I,1) - TY1(I,1)
9078  CONTINUE
CE
C  SURFACE AIR TEMPTERATURE CONTRIBUTION

      DO 62 L=1,LL
CB
      DO 9080 I =1,M
      TY4(I,L) = TLE(I,LL+1)
      TY3(I,L) = TV2(I,LL+1)
9080  CONTINUE
CE
62    CONTINUE
CB
      DO 9082 I =1,MLLM0
      TY1(I,1) =  X4(I,1) * TY4(I,1) +  X2(I,1)
      TY1(I,1) = TY1(I,1) * TY4(I,1) + ONE
      TY1(I,1) = TY1(I,1) * TY3(I,1) * WVO(I,1)

      TY6(I,1) = TY6(I,1) + TY1(I,1)
9082  CONTINUE
CE
C  9.6 MICRON

      DO 44 L=1,LL
CB
      DO 9084 I =1,M
      X2(I,L) = OUR(I,LL+1) - OUR(I,L)
      X4(I,L) = OVR(I,LL+1) - OVR(I,L)
9084  CONTINUE
CE
44    CONTINUE

      CALL TRO3(X1,X2,X4,WVO,TY1,OZNE,MLLM0)

      DO 9086 I =1,M
      TY3(I,LL+1) = TV3(I,LL+2) - TV3(I,LL+1)
9086  CONTINUE

      DO 70 L=1,LL
      DO 9088 I =1,M
      TY2(I,L) = TY3(I,LL+1) * WVO(I,L)
9088  CONTINUE
70    CONTINUE

C   15 MICRON

      DO 46 L=1,LL
CB
      DO 9090 I =1,M
      X2(I,L) = PCV(I,LL+1) - PCV(I,L)
      X4(I,L) = PCW(I,LL+1) - PCW(I,L)
9090  CONTINUE
CE
46    CONTINUE

      CALL TRCO2(X2,X4,X3,X1,WVO,TY1,MLLM0)

      DO 9092 I =1,M
      TY2(I,LL+1) = TV4(I,LL+2) - TV4(I,LL+1)
9092  CONTINUE

      DO 72 L=1,LL
      DO 9094 I =1,M
      TY2(I,L) = TY2(I,L) + TY2(I,LL+1) * WVO(I,L)
9094  CONTINUE
72    CONTINUE

C  GATHER ALL FOUR CONTRIBUTIONS

      DO 9096 I =1,MLLM0
      TY6(I,1) = TY6(I,1) - TY2(I,1)
9096  CONTINUE

C  PROBABILITY OF A CLEAR LINE OF SIGHT (TY2)

CB
      DO 9098 I =1,M
      TY2(I,LL+1) = ONE
      TY1(I,LL+1) = ONE
9098  CONTINUE
CE
      DO 50 L=1,LL
CB
      DO 9100 I =1,M
      TY2(I,LL+1-L) = TY2(I,LL+2-L) * CLU(I,LL+1-L)
      TY1(I,LL+1-L) = TY1(I,LL+2-L)
      IF(TY1(I,LL+1-L) .GT. CLO(I,LL+1-L) ) THEN
      TY1(I,LL+1-L) = CLO(I,LL+1-L)
      ENDIF
9100  CONTINUE
CE
50    CONTINUE
      DO 9104 I =1,MLLM0
      TY2(I,1) = TY2(I,1) * TY1(I,1)
9104  CONTINUE

C   ADD TO FLUXES

      DO 174 L=1,LL
CB
      DO 9106 I =1,M
      FLC(I,L) = FLC(I,L) + TY6(I,L)
      TY6(I,L) = TY6(I,L) * TY2(I,L)
      FLX(I,L) = FLX(I,L) + TY6(I,L)
9106  CONTINUE
CE
174   CONTINUE

C**** EXCHANGE INTEGRAL **********

      DO 130 L=1,LL


C  H20 CENTER

      DO 9108 I =1,M
      TY1(I,2) = HALF * (VBR(I,L) + VBR(I,L+1))
9108  CONTINUE

      DO 102 K=1,L
      DO 9110 I =1,M
      X2(I,K) = TY1(I,2) - VBR(I,K)
9110  CONTINUE
102   CONTINUE
      DO 112 K=L+1,LL+1
      DO 9112 I =1,M
      X2(I,K) = VBR(I,K) - TY1(I,2)
9112  CONTINUE
112   CONTINUE

      DO 118 K=1,LL+1
CB
      DO 9114 I =1,M
      TY2(I,K) =  TLE(I,L  )
      TY3(I,K) =  TLE(I,L+1)
9114  CONTINUE
CE
118   CONTINUE

      CALL GFUN2(X2,WVO, X1, X3,TY4,TY5,MLLP1)

      DO 120 K=1,LL+1
CB
      DO 9116 I =1,M
      TY6(I,K) =  TV1(I,L  )
      TY5(I,K) =  TV1(I,L+1)
9116  CONTINUE
CE
120   CONTINUE
CB
      DO 9118 I =1,MLLP1
      TY1(I,1) =  X3(I,1) * TY2(I,1) +  X1(I,1)
      TY1(I,1) = TY1(I,1) * TY2(I,1) + ONE
      TY4(I,1) = TY1(I,1) * WVO(I,1) * TY6(I,1)

      TY1(I,1) =  X3(I,1) * TY3(I,1) +  X1(I,1)
      TY1(I,1) = TY1(I,1) * TY3(I,1) + ONE
      TY1(I,1) = TY1(I,1) * WVO(I,1) * TY5(I,1)

      TY4(I,1) = TY4(I,1) - TY1(I,1)
9118  CONTINUE
CE
C  H2O WINGS
CB
      DO 9120 I =1,M
      TY1(I,1) = HALF * (UBR(I,L) + UBR(I,L+1))
      TY1(I,3) = HALF * (WBR(I,L) + WBR(I,L+1))
9120  CONTINUE
CE
      DO 100 K=1,L
CB
      DO 9122 I =1,M
      X1(I,K) = TY1(I,1) - UBR(I,K)
      X3(I,K) = TY1(I,3) - WBR(I,K)
9122  CONTINUE
CE
100   CONTINUE
      DO 110 K=L+1,LL+1
CB
      DO 9124 I =1,M
      X1(I,K) = UBR(I,K) - TY1(I,1)
      X3(I,K) = WBR(I,K) - TY1(I,3)
9124  CONTINUE
CE
110   CONTINUE

      CALL GFUN1(X1,X3,WVO, X2, X4,TY1,TY5,TY6,MLLP1)

      DO 122 K=1,LL+1
CB
      DO 9126 I =1,M
      TY6(I,K) =  TV2(I,L  )
      TY5(I,K) =  TV2(I,L+1)
9126  CONTINUE
CE
122   CONTINUE
CB
      DO 9128 I =1,MLLP1
      TY1(I,1) =  X4(I,1) * TY2(I,1) +  X2(I,1)
      TY1(I,1) = TY1(I,1) * TY2(I,1) + ONE
      TY1(I,1) = TY1(I,1) * WVO(I,1) * TY6(I,1)
      TY4(I,1) = TY4(I,1) + TY1(I,1)

      TY1(I,1) =  X4(I,1) * TY3(I,1) +  X2(I,1)
      TY1(I,1) = TY1(I,1) * TY3(I,1) + ONE
      TY1(I,1) = TY1(I,1) * WVO(I,1) * TY5(I,1)
      TY4(I,1) = TY4(I,1) - TY1(I,1)
9128  CONTINUE
CE
C  9.6 MICRON
CB
      DO 9130 I =1,M
      TY2(I,3) = HALF * (OUR(I,L) + OUR(I,L+1))
      TY3(I,1) = HALF * (OVR(I,L) + OVR(I,L+1))
9130  CONTINUE
CE
      DO 104 K=1,L
CB
      DO 9132 I =1,M
      X2(I,K) = TY2(I,3) - OUR(I,K)
      X4(I,K) = TY3(I,1) - OVR(I,K)
9132  CONTINUE
CE
104   CONTINUE
      DO 114 K=L+1,LL+1
CB
      DO 9134 I =1,M
      X2(I,K) = OUR(I,K) - TY2(I,3)
      X4(I,K) = OVR(I,K) - TY3(I,1)
9134  CONTINUE
CE
114   CONTINUE

      CALL TRO3(X1,X2,X4,WVO,TY1,OZNE,MLLP1)

      DO 124 K=1,LL+1
      DO 9136 I =1,M
      TY3(I,K) =  TV3(I,L) - TV3(I,L+1)
9136  CONTINUE
124   CONTINUE
CB
      DO 9138 I =1,MLLP1
      TY3(I,1) = TY3(I,1) * WVO(I,1)
      TY4(I,1) = TY4(I,1) + TY3(I,1)
9138  CONTINUE
CE
C  15. MICRON
CB
      DO 9140 I =1,M
      TY2(I,1) = HALF * (PCV(I,L) + PCV(I,L+1))
      TY2(I,2) = HALF * (PCW(I,L) + PCW(I,L+1))
9140  CONTINUE
CE
      DO 106 K=1,L
CB
      DO 9142 I =1,M
      X2(I,K) = TY2(I,1) - PCV(I,K)
      X4(I,K) = TY2(I,2) - PCW(I,K)
9142  CONTINUE
CE
106   CONTINUE
      DO 116 K=L+1,LL+1
CB
      DO 9144 I =1,M
      X2(I,K) = PCV(I,K) - TY2(I,1)
      X4(I,K) = PCW(I,K) - TY2(I,2)
9144  CONTINUE
CE
116   CONTINUE

      CALL TRCO2(X2,X4,X3,X1,WVO,TY1,MLLP1)

      DO 125 K=1,LL+1
      DO 9146 I =1,M
      TY2(I,K) =  TV4(I,L) - TV4(I,L+1)
9146  CONTINUE
125   CONTINUE

CB
      DO 9148 I =1,MLLP1
      TY2(I,1) = TY2(I,1) * WVO(I,1)
      TY4(I,1) = TY4(I,1) + TY2(I,1)
9148  CONTINUE
CE

C  PROBABILITY OF A CLEAR LINE OF SIGHT (TY5)

CB
      DO 9150 I =1,M
      TY5(I,L) = CLU(I,L)
      TY1(I,L) = CLO(I,L)
9150  CONTINUE
CE
      IF (L .GT. 1) THEN
      DO 80 K=L,2,-1
CB
      DO 9152 I =1,M
      TY5(I,K-1) = TY5(I,K) * CLU(I,K-1)
      TY1(I,K-1) = TY1(I,K)
      IF(TY1(I,K-1) .GT. CLO(I,K-1)) TY1(I,K-1) = CLO(I,K-1)
9152  CONTINUE
CE
80    CONTINUE
      ENDIF

CB
      DO 9154 I =1,M
      TY5(I,L+1) = CLU(I,L)
      TY1(I,L+1) = CLO(I,L)
9154  CONTINUE
CE
      IF (L .LT. LL) THEN
      DO 90 K=L,LL-1
CB
      DO 9156 I =1,M
      TY5(I,K+2) = TY5(I,K+1) * CLU(I,K+1)
      TY1(I,K+2) = TY1(I,K+1)
      IF(TY1(I,K+2) .GT. CLO(I,K+1)) TY1(I,K+2) = CLO(I,K+1)
9156  CONTINUE
CE
90    CONTINUE
      ENDIF

      DO 9158 I =1,MLLP1
      TY5(I,1) = TY5(I,1) * TY1(I,1)
9158  CONTINUE

C   ADD TO FLUXES

      DO 178 K=1,LL+1
CB
      DO 9160 I =1,M
      FLC(I,K) = FLC(I,K) + TY4(I,K)
      TY4(I,K) = TY4(I,K) * TY5(I,K)
      FLX(I,K) = FLX(I,K) + TY4(I,K)
9160  CONTINUE
CE
178   CONTINUE

130   CONTINUE
CB
      DO 9162 I =1,M
      TY1(I,1   ) = TV1(I,1   ) + TV2(I,1   )
      TY1(I,1   ) = TY1(I,1   ) + TV3(I,1   )
      TY1(I,1   ) = TY1(I,1   ) + TV4(I,1   )

      FLX(I,1   ) = FLX(I,1   ) - TY1(I,1   )
      FLC(I,1   ) = FLC(I,1   ) - TY1(I,1   )
9162  CONTINUE
CE
CB
      DO 9164 I =1,M
      TV1(I,LL+1) = TV1(I,LL+1) - TV1(I,LL+2)
      TV2(I,LL+1) = TV2(I,LL+1) - TV2(I,LL+2)
      TV3(I,LL+1) = TV3(I,LL+1) - TV3(I,LL+2)
      TV4(I,LL+1) = TV4(I,LL+1) - TV4(I,LL+2)

      ST4(I) = TV1(I,LL+1) + TV2(I,LL+1)
      ST4(I) = ST4(I) + TV3(I,LL+1)
      ST4(I) = ST4(I) + TV4(I,LL+1)

      FLX(I,LL+1) = FLX(I,LL+1) + ST4(I)
      FLC(I,LL+1) = FLC(I,LL+1) + ST4(I)

      ST4(I) = TV1(I,LL+2) + TV2(I,LL+2)
      ST4(I) = ST4(I) + TV3(I,LL+2)
      ST4(I) = ST4(I) + TV4(I,LL+2)
9164  CONTINUE
CE

      RETURN
      END
      SUBROUTINE OPTTHK(TLO,LEN,TAU)
C
      PARAMETER (ZERO=0.0,      TICE=273.16)
      PARAMETER (TC1=TICE-82.5, TC2=TICE-10.0, TC3=TICE)
      PARAMETER (CFC=2.E-6,     BFC=8.E-2)
      PARAMETER (EMX=CFC*(TC1-TC2)*(TC1-TC2))
      PARAMETER (CC1=EMX/(TC2-TC3), CC2=BFC/(TC2-TC3))
C
      DIMENSION TLO(LEN), TAU(LEN)
C
      DO 10 L = 1,LEN
       TAU(L) = TLO(L) - TC1
       IF(TAU(L) .LT. ZERO) TAU(L) = ZERO
       TAU(L) = TAU(L)*TAU(L)*CFC
       IF(TAU(L) .GT. EMX)
     1      TAU(L) = ((TLO(L)-TC3)*CC1) + (TC2-TLO(L))*CC2
       IF(TAU(L) .GT. BFC) TAU(L) = BFC
  10  CONTINUE
C
      RETURN
      END
      SUBROUTINE GFUN1(X1,X3,WW,YW,ZW,TX,TY,TZ,MLEN)

      PARAMETER (ZERO=0.,ONE=1.)
      PARAMETER (X1MAX=.06,X3MAX=8.)
C
      PARAMETER
     *  (  B1=32.2095,    B2=52.85,   B3=-1990.63, B4=199., B5=333.244
     *,    B6=.534874,    B7=74.144,  B8=24.7442,  B9=.43368

     *,    C1=-4.92303   , C2= .134927   , C3= 257.949, C4= ZERO
     *,    C5= .315851   , C6= .440720   , C7= 40.2298, C8= 7.79555E-3

     *,    D1= .798951   , D2=-.363684   , D3= 62.5692, D4=-3.81305E-3
     *,    D5=-.285241   , D6= .071006   , D7= 70.2912, D8= 6.21039E-4

     *,    E1=-1.47769E-3, E2= .201563E-3, E3=-4.79279, E4= ZERO
     *,    E5= 10.9460E-3, E6=-2.30007E-3, E7= 63.5190, E8=-2.91325E-5

     *,    F1=-10.1311E-3, F2= 1.80265E-3, F3= 98.4758, F4= 1.43812E-5
     *,    F5= 2.35443E-3, F6=-.371430E-3, F7= 120.228, F8=-2.39016E-6 )

      DIMENSION X1(MLEN), X3(MLEN)
      DIMENSION TX(MLEN), TY(MLEN), TZ(MLEN)
      DIMENSION WW(MLEN), YW(MLEN), ZW(MLEN)

      DO I = 1,MLEN
      IF(X1(I) .GT. X1MAX) X1(I) = X1MAX
      IF(X3(I) .GT. X3MAX) X3(I) = X3MAX
      ENDDO

      DO I = 1,MLEN
C A1
      WW(I) = B1         * X1(I) + ONE
      TX(I) = B2         * X1(I) + ONE
      WW(I) = WW(I) / TX(I)
C A2
      TX(I) = B3         * X1(I) + B4
      TX(I) = TX(I) * X1(I) + B6
      TY(I) = B5         * X1(I) + ONE
      TX(I) = TX(I) / TY(I)
C A3
      TY(I) = B7         * X1(I) + ONE
      TZ(I) = B8         * X1(I) + B9
      TY(I) = TY(I) / TZ(I)
C WW
      TZ(I) = SQRT(X3(I))
      TY(I) = TY(I) * TZ(I) + ONE

      TX(I) = TX(I) * TZ(I)
      WW(I) = WW(I) + TX(I)
      WW(I) = WW(I) / TY(I)
      IF(WW(I) .LT. ZERO) WW(I) = ZERO
C  B1
      TX(I) = C1         * X1(I) + C2
      TX(I) = TX(I) * X1(I) + C4
      YW(I) = C3         * X1(I) + ONE
      TX(I) = TX(I) / YW(I)
C  B2
      TY(I) = C5         * X1(I) + C6
      TY(I) = TY(I) * X1(I) + C8
      YW(I) = C7         * X1(I) + ONE
      TY(I) = TY(I) / YW(I)

      TY(I) = TY(I) * TZ(I)
      TX(I) = TX(I) + TY(I)
C  B3
      TY(I) = D1         * X1(I) + D2
      TY(I) = TY(I) * X1(I) + D4
      YW(I) = D3         * X1(I) + ONE
      TY(I) = TY(I) / YW(I)

      TY(I) = TY(I) * X3(I)
      TX(I) = TX(I) + TY(I)
C  B4
      TY(I) = D5         * X1(I) + D6
      TY(I) = TY(I) * X1(I) + D8
      YW(I) = D7         * X1(I) + ONE
      TY(I) = TY(I) / YW(I)
C YW
      TY(I) = TY(I) * X3(I)
      TY(I) = TY(I) * TZ(I)
      YW(I) = TX(I) + TY(I)
C  C1
      TX(I) = E1         * X1(I) + E2
      TX(I) = TX(I) * X1(I) + E4
      ZW(I) = E3         * X1(I) + ONE
      TX(I) = TX(I) / ZW(I)
C  C2
      TY(I) = E5         * X1(I) + E6
      TY(I) = TY(I) * X1(I) + E8
      ZW(I) = E7         * X1(I) + ONE
      TY(I) = TY(I) / ZW(I)

      TY(I) = TY(I) * TZ(I)
      TX(I) = TX(I) + TY(I)
C  C3
      TY(I) = F1         * X1(I) + F2
      TY(I) = TY(I) * X1(I) + F4
      ZW(I) = F3         * X1(I) + ONE
      TY(I) = TY(I) / ZW(I)

      TY(I) = TY(I) * X3(I)
      TX(I) = TX(I) + TY(I)
C  C4
      TY(I) = F5         * X1(I) + F6
      TY(I) = TY(I) * X1(I) + F8
      ZW(I) = F7         * X1(I) + ONE
      TY(I) = TY(I) / ZW(I)
C ZW
      TY(I) = TY(I) * X3(I)
      TY(I) = TY(I) * TZ(I)
      ZW(I) = TX(I) + TY(I)
      ENDDO

      RETURN
      END

      SUBROUTINE GFUN2(X2,WV,YV,ZV,TX,TZ,MLEN)

      PARAMETER (ZERO=0.,ONE=1.)
      PARAMETER (X2MAX=2.)

      PARAMETER
     *    (A1=  176.396  , A2= .331236E+2, A3=.922411E+1
     *,    A4=-.0187096  , A5= .085107E-1, A6=.323105E-1
     *,    A7=-.016741E-3, A8=-.092229E-3, A9=.239186E-3   )


      DIMENSION X2(MLEN)
      DIMENSION TX(MLEN), TZ(MLEN)
      DIMENSION WV(MLEN), YV(MLEN), ZV(MLEN)


      DO 10 I = 1,MLEN
      IF(X2(I) .GT. X2MAX) X2(I) = X2MAX

C WV
      TZ(I) = SQRT(X2(I))
      TX(I) = A1         * X2(I) + A2
      TX(I) = TX(I) * X2(I) + ONE
      WV(I) = A3         * TZ(I) + TX(I)
      WV(I) = ONE        / WV(I)
      IF(WV(I) .LT. ZERO) WV(I) = ZERO
C YV
      YV(I) = A4         * X2(I) + A5
      YV(I) = YV(I) * TZ(I)
      YV(I) = A6         * X2(I) + YV(I)
C ZV
      TX(I) = A7         * X2(I) + A9
      TX(I) = A8         * TZ(I) + TX(I)
      ZV(I) = TX(I) * X2(I)
  10  CONTINUE

      RETURN
      END
      SUBROUTINE TRCO2(X1,X2,X3,X4,TA,TS,MLEN)

      PARAMETER (ZERO=0.,ONE=1.)

      PARAMETER (A2=15.1, A3=3.1)
      PARAMETER (B2=0.90, B3=.04)
      PARAMETER (C2=16.0, C3=6.7)
      PARAMETER (D1=.83,  D2=27.0)
      PARAMETER (F1=ONE/2.6)
      PARAMETER (EXPMAX=-10.)

      DIMENSION X1(MLEN), X2(MLEN), X3(MLEN), X4(MLEN)
      DIMENSION TA(MLEN), TS(MLEN)


C******* BEGIN 15 MICRON BAND   *****************
C******  TRANSMITTANCE RETURNED IN TA ***********

      DO 10 I = 1,MLEN
C====>  CO2 BAND: CENTER

C  X1 ** .5625 ( REPLACES 0.56 IN CHOU AND PENG)

      TA(I) = SQRT(X1(I))
      TS(I) = SQRT(TA(I))
      TS(I) = SQRT(TS(I))
      TS(I) = SQRT(TS(I))
      TA(I) = TA(I) *  TS(I)

C  TRANSMITTANCE

      TA(I) = TA(I) * A2 + ONE
      X1(I) = A3 * X1(I) / TA(I)

C====>  CO2 BAND: WINGS

C  X1 ** .5625 ( REPLACES 0.57 IN CHOU AND PENG)

      TA(I) = SQRT(X2(I))
      TS(I) = SQRT(TA(I))
      TS(I) = SQRT(TS(I))
      TS(I) = SQRT(TS(I))
      TA(I) = TA(I) *  TS(I)

C  TRANSMITTANCE

      TA(I) = TA(I) * B2 + ONE
      X2(I) = B3 * X2(I) / TA(I)

C====>  H2O LINE OVERLAP

C  X3 ** .59375 ( REPLACES 0.60 IN DOC.)

      TA(I) = SQRT(X3(I))
      TS(I) = SQRT(TA(I))
      TS(I) = SQRT(TS(I))
      TS(I) = SQRT(TS(I))
      TA(I) = TA(I) *  TS(I)
      TS(I) = SQRT(TS(I))
      TA(I) = TA(I) *  TS(I)

C  TRANSMITTANCE

      TA(I) = TA(I) * C2 + ONE
      X3(I) = C3 * X3(I) / TA(I)

C====>  H2O E-TYPE OVERLAP

C  X4 ** .8125  ( REPLACES 0.83 IN DOC.)

      TA(I) = SQRT(X4(I))
      TS(I) = SQRT(TA(I))
      TA(I) = TA(I) *  TS(I)
      TS(I) = SQRT(TS(I))
      TS(I) = SQRT(TS(I))
      TA(I) = TA(I) *  TS(I)

C  TRANSMITTANCE

      X4(I) = TA(I) * D2 + X3(I)

C====>  TOTAL TANSMITTANCE

      X1(I) = -X1(I) - X4(I)
      X2(I) = -X2(I) - X4(I)

      IF ( X3(I) .LT. EXPMAX ) X3(I) = EXPMAX
      IF ( X4(I) .LT. EXPMAX ) X4(I) = EXPMAX
  10  CONTINUE
      CALL EXPSUB (X1,MLEN,X3)
      CALL EXPSUB(X2,MLEN,X4)

      DO 11 I = 1,MLEN
      X3(I) = X3(I) - X4(I)
      TA(I) = X4(I) + X3(I) * F1
  11  CONTINUE

C******* END 15 MICRON BAND   *****************

      RETURN
      END


      SUBROUTINE TRO3(X4,X6,X5,TX,TS,OZONE,MLEN)

#include <vrslv.com>
C
      PARAMETER (ZERO=0.,ONE=1.)

      PARAMETER (E1=9.79)
      PARAMETER (G2=345.28*4./.8796)
      PARAMETER (G4=-4.398)
      PARAMETER (G5=81.21/120.)
      PARAMETER (EXPMAX = 10. )

      DIMENSION X4(MLEN), X5(MLEN), X6(MLEN)
      DIMENSION TX(MLEN), TS(MLEN)
      DIMENSION TTMP(IMR*(JMR+1)*NLAYR)

      LOGICAL OZONE

C******  BEGIN 9.6 MICRON BAND  *********************
C******  TRANSMITTANCE RETURNED IN TX ***************

C  H2O CONTINUUM OVERLAP

      DO 10 I = 1,MLEN
      TS(I) = X4(I) * E1
      IF ( TS(I).GT.EXPMAX ) TS(I) = EXPMAX
      TS(I) = -TS(I)
  10  CONTINUE
      CALL EXPSUB(TS,MLEN,TX)

C  OZONE: EQS(6-8) Rosenfield et al. (1987) JAS pg 862

      IF (OZONE) THEN
      DO 11 I = 1,MLEN
      IF ( X6(I).NE.ZERO .AND. X5(I).NE.ZERO )THEN
      TS(I) = X5(I) * X5(I) / X6(I)
      TS(I) = ONE        + TS(I) * G2
      TS(I) = SQRT(TS(I)) - ONE
      TS(I) = X6(I) * TS(I) / X5(I)
      TS(I) = TS(I) * G4
      ENDIF
  11  CONTINUE
      CALL EXPSUB(TS,MLEN,TTMP)
      DO 12 I = 1,MLEN
      TS(I) = ONE        - TTMP(I)
      TS(I) = ONE        - TS(I) * G5
      IF ( X6(I).EQ.ZERO .OR. X5(I).EQ.ZERO )THEN
      TS(I) = ONE
      ENDIF
      TX(I) = TX(I) * TS(I)
  12  CONTINUE
      ENDIF

C******  END   9.6 MICRON BAND  *********************

      RETURN
      END
      SUBROUTINE EXPSUB(ARG,LEN,OUT)
C
      PARAMETER (ONE=1.0)
      DIMENSION ARG(LEN), OUT(LEN)
      DIMENSION EXPCOEF(4)
      DATA EXPCOEF/1., -2.507213E-1, 2.92732E-2, -3.8278E-3/
C
      DO 10 I = 1,LEN
       OUT(I) = EXPCOEF(4)
       OUT(I) = OUT(I) * ARG(I) + EXPCOEF(3)
       OUT(I) = OUT(I) * ARG(I) + EXPCOEF(2)
       OUT(I) = OUT(I) * ARG(I) + EXPCOEF(1)
       OUT(I) = OUT(I) * OUT(I)
       OUT(I) = OUT(I) * OUT(I)
       OUT(I) = ONE /(OUT(I))
 10   CONTINUE
C
      RETURN
      END
      SUBROUTINE POLY(P,X,B,N,LEN)
      DIMENSION P(LEN), X(LEN), B(N)
C
      DO 10 I = 1,LEN
      P(I)  = B(N)
  10  CONTINUE
      IF (N .GT. 1) THEN
         DO 100 L=N,2,-1
         DO 11 I = 1,LEN
         P(I)  = P(I) * X(I) + B(L-1)
  11     CONTINUE
100      CONTINUE
      ENDIF
C
      RETURN
      END
       SUBROUTINE TURBIO (N,NYMD,NHMS,NDTURB,QBEG,QBUDGET,
     .                    IMAX,JMAX,KMAX)
#include <vrslv.com>
#include <pblparm.com>
#include <vmetric.com>
#include <vstate.com>
#include <vphyscom.com>
#include <stripmine.com>

 
#if macro
cmic$ taskcommon mspace 
#endif
      COMMON /MSPACE/ WRKFLX (ISTRIP,36*NLAYR+31)
      COMMON /MSPACE/ WRKLEN (ISTRIP,10*NLAYR-7 )
      COMMON /MSPACE/ WRKTRB (ISTRIP,5*NLAYR-5 )
      COMMON /MSPACE/ WRKSF1 (ISTRIP,37         )
      COMMON /MSPACE/ WRKSF2 (ISTRIP,42         )
      COMMON /MSPACE/ WRKSF3 (ISTRIP,26         )
 
      DIMENSION PKZ(IMR,JMR+1,NLAYR)
      DIMENSION PLZ(IMR,JMR+1,NLAYR)

      DIMENSION       U(ISTRIP,NLAYR+1),   V(ISTRIP,NLAYR+1)
      DIMENSION      TH(ISTRIP,NLAYR+1),  SH(ISTRIP,NLAYR+1)
      DIMENSION       P(ISTRIP,NLAYR),    PE(ISTRIP,NLAYR+1)
      DIMENSION      PK(ISTRIP,NLAYR),   PKE(ISTRIP,NLAYR+1)
      DIMENSION      QQ(ISTRIP,NLAYR),   THV(ISTRIP,NLAYR+1)
      DIMENSION     SRI(ISTRIP,NLAYR),   SET(ISTRIP,NLAYR)
      DIMENSION     SEU(ISTRIP,NLAYR)
      DIMENSION STUFLUX(ISTRIP,NLAYR), STVFLUX(ISTRIP,NLAYR)
      DIMENSION STTFLUX(ISTRIP,NLAYR), STQFLUX(ISTRIP,NLAYR)
      DIMENSION FRQTRB(ISTRIP,NLAYR-1)
      DIMENSION   STWET(ISTRIP), STRFLX(ISTRIP)
      DIMENSION    STZ0(ISTRIP),  STEVE(ISTRIP)
      DIMENSION  STCLHG(ISTRIP), STHCPG(ISTRIP)
      DIMENSION  STCNDI(ISTRIP)
      DIMENSION  STDIAG(ISTRIP),  TENDS(ISTRIP)
      DIMENSION  SUSTAR(ISTRIP),    SZ0(ISTRIP)
      DIMENSION pbldpth(ISTRIP),  STSHG(ISTRIP)
      DIMENSION  SDTSRF(ISTRIP),    STG(ISTRIP),    SQS(ISTRIP)
      DIMENSION     SCT(ISTRIP),    SCU(ISTRIP), SWINDS(ISTRIP)
      DIMENSION    SDTG(ISTRIP),    STS(ISTRIP),    SQG(ISTRIP)
      INTEGER    STWATR(ISTRIP)
c
      dimension stu2m(istrip),stv2m(istrip),stt2m(istrip),stq2m(istrip)
      dimension stu10m(istrip),stv10m(istrip),
     1                              stt10m(istrip),stq10m(istrip)

      DIMENSION    CLHG(IJALL),   HCAPG(IJALL)
      DIMENSION     SHG(IJALL), CONDICE(IJALL)
      DIMENSION     WET(IJALL),      Z0(IJALL)
      DIMENSION TMPDIAG(IJALL),  RADFLX(IJALL)
      DIMENSION   THTGZ(IJALL),     EVE(IJALL)
      DIMENSION    TMP1(IJALL),    TMP2(IJALL)
      DIMENSION    TMP3(IJALL),   ITMP1(IJALL)
      DIMENSION   ITMP2(IJALL)

C  INTEGER VARIABLES FOR LWI CALCULATIONS
C  ======================================
      INTEGER WATER  (IMR,JMR+1)
      INTEGER SNOWY  (IMR,JMR+1)
      INTEGER ICY    (IMR,JMR+1)
      INTEGER FROSTY (IMR,JMR+1)
      INTEGER DIRT   (IMR,JMR+1)
      INTEGER SEAICY (IMR,JMR+1)
      LOGICAL QBUDGET
      LOGICAL QBEG
      LOGICAL TPROF
      PARAMETER ( ALBSNO = 0.4 )
C
      DATA TPROF/.FALSE./
C
C   COMPUTE VARIABLES THAT DO NOT CHANGE
C
       SECDAY   = GETCON('SDAY')
       SDAYOPI2 = GETCON('SDAY') / (PI*TWO)
       RGAS     = GETCON('RGAS')
       CP       = GETCON('CP')
       ALHL     = GETCON('LATENT HEAT COND')
       FACEPS   = GETCON('EPSFAC')
       GRAV     = GETCON('GRAVITY')
       CALTOJ   = GETCON('CALTOJ')
       VIRTCON  = GETCON('VIRTCON')
       HEATW    = GETCON('HEATW')
       HEATI    = GETCON('HEATI')
       UNDEF    = GETCON('UNDEF')

      TIMSTP    = MODHMS(NDTURB,0)
      PTKAP     = EXPBYK(PTOP)

       ISTNP1   =   ISTRIP * NLAYP1
       ISTNLAY  =   ISTRIP * NLAY
       ITRTRB   = ( TIMSTP / DELTTRB ) + ZP1
       DTTRB    =   TIMSTP / FLOAT(ITRTRB)
       EDLE     =   ED * ZP2
C
C   COEFFICIENT OF VISCOSITY (BACKGROUND MOMENTUM DIFFUSION)
C
       RMU     = FMU * TREF * RGAS / PREF
       CLTJ10  = TEN * CALTOJ
       ATIMSTP = ONE / TIMSTP

C **********************************************************************
C ****                UPDATE SURFACE SPECIFICATION                  ****
C **********************************************************************
      DO 9106 I = 1,IJALL
                             WATER(I,1) = 0
                              DIRT(I,1) = 0
                               ICY(I,1) = 0
      IF(ZLWI(I,1).EQ.ONE  ) WATER(I,1) = 1
      IF(ZLWI(I,1).EQ.TWO  )  DIRT(I,1) = 1
      IF(ZLWI(I,1).EQ.THREE)   ICY(I,1) = 1
                                 SEAICY(I,1) = 0
      IF (THKZ(I,1).GT.ZERO )    SEAICY(I,1) = 1
      IF ( WATER(I,1).EQ.1 .AND. SEAICY(I,1).EQ.0 )THEN
        WATER(I,1) = 1
       ELSE
        WATER(I,1) = 0
       ENDIF
      IF( ICY(I,1).EQ.1 .OR. SEAICY(I,1).EQ.1 ) ICY(I,1) = 1
                                                     SNOWY(I,1) = 0
      IF( DIRT(I,1).EQ.1 .AND. ALBZ(I,1).GT.ALBSNO ) SNOWY(I,1) = 1
      IF( DIRT(I,1).EQ.1 .AND. ALBZ(I,1).LE.ALBSNO ) THEN
        DIRT(I,1) = 1
       ELSE
        DIRT(I,1) = 0
      ENDIF
                                                  FROSTY(I,1) = 0
      IF( DIRT(I,1).EQ.1 .AND. TGZ(I,1).LE.TICE ) FROSTY(I,1) = 1
 9106 CONTINUE
C
C   COMPUTE P ** KAPPA AT LAYER CENTERS
C
      CALL PKAPPA ( PZ(1,1,N),PKHT(1,1,1),PKZ,IM,JNP,NLAY )
C
C   SURFACE PARAMETERS - WET AND TG
C
      DO 9001 I =1,IJALL
       WET(I) = ONE
 9001 CONTINUE
      DO 9100 I =1,IJALL
       IF ( GWZ(I,1).LT.Z1PEM20 ) GWZ(I,1) = ZERO
       IF ( (SNOWY(I,1).EQ.1) .AND. (TGZ(I,1).GT.TICE) )
     1       TGZ(I,1) = TICE
       IF ( DIRT(I,1).EQ.1 ) WET(I) = GWZ(I,1)
 9100 CONTINUE
C
C   ROUGHNESS LENGTH Z0
C
      DO 9102 I =1,IJALL
       IF ( SEAICY(I,1).EQ.1 )THEN
        Z0(I) = Z1PEM4
       ELSE
        Z0(I) = SRZ(I,1)
       ENDIF
 9102 CONTINUE
C
C   EVAPOTRANSPIRATION COEFFICIENT EVE
C
      DO 9002 I =1,IJALL
                           EVE(I) = WET(I) / FWET
      IF ( EVE(I).GT.ONE ) EVE(I) = ONE
 9002 CONTINUE
C
C   COMPUTE HEAT CAPACITY OF THE GROUND AND LATENT HEAT OF
C                                              EVAPORATION/SUBLIMATION
      DO 9003 I =1,IJALL
        CLHG(I) = HEATW
       HCAPG(I) = ZP386 + ZP15 * WET(I)
       HCAPG(I) = (ONE + WET(I)) * HCAPG(I)
       HCAPG(I) = (Z2PEM3*SDAYOPI2) * HCAPG(I)
       HCAPG(I) = SQRT ( HCAPG(I) )
       IF ( SNOWY(I,1).EQ.1 ) THEN
         CLHG(I) = HEATI
        HCAPG(I) = TWOP3
       ENDIF
       IF ( ICY(I,1).EQ.1 ) THEN
         CLHG(I) = HEATI
        HCAPG(I) = FIVEP1
       ENDIF
        CLHG(I) = CLHG(I) * CALTOJ
       HCAPG(I) = HCAPG(I) * CLTJ10
 9003 CONTINUE
C
C   VALUE OF SH AT GROUND
C
      DO 9028 I =1,IJALL
       THTGZ(I) = PZ(I,1,N) + PTOP
9028  CONTINUE
      CALL QSTARS (TGZ,THTGZ,TMP1,TMP2,TMP3,ITMP1,ITMP2,IJALL,SHG)
      DO 9030 I =1,IJALL
      THTGZ(I) = TGZ(I,1) / PKHT(I,1,NLAY)
9030  CONTINUE
C   COMPUTE NET RADIATIVE FLUX INTO THE GROUND
C
      DO 9032 I =1,IJALL
      RADFLX(I) = RADSWG(I,1) * RADSWT(I,1) - RADLWG(I,1)
9032  CONTINUE
C   ADD HEAT CONDUCTION THROUGH ICE TO NET RADIATIVE FLUX
C
      CONST = ( CTI / HICE ) * CLTJ10
      DO 9104 I =1,IJALL
       IF ( SEAICY(I,1).EQ.1 ) THEN
        CONDICE(I) = TICE - TGZ(I,1)
        RADFLX(I) = RADFLX(I) + CONST * CONDICE(I)
       ENDIF
9104  CONTINUE

C**********************************************************************
C                        LOOP OVER REGIONS
C**********************************************************************

#if macro
cmic$ do all shared (alhl, clhg, cp, dqturb, dsig, dttrb, dtturb)
cmic$*       shared (duturb, dvturb, edle, eve, faceps, grav, hcapg)
cmic$*       shared (ict, icu, idtsrf, ieflux, iet, ieu, ievap)
cmic$*       shared (ifrqtrb, ihflux, ijall, imr, ipbl, iq10m)
cmic$*       shared (iq2m, iqg, iri, istnlay, istnp1, istrip, it10m)
cmic$*       shared (it2m, itqflux, itrtrb, ittflux, ituflux, itvflux)
cmic$*       shared (iu10m, iu2m, iuflux, iustar, iv10m, iv2m)
cmic$*       shared (ivflux, iwinds, iz0, jmr, n, nhms, nlay, nlaym1)
cmic$*       shared (nlayp1, nlayr, npcs, nprogs2, nprogu2, nymd)
cmic$*       shared (one, pkht, pkz, ptkap, ptop, pz, qbeg, qdiag)
cmic$*       shared (qqz, qz, radflx, rgas, rmu, shg, sig, sige)
cmic$*       shared (thtgz, tprof, tz, undef, uz, virtcon, vz)
cmic$*       shared (water, wet, z0, zero)
cmic$*       private (frqtrb, i, l, nn, p, pbldpth, pe, pk, pke)
cmic$*       private (qq, sct, scu, set, seu, sh, sri, stclhg)
cmic$*       private (stdiag, steve, sthcpg, stq10m, stq2m, stqflux)
cmic$*       private (strflx, stshg, stt10m, stt2m, sttflux, stu10m)
cmic$*       private (stu2m, stuflux, stv10m, stv2m, stvflux, stwatr)
cmic$*       private (stwet, stz0, sustar, swinds, sz0, tends)
cmic$*       private (th, thv, tmpdiag, u, v)
#endif

      DO 2000 NN = 1, NPCS

       CALL STRIP  (WET,STWET,IJALL,ISTRIP,1,NN)
       CALL STRIP  (Z0,STZ0,IJALL,ISTRIP,1,NN)
       CALL STRIP  (EVE,STEVE,IJALL,ISTRIP,1,NN)
       CALL STRIP  (CLHG,STCLHG,IJALL,ISTRIP,1,NN)
       CALL STRIP  (HCAPG,STHCPG,IJALL,ISTRIP,1,NN)
       CALL STRIP  (SHG,STSHG,IJALL,ISTRIP,1,NN)
       CALL STRIP  (RADFLX,STRFLX,IJALL,ISTRIP,1,NN)
       CALL STRIP  (UZ(1,1,1,N),U,IJALL,ISTRIP,NLAY,NN)
       CALL STRIP  (VZ(1,1,1,N),V,IJALL,ISTRIP,NLAY,NN)
       CALL STRIP  (TZ(1,1,1,N),TH,IJALL,ISTRIP,NLAY,NN)
       CALL STRIP  (THTGZ,TH(1,NLAYP1),IJALL,ISTRIP,1,NN)
       CALL STRIP  (QZ(1,1,1,N),SH,IJALL,ISTRIP,NLAY,NN)
       CALL STRIP  (PZ(1,1,N),PE(1,NLAYP1),IJALL,ISTRIP,1,NN)
       CALL STRIP  (PKZ,PK,IJALL,ISTRIP,NLAY,NN)
       CALL STRIP  (PKHT(1,1,1),PKE(1,2),IJALL,ISTRIP,NLAY,NN)
       CALL STRIP  (QQZ,QQ,IJALL,ISTRIP,NLAYM1,NN)
       CALL STRINT (WATER,STWATR,IJALL,ISTRIP,1,NN)

      DO I =1,ISTRIP
        PE(I,1)    = PTOP 
       PKE(I,1)    = PTKAP
       U(I,NLAYP1) = ZERO
       V(I,NLAYP1) = ZERO
      ENDDO   

       DO 1000 L = 1, NLAY
       DO 9038 I = 1, ISTRIP
         P(I,L) = SIG (L)   * PE(I,NLAYP1) + PTOP
      PE(I,L+1) = SIGE(L+1) * PE(I,NLAYP1) + PTOP
 9038 CONTINUE
 1000 CONTINUE

      DO I =1,ISTRIP
      SH(I,NLAYP1) = SH(I,NLAY) + STEVE(I)*( STSHG(I)-SH(I,NLAY) )
      IF ( SH(I,NLAY) .GT. STSHG(I) ) SH(I,NLAYP1) = STSHG(I)
      enddo
      IF(IQG.GT.0) then
      do i=1,istrip
      tmpdiag(i) = sh(i,nlayp1)*1000
      enddo
      CALL PSTBMP (tmpdiag,QDIAG(1,1,IQG),ISTRIP,IJALL,1,NN)
      endif

C COMPUTE VIRTUAL POTENTIAL TEMPERATURES
C --------------------------------------
      DO I =1,ISTNP1
      THV(I,1) = ONE + VIRTCON * SH(I,1)
      THV(I,1) = TH(I,1) * THV(I,1)
      enddo
      DO I =1,ISTRIP
      SH(I,NLAYP1) = STSHG(I)
      enddo

C ZERO OUT QQ ARRAY FOR BEGIN CASE
C --------------------------------
      IF(QBEG) then
      DO I =1,ISTNLAY
      QQ(I,1) = ZERO
      enddo
      endif

C *********************************************************************
C ** INCREMENT DIAGNOSTIC ARRAYS FOR QUANTITIES CALCULATED BEFORE TRBFL
C *********************************************************************

      DO 9048 I =1,ISTRIP
      STDIAG(I) = ( THV(I,NLAYP1)-THV(I,NLAY) ) / PKE(I,NLAYP1)
9048  CONTINUE
      IF(IDTSRF.GT.0) then
      CALL PSTBMP (STDIAG,QDIAG(1,1,IDTSRF),ISTRIP,IJALL,1,NN)
      endif

C CALL TRBFLX
C -----------
      CALL TRBFLX(NN,TH,THV,SH,U,V,QQ,P,PE,PK,PKE,DSIG,STWATR,STZ0,
     1 STEVE,STHCPG,STCLHG,STRFLX,DTTRB,ITRTRB,RMU,EDLE,QBEG,TPROF,
     2 ITUFLUX,ITVFLUX,ITTFLUX,ITQFLUX,IRI,ICT,ICU,IET,IEU,IWINDS,
     3 IHFLUX,IEFLUX,IUFLUX,IVFLUX,IEVAP,iustar,iz0,ifrqtrb,ipbl,
     4 iu2m,iv2m,it2m,iq2m,iu10m,iv10m,it10m,iq10m,
     5 STUFLUX,STVFLUX,STTFLUX,STQFLUX,SRI,SET,SEU,SWINDS,
     6 sustar,sz0,frqtrb,pbldpth,sct,scu,
     7 stu2m,stv2m,stt2m,stq2m,stu10m,stv10m,stt10m,stq10m,
     8 istrip,nlayr,NYMD,NHMS,grav,cp,rgas,faceps,virtcon,undef)

      CALL PASTE (TH(1,NLAYP1),THTGZ,ISTRIP,IJALL,1,NN)
      CALL PASTE (QQ,QQZ,ISTRIP,IJALL,NLAY,NN)

C TENDENCY UPDATES
C ----------------
      DO  1500 L=1,NLAY
      CALL STRIP (UZ(1,1,L,N),TMPDIAG,IJALL,ISTRIP,1,NN)
      DO I =1,ISTRIP
      TENDS(I) = ( U(I,L)-TMPDIAG(I) )
      enddo
      CALL PASTE (TENDS,DUTURB(1,1,L),ISTRIP,IJALL,1,NN)
      CALL STRIP (VZ(1,1,L,N),TMPDIAG,IJALL,ISTRIP,1,NN)
      DO I =1,ISTRIP
      TENDS(I) = ( V(I,L)-TMPDIAG(I) )
      enddo
      CALL PASTE (TENDS,DVTURB(1,1,L),ISTRIP,IJALL,1,NN)
      CALL STRIP (TZ(1,1,L,N),TMPDIAG,IJALL,ISTRIP,1,NN)
      DO I =1,ISTRIP
      TENDS(I) = ( TH(I,L)-TMPDIAG(I) )
      enddo
      CALL PASTE (TENDS,DTTURB(1,1,L),ISTRIP,IJALL,1,NN)
      CALL STRIP (QZ(1,1,L,N),TMPDIAG,IJALL,ISTRIP,1,NN)
      DO I =1,ISTRIP
      TENDS(I) = ( SH(I,L)-TMPDIAG(I) )
      enddo
      CALL PASTE (TENDS,DQTURB(1,1,L),ISTRIP,IJALL,1,NN)
 1500 CONTINUE

C *********************************************************************
C **** INCREMENT DIAGNOSTIC ARRAYS FOR QUANTITIES SAVED IN TRBFLX
C *********************************************************************
C Note: The order, logic, and scaling of the heat and moisture flux
C       diagnostics IS CRITICAL!
C ------------------------------
      IF(IEVAP.GT.0) THEN
      DO I=1,ISTRIP
      tmpdiag(i) = STQFLUX(I,NLAY) * 86400
      enddo
      CALL PSTBMP(tmpdiag,QDIAG(1,1,IEVAP),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(IEFLUX.GT.0) THEN
      DO I=1,ISTRIP
      tmpdiag(i) = STQFLUX(I,NLAY) * ALHL
      enddo
      CALL PSTBMP (tmpdiag,QDIAG(1,1,IEFLUX),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(IHFLUX.GT.0) THEN
      DO I=1,ISTRIP
      tmpdiag(i) = STTFLUX(I,NLAY) * CP
      enddo
      CALL PSTBMP (tmpdiag,QDIAG(1,1,IHFLUX),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(ITUFLUX.GT.0) THEN
      CALL PSTBMP (STUFLUX,QDIAG(1,1,ITUFLUX),ISTRIP,IJALL,NLAY,NN)
      ENDIF
      IF(ITVFLUX.GT.0) THEN
      CALL PSTBMP (STVFLUX,QDIAG(1,1,ITVFLUX),ISTRIP,IJALL,NLAY,NN)
      ENDIF
      IF(ITTFLUX.GT.0) THEN
      do L=1,nlay
      do i=1,istrip
      STTFLUX(I,L) = STTFLUX(I,L) * CP
      enddo
      enddo
      CALL PSTBMP (STTFLUX,QDIAG(1,1,ITTFLUX),ISTRIP,IJALL,NLAY,NN)
      ENDIF
      IF(ITQFLUX.GT.0) THEN
      do L=1,nlay
      do i=1,istrip
      STQFLUX(I,L) = STQFLUX(I,L) * ALHL
      enddo
      enddo
      CALL PSTBMP (STQFLUX,QDIAG(1,1,ITQFLUX),ISTRIP,IJALL,NLAY,NN)
      ENDIF
      IF(IRI.GT.0) CALL PSTBMP(SRI,QDIAG(1,1,IRI),ISTRIP,IJALL,NLAY,NN)
      IF(IET.GT.0) CALL PSTBMP(SET,QDIAG(1,1,IET),ISTRIP,IJALL,NLAY,NN)
      IF(IEU.GT.0) CALL PSTBMP(SEU,QDIAG(1,1,IEU),ISTRIP,IJALL,NLAY,NN)
      IF(ICT.GT.0) THEN
      CALL PSTBMP (SCT,QDIAG(1,1,ICT),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(ICU.GT.0) THEN
      CALL PSTBMP (SCU,QDIAG(1,1,ICU),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(IWINDS.GT.0) THEN
      CALL PSTBMP (SWINDS,QDIAG(1,1,IWINDS),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(IUFLUX.GT.0) THEN
      CALL PSTBMP (STUFLUX(1,NLAY),QDIAG(1,1,IUFLUX),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(IVFLUX.GT.0) THEN
      CALL PSTBMP (STVFLUX(1,NLAY),QDIAG(1,1,IVFLUX),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(iustar.GT.0) THEN
      CALL PSTBMP (sustar,QDIAG(1,1,iustar),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(iz0.GT.0) THEN
      CALL PSTBMP (sz0,QDIAG(1,1,iz0),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(ifrqtrb.GT.0) THEN
      CALL PSTBMP (frqtrb,QDIAG(1,1,ifrqtrb),ISTRIP,IJALL,NLAYR-1,NN)
      ENDIF
      IF(ipbl.GT.0) THEN
      CALL PSTBMP (pbldpth,QDIAG(1,1,ipbl),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(iu2m.GT.0) THEN
      CALL PSTBMP (stu2m,QDIAG(1,1,iu2m),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(iv2m.GT.0) THEN
      CALL PSTBMP (stv2m,QDIAG(1,1,iv2m),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(it2m.GT.0) THEN
      CALL PSTBMP (stt2m,QDIAG(1,1,it2m),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(iq2m.GT.0) THEN
      CALL PSTBMP (stq2m,QDIAG(1,1,iq2m),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(iu10m.GT.0) THEN
      CALL PSTBMP (stu10m,QDIAG(1,1,iu10m),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(iv10m.GT.0) THEN
      CALL PSTBMP (stv10m,QDIAG(1,1,iv10m),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(it10m.GT.0) THEN
      CALL PSTBMP (stt10m,QDIAG(1,1,it10m),ISTRIP,IJALL,1,NN)
      ENDIF
      IF(iq10m.GT.0) THEN
      CALL PSTBMP (stq10m,QDIAG(1,1,iq10m),ISTRIP,IJALL,1,NN)
      ENDIF
 2000 CONTINUE

C**********************************************************************
C   END REGIONS LOOP
C**********************************************************************
C **** INCREMENT DIAGNOSTIC ARRAY FOR GROUND AND SURFACE TEMPERATURES,
C           GROUND TEMP TENDENCY, AND GROUND HUMIDITY
C
      IF(ITG.GT.0) then
      DO I =1,IJALL
      QDIAG(I,1,ITG) = QDIAG(I,1,ITG) + TGZ(I,1)
      enddo
      endif
      IF(IQS.GT.0) then
      DO I =1,IJALL
      QDIAG(I,1,IQS) = QDIAG(I,1,IQS) + SHG(I)*1000
      enddo
      endif
      IF(ITS.GT.0) then
      DO I =1,IJALL
      TMPDIAG(I) = TZ(I,1,NLAY,N) * PKHT(I,1,NLAY)
      enddo
      DO I =1,IJALL
      QDIAG(I,1,ITS) = QDIAG(I,1,ITS) + TMPDIAG(I)
      enddo
      ENDIF
      IF(IDTG.GT.0) then
      DO I =1,IJALL
      TMPDIAG(I) = THTGZ(I) * PKHT(I,1,NLAY) - TGZ(I,1)
      enddo
      DO I =1,IJALL
      QDIAG(I,1,IDTG) = QDIAG(I,1,IDTG) + TMPDIAG(I)
      enddo
      ENDIF
 
C COMPUTE GROUND TEMPERATURE AND PREVENT ICE OR SNOW FROM MELTING
C ---------------------------------------------------------------
      DO I =1,IJALL
      TGZ(I,1) = THTGZ(I) * PKHT(I,1,NLAY)
      IF( (ICY(I,1).EQ.1 .OR. SNOWY(I,1).EQ.1)
     .                   .AND.  TGZ(I,1).GT.TICE ) TGZ(I,1)=TICE
      enddo

C **********************************************************************
C ****   ADD NEWTONIAN COOLING AND RAYLEIGH FRICTION ABOVE 10 MB    ****
C **********************************************************************

      if ( ptop.lt.10.0 ) then
      DO L=1,NLAYR
      DO I=1,IJALL
      plz( I,1,L ) = sig(L)*pz(i,1,n) + ptop
      ENDDO
      ENDDO
                    KZT = 52
      CALL RAYLEI ( KZT,IM,JNP,NLAY,NYMD,NHMS,
     .              UZ(1,1,1,N),VZ(1,1,1,N),
     .              TZ(1,1,1,N),PZ(1,1,N),PLZ,PKZ,
     .              DUTURB,DVTURB,DTTURB )
      endif

C *********************************************************************
C ****           INCREMENT DIAGNOSTIC ARRAYS FOR TENDENCIES        ****
C *********************************************************************
 
      IF(ITURBU.GT.0) then
      DO L = 1,NLAY
      DO I =1,IJALL
      QDIAG(I,1,ITURBU+L-1) =  QDIAG(I,1,ITURBU+L-1)
     .                      + DUTURB(I,1,L) * ATIMSTP * SECDAY
      enddo
      enddo
      endif

      IF(ITURBV.GT.0) then
      DO L = 1,NLAY
      DO I =1,IJALL
      QDIAG(I,1,ITURBV+L-1) =  QDIAG(I,1,ITURBV+L-1)
     .                      + DVTURB(I,1,L) * ATIMSTP * SECDAY
      enddo
      enddo
      endif

      IF(ITURBQ.GT.0) then
      DO L = 1,NLAY
      DO I =1,IJALL
      QDIAG(I,1,ITURBQ+L-1) =  QDIAG(I,1,ITURBQ+L-1)
     .                      + DQTURB(I,1,L) * ATIMSTP * SECDAY * 1000
      enddo
      enddo
      endif

      IF(ITURBT.GT.0) then
      DO L = 1,NLAY
      DO I =1,IJALL
      QDIAG(I,1,ITURBT+L-1) =  QDIAG(I,1,ITURBT+L-1)
     .                      + DTTURB(I,1,L) * PKZ(I,1,L)*ATIMSTP*SECDAY
      enddo
      enddo
      endif

C PI-WEIGHT THE TENDENCIES
C ------------------------
      DO I =1,IJALL
      THTGZ(I) = PZ(I,1,N) * ATIMSTP
      enddo
      DO 1502 L =1,NLAY
      DO 9082 I =1,IJALL
       DUTURB(I,1,L) = DUTURB(I,1,L)*THTGZ(I)
       DVTURB(I,1,L) = DVTURB(I,1,L)*THTGZ(I)
       DTTURB(I,1,L) = DTTURB(I,1,L)*THTGZ(I)
       DQTURB(I,1,L) = DQTURB(I,1,L)*THTGZ(I)
 9082 CONTINUE
 1502 CONTINUE

C *********************************************************************
C ****                BUMP DIAGNOSTIC COUNTERS                      ***
C *********************************************************************
      NTURBT  = NTURBT  + 1
      NTURBQ  = NTURBQ  + 1
      NTURBU  = NTURBU  + 1
      NTURBV  = NTURBV  + 1
      NTUFLUX = NTUFLUX + 1
      NTVFLUX = NTVFLUX + 1
      NTTFLUX = NTTFLUX + 1
      NTQFLUX = NTQFLUX + 1
      NWINDS  = NWINDS  + 1
      NET     = NET     + 1
      NEU     = NEU     + 1
      NRI     = NRI     + 1
      NCT     = NCT     + 1
      NCU     = NCU     + 1
      NTG     = NTG     + 1
      NTS     = NTS     + 1
      NDTG    = NDTG    + 1
      NQG     = NQG     + 1
      NQS     = NQS     + 1
      NHFLUX  = NHFLUX  + 1
      NEFLUX  = NEFLUX  + 1
      NEVAP   = NEVAP   + 1
      NUFLUX  = NUFLUX  + 1
      NVFLUX  = NVFLUX  + 1
      NDTSRF  = NDTSRF  + 1
      NUSTAR  = NUSTAR  + 1
      NZ0     = NZ0     + 1
      NFRQTRB = NFRQTRB + 1
      NPBL    = NPBL    + 1
      nu2m    = nu2m    + 1
      nv2m    = nv2m    + 1
      nt2m    = nt2m    + 1
      nq2m    = nq2m    + 1
      nu10m   = nu10m   + 1
      nv10m   = nv10m   + 1
      nt10m   = nt10m   + 1
      nq10m   = nq10m   + 1
      RETURN
      END
      SUBROUTINE TRBFLX (NN,TH,THV,SH,U,V,QQ,PL,PLE,PLK,PLKE,DSIG,
     1  IWATER,Z0,EVE,HCAPG,CLHG,RADFLX,DTAU,ITRTRB,KMBG,KHBG,
     2  QBEG,TPROF,ITUFLUX,ITVFLUX,ITTFLUX,ITQFLUX,IRI,ICT,ICU,
     3  IET,IEU,IWINDS,IHFLUX,IEFLUX,IUFLUX,IVFLUX,IEVAP,iustar,iz0,
     4  ifrqtrb,ipbl,iu2m,iv2m,it2m,iq2m,iu10m,iv10m,it10m,iq10m,
     5  WU,WV,WT,WSH,SRI,ET,EU,SWINDS,sustar,sz0,freqdg,pbldpth,
     6  sct,scu,stu2m,stv2m,stt2m,stq2m,stu10m,stv10m,stt10m,stq10m,
     7  irun,nlev,NYMD,NHMS,grav,cp,rgas,faceps,virtcon,undef)
C**********************************************************************
C  SUBROUTINE TRBFLX - COMPUTES TURBULENT ADJUSTMENTS TO ATMOSPHERIC
C                       PROFILE
C                    - CALLED FROM PBL DRIVER
C
C   ARGUMENTS ::
C
C     INPUT:
C     ------
C    TH            -         POTENTIAL TEMPERATURE PROFILE
C    THV           -         VIRTUAL POTENTIAL TEMPERATURE PROFILE
C    SH            -         SPECIFIC HUMIDITY PROFILE
C    U             -         U - COMPONENT OF WIND PROFILE
C    V             -         V - COMPONENT OF WIND PROFILE
C    QQ            -         TURBULENT KINETIC ENERGY
C    PL            -         EVEN LEVEL PRESSURES
C    PLE           -         EDGE LEVEL PRESSURES
C    PLK           -         EVEN LEVEL PRESSURES ** KAPPA
C    PLKE          -         EDGE LEVEL PRESSURES ** KAPPA
C    DSIG          -         SIGMA INTERVALS
C    WATER         -         BIT ARRAY - '1' OVER OCEANS
C    Z0            -         SURFACE ROUGHNESS
C    EVE           -         MOISTURE AVAILABILITY
C    HCAPG         -         HEAT CAPACITY OF GROUND
C    CLHG          -         LATENT HEAT COEFFICIENT AT GROUND
C    RADFLX        -         NET RADIATIVE PLUS CONDUCTIVE HEAT FLUX
C                              CONVERGENCE INTO GROUND (LY/SEC)
C    DTAU          -         TIME CHANGE PER ITERATION OF TRBLFX
C    ITRTRB        -         NUMBER OF ITERATIONS OF TRBLFX
C    KMBG          -         BACKGROUND VALUE OF MOMENTUM TRANSFER COEF
C    KHBG          -         BACKGROUND VALUE OF HEAT TRANSFER COEF
C    NLEV          -         NUMBER OF ATMOSPHERIC LEVELS TO CALCULATE
C    QBEG          -         LOGICAL .TRUE. FOR INITIAL START OF GCM
C    TPROF         -         LOGICAL .TRUE. TO CALCULATE PT BY PT DIAGS
C
C     OUTPUT:
C     -------
C    PROFILES RETURNED WITH UPDATED VALUES
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
      PARAMETER (IMJMR = IMR * (JMR+1) )

#if macro
cmic$ taskcommon mspace
#endif
      COMMON /MSPACE/ WRKFLX (ISTRIP,36*NLAYR+31)
      COMMON /MSPACE/ WRKLEN (ISTRIP,10*NLAYR-7 )
      COMMON /MSPACE/ WRKTRB (ISTRIP,5*NLAYR-5 )
      COMMON /MSPACE/ WRKSF1 (ISTRIP,37         )
      COMMON /MSPACE/ WRKSF2 (ISTRIP,42         )
      COMMON /MSPACE/ WRKSF3 (ISTRIP,26         )

      DIMENSION TH(ISTRIP,NLEV+1),THV(ISTRIP,NLEV+1),SH(ISTRIP,NLEV+1)
      DIMENSION U(ISTRIP,NLEV+1),V(ISTRIP,NLEV+1),QQ(ISTRIP,NLEV)
      DIMENSION PL(ISTRIP,NLEV),PLE(ISTRIP,NLEV+1),PLK(ISTRIP,NLEV)
      DIMENSION PLKE(ISTRIP,NLEV+1),DSIG(NLEV)
      DIMENSION IWATER(ISTRIP),Z0(ISTRIP),EVE(ISTRIP)
      DIMENSION HCAPG(ISTRIP),CLHG(ISTRIP),RADFLX(ISTRIP)
      real KMBG,KHBG
      LOGICAL QBEG,TPROF
C Diagnostic Variables
C --------------------
      DIMENSION SWINDS(istrip)
      DIMENSION    SRI(istrip,nlayr), ET(istrip,nlayr)
      DIMENSION    EU (istrip,nlayr)
      DIMENSION    WT (istrip,nlayr)
      DIMENSION    WSH(istrip,nlayr), WU(istrip,nlayr)
      DIMENSION    WV (istrip,nlayr), pbldpth(istrip)
      DIMENSION sustar(istrip),          sz0(istrip)
      DIMENSION    sct(istrip),          scu(istrip)
      dimension stu2m(istrip),stv2m(istrip),stt2m(istrip),stq2m(istrip)
      dimension stu10m(istrip),stv10m(istrip),
     1                              stt10m(istrip),stq10m(istrip)
      DIMENSION freqdg(istrip,nlayr-1)
C Dynamic Variables
C -----------------
      DIMENSION BETAEV (istrip),   AHS (istrip), HS(istrip)
      DIMENSION DTHR   (istrip),   XX  (istrip), YY(istrip), CU(istrip)
      DIMENSION CT     (istrip),  USTAR(istrip)
      DIMENSION RIB    (istrip),   ZETA(istrip), WS(istrip)
      DIMENSION DTHS   (istrip), DELTHS(istrip)
      DIMENSION DTHL   (istrip), DELTHL(istrip)
      DIMENSION TG     (istrip),    SHG(istrip)
      DIMENSION RIBIN  (istrip),   CUIN(istrip)
      DIMENSION CTIN   (istrip), ZETAIN(istrip)
      DIMENSION USTARIN(istrip), RHOSIN(istrip), Z0IN(istrip)
      DIMENSION    TMP1(istrip),   TMP2(istrip)
      DIMENSION    TMP3(istrip),  ITMP1(istrip)
      DIMENSION   ITMP2(istrip)
      dimension qqcolmin(istrip),qqcolmax(istrip),levpbl(istrip)
      equivalence ( betaev(1)  , wrkflx(1, 0*nlayr + 01) )
      equivalence ( ahs   (1)  , wrkflx(1, 0*nlayr + 02) )
      equivalence ( hs    (1)  , wrkflx(1, 0*nlayr + 03) )
      equivalence ( dthr  (1)  , wrkflx(1, 0*nlayr + 04) )
      equivalence ( xx    (1)  , wrkflx(1, 0*nlayr + 05) )
      equivalence ( yy    (1)  , wrkflx(1, 0*nlayr + 06) )
      equivalence ( cu    (1)  , wrkflx(1, 0*nlayr + 07) )
      equivalence ( ct    (1)  , wrkflx(1, 0*nlayr + 08) )
      equivalence ( ustar (1)  , wrkflx(1, 0*nlayr + 09) )
      equivalence ( rib   (1)  , wrkflx(1, 0*nlayr + 10) )
      equivalence ( zeta  (1)  , wrkflx(1, 0*nlayr + 11) )
      equivalence ( ws    (1)  , wrkflx(1, 0*nlayr + 12) )
      equivalence ( dths  (1)  , wrkflx(1, 0*nlayr + 13) )
      equivalence ( delths(1)  , wrkflx(1, 0*nlayr + 14) )
      equivalence ( dthl  (1)  , wrkflx(1, 0*nlayr + 15) )
      equivalence ( delthl(1)  , wrkflx(1, 0*nlayr + 16) )
      equivalence ( tg    (1)  , wrkflx(1, 0*nlayr + 17) )
      equivalence ( shg   (1)  , wrkflx(1, 0*nlayr + 18) )
      equivalence ( ribin (1)  , wrkflx(1, 0*nlayr + 19) )
      equivalence ( cuin  (1)  , wrkflx(1, 0*nlayr + 20) )
      equivalence ( ctin  (1)  , wrkflx(1, 0*nlayr + 21) )
      equivalence ( zetain(1)  , wrkflx(1, 0*nlayr + 22) )
      equivalence ( ustarin(1) , wrkflx(1, 0*nlayr + 23) )
      equivalence ( rhosin (1) , wrkflx(1, 0*nlayr + 24) )
      equivalence ( z0in   (1) , wrkflx(1, 0*nlayr + 25) )
      equivalence ( tmp1   (1) , wrkflx(1, 0*nlayr + 26) )
      equivalence ( tmp2   (1) , wrkflx(1, 0*nlayr + 27) )
      equivalence ( tmp3   (1) , wrkflx(1, 0*nlayr + 28) )
      equivalence (itmp1   (1) , wrkflx(1, 0*nlayr + 29) )
      equivalence (itmp2   (1) , wrkflx(1, 0*nlayr + 30) )
C Dynamic Variables
C -----------------
      DIMENSION ADZ1   (istrip,nlayr  ), DZ1TMP(istrip,nlayr)
      DIMENSION DZ3    (istrip,nlayr  ), TEMP  (istrip,nlayr)
      DIMENSION DV     (istrip,nlayr  ), DTHV  (istrip,nlayr)
      DIMENSION DPK    (istrip,nlayr  ), STRT  (istrip,nlayr)
      DIMENSION DW2    (istrip,nlayr  ), RI    (istrip,nlayr)
      DIMENSION RHOZPK (istrip,nlayr  ), Q     (istrip,nlayr)
      DIMENSION RIINIT (istrip,nlayr  ), DU    (istrip,nlayr)
      DIMENSION QQINIT (istrip,nlayr  ), RHOKDZ(istrip,nlayr)
      DIMENSION RHODZ2 (istrip,nlayr  )
      REAL      KM     (istrip,nlayr  ),     KH(istrip,nlayr)
      equivalence ( adz1  (1,1), wrkflx(1, 0*nlayr + 31) )
      equivalence ( dz1tmp(1,1), wrkflx(1, 1*nlayr + 31) )
      equivalence ( dz3   (1,1), wrkflx(1, 2*nlayr + 31) )
      equivalence ( temp  (1,1), wrkflx(1, 3*nlayr + 31) )
      equivalence ( dv    (1,1), wrkflx(1, 4*nlayr + 31) )
      equivalence ( dthv  (1,1), wrkflx(1, 5*nlayr + 31) )
      equivalence ( dpk   (1,1), wrkflx(1, 6*nlayr + 31) )
      equivalence ( strt  (1,1), wrkflx(1, 7*nlayr + 31) )
      equivalence ( dw2   (1,1), wrkflx(1, 8*nlayr + 31) )
      equivalence ( ri    (1,1), wrkflx(1, 9*nlayr + 31) )
      equivalence ( rhozpk(1,1), wrkflx(1,10*nlayr + 31) )
      equivalence ( q     (1,1), wrkflx(1,11*nlayr + 31) )
      equivalence ( riinit(1,1), wrkflx(1,12*nlayr + 31) )
      equivalence ( du    (1,1), wrkflx(1,13*nlayr + 31) )
      equivalence ( qqinit(1,1), wrkflx(1,14*nlayr + 31) )
      equivalence ( rhokdz(1,1), wrkflx(1,15*nlayr + 31) )
      equivalence ( rhodz2(1,1), wrkflx(1,16*nlayr + 31) )
      equivalence ( km    (1,1), wrkflx(1,17*nlayr + 31) )
      equivalence ( kh    (1,1), wrkflx(1,18*nlayr + 31) )
      DIMENSION DELTH  (istrip,nlayr+1), DELSH (istrip,nlayr+1)
      DIMENSION FLXFAC (istrip,nlayr+1), DTHG  (istrip,nlayr+1)
      DIMENSION FLXFPK (istrip,nlayr+1)
      equivalence ( delth (1,1), wrkflx(1,19*nlayr + 31) )
      equivalence ( delsh (1,1), wrkflx(1,20*nlayr + 32) )
      equivalence ( flxfac(1,1), wrkflx(1,21*nlayr + 33) )
      equivalence ( dthg  (1,1), wrkflx(1,22*nlayr + 34) )
      equivalence ( flxfpk(1,1), wrkflx(1,23*nlayr + 35) )
      DIMENSION ADZ2   (istrip,nlayr-1), RHODZ1(istrip,nlayr-1)
      DIMENSION VKZE   (istrip,nlayr-1), VKZM  (istrip,nlayr-1)
      DIMENSION XL     (istrip,nlayr-1), QXLM  (istrip,nlayr-1)
      DIMENSION QQE    (istrip,nlayr-1), QE    (istrip,nlayr-1)
      DIMENSION P3     (istrip,nlayr-1), XQ    (istrip,nlayr-1)
      DIMENSION XLDIAG (istrip,nlayr-1), FLXFCE(istrip,nlayr-1)
      equivalence ( adz2  (1,1), wrkflx(1,24*nlayr + 36) )
      equivalence ( rhodz1(1,1), wrkflx(1,25*nlayr + 35) )
      equivalence ( vkze  (1,1), wrkflx(1,26*nlayr + 34) )
      equivalence ( vkzm  (1,1), wrkflx(1,27*nlayr + 33) )
      equivalence ( xl    (1,1), wrkflx(1,28*nlayr + 32) )
      equivalence ( qxlm  (1,1), wrkflx(1,29*nlayr + 31) )
      equivalence ( qqe   (1,1), wrkflx(1,30*nlayr + 30) )
      equivalence ( qe    (1,1), wrkflx(1,31*nlayr + 29) )
      equivalence ( p3    (1,1), wrkflx(1,32*nlayr + 28) )
      equivalence ( xq    (1,1), wrkflx(1,33*nlayr + 27) )
      equivalence ( xldiag(1,1), wrkflx(1,34*nlayr + 26) )
      equivalence ( flxfce(1,1), wrkflx(1,35*nlayr + 25) )
      LOGICAL LUFU,LVFU,LEFU,LHFU,LUFS,LVFS,LEFS,LHFS,LEU,LET,LCU,LCT
      LOGICAL FIRST,LAST
      DIMENSION IBITSTB(ISTRIP,NLAYR),IBITSTR(ISTRIP),INTQ(ISTRIP,NLAYR)
c
      AITR   = ONE / FLOAT(ITRTRB)
      ISTNLV = ISTRIP * NLEV
      NLEVM1 = NLEV - 1
      NLEVM2 = NLEV - 2
      NLEVP1 = NLEV + 1
      ISTNM1 = ISTRIP * NLEVM1
      ISTNM2 = ISTRIP * NLEVM2
      ISTNP1 = ISTRIP * NLEVP1
      GBYCP  = GRAV / CP
C CALL POINT BY POINT DIAGNOSTIC ROUTINE FOR 'BEFORE' VALUES
C ----------------------------------------------------------
      IF(TPROF) THEN
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,PL,'MID PRESSURE    ')
      CALL PNTPRF (NN,ISTRIP,NLEV+1,NYMD,NHMS,PLE,'EDGE PRESSURE   ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,PLK,'MID P ** KAPPA  ')
      CALL PNTPRF (NN,ISTRIP,NLEV+1,NYMD,NHMS,PLKE,'EDGE P ** KAPPA ')
      CALL PNTPRF (NN,ISTRIP,NLEV+1,NYMD,NHMS,TH,'THETA BEFORE    ')
      CALL PNTPRF (NN,ISTRIP,NLEV+1,NYMD,NHMS,THV,'THETA V BEFORE  ')
      CALL PNTPRF (NN,ISTRIP,NLEV+1,NYMD,NHMS,SH,'Q BEFORE        ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,U,'U WIND BEFORE   ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,V,'V WIND BEFORE   ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,HCAPG,'HEAT CAP GROUND ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,CLHG,'LAT HEAT AT GRND')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,RADFLX,'NET SFC RAD     ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,KHBG,'BACKGR HEAT TRAN')
      ENDIF
C SET INITIAL NUMBER OF ITERATIONS OF SFCFLX
C ------------------------------------------
      N = 2
C DETERMINE IF INITIAL START
C --------------------------
      INIT = 0
      IF(QBEG) INIT = 1
C SET DIAGNOSTIC LOGICALS AND INITIALIZE DIAGNOSTIC ARRAYS
C --------------------------------------------------------
      LHFU = .FALSE.
      LEFU = .FALSE.
      LUFU = .FALSE.
      LVFU = .FALSE.
      LHFS = .FALSE.
      LEFS = .FALSE.
      LUFS = .FALSE.
      LVFS = .FALSE.
      LEU  = .FALSE.
      LET  = .FALSE.
      LCU  = .FALSE.
      LCT  = .FALSE.
      IF((ITTFLUX.NE.0).OR.TPROF)       LHFU = .TRUE.
      IF((ITQFLUX.NE.0).OR.TPROF)       LEFU = .TRUE.
      IF((ITUFLUX.NE.0).OR.TPROF)       LUFU = .TRUE.
      IF((ITVFLUX.NE.0).OR.TPROF)       LVFU = .TRUE.
      IF((IEU    .NE.0).OR.TPROF)       LEU  = .TRUE.
      IF((IET    .NE.0).OR.TPROF)       LET  = .TRUE.
      IF((ICU    .NE.0).OR.TPROF)       LCU  = .TRUE.
      IF((ICT    .NE.0).OR.TPROF)       LCT  = .TRUE.
      IF((IHFLUX .NE.0).AND.(.NOT.LHFU)) LHFS = .TRUE.
      IF((IEFLUX .NE.0).AND.(.NOT.LEFU)) LEFS = .TRUE.
      IF((IEVAP  .NE.0).AND.(.NOT.LEFU)) LEFS = .TRUE.
      IF((IUFLUX .NE.0).AND.(.NOT.LUFU)) LUFS = .TRUE.
      IF((IVFLUX .NE.0).AND.(.NOT.LVFU)) LVFS = .TRUE.
      if (lhfu) then
      do I =1,istnlv
      wt(i,1) = zero
      enddo
      endif
      if (lefu) then
      do I =1,istnlv
      wsh(i,1) = zero
      enddo
      endif
      if (lufu) then
      do I =1,istnlv
      wu(i,1) = zero
      enddo
      endif
      if (lvfu) then
      do I =1,istnlv
      wv(i,1) = zero
      enddo
      endif
      if (leu) then
      do I =1,istnlv
      eu(i,1) = zero
      enddo
      endif
      if (let) then
      do I =1,istnlv
      et(i,1) = zero
      enddo
      endif
      if (tprof) then
      DO I =1,ISTNM1
      XLDIAG(I,1) = ZERO
      enddo
      endif
      if (lhfs) then
      do I =1,ISTRIP
      wt(i,nlev) = zero
      enddo
      endif
      if (lefs) then
      do I =1,ISTRIP
      wsh(i,nlev) = zero
      enddo
      endif
      if (lufs) then
      do I =1,ISTRIP
      wu(i,nlev) = zero
      enddo
      endif
      if (lvfs) then
      do I =1,ISTRIP
      wv(i,nlev) = zero
      enddo
      endif
      if (lcu) then
      do I =1,ISTRIP
      scu(i) = zero
      enddo
      endif
      if (lct) then
      do I =1,ISTRIP
      sct(i) = zero
      enddo
      endif
      if (ipbl.ne.0) then
      do I =1,ISTRIP
      pbldpth(i) = zero
      enddo
      endif
      if (iustar.ne.0) then
      do I =1,ISTRIP
      sustar(i) = zero
      enddo
      endif
      if (iz0.ne.0) then
      do I =1,ISTRIP
      sz0(i) = zero
      enddo
      endif
      IF(ifrqtrb.NE.0) THEN
      do I =1,ISTNM1
      FREQDG(I,1) = ZERO
      enddo
      ENDIF
      if (iu2m.ne.0) then
      do I =1,ISTRIP
      stu2m(i) = zero
      enddo
      endif
      if (iv2m.ne.0) then
      do I =1,ISTRIP
      stv2m(i) = zero
      enddo
      endif
      if (it2m.ne.0) then
      do I =1,ISTRIP
      stt2m(i) = zero
      enddo
      endif
      if (iq2m.ne.0) then
      do I =1,ISTRIP
      stq2m(i) = zero
      enddo
      endif
      if (iu10m.ne.0) then
      do I =1,ISTRIP
      stu10m(i) = zero
      enddo
      endif
      if (iv10m.ne.0) then
      do I =1,ISTRIP
      stv10m(i) = zero
      enddo
      endif
      if (it10m.ne.0) then
      do I =1,ISTRIP
      stt10m(i) = zero
      enddo
      endif
      if (iq10m.ne.0) then
      do I =1,ISTRIP
      stq10m(i) = zero
      enddo
      endif
C COMPUTE VERTICAL GRID
C ---------------------
      DO 9038 I =1,ISTNLV
      ADZ1(I,1) = (CP/GRAV)*(PLKE(I,2)-PLKE(I,1))
      ADZ1(I,1) = THV(I,1) * ADZ1(I,1)
      DZ1TMP(I,1) = ADZ1(I,1)
9038  CONTINUE
      DO 9040 I =1,ISTNM1
      ADZ2(I,1) = AHALF * (ADZ1(I,1)+ADZ1(I,2))
9040  CONTINUE
C DEPTH HS OF SURFACE LAYER
C -------------------------
      DO 9042 I =1,ISTRIP
      HS(I) = AHALF * ADZ1(I,NLEV)
9042  CONTINUE
C ALPHA * LAYER DEPTHS FOR TRBLEN
C -------------------------------
      DO 9044 I =1,ISTRIP
      DZ3(I,1) = HALPHA * ADZ1(I,1)
9044  CONTINUE
      DO 9046 I =1,ISTNM2
      DZ3(I,2) = ALPHA * ADZ1(I,2)
9046  CONTINUE
      DO 9048 I =1,ISTRIP
      DZ3(I,NLEV) = ALPHA * HS(I)
9048  CONTINUE
C VK * HEIGHTS AT MID AND EDGE LEVELS
C -----------------------------------
      DO 9050 I =1,ISTNM1
      TEMP(I,2) = VK * ADZ1(I,2)
9050  CONTINUE
      DO 9052 I =1,ISTRIP
      VKZE(I,NLEVM1) = TEMP(I,NLEV)
9052  CONTINUE
      DO 100 LL = 2,NLEVM1
      L = NLEV - LL
      LP1 = L + 1
      DO 9054 I =1,ISTRIP
      VKZE(I,L) = VKZE(I,LP1) + TEMP(I,LP1)
9054  CONTINUE
 100  CONTINUE
      DO 9056 I =1,ISTNM1
      VKZM(I,1) = VKZE(I,1) - AHALF * TEMP(I,2)
9056  CONTINUE
C COMPUTE RHO BY DZ AT MID AND EDGE LEVELS
C ----------------------------------------
      DO 200 L = 1,NLEVM1
       LP1 = L + 1
       FAC1 = DSIG(L) / ( DSIG(L) + DSIG(LP1) )
       FAC2 = ONE - FAC1
      DO 9058 I =1,ISTRIP
       RHODZ2(I,L) = FAC1 * THV(I,LP1)
       RHODZ2(I,L) = RHODZ2(I,L) + FAC2 * THV(I,L)
9058  CONTINUE
 200  CONTINUE
      DO 9060 I =1,ISTNM1
       RHODZ2(I,1) = (RGAS*ZP01) * RHODZ2(I,1)
       TEMP(I,1)   = PLKE(I,2) * ADZ2(I,1)
       RHODZ2(I,1) = TEMP(I,1) * RHODZ2(I,1)
       RHODZ2(I,1) = PLE(I,2) / RHODZ2(I,1)
       RHOZPK(I,1) = RHODZ2(I,1) * PLKE(I,2)
       RHODZ1(I,1) = (RGAS*ZP01) * THV(I,2)
       TEMP(I,1)   = PLK(I,2) * ADZ1(I,2)
       RHODZ1(I,1) = TEMP(I,1) * RHODZ1(I,1)
       RHODZ1(I,1) = PL(I,2) / RHODZ1(I,1)
9060  CONTINUE
C COMPUTE FLXFAC FOR LAYERS AND EDGES
C COMPUTE DTG / DT DUE TO RADIATION AND HEAT CONDUCTION THROUGH ICE
C -----------------------------------------------------------------
      DO 9062 I =1,ISTNLV
      FLXFPK(I,1) = PLE(I,2) - PLE(I,1)
      FLXFPK(I,1) = FLXFPK(I,1) * PLK(I,1)
      FLXFPK(I,1) = (GRAV*DTAU*ZP01) / FLXFPK(I,1)
9062  CONTINUE
      DO 9064 I =1,ISTRIP
      FLXFPK(I,NLEVP1) = ZERO
9064  CONTINUE
      DO 9066 I =1,ISTRIP
      IF (IWATER(I).EQ.0 ) FLXFPK(I,NLEVP1) =
     1   DTAU / ( HCAPG(I) * PLKE(I,NLEVP1) )
9066  CONTINUE
      DO 9068 I =1,ISTNLV
      FLXFAC(I,1) = FLXFPK(I,1) * PLK(I,1)
9068  CONTINUE
      DO 9070 I =1,ISTRIP
      FLXFAC(I,NLEVP1) = CLHG(I) * FLXFPK(I,NLEVP1)
9070  CONTINUE
      DO 9072 I =1,ISTRIP
      DTHR(I) = FLXFPK(I,NLEVP1) * RADFLX(I)
9072  CONTINUE
      DO 9074 I =1,ISTRIP
      FLXFPK(I,NLEVP1) = CP * FLXFPK(I,NLEVP1)
9074  CONTINUE
      DO 9076 I =1,ISTNM1
      FLXFCE(I,1) = PL(I,2) - PL(I,1)
9076  CONTINUE
      DO 9078 I =1,ISTNM1
      FLXFCE(I,1) = (GRAV*DTAU*ZP01) / FLXFCE(I,1)
9078  CONTINUE
      DO 9080 I =1,ISTRIP
      BETAEV(I) = EVE(I)
9080  CONTINUE
      DO 9082 I =1,ISTRIP
      IF(SH(I,NLEV).GT.SH(I,NLEVP1)) BETAEV(I) = ONE
9082  CONTINUE
C COMPUTE RECIPROCALS OF DZ1, DZ2, HS
C -----------------------------------
      DO 9084 I =1,ISTNLV
      ADZ1(I,1) = ONE / ADZ1(I,1)
9084  CONTINUE
      DO 9086 I =1,ISTNM1
      ADZ2(I,1) = ONE / ADZ2(I,1)
9086  CONTINUE
      DO 9088 I =1,ISTRIP
      AHS(I) = ONE / HS(I)
9088  CONTINUE
C COMPUTE GRADIENTS OF P**KAPPA
C -----------------------------
      DO 9090 I =1,ISTNM1
      DPK(I,1) = ( PLK(I,2)-PLK(I,1) ) * ADZ2(I,1)
9090  CONTINUE
      DO 9092 I =1,ISTRIP
      DPK(I,NLEV) = GBYCP / THV(I,NLEV)
9092  CONTINUE
C INITIALIZE Q ARRAY
C ------------------
      DO 9094 I =1,ISTNM1
      Q(I,1) = TWO * QQ(I,1)
      Q(I,1) = SQRT( Q(I,1) )
9094  CONTINUE
      FIRST = .TRUE.
      LAST = .FALSE.
C**********************************************************************
C**********************************************************************
C                              MAIN LOOP
C
      DO 2000 ITER = 1, ITRTRB
C
      IF ( ITER .GE. ITRTRB ) LAST = .TRUE.
C
C   SURFACE FLUX TRANSFER COEFFICIENTS
C
CI
       CALL SFCFLX(NN,U(1,NLEV),V(1,NLEV),
     1   THV(1,NLEV),
     2   THV(1,NLEVP1),TH(1,NLEV),TH(1,NLEVP1),
     3   SH(1,NLEV),SH(1,NLEVP1),PLK(1,NLEV),
     4   PLKE(1,NLEVP1),PLE(1,NLEVP1),Z0,
     5   IWATER,HS,AHS,BETAEV,
     6   FIRST,LAST,N,ISTRIP,aitr,RHODZ2(1,NLEV),RHOZPK(1,NLEV),
     7   KH(1,NLEV),KM(1,NLEV),USTAR,
     8   XX,YY,CU,
     9   CT,RIB,ZETA,WS,
     1   stu2m,stv2m,stt2m,stq2m,stu10m,stv10m,stt10m,stq10m,
     2   iu2m,iv2m,it2m,iq2m,iu10m,iv10m,it10m,iq10m,cp,rgas,undef)
CI
C
       N = 1
C
C   SET VALUES OF TURBULENT VELOCITY AND KINETIC ENERGY AT THE GROUND
C
CB
      DO 9098 I =1,ISTRIP
      Q(I,NLEV) = QBUSTR * USTAR(I)
      QQ(I,NLEV) = AHALF * Q(I,NLEV) * Q(I,NLEV)
9098  CONTINUE
CE
C
C GRADIENTS
C ---------
      DO 9100 I =1,ISTNM1
        DU(I,1) = (   U(I,1)-  U(I,2) ) * ADZ2(I,1)
        DV(I,1) = (   V(I,1)-  V(I,2) ) * ADZ2(I,1)
      DTHV(I,1) = ( THV(I,1)-THV(I,2) ) * ADZ2(I,1)
9100  CONTINUE
C GRADIENTS AT THE TOP OF THE SURFACE LAYER
C -----------------------------------------
      DO 9102 I =1,ISTRIP
      DU(I,NLEV) = CU(I)*XX(I)*AHS(I)*RVK
      DV(I,NLEV) = V(I,NLEV) * DU(I,NLEV)
      DU(I,NLEV) = U(I,NLEV) * DU(I,NLEV)
      DTHV(I,NLEV) = CT(I) * YY(I) *
     1 ((THV(I,NLEV)-THV(I,NLEVP1)) * RVK)* AHS(I)
9102  CONTINUE
C CALCULATE BRUNT-VAISALA FREQUENCIES, SHEARS, RICHARDSON NUMBERS
C ---------------------------------------------------------------
      DO 9104 I =1,ISTNLV
      STRT(I,1) = CP * DTHV(I,1) * DPK(I,1)
      DW2(I,1) = DU(I,1) * DU(I,1) + DV(I,1) * DV(I,1)
      IF ( DW2(I,1) .LE. Z1PEM4 ) DW2(I,1) = Z1PEM4
      RI(I,1) = STRT(I,1) / DW2(I,1)
9104  CONTINUE
C FILL RICHARDSON NUMBER AND SURFACE WIND DIAGNOSTICS
C       (THOSE NEEDED FROM FIRST TRBFLX ITERATION)
C ---------------------------------------------------
      DO 9106 I =1,ISTNM1
      SRI(I,1) = RI(I,1)
9106  CONTINUE
      DO 9108 I =1,ISTRIP
      SRI(I,NLEV) = RIB(I)
9108  CONTINUE
      DO 9110 I =1,ISTRIP
      SWINDS(I) = WS(I)
9110  CONTINUE
C INITIALIZE KH, KM, QE AND P3 AND ELIMINATE SMALL QQ
C ---------------------------------------------------
      DO 9112 I =1,ISTNM1
      KH(I,1) = ZERO
      KM(I,1) = ZERO
      QQE(I,1) = ZERO
      QE(I,1) = ZERO
      P3(I,1) = ZERO
9112  CONTINUE
      DO 9414 I = 1,ISTNM1
       IBITSTB(I,1) = 0
 9414 CONTINUE
      DO 9314 I = 1,ISTNM1
       IF ( QQ(I,1) .GT. Z1PEM8 ) THEN
        INTQ(I,1) = 1
       ELSE
        INTQ(I,1) = 0
       ENDIF
 9314 CONTINUE
      DO 9114 I = 1,ISTNM1
       IF ( QQ(I,1).LE.Z1PEM8 ) THEN
        QQ(I,1) = ZERO
        Q(I,1) = ZERO
       ENDIF
 9114 CONTINUE
C
       DO 300 LMINQ = 1,NLEVM1
        IBIT = 0
        DO 9116 I = 1,ISTRIP
         IF ( QQ(I,LMINQ).GT.Z1PEM8 ) IBIT = IBIT + 1
 9116   CONTINUE
        IF(IBIT.GE.1)GO TO 310
 300   CONTINUE
       LMINQ = NLEV-1
 310   CONTINUE
       LMINQ1 = 1
       IF(LMINQ.GT.1)LMINQ1 = LMINQ - 1
C LENGTH SCALE
C ------------
       CALL TRBLEN(STRT,DW2,DZ3,Q,VKZE,VKZM,DTHV,DPK,DU,DV,XL,QXLM,
     1  NLEV,INIT,LMIN,LMINQ,LMINQ1,CP)
C QE AND DIMENSIONLESS COEFFS FROM LEVEL 2 MODEL
C ----------------------------------------------
      IF( LMIN .LT. NLEV ) THEN
       NLEVML = NLEV - LMIN
        CALL TRBL20(RI(1,LMIN),STRT(1,LMIN),DW2(1,LMIN),XL(1,LMIN),
     1   KM(1,LMIN),KH(1,LMIN),QE(1,LMIN),QQE(1,LMIN),IBITSTB(1,LMIN),
     2   NLEVML)
      ENDIF
C FOR INITIAL START ONLY : USE EQUILIBRIUM MODEL
C ----------------------------------------------
      IF ( INIT .EQ. 1 ) THEN
      DO 9180 I =1,ISTNM1
       QQ(I,1) = QQE(I,1)
       Q(I,1) = QE(I,1)
9180  CONTINUE
       INIT = 2
        CALL TRBLEN(STRT,DW2,DZ3,Q,VKZE,VKZM,DTHV,DPK,DU,DV,XL,QXLM,
     1    NLEV,INIT,LMIN,LMINQ,LMINQ1,CP)
       INIT = 0
       GO TO 550
      ENDIF
C DIMENSIONLESS COEFFS AND P3 (Q LE QE)
C -------------------------------------
      IF( LMIN .LT. NLEV ) THEN
       ISTNML = ISTRIP * NLEVML
       DO 9320 I = 1,ISTNML
        IF (  (IBITSTB(I,LMIN).EQ.1)  .AND.
     1            ( Q(I,LMIN) .LE. QE(I,LMIN) ) ) THEN
         IBITSTB(I,LMIN) = 1
        ELSE
         IBITSTB(I,LMIN) = 0
        ENDIF
 9320  CONTINUE
       DO 9220 I = 1,ISTNML
        IF(IBITSTB(I,LMIN).EQ.1 ) THEN
         TEMP(I,LMIN) = Q(I,LMIN) / QE(I,LMIN)
         KH(I,LMIN) = TEMP(I,LMIN) * KH(I,LMIN)
         KM(I,LMIN) = TEMP(I,LMIN) * KM(I,LMIN)
        ENDIF
        TEMP(I,LMIN) = ZP01 * QQE(I,LMIN)
        IF((IBITSTB(I,LMIN).EQ.1) .AND.
     1        ( QQ(I,LMIN) .LE. TEMP(I,LMIN) )) THEN
         QQ(I,LMIN) = TEMP(I,LMIN)
         Q(I,LMIN) = ZP1 * QE(I,LMIN)
        ENDIF
        IF(IBITSTB(I,LMIN).EQ.1 ) P3(I,LMIN) = (TWO*B3) *
     1            ( QE(I,LMIN) - Q(I,LMIN) )
 9220  CONTINUE
      ENDIF
C DIMENSIONLESS COEFFS AND P3 (Q GT QE)
C -------------------------------------
       NLEVML = NLEV - LMINQ
       CALL TRBL25(Q(1,LMINQ),XL(1,LMINQ),STRT(1,LMINQ),DW2(1,LMINQ),
     1    IBITSTB(1,LMINQ),INTQ(1,LMINQ),KM(1,LMINQ),KH(1,LMINQ),
     2    P3(1,LMINQ),NLEVML)
C CALCULATE SOURCE TERM P3
C ------------------------
      IF ( LMINQ .LT. LMIN ) THEN
       LMIN = LMINQ
       ISTNML = ISTRIP * ( NLEV - LMIN )
      ENDIF
      IF( LMIN .LT. NLEV ) THEN
      DO 9122 I =1,ISTNML
       P3(I,LMIN) = P3(I,LMIN) * DTAU / XL(I,LMIN)
       TEMP(I,LMIN) = QQE(I,LMIN) * P3(I,LMIN)
       XQ(I,LMIN) = QQE(I,LMIN) - QQ(I,LMIN)
9122  CONTINUE
       DO 9216 I = 1,ISTNML
        IF( ( (IBITSTB(I,LMIN).EQ.1) .AND.
     1            ( XQ(I,LMIN) .LT. TEMP(I,LMIN) ) )
     2                  .OR.
     3 ( (IBITSTB(I,LMIN).EQ.0) .AND.
     4            ( XQ(I,LMIN) .GT. TEMP(I,LMIN) ) ) )
     5  P3(I,LMIN) = XQ(I,LMIN) / QQE(I,LMIN)
 9216  CONTINUE
      ENDIF
 550  CONTINUE
C DIAGNOSTIC PROFILES : INITIAL RI AND QQ
C ---------------------------------------
      IF ( TPROF .AND. FIRST ) THEN
      DO 9118 I =1,ISTRIP
       RIBIN(I) = RIB(I)
       CUIN(I) = CU(I)
       CTIN(I) = CT(I)
       USTARIN(I) = USTAR(I)
       RHOSIN(I) = RHODZ2(I,NLEV)
       Z0IN(I) = Z0(I)
       ZETAIN(I) = ZETA(I)
9118  CONTINUE
      DO 9120 I =1,ISTNLV
       RIINIT(I,1) = RI(I,1)
       QQINIT(I,1) = QQ(I,1)
9120  CONTINUE
      ENDIF
C UPDATE TURBULENT KINETIC ENERGY QQ
C ----------------------------------
      NLEVMQ = NLEV - LMINQ1
      ISTNMQ = ISTRIP * NLEVMQ
      DO 9306 I =1,ISTNMQ
      RHOKDZ(I,LMINQ1) = RHODZ1(I,LMINQ1)
     1      * QXLM(I,LMINQ1)
9306  CONTINUE
       CALL TRBDIF(QQ(1,LMINQ1),P3(1,LMINQ1),RHOKDZ(1,LMINQ1),
     1       FLXFCE(1,LMINQ1),DTHS,DELTHS,NLEVMQ,1,Z1PEM20)
C
C   DIMENSIONAL DIFFUSION COEFFS INCLUDING BACKGROUND AMOUNTS
C
      IF(LMINQ1.GT.1)THEN
       ISTLMQ = ISTRIP * (LMINQ1-1)
CB
      DO 9124 I =1,ISTLMQ
       KM(I,1) = KMBG
       KH(I,1) = KHBG
9124  CONTINUE
CE
      ENDIF
C
CB
      DO 9126 I =1,ISTNMQ
      Q(I,LMINQ1) = TWO * QQ(I,LMINQ1)
      Q(I,LMINQ1) = SQRT(Q(I,LMINQ1))
      XQ(I,LMINQ1) = XL(I,LMINQ1) * Q(I,LMINQ1)
      KM(I,LMINQ1)=XQ(I,LMINQ1)*KM(I,LMINQ1)+KMBG
      KH(I,LMINQ1)=XQ(I,LMINQ1)*KH(I,LMINQ1)+KHBG
9126  CONTINUE
CE
C
C   CALCULATE INTERNAL FLUXES AND UPDATE PROGNOSTIC VARIABLES: TH AND S
C
      DO 9128 I =1,ISTNLV
      TEMP(I,1) = RHOZPK(I,1) * KH(I,1)
9128  CONTINUE
      DO 9130 I =1,ISTNLV
      DELTH(I,1) = ZERO
9130  CONTINUE
      DO 9132 I =1,ISTRIP
      DELTH(I,NLEVP1) = ONE
9132  CONTINUE
       CALL TRBDIF(TH,DELTH,TEMP,FLXFPK,DTHS,DELTHS,NLEV,2,ZERO)
      DO 9134 I =1,ISTNLV
      RHOKDZ(I,1) = RHODZ2(I,1) * KH(I,1)
9134  CONTINUE
      DO 9136 I =1,ISTRIP
      RHOKDZ(I,NLEV) = BETAEV(I)*RHOKDZ(I,NLEV)
9136  CONTINUE
C
      DO 9138 I =1,ISTNLV
      DELSH(I,1) = ZERO
9138  CONTINUE
      DO 9140 I =1,ISTRIP
      DELSH(I,NLEVP1) = FACEPS * SH(I,NLEVP1) /
     1 ( TH(I,NLEVP1)*TH(I,NLEVP1)*PLKE(I,NLEVP1) )
9140  CONTINUE
       CALL TRBDIF(SH,DELSH,RHOKDZ,FLXFAC,DTHL,DELTHL,NLEV,2,ZERO)
C
C   UPDATE THG AND THE TH AND SH PROFILES
C
      DO 9142 I =1,ISTRIP
      DTHG(I,2) = ONE - DELTHS(I) - DELTHL(I)
9142  CONTINUE
      DO 9144 I =1,ISTRIP
      DTHG(I,1) = (DTHR(I)+DTHS(I)+DTHL(I))
     1      / DTHG(I,2)
9144  CONTINUE
      DO 600 L = 2,NLEVP1
      DO 9146 I =1,ISTRIP
       DTHG(I,L) = DTHG(I,1)
9146  CONTINUE
 600  CONTINUE
      DO 9148 I =1,ISTNP1
      TH(I,1) = TH(I,1) +
     1      DELTH(I,1) * DTHG(I,1)
9148  CONTINUE
      DO 9150 I =1,ISTNP1
      SH(I,1) = SH(I,1) +
     1      DELSH(I,1) * DTHG(I,1)
9150  CONTINUE
C
C ( FILL DIAGNOSTIC ARRAYS IF REQUIRED )
C
      DO 9152 I =1,ISTNLV
      IF(LHFU) WT(I,1) = WT(I,1) + TEMP(I,1) *
     1                            ( TH(I,2) - TH(I,1) )
9152  CONTINUE
      DO 9154 I =1,ISTNLV
      IF(LEFU) WSH(I,1) = WSH(I,1) + RHOKDZ(I,1) *
     1                            ( SH(I,2) - SH(I,1) )
9154  CONTINUE
      DO 9156 I =1,ISTRIP
      IF(LHFS) WT(I,NLEV) = WT(I,NLEV) +
     1 TEMP(I,NLEV) * ( TH(I,NLEVP1)-TH(I,NLEV) )
9156  CONTINUE
      DO 9158 I =1,ISTRIP
      IF(LEFS) WSH(I,NLEV) = WSH(I,NLEV) +
     1 RHOKDZ(I,NLEV) * ( SH(I,NLEVP1)-SH(I,NLEV) )
9158  CONTINUE
C
C   COMPUTE NEW VALUES OF THV AND SH(NLEVP1)
C
      IF(TPROF.OR.(.NOT.LAST)) THEN
      DO 9160 I =1,ISTRIP
       TG(I) = TH(I,NLEVP1) * PLKE(I,NLEVP1)
9160  CONTINUE
CI
      CALL QSTARS (TG,PLE(1,NLEVP1),TMP1,TMP2,TMP3,ITMP1,ITMP2,
     1     ISTRIP,SHG)
CI
      DO 9162 I =1,ISTRIP
       BETAEV(I) = EVE(I)
9162  CONTINUE
      DO 9164 I =1,ISTRIP
       IF(SHG(I).LE.SH(I,NLEV)) BETAEV(I)= ONE
9164  CONTINUE
      DO 9166 I =1,ISTRIP
       SH(I,NLEVP1) = SH(I,NLEV) + BETAEV(I) *
     1              ( SHG(I) - SH(I,NLEV) )
9166  CONTINUE
      DO 9168 I =1,ISTNP1
       THV(I,1) = TH(I,1) * (ONE+VIRTCON*SH(I,1))
9168  CONTINUE
      DO 9170 I =1,ISTRIP
       SH(I,NLEVP1) = SHG(I)
9170  CONTINUE
      ENDIF
C
C   CALCULATE INTERNAL FLUXES AND UPDATE PROGNOSTIC VARIABLES: U AND V
C
      DO 9172 I =1,ISTNLV
      RHOKDZ(I,1) = RHODZ2(I,1) * KM(I,1)
9172  CONTINUE
       CALL TRBDIF(U,V,RHOKDZ,FLXFAC,DTHS,DELTHS,NLEV,3,ZERO)
C ( FILL DIAGNOSTIC ARRAYS IF REQUIRED )
      if(lufu) then
      DO 9174 I =1,ISTNLV
      WU(I,1) = WU(I,1) + RHOKDZ(I,1) * ( U(I,2) - U(I,1) )
9174  CONTINUE
      endif
      if(lvfu) then
      DO 9176 I =1,ISTNLV
      WV(I,1) = WV(I,1) + RHOKDZ(I,1) * ( V(I,2) - V(I,1) )
9176  CONTINUE
      endif
      if(lufs) then
      DO 9178 I =1,ISTRIP
      WU(I,NLEV) = WU(I,NLEV) - RHOKDZ(I,NLEV) * U(I,NLEV)
9178  CONTINUE
      endif
      if(lvfs) then
      DO 9182 I =1,ISTRIP
      WV(I,NLEV) = WV(I,NLEV) - RHOKDZ(I,NLEV) * V(I,NLEV)
9182  CONTINUE
      endif
      IF(ifrqtrb.NE.0) THEN
      DO 9300 I = 1,ISTNM1
      IF ( QQ(I,1) .GT. QQMIN ) THEN
         IBITSTB(I,1) = 1
         ELSE
         IBITSTB(I,1) = 0
      ENDIF
      IF( IBITSTB(I,1).EQ.1 ) FREQDG(I,1) = FREQDG(I,1) + aitr
 9300 CONTINUE
      ENDIF
      IF(ipbl.NE.0) THEN
       do i = 1,istrip
        qqcolmin(i) = qq(i,nlev)*0.1
        qqcolmax(i) = qq(i,nlev)
        levpbl(i) = nlev
       enddo
       DO L = nlev-1,1,-1
        DO I = 1,ISTRIP
         IF ( (qq(i,l).gt.qqcolmax(I)).and.(levpbl(i).eq.nlev))then
          qqcolmax(i) = qq(i,l)
          qqcolmin(i) = 0.1*qqcolmax(I)
         endif
         if((qq(i,l).lt.qqcolmin(i)).and.(levpbl(i).eq.nlev))
     1                                                levpbl(i)=l
        enddo
       enddo
       do i = 1,istrip
        lp = levpbl(i)
        if(lp.lt.nlev)then
         pbldpth(I) = pbldpth(I) + ( (PLE(I,nlev+1)-PLE(I,Lp+2)) +
     1    ( (ple(i,lp+2)-ple(i,lp+1))*(qq(i,lp+1)-qqcolmin(i))
     2          / (qq(i,lp+1)-qq(i,lp)) ) ) * aitr
        else
         pbldpth(I) = pbldpth(I) + ( (PLE(I,nlev+1)-PLE(I,2)) +
     1    ( (ple(i,2)-ple(i,1))*(qq(i,1)-qqcolmin(i))
     2          / qq(i,1) ) ) * aitr
        endif
       enddo
      ENDIF
      if( iustar.ne.0 ) then
      do i=1,istrip
      sustar(i) = sustar(i) + aitr*ustar(i)
      enddo
      endif
      if( iz0.ne.0 ) then
      do i=1,istrip
      sz0(i) = sz0(i) + aitr*z0(i)
      enddo
      endif
      IF(LEU) then
      DO I =1,ISTNLV
      EU(I,1) = EU(I,1) + AITR*KM(I,1)
      enddo
      endif
      IF(LET) then
      DO I =1,ISTNLV
      ET(I,1) = ET(I,1) + AITR*KH(I,1)
      enddo
      endif
      IF(LCU) then
      DO I =1,ISTRIP
      scu(I) = scu(I) + AITR*cu(I)
      enddo
      endif
      IF(LCT) then
      DO I =1,ISTRIP
      sct(I) = sct(I) + AITR*ct(I)
      enddo
      endif
      IF(tprof) then
      do i=1,ISTNM1
      XLDIAG(I,1) = XLDIAG(I,1) + AITR*XL(I,1)
      enddo
      endif
      FIRST = .FALSE.
C
C   END OF MAIN LOOP
C
 2000 CONTINUE
      DO 9194 I =1,ISTNLV
      IF(TPROF.OR.LUFU)  WU(I,1) =  WU(I,1) * AITR
      IF(TPROF.OR.LVFU)  WV(I,1) =  WV(I,1) * AITR
      IF(TPROF.OR.LHFU)  WT(I,1) =  WT(I,1) * AITR
      IF(TPROF.OR.LEFU) WSH(I,1) = WSH(I,1) * AITR
9194  CONTINUE
      DO 9196 I =1,ISTRIP
      IF(LUFS)  WU(I,NLEV) =  WU(I,NLEV) * AITR
      IF(LVFS)  WV(I,NLEV) =  WV(I,NLEV) * AITR
      IF(LHFS)  WT(I,NLEV) =  WT(I,NLEV) * AITR
      IF(LEFS) WSH(I,NLEV) = WSH(I,NLEV) * AITR
9196  CONTINUE
C
C  IF TPROF, CALL POINT BY POINT DIAGNOSTIC ROUTINE FOR 'AFTER' VALUES
C
       IF(TPROF)THEN
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,QQINIT,'TKE BEFORE      ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,RIINIT,'RI BEFORE       ')
      CALL PNTPRF (NN,ISTRIP,NLEV+1,NYMD,NHMS,TH,'THETA AFTER     ')
      CALL PNTPRF (NN,ISTRIP,NLEV+1,NYMD,NHMS,THV,'THETA V AFTER   ')
      CALL PNTPRF (NN,ISTRIP,NLEV+1,NYMD,NHMS,SH,'Q AFTER         ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,U,'U WIND AFTER    ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,V,'V WIND AFTER    ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,QQ,'TKE AFTER       ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,RI,'RI AFTER        ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,WU,'TRB U FLUX      ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,WV,'TRB V FLUX      ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,WT,'TRB T FLUX      ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,WSH,'TRB Q FLUX      ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,EU,'EDDY COEF MOM   ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,ET,'EDDY COEF HEAT  ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,XLDIAG,'LENGTH SCALE    ')
      CALL PNTPRF (NN,ISTRIP,NLEV,NYMD,NHMS,DZ1TMP,'LAYER HEIGHTS   ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,SHG,'Q GROUND        ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,RIBIN,'RIB INITIAL     ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,CUIN,'CU INITIAL      ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,CTIN,'CT INITIAL      ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,USTARIN,'USTAR INITIAL   ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,RHOSIN,'RHO SFC INITIAL ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,Z0IN,'Z0 INITIAL      ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,ZETAIN,'ZETA INITIAL    ')
      CALL PNTPRF (NN,ISTRIP,1,NYMD,NHMS,BETAEV,'BETA COEFF      ')
       ENDIF
      RETURN
      END
       SUBROUTINE SFCFLX(NN,VUS,VVS,VTHV1,VTHV2,VTH1,VTH2,VSH1,
     1      VSH2,VPK,VPKE,VPE,VZ0,IVWATER,VHS,
     2      VAHS,VBETAEV,FIRST,LAST,N,IRUN,aitr,VRHO,VRHOZPK,VKH,VKM,
     3      VUSTAR,VXX,VYY,VCU,VCT,VRIB,VZETA,VWS,
     4      stu2m,stv2m,stt2m,stq2m,stu10m,stv10m,stt10m,stq10m,
     5      iu2m,iv2m,it2m,iq2m,iu10m,iv10m,it10m,iq10m,cp,rgas,undef)
C**********************************************************************
C  SUBROUTINE SFCFLX - COMPUTES SURFACE TRANSFER COEFFICIENTS
C                    - CALLED FROM TRBFLX
C
C   ARGUMENTS ::
C
C     INPUT:
C     ------
C    US            -         U - COMPONENT OF SURFACE WIND
C    VS            -         V - COMPONENT OF SURFACE WIND
C    THV1          -         VIRTUAL POTENTIAL TEMPERATURE AT NLAY
C    THV2          -         VIRTUAL POTENTIAL TEMPERATURE AT GROUND
C    TH1           -         POTENTIAL TEMPERATURE AT NLAY
C    TH2           -         POTENTIAL TEMPERATURE AT GROUND
C    SH1           -         SPECIFIC HUMIDITY AT NLAY
C    SH2           -         SPECIFIC HUMIDITY AT GROUND
C    PK            -         EVEN LEVEL PRESSURE ** KAPPA AT LEVEL NLAY
C    PKE           -         EDGE LEVEL PRESSURE ** KAPPA AT GROUND
C    PE            -         SURFACE PRESSURE
C    Z0            -         SURFACE ROUGHNESS
C    WATER         -         BIT ARRAY - '1' OVER OCEANS
C    HS            -         DEPTH OF SURFACE LAYER
C    AHS           -         ONE / HS
C    BETAEV        -         EVAPOTRANSPIRATION COEFFICIENT
C    FIRST         -         LOGICAL .TRUE. FOR FIRST TRBFLX ITERATION
C    LAST          -         LOGICAL .TRUE. FOR LAST TRBFLX ITERATION
C    N             -         NUMBER OF SFCFLX ITERATIONS
C     OUTPUT:
C     -------
C    RHO           -         DENSITY AT 10M HEIGHT
C    RHOZPK        -         RHO * P**K AT THE SURFACE
C    KH            -         HEAT TRANSFER COEFFICIENT (CT*USTAR)
C    KM            -         MOMENTUM TRANSFER COEFFICIENT (CU*USTAR)
C    USTAR         -         FRICTION VELOCITY
C    XX            -         PHIM(ZETA) - DIMENSIONLESS WIND SHEAR
C    YY            -         PHIH(ZETA) - DIMENSIONLESS TEMP GRADIENT
C    CU            -         MOMENTUM TRANSPORT COEFFICIENT
C    CT            -         HEAT TRANSPORT COEFFICIENT
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>

       DIMENSION VUS(IRUN),VVS(IRUN),VTHV1(IRUN),VTHV2(IRUN)
       DIMENSION VTH1(IRUN),VTH2(IRUN),VSH1(IRUN),VSH2(IRUN)
       DIMENSION VPK(IRUN),VPKE(IRUN),VPE(IRUN)
       DIMENSION VZ0(IRUN),IVWATER(IRUN),VHS(IRUN),VAHS(IRUN)
       DIMENSION VBETAEV(IRUN),VRHO(IRUN),VRHOZPK(IRUN)
       DIMENSION VKM(IRUN),VKH(IRUN),VUSTAR(IRUN),VXX(IRUN)
       DIMENSION VYY(IRUN),VCU(IRUN),VCT(IRUN),VRIB(IRUN)
       DIMENSION VZETA(IRUN),VWS(IRUN)
       dimension stu2m(irun),stv2m(irun),stt2m(irun),stq2m(irun)
       dimension stu10m(irun),stv10m(irun),stt10m(irun),stq10m(irun)
       LOGICAL FIRST,LAST
C
 
#if macro
cmic$ taskcommon mspace, mwater 
#endif
      COMMON /MSPACE/ WRKFLX (ISTRIP,36*NLAYR+31)
      COMMON /MSPACE/ WRKLEN (ISTRIP,10*NLAYR-7 )
      COMMON /MSPACE/ WRKTRB (ISTRIP,5*NLAYR-5 )
      COMMON /MSPACE/ WRKSF1 (ISTRIP,37         )
      COMMON /MSPACE/ WRKSF2 (ISTRIP,42         )
      COMMON /MSPACE/ WRKSF3 (ISTRIP,26         )
 
       LOGICAL LWATER
       COMMON /MWATER/ LWATER, IVBITRIB(ISTRIP)

       DIMENSION VHZ(ISTRIP)
       DIMENSION VH0(ISTRIP)
       DIMENSION VPSIM(ISTRIP),VAPSIM(ISTRIP)
       DIMENSION VPSIG(ISTRIP),VPSIHG(ISTRIP)
       DIMENSION VTEMP(ISTRIP),VDZETA(ISTRIP)
       DIMENSION VDZ0(ISTRIP),VDPSIM(ISTRIP)
       DIMENSION VDPSIH(ISTRIP),VZH(ISTRIP)
       DIMENSION VXX0(ISTRIP),VYY0(ISTRIP)
       DIMENSION VAPSIHG(ISTRIP),VRIB1(ISTRIP),VWS1(ISTRIP)
       DIMENSION VPSIH(ISTRIP),VZETAL(ISTRIP)
       DIMENSION VZ0L(ISTRIP),VPSIH2(ISTRIP)
       dimension psihdiag(istrip),psimdiag(istrip)
C
       EQUIVALENCE( WRKSF1(1, 1),    VHZ(1))
       EQUIVALENCE( WRKSF1(1, 2),  VPSIM(1))
       EQUIVALENCE( WRKSF1(1, 3), VAPSIM(1))
       EQUIVALENCE( WRKSF1(1, 4),  VPSIG(1))
       EQUIVALENCE( WRKSF1(1, 5), VPSIHG(1))
       EQUIVALENCE( WRKSF1(1, 6),  VTEMP(1))
       EQUIVALENCE( WRKSF1(1, 7), VDZETA(1))
       EQUIVALENCE( WRKSF1(1, 8),   VDZ0(1))
       EQUIVALENCE( WRKSF1(1, 9), VDPSIM(1))
       EQUIVALENCE( WRKSF1(1,10), VDPSIH(1))
       EQUIVALENCE( WRKSF1(1,11),    VZH(1))
       EQUIVALENCE( WRKSF1(1,12),   VXX0(1))
       EQUIVALENCE( WRKSF1(1,13),   VYY0(1))
       EQUIVALENCE( WRKSF1(1,14),VAPSIHG(1))
       EQUIVALENCE( WRKSF1(1,15),  VRIB1(1))
       EQUIVALENCE( WRKSF1(1,16),   VWS1(1))
       EQUIVALENCE( WRKSF1(1,17),  VPSIH(1))
       EQUIVALENCE( WRKSF1(1,18), VZETAL(1))
       EQUIVALENCE( WRKSF1(1,19),   VZ0L(1))
       EQUIVALENCE( WRKSF1(1,20), VPSIH2(1))
       EQUIVALENCE( WRKSF1(1,36),    VH0(1))
C
C     DETERMINE SURFACE WIND MAGNITUDE AND BULK RICHARDSON NUMBER
C
      DO 9000 I = 1,IRUN
       VWS(I) = VUS(I) * VUS(I) + VVS(I) * VVS(I)
       IF ( VWS(I) .LE. Z1PEM4) VWS(I) = Z1PEM4
       VRIB(I) = ( CP * (VPKE(I)-VPK(I)) ) *
     1       (VTHV1(I) - VTHV2(I)) / VWS(I)
       VWS(I) = SQRT( VWS(I) )
 9000 CONTINUE
C
C     INITIALIZATION (FIRST TRBFLX ITERATION)
C  INITIAL GUESS FOR ROUGHNESS LENGTH Z0 OVER WATER
C
      IF (.NOT. FIRST) GO TO 100
C
      IWATER = 0
      DO 9002 I = 1,IRUN
       IF (IVWATER(I).EQ.1)  IWATER = IWATER + 1
 9002 CONTINUE
      LWATER = .FALSE.
      IF(IWATER.GE.1)LWATER = .TRUE.
C
      IF(LWATER)THEN
       DO 9004 I = 1,IRUN
        IF (IVWATER(I).EQ.1) VZ0(I) = 0.0003
 9004  CONTINUE
      ENDIF
      do i = 1,irun
       vh0(i) = h0byz0 * vz0(i)
       if(vz0(i).ge.z0vegm)vh0(i) = h0veg
       enddo
C
C     CU AND PSIHG FOR NEUTRALLY STRATIFIED FLOW
C
      DO 9006 I = 1,IRUN
       VHZ(I) = VHS(I) / VZ0(I)
       VPSIM(I) = LOG( VHZ(I) )
       VAPSIM(I) = ONE / VPSIM(I)
       VCU(I) = VK * VAPSIM(I)
       VUSTAR(I) = VCU(I) * VWS(I)
C
       VPSIG(I) = VH0(I) * VUSTAR(I) - USTH0S
       if(VPSIG(I).lt.zero)  VPSIG(I) = zero
       VPSIG(I) = SQRT( VPSIG(I) )
       VPSIG(I) = BMDL * VPSIG(I)
       VPSIHG(I) = VPSIM(I) + VPSIG(I)
 9006 CONTINUE
C
C     LINEAR CORRECTION FOR ERROR IN ROUGHNESS LENGTH Z0
C
      IF(LWATER)THEN
       DO 9008 I = 1,IRUN
        VTEMP(I) = ZERO
 9008  CONTINUE
       CALL LINADJ(NN,VRIB,VRIB,VWS,
     1  VWS,VZ0,VUSTAR,IVWATER,
     2  VAPSIM,VTEMP,VTEMP,
     3  VTEMP,VTEMP,VTEMP,
     4  VTEMP,VTEMP,1,.TRUE.,IRUN,VDZETA,
     5  VDZ0,VDPSIM,VDPSIH,
     6  IVBITRIB,WRKSF1(1,22))
       DO 9010 I = 1,IRUN
        IF ( IVWATER(I).EQ.1 ) THEN
         VCU(I) = VCU(I) * (ONE - VDPSIM(I)*VAPSIM(I))
         VZ0(I) = VZ0(I) + VDZ0(I)
         ENDIF
         IF ( IVWATER(I).EQ.1) THEN
         IF ( VZ0(I) .LE. Z0MIN ) VZ0(I) = Z0MIN
         vh0(i) = h0byz0 * vz0(i)
         VPSIG(I) = VH0(I) * VCU(I) * VWS(I) - USTH0S
         if(VPSIG(I).lt.zero)  VPSIG(I) = zero
         VPSIG(I) = SQRT( VPSIG(I) )
         VPSIG(I) = BMDL * VPSIG(I)
         VPSIHG(I) = VPSIM(I) + VDPSIH(I) + VPSIG(I)
        ENDIF
 9010  CONTINUE
      ENDIF
C
C  INITIAL GUESS FOR STABILITY PARAMETER ZETA
C
      DO 9012 I = 1,IRUN
       VZETA(I) = VK2 * VRIB(I) / (VCU(I) * VCU(I) * VPSIHG(I))
 9012 CONTINUE
C
C  RECOMPUTE CU, ESTIMATE PSIHG AND UPDATE ZETA AND Z0
C
      DO 9014 I = 1,IRUN
       VZH(I) = VZ0(I) * VAHS(I)
 9014 CONTINUE
      CALL PSI (VZETA,VZH,VPSIM,
     1    VTEMP,IRUN,VXX,VXX0,VYY,
     2    VYY0,2)
      DO 9016 I = 1,IRUN
       VCU(I) = VK / VPSIM(I)
       VPSIG(I) = VH0(I) * VCU(I) * VWS(I) - USTH0S
       if(VPSIG(I).lt.zero)  VPSIG(I) = zero
       VPSIG(I) = SQRT(VPSIG(I))
       VPSIG(I) = BMDL * VPSIG(I)
       VPSIHG(I) = VPSIM(I) + VPSIG(I)
       VZETA(I) = VK2 * VRIB(I) / (VCU(I) * VCU(I) * VPSIHG(I))
 9016 CONTINUE
C
      IF(LWATER)THEN
CCCOOOMMMM ADDED 'WHERE WATER'
       DO 9018 I = 1,IRUN
        IF (IVWATER(I).EQ.1) VUSTAR(I) = VCU(I) * VWS(I)
 9018  CONTINUE
        CALL ZCSUB ( VUSTAR,VHZ,IVWATER,
     1    .FALSE.,WRKSF1(1,22),WRKSF1(1,27),
     2     WRKSF1(1,28),WRKSF1(1,30),WRKSF1(1,32) ,IRUN,VTEMP)
       DO 9020 I = 1,IRUN
        IF (IVWATER(I).EQ.1 ) then
         VZ0(I) = VTEMP(I)
         IF ( VZ0(I) .LE. Z0MIN ) VZ0(I) = Z0MIN
         vh0(i) = h0byz0 * vz0(i)
        endif
 9020  CONTINUE
      ENDIF
C
      GO TO 125
C
C  LINEARLY UPDATE ZETA AND Z0 FOR SECOND OR GREATER TRBFLX ITERATION
C
 100  CONTINUE
C
      CALL LINADJ(NN,VRIB1,VRIB,VWS1,
     1  VWS,VZ0,VUSTAR,IVWATER,
     2  VAPSIM,VAPSIHG,VPSIH,
     3  VPSIG,VXX,VXX0,
     4  VYY,VYY0,2,LWATER,IRUN,VDZETA,
     5  VDZ0,VDPSIM,VDPSIH,
     6  IVBITRIB,WRKSF1(1,22))
C
      DO 9022 I = 1,IRUN
       VZETA(I) = VZETA(I) + VZETAL(I) * VDZETA(I)
       IF (IVBITRIB(I).EQ.1 )VZETA(I) =
     1       VPSIM(I) * VPSIM(I) * VRIB(I) * VCT(I) * RVK
 9022 CONTINUE
C
      IF ( LWATER ) THEN
       DO 9024 I = 1,IRUN
        IF (IVWATER(I).EQ.1 ) then
         VZ0(I) = VZ0(I) + VZ0L(I) * VDZ0(I)
         IF (VZ0(I) .LE. Z0MIN ) VZ0(I) = Z0MIN
         vh0(i) = h0byz0 * vz0(i)
        endif
 9024  CONTINUE
      ENDIF
C
 125  CONTINUE
C
C  ITERATIVE LOOP - N ITERATIONS
C     COMPUTE CU AND CT
C
      DO 200 ITER = 1,N
       DO 9026 I = 1,IRUN
        VZH(I) = VZ0(I) * VAHS(I)
 9026  CONTINUE
       CALL PSI (VZETA,VZH,VPSIM,
     1    VPSIH,IRUN,VXX,VXX0,VYY,
     2    VYY0,1)
       DO 9028 I = 1,IRUN
        VCU(I) = VK / VPSIM(I)
        VUSTAR(I) = VCU(I) * VWS(I)
C
        VPSIG(I) = VH0(I) * VUSTAR(I) - USTH0S
        if(VPSIG(I).lt.zero)  VPSIG(I) = zero
        VPSIG(I) = SQRT(VPSIG(I))
        VPSIG(I) = BMDL * VPSIG(I)
        VPSIHG(I) = VPSIH(I) + VPSIG(I)
C
C  LINEAR CORRECTIONS FOR CU, CT, ZETA, AND Z0
C
        VAPSIM(I) = VCU(I) * RVK
        VAPSIHG(I) = ONE / VPSIHG(I)
        VRIB1(I) = VAPSIM(I) * VAPSIM(I) * VPSIHG(I) * VZETA(I)
 9028  CONTINUE
C
       ITYPE = 3
       IF(ITER.EQ.N) ITYPE = 4
       IF( (ITYPE.EQ.4) .AND. (.NOT.LAST) ) ITYPE = 5
C
       CALL LINADJ(NN,VRIB1,VRIB,VWS,
     1  VWS,VZ0,VUSTAR,IVWATER,
     2  VAPSIM,VAPSIHG,VPSIH,
     3  VPSIG,VXX,VXX0,
     4  VYY,VYY0,ITYPE,LWATER,IRUN,VDZETA,
     5  VDZ0,VDPSIM,VDPSIH,
     6  IVBITRIB,WRKSF1(1,22))
C
C  UPDATES OF ZETA, Z0, CU AND CT
C
       IF (ITYPE.EQ.5) THEN
        DO 9030 I = 1,IRUN
         VZETAL(I) = VZETA(I)
         VZ0L(I) = VZ0(I)
 9030   CONTINUE
       ENDIF
C
       DO 9032 I = 1,IRUN
        VZETA(I) = VZETA(I) * ( ONE + VDZETA(I) )
        IF (IVBITRIB(I).EQ.1 ) VZETA(I) =
     1        VPSIM(I) * VPSIM(I) * VRIB(I) * VAPSIHG(I)
 9032  CONTINUE
C
       IF ( LWATER ) THEN
        DO 9034 I = 1,IRUN
         IF (IVWATER(I).EQ.1 ) then
          VZ0(I) = VZ0(I) * ( ONE + VDZ0(I) )
          IF (VZ0(I) .LE. Z0MIN ) VZ0(I) = Z0MIN
          vh0(i) = h0byz0 * vz0(i)
         endif
 9034   CONTINUE
       ENDIF
C
       IF ( ITER .EQ. N ) THEN
        DO 9036 I = 1,IRUN
         VPSIM(I) = VPSIM(I) + VDPSIM(I)
         VCU(I) = VK / VPSIM(I)
         VUSTAR(I) = VCU(I) * VWS(I)
C
         VPSIG(I) = VH0(I) * VUSTAR(I) - USTH0S
         if(VPSIG(I).lt.zero)  VPSIG(I) = zero
         VPSIG(I) = SQRT(VPSIG(I))
         VPSIG(I) = BMDL * VPSIG(I)
         VPSIHG(I) = VPSIH(I) + VDPSIH(I) + VPSIG(I)
         VCT(I) = VK / VPSIHG(I)
 9036   CONTINUE
       ENDIF
C
C  SAVE VALUES OF RIB AND WS FOR NEXT ITERATION OF TRBFLX
C
       IF (ITYPE.EQ.5) THEN
        DO 9038 I = 1,IRUN
         VRIB1(I) = VRIB(I)
         VWS1(I) = VWS(I)
 9038   CONTINUE
       ENDIF
C
 200  CONTINUE
C
C  CALCULATE RHO-SURFACE ( KG / M**3 )
C
      IF (FIRST) THEN
       DO I = 1,IRUN
        VTEMP(I) =  TEN * VAHS(I) * VZETA(I)
        VZH(I) = VZ0(I) * ZP1
       ENDDO
       CALL PSI (VTEMP,VZH,VHZ,
     1    VPSIH2,IRUN,VHZ,VHZ,VHZ,
     2    VHZ,3)
       DO I = 1,IRUN
        VTEMP(I) = ( VPSIH2(I) + VPSIG(I) ) / VPSIHG(I)
        VRHO(I) = VPKE(I)*( VTH2(I) + VTEMP(I) * (VTH1(I)-VTH2(I)) )
        VRHO(I) = VPE(I)*HUNDRD / ( RGAS * VRHO(I) )
       ENDDO
      ENDIF
C
C interpolate uvtq to 2m and to 10 meters for diagnostic output
C  use psih and psim which represent non-dim change from ground
C                 to specified level
C and multiply theta by surface p**kappa to get temperatures
C
       if((iu2m+iv2m+it2m+iq2m).ne.0)then
        do i = 1,irun
         vtemp(i) = two * vahs(i) * vzeta(i)
         vzh(i) = vz0(i) * ahalf
         if(vz0(i).ge.two)vzh(i) = 0.9
        enddo
        call psi(vtemp,vzh,psimdiag,psihdiag,irun,vhz,vhz,vhz,vhz,1)
        do i = 1,irun
         stu2m(i) = (psimdiag(i)/vpsim(i) * vus(i))
         stv2m(i) = (psimdiag(i)/vpsim(i) * vvs(i))
         stt2m(i) = ( (vth2(i) + ((psihdiag(i)+vpsig(i))/vpsihg(i))*
     1                             (vth1(i)-vth2(i))) ) * vpke(i)
         stq2m(i) = (vsh2(i) + ((psihdiag(i)+vpsig(i))/vpsihg(i))*
     1                                             (vsh1(i)-vsh2(i)))
         if(vz0(i).ge.two)then
          stu2m(i) = UNDEF
          stv2m(i) = UNDEF
          stt2m(i) = UNDEF
          stq2m(i) = UNDEF
         endif
        enddo
       endif
       if((iu10m+iv10m+it10m+iq10m).ne.0)then
        do i = 1,irun
         vtemp(i) = ten * vahs(i) * vzeta(i)
         vzh(i) = vz0(i) * zp1
        enddo
        call psi(vtemp,vzh,psimdiag,psihdiag,irun,vhz,vhz,vhz,vhz,1)
        do i = 1,irun
         stu10m(i) = (psimdiag(i)/vpsim(i) * vus(i))
         stv10m(i) = (psimdiag(i)/vpsim(i) * vvs(i))
         stt10m(i) = ( (vth2(i) + ((psihdiag(i)+vpsig(i))/vpsihg(i))*
     1                            (vth1(i)-vth2(i))) ) * vpke(i)
         stq10m(i) = (vsh2(i) + ((psihdiag(i)+vpsig(i))/vpsihg(i))*
     1                                             (vsh1(i)-vsh2(i)))
        enddo
       endif
C
C  EVALUATE TURBULENT TRANSFER COEFFICIENTS
C
      DO 9044 I = 1,IRUN
       VRHOZPK(I) = VRHO(I) * VPKE(I)
       VKH(I) = VUSTAR(I) * VCT(I)
       VKM(I) = VUSTAR(I) * VCU(I)
 9044 CONTINUE
C
      RETURN
      END
      SUBROUTINE STRINT(A,B,IA,IB,L,K)
      DIMENSION A(IA,L), B(IB,L)
      INTEGER A,B
      INTEGER   OFFSET
      SAVE
      OFFSET = IB*(K-1)
      LEN = MIN(IB,IA-OFFSET)
      OFFSET = OFFSET+1
      IF(LEN.EQ.IB) THEN
        DO 100 J=1,L
        DO 100 I=1,LEN
        B(I,J) = A(I+OFFSET-1,J)
100     CONTINUE
      ELSE
        DO 200 J=1,L
        DO 300 I=1,LEN
        B(I,J) = A(I+OFFSET-1,J)
300     CONTINUE
        DO 400 I=1,IB-LEN
        B(LEN+I,J) = A(I+OFFSET-1,J)
400     CONTINUE
200     CONTINUE
      ENDIF
      RETURN
      END
      SUBROUTINE PHI(Z,PHIM,PHIH,IFLAG,N,I1,I2,ZSTAR,E1,E2,TEMP1,
     1     INT72,INTMAX)
C**********************************************************************
C
C  FUNCTION PHI - SOLVES KEYPS EQUATIONS
C               - CALLED FROM PSI
C
C  DESCRIPTION OF PARAMETERS
C     Z     -  INPUTED VALUE OF MONIN- OBUKHOV STABILITY PARAMETER ZETA
C               TIMES APPROPRIATE CONSTANT
C     PHIM  -  OUTPUTED SOLUTION OF KEYPS EQUATION FOR MOMENTUM
C     PHIH  -  OUTPUTED SOLUTION OF KEYPS EQUATION FOR SCALARS
C     IFLAG -  FLAG TO DETERMINE IF X IS NEEDED (IFLAG=2), Y IS NEEDED
C                  (IFLAG=3), OR BOTH (IFLAG=1)
C     N     -  LENGTH OF VECTOR TO BE SOLVED
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
      DIMENSION PHIM(N),PHIH(N),Z(N)
      DIMENSION INT72(ISTRIP), INTMAX(ISTRIP)
      DIMENSION ZSTAR(ISTRIP),I1(ISTRIP),I2(ISTRIP)
      DIMENSION E1(ISTRIP),E2(ISTRIP),TEMP1(ISTRIP)
C
      DIMENSION PHIM0(385),ZLINM1(75),ZLINM2(75),ZLINM3(36)
      DIMENSION ZLOGM1(74),ZLOGM2(75),ZLOGM3(50)
      DIMENSION PHIH0(385),ZLINH1(75),ZLINH2(75),ZLINH3(36)
      DIMENSION ZLOGH1(74),ZLOGH2(75),ZLOGH3(50)
      EQUIVALENCE (PHIM0(1),ZLINM1(1)),(PHIM0(76),ZLINM2(1))
      EQUIVALENCE (PHIM0(151),ZLINM3(1))
      EQUIVALENCE (PHIM0(187),ZLOGM1(1)),(PHIM0(261),ZLOGM2(1))
      EQUIVALENCE (PHIM0(336),ZLOGM3(1))
      EQUIVALENCE (PHIH0(1),ZLINH1(1)),(PHIH0(76),ZLINH2(1))
      EQUIVALENCE (PHIH0(151),ZLINH3(1))
      EQUIVALENCE (PHIH0(187),ZLOGH1(1)),(PHIH0(261),ZLOGH2(1))
      EQUIVALENCE (PHIH0(336),ZLOGH3(1))
C
       DATA ZLOGM1/
     .             0.697894,0.678839,0.659598,0.640260,
     .  0.620910,0.601628,0.582486,0.563550,0.544877,
     .  0.526519,0.508516,0.490903,0.473708,0.456951,
     .  0.440649,0.424812,0.409446,0.394553,0.380133,
     .  0.366182,0.352695,0.339664,0.327082,0.314938,
     .  0.303222,0.291923,0.281029,0.270528,0.260409,
     .  0.250659,0.241267,0.232221,0.223509,0.215119,
     .  0.207041,0.199264,0.191776,0.184568,0.177628,
     .  0.170949,0.164519,0.158331,0.152374,0.146641,
     .  0.141123,0.135813,0.130702,0.125783,0.121048,
     .  0.116492,0.112107,0.107887,0.103826,0.0999177,
     .  0.0961563,0.0925364,0.0890528,0.0857003,0.0824739,
     .  0.0793690,0.0763810,0.0735054,0.0707380,0.0680749,
     .  0.0655120,0.0630455,0.0606720,0.0583877,0.0561895,
     .  0.0540740,0.0520382,0.0500790,0.0481936,0.0463791/
       DATA ZLOGM2/
     .  0.0446330,0.0429526,0.0413355,0.0397792,0.0382816,
     .  0.0368403,0.0354533,0.0341185,0.0328340,0.0315978,
     .  0.0304081,0.0292633,0.0281616,0.0271013,0.0260809,
     .  0.0250990,0.0241540,0.0232447,0.0223695,0.0215273,
     .  0.0207168,0.0199369,0.0191862,0.0184639,0.0177687,
     .  0.0170998,0.0164560,0.0158364,0.0152402,0.0146664,
     .  0.0141142,0.0135828,0.0130714,0.0125793,0.0121057,
     .  0.0116499,0.0112113,0.0107892,0.0103830,0.999210E-2,
     .  0.961590E-2,0.925387E-2,0.890547E-2,0.857018E-2,0.824752E-2,
     .  0.793701E-2,0.763818E-2,0.735061E-2,0.707386E-2,0.680754E-2,
     .  0.655124E-2,0.630459E-2,0.606722E-2,0.583880E-2,0.561897E-2,
     .  0.540742E-2,0.520383E-2,0.500791E-2,0.481937E-2,0.463792E-2,
     .  0.446331E-2,0.429527E-2,0.413355E-2,0.397793E-2,0.382816E-2,
     .  0.368403E-2,0.354533E-2,0.341185E-2,0.328340E-2,0.315978E-2,
     .  0.304082E-2,0.292633E-2,0.281616E-2,0.271013E-2,0.260809E-2/
       DATA ZLOGM3/
     .  0.250990E-2,0.241541E-2,0.232447E-2,0.223695E-2,0.215273E-2,
     .  0.207168E-2,0.199369E-2,0.191862E-2,0.184639E-2,0.177687E-2,
     .  0.170998E-2,0.164560E-2,0.158364E-2,0.152402E-2,0.146664E-2,
     .  0.141142E-2,0.135828E-2,0.130714E-2,0.125793E-2,0.121057E-2,
     .  0.116499E-2,0.112113E-2,0.107892E-2,0.103830E-2,0.999210E-3,
     .  0.961590E-3,0.925387E-3,0.890547E-3,0.857018E-3,0.824752E-3,
     .  0.793701E-3,0.763818E-3,0.735061E-3,0.707386E-3,0.680754E-3,
     .  0.655124E-3,0.630459E-3,0.606722E-3,0.583880E-3,0.561897E-3,
     .  0.540742E-3,0.520383E-3,0.500791E-3,0.481937E-3,0.463792E-3,
     .  0.446331E-3,0.429527E-3,0.413355E-3,0.397793E-3,0.382816E-3/
       DATA ZLOGH1/
     .             0.640529,0.623728,0.606937,0.590199,
     .  0.573552,0.557032,0.540672,0.524504,0.508553,
     .  0.492843,0.477397,0.462232,0.447365,0.432809,
     .  0.418574,0.404670,0.391103,0.377878,0.364999,
     .  0.352468,0.340284,0.328447,0.316954,0.305804,
     .  0.294992,0.284514,0.274364,0.264538,0.255028,
     .  0.245829,0.236933,0.228335,0.220026,0.211999,
     .  0.204247,0.196762,0.189537,0.182564,0.175837,
     .  0.169347,0.163088,0.157051,0.151231,0.145620,
     .  0.140211,0.134998,0.129974,0.125133,0.120469,
     .  0.115975,0.111645,0.107475,0.103458,0.995895E-1,
     .  0.958635E-1,0.922753E-1,0.888199E-1,0.854925E-1,0.822886E-1,
     .  0.792037E-1,0.762336E-1,0.733739E-1,0.706208E-1,0.679704E-1,
     .  0.654188E-1,0.629625E-1,0.605979E-1,0.583217E-1,0.561306E-1,
     .  0.540215E-1,0.519914E-1,0.500373E-1,0.481564E-1,0.463460E-1/
       DATA ZLOGH2/
     .  0.446034E-1,0.429263E-1,0.413120E-1,0.397583E-1,0.382629E-1,
     .  0.368237E-1,0.354385E-1,0.341053E-1,0.328222E-1,0.315873E-1,
     .  0.303988E-1,0.292550E-1,0.281541E-1,0.270947E-1,0.260750E-1,
     .  0.250937E-1,0.241494E-1,0.232405E-1,0.223658E-1,0.215240E-1,
     .  0.207139E-1,0.199342E-1,0.191839E-1,0.184618E-1,0.177669E-1,
     .  0.170981E-1,0.164545E-1,0.158351E-1,0.152390E-1,0.146653E-1,
     .  0.141133E-1,0.135820E-1,0.130707E-1,0.125786E-1,0.121051E-1,
     .  0.116494E-1,0.112108E-1,0.107888E-1,0.103826E-1,0.999177E-2,
     .  0.961561E-2,0.925360E-2,0.890523E-2,0.856997E-2,0.824733E-2,
     .  0.793684E-2,0.763803E-2,0.735048E-2,0.707375E-2,0.680743E-2,
     .  0.655114E-2,0.630450E-2,0.606715E-2,0.583873E-2,0.561891E-2,
     .  0.540737E-2,0.520379E-2,0.500787E-2,0.481933E-2,0.463789E-2,
     .  0.446328E-2,0.429524E-2,0.413353E-2,0.397790E-2,0.382814E-2,
     .  0.368401E-2,0.354532E-2,0.341184E-2,0.328338E-2,0.315977E-2,
     .  0.304081E-2,0.292632E-2,0.281615E-2,0.271012E-2,0.260809E-2/
       DATA ZLOGH3/
     .  0.250990E-2,0.241540E-2,0.232446E-2,0.223695E-2,0.215273E-2,
     .  0.207168E-2,0.199368E-2,0.191862E-2,0.184639E-2,0.177687E-2,
     .  0.170997E-2,0.164559E-2,0.158364E-2,0.152402E-2,0.146664E-2,
     .  0.141142E-2,0.135828E-2,0.130714E-2,0.125793E-2,0.121057E-2,
     .  0.116499E-2,0.112113E-2,0.107892E-2,0.103830E-2,0.999209E-3,
     .  0.961590E-3,0.925387E-3,0.890546E-3,0.857018E-3,0.824752E-3,
     .  0.793700E-3,0.763818E-3,0.735061E-3,0.707386E-3,0.680754E-3,
     .  0.655124E-3,0.630459E-3,0.606722E-3,0.583880E-3,0.561897E-3,
     .  0.540742E-3,0.520383E-3,0.500791E-3,0.481937E-3,0.463792E-3,
     .  0.446331E-3,0.429527E-3,0.413355E-3,0.397793E-3,0.382816E-3/
C
       DATA ZLINM1/
     &  0.964508,0.962277,0.960062,0.957863,0.955680,
     &  0.953512,0.951359,0.949222,0.947100,0.944992,
     &  0.942899,0.940821,0.938758,0.936709,0.934673,
     &  0.932652,0.930645,0.928652,0.926672,0.924706,
     &  0.922753,0.920813,0.918886,0.916973,0.915072,
     &  0.913184,0.911308,0.909445,0.907594,0.905756,
     &  0.903930,0.902115,0.900313,0.898522,0.896743,
     &  0.894975,0.893219,0.891475,0.889741,0.888019,
     &  0.886307,0.884607,0.882917,0.881238,0.879569,
     &  0.877911,0.876264,0.874626,0.872999,0.871382,
     &  0.869775,0.868178,0.866591,0.865013,0.863445,
     &  0.861887,0.860338,0.858798,0.857268,0.855747,
     &  0.854235,0.852732,0.851238,0.849753,0.848277,
     &  0.846809,0.845350,0.843900,0.842458,0.841025,
     &  0.839599,0.838182,0.836774,0.835373,0.833980/
       DATA ZLINM2/
     &  0.832596,0.831219,0.829850,0.828489,0.827136,
     &  0.825790,0.824451,0.823121,0.821797,0.820481,
     &  0.819173,0.817871,0.816577,0.815289,0.814009,
     &  0.812736,0.811470,0.810210,0.808958,0.807712,
     &  0.806473,0.805240,0.804015,0.802795,0.801582,
     &  0.800376,0.799176,0.797982,0.796794,0.795613,
     &  0.794438,0.793269,0.792106,0.790949,0.789798,
     &  0.788652,0.787513,0.786380,0.785252,0.784130,
     &  0.783014,0.781903,0.780798,0.779698,0.778604,
     &  0.777516,0.776432,0.775354,0.774282,0.773215,
     &  0.772153,0.771096,0.770044,0.768998,0.767956,
     &  0.766920,0.765888,0.764862,0.763840,0.762824,
     &  0.761812,0.760805,0.759803,0.758805,0.757813,
     &  0.756824,0.755841,0.754862,0.753888,0.752918,
     &  0.751953,0.750992,0.750035,0.749083,0.748136/
       DATA ZLINM3/
     &  0.747192,0.746253,0.745318,0.744388,0.743462,
     &  0.742539,0.741621,0.740707,0.739798,0.738892,
     &  0.737990,0.737092,0.736198,0.735308,0.734423,
     &  0.733540,0.732662,0.731788,0.730917,0.730050,
     &  0.729187,0.728328,0.727472,0.726620,0.725772,
     &  0.724927,0.724086,0.723248,0.722414,0.721584,
     &  0.720757,0.719933,0.719113,0.718296,0.717483,
     &  0.716673/
       DATA ZLINH1/
     &  0.936397,0.932809,0.929287,0.925827,0.922429,
     &  0.919089,0.915806,0.912579,0.909405,0.906284,
     &  0.903212,0.900189,0.897214,0.894284,0.891399,
     &  0.888558,0.885759,0.883001,0.880283,0.877603,
     &  0.874962,0.872357,0.869788,0.867255,0.864755,
     &  0.862288,0.859854,0.857452,0.855081,0.852739,
     &  0.850427,0.848144,0.845889,0.843662,0.841461,
     &  0.839287,0.837138,0.835014,0.832915,0.830841,
     &  0.828789,0.826761,0.824755,0.822772,0.820810,
     &  0.818869,0.816949,0.815050,0.813170,0.811310,
     &  0.809470,0.807648,0.805845,0.804060,0.802293,
     &  0.800543,0.798811,0.797095,0.795396,0.793714,
     &  0.792047,0.790396,0.788761,0.787141,0.785535,
     &  0.783945,0.782369,0.780807,0.779259,0.777724,
     &  0.776204,0.774696,0.773202,0.771720,0.770251/
       DATA ZLINH2/
     &  0.768795,0.767351,0.765919,0.764499,0.763091,
     &  0.761694,0.760309,0.758935,0.757571,0.756219,
     &  0.754878,0.753547,0.752226,0.750916,0.749616,
     &  0.748326,0.747045,0.745775,0.744514,0.743262,
     &  0.742020,0.740787,0.739563,0.738348,0.737141,
     &  0.735944,0.734755,0.733574,0.732402,0.731238,
     &  0.730083,0.728935,0.727795,0.726664,0.725539,
     &  0.724423,0.723314,0.722213,0.721119,0.720032,
     &  0.718952,0.717880,0.716815,0.715756,0.714704,
     &  0.713660,0.712621,0.711590,0.710565,0.709547,
     &  0.708534,0.707529,0.706529,0.705536,0.704549,
     &  0.703567,0.702592,0.701623,0.700660,0.699702,
     &  0.698750,0.697804,0.696863,0.695928,0.694998,
     &  0.694074,0.693155,0.692241,0.691333,0.690430,
     &  0.689532,0.688639,0.687751,0.686868,0.685990/
       DATA ZLINH3/
     &  0.685117,0.684249,0.683386,0.682527,0.681673,
     &  0.680824,0.679979,0.679139,0.678303,0.677472,
     &  0.676645,0.675823,0.675005,0.674191,0.673381,
     &  0.672576,0.671775,0.670978,0.670185,0.669396,
     &  0.668611,0.667830,0.667054,0.666281,0.665512,
     &  0.664746,0.663985,0.663227,0.662473,0.661723,
     &  0.660977,0.660234,0.659495,0.658759,0.658027,
     &  0.657298/
C
      IBIT1 = 0
      IBIT2 = 0
C
      DO 9000 I = 1,N
       IF(Z(I).GE.ZP15)IBIT1 = IBIT1 + 1
       IF(Z(I).GT.TWO )IBIT2 = IBIT2 + 1
 9000 CONTINUE
C
      IF( IBIT1 .LE. 0 ) GO TO 200
C
      DO 9002 I = 1,N
       ZSTAR(I)    = HUNDRD * Z(I) - FORTN
 9002 CONTINUE
C
      IF( IBIT2 .LE. 0 ) GO TO 60
      DO 9004 I = 1,N
       TEMP1(I) = Z(I)*AHALF
       IF( Z(I) .LE. TWO )TEMP1(I) = ONE
       TEMP1(I) = LOG10(TEMP1(I))
       TEMP1(I) = (TEMP1(I) + Z9P3) * TWENTY
       IF( Z(I) .GT. TWO ) ZSTAR(I) = TEMP1(I)
       IF( Z(I).GT.Z1P78E10 ) ZSTAR(I) = Z384P9
 9004  CONTINUE
C
 60    CONTINUE
C
      DO 9006 I = 1,N
       I1(I) = ZSTAR(I)
       I2(I) = I1(I) + 1
       TEMP1(I) = ZSTAR(I) - I1(I)
C
 9006  CONTINUE
C
      IF( IFLAG .GT. 2 ) GO TO 100
       DO 9008 I = 1,N
       if( z(i).ge.zp15 ) then
       E1(I) = PHIM0( I1(I) )
       E2(I) = PHIM0( I2(I) )
       PHIM(I)  = TEMP1(I) * ( E2(I)-E1(I) )
       PHIM(I)  = PHIM(I) +   E1(I)
       endif
 9008  CONTINUE

  100 CONTINUE
C
      IF( IFLAG .EQ. 2 ) GO TO 200
       DO 9010 I = 1,N
       if( z(i).ge.zp15 ) then
       E1(I) = PHIH0( I1(I) )
       E2(I) = PHIH0( I2(I) )
       PHIH(I)  = TEMP1(I) * ( E2(I)-E1(I) )
       PHIH(I)  = PHIH(I) +   E1(I)
       endif
 9010  CONTINUE

  200 CONTINUE
      IF( IBIT1 .GE. N ) GO TO 500
C
       DO 9012 I = 1,N
       ZSTAR(I) = -Z(I)
 9012  CONTINUE
C
      IF( IFLAG .GT. 2 ) GO TO 300
       DO 9014 I = 1,N
       IF( Z(I) .LT. ZP15 ) PHIM(I) = ONE + ZSTAR(I)
     2     *(ZP25+ZSTAR(I)*(ZP09375+ZSTAR(I)*
     3     (ZP03125+ZP007 * ZSTAR(I))))
 9014  CONTINUE
C
  300 CONTINUE
      IF( IFLAG .EQ. 2 ) GO TO 500
       DO 9016 I = 1,N
       IF( Z(I) .LT. ZP15 ) THEN
       PHIH(I) =ONE+ Z(I) * (AHALF+ZSTAR(I)*(ZP375+ZSTAR(I)*
     1     (AHALF+ZSTAR(I)*(ZP82ETC+ZSTAR(I)*
     2     (ONEP5+TWOP9*ZSTAR(I))))))
       PHIH(I) = ONE / PHIH(I)
      ENDIF
 9016  CONTINUE
C
  500 CONTINUE
      RETURN
      END
      SUBROUTINE PSI(VZZ,VZH,VPSIM,VPSIH,IRUN,VX,VXS,VY,VYS,IFLAG)
C**********************************************************************
C
C  SUBROUTINE PSI - DETERMINES DIMENSIONLESS WIND AND
C                    SCALAR PROFILES IN SURFACE LAYER
C                 - CALLED FROM SFCFLX
C
C  DESCRIPTION OF PARAMETERS
C     ZZ   -  INPUTED VALUE OF MONIN- OBUKHOV STABILITY PARAMETER ZETA
C     ZH   -  INPUTED VALUE OF PBL HEIGHT DIVIDED BY Z0
C     PSIM -  OUTPUTED VALUE OF DIMENSIONLESS WIND
C     PSIH -  OUTPUTED VALUE OF DIMENSIONLESS SCALAR
C     X    -  OUTPUTED VALUE OF PHIM(ZETA)
C     XS   -  OUTPUTED VALUE OF PHIM(ZETA0)
C     Y    -  OUTPUTED VALUE OF PHIH(ZETA)
C     YS   -  OUTPUTED VALUE OF PHIH(ZETA0)
C     IFLAG-  FLAG TO DETERMINE IF CU IS NEEDED (IFLAG=2),
C                  IF CT IS NEEDED (IFLAG=3), OR BOTH (IFLAG=1)
C  SUBPROGRAMS NEEDED
C     PHI  -  COMPUTES SIMILARITY FUNCTION FOR MOMENTUM AND SCALARS
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
      DIMENSION VZZ(IRUN),VZH(IRUN),VPSIM(IRUN),VPSIH(IRUN),
     1     VX(IRUN),VXS(IRUN),VY(IRUN),VYS(IRUN)
C
      PARAMETER (IMJMR = IMR * (JMR+1) )
 
#if macro
cmic$ taskcommon mspace 
#endif
      COMMON /MSPACE/ WRKFLX (ISTRIP,36*NLAYR+31)
      COMMON /MSPACE/ WRKLEN (ISTRIP,10*NLAYR-7 )
      COMMON /MSPACE/ WRKTRB (ISTRIP,5*NLAYR-5 )
      COMMON /MSPACE/ WRKSF1 (ISTRIP,37         )
      COMMON /MSPACE/ WRKSF2 (ISTRIP,42         )
      COMMON /MSPACE/ WRKSF3 (ISTRIP,26         )
 
      DIMENSION INTSTB(ISTRIP),INT72(ISTRIP),INTZ0(ISTRIP)
      DIMENSION  WRKPHI(ISTRIP,12)
      DIMENSION  ZZ0(ISTRIP),Z(ISTRIP),Z2(ISTRIP),Z1(ISTRIP),Z0(ISTRIP)
      DIMENSION  X0(ISTRIP),X1(ISTRIP),Y0(ISTRIP),Y1(ISTRIP)
      DIMENSION  XX(ISTRIP),XXS(ISTRIP),YY(ISTRIP),YYS(ISTRIP)
      DIMENSION  PSIMM(ISTRIP),PSIHH(ISTRIP)
      DIMENSION  PSI2(ISTRIP),TEMP(ISTRIP)
      DIMENSION  HZ(ISTRIP),ARG0(ISTRIP),ARG1(ISTRIP),DX(ISTRIP)
      DIMENSION  X0NUM(ISTRIP),X1NUM(ISTRIP),X0DEN(ISTRIP)
      DIMENSION  X1DEN(ISTRIP),Y1DEN(ISTRIP),Z2ZWM(ISTRIP)
C
       EQUIVALENCE( WRKSF2(1, 1), WRKPHI(1,1))
       EQUIVALENCE( WRKSF2(1,13),    ZZ0(1  ))
       EQUIVALENCE( WRKSF2(1,14),      Z(1  ))
       EQUIVALENCE( WRKSF2(1,15),     Z2(1  ))
       EQUIVALENCE( WRKSF2(1,16),     Z1(1  ))
       EQUIVALENCE( WRKSF2(1,17),     Z0(1  ))
       EQUIVALENCE( WRKSF2(1,18),     X0(1  ))
       EQUIVALENCE( WRKSF2(1,19),     X1(1  ))
       EQUIVALENCE( WRKSF2(1,20),     Y0(1  ))
       EQUIVALENCE( WRKSF2(1,21),     Y1(1  ))
       EQUIVALENCE( WRKSF2(1,22),     XX(1  ))
       EQUIVALENCE( WRKSF2(1,23),    XXS(1  ))
       EQUIVALENCE( WRKSF2(1,24),     YY(1  ))
       EQUIVALENCE( WRKSF2(1,25),    YYS(1  ))
       EQUIVALENCE( WRKSF2(1,26),  PSIMM(1  ))
       EQUIVALENCE( WRKSF2(1,27),  PSIHH(1  ))
       EQUIVALENCE( WRKSF2(1,28),   PSI2(1  ))
       EQUIVALENCE( WRKSF2(1,29),   TEMP(1  ))
       EQUIVALENCE( WRKSF2(1,30),     HZ(1  ))
       EQUIVALENCE( WRKSF2(1,31),   ARG0(1  ))
       EQUIVALENCE( WRKSF2(1,32),   ARG1(1  ))
       EQUIVALENCE( WRKSF2(1,33),     DX(1  ))
       EQUIVALENCE( WRKSF2(1,34),  X0NUM(1  ))
       EQUIVALENCE( WRKSF2(1,35),  X1NUM(1  ))
       EQUIVALENCE( WRKSF2(1,36),  X0DEN(1  ))
       EQUIVALENCE( WRKSF2(1,37),  X1DEN(1  ))
       EQUIVALENCE( WRKSF2(1,38),  Y1DEN(1  ))
       EQUIVALENCE( WRKSF2(1,39),  Z2ZWM(1  ))
C
      CM3 =   sqrt( ZP2/CM1-ZP01 )
      CM4 =   ONE/CM3
      CM5 =  (TEN-CM1) / (TEN*CM1*CM3)
      CM8 =   SIX * LOG(CM8ARG)
C
      DO 9000 I = 1,IRUN
       VPSIM(I) = ZERO
       VPSIH(I) = ZERO
       VX(I) = ZERO
       VXS(I) = ZERO
       VY(I) = ZERO
       VYS(I) = ZERO
       ZZ0(I) = VZH(I)*VZZ(I)
 9000 CONTINUE
      IBIT = 0
      DO 9122 I = 1,IRUN
       IF(VZZ(I).LE.ZM1PEM7)IBIT = IBIT + 1
 9122 CONTINUE
      DO 9022 I = 1,IRUN
       IF(VZZ(I).LE.ZM1PEM7)THEN
        INTSTB(I) = 1
       ELSE
        INTSTB(I) = 0
       ENDIF
 9022 CONTINUE
C
C ****************************************
C *****    UNSTABLE SURFACE LAYER    *****
C ****************************************
C
      IF(IBIT.LE.0)  GO TO 100
C
      INDEX = 0
      DO 9002 I = 1,IRUN
       IF (INTSTB(I).EQ.1)THEN
        INDEX = INDEX + 1
        Z(INDEX) = VZZ(I)
        Z0(INDEX) = ZZ0(I)
       ENDIF
 9002 CONTINUE
C
      DO 9004 I = 1,IBIT
       Z(I) = EGHTNM * Z(I)
       Z0(I) = EGHTNM * Z0(I)
 9004 CONTINUE
C
      CALL PHI( Z,X1,Y1,IFLAG,IBIT ,
     1   WRKPHI(1,1),WRKPHI(1,3),WRKPHI(1,5),WRKPHI(1,6),WRKPHI(1,7),
     2   WRKPHI(1,8),WRKPHI(1,9),WRKPHI(1,11) )
      CALL PHI( Z0,X0,Y0,IFLAG,IBIT ,
     1   WRKPHI(1,1),WRKPHI(1,3),WRKPHI(1,5),WRKPHI(1,6),WRKPHI(1,7),
     2   WRKPHI(1,8),WRKPHI(1,9),WRKPHI(1,11) )
C
C ****************************
C *****    COMPUTE PSIM  *****
C ****************************
C
      IF(IFLAG.GE.3) GO TO 75
C
      DO 9006 I = 1,IBIT
       ARG1(I) = ONE - X1(I)
       IF ( Z(I) .LT. ZP013 ) ARG1(I) =
     1        Z(I) * ( ZP25 -  ZP09375 * Z(I) )
C
       ARG0(I)  = ONE - X0(I)
       IF ( Z0(I) .LT. ZP013 ) ARG0(I) =
     1       Z0(I) * ( ZP25 -  ZP09375 * Z0(I) )
C
       ARG1(I) = ARG1(I) * ( ONE+X0(I) )
       ARG0(I) = ARG0(I) * ( ONE+X1(I) )
       DX(I) = X1(I) - X0(I)
       ARG1(I) = ARG1(I) / ARG0(I)
       ARG0(I) = -DX(I) / ( ONE + X1(I)*X0(I) )
       ARG0(I) = ATAN( ARG0(I) )
       ARG1(I) = LOG( ARG1(I) )
       PSI2(I) = TWO * ARG0(I) + ARG1(I)
       PSI2(I) = PSI2(I) + DX(I)
 9006 CONTINUE
C
      INDEX = 0
      DO 9008 I = 1,IRUN
       IF( INTSTB(I).EQ.1 ) THEN
        INDEX = INDEX + 1
        VPSIM(I) = PSI2(INDEX)
        VX(I) = X1(INDEX)
        VXS(I) = X0(INDEX)
       ENDIF
 9008 CONTINUE
C
C ****************************
C *****    COMPUTE PSIH  *****
C ****************************
C
      IF(IFLAG.EQ.2) GO TO 100
C
  75  CONTINUE
      DO 9010 I = 1,IBIT
       ARG1(I) = ONE - Y1(I)
       IF( Z(I) .LT. ZP0065 ) ARG1(I) =
     1       Z(I) * ( AHALF -  ZP625 * Z(I) )
C
       ARG0(I)  = ONE - Y0(I)
       IF( Z0(I) .LT. ZP0065 ) ARG0(I) =
     1       Z0(I) * ( AHALF -  ZP625 * Z0(I) )
C
       ARG1(I) = ARG1(I) * ( ONE + Y0(I) )
       ARG0(I) = ARG0(I) * ( ONE + Y1(I) )
       ARG1(I) = ARG1(I) / ARG0(I)
       PSI2(I) = LOG( ARG1(I) )
       PSI2(I) = PSI2(I) - Y1(I) + Y0(I)
 9010 CONTINUE
C
      INDEX = 0
      DO 9012 I = 1,IRUN
       IF( INTSTB(I).EQ.1 ) THEN
       INDEX = INDEX + 1
       VPSIH(I) = PSI2(INDEX)
       VY(I) = Y1(INDEX)
       VYS(I) = Y0(INDEX)
       ENDIF
 9012 CONTINUE
C
C **************************************
C *****    STABLE SURFACE LAYER    *****
C **************************************
C
  100 CONTINUE
      IBIT = 0
      DO 9114 I = 1,IRUN
       IF(VZZ(I).GT.ZM1PEM7)THEN
        IBIT = IBIT + 1
       ENDIF
 9114 CONTINUE
      DO 9014 I = 1,IRUN
       IF(VZZ(I).GT.ZM1PEM7)THEN
        INTSTB(I) = 1
       ELSE
        INTSTB(I) = 0
       ENDIF
 9014 CONTINUE
      IF(IBIT.LE.0)  GO TO 300
      INDEX = 0
CDIR$ NOVECTOR
      DO 9016 I = 1,IRUN
       IF (INTSTB(I).EQ.1)THEN
        INDEX = INDEX + 1
        Z(INDEX) = VZZ(I)
        Z0(INDEX) = ZZ0(I)
        ARG1(INDEX) = VZH(I)
       ENDIF
 9016 CONTINUE
CDIR$ VECTOR

      DO 9018 I = 1,IBIT
       HZ(I) = ONE / ARG1(I)
       Z1(I) = Z(I)
       Z2(I) = ZWM
C
       IF ( Z(I) .GT. ZWM ) THEN
        Z1(I) = ZWM
        Z2(I) = Z(I)
       ENDIF
C
       IF ( Z0(I) .GT. Z0M ) THEN
        Z0(I) = Z0M
        INTZ0(I) = 1
       ELSE
        INTZ0(I) = 0
       ENDIF
C
       X1NUM(I) = ONE + FIVE * Z1(I)
       X0NUM(I) = ONE + FIVE * Z0(I)
       X1DEN(I) = ONE / (ONE + CM1 * (X1NUM(I) * Z1(I)) )
       X0DEN(I) = ONE + CM1 * (X0NUM(I) * Z0(I))
C
       IF ( (INTZ0(I).EQ.1) .OR. (Z(I).GT.ZWM) )
     1      HZ(I) = Z1(I) / Z0(I)
       ARG1(I) = HZ(I)*HZ(I)*X0DEN(I)*X1DEN(I)
       ARG1(I) = LOG( ARG1(I) )
       ARG1(I) = AHALF * ARG1(I)
       ARG0(I) = (Z1(I) + ZP1) * (Z0(I) + ZP1)
       ARG0(I) = CM3 + ARG0(I) * CM4
       ARG0(I) = ( Z1(I) - Z0(I) ) / ARG0(I)
       ARG0(I) = ATAN( ARG0(I) )
       TEMP(I) = ARG1(I) + CM5 * ARG0(I)
C
       X0(I) = X0NUM(I) / X0DEN(I)
       IF ( INTZ0(I).EQ.1 ) X0(I) = ZERO
       Z2ZWM(I) = Z2(I) * RZWM
 9018 CONTINUE
C
C ****************************
C *****    COMPUTE PSIM  *****
C ****************************
C
      IF( IFLAG.GE.3 ) GO TO 225
C
      DO 9020 I = 1,IBIT
       X1(I) = X1NUM(I) * X1DEN(I)
       ARG1(I) = LOG( Z2ZWM(I) )
       PSI2(I) = TEMP(I) + CM6 * ARG1(I)
 9020 CONTINUE
C
      INDEX = 0
      DO 9030 I = 1,IRUN
       IF( INTSTB(I).EQ.1 ) THEN
       INDEX = INDEX + 1
       VPSIM(I) = PSI2(INDEX)
       VX(I) = X1(INDEX)
       VXS(I) = X0(INDEX)
       ENDIF
 9030 CONTINUE
C
C ****************************
C *****    COMPUTE PSIH  *****
C ****************************
C
       IF(IFLAG.EQ.2)GO TO 300
C
  225 CONTINUE
      DO 9024 I = 1,IBIT
       Y1DEN(I) = ONE + CM1 * ( X1NUM(I) * Z(I) )
       Y1(I) = X1NUM(I) / Y1DEN(I)
       ARG1(I) = CM7 * Z2ZWM(I) / ( CM2 + Z2(I) )
       ARG0(I) = SIX
       IF ( Z2(I) .GT. ZCM ) THEN
        Y1(I) = YCM
        ARG1(I) = Z2(I) * RZCM
        ARG0(I) = YCM
        TEMP(I) = TEMP(I) + CM8
       ENDIF
       ARG1(I) = LOG( ARG1(I) )
       PSI2(I) = TEMP(I) + ARG0(I) * ARG1(I)
 9024 CONTINUE
C
      INDEX = 0
      DO 9026 I = 1,IRUN
       IF( INTSTB(I).EQ.1 ) THEN
       INDEX = INDEX + 1
       VPSIH(I) = PSI2(INDEX)
       VY(I) = Y1(INDEX)
       VYS(I) = X0(INDEX)
       ENDIF
 9026 CONTINUE
C
  300 CONTINUE
C
      RETURN
      END
      SUBROUTINE TRBLEN (STRT,DW2,DZ3,Q,VKZE,VKZM,DTHV,DPK,DU,DV,XL,
     .                   QXLM,NLEV,INIT,LMIN,LMINQ,LMINQ1,CP )
C**********************************************************************
C
C  SUBROUTINE TRBLEN - COMPUTES TURBULENT LENGTH SCALE
C                    - CALLED FROM TRBFLX
C   ARGUMENTS ::
C
C     INPUT:
C     ------
C    STRT          -         BRUNT VAISALA FREQUENCY
C    DW2           -         SQUARED SHEAR
C    DZ3           -         LAYER THICKNESS FOR LENGTH SCALE CALC.
C    Q             -         TURBULENCE VELOCITY
C    VKZE          -         VK * Z AT LAYER EDGES
C    VKZM          -         VK * Z AT LAYER CENTERS
C    DTHV          -         VERTICAL GRADIENT OF THV
C    DPK           -         VERTICAL GRADIENT OF PK
C    DU            -         VERTICAL GRADIENT OF U
C    DV            -         VERTICAL GRADIENT OF V
C    NLEV          -         NUMBER OF ATMOSPHERIC LEVELS
C    INIT          -         INPUT FLAG : 1 = INITIAL START
C                                         2 = 2ND CALL FOR INITIAL STAR
C                                         0 = NON-INITIAL START
C
C     OUTPUT:
C     -------
C    XL            -         TURBULENT LENGTH SCALE
C    QXLM          -         TURBULENT LENGTH SCALE * Q AT LAYER CENTER
C    LMIN          -         HIGHEST LAYER WHERE INSTABILITY OCCURS
C    LMINQ         -         HIGHEST LAYER WHERE TURBULENCE OCCURS
C
C   SUBPROGRAMS NEEDED ::
C
C     TRBITP -  INTERPOLATES TO HEIGHT WHERE RI = RICR
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
 
#if macro
cmic$ taskcommon mspace 
#endif
      COMMON /MSPACE/ WRKFLX (ISTRIP,36*NLAYR+31)
      COMMON /MSPACE/ WRKLEN (ISTRIP,10*NLAYR-7 )
      COMMON /MSPACE/ WRKTRB (ISTRIP,5*NLAYR-5 )
      COMMON /MSPACE/ WRKSF1 (ISTRIP,37         )
      COMMON /MSPACE/ WRKSF2 (ISTRIP,42         )
      COMMON /MSPACE/ WRKSF3 (ISTRIP,26         )
 
      DIMENSION STRT(ISTRIP,NLEV),   DW2(ISTRIP,NLEV), DZ3(ISTRIP,NLEV)
      DIMENSION DTHV(ISTRIP,NLEV),   DPK(ISTRIP,NLEV),  DU(ISTRIP,NLEV)
      DIMENSION   DV(ISTRIP,NLEV),     Q(ISTRIP,NLEV)
      DIMENSION VKZM(ISTRIP,NLEV-1),VKZE(ISTRIP,NLEV-1)
      DIMENSION QXLM(ISTRIP,NLEV-1),  XL(ISTRIP,NLEV-1)
C
      DIMENSION DZITRP(ISTRIP,NLAYR-1), STBFCN(ISTRIP,NLAYR)
      DIMENSION    XL0(ISTRIP,NLAYR),       Q1(ISTRIP,NLAYR-1)
      DIMENSION WRKIT1(ISTRIP,NLAYR-1)
      DIMENSION WRKIT2(ISTRIP,NLAYR-1)
      DIMENSION WRKIT3(ISTRIP,NLAYR-1)
      DIMENSION WRKIT4(ISTRIP,NLAYR-1)
C
      INTEGER INT1(ISTRIP,NLAYR), INT2(ISTRIP,NLAYR-1)
C
      EQUIVALENCE( DZITRP(1,1), WRKLEN(1,1          ) )
      EQUIVALENCE( STBFCN(1,1), WRKLEN(1,1+  NLAYR-1) )
      EQUIVALENCE(    XL0(1,1), WRKLEN(1,1+2*NLAYR-1) )
      EQUIVALENCE(     Q1(1,1), WRKLEN(1,1+3*NLAYR-1) )
      EQUIVALENCE( WRKIT1(1,1), WRKLEN(1,1+4*NLAYR-2) )
      EQUIVALENCE( WRKIT2(1,1), WRKLEN(1,1+5*NLAYR-3) )
      EQUIVALENCE( WRKIT3(1,1), WRKLEN(1,1+6*NLAYR-4) )
      EQUIVALENCE( WRKIT4(1,1), WRKLEN(1,1+7*NLAYR-5) )
      NLEVM1 = NLEV - 1
      NLEVP1 = NLEV + 1
      ISTNLV = ISTRIP * NLEV
      ISTNM1 = ISTRIP * NLEVM1
C
      IF ( INIT.EQ.2 ) GO TO 1200
C
C COMPUTE DEPTHS OF UNSTABLE LAYERS
C =================================
      DO 10 I=1,ISTNLV
          STBFCN(I,1) = STRT(I,1) - RICR * DW2(I,1)
                                INT1(I,1) = 0
      IF( STBFCN(I,1).LE.ZERO ) INT1(I,1) = 1
   10 CONTINUE
      DO 20 I=1,ISTNM1
                                                INT2(I,1) = 0
      IF( INT1(I,1).EQ.1 .XOR. INT1(I,2).EQ.1 ) INT2(I,1) = 1
   20 CONTINUE
C
      DO 40 LMIN = 1,NLEV
      IBIT = 0
      DO 30 I=1,ISTRIP
      IBIT = IBIT + INT1(I,LMIN)
   30 CONTINUE
      IF(IBIT.GE.1) GO TO 50
   40 CONTINUE
      LMIN = NLEVP1
   50 CONTINUE
C
      DO 60 I=1,ISTNM1
      XL0(I,1) = ZERO
   60 CONTINUE
      DO 70 I=1,ISTRIP
      XL0(I,NLEV) = DZ3(I,NLEV)
   70 CONTINUE
C
      IF(LMIN.GE.NLEVP1) GOTO 1100
                         LMIN1 = LMIN - 1
      IF(LMIN1.EQ.0)     LMIN1 = 1
         NLEVML = NLEV - LMIN1
         ISTNML = ISTRIP*NLEVML
      CALL TRBITP ( STBFCN(1,LMIN1),INT2(1,LMIN1),DTHV(1,LMIN1),
     .                 DPK(1,LMIN1),  DU(1,LMIN1),  DV(1,LMIN1),
     .              DZITRP(1,LMIN1),  NLEVML,
     .              WRKIT1,WRKIT2,WRKIT3,WRKIT4,CP )
      LP1 = LMIN1 + 1
C
      DO 80 I=1,ISTNML
                                                     INT2(I,LMIN1) = 0
      IF( INT1(I,LMIN1).EQ.1 .OR. INT1(I,LP1).EQ.1 ) INT2(I,LMIN1) = 1
      IF( INT2(I,LMIN1).EQ.1 )
     .     XL0(I,LMIN1) = (AHALF+DZITRP(I,LMIN1)) * DZ3(I,LP1)
   80 CONTINUE
      DO 90 I=1,ISTRIP
      INT2(I,NLEVM1) = INT1(I,NLEV)
   90 CONTINUE
C
      DO 100 I=1,ISTNML
      IF( INT2(I,LMIN1).EQ.1 ) THEN
      XL0(I,LP1) = XL0(I,LP1) + ( (AHALF-DZITRP(I,LMIN1)) * DZ3(I,LP1) )
      ENDIF
  100 CONTINUE
      IF (LMIN.GT.1) GOTO 400
      DO 110 I=1,ISTRIP
      IF( INT1(I,1).EQ.1 ) XL0(I,1) = XL0(I,1) + DZ3(I,1)
  110 CONTINUE
  400 CONTINUE
C
      LMINP = LMIN + 1
      IF(LMINP.GT.NLEVM1) GOTO 550
      DO 500 L = LMINP,NLEVM1
           LM1 = L-1
      DO 120 I = 1,ISTRIP
      IF( INT1(I,LM1).EQ.1 ) XL0(I,L) = XL0(I,L) + XL0(I,LM1)
  120 CONTINUE
  500 CONTINUE
  550 CONTINUE
      IF(LMIN.GT.NLEVM1) GOTO 600
      DO 130 I = 1,ISTRIP
      IF( INT1(I,NLEVM1).EQ.1 .AND. INT1(I,NLEV).EQ.1 ) THEN
      XL0(I,NLEV) = XL0(I,NLEV) + XL0(I,NLEVM1)
      ENDIF
  130 CONTINUE
      IF(LMIN.GT.NLEV) GOTO 1100
  600 CONTINUE
      DO 1000 LL  = LMIN,NLEV
              L   = NLEVM1 + LMIN - LL
              LP1 = L+1
      DO 140 I = 1,ISTRIP
      IF( INT1(I,LP1).EQ.1 ) THEN
      IF( INT1(I,L)  .EQ.1 ) THEN
      XL0(I,L) = XL0(I,LP1)
      ELSE
      XL0(I,L) = XL0(I,L) + XL0(I,LP1)
      ENDIF
      ENDIF
  140 CONTINUE
 1000 CONTINUE
 1100 CONTINUE
C
      DO 150 I = 1,ISTNLV
      IF( XL0(I,1).LT.XL0CNV ) XL0(I,1) = XL0CNV
  150 CONTINUE
C
C *********************************************************************
C ****          DETERMINE MIXING LENGTHS FOR STABLE LAYERS          ***
C *********************************************************************
C
      IF(INIT.EQ.1) GOTO 1400
C
      IF(LMINQ.GT.1) THEN
                    ISTLMQ = ISTRIP * LMINQ1
      DO 160 I  = 1,ISTLMQ
      INT2(I,1) = 1 - INT1(I,1)
  160 CONTINUE
      ENDIF
      IF(LMINQ.LT.NLEV) THEN
        ISTNMQ = ISTRIP * (NLEV-LMINQ)
      DO 170 I = 1,ISTNMQ
      IF( INT1(I,LMINQ).EQ.0 ) THEN
           XL0(I,LMINQ) =         Q(I,LMINQ) / XL0(I,LMINQ)
           XL0(I,LMINQ) =       XL0(I,LMINQ) * XL0(I,LMINQ) + Z1PEM20
           XL0(I,LMINQ) =    STBFCN(I,LMINQ) + XL0(I,LMINQ)
           XL0(I,LMINQ) = SQRT( XL0(I,LMINQ) )
           XL0(I,LMINQ) =         Q(I,LMINQ) / XL0(I,LMINQ)
      ENDIF
                                   INT2(I,LMINQ)  = 0
      IF( XL0(I,LMINQ).LT.XL0MIN ) INT2(I,LMINQ)  = 1
  170 CONTINUE
      ENDIF
C
 1200 CONTINUE
C
      IF(INIT.EQ.2) THEN
      DO 180 I = 1,ISTNM1
      INT2(I,1) = 1 - INT1(I,1)
  180 CONTINUE
      ENDIF
      DO 190 I = 1,ISTNM1
      IF( INT2(I,1).EQ.1 ) XL0(I,1) = XL0MIN
  190 CONTINUE
C
C *********************************************************************
C ****             LENGTH SCALE XL FROM XL0 AND VKZE              ****
C *********************************************************************
C
 1400 CONTINUE
C
      DO 200 I = 1,ISTNM1
      XL(I,1) = XL0(I,1) * VKZE(I,1) / ( XL0(I,1)+VKZE(I,1) )
  200 CONTINUE
C
C *********************************************************************
C ****       CLMT53 TIMES Q TIMES LENGTH SCALE AT MID LEVELS        ***
C *********************************************************************
C
      IF(INIT.EQ.1) GOTO 1700
                   ISTNMQ = ISTRIP * (NLEV-LMINQ1)
      DO 210 I = 1,ISTNMQ
                                              Q1(I,LMINQ1) = Q(I,LMINQ1)
                                            INT1(I,LMINQ1) = 0
      IF(    Q(I,LMINQ1).LE.Q(I,LMINQ1+1) ) INT1(I,LMINQ1) = 1
      IF( INT1(I,LMINQ1).EQ.1 )  THEN
           XL0(I,LMINQ1) = XL0(I,LMINQ1+1)
            Q1(I,LMINQ1) =   Q(I,LMINQ1+1)
      ENDIF
  210 CONTINUE
C
      DO 240 I = 1,ISTNMQ
      QXLM(I,LMINQ1) =   XL0(I,LMINQ1)*VKZM(I,LMINQ1)
     .               / ( XL0(I,LMINQ1)+VKZM(I,LMINQ1) )
      QXLM(I,LMINQ1) = CLMT53 * Q1(I,LMINQ1)*QXLM(I,LMINQ1)
  240 CONTINUE
C
 1700 CONTINUE
C
      RETURN
      END
      SUBROUTINE TRBITP ( STBFCN,INTCHG,DTHV,DPK,DU,DV,DZITRP,NLEV,
     .                    AAA,BBB,CCC,DDD,CP )
C**********************************************************************
C
C  SUBROUTINE TRBITP - INTERPOLATES TO THE HEIGHT WHERE RI EQUALS RICR
C                    - CALLED FROM TRBLEN
C   ARGUMENTS ::
C
C     INPUT:
C     ------
C    STBFCN        -         DTHV * DPK - RICR*( DU*DU + DV*DV)
C    INTCHG        -         INT '1' AT LEVELS WHERE STBFCN CHANGES SIG
C    DTHV          -         VERTICAL GRADIENT OF THV
C    DPK           -         VERTICAL GRADIENT OF PK
C    DU            -         VERTICAL GRADIENT OF U
C    DV            -         VERTICAL GRADIENT OF V
C    NLEV          -         NUMBER OF LEVELS TO BE PROCESSED
C
C     OUTPUT:
C     -------
C    DZITRP        -         INTERPOLATION COEFFICIENT
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
      DIMENSION STBFCN(ISTRIP,NLEV), INTCHG(ISTRIP,NLEV)
      DIMENSION DTHV  (ISTRIP,NLEV), DPK   (ISTRIP,NLEV)
      DIMENSION DU    (ISTRIP,NLEV), DV    (ISTRIP,NLEV)
      DIMENSION DZITRP(ISTRIP,NLEV)
      DIMENSION AAA   (ISTRIP,NLEV), BBB   (ISTRIP,NLEV)
      DIMENSION CCC   (ISTRIP,NLEV), DDD   (ISTRIP,NLEV)
C
C *********************************************************************
C ****           QUADRATIC INTERPOLATION OF RI TO RICR VIA          ***
C ****           LINEAR INTERPOLATION OF DTHV, DPK, DU & DV         ***
C *********************************************************************
C
      ISTNLV = ISTRIP*NLEV
      DO 10 I=1,ISTNLV
      DZITRP(I,1) = ZERO
   10 CONTINUE
      DO 20 I=1,ISTNLV
      IF( INTCHG(I,1).EQ.1 ) THEN
      DDD(I,1) = (         CP  *(DTHV(I,2)*DPK(I,1)
     .                         + DTHV(I,1)*DPK(I,2)) )
     .         - ( (TWO*RICR)  * ( DU(I,2)* DU(I,1)
     .                         +   DV(I,2)* DV(I,1)) )
      AAA(I,1) = STBFCN(I,1) + STBFCN(I,2)
      BBB(I,1) = STBFCN(I,1) - STBFCN(I,2)
      CCC(I,1) =            ONE / BBB(I,1)
      DZITRP(I,1) = AAA(I,1) * CCC(I,1)
         AAA(I,1) = AAA(I,1) - DDD(I,1)
         DDD(I,1) =        (   DDD(I,1) *    DDD(I,1) )
     .            - FOUR * (STBFCN(I,2) * STBFCN(I,1) )
         DDD(I,1) = DDD(I,1)*CCC(I,1)*CCC(I,1)
         DDD(I,1) =    SQRT( DDD(I,1) )
      ENDIF
C
      IF( INTCHG(I,1).EQ.1 .AND. AAA(I,1).NE.ZERO ) THEN
      DZITRP(I,1) = ( BBB(I,1)*(ONE-DDD(I,1)) ) / AAA(I,1)
      ENDIF
C
      DZITRP(I,1) = AHALF * DZITRP(I,1)
   20 CONTINUE
C
      RETURN
      END
      SUBROUTINE TRBL20 ( RI,STRT,DW2,XL,ZKM,ZKH,QE,QQE,INTSTB,NLEV )
C**********************************************************************
C
C  SUBROUTINE TRBL20 - COMPUTES QE AND DIMLESS COEFS FROM
C                       MELLOR-YAMADA LEVEL 2 MODEL
C                    - CALLED FROM AND FROM TRBFLX
C   ARGUMENTS ::
C
C     INPUT:
C     ------
C    RI            -         RICHARDSON NUMBER
C    STRT          -         BRUNT VAISALA FREQUENCY
C    DW2           -         SQUARED SHEAR
C    XL            -         TURBULENT LENGTH SCALE
C    NLEV          -         NUMBER OF LEVELS TO BE PROCESSED
C
C     OUTPUT:
C     -------
C    ZKM           -         MOMENTUM TRANSPORT COEFFICIENT
C    ZKH           -         HEAT TRANSPORT COEFFICIENT
C    QE            -         EQUILIBRIUM TURBULENT VELOCITY SCALE
C    QQE           -         EQUILIBRIUM TURBULENT KINETIC ENERGY
C    BITSTB        -         BIT '1' WHERE QE GREATER THAN ZERO
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
 
#if macro
cmic$ taskcommon mspace 
#endif
      COMMON /MSPACE/ WRKFLX (ISTRIP,36*NLAYR+31)
      COMMON /MSPACE/ WRKLEN (ISTRIP,10*NLAYR-7 )
      COMMON /MSPACE/ WRKTRB (ISTRIP,5*NLAYR-5 )
      COMMON /MSPACE/ WRKSF1 (ISTRIP,37         )
      COMMON /MSPACE/ WRKSF2 (ISTRIP,42         )
      COMMON /MSPACE/ WRKSF3 (ISTRIP,26         )
 
      DIMENSION RI(ISTRIP,NLEV), STRT(ISTRIP,NLEV), DW2(ISTRIP,NLEV)
      DIMENSION XL(ISTRIP,NLEV),  ZKM(ISTRIP,NLEV), ZKH(ISTRIP,NLEV)
      DIMENSION QE(ISTRIP,NLEV),  QQE(ISTRIP,NLEV)
      INTEGER INTSTB(ISTRIP,NLAYR)
      DIMENSION     EE(ISTRIP,NLAYR-1),  RF(ISTRIP,NLAYR-1)
      EQUIVALENCE ( EE(1,1), WRKTRB(1,1        ) )
      EQUIVALENCE ( RF(1,1), WRKTRB(1,1+NLAYR-1) )
C
      ISTNLV = ISTRIP * NLEV
C
C *********************************************************************
C ****               COMPUTE FLUX RICHARDSON NUMBER                 ***
C *********************************************************************
C
      DO 10 I=1,ISTNLV
      EE(I,1) = D1HALF * RI(I,1) + D2HALF
      RF(I,1) = EE(I,1)* EE(I,1)
      RF(I,1) = RF(I,1)- D3*RI(I,1)
      RF(I,1) = SQRT( RF(I,1) )
      RF(I,1) = EE(I,1) - RF(I,1)
C
      IF( RI(I,1).LE.Z1PEM4 .AND. RI(I,1).GE.ZM1PEM4 ) THEN
          RF(I,1) = D3B2*RI(I,1)
      ENDIF
C
C *********************************************************************
C ****           QE AND DIMENSIONLESS DIFFUSION COEFICIENTS         ***
C ****                   FROM LEVEL 2 CLOSURE MODEL                 ***
C *********************************************************************
C
      IF( RI(I,1).LT.RICR .AND. RF(I,1).LT.RFC ) THEN
      ZKH(I,1) =   ( RFC-RF(I,1) ) / (ONE-RF(I,1))
      ZKM(I,1) =                CM * (RF1-RF(I,1))
      ZKM(I,1) = ZKH(I,1)*ZKM(I,1) / (RF2-RF(I,1))
      ZKH(I,1) = CH      *ZKH(I,1)
       QE(I,1) = ZKM(I,1)*DW2(I,1) - ZKH(I,1)*STRT(I,1)
      ENDIF
C
      IF( QE(I,1).LT.Z1PEM14 ) THEN
      INTSTB(I,1) = 0
          QE(I,1) = ZERO
      ELSE
      INTSTB(I,1) = 1
          QE(I,1) = B1*QE(I,1)
          QE(I,1) = SQRT( QE(I,1) )
          QE(I,1) = XL(I,1)*QE(I,1)
      ENDIF
      QQE(I,1) = AHALF * QE(I,1) * QE(I,1)
   10 CONTINUE
C
      RETURN
      END
      SUBROUTINE TRBL25 ( Q,XL,STRT,DW2,INTSTB,INTQ,ZKM,ZKH,P3,NLEV )
C**********************************************************************
C
C  SUBROUTINE TRBL25 - COMPUTES P3 AND DIMLESS COEFS FROM
C                       MELLOR-YAMADA LEVEL 2.5 MODEL
C                    - CALLED FROM TRBFLX
C
C   ARGUMENTS ::
C
C     INPUT:
C     ------
C    Q             -         TURBULENCE VELOCITY
C    XL            -         TURBULENT LENGTH SCALE
C    STRT          -         BRUNT VAISALA FREQUENCY
C    DW2           -         SQUARED SHEAR
C    BITSTB        -         BIT '1' WHERE QE GREATER THAN ZERO
C    NLEV          -         NUMBER OF LEVELS TO BE PROCESSED
C
C     OUTPUT:
C     -------
C    ZKM           -         MOMENTUM TRANSPORT COEFFICIENT
C    ZKH           -         HEAT TRANSPORT COEFFICIENT
C    P3            -         PRODUCTION RATE OF TURBULENT KINETIC ENERG
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
 
#if macro
cmic$ taskcommon mspace 
#endif
      COMMON /MSPACE/ WRKFLX (ISTRIP,36*NLAYR+31)
      COMMON /MSPACE/ WRKLEN (ISTRIP,10*NLAYR-7 )
      COMMON /MSPACE/ WRKTRB (ISTRIP,5*NLAYR-5 )
      COMMON /MSPACE/ WRKSF1 (ISTRIP,37         )
      COMMON /MSPACE/ WRKSF2 (ISTRIP,42         )
      COMMON /MSPACE/ WRKSF3 (ISTRIP,26         )
 
      DIMENSION   Q(ISTRIP,NLEV),  XL(ISTRIP,NLEV), STRT(ISTRIP,NLEV)
      DIMENSION DW2(ISTRIP,NLEV)
      DIMENSION ZKM(ISTRIP,NLEV), ZKH(ISTRIP,NLEV),   P3(ISTRIP,NLEV)
C
      DIMENSION F2(ISTRIP,NLAYR-1),  F3(ISTRIP,NLAYR-1)
      DIMENSION F4(ISTRIP,NLAYR-1),  XQ(ISTRIP,NLAYR-1)
      EQUIVALENCE ( F2(1,1), WRKTRB(1,1          ) )
      EQUIVALENCE ( F3(1,1), WRKTRB(1,1+  NLAYR-1) )
      EQUIVALENCE ( F4(1,1), WRKTRB(1,1+2*NLAYR-2) )
      EQUIVALENCE ( XQ(1,1), WRKTRB(1,1+3*NLAYR-3) )
      INTEGER INTSTB(ISTRIP,NLAYR), INTQ(ISTRIP,NLAYR)
C
      ISTNLV = ISTRIP * NLEV
C
C *********************************************************************
C ****           P3 AND DIMENSIONLESS DIFFUSION COEFICIENTS         ***
C ****                  FROM LEVEL 2.5 CLOSURE MODEL                ***
C *********************************************************************
C
      DO 10 I=1,ISTNLV
      IF( INTQ(I,1).EQ.1 .AND. INTSTB(I,1).EQ.0 ) THEN
        XQ(I,1) = XL(I,1)  /      Q(I,1)
        XQ(I,1) = XQ(I,1)  *     XQ(I,1)
      STRT(I,1) = XQ(I,1)  *   STRT(I,1)
       DW2(I,1) = XQ(I,1)  *    DW2(I,1)
        F2(I,1) = ONE+FF2  *   STRT(I,1)
        F3(I,1) = ONE+FF3  *   STRT(I,1)
        F4(I,1) = ONE+FF4  *   STRT(I,1)
       ZKH(I,1) = (F4(I,1) *     F2(I,1))
     .     + A4 * (F3(I,1) *    DW2(I,1))
       ZKH(I,1) = (F2(I,1) + A3*DW2(I,1))
     .          / ZKH(I,1)
       ZKM(I,1) = A1 * (F3(I,1)*ZKH(I,1)+A5)
     .               /  F2(I,1)
       ZKH(I,1) = A2 * ZKH(I,1)
        P3(I,1) = ZKH(I,1)*STRT(I,1) + B3
        P3(I,1) = TWO * ( ZKM(I,1)*DW2(I,1) - P3(I,1) )
        P3(I,1) = P3(I,1)*Q(I,1)
C
      ENDIF
   10 CONTINUE
C
      RETURN
      END
      SUBROUTINE TRBDIF ( XX1,XX2,RHOKDZ,FLXFAC,DXX1G,DXX2G,NLEV,
     .                    ITYPE,EPSL )
C
C**********************************************************************
C
C   ARGUMENTS ::
C
C     INPUT:
C     ------
C    XX1           -         FIRST PROPERTY TO BE DIFFUSED
C                              (INPUT INCLUDES FORWARD PRODUCTION TERM)
C    XX2           -         SECOND PROPERTY TO BE DIFFUSED (V-WIND)
C                              (INPUT INCLUDES FORWARD PRODUCTION TERM)
C                                             -OR-
C                            CHANGE IN XX1 DUE TO UNIT CHANGE IN THG
C                               (TH OR SH PROFILES)
C                                             -OR-
C                            BACKWARD PRODUCTION TERM (QQ)
C    RHOKDZ        -         RHO * K * WEIGHT / DZ AT INTERFACES
C    FLXFAC        -         G * DT / (DP*WEIGHT) AT EDGES
C    NLEV          -         NUMBER OF ATMOSPHERIC LEVELS
C    ITYPE         -         INTEGER FLAG FOR INPUT TYPE
C                              1 = QQ: COMPUTE BACKWARD PRODUCTION AND
C                                  USE UNDERFLOW CUTOFF
C                              2 = TH OR SH: COMPUTE TENDENCY DUE TO
C                                  SURFACE PERTURBATION
C                              3 = U AND V: COMPUTE BOTH FIELDS
C    EPSL          -         UNDERFLOW CUTOFF CRITERION (QQ ONLY)
C
C     OUTPUT:
C     ------
C    XX1           -         NEW VALUE RETURNED
C    XX2           -         NEW VALUE RETURNED
C    DXX1G         -         SOURCE TERM FOR XX1 AT GROUND
C    DXX1G         -         SOURCE TERM FOR XX2 AT GROUND
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
 
#if macro
cmic$ taskcommon mspace 
#endif
      COMMON /MSPACE/ WRKFLX (ISTRIP,36*NLAYR+31)
      COMMON /MSPACE/ WRKLEN (ISTRIP,10*NLAYR-7 )
      COMMON /MSPACE/ WRKTRB (ISTRIP,5*NLAYR-5 )
      COMMON /MSPACE/ WRKSF1 (ISTRIP,37         )
      COMMON /MSPACE/ WRKSF2 (ISTRIP,42         )
      COMMON /MSPACE/ WRKSF3 (ISTRIP,26         )
 
      DIMENSION    XX1(ISTRIP,NLEV+1),    XX2(ISTRIP,NLEV+1)
      DIMENSION RHOKDZ(ISTRIP,NLEV)  , FLXFAC(ISTRIP,NLEV+1)
      DIMENSION  DXX1G(ISTRIP)    ,     DXX2G(ISTRIP)
C
      DIMENSION AA(ISTRIP,NLAYR), BB(ISTRIP,NLAYR), CC(ISTRIP,NLAYR+1)
C
      EQUIVALENCE ( AA(1,1), WRKTRB(1,1         ) )
      EQUIVALENCE ( BB(1,1), WRKTRB(1,1+  NLAYR ) )
      EQUIVALENCE ( CC(1,1), WRKTRB(1,1+2*NLAYR ) )
      ISTNLV = ISTRIP * NLEV
      ISTNM1 = ISTNLV - ISTRIP
      NLEVP1 = NLEV + 1
      ISTNLX = ISTNM1
      IF(ITYPE.EQ.2) ISTNLX = ISTNLV
C
C   DEFINE MATRIX
C
      DO 10 I=1,ISTRIP
      CC(I,1) = ZERO
   10 CONTINUE
      DO 20 I=1,ISTNLX
      CC(I,2) = RHOKDZ(I,1) * FLXFAC(I,2)
   20 CONTINUE
      DO 30 I=1,ISTNLV
      BB(I,1) =   RHOKDZ(I,1) * FLXFAC(I,1)
      AA(I,1) = ONE + CC(I,1) + BB(I,1)
   30 CONTINUE
C
C     ADD IMPLICIT BACKWARD FORCING FOR QQ
      IF(ITYPE.EQ.1) THEN
      DO 40 I=1,ISTNLV
      AA(I,1) = AA(I,1) - XX2(I,1)
   40 CONTINUE
      ENDIF
C
C     SOLVE MATRIX EQUATION FOR XX1
      CALL VTRI0(AA,BB,CC,XX1,XX1,NLEV)
C
      IF(ITYPE.EQ.2) THEN
C     COMPUTE CHANGE AT SURFACE
C
      DO 50 I=1,ISTRIP
      DXX1G(I) = CC(I,NLEVP1) * ( XX1(I,NLEV)-XX1(I,NLEVP1) )
   50 CONTINUE
C
C     SOLVE MATRIX FOR SURFACE PERTURBATION
      CALL VTRI1(AA,BB,XX2,NLEV)
      DO 60 I=1,ISTRIP
      DXX2G(I) = CC(I,NLEVP1) * ( XX2(I,NLEV)-XX2(I,NLEVP1) )
   60 CONTINUE
      ENDIF
C
C     SOLVE MATRIX EQUATION FOR XX2
C
      IF(ITYPE.EQ.3) CALL VTRI2 (AA,BB,CC,XX2,XX2,NLEV)
C
C     ELIMINATE UNDERFLOW
      IF(ITYPE.EQ.1) THEN
      DO 70 I=1,ISTNLV
      IF( XX1(I,1).LT.EPSL ) XX1(I,1) = ZERO
   70 CONTINUE
      ENDIF
C
      RETURN
      END
      SUBROUTINE VTRI0 ( A,B,C,F,Y,K)
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
      DIMENSION A(ISTRIP,K),B(ISTRIP,K),C(ISTRIP,K),Y(ISTRIP,K+1)
      DIMENSION F(ISTRIP,K)
C
      DO 9000 I = 1,ISTRIP
       A(I,1) = ONE / A(I,1)
 9000 CONTINUE
C
      DO 100 L = 2,K
       LM1 = L - 1
       DO 9002 I = 1,ISTRIP
        C(I,L) = C(I,L) * A(I,LM1)
        A(I,L) = ONE / ( A(I,L) - B(I,LM1) * C(I,L) )
        F(I,L) = F(I,L) + F(I,LM1) * C(I,L)
 9002  CONTINUE
 100  CONTINUE
C
      DO 200 L = K,1,-1
       DO 9004 I = 1,ISTRIP
       Y(I,L) = (F(I,L) + B(I,L) * Y(I,L+1)) * A(I,L)
 9004  CONTINUE
 200  CONTINUE
C
      RETURN
      END
C
      SUBROUTINE VTRI1 ( A,B,Y,K)
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
      DIMENSION A(ISTRIP,K),B(ISTRIP,K),Y(ISTRIP,K+1)
C
      DO 200 L = K,1,-1
       DO 9000 I = 1,ISTRIP
        Y(I,L) = B(I,L) * Y(I,L+1) * A(I,L)
 9000  CONTINUE
 200  CONTINUE
C
      RETURN
      END
C
      SUBROUTINE VTRI2 ( A,B,C,F,Y,K)
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
      DIMENSION A(ISTRIP,K),B(ISTRIP,K),C(ISTRIP,K),F(ISTRIP,K)
      DIMENSION Y(ISTRIP,K+1)
C
      DO 100 L = 2,K
       DO 9000 I = 1,ISTRIP
        F(I,L) = F(I,L) + F(I,L-1) * C(I,L)
 9000  CONTINUE
 100  CONTINUE
C
      DO 200 L = K,1,-1
       DO 9002 I = 1,ISTRIP
        Y(I,L) = (F(I,L) + B(I,L) * Y(I,L+1)) *  A(I,L)
 9002  CONTINUE
 200  CONTINUE
C
      RETURN
      END
      SUBROUTINE LINADJ ( NN,VRIB1,VRIB2,VWS1,VWS2,VZ1,VUSTAR,IWATER,
     1 VAPSIM, VAPSIHG,VPSIH,VPSIG,VX,VX0,VY,VY0,ITYPE,LWATER,IRUN,
     1 VDZETA,VDZ0,VDPSIM,VDPSIH,INTRIB,WRKZSEA)
C
C**********************************************************************
C
C  ARGUMENTS ::
C
C     INPUT:
C     ------
C    RIB1          -         BULK RICHARDSON NUMBER OF INPUT STATE
C    RIB2          -         DESIRED BULK RICH NUMBER OF OUTPUT STATE
C    WS1           -         SURFACE WIND SPEED OF INPUT STATE
C    WS2           -         DESIRED SURFACE WIND SPEED OF OUTPUT STATE
C    Z1            -         INPUT VALUE OF ROUGHNESS HEIGHT
C    USTAR         -         INPUT VALUE OF CU * WS
C    WATER         -         BIT ARRAY - '1' WHERE OCEAN
C    APSIM         -         (1/PSIM)
C    APSIHG        -         ( 1 / (PSIH+PSIG) )
C    PSIH          -         NON-DIM TEMP GRADIENT
C    PSIG          -         PSIH FOR THE MOLECULAR LAYER
C    X             -         PHIM(ZETA) - DERIVATIVE OF PSIM
C    X0            -         PHIM(ZETA0)
C    Y             -         PHIH(ZETA) - DERIVATIVE OF PSIH
C    Y0            -         PHIH(ZETA0)
C    ITYPE         -         INTEGER FLAG :
C                               1 = NEUTRAL ADJUSTMENT
C                               2 = ADJ FOR 2ND OR GREATER TRBFLX ITER
C                               3 - 5 = ADJUSTMENT INSIDE LOOP
C                               4 - 5 = ADJUST CU AND CT
C                               5 = PREPARATION FOR ITYPE = 2
C    LWATER        -         LOGICAL - .TRUE. IF THERE ARE WATER POINTS
C
C     OUTPUT:
C     -------
C    DZETA         -         D LOG ZETA
C    DZ0           -         D Z0 (ITYPE 1) OR D LOG Z0 (ITYPE 2-5)
C    DPSIM         -         D PSIM
C    DPSIH         -         D PSIH
C    BITRIB        -         BIT ARRAY - '1' WHERE RIB1 = 0
C
C**********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
      DIMENSION VRIB1(IRUN),VRIB2(IRUN)
      DIMENSION VWS1(IRUN),VWS2(IRUN),VZ1(IRUN),VUSTAR(IRUN)
      DIMENSION VAPSIM(IRUN),VAPSIHG(IRUN)
      DIMENSION VPSIH(IRUN),VPSIG(IRUN),VX(IRUN)
      DIMENSION VX0(IRUN),VY(IRUN),VY0(IRUN)
      DIMENSION VDZETA(IRUN),VDZ0(IRUN),VDPSIM(IRUN)
      DIMENSION VDPSIH(IRUN)
      DIMENSION IWATER(IRUN),INTRIB(IRUN)
      LOGICAL LWATER
      DIMENSION WRKZSEA(ISTRIP,13)
C
      PARAMETER (IMJMR = IMR * (JMR+1) )
 
#if macro
cmic$ taskcommon mspace 
#endif
      COMMON /MSPACE/ WRKFLX (ISTRIP,36*NLAYR+31)
      COMMON /MSPACE/ WRKLEN (ISTRIP,10*NLAYR-7 )
      COMMON /MSPACE/ WRKTRB (ISTRIP,5*NLAYR-5 )
      COMMON /MSPACE/ WRKSF1 (ISTRIP,37         )
      COMMON /MSPACE/ WRKSF2 (ISTRIP,42         )
      COMMON /MSPACE/ WRKSF3 (ISTRIP,26         )
 
      DIMENSION VX0PSIM(ISTRIP),VG(ISTRIP),VG0(ISTRIP),VR1MG0(ISTRIP)
      DIMENSION VZ2(ISTRIP),VDZSEA(ISTRIP),VAZ0(ISTRIP),VXNUM1(ISTRIP)
      DIMENSION VPSIGB2(ISTRIP),VDX(ISTRIP),VDXPSIM(ISTRIP),VDY(ISTRIP)
      DIMENSION VXNUM2(ISTRIP),VDEN(ISTRIP),VAWS1(ISTRIP),VXNUM3(ISTRIP)
      DIMENSION VXNUM(ISTRIP),VDZETA1(ISTRIP),VDZETA2(ISTRIP)
      DIMENSION VZCOEF2(ISTRIP),VZCOEF1(ISTRIP),VTEMP(ISTRIP)
      DIMENSION VDPSIMC(ISTRIP),VDPSIHC(ISTRIP)
      DIMENSION VINT1(ISTRIP),VINT2(ISTRIP)
C
       EQUIVALENCE( WRKSF3(1, 1),VX0PSIM(1))
       EQUIVALENCE( WRKSF3(1, 2),     VG(1))
       EQUIVALENCE( WRKSF3(1, 3),    VG0(1))
       EQUIVALENCE( WRKSF3(1, 4), VR1MG0(1))
       EQUIVALENCE( WRKSF3(1, 5),    VZ2(1))
       EQUIVALENCE( WRKSF3(1, 6), VDZSEA(1))
       EQUIVALENCE( WRKSF3(1, 7),   VAZ0(1))
       EQUIVALENCE( WRKSF3(1, 8), VXNUM1(1))
       EQUIVALENCE( WRKSF3(1, 9),VPSIGB2(1))
       EQUIVALENCE( WRKSF3(1,10),    VDX(1))
       EQUIVALENCE( WRKSF3(1,11),VDXPSIM(1))
       EQUIVALENCE( WRKSF3(1,12),    VDY(1))
       EQUIVALENCE( WRKSF3(1,13), VXNUM2(1))
       EQUIVALENCE( WRKSF3(1,14),   VDEN(1))
       EQUIVALENCE( WRKSF3(1,15),  VAWS1(1))
       EQUIVALENCE( WRKSF3(1,16), VXNUM3(1))
       EQUIVALENCE( WRKSF3(1,17),  VXNUM(1))
       EQUIVALENCE( WRKSF3(1,18),VDZETA1(1))
       EQUIVALENCE( WRKSF3(1,19),VDZETA2(1))
       EQUIVALENCE( WRKSF3(1,20),VZCOEF2(1))
       EQUIVALENCE( WRKSF3(1,21),VZCOEF1(1))
       EQUIVALENCE( WRKSF3(1,22),  VTEMP(1))
       EQUIVALENCE( WRKSF3(1,23),VDPSIMC(1))
       EQUIVALENCE( WRKSF3(1,24),VDPSIHC(1))
C
C   COMPUTE X0/PSIM, 1/Z0, G, G0, 1/(1-G0),
C     DEL LOG Z0, D LOG ZO / D USTAR
C
CCCOOOMMMM ADDED 'WHERE WATER'
      IF ( (ITYPE.EQ.1) .AND. LWATER ) THEN
       DO 9000 I = 1,IRUN
        IF (IWATER(I).EQ.1) VX0PSIM(I) = VAPSIM(I)
 9000  CONTINUE
      ENDIF
      IF ( ITYPE .GE. 3 ) THEN
       DO 9002 I = 1,IRUN
        VX0PSIM(I) = VX0(I) * VAPSIM(I)
 9002  CONTINUE
      ENDIF
      IF ( ITYPE .NE. 2 ) THEN
C
       DO 9004 I = 1,IRUN
        VDZ0(I) = ZERO
        VG(I) = ZERO
        VG0(I) = ZERO
        VR1MG0(I) = ONE
 9004  CONTINUE
C
       IF ( LWATER ) THEN
        CALL ZCSUB ( VUSTAR,VDZSEA,IWATER,
     1    .TRUE.,WRKZSEA(1,1),WRKZSEA(1,6),
     2     WRKZSEA(1,7),WRKZSEA(1,9),WRKZSEA(1,11) ,IRUN,VZ2)
C
        DO 9006 I = 1,IRUN
         IF ( IWATER(I).EQ.1) THEN
          VAZ0(I) = ONE / VZ1(I)
          VG(I) = VDZSEA(I) * VAZ0(I)
          VG0(I) = VX0PSIM(I) * VG(I)
          VR1MG0(I) = ONE / ( ONE - VG0(I) )
          VDZ0(I) = ( VZ2(I) - VZ1(I) ) * VR1MG0(I)
         ENDIF
 9006   CONTINUE
       ENDIF
      ENDIF
C
      IF ( LWATER .AND. (ITYPE.GE.3) ) THEN
       DO 9008 I = 1,IRUN
        IF (IWATER(I).EQ.1) VDZ0(I) = VDZ0(I) * VAZ0(I)
 9008  CONTINUE
      ENDIF
C
C   COMPUTE NUM1,NUM2,NUM3, DEN
C
      IF (ITYPE.GE.3) THEN
       DO 9010 I = 1,IRUN
        VXNUM1(I) = ZERO
        IF (VRIB1(I).EQ.ZERO) THEN
         INTRIB(I) = 1
        ELSE
         INTRIB(I) = 0
        ENDIF
        IF ( INTRIB(I).EQ.0 ) VXNUM1(I) = ONE / VRIB1(I)
        VPSIGB2(I) = zero
        if(vpsig(i).gt.zero)VPSIGB2(I) =
     1        ahalf * ( vpsig(i)*vpsig(i) + b2uhs ) / vpsig(i)
        VDX(I) = VX(I) - VX0(I)
        VDXPSIM(I) = VDX(I) * VAPSIM(I)
        VDY(I) = VY(I) - VY0(I)
        VXNUM3(I) = - VPSIGB2(I)
C
        IF ( LWATER ) THEN
CCCOOOMMMM ADDED 'WHERE WATER'
         IF (IWATER(I).EQ.1) THEN
          VDXPSIM(I) = VDXPSIM(I) * VR1MG0(I)
          VXNUM3(I) = VXNUM3(I) + VG(I) * ( VY0(I) - VPSIGB2(I) )
          VXNUM2(I) = VY0(I) - VPSIGB2(I) - VX0PSIM(I) * VPSIGB2(I)
          VXNUM2(I) = (VXNUM2(I) * VAPSIHG(I)) - TWO * VX0PSIM(I)
          VXNUM2(I) = VXNUM2(I) * VDZ0(I)
         ENDIF
        ENDIF
C
        VDEN(I) = VDY(I) + VDXPSIM(I) * VXNUM3(I)
        VDEN(I) = ( ONE + VDEN(I) * VAPSIHG(I) ) - TWO * VDXPSIM(I)
 9010  CONTINUE
      ENDIF
C
      IF (ITYPE.EQ.5) THEN
       DO 9012 I = 1,IRUN
        VAWS1(I) = VR1MG0(I) / VWS1(I)
        VXNUM3(I) = VXNUM3(I) * VAPSIHG(I)
C
        IF ( LWATER ) THEN
CCCOOOMMMM ADDED 'WHERE WATER'
         IF(IWATER(I).EQ.1) THEN
          VXNUM3(I) = VXNUM3(I) - TWO * VG0(I)
          VXNUM3(I) = VAWS1(I) * VXNUM3(I)
         ENDIF
        ENDIF
 9012  CONTINUE
      ENDIF
C
C   COMPUTE D LOG ZETA
C
      IF (ITYPE.GE.2) THEN
       DO 9014 I = 1,IRUN
        VXNUM(I) = VRIB2(I) - VRIB1(I)
        IF( (VX0(I).GT.XX0MAX).AND.(VXNUM(I).GE.ZERO) )VXNUM(I) = ZERO
        VXNUM(I) = VXNUM1(I) * VXNUM(I)
 9014  CONTINUE
      ENDIF
C
      IF ( ITYPE.EQ.2 )THEN
       DO 9016 I = 1,IRUN
        VDZETA1(I) = VDZETA(I)
        VXNUM(I) = VXNUM(I) + VXNUM3(I) * ( VWS2(I) - VWS1(I) )
 9016  CONTINUE
      ENDIF
C
      IF (ITYPE.GE.3) THEN
       DO 9018 I = 1,IRUN
        VDZETA1(I) = VXNUM(I)
        IF(LWATER.AND.(IWATER(I).EQ.1)) VXNUM(I) = VXNUM(I) + VXNUM2(I)
        IF ( VDEN(I) .LT.ZP1 ) VDEN(I) = ZP1
 9018  CONTINUE
      ENDIF
C
      IF (ITYPE.GE.2) THEN
       DO 9020 I = 1,IRUN
        VDZETA(I) = VXNUM(I) / VDEN(I)
 9020  CONTINUE
      ENDIF
      IF (ITYPE.GE.3) THEN
       DO 9022 I = 1,IRUN
        IF ( (VRIB2(I).EQ.ZERO) .OR. (VDZETA(I).LE.ONENEG) ) THEN
         VINT1(I) = 1
        ELSE
         VINT1(I) = 0
        ENDIF
        IF ( VINT1(I).EQ.1 ) VDZETA(I) = VDZETA1(I)
 9022  CONTINUE
      ENDIF
      IF (ITYPE.EQ.2) THEN
       DO 9024 I = 1,IRUN
        VDZETA2(I) = VDZETA(I) + VDZETA1(I)
        IF ( (VRIB2(I).EQ.ZERO) .OR. (VDZETA2(I).LE.ONENEG) ) THEN
         VINT1(I) = 1
        ELSE
         VINT1(I) = 0
        ENDIF
        IF(VINT1(I).EQ.1)VDZETA(I)=VXNUM1(I)*VRIB2(I) - ONE - VDZETA1(I)
 9024  CONTINUE
      ENDIF

C
C   COMPUTE D LOG Z0
C
      IF ( LWATER .AND. (ITYPE.GE.3) )THEN
       DO 9026 I = 1,IRUN
        IF( IWATER(I).EQ.1 ) THEN
         VZCOEF2(I) = VG(I) * VDXPSIM(I)
         VDZ0(I) = VDZ0(I) - VZCOEF2(I) * VDZETA(I)
        ENDIF
 9026  CONTINUE
      ENDIF
C
      IF ( LWATER .AND. (ITYPE.EQ.5) ) THEN
       DO 9028 I = 1,IRUN
        IF(IWATER(I).EQ.1) VZCOEF1(I) = VG(I) * VAWS1(I)
 9028  CONTINUE
      ENDIF
C
      IF ( LWATER .AND. (ITYPE.EQ.2) ) THEN
       DO 9030 I = 1,IRUN
        IF (IWATER(I).EQ.1) VDZ0(I) =
     1   VZCOEF1(I) * ( VWS2(I) - VWS1(I) ) - VZCOEF2(I) * VDZETA(I)
 9030  CONTINUE
      ENDIF
C
C   CALCULATE D PSIM AND D PSIH
C
      IF ( (ITYPE.EQ.1) .AND. LWATER ) THEN
       DO 9032 I = 1,IRUN
        IF (IWATER(I).EQ.1) THEN
         VDPSIM(I) = - VDZ0(I) * VAZ0(I)
         VDPSIH(I) = VDPSIM(I)
        ENDIF
 9032  CONTINUE
      ENDIF
C
      IF (ITYPE.GE.3) THEN
       DO 9034 I = 1,IRUN
        VDPSIM(I) = VDX(I) * VDZETA(I)
        VDPSIH(I) = VDY(I) * VDZETA(I)
        IF ( LWATER ) THEN
         IF (IWATER(I).EQ.1 ) THEN
          VDPSIM(I) = VDPSIM(I) - VX0(I) * VDZ0(I)
          VDPSIH(I) = VDPSIH(I) - VY0(I) * VDZ0(I)
         ENDIF
        ENDIF
 9034  CONTINUE
      ENDIF
C
C   PREVENT OVERCORRECTION OF PSIM OR PSIH FOR UNSTABLE CASE
C
      IF (ITYPE.GE.4) THEN
       DO 9036 I = 1,IRUN
        VDPSIMC(I) = ZP9NEG - VDPSIM(I) * VAPSIM(I)
        VDPSIHC(I) = VPSIH(I) * ZP9NEG - VDPSIH(I)
        IF ( VDPSIMC(I).GT.ZERO  ) THEN
         VINT1(I) = 1
        ELSE
         VINT1(I) = 0
        ENDIF
        IF ( VDPSIHC(I).GT.ZERO  ) THEN
         VINT2(I) = 1
        ELSE
         VINT2(I) = 0
        ENDIF
        VDZETA1(I) = ZERO
        IF(VINT1(I).EQ.1) VDZETA1(I) = VDPSIMC(I) / VDXPSIM(I)
        IF((VINT1(I).EQ.1).OR.(VINT2(I).EQ.1)) VTEMP(I) =
     1        VDY(I) + VY0(I) * VG(I) * VDXPSIM(I)
        IF (VINT2(I).EQ.1) then
             VDZETA2(I) =  VDPSIHC(I) / VTEMP(I)
        IF ( VDZETA2(I).LT.VDZETA1(I) ) VDZETA1(I) = VDZETA2(I)
        endif
        IF((VINT1(I).EQ.1).OR.(VINT2(I).EQ.1)) THEN
         VDZETA(I) = VDZETA1(I) + VDZETA(I)
         VDPSIM(I) = VDPSIM(I) + VDX(I) * VR1MG0(I) * VDZETA1(I)
         VDPSIH(I) = VDPSIH(I) + VTEMP(I) * VDZETA1(I)
         IF ( IWATER(I).EQ.1 )
     1     VDZ0(I) = VDZ0(I) - VG(I) * VDXPSIM(I) * VDZETA1(I)
        ENDIF
 9036  CONTINUE
      ENDIF
C
      RETURN
      END
      SUBROUTINE ZCSUB (VUSTAR,VDZSEA,IWATER,LDZSEA,AA,TEMP,INT2,INT3,
     1         INT4,IRUN,VZSEA)
C**********************************************************************
C  FUNCTION ZSEA
C  PURPOSE
C     COMPUTES Z0 AS A FUNCTION OF USTAR OVER WATER SURFACES
C  USAGE
C     CALLED BY SFCFLX
C  DESCRIPTION OF PARAMETERS
C     USTAR -  INPUTED VALUE OF SURFACE-STRESS VELOCITY
C     DZSEA -  OUTPUTED VALUE OF DERIVATIVE  D(ZSEA)/D(USTAR)
C     WATER -  INPUTED BIT VECTOR TO DETERMINE WATER POINTS
C     LDZSEA-  LOGICAL FLAG TO DETERMINE IF DZSEA SHOULD BE COMPUTED
C     ZSEA  -  OUTPUTED VALUE OF ROUGHNESS LENGTH
C  SUBPROGRAMS NEEDED
C     NONE
C  RECORD OF MODIFICATIONS
C  REMARKS:
C        COMPUTE ROUGHNESS LENGTH FOR OCEAN POINTS
C          BASED ON FUNCTIONS OF LARGE AND POND
C          AND OF KONDO --- DESIGNED FOR K = .4
C *********************************************************************
C
#include <vrslv.com>
#include <pblparm.com>
#include <stripmine.com>
C
      DIMENSION VZSEA(IRUN),VUSTAR(IRUN),VDZSEA(IRUN)
      DIMENSION IWATER(IRUN)
      DIMENSION AA(IRUN ,5),TEMP(IRUN)
      LOGICAL LDZSEA
      DIMENSION INT2(IRUN ), INT3(IRUN ), INT4(IRUN)
C
      DIMENSION AA1(5),AA2(5),AA3(5),AA4(5)
      DATA AA1/.2030325E-5,0.0,0.0,0.0,0.0/
      DATA AA2/-0.402451E-08,0.239597E-04,0.117484E-03,0.191918E-03,
     1         0.395649E-04/
      DATA AA3/-0.237910E-04,0.228221E-03,-0.860810E-03,0.176543E-02,
     1         0.784260E-04/
      DATA AA4/-0.343228E-04,0.552305E-03,-0.167541E-02,0.250208E-02,
     1         -0.153259E-03/
C
C**********************************************************************
C*****              LOWER CUTOFF CONDITION FOR USTAR                ***
C**********************************************************************
C
      DO 9000 I = 1,IRUN
       IF(VUSTAR(I) .LT. ONEPEM6)THEN
        INT3(I) = 1
       ELSE
        INT3(I) = 0
       ENDIF
 9000 CONTINUE
      DO 9002 I = 1,IRUN
       IF(INT3(I).EQ.1) VUSTAR(I) = ONEPEM6
 9002 CONTINUE
C
C***********************************
C*****  LOAD THE ARRAY A(I,K)  *****
C***********************************
C
      DO 9004 I = 1,IRUN
       IF( (VUSTAR(I) .GT. USTMX1) .AND. (IWATER(I).EQ.1) ) THEN
        INT4(I) = 1
       ELSE
        INT4(I) = 0
       ENDIF
 9004 CONTINUE
      DO 9006 I = 1,IRUN
       IF(VUSTAR(I) .GT. USTMX2) THEN
        INT3(I) = 1
       ELSE
        INT3(I) = 0
       ENDIF
 9006 CONTINUE
      DO 9008 I = 1,IRUN
       IF(VUSTAR(I) .GE. USTMX3) THEN
        INT2(I) = 1
       ELSE
        INT2(I) = 0
       ENDIF
 9008 CONTINUE
C
      DO 100 K=1,5
       DO 9010 I = 1,IRUN
        AA(I,K) = AA1(K)
        IF( INT2(I).EQ.1 )  AA(I,K) = AA2(K)
        IF( INT3(I).EQ.1 )  AA(I,K) = AA3(K)
        IF( INT4(I).EQ.1 )  AA(I,K) = AA4(K)
 9010  CONTINUE
  100 CONTINUE
C
C********************************************************
C*****  EVALUATE THE ENHANCED POLYNOMIAL FOR ZSEA  *****
C********************************************************
C
      DO 9012 I = 1,IRUN
       VDZSEA(I)  =  ( AA(I,4) + AA(I,5) * VUSTAR(I) ) * VUSTAR(I)
       VZSEA(I)  =  AA(I,2) + ( AA(I,3) + VDZSEA(I) ) * VUSTAR(I)
       TEMP(I) = AA(I,1) / VUSTAR(I)
       VZSEA(I)  =  VZSEA(I) + TEMP(I)
 9012 CONTINUE
C
C**********************************************************************
C*****        EVALUATE THE DERIVATIVE DZSEA IF LDZSEA IS TRUE       ***
C**********************************************************************
C
      IF( LDZSEA ) THEN
       DO 9014 I = 1,IRUN
        VDZSEA(I)  =  THREE * VDZSEA(I) -(AA(I,4)*VUSTAR(I) - AA(I,3))
        VDZSEA(I)  =  VDZSEA(I) * VUSTAR(I) - TEMP(I)
 9014  CONTINUE
      ENDIF
C
      RETURN
      END
      SUBROUTINE RAYLEI ( KZT,IM,JNP,NLAY,NYMD,NHMS,
     .                    UZ,VZ,TZ,PZ,PLZ,PKZ,
     .                    DUDT,DVDT,DTDT )
C **********************************************************************
C
C  PURPOSE
C     TO IMPLIMENT RAYLEIGH FRICTION AND NEWTONIAN COOLING ABOVE 'UPLIMT
C
C  ARGUMENTS   DESCRIPTION
C     KZT ..... UNIT TO READ CLIMATOLOGY TEMPERATURES FROM
C     IM ...... NUMBER OF LONGITUDE POINTS IN MODEL
C     JNP ..... NUMBER OF LATITUDE POINTS IN MODEL
C     NLAY .... NUMBER OF VERTICAL LEVELS IN MODEL
C     NYMD .... CURRENT YYMMDD
C     NHMS .... CURRENT HHMMSS
C     UZ ...... VALUE OF U-WIND IN MODEL FOR N-1 TIME STEP
C     VZ ...... VALUE OF V-WIND IN MODEL FOR N-1 TIME STEP
C     TZ ...... VALUE OF TEMPERATURE IN MODEL FOR N-1 TIME STEP
C     PZ ...... VALUE OF PI IN MODEL FOR N-1 TIME STEP
C     PLZ ..... PRESSURE (MB) OF ALL POINTS IN MODEL (BOTTOM TO TOP)
C     PKZ ..... P**KAPPA AT MODEL LEVELS
C     DUDT .... VALUE OF U-WIND TENDENCY
C     DVDT .... VALUE OF V-WIND TENDENCY
C     DTDT .... VALUE OF TEMPERATURE TENDENCY
C
C **********************************************************************

      PARAMETER (ZERO=0.0, ONE=1.0 )
      PARAMETER (NUMLEVS=24, NUMLATS=17, UPLIMT=10.)

      REAL PRDAT(NUMLEVS), ZTLAT(NUMLATS), ZTDAT(NUMLATS,NUMLEVS,12),
     .     SLOPE, T1, T2, TFACM, TFACP,
     .     CRF(NUMLEVS), CNC(NUMLEVS), ZTTEMP(JNP,NUMLEVS),
     .     ZT(IM,JNP,NLAY), COEFNC(IM,JNP,NLAY), COEFRF(IM,JNP,NLAY)

      LOGICAL FIRST
      INTEGER MON, DAY, SEC, DAYS(12), IMNM, IMNP, IX(JNP)

      REAL XLAT(500)
      REAL DUDT(IM,JNP,NLAY),    DVDT(IM,JNP,NLAY),
     .     DTDT(IM,JNP,NLAY),
     .       UZ(IM,JNP,NLAY),      VZ(IM,JNP,NLAY),
     .       TZ(IM,JNP,NLAY),
     .      PLZ(IM,JNP,NLAY),      PZ(IM,JNP),
     .      PKZ(IM,JNP,NLAY)

      DATA DAYS /31,28,31,30,31,30,31,31,30,31,30,31/
      DATA FIRST/.TRUE./

      SAVE
      NSECF(N) = N/10000*3600 + MOD(N,10000)/100* 60 + MOD(N,100)
      NMONF(N) = MOD(N,10000)/100
      NDAYF(N) = MOD(N,100)

C **********************************************************************
C ****   READ PRESSURE LEVELS, RAYLEIGH AND NEWTONIAN COEFFICIENTS,  ***
C ****                AND CLIMATOLOGY TEMPERATURES                   ***
C **********************************************************************

      IF(FIRST) THEN
      DO 10 J=1,NUMLATS
      ZTLAT(J) = -90. + J*10.
 10   CONTINUE
      DO 12 J=1,JNP
       XLAT(J) = -90. + (J-1)*180./(JNP-1)
 12   CONTINUE
      REWIND KZT
      READ  (KZT) PRDAT, CRF, CNC, ZTDAT
      FIRST = .FALSE.
      ENDIF

C **********************************************************************
C ****           SET PROPER FACTORS FOR TIME INTERPOLATION           ***
C **********************************************************************

      SEC = NSECF(NHMS)
      MON = NMONF(NYMD)
      DAY = NDAYF(NYMD)

      MIDMON = DAYS(MON)/2+1

      IF(DAY.LT.MIDMON) THEN
        IMNM = MON-1
        IMNP = MON
        IF(IMNM.LT.1) IMNM = 12
        ISC  = (DAY+DAYS(IMNM))*86400+SEC
       ELSE
         IMNM = MON
         IMNP = MON+1
         IF(IMNP.GT.12) IMNP = 1
         ISC  = DAY*86400+SEC
       ENDIF

       ISCM = (DAYS(IMNM)/2+1)*86400
       ISCP = (DAYS(IMNP)/2+1+DAYS(IMNM))*86400
      TFACP = (ISC-ISCM) / real(ISCP-ISCM)
      TFACM =  ONE-TFACP

C **********************************************************************
C ****        INTERPOLATE TEMPERATURE DATA TO MODEL LATITUDES        ***
C **********************************************************************

      DO 30 K=1,NUMLEVS
      DO 31 J=1,JNP
         DO 32 I=2,NUMLATS-1
         IF(ZTLAT(I).GE.XLAT(J)) THEN
         IX(J) = I
         GOTO 31
         ENDIF
 32      CONTINUE
      IX(J) = NUMLATS
 31   CONTINUE

C **********************************************************************
C ****       SAVE RESULT FROM TIME AND LATITUDE INTERPOLATION        ***
C **********************************************************************

      DO J=1,JNP
         T1 = ZTDAT(IX(J)-1,K,IMNM)*TFACM + ZTDAT(IX(J)-1,K,IMNP)*TFACP
         T2 = ZTDAT(IX(J)  ,K,IMNM)*TFACM + ZTDAT(IX(J)  ,K,IMNP)*TFACP
            SLOPE = (  XLAT(J)    -ZTLAT(IX(J)-1) )
     .            / ( ZTLAT(IX(J))-ZTLAT(IX(J)-1) )
      ZTTEMP(J,K) = T1 + SLOPE*(T2-T1)
      ENDDO
 30   CONTINUE

C **********************************************************************
C ****    INTERPOLATE LATITUDE TEMPERATURE DATA TO MODEL PRESSURES   ***
C **********************************************************************

      DO 40 L=1,NLAY
        DO 44 I=1,IM
          DO 41 J=1,JNP
            IF( PLZ(I,J,L).LE.UPLIMT ) THEN
              DO 42 N=2,NUMLEVS-1
                IF(PRDAT(N).LE.PLZ(I,J,L)) THEN
                  IX(J) = N
                  GOTO 41
                 ENDIF
 42            CONTINUE
              IX(J) = NUMLEVS
             ENDIF
 41        CONTINUE

          DO 43 J=1,JNP
          IF( PLZ(I,J,L).GT.UPLIMT ) THEN
           COEFRF(I,J,L) = 0.0
           COEFNC(I,J,L) = 0.0
          ELSE
                   SLOPE = (   PLZ(I,J,L)  -PRDAT(IX(J)) )
     .                   / ( PRDAT(IX(J)-1)-PRDAT(IX(J)) )
           COEFRF(I,J,L) = CRF(IX(J)) + SLOPE*(CRF(IX(J)-1)-CRF(IX(J)))
           COEFNC(I,J,L) = CNC(IX(J)) + SLOPE*(CNC(IX(J)-1)-CNC(IX(J)))
               ZT(I,J,L) =  ZTTEMP(J,IX(J)) + SLOPE
     .                   * (ZTTEMP(J,IX(J)-1)-ZTTEMP(J,IX(J)))
           ENDIF
 43        CONTINUE
 44      CONTINUE

 40    CONTINUE

C **********************************************************************
C ****          COMPUTE CLIMATOLOGICAL POTENTIAL TEMPERATURE         ***
C **********************************************************************

      DO L=1,NLAY
      DO I=1,IM*JNP
      IF( PLZ(I,1,L).LE.UPLIMT ) ZT(I,1,L) = ZT(I,1,L)/PKZ(I,1,L)
      ENDDO
      ENDDO

C **********************************************************************
C ****    APPLY RAYLEIGH FRICTION AND NEWTONIAN COOLING TO FIELDS    ***
C **********************************************************************

      DO L=1,NLAY
      DO I=1,IM*JNP
        DUDT(I,1,L) =   DUDT(I,1,L)
     .              - COEFRF(I,1,L)*PZ(I,1)* UZ(I,1,L)
        DVDT(I,1,L) =   DVDT(I,1,L)
     .              - COEFRF(I,1,L)*PZ(I,1)* VZ(I,1,L)
c       DTDT(I,1,L) =   DTDT(I,1,L)
c    .              - COEFNC(I,1,L)*PZ(I,1)*(TZ(I,1,L)-ZT(I,1,L))
      ENDDO
      ENDDO

C **********************************************************************
C ****             REMOVE LONGWAVE COOLING ABOVE UPLIMT              ***
C **********************************************************************

c     DO  I=1,IM*JNP*NLAY
c     IF( PLZ(I,1,1).LE.UPLIMT )  THEN
c     DTDT(I,1,1) = DTDT(I,1,1) - DTRADLW(I,1,1)
c     ENDIF
c     ENDDO

      RETURN
      END
      SUBROUTINE RESTART (LU,IOFLAG,NYMD,NHMS,NB,NYMD0,NHMS0,JOB,NTYPE)         
C***********************************************************************        
C  PURPOSE                                                                      
C     Reads or Writes a Model Restart                                           
C  ARGUMENTS   DESCRIPTION                                                      
C     LU       Unit to read or write                                            
C     IOFLAG   Flag for IO:  1 to Read  a Restart                               
C                            2 to Write a Restart                               
C                           -2 to Write a Short Restart (No Tendencies)         
C     NYMD     CURRENT YYMMDD                                                   
C     NHMS     CURRENT HHMMSS                                                   
C     NB       CURRENT TIME INDEX                                               
C     NYMD0    YYMMDD OF BEGINNING OF EXPERIMENT                                
C     NHMS0    HHMMSS OF BEGINNING OF EXPERIMENT                                
C     JOB      CHARACTER*8 JOB IDENTIFIER                                       
C     NTYPE    RESTART RECORD TYPE:  -1  BEFORE ANALYSIS                        
C                                     0  AFTER  ANALYSIS                        
C                                     1  AFTER  ANALYSIS AND INITIALIZED        
C                                    1_  ANALYSIS TENDENCIES ON RESTART         
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
                                                                                
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <vphyscom.com>                                                         
#include <vmetric.com>                                                          
#include <iau.com>                                                          
                                                                                
      PARAMETER ( ZERO  = 0.0 )                                                 
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
      CHARACTER*8     JOB                                                       
      logical         dimchk                                                    
      REAL            QT, ZSIGE, ZPTOP, ZPMEAN                                  
      COMMON /SPACE/  QT(IMR,JMR+1)                                             
      COMMON /SPACE/ ZSIGE(NLAYR+1)                                             
                                                                                
C **********************************************************************        
C ****                 READ THE RESTART HEADER                      ****        
C **********************************************************************        
                                                                                
      IF(IOFLAG.EQ.1) THEN                                                      
      READ (LU) NYMD, NHMS, NB, NYMD0, NHMS0, IMAX, JMAXM1, KMAX,               
     .          KPROGS1, KPROGU1, KPROGS2, KPROGU2,                             
     .          KPHYS, NLWCLD, NSWCLD, JOB, NTYPE,         
     .          ZSIGE, ZPTOP, ZPMEAN, IRAS   
                                                                                
C **********************************************************************        
C ****                   CHECK CONSISTANCY                          ****        
C **********************************************************************        
                                                                                
      dimchk = imax.eq.imr .and. jmaxm1.eq.jmr .and. kmax.eq.nlayr              
                                                                                
      IF ( .not.dimchk ) then                                                   
      WRITE(6,9999) IMAX,    IMR,                                               
     .              JMAXM1,  JMR,                                               
     .              KMAX,    NLAYR                                              
 9999 FORMAT(1X,//,1x,'********* N O T E ***************',//,                   
     .       1X,'RESTART horizontal dimensions not consistant with ',           
     .          'loaded TEXT module:',/,                                        
     .       1X,'Read IMAX    as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read JMAX-1  as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KMAX    as ',I3,', Dimensioned as ',I3,//)                
      stop                                                                      
      endif                                                                     
                                                                                
      IF ( KPROGS1.NE.NPROGS1 .OR. KPROGU1.NE.NPROGU1 .OR.                      
     .     KPROGS2.NE.NPROGS2 .OR. KPROGU2.NE.NPROGU2 .OR.                      
     .     KPHYS .NE.NPHYS ) THEN                                               
      WRITE(6,9998) KPROGS1, NPROGS1,                                           
     .              KPROGU1, NPROGU1,                                           
     .              KPROGS2, NPROGS2,                                           
     .              KPROGU2, NPROGU2,                                           
     .              KPHYS,   NPHYS                                              
 9998 FORMAT(1X,//,1x,'********* N O T E ***************',//,                   
     .       1X,'RESTART record fields and/or tendencies ',/,                   
     .       1X,'        not consistant with loaded TEXT module.',/,            
     .       1X,'        SKIPPING or ZERO FILL values will be used:',//,        
     .       1X,'Read KPROGS1 as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KPROGU1 as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KPROGS2 as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KPROGU2 as ',I3,', Dimensioned as ',I3,/,                 
     .       1X,'Read KPHYS   as ',I3,', Dimensioned as ',I3,/)                 
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                    LOAD SIGMA LEVELS                         ****        
C **********************************************************************        
                                                                                
      PRINT *                                                                   
      PRINT *, 'Reading Initial Conditions for ',NLAYR,' Sigma levels'          
      PRINT *, 'Model Top Pressure: ',ZPTOP,' mb.'                              
      PRINT *                                                                   
      PTOP    = ZPTOP                                                           
      DO 5 L=1,NLAYR+1                                                          
      SIGE(L) = ZSIGE(L)                                                        
      WRITE(6,8888) L,SIGE(L)                                                   
    5 CONTINUE                                                                  
 8888 FORMAT(1X,'L = ',I2,2X,'SIGE = ',F8.6)                                    
      PRINT *                                                                   
                                                                                
C **********************************************************************        
C ****       READ THE 1 TIME LEVEL SURFACE PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      nqmax = min( kprogs1,nprogs1 )                                            
                                                                                
      DO 10 NQ=1,nqmax                                                          
      READ (LU)  QT                                                             
      DO 10 I =1,IMJMR                                                          
      PROGS1(I,1,NQ) = QT(I,1)                                                  
   10 CONTINUE                                                                  
                                                                                
      if( kprogs1.gt.nprogs1 ) then                                             
      DO 11 NQ=1,kprogs1-nprogs1                                                
      READ (LU)  QT                                                             
   11 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nprogs1.gt.kprogs1 ) then                                             
      DO 12 NQ=1,nprogs1-kprogs1                                                
      DO 12 I =1,IMJMR                                                          
      PROGS1(I,1,NQ+nqmax) = zero                                               
   12 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****     READ THE 1 TIME LEVEL UPPER-AIR PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      nqmax = min( kprogu1,nprogu1 )                                            
                                                                                
      DO 20 NQ=1,nqmax                                                          
      DO 20  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
      DO 20 I =1,IMJMR                                                          
      PROGU1(I,1,L,NQ) = QT(I,1)                                                
   20 CONTINUE                                                                  
                                                                                
      if( kprogu1.gt.nprogu1 ) then                                             
      DO 21 NQ=1,kprogu1-nprogu1                                                
      DO 21  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
   21 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nprogu1.gt.kprogu1 ) then                                             
      DO 22 NQ=1,nprogu1-kprogu1                                                
      DO 22  L=1,NLAYR                                                          
      DO 22  I=1,IMJMR                                                          
      PROGU1(I,1,L,NQ+nqmax) = zero                                             
   22 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****       READ THE 2 TIME LEVEL SURFACE PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      nqmax = min( kprogs2,nprogs2 )                                            
                                                                                
      DO 30 NT=1,2                                                              
      DO 30 NQ=1,nqmax                                                          
      READ (LU)  QT                                                             
      DO 30 I =1,IMJMR                                                          
      PROGS2(I,1,NQ,NT) = QT(I,1)                                               
   30 CONTINUE                                                                  
                                                                                
      if( kprogs2.gt.nprogs2 ) then                                             
      DO 31 NT=1,2                                                              
      DO 31 NQ=1,kprogs2-nprogs2                                                
      READ (LU)  QT                                                             
   31 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nprogs2.gt.kprogs2 ) then                                             
      DO 32 NT=1,2                                                              
      DO 32 NQ=1,nprogs2-kprogs2                                                
      DO 32  I=1,IMJMR                                                          
      PROGS2(I,1,NQ,NT+nqmax) = zero                                            
   32 CONTINUE                                                                  
      endif                                                                     
                                                                                
C COMPUTE MEAN SURFACE PRESSURE                                                 
C -----------------------------                                                 
      CALL SETMET                                                               
      IF (ZPMEAN.EQ.ZERO) THEN                                                  
      PIMEAN =  PMEAN( PZ(1,1,NB),IMR,JMR+1 )                                   
      ELSE                                                                      
      PIMEAN = ZPMEAN                                                           
      ENDIF                                                                     
                                                                                
      PRINT *                                                                   
      PRINT *, 'Mean Surface Pressure: ',PIMEAN,' mb.'                          
      PRINT *                                                                   
                                                                                
C **********************************************************************        
C ****     READ THE 2 TIME LEVEL UPPER-AIR PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      nqmax = min( kprogu2,nprogu2 )                                            
                                                                                
      DO 40 NT=1,2                                                              
      DO 40 NQ=1,nqmax                                                          
      DO 40  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
      DO 40 I =1,IMJMR                                                          
      PROGU2(I,1,L,NQ,NT) = QT(I,1)                                             
   40 CONTINUE                                                                  
                                                                                
      if( kprogu2.gt.nprogu2 ) then                                             
      DO 41 NT=1,2                                                              
      DO 41 NQ=1,kprogu2-nprogu2                                                
      DO 41  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
   41 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nprogu2.gt.kprogu2 ) then                                             
      DO 42 NT=1,2                                                              
      DO 42 NQ=1,nprogu2-kprogu2                                                
      DO 42  L=1,NLAYR                                                          
      DO 42  I=1,IMJMR                                                          
      PROGU2(I,1,L,NQ,NT+nqmax) = zero                                          
   42 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****                  READ THE PHYSICS TENDENCIES                 ****        
C **********************************************************************        
                                                                                
      nqmax = min( kphys,nphys )                                                
                                                                                
      if( nqmax.ne.0 ) then                                                     
      DO 50 NQ=1,nqmax                                                          
      DO 50  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
      DO 50 I =1,IMJMR                                                          
      PHYDUM(I,1,L,NQ) = QT(I,1)                                                
   50 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( kphys.gt.nphys ) then                                                 
      DO 51 NQ=1,kphys-nphys                                                    
      DO 51  L=1,NLAYR                                                          
      READ (LU)  QT                                                             
   51 CONTINUE                                                                  
      endif                                                                     
                                                                                
      if( nphys.gt.kphys ) then                                                 
      DO 52 NQ=1,nphys-kphys                                                    
      DO 52  L=1,NLAYR                                                          
      DO 52  I=1,IMJMR                                                          
      PHYDUM(I,1,L,NQ+nqmax) = zero                                             
   52 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****        READ THE NET RADIATION (LW,SW) AT THE GROUND          ****        
C **********************************************************************        
                                                                                
      READ (LU)  QT                                                             
      DO 60 I =1,IMJMR                                                          
      RADLWG(I,1) = QT(I,1)                                                     
   60 CONTINUE                                                                  
      READ (LU)  QT                                                             
      DO 70 I =1,IMJMR                                                          
      RADSWG(I,1) = QT(I,1)                                                     
   70 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****                 READ THE ANALYSIS TENDENCIES                 ****        
C **********************************************************************        
                                                                                
      if( abs(ntype/10).eq.1 ) then
      PRINT *, 'Reading Analysis Tendencies'                         
      PRINT *                                                                   
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DPANAL(I,1) = QT(I,1)                                                     
      ENDDO                                                                     

      DO L = 1,NLAYR
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DUANAL(I,1,L) = QT(I,1)                                                     
      ENDDO                                                                     
      ENDDO                                                                     

      DO L = 1,NLAYR
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DVANAL(I,1,L) = QT(I,1)                                                     
      ENDDO                                                                     
      ENDDO                                                                     

      DO L = 1,NLAYR
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DTANAL(I,1,L) = QT(I,1)                                                     
      ENDDO                                                                     
      ENDDO                                                                     

      DO L = 1,NLAYR
      READ (LU)  QT                                                             
      DO I = 1,IMJMR                                                          
      DQANAL(I,1,L) = QT(I,1)                                                     
      ENDDO                                                                     
      ENDDO                                                                     
      else
      PRINT *, 'No Analysis Tendencies on RESTART'                         
      PRINT *                                                                   
      endif
                                                                                
      ENDIF                                                                     
                                                                                
C **********************************************************************        
C ****                 WRITE THE RESTART HEADER                     ****        
C **********************************************************************        
                                                                                
      IF(IOFLAG.EQ. 2 .or. IOFLAG.EQ.-2) THEN                                   
      IF(IOFLAG.EQ. 2) MPHYS = NPHYS                                            
      IF(IOFLAG.EQ.-2) MPHYS = 0                                                
                                                                                
      ZPMEAN   = PIMEAN                                                         
      ZPTOP    = PTOP                                                           
      DO 205 L=1,NLAYR+1                                                        
      ZSIGE(L) = SIGE(L)                                                        
  205 CONTINUE                                                                  
                                                                                
      WRITE(LU) NYMD, NHMS, NB, NYMD0, NHMS0, IMR, JMR, NLAYR,                  
     .          NPROGS1, NPROGU1, NPROGS2, NPROGU2,                             
     .          MPHYS, NLWCLD, NSWCLD, JOB, NTYPE,         
     .          ZSIGE, ZPTOP, ZPMEAN, IRAS   
                                                                                
C **********************************************************************        
C ****      WRITE THE 1 TIME LEVEL SURFACE PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      DO 210 NQ=1,NPROGS1                                                       
      DO 215 I =1,IMJMR                                                         
      QT(I,1)  = PROGS1(I,1,NQ)                                                 
  215 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  210 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****    WRITE THE 1 TIME LEVEL UPPER-AIR PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      DO 220 NQ=1,NPROGU1                                                       
      DO 220  L=1,NLAYR                                                         
      DO 225 I =1,IMJMR                                                         
      QT(I,1)  = PROGU1(I,1,L,NQ)                                               
  225 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  220 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****      WRITE THE 2 TIME LEVEL SURFACE PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      DO 230 NT=1,2                                                             
      DO 230 NQ=1,NPROGS2                                                       
      DO 235 I =1,IMJMR                                                         
      QT(I,1)  = PROGS2(I,1,NQ,NT)                                              
  235 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  230 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****    WRITE THE 2 TIME LEVEL UPPER-AIR PROGNOSTIC FIELDS        ****        
C **********************************************************************        
                                                                                
      DO 240 NT=1,2                                                             
      DO 240 NQ=1,NPROGU2                                                       
      DO 240  L=1,NLAYR                                                         
      DO 245 I =1,IMJMR                                                         
      QT(I,1)  = PROGU2(I,1,L,NQ,NT)                                            
  245 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  240 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****                 WRITE THE PHYSICS TENDENCIES                 ****        
C **********************************************************************        
                                                                                
      IF (IOFLAG.eq.2) then                                                     
      DO 250 NQ=1,NPHYS                                                         
      DO 250  L=1,NLAYR                                                         
      DO 255 I =1,IMJMR                                                         
      QT(I,1)  = PHYDUM(I,1,L,NQ)                                               
  255 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
  250 CONTINUE                                                                  
      endif                                                                     
                                                                                
C **********************************************************************        
C ****       WRITE THE NET RADIATION (LW,SW) AT THE GROUND          ****        
C **********************************************************************        
                                                                                
      DO 260 I =1,IMJMR                                                         
      QT(I,1)  = RADLWG(I,1)                                                    
  260 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
      DO 270 I =1,IMJMR                                                         
      QT(I,1)  = RADSWG(I,1)                                                    
  270 CONTINUE                                                                  
      WRITE (LU)  QT                                                            
                                                                                
C **********************************************************************        
C ****                WRITE THE ANALYSIS TENDENCIES                 ****        
C **********************************************************************        
                                                                                
      if( abs(ntype/10).eq.1 ) then
      DO I = 1,IMJMR                                                          
      QT(I,1) = DPANAL(I,1)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             

      DO L = 1,NLAYR
      DO I = 1,IMJMR                                                          
      QT(I,1) = DUANAL(I,1,L)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             
      ENDDO                                                                     

      DO L = 1,NLAYR
      DO I = 1,IMJMR                                                          
      QT(I,1) = DVANAL(I,1,L)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             
      ENDDO                                                                     

      DO L = 1,NLAYR
      DO I = 1,IMJMR                                                          
      QT(I,1) = DTANAL(I,1,L)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             
      ENDDO                                                                     

      DO L = 1,NLAYR
      DO I = 1,IMJMR                                                          
      QT(I,1) = DQANAL(I,1,L)
      ENDDO                                                                     
      WRITE (LU)  QT                                                             
      ENDDO                                                                     
      endif
                                                                                
      ENDIF                                                                     
      RETURN                                                                    
      END                                                                       
      SUBROUTINE GETBCS (KBC, NYMD, NHMS, TGZIJ,                                
     .                   PHIIJ, TVAIJ, SOLIJ,                                   
     .                   SSTIJ, ALBIJ, WETIJ, THKIJ, SRFIJ, ZLWIJ,              
     .                   IMJM)                                                  
C***********************************************************************        
C  PURPOSE                                                                      
C     TO READ AND UPDATE GCM BOUNDARY CONDITIONS                                
C                                                                               
C  ARGUMENTS   DESCRIPTION                                                      
C     KBC .... UNIT TO READ BCS                                                 
C     NYMD ... YYMMDD TO INTERPOLATE TO                                         
C     NHMS ... HHMMSS TO INTERPOLATE TO                                         
C     TGZIJ .. GROUND TEMPERATURES (TO BE UPDATED OVER OCEAN)                   
C     PHIIJ .. SURFACE GEOPOTENTIAL                                             
C     TVAIJ .. SURFACE GEOPOTENTIAL VARIANCE                                    
C     SOLIJ .. INCIDENT SOLAR RADIATION AT THE TOP OF THE ATMOSPHERE            
C     SSTIJ .. SEA-SURFACE TEMPERATURE                                          
C     ALBIJ .. SURFACE ALBEDO         (0-1)                                     
C     WETIJ .. SURFACE GROUND WETNESS (0-1)                                     
C     THKIJ .. SEA-ICE THICKNESS                                                
C     SRFIJ .. SURFACE ROUGHNESS                                                
C     ZLWIJ .. LAND-WATER-GLACIER FLAG                                          
C     IMJM ... HORIZONTAL DIMENSION OF BOUNDARY CONDITIONS                      
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
                                                                                
      PARAMETER ( IRUN  = IMR*(JMR+1) )                                         
      PARAMETER ( ZERO   = 0.0 )                                                
      PARAMETER ( ONE    = 1.0 )                                                
      PARAMETER ( TWO    = 2.0 )                                                
      PARAMETER ( THREE  = 3.0 )                                                
      PARAMETER ( FOUR   = 4.0 )                                                
      PARAMETER ( FIVE   = 5.0 )                                                
      PARAMETER ( ALBSNO = 0.4 )                                                
      PARAMETER ( ALBMIN = 0.4 )                                                
      PARAMETER ( ALBMAX = 1.0 )                                                
      PARAMETER ( TICE   = 273.16 )                                             
      PARAMETER ( SSTMIN = 273.16 )                                             
      PARAMETER ( SSTMAX = 310.00 )                                             
                                                                                
      INTEGER   MONTH, DAY, SEC, KBC                                            
                                                                                
      DIMENSION TGZIJ(IMJM), PHIIJ(IMJM), TVAIJ(IMJM)                           
      DIMENSION SSTIJ(IMJM), ALBIJ(IMJM), WETIJ(IMJM)                           
      DIMENSION THKIJ(IMJM), SRFIJ(IMJM), ZLWIJ(IMJM)                           
      DIMENSION SOLIJ(IMJM)                                                     
         
      character*4 year                                                         
      character*3 cmon1, cmon2                                                       
      character*4 cnam  
                                                               
      INTEGER   DAYS(13)                                                        
      DATA      DAYS /31,31,28,31,30,31,30,31,31,30,31,30,31/                   
      DIMENSION ALAT(JMR+1), ALON(IMR)                                          
                                                                                
      COMMON /BCTEMP/ TOPG(IRUN), TOPV(IRUN), FLWI(IRUN)                        
      COMMON /BCTEMP/ SSTM(IRUN), SSTP(IRUN)                                    
      COMMON /BCTEMP/ WETM(IRUN), WETP(IRUN)                                    
      COMMON /BCTEMP/ ALBM(IRUN), ALBP(IRUN)                                    
      COMMON /BCTEMP/ THKM(IRUN), THKP(IRUN)                                    
      COMMON /BCTEMP/ SRFM(IRUN), SRFP(IRUN)                                    
                                                                                
      COMMON /SPACE/   COSZ(IRUN)                                               
      COMMON /SPACE/    VT1(IRUN)                                               
                                                                                
      DATA IMONP/0/, IMONM/0/, IKBC/0/                                          
      SAVE    imonp, imonm, ikbc                                                
                                                                                
      NSECF(N) = N/10000*3600 + MOD(N,10000)/100* 60 + MOD(N,100)               
      NMONF(N) = MOD(N,10000)/100                                               
      NDAYF(N) = MOD(N,100)                                                     
                                                                                
C*********************************************************************          
C****     FIND PROPER MONTH BOUNDARIES FROM 14-MONTH DATASET      ****          
C*********************************************************************          
                                                                                
      SEC    = NSECF(NHMS)                                                      
      MONTH  = NMONF(NYMD)                                                      
      DAY    = NDAYF(NYMD)                                                      
                                                                                
      MIDMON = DAYS(MONTH)/2 + 1                                                
                                                                                
      IF(DAY.LT.MIDMON) THEN
         imnm = month
         imnp = month + 1
      ELSE                                                                      
         IMNM = MONTH + 1                                                       
         IMNP = MONTH + 2                                                       
      ENDIF                                                                     
                                                                                
C*********************************************************************          
C****           READ DATA SET TWICE TO GET BOTH MONTHS            ****          
C*********************************************************************          
                                                                                
      IF(IMONM.NE.IMNM .OR. IMONP.NE.IMNP .OR. IKBC.NE.KBC) THEN                
                                                                                
         IMONM = IMNM                                                           
         IMONP = IMNP                                                           
         IKBC  = KBC                                                            
                                                                                
C  FIRST READ OF DATA                                                           
C====================                                                           
         REWIND KBC                                                             
         READ  (KBC) IMJMBC,  year                                                     
         IF( IMJMBC.NE.IMJM ) THEN                                              
         PRINT *, 'BAD B.C. DATASET'                                            
         PRINT *, 'MODEL DIMENSION ',IMJM                                       
         PRINT *, 'DATA  DIMENSION ',IMJMBC                                     
         PRINT *, 'YEAR ',YEAR                                     
         STOP                                                                   
         ENDIF                                                                  
         READ  (KBC)  cnam, TOPG                                                      
         READ  (KBC)  cnam, TOPV                                                      
         READ  (KBC)  cnam, FLWI                                                      
                                                                                
         DO 200 K=1,IMONM                                                       
         READ (KBC) cmon1, cnam, SSTM                                                      
         READ (KBC) cmon1, cnam, ALBM                                                      
         READ (KBC) cmon1, cnam, WETM                                                      
         READ (KBC) cmon1, cnam, THKM                                                      
         READ (KBC) cmon1, cnam, SRFM                                                      
 200     CONTINUE                                                               
                                                                                
C  SECOND READ OF DATA                                                          
C=====================                                                          
         REWIND KBC                                                             
         READ  (KBC)  IMJMBC, year                                                     
         READ  (KBC)  cnam, TOPG                                                      
         READ  (KBC)  cnam, TOPV                                                      
         READ  (KBC)  cnam, FLWI                                                      
                                                                                
         DO 300 K=1,IMONP                                                       
         READ (KBC) cmon2, cnam, SSTP                                                      
         READ (KBC) cmon2, cnam, ALBP                                                      
         READ (KBC) cmon2, cnam, WETP                                                      
         READ (KBC) cmon2, cnam, THKP                                                      
         READ (KBC) cmon2, cnam, SRFP                                                      
 300     CONTINUE 
      
      mbefor = imonm-1
      mafter = imonp-1                                                        
      WRITE(6,1000) NYMD,NHMS, mbefor,cmon1, mafter,cmon2, year, KBC                               
 1000 FORMAT(1X,'Updating Boundary Conditions for   NYMD: ',I6,2X,              
     .        'NHMS: ',I6,/,1X,'Reading Months ',I2,' (',a3,') and ',
     .         I2,' (',a3,')',2X,'Year: ',a4,' From Unit ',I2)                                                  
      ENDIF                                                                     
                                                                                
C*********************************************************************          
C****             INTERPOLATE DATA TO CURRENT TIME                ****          
C*********************************************************************          
                                                                                
      IF(DAY.LT.MIDMON) THEN                                                    
               ISC  = (DAY+DAYS(IMONM))*86400 + SEC                             
      ELSE                                                                      
               ISC  = (DAY            )*86400 + SEC                             
      ENDIF                                                                     
                                                                                
      ISCM = (DAYS(IMONM)/2+1               )*86400                             
      ISCP = (DAYS(IMONP)/2+1 + DAYS(IMONM) )*86400                             
      
      FACP = (ISC-ISCM) / REAL(ISCP-ISCM)                                       
      FACM = ONE-FACP                                                           
                                                                                
      DO 20 I=1,IRUN                                                            
      PHIIJ(I) = TOPG(I)                                                        
      TVAIJ(I) = TOPV(I)                                                        
      ZLWIJ(I) = FLWI(I)                                                        
      VT1  (I) = SRFM(I) * FACM                                                 
      SRFIJ(I) = VT1 (I) + SRFP(I) * FACP                                       
      VT1  (I) = ALBM(I) * FACM                                                 
      ALBIJ(I) = VT1 (I) + ALBP(I) * FACP                                       
      VT1  (I) = WETM(I) * FACM                                                 
      WETIJ(I) = VT1 (I) + WETP(I) * FACP                                       
      VT1  (I) = THKM(I) * FACM                                                 
      THKIJ(I) = VT1 (I) + THKP(I) * FACP                                       
      VT1  (I) = SSTM(I) * FACM                                                 
      SSTIJ(I) = VT1 (I) + SSTP(I) * FACP                                       
   20 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****     COMPUTE INCIDENT SOLAR RADIATION AT TOP OF ATMOSPHERE    ****        
C **********************************************************************        
C                                                                               
      S0 = GETCON('S0')                                                         
      DO 400 J=1,JMR+1                                                          
      ALAT(J) = - 90. + (J-1)*180./JMR                                          
  400 CONTINUE                                                                  
      DO 500 I=1,IMR                                                            
      ALON(I) = -180. + (I-1)*360./IMR                                          
  500 CONTINUE                                                                  
                                                                                
      CALL ASTRO ( NYMD,NHMS, ALAT,ALON, IMR,JMR+1, COSZ,RA )                   
      DO 30 I=1,IRUN                                                            
      SOLIJ(I) = S0 * (ONE/RA**2) * COSZ(I)                                     
   30 CONTINUE                                                                  
                                                                                
C **********************************************************************        
C ****     RESOLVE INCONSISTANCIES IN NEW SURFACE FIELD SETTINGS    ****        
C **********************************************************************        
C                                                                               
C LAND & SEA ICE COVERED POINTS HAVE ALBEDO BETWEEN 0.40 AND 1.00               
C ===============================================================               
                                                                                
      DO 40 I=1,IRUN                                                            
      IF( (ZLWIJ(I).EQ.THREE .OR.                                               
     .     THKIJ(I).GT.ZERO) .AND.                                              
     .     ALBIJ(I).LT.ALBMIN ) ALBIJ(I) = ALBMIN                               
                                                                                
      IF( (ZLWIJ(I).EQ.THREE .OR.                                               
     .     THKIJ(I).GT.ZERO) .AND.                                              
     .     ALBIJ(I).GT.ALBMAX ) ALBIJ(I) = ALBMAX                               
C                                                                               
C OPEN WATER POINTS HAVE SSTS BETWEEN TICE AND 310.0                            
C ==================================================                            
                                                                                
      IF(  ZLWIJ(I).EQ.ONE  .AND.                                               
     .     THKIJ(I).EQ.ZERO .AND.                                               
     .     SSTIJ(I).LT.SSTMIN ) SSTIJ(I) = SSTMIN                               
                                                                                
      IF(  ZLWIJ(I).EQ.ONE  .AND.                                               
     .     THKIJ(I).EQ.ZERO .AND.                                               
     .     SSTIJ(I).GT.SSTMAX ) SSTIJ(I) = SSTMAX                               
   40 CONTINUE                                                                  
C                                                                               
C*********************************************************************          
C****   UPDATE GROUND TEMPERATURE OVER OCEANS AND GROUND WETNESS  ****          
C*********************************************************************          
                                                                                
      DO 50 I=1,IRUN                                                            
      IF( ZLWIJ(I).EQ.ONE .AND. THKIJ(I).EQ.ZERO )   TGZIJ(I) = SSTIJ(I)        
      IF( ZLWIJ(I).EQ.ONE .OR.  ZLWIJ(I).EQ.THREE .OR.                          
     .    ZLWIJ(I).EQ.TWO .AND. ALBIJ(I).GT.ALBSNO ) WETIJ(I) = ONE             
   50 CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE SETBCS (KBC,NYMD,NHMS,IRUN)                                    
C***********************************************************************        
C  PURPOSE                                                                      
C     Driver for GETBCS which fills boundary condition common block             
C  ARGUMENTS   DESCRIPTION                                                      
C     KBC ...... Unit to read BOUNDARY CONDITIONS                               
C     NYMD ..... Current YYMMDD                                                 
C     NHMS ..... Current HHMMSS                                                 
C     IRUN ..... Horizontal Resolution (IMxJM)                                  
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
                                                                                
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
                                                                                
      CALL GETBCS (KBC, NYMD, NHMS, TGZ, PHISZ, TVAZ, RADSWT,                   
     .             SSTZ, ALBZ, GWZ, THKZ, SRZ, ZLWI, IRUN)                      
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE ASTRO ( NYMD,NHMS, ALAT,ALON, IM,JM, COSZ,RA )                 
C***********************************************************************        
C                                                                               
C  INPUT:                                                                       
C  ======                                                                       
C    NYMD     : CURRENT YYMMDD                                                  
C    NHMS     : CURRENT HHMMSS                                                  
C    ALAT(JM) : LATITUDES  IN DEGREES.                                          
C    ALON(IM) : LONGITUDES IN DEGREES. (0 = GREENWICH, + = EAST).               
C    IM, JM   : # OF LONS AND LATS.                                             
C                                                                               
C  OUTPUT:                                                                      
C  =======                                                                      
C    COSZ(IM,JM) : COSINE OF ZENITH ANGLE.                                      
C    RA          : EARTH-SUN DISTANCE IN UNITS OF                               
C                  THE ORBIT'S SEMI-MAJOR AXIS.                                 
C                                                                               
C  NOTE:                                                                        
C  =====                                                                        
C  THE INSOLATION AT THE TOP OF THE ATMOSPHERE IS:                              
C                                                                               
C  S(I,J) = (SOLAR CONSTANT)*(1/RA**2)*COSZ(I,J),                               
C                                                                               
C  WHERE:                                                                       
C  RA AND COSZ(I,J) ARE THE TWO OUTPUTS OF THIS SUBROUTINE.                     
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
      DIMENSION COSZ(IM,JM), ALAT(JM), ALON(IM)                                 
      INTEGER YEAR, DAY, SEC                                                    
                                                                                
      REAL PI, ZERO, ONE, TWO, SIX, DG2RD, YRLEN, EQNX, OB, ECC, PER            
      REAL DAYLEN, FAC, THM, THP, THNOW, ZS, ZC, SJ, CJ                         
      PARAMETER (PI = 3.1415926535898)                                          
      PARAMETER (ZERO=0.,ONE=1.,TWO=2.,SIX=6.)                                  
      PARAMETER (DG2RD=PI/180.)                                                 
                                                                                
      INTEGER    DAYSCY                                                         
      PARAMETER (YRLEN=365.25,DAYSCY=365*4+1)                                   
      PARAMETER (EQNX=80.9028)                                                  
      PARAMETER (OB=23.45*DG2RD, ECC=.0167, PER=102.0*DG2RD)                    
      PARAMETER (DAYLEN=86400.)                                                 
                                                                                
      REAL      TH(DAYSCY),T0,T1,T2,T3,T4,FUN,Y,HC(600),MNDY(12,4)              
                                                                                
      LOGICAL FIRST                                                             
      DATA    FIRST/.TRUE./                                                     
      SAVE                                                                      
                                                                                
      DATA MNDY /0,31,60,91,121,152,182,213,244,274,305,335,366,                
     .           397,34*0 /                                                     
                                                                                
      FUN(Y) = (TWO*PI/((ONE-ECC**2)**1.5))*(ONE/YRLEN)                         
     .       * (ONE - ECC*COS(Y-PER)) ** 2                                      
      NSECF(N) = N/10000*3600 + MOD(N,10000)/100* 60 + MOD(N,100)               
C                                                                               
C***********************************************************************        
C*                          SET CURRENT TIME                           *        
C***********************************************************************        
C                                                                               
      YEAR  = NYMD / 10000                                                      
      MONTH = MOD(NYMD,10000) / 100                                             
      DAY   = MOD(NYMD,100)                                                     
      SEC   = NSECF(NHMS)                                                       
C                                                                               
C***********************************************************************        
C*                 COMPUTE DAY-ANGLES FOR 4-YEAR CYCLE                 *        
C***********************************************************************        
C                                                                               
      IF(FIRST) THEN                                                            
           DO 100 I=15,48                                                       
           MNDY(I,1) = MNDY(I-12,1) + 365                                       
100        CONTINUE                                                             
                                                                                
           KM  = INT(EQNX) + 1                                                  
           FAC = KM-EQNX                                                        
           T0 = ZERO                                                            
           T1 = FUN(T0         )*FAC                                            
           T2 = FUN(ZERO+T1/TWO)*FAC                                            
           T3 = FUN(ZERO+T2/TWO)*FAC                                            
           T4 = FUN(ZERO+T3    )*FAC                                            
           TH(KM) = (T1 + TWO*(T2 + T3) + T4) / SIX                             
                                                                                
           DO 200 K=2,DAYSCY                                                    
           T1 = FUN(TH(KM)       )                                              
           T2 = FUN(TH(KM)+T1/TWO)                                              
           T3 = FUN(TH(KM)+T2/TWO)                                              
           T4 = FUN(TH(KM)+T3    )                                              
           KP = MOD(KM,DAYSCY) + 1                                              
           TH(KP) = TH(KM) + (T1 + TWO*(T2 + T3) + T4) / SIX                    
           KM = KP                                                              
 200       CONTINUE                                                             
                                                                                
           FIRST=.FALSE.                                                        
      ENDIF                                                                     
C                                                                               
C***********************************************************************        
C*            COMPUTE EARTH-SUN DISTANCE TO CURRENT SECOND             *        
C***********************************************************************        
C                                                                               
      IDAY   = DAY + MNDY(MONTH,MOD(YEAR,4)+1)                                  
      IDAYP1 = MOD( IDAY,DAYSCY) + 1                                            
      THM    = MOD( TH(IDAY)  ,TWO*PI)                                          
      THP    = MOD( TH(IDAYP1),TWO*PI)                                          
                                                                                
      IF(THP.LT.THM) THP = THP + TWO*PI                                         
      FAC   = FLOAT(SEC)/DAYLEN                                                 
      THNOW = THM*(ONE-FAC) + THP*FAC                                           
                                                                                
      ZS = SIN(THNOW) * SIN(OB)                                                 
      ZC = SQRT(ONE-ZS*ZS)                                                      
      RA = (1.-ECC*ECC) / ( ONE-ECC*COS(THNOW-PER) )                            
C                                                                               
C***********************************************************************        
C*                 COMPUTE COSINE OF THE ZENITH ANGLE                  *        
C***********************************************************************        
C                                                                               
           FAC = FAC*TWO*PI + PI                                                
      DO 300 I=1,IMR                                                            
      HC(I) = COS( FAC+ALON(I)*DG2RD )                                          
  300 CONTINUE                                                                  
                                                                                
      DO 400 J=1,JM                                                             
                SJ = SIN(ALAT(J)*DG2RD)                                         
                CJ = SQRT(ONE-SJ*SJ)                                            
      DO 400 I=1,IMR                                                            
      COSZ(I,J) = SJ*ZS + CJ*ZC*HC(I)                                           
400   CONTINUE                                                                  
                                                                                
      DO 500 I=1,IMJMR                                                          
      IF ( COSZ(I,1).LT.ZERO ) COSZ(I,1) = ZERO                                 
500   CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE DIAGDRV                                                
C***********************************************************************
C                                                                       
C  PURPOSE                                                              
C     SET POINTER LOCATIONS FOR DIAGNOSTICS TURNED ON IN NAMELIST       
C  I/O DDNAME  DESCRIPTION                                              
C     11       REWINDABLE COPY OF CARD-IMAGE DATA FROM UNIT 5 (TEMPNL)  
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
C                                                                       
#include <vrslv.com>                                                    
C                                                                       
      PARAMETER ( NDIAGT  = 500)                                        
                                                                        
      LOGICAL LDIAG (NDIAGT)                                            
      COMMON  /DIAG/ NDIAGMX, QDIAG (IMR,JMR+1,1)     
C                                                                       
C ======================================================================
C ====               D I A G N O S T I C    M E N U                 ====
C ======================================================================
C                                                                       
C   #    NAME      UNITS   LEVELS             DESCRIPTION               
C  ===  =======   =======  ======  =====================================
C   1    UFLUX    N/M**2      1    U-MOMENTUM SURFACE STRESS
C   2    VFLUX    N/M**2      1    V-MOMENTUM SURFACE STRESS
C   3    HFLUX    W/M**2      1    SURFACE FLUX OF SENSIBLE    HEAT     
C   4    EFLUX    W/M**2      1    SURFACE FLUX OF EVAPORATIVE HEAT     
C   5    QICE     W/M**2      1    HEAT CONDUCTION THROUGH SEA-ICE      
C   6    RADLWG   W/M**2      1    NET UPWARD   LW RADIATION AT GROUND  
C   7    RADSWG   W/M**2      1    NET DOWNWARD SW RADIATION AT GROUND  
C   8    RI                 NLAY   RICHARDSON NUMBER               
C   9    CT          M/SEC    1    SURFACE DRAG COEFFICIENT FOR T AND Q 
C  10    CU          M/SEC    1    SURFACE DRAG COEFFICIENT FOR U AND V 
C  11    ET       M**2/SEC  NLAY   DIFFUSIVITY  COEFFICIENT FOR T AND Q 
C  12    EU       M**2/SEC  NLAY   DIFFUSIVITY  COEFFICIENT FOR U AND V 
C  13    TURBU   M/SEC/DAY  NLAY   U-MOMENTUM  CHANGES DUE TO TURBULENCE
C  14    TURBV   M/SEC/DAY  NLAY   V-MOMENTUM  CHANGES DUE TO TURBULENCE
C  15    TURBT     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO TURBULENCE
C  16    TURBQ    G/DK/DAY  NLAY   MOISTURE    CHANGES DUE TO TURBULENCE
C  17    MOISTT    DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO MOIST PROC
C  18    MOISTQ   G/DK/DAY  NLAY   MOISTURE    CHANGES DUE TO MOIST PROC
C  19    RADLW     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO LW RAD.   
C  20    RADSW     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO SW RAD.   
C  21    PREACC     MM/DAY   1     TOTAL PRECIPITATION                  
C  22    PRECON     MM/DAY   1     CONVECTIVE PRECIPITATION             
C  23    TUFLUX     N/M**2  NLAY   TURBULENT FLUX OF U-MOMENTUM         
C  24    TVFLUX     N/M**2  NLAY   TURBULENT FLUX OF V-MOMENTUM         
C  25    TTFLUX     W/M**2  NLAY   TURBULENT FLUX OF TEMPERATURE        
C  26    TQFLUX     W/M**2  NLAY   TURBULENT FLUX OF MOISTURE           
C  27    CN          M/SEC   1     NEUTRAL DRAG COEFFICIENT             
C  28    WINDS       M/SEC   1     SURFACE WIND SPEED                   
C  29    DTSRF         DEG   1     AIR/SURFACE VIRTUAL TEMPERATURE DIFF.
C  30    TG            DEG   1     GROUND TEMPERATURE                   
C  31    TS            DEG   1     SURFACE AIR TEMPERATURE              
C  32    DTG           DEG   1     GROUND TEMPERATURE ADJUSTMENT        
C  33    QG           G/KG   1     GROUND SPECIFIC HUMIDITY             
C  34    QS           G/KG   1     SATURATION SURFACE SPECIFIC HUMIDITY 
C  35    TGRLW         DEG   1     GROUND TEMP SEEN BY LONGWAVE RAD     
C  36    ST4        W/M**2   1     SIGMA T**4                           
C  37    OLR        W/M**2   1     OUTGOING LONGWAVE RADIATION          
C  38    OLRCLR     W/M**2   1     OUTGOING LONGWAVE RADIATION CLEAR SKY
C  39    LWGCLR     W/M**2   1     SURFACE LONGWAVE FLUX CLEAR SKY      
C  40    LWCLR     DEG/DAY  NLAY   LW CLEAR SKY HEATING RATES           
C  41    TLW           DEG  NLAY   TEMPERATURE SEEN BY LONGWAVE RAD     
C  42    SHLW          G/G  NLAY   SPEC. HUM.  SEEN BY LONGWAVE RAD     
C  43    OZLW          G/G  NLAY   OZONE       SEEN BY LONGWAVE RAD     
C  44    CLMOLW             NLAY   MAXIMUM OVERLAP CLOUD FRACTION FOR LW
C  45    CLROLW             NLAY   RANDOM  OVERLAP CLOUD FRACTION FOR LW
C  46    CLMOSW             NLAY   MAXIMUM OVERLAP CLOUD FRACTION FOR SW
C  47    CLROSW             NLAY   RANDOM  OVERLAP CLOUD FRACTION FOR SW
C  48    RADSWT              1     INCIDENT SW RADIATION AT TOP OF ATMS.
C  49    FREQMD              1     FREQUENCY OF MID-LEVEL CONVECTION    
C  50    FREQPN             NLAY   FREQUENCY OF PEN-LEVEL CONVECTION    
C  51    FREQLW              1     FREQUENCY OF LOW-LEVEL CONVECTION    
C  52    EVAP       MM/DAY   1     SURFACE EVAPORATION                  
C  53    DPDT       MB/SEC   1     SURFACE PRESSURE TENDENCY            
C  54    ANALU     M/S/SEC  NLAY   ANALYSIS INCREMENT OF U              
C  55    ANALV     M/S/SEC  NLAY   ANALYSIS INCREMENT OF V              
C  56    ANALT MB*THET/SEC  NLAY   ANALYSIS INCREMENT OF PI*THETA/PKAPPA    
C  57    ANALQ MB*G/G /SEC  NLAY   ANALYSIS INCREMENT OF PI*Q           
C  58    OMEGA      NB/SEC  NLAY   VERTICAL VELOCITY OMEGA (DPDT)       
C  59    DUDT      M/S/DAY  NLAY   TOTAL  U TENDENCY    (C-GRID)           
C  60    DVDT      M/S/DAY  NLAY   TOTAL  V TENDENCY    (C-GRID)           
C  61    DTDT      DEG/DAY  NLAY   TOTAL  T TENDENCY    (C-GRID)           
C  62    DQDT     G/KG/DAY  NLAY   TOTAL  Q TENDENCY    (C-GRID)           
C  63    VORT      SEC**-1  NLAY   RELATIVE VORTICITY   (C-GRID)          
C  64    PGV       M/S/DAY  NLAY   V PRESSURE GRADIENT TENDENCY (A-GRID)
C  65    CORU      M/S/DAY  NLAY   U CORIOLIS TENDENCY  (A-GRID)         
C  66    CORV      M/S/DAY  NLAY   V CORIOLIS TENDENCY  (A-GRID)         
C  67    USTAR     M/S      1      USTAR                                
C  68    Z0        M        1      SURFACE ROUGHNESS                    
C  69    FRQTRB             NLAY-1 FREQUENCY OF TURBULENCE              
C  70    PBL       MB       1      PBL DEPTH                            
C  71    SWCLR     DEG/DAY  NLAY   CLEAR-SKY SW HEATING RATES           
C  72    OSR       W/M**2   1      OUTGOING SW RADIATION                
C  73    OSRCLR    W/M**2   1      CLEAR-SKY OSR                        
C  74    CLDMAS    KG*M/S   NLAY   CLOUD MASS FLUX                      
C  75    UAVE      M/SEC    NLAY   TIME-AVERAGED U                      
C  76    VAVE      M/SEC    NLAY   TIME-AVERAGED V                      
C  77    TAVE      DEG      NLAY   TIME-AVERAGED T                      
C  78    QAVE      G/G      NLAY   TIME-AVERAGED Q                      
C  79    MDIV      MB/DAY   NLAY   MASS DIVERGENCE                      
C  80    PAVE      MB       1      TIME-AVERAGED PS-PTOP                
C  81    QQAVE     (M/S)**2 NLAY   TIME-AVERAGE QQ                      
C  82    SWGCLR    W/M**2   1      SURFACE SHORTWAVE FLUX CLEAR SKY     
C  83    ANALP     MB/DAY   1      ANALYSIS INCREMENT OF P              
C  84    SDIAG1             1      USER-DEFINED SURFACE   DIAGNOSTIC-1    
C  85    SDIAG2             1      USER-DEFINED SURFACE   DIAGNOSTIC-2    
C  86    UDIAG1             NLAY   USER-DEFINED UPPER-AIR DIAGNOSTIC-1    
C  87    UDIAG2             NLAY   USER-DEFINED UPPER-AIR DIAGNOSTIC-2    
C  88    DIABU     M/S/DAY  NLAY   TOTAL DIABATIC FORCING ON U-WIND       
C  89    DIABV     M/S/DAY  NLAY   TOTAL DIABATIC FORCING ON V-WIND       
C  90    DIABT     DEG/DAY  NLAY   TOTAL DIABATIC FORCING ON TEMPERATURE  
C  91    DIABQ    G/KG/DAY  NLAY   TOTAL DIABATIC FORCING ON SPEC. HUM.   
C  92    PSIGDOT    MB/DAY  NLAY   VERTICAL SIGMA VELOCITY PI*D(SIGMA)/DT   
C  93    PGU       M/S/DAY  NLAY   U PRESSURE GRADIENT TENDENCY (A-GRID)
C  94    PUFLX     M/S/DAY  NLAY   3-D FLUX CONVERGENCE OF PU   (A-GRID)
C  95    PVFLX     M/S/DAY  NLAY   3-D FLUX CONVERGENCE OF PV   (A-GRID)
C  96    PTFLX     DEG/DAY  NLAY   3-D FLUX CONVERGENCE OF PT   (A-GRID)
C  97    PQFLX    G/KG/DAY  NLAY   3-D FLUX CONVERGENCE OF PQ   (A-GRID)
C  98    DPUDT     M/S/DAY  NLAY   TOTAL PU TENDENCY            (A-GRID)
C  99    DPVDT     M/S/DAY  NLAY   TOTAL PV TENDENCY            (A-GRID)
C 100    DPTDT     DEG/DAY  NLAY   TOTAL PT TENDENCY            (A-GRID)
C 101    DPQDT    G/KG/DAY  NLAY   TOTAL PQ TENDENCY            (A-GRID)
C 102    VINTUQ M/SEC G/KG  1      VERTICALLY INTEGRATED U*Q    (C-GRID)
C 103    VINTVQ M/SEC G/KG  1      VERTICALLY INTEGRATED V*Q    (C-GRID)
C 104    VINTUT M/SEC  DEG  1      VERTICALLY INTEGRATED U*T    (C-GRID)
C 105    VINTVT M/SEC  DEG  1      VERTICALLY INTEGRATED V*T    (C-GRID)
C 106    CLDFRC             1      2-DIMENSIONAL TOTAL CLOUD FRACTION   
C 107    QINT               1      PRECIPITABLE WATER   (GM/CM**2)
C 108    U2M                1      U AT  2 METERS (M/SEC)
C 109    V2M                1      V AT  2 METERS (M/SEC)
C 110    T2M                1      T AT  2 METERS (DEG)
C 111    Q2M                1      Q AT  2 METERS (G/KG)
C 112    U10M               1      U AT 10 METERS (M/SEC)
C 113    V10M               1      V AT 10 METERS (M/SEC)
C 114    T10M               1      T AT 10 METERS (DEG)
C 115    Q10M               1      Q AT 10 METERS (G/KG)
C 116    DTRAIN    KG*M/S   NLAY   CONVECTIVE CLOUD DETRAINMENT               
C 117    QFILL     G/KG/DAY NLAY   FILLING OF NEGATIVE SPECIFIC HUMIDITY            
C                                                                       
C **********************************************************************
C **********************************************************************

      LOGICAL   LUFLUX                                                  
      LOGICAL   LVFLUX                                                  
      LOGICAL   LHFLUX                                                  
      LOGICAL   LEFLUX                                                  
      LOGICAL   LQICE                                                   
      LOGICAL   LRADLWG                                                 
      LOGICAL   LRADSWG                                                 
      LOGICAL   LRI                                                     
      LOGICAL   LCT                                                     
      LOGICAL   LCU                                                     
      LOGICAL   LET                                                     
      LOGICAL   LEU                                                     
      LOGICAL   LTURBU                                                  
      LOGICAL   LTURBV                                                  
      LOGICAL   LTURBT                                                  
      LOGICAL   LTURBQ                                                  
      LOGICAL   LMOISTT                                                 
      LOGICAL   LMOISTQ                                                 
      LOGICAL   LRADLW                                                  
      LOGICAL   LRADSW                                                  
      LOGICAL   LPREACC                                                 
      LOGICAL   LPRECON                                                 
      LOGICAL   LTUFLUX                                                 
      LOGICAL   LTVFLUX                                                 
      LOGICAL   LTTFLUX                                                 
      LOGICAL   LTQFLUX                                                 
      LOGICAL   LCN                                                     
      LOGICAL   LWINDS                                                  
      LOGICAL   LDTSRF                                                  
      LOGICAL   LTG                                                     
      LOGICAL   LTS                                                     
      LOGICAL   LDTG                                                    
      LOGICAL   LQG                                                     
      LOGICAL   LQS                                                     
      LOGICAL   LTGRLW                                                  
      LOGICAL   LST4                                                    
      LOGICAL   LOLR                                                    
      LOGICAL   LOLRCLR                                                 
      LOGICAL   LLWGCLR                                                 
      LOGICAL   LLWCLR                                                  
      LOGICAL   LTLW                                                    
      LOGICAL   LSHLW                                                   
      LOGICAL   LOZLW                                                   
      LOGICAL   LCLMOLW                                                 
      LOGICAL   LCLROLW                                                 
      LOGICAL   LCLMOSW                                                 
      LOGICAL   LCLROSW                                                 
      LOGICAL   LRADSWT                                                 
      LOGICAL   LFREQMD                                                 
      LOGICAL   LFREQPN                                                 
      LOGICAL   LFREQLW                                                 
      LOGICAL   LEVAP                                                   
      LOGICAL   LDPDT                                                   
      LOGICAL   LANALU                                                  
      LOGICAL   LANALV                                                  
      LOGICAL   LANALT                                                  
      LOGICAL   LANALQ                                                  
      LOGICAL   LOMEGA                                                  
      LOGICAL   LDUDT                                                  
      LOGICAL   LDVDT                                                  
      LOGICAL   LDTDT                                                  
      LOGICAL   LDQDT                                                  
      LOGICAL   LVORT                                                   
      LOGICAL   LPGV                                                    
      LOGICAL   LCORU                                                   
      LOGICAL   LCORV                                                   
      LOGICAL   LUSTAR                                                  
      LOGICAL   LZ0                                                     
      LOGICAL   LFRQTRB                                                 
      LOGICAL   LPBL                                                    
      LOGICAL   LSWCLR                                                  
      LOGICAL   LOSR                                                    
      LOGICAL   LOSRCLR                                                 
      LOGICAL   LCLDMAS                                                 
      LOGICAL   LUAVE                                                   
      LOGICAL   LVAVE                                                   
      LOGICAL   LTAVE                                                   
      LOGICAL   LQAVE                                                   
      LOGICAL   LMDIV                                                   
      LOGICAL   LPAVE                                                   
      LOGICAL   LQQAVE                                                  
      LOGICAL   LSWGCLR                                                 
      LOGICAL   LANALP                                                  
      LOGICAL   LSDIAG1                                                  
      LOGICAL   LSDIAG2                                                  
      LOGICAL   LUDIAG1                                                  
      LOGICAL   LUDIAG2                                                  
      LOGICAL   LDIABU                                                   
      LOGICAL   LDIABV                                                   
      LOGICAL   LDIABT                                                   
      LOGICAL   LDIABQ                                                   
      LOGICAL   LPSIGDOT                                                 
      LOGICAL   LPGU                                                     
      LOGICAL   LPUFLX                                                   
      LOGICAL   LPVFLX                                                   
      LOGICAL   LPTFLX                                                   
      LOGICAL   LPQFLX                                                   
      LOGICAL   LDPUDT                                                   
      LOGICAL   LDPVDT                                                   
      LOGICAL   LDPTDT                                                   
      LOGICAL   LDPQDT                                                   
      LOGICAL   LVINTUQ                                                  
      LOGICAL   LVINTVQ                                                  
      LOGICAL   LVINTUT                                                  
      LOGICAL   LVINTVT                                                  
      LOGICAL   LCLDFRC                                                  
      LOGICAL   LQINT                                                    
      LOGICAL   LU2M                                                    
      LOGICAL   LV2M                                                    
      LOGICAL   LT2M                                                    
      LOGICAL   LQ2M                                                    
      LOGICAL   LU10M                                                    
      LOGICAL   LV10M                                                    
      LOGICAL   LT10M                                                    
      LOGICAL   LQ10M                                                    
      LOGICAL   LDTRAIN                                                  
      LOGICAL   LQFILL                                                   
C                                                                       
      EQUIVALENCE ( LDIAG( 1) ,  LUFLUX  )                              
      EQUIVALENCE ( LDIAG( 2) ,  LVFLUX  )                              
      EQUIVALENCE ( LDIAG( 3) ,  LHFLUX  )                              
      EQUIVALENCE ( LDIAG( 4) ,  LEFLUX  )                              
      EQUIVALENCE ( LDIAG( 5) ,  LQICE   )                              
      EQUIVALENCE ( LDIAG( 6) ,  LRADLWG )                              
      EQUIVALENCE ( LDIAG( 7) ,  LRADSWG )                              
      EQUIVALENCE ( LDIAG( 8) ,  LRI     )                              
      EQUIVALENCE ( LDIAG( 9) ,  LCT     )                              
      EQUIVALENCE ( LDIAG(10) ,  LCU     )                              
      EQUIVALENCE ( LDIAG(11) ,  LET     )                              
      EQUIVALENCE ( LDIAG(12) ,  LEU     )                              
      EQUIVALENCE ( LDIAG(13) ,  LTURBU  )                              
      EQUIVALENCE ( LDIAG(14) ,  LTURBV  )                              
      EQUIVALENCE ( LDIAG(15) ,  LTURBT  )                              
      EQUIVALENCE ( LDIAG(16) ,  LTURBQ  )                              
      EQUIVALENCE ( LDIAG(17) ,  LMOISTT )                              
      EQUIVALENCE ( LDIAG(18) ,  LMOISTQ )                              
      EQUIVALENCE ( LDIAG(19) ,  LRADLW  )                              
      EQUIVALENCE ( LDIAG(20) ,  LRADSW  )                              
      EQUIVALENCE ( LDIAG(21) ,  LPREACC )                              
      EQUIVALENCE ( LDIAG(22) ,  LPRECON )                              
      EQUIVALENCE ( LDIAG(23) ,  LTUFLUX )                              
      EQUIVALENCE ( LDIAG(24) ,  LTVFLUX )                              
      EQUIVALENCE ( LDIAG(25) ,  LTTFLUX )                              
      EQUIVALENCE ( LDIAG(26) ,  LTQFLUX )                              
      EQUIVALENCE ( LDIAG(27) ,  LCN     )                              
      EQUIVALENCE ( LDIAG(28) ,  LWINDS  )                              
      EQUIVALENCE ( LDIAG(29) ,  LDTSRF  )                              
      EQUIVALENCE ( LDIAG(30) ,  LTG     )                              
      EQUIVALENCE ( LDIAG(31) ,  LTS     )                              
      EQUIVALENCE ( LDIAG(32) ,  LDTG    )                              
      EQUIVALENCE ( LDIAG(33) ,  LQG     )                              
      EQUIVALENCE ( LDIAG(34) ,  LQS     )                              
      EQUIVALENCE ( LDIAG(35) ,  LTGRLW  )                              
      EQUIVALENCE ( LDIAG(36) ,  LST4    )                              
      EQUIVALENCE ( LDIAG(37) ,  LOLR    )                              
      EQUIVALENCE ( LDIAG(38) ,  LOLRCLR )                              
      EQUIVALENCE ( LDIAG(39) ,  LLWGCLR )                              
      EQUIVALENCE ( LDIAG(40) ,  LLWCLR  )                              
      EQUIVALENCE ( LDIAG(41) ,  LTLW    )                              
      EQUIVALENCE ( LDIAG(42) ,  LSHLW   )                              
      EQUIVALENCE ( LDIAG(43) ,  LOZLW   )                              
      EQUIVALENCE ( LDIAG(44) ,  LCLMOLW )                              
      EQUIVALENCE ( LDIAG(45) ,  LCLROLW )                              
      EQUIVALENCE ( LDIAG(46) ,  LCLMOSW )                              
      EQUIVALENCE ( LDIAG(47) ,  LCLROSW )                              
      EQUIVALENCE ( LDIAG(48) ,  LRADSWT )                              
      EQUIVALENCE ( LDIAG(49) ,  LFREQMD )                              
      EQUIVALENCE ( LDIAG(50) ,  LFREQPN )                              
      EQUIVALENCE ( LDIAG(51) ,  LFREQLW )                              
      EQUIVALENCE ( LDIAG(52) ,  LEVAP   )                              
      EQUIVALENCE ( LDIAG(53) ,  LDPDT   )                              
      EQUIVALENCE ( LDIAG(54) ,  LANALU  )                              
      EQUIVALENCE ( LDIAG(55) ,  LANALV  )                              
      EQUIVALENCE ( LDIAG(56) ,  LANALT  )                              
      EQUIVALENCE ( LDIAG(57) ,  LANALQ  )                              
      EQUIVALENCE ( LDIAG(58) ,  LOMEGA  )                              
      EQUIVALENCE ( LDIAG(59) ,  LDUDT   )                              
      EQUIVALENCE ( LDIAG(60) ,  LDVDT   )                              
      EQUIVALENCE ( LDIAG(61) ,  LDTDT   )                              
      EQUIVALENCE ( LDIAG(62) ,  LDQDT   )                              
      EQUIVALENCE ( LDIAG(63) ,  LVORT   )                              
      EQUIVALENCE ( LDIAG(64) ,  LPGV    )                              
      EQUIVALENCE ( LDIAG(65) ,  LCORU   )                              
      EQUIVALENCE ( LDIAG(66) ,  LCORV   )                              
      EQUIVALENCE ( LDIAG(67) ,  LUSTAR  )                              
      EQUIVALENCE ( LDIAG(68) ,  LZ0     )                              
      EQUIVALENCE ( LDIAG(69) ,  LFRQTRB )                              
      EQUIVALENCE ( LDIAG(70) ,  LPBL    )                              
      EQUIVALENCE ( LDIAG(71) ,  LSWCLR  )                              
      EQUIVALENCE ( LDIAG(72) ,  LOSR    )                              
      EQUIVALENCE ( LDIAG(73) ,  LOSRCLR )                              
      EQUIVALENCE ( LDIAG(74) ,  LCLDMAS )                              
      EQUIVALENCE ( LDIAG(75) ,  LUAVE   )                              
      EQUIVALENCE ( LDIAG(76) ,  LVAVE   )                              
      EQUIVALENCE ( LDIAG(77) ,  LTAVE   )                              
      EQUIVALENCE ( LDIAG(78) ,  LQAVE   )                              
      EQUIVALENCE ( LDIAG(79) ,  LMDIV   )                              
      EQUIVALENCE ( LDIAG(80) ,  LPAVE   )                              
      EQUIVALENCE ( LDIAG(81) ,  LQQAVE  )                              
      EQUIVALENCE ( LDIAG(82) ,  LSWGCLR )                              
      EQUIVALENCE ( LDIAG(83) ,  LANALP  )                              
      EQUIVALENCE ( LDIAG(84) ,  LSDIAG1 )                              
      EQUIVALENCE ( LDIAG(85) ,  LSDIAG2 )                              
      EQUIVALENCE ( LDIAG(86) ,  LUDIAG1 )                              
      EQUIVALENCE ( LDIAG(87) ,  LUDIAG2 )                              
      EQUIVALENCE ( LDIAG(88) ,  LDIABU  )                              
      EQUIVALENCE ( LDIAG(89) ,  LDIABV  )                              
      EQUIVALENCE ( LDIAG(90) ,  LDIABT  )                              
      EQUIVALENCE ( LDIAG(91) ,  LDIABQ  )                              
      EQUIVALENCE ( LDIAG(92) ,  LPSIGDOT)                              
      EQUIVALENCE ( LDIAG(93) ,  LPGU    )                              
      EQUIVALENCE ( LDIAG(94) ,  LPUFLX  )                              
      EQUIVALENCE ( LDIAG(95) ,  LPVFLX  )                              
      EQUIVALENCE ( LDIAG(96) ,  LPTFLX  )                              
      EQUIVALENCE ( LDIAG(97) ,  LPQFLX  )                              
      EQUIVALENCE ( LDIAG(98) ,  LDPUDT  )                              
      EQUIVALENCE ( LDIAG(99) ,  LDPVDT  )                              
      EQUIVALENCE ( LDIAG(100),  LDPTDT  )                              
      EQUIVALENCE ( LDIAG(101),  LDPQDT  )                              
      EQUIVALENCE ( LDIAG(102),  LVINTUQ )                              
      EQUIVALENCE ( LDIAG(103),  LVINTVQ )                              
      EQUIVALENCE ( LDIAG(104),  LVINTUT )                              
      EQUIVALENCE ( LDIAG(105),  LVINTVT )                              
      EQUIVALENCE ( LDIAG(106),  LCLDFRC )                              
      EQUIVALENCE ( LDIAG(107),  LQINT   )                              
      EQUIVALENCE ( LDIAG(108),  LU2M    )                              
      EQUIVALENCE ( LDIAG(109),  LV2M    )                              
      EQUIVALENCE ( LDIAG(110),  LT2M    )                              
      EQUIVALENCE ( LDIAG(111),  LQ2M    )                              
      EQUIVALENCE ( LDIAG(112),  LU10M   )                              
      EQUIVALENCE ( LDIAG(113),  LV10M   )                              
      EQUIVALENCE ( LDIAG(114),  LT10M   )                              
      EQUIVALENCE ( LDIAG(115),  LQ10M   )                              
      EQUIVALENCE ( LDIAG(116),  LDTRAIN )                              
      EQUIVALENCE ( LDIAG(117),  LQFILL  )                              
C                                                                       
C  DIAGNOSTICS INPUT NAMELIST                                           
C  ==========================                                           
C                                                                       
      NAMELIST /INDIAG/ LUFLUX  , LVFLUX  , LHFLUX  , LEFLUX  ,         
     .                  LQICE   , LRADLWG , LRADSWG , LRI     ,         
     .                  LCT     , LCU     , LET     , LEU     ,         
     .                  LTURBU  , LTURBV  , LTURBT  , LTURBQ  ,         
     .                  LMOISTT , LMOISTQ , LRADLW  , LRADSW  ,         
     .                  LPREACC , LPRECON , LTUFLUX , LTVFLUX ,         
     .                  LTTFLUX , LTQFLUX , LCN     , LWINDS  ,         
     .                  LDTSRF  , LTG     , LTS     , LDTG    ,         
     .                  LQG     , LQS     , LTGRLW  , LST4    ,         
     .                  LOLR    , LOLRCLR , LLWGCLR , LLWCLR  ,         
     .                  LTLW    , LSHLW   , LOZLW   , LCLMOLW ,         
     .                  LCLROLW , LCLMOSW , LCLROSW , LRADSWT ,         
     .                  LFREQMD , LFREQPN , LFREQLW , LEVAP   ,         
     .                  LDPDT   , LANALU  , LANALV  , LANALT  ,         
     .                  LANALQ  , LOMEGA  , LDUDT   , LDVDT   ,         
     .                  LDTDT   , LDQDT   , LVORT   , LPGV    ,         
     .                  LCORU   , LCORV   , LUSTAR  , LZ0     ,         
     .                  LFRQTRB , LPBL    , LSWCLR  , LOSR    ,         
     .                  LOSRCLR , LCLDMAS , LUAVE   , LVAVE   ,         
     .                  LTAVE   , LQAVE   , LMDIV   , LPAVE   ,         
     .                  LQQAVE  , LSWGCLR , LANALP  , LSDIAG1 ,
     .                  LSDIAG2 , LUDIAG1 , LUDIAG2 , LDIABU  ,
     .                  LDIABV  , LDIABT  , LDIABQ  , LPSIGDOT,
     .                  LPGU    , LPUFLX  , LPVFLX  , LPTFLX  ,
     .                  LPQFLX  , LDPUDT  , LDPVDT  , LDPTDT  ,
     .                  LDPQDT  , LVINTUQ , LVINTVQ , LVINTUT ,
     .                  LVINTVT , LCLDFRC , LQINT   , LU2M    ,
     .                  LV2M    , LT2M    , LQ2M    , LU10M   ,
     .                  LV10M   , LT10M   , LQ10M   , LDTRAIN ,
     .                  LQFILL
                                                                        
C **********************************************************************
C ****               INITIALIZE DIAGNOSTIC LOGICALS                 ****
C **********************************************************************
                                                                        
      PRINT *                                                           
      PRINT *, 'Total Number of Diagnostics Allowed (NDIAGMX)'
      PRINT *, 'Defined by Users Application:  ',NDIAGMX
      PRINT *                                                           
                                                                        
      DO 10 N = 1,NDIAGT                                                
      LDIAG(N) = .FALSE.                                                
   10 CONTINUE                                                          
                                                                        
C **********************************************************************
C ****                  READ DIAGNOSTIC NAMELIST                    ****
C **********************************************************************
                                                                        
      REWIND 11                                                         
      WRITE (6 ,1000)                                                   
      READ  (11,INDIAG)                                                 
      WRITE (6 ,INDIAG)                                                 
                                                                        
C **********************************************************************
C ****       CALCULATE POINTERS FOR DIAGNOSTICS SET TO .TRUE.       ****
C **********************************************************************
                                                                        
      DO 20 N = 1,NDIAGT                                                
      IF(     LDIAG(N) ) THEN                                           
      CALL SETDIAG (N)                                                  
      CALL CLRDIAG (N)                                                  
      ENDIF                                                             
   20 CONTINUE                                                          
                                                                        
 1000 FORMAT(1X,'READING DIAGNOSTIC NAMELIST:')                         
      RETURN                                                            
      END                                                               
      SUBROUTINE DIAGDRVN (NFREQ)                                               
C***********************************************************************
C                                                                       
C  PURPOSE                                                              
C     Set Pointer Locations for Diagnostics turned on in NAMELIST       
C  OUTPUT
C     NDIAG(500)    HHMMSS Frequencies of Diagnostic Output
C                                                                       
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************
C                                                                       
#include <vrslv.com>                                                    
C                                                                       
      PARAMETER ( NDIAGT  = 500)                                        
                                                                        
      INTEGER NFREQ (NDIAGT)                                            
      INTEGER NDIAG (NDIAGT)                                            
      COMMON  /DIAG/ NDIAGMX, QDIAG (IMR,JMR+1,1)     
C                                                                       
C ======================================================================
C ====               D I A G N O S T I C    M E N U                 ====
C ======================================================================
C                                                                       
C   #    NAME      UNITS   LEVELS             DESCRIPTION               
C  ===  =======   =======  ======  =====================================
C   1    UFLUX    N/M**2      1    U-MOMENTUM SURFACE STRESS
C   2    VFLUX    N/M**2      1    V-MOMENTUM SURFACE STRESS
C   3    HFLUX    W/M**2      1    SURFACE FLUX OF SENSIBLE    HEAT     
C   4    EFLUX    W/M**2      1    SURFACE FLUX OF EVAPORATIVE HEAT     
C   5    QICE     W/M**2      1    HEAT CONDUCTION THROUGH SEA-ICE      
C   6    RADLWG   W/M**2      1    NET UPWARD   LW RADIATION AT GROUND  
C   7    RADSWG   W/M**2      1    NET DOWNWARD SW RADIATION AT GROUND  
C   8    RI                 NLAY   RICHARDSON NUMBER               
C   9    CT          M/SEC    1    SURFACE DRAG COEFFICIENT FOR T AND Q 
C  10    CU          M/SEC    1    SURFACE DRAG COEFFICIENT FOR U AND V 
C  11    ET       M**2/SEC  NLAY   DIFFUSIVITY  COEFFICIENT FOR T AND Q 
C  12    EU       M**2/SEC  NLAY   DIFFUSIVITY  COEFFICIENT FOR U AND V 
C  13    TURBU   M/SEC/DAY  NLAY   U-MOMENTUM  CHANGES DUE TO TURBULENCE
C  14    TURBV   M/SEC/DAY  NLAY   V-MOMENTUM  CHANGES DUE TO TURBULENCE
C  15    TURBT     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO TURBULENCE
C  16    TURBQ    G/DK/DAY  NLAY   MOISTURE    CHANGES DUE TO TURBULENCE
C  17    MOISTT    DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO MOIST PROC
C  18    MOISTQ   G/DK/DAY  NLAY   MOISTURE    CHANGES DUE TO MOIST PROC
C  19    RADLW     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO LW RAD.   
C  20    RADSW     DEG/DAY  NLAY   TEMPERATURE CHANGES DUE TO SW RAD.   
C  21    PREACC     MM/DAY   1     TOTAL PRECIPITATION                  
C  22    PRECON     MM/DAY   1     CONVECTIVE PRECIPITATION             
C  23    TUFLUX     N/M**2  NLAY   TURBULENT FLUX OF U-MOMENTUM         
C  24    TVFLUX     N/M**2  NLAY   TURBULENT FLUX OF V-MOMENTUM         
C  25    TTFLUX     W/M**2  NLAY   TURBULENT FLUX OF TEMPERATURE        
C  26    TQFLUX     W/M**2  NLAY   TURBULENT FLUX OF MOISTURE           
C  27    CN          M/SEC   1     NEUTRAL DRAG COEFFICIENT             
C  28    WINDS       M/SEC   1     SURFACE WIND SPEED                   
C  29    DTSRF         DEG   1     AIR/SURFACE VIRTUAL TEMPERATURE DIFF.
C  30    TG            DEG   1     GROUND TEMPERATURE                   
C  31    TS            DEG   1     SURFACE AIR TEMPERATURE              
C  32    DTG           DEG   1     GROUND TEMPERATURE ADJUSTMENT        
C  33    QG           G/KG   1     GROUND SPECIFIC HUMIDITY             
C  34    QS           G/KG   1     SATURATION SURFACE SPECIFIC HUMIDITY 
C  35    TGRLW         DEG   1     GROUND TEMP SEEN BY LONGWAVE RAD     
C  36    ST4        W/M**2   1     SIGMA T**4                           
C  37    OLR        W/M**2   1     OUTGOING LONGWAVE RADIATION          
C  38    OLRCLR     W/M**2   1     OUTGOING LONGWAVE RADIATION CLEAR SKY
C  39    LWGCLR     W/M**2   1     SURFACE LONGWAVE FLUX CLEAR SKY      
C  40    LWCLR     DEG/DAY  NLAY   LW CLEAR SKY HEATING RATES           
C  41    TLW           DEG  NLAY   TEMPERATURE SEEN BY LONGWAVE RAD     
C  42    SHLW          G/G  NLAY   SPEC. HUM.  SEEN BY LONGWAVE RAD     
C  43    OZLW          G/G  NLAY   OZONE       SEEN BY LONGWAVE RAD     
C  44    CLMOLW             NLAY   MAXIMUM OVERLAP CLOUD FRACTION FOR LW
C  45    CLROLW             NLAY   RANDOM  OVERLAP CLOUD FRACTION FOR LW
C  46    CLMOSW             NLAY   MAXIMUM OVERLAP CLOUD FRACTION FOR SW
C  47    CLROSW             NLAY   RANDOM  OVERLAP CLOUD FRACTION FOR SW
C  48    RADSWT              1     INCIDENT SW RADIATION AT TOP OF ATMS.
C  49    FREQMD              1     FREQUENCY OF MID-LEVEL CONVECTION    
C  50    FREQPN             NLAY   FREQUENCY OF PEN-LEVEL CONVECTION    
C  51    FREQLW              1     FREQUENCY OF LOW-LEVEL CONVECTION    
C  52    EVAP       MM/DAY   1     SURFACE EVAPORATION                  
C  53    DPDT       MB/SEC   1     SURFACE PRESSURE TENDENCY            
C  54    ANALU     M/S/SEC  NLAY   ANALYSIS INCREMENT OF U              
C  55    ANALV     M/S/SEC  NLAY   ANALYSIS INCREMENT OF V              
C  56    ANALT MB*THET/SEC  NLAY   ANALYSIS INCREMENT OF PI*THETA/PKAPPA    
C  57    ANALQ MB*G/G /SEC  NLAY   ANALYSIS INCREMENT OF PI*Q           
C  58    OMEGA      NB/SEC  NLAY   VERTICAL VELOCITY OMEGA (DPDT)       
C  59    DUDT      M/S/DAY  NLAY   TOTAL  U TENDENCY    (C-GRID)           
C  60    DVDT      M/S/DAY  NLAY   TOTAL  V TENDENCY    (C-GRID)           
C  61    DTDT      DEG/DAY  NLAY   TOTAL  T TENDENCY    (C-GRID)           
C  62    DQDT     G/KG/DAY  NLAY   TOTAL  Q TENDENCY    (C-GRID)           
C  63    VORT      SEC**-1  NLAY   RELATIVE VORTICITY   (C-GRID)          
C  64    PGV       M/S/DAY  NLAY   V PRESSURE GRADIENT TENDENCY (A-GRID)
C  65    CORU      M/S/DAY  NLAY   U CORIOLIS TENDENCY  (A-GRID)         
C  66    CORV      M/S/DAY  NLAY   V CORIOLIS TENDENCY  (A-GRID)         
C  67    USTAR     M/S      1      USTAR                                
C  68    Z0        M        1      SURFACE ROUGHNESS                    
C  69    FRQTRB             NLAY-1 FREQUENCY OF TURBULENCE              
C  70    PBL       MB       1      PBL DEPTH                            
C  71    SWCLR     DEG/DAY  NLAY   CLEAR-SKY SW HEATING RATES           
C  72    OSR       W/M**2   1      OUTGOING SW RADIATION                
C  73    OSRCLR    W/M**2   1      CLEAR-SKY OSR                        
C  74    CLDMAS    KG*M/S   NLAY   CLOUD MASS FLUX                      
C  75    UAVE      M/SEC    NLAY   TIME-AVERAGED U                      
C  76    VAVE      M/SEC    NLAY   TIME-AVERAGED V                      
C  77    TAVE      DEG      NLAY   TIME-AVERAGED T                      
C  78    QAVE      G/G      NLAY   TIME-AVERAGED Q                      
C  79    MDIV      MB/DAY   NLAY   MASS DIVERGENCE                      
C  80    PAVE      MB       1      TIME-AVERAGED PS-PTOP                
C  81    QQAVE     (M/S)**2 NLAY   TIME-AVERAGE QQ                      
C  82    SWGCLR    W/M**2   1      SURFACE SHORTWAVE FLUX CLEAR SKY     
C  83    ANALP     MB/DAY   1      ANALYSIS INCREMENT OF P              
C  84    SDIAG1             1      USER-DEFINED SURFACE   DIAGNOSTIC-1    
C  85    SDIAG2             1      USER-DEFINED SURFACE   DIAGNOSTIC-2    
C  86    UDIAG1             NLAY   USER-DEFINED UPPER-AIR DIAGNOSTIC-1    
C  87    UDIAG2             NLAY   USER-DEFINED UPPER-AIR DIAGNOSTIC-2    
C  88    DIABU     M/S/DAY  NLAY   TOTAL DIABATIC FORCING ON U-WIND       
C  89    DIABV     M/S/DAY  NLAY   TOTAL DIABATIC FORCING ON V-WIND       
C  90    DIABT     DEG/DAY  NLAY   TOTAL DIABATIC FORCING ON TEMPERATURE  
C  91    DIABQ    G/KG/DAY  NLAY   TOTAL DIABATIC FORCING ON SPEC. HUM.   
C  92    PSIGDOT    MB/DAY  NLAY   VERTICAL SIGMA VELOCITY PI*D(SIGMA)/DT   
C  93    PGU       M/S/DAY  NLAY   U PRESSURE GRADIENT TENDENCY (A-GRID)
C  94    PUFLX     M/S/DAY  NLAY   3-D FLUX CONVERGENCE OF PU   (A-GRID)
C  95    PVFLX     M/S/DAY  NLAY   3-D FLUX CONVERGENCE OF PV   (A-GRID)
C  96    PTFLX     DEG/DAY  NLAY   3-D FLUX CONVERGENCE OF PT   (A-GRID)
C  97    PQFLX    G/KG/DAY  NLAY   3-D FLUX CONVERGENCE OF PQ   (A-GRID)
C  98    DPUDT     M/S/DAY  NLAY   TOTAL PU TENDENCY            (A-GRID)
C  99    DPVDT     M/S/DAY  NLAY   TOTAL PV TENDENCY            (A-GRID)
C 100    DPTDT     DEG/DAY  NLAY   TOTAL PT TENDENCY            (A-GRID)
C 101    DPQDT    G/KG/DAY  NLAY   TOTAL PQ TENDENCY            (A-GRID)
C 102    VINTUQ M/SEC G/KG  1      VERTICALLY INTEGRATED U*Q    (C-GRID)
C 103    VINTVQ M/SEC G/KG  1      VERTICALLY INTEGRATED V*Q    (C-GRID)
C 104    VINTUT M/SEC  DEG  1      VERTICALLY INTEGRATED U*T    (C-GRID)
C 105    VINTVT M/SEC  DEG  1      VERTICALLY INTEGRATED V*T    (C-GRID)
C 106    CLDFRC             1      2-DIMENSIONAL TOTAL CLOUD FRACTION   
C 107    QINT               1      PRECIPITABLE WATER   (GM/CM**2)
C 108    U2M                1      U AT  2 METERS (M/SEC)
C 109    V2M                1      V AT  2 METERS (M/SEC)
C 110    T2M                1      T AT  2 METERS (DEG)
C 111    Q2M                1      Q AT  2 METERS (G/KG)
C 112    U10M               1      U AT 10 METERS (M/SEC)
C 113    V10M               1      V AT 10 METERS (M/SEC)
C 114    T10M               1      T AT 10 METERS (DEG)
C 115    Q10M               1      Q AT 10 METERS (G/KG)
C 116    DTRAIN    KG*M/S   NLAY   CONVECTIVE CLOUD DETRAINMENT               
C 117    QFILL     G/KG/DAY NLAY   FILLING OF NEGATIVE SPECIFIC HUMIDITY            
C                                                                       
C **********************************************************************
C **********************************************************************
C                                                                       
      INTEGER   NUFLUX                                                  
      INTEGER   NVFLUX                                                  
      INTEGER   NHFLUX                                                  
      INTEGER   NEFLUX                                                  
      INTEGER   NQICE                                                   
      INTEGER   NRADLWG                                                 
      INTEGER   NRADSWG                                                 
      INTEGER   NRI                                                     
      INTEGER   NCT                                                     
      INTEGER   NCU                                                     
      INTEGER   NET                                                     
      INTEGER   NEU                                                     
      INTEGER   NTURBU                                                  
      INTEGER   NTURBV                                                  
      INTEGER   NTURBT                                                  
      INTEGER   NTURBQ                                                  
      INTEGER   NMOISTT                                                 
      INTEGER   NMOISTQ                                                 
      INTEGER   NRADLW                                                  
      INTEGER   NRADSW                                                  
      INTEGER   NPREACC                                                 
      INTEGER   NPRECON                                                 
      INTEGER   NTUFLUX                                                 
      INTEGER   NTVFLUX                                                 
      INTEGER   NTTFLUX                                                 
      INTEGER   NTQFLUX                                                 
      INTEGER   NCN                                                     
      INTEGER   NWINDS                                                  
      INTEGER   NDTSRF                                                  
      INTEGER   NTG                                                     
      INTEGER   NTS                                                     
      INTEGER   NDTG                                                    
      INTEGER   NQG                                                     
      INTEGER   NQS                                                     
      INTEGER   NTGRLW                                                  
      INTEGER   NST4                                                    
      INTEGER   NOLR                                                    
      INTEGER   NOLRCLR                                                 
      INTEGER   NLWGCLR                                                 
      INTEGER   NLWCLR                                                  
      INTEGER   NTLW                                                    
      INTEGER   NSHLW                                                   
      INTEGER   NOZLW                                                   
      INTEGER   NCLMOLW                                                 
      INTEGER   NCLROLW                                                 
      INTEGER   NCLMOSW                                                 
      INTEGER   NCLROSW                                                 
      INTEGER   NRADSWT                                                 
      INTEGER   NFREQMD                                                 
      INTEGER   NFREQPN                                                 
      INTEGER   NFREQLW                                                 
      INTEGER   NEVAP                                                   
      INTEGER   NDPDT                                                   
      INTEGER   NANALU                                                  
      INTEGER   NANALV                                                  
      INTEGER   NANALT                                                  
      INTEGER   NANALQ                                                  
      INTEGER   NOMEGA                                                  
      INTEGER   NDUDT                                                  
      INTEGER   NDVDT                                                  
      INTEGER   NDTDT                                                  
      INTEGER   NDQDT                                                  
      INTEGER   NVORT                                                   
      INTEGER   NPGV                                                    
      INTEGER   NCORU                                                   
      INTEGER   NCORV                                                   
      INTEGER   NUSTAR                                                  
      INTEGER   NZ0                                                     
      INTEGER   NFRQTRB                                                 
      INTEGER   NPBL                                                    
      INTEGER   NSWCLR                                                  
      INTEGER   NOSR                                                    
      INTEGER   NOSRCLR                                                 
      INTEGER   NCLDMAS                                                 
      INTEGER   NUAVE                                                   
      INTEGER   NVAVE                                                   
      INTEGER   NTAVE                                                   
      INTEGER   NQAVE                                                   
      INTEGER   NMDIV                                                   
      INTEGER   NPAVE                                                   
      INTEGER   NQQAVE                                                  
      INTEGER   NSWGCLR                                                 
      INTEGER   NANALP                                                  
      INTEGER   NSDIAG1                                                  
      INTEGER   NSDIAG2                                                  
      INTEGER   NUDIAG1                                                  
      INTEGER   NUDIAG2                                                  
      INTEGER   NDIABU                                                   
      INTEGER   NDIABV                                                   
      INTEGER   NDIABT                                                   
      INTEGER   NDIABQ                                                   
      INTEGER   NPSIGDOT                                                 
      INTEGER   NPGU                                                     
      INTEGER   NPUFLX                                                   
      INTEGER   NPVFLX                                                   
      INTEGER   NPTFLX                                                   
      INTEGER   NPQFLX                                                   
      INTEGER   NDPUDT                                                   
      INTEGER   NDPVDT                                                   
      INTEGER   NDPTDT                                                   
      INTEGER   NDPQDT                                                   
      INTEGER   NVINTUQ                                                  
      INTEGER   NVINTVQ                                                  
      INTEGER   NVINTUT                                                  
      INTEGER   NVINTVT                                                  
      INTEGER   NCLDFRC                                                  
      INTEGER   NQINT                                                    
      INTEGER   NU2M                                                    
      INTEGER   NV2M                                                    
      INTEGER   NT2M                                                    
      INTEGER   NQ2M                                                    
      INTEGER   NU10M                                                    
      INTEGER   NV10M                                                    
      INTEGER   NT10M                                                    
      INTEGER   NQ10M                                                    
      INTEGER   NDTRAIN                                                  
      INTEGER   NQFILL                                                   
C                                                                       
      EQUIVALENCE ( NDIAG( 1) ,  NUFLUX  )                              
      EQUIVALENCE ( NDIAG( 2) ,  NVFLUX  )                              
      EQUIVALENCE ( NDIAG( 3) ,  NHFLUX  )                              
      EQUIVALENCE ( NDIAG( 4) ,  NEFLUX  )                              
      EQUIVALENCE ( NDIAG( 5) ,  NQICE   )                              
      EQUIVALENCE ( NDIAG( 6) ,  NRADLWG )                              
      EQUIVALENCE ( NDIAG( 7) ,  NRADSWG )                              
      EQUIVALENCE ( NDIAG( 8) ,  NRI     )                              
      EQUIVALENCE ( NDIAG( 9) ,  NCT     )                              
      EQUIVALENCE ( NDIAG(10) ,  NCU     )                              
      EQUIVALENCE ( NDIAG(11) ,  NET     )                              
      EQUIVALENCE ( NDIAG(12) ,  NEU     )                              
      EQUIVALENCE ( NDIAG(13) ,  NTURBU  )                              
      EQUIVALENCE ( NDIAG(14) ,  NTURBV  )                              
      EQUIVALENCE ( NDIAG(15) ,  NTURBT  )                              
      EQUIVALENCE ( NDIAG(16) ,  NTURBQ  )                              
      EQUIVALENCE ( NDIAG(17) ,  NMOISTT )                              
      EQUIVALENCE ( NDIAG(18) ,  NMOISTQ )                              
      EQUIVALENCE ( NDIAG(19) ,  NRADLW  )                              
      EQUIVALENCE ( NDIAG(20) ,  NRADSW  )                              
      EQUIVALENCE ( NDIAG(21) ,  NPREACC )                              
      EQUIVALENCE ( NDIAG(22) ,  NPRECON )                              
      EQUIVALENCE ( NDIAG(23) ,  NTUFLUX )                              
      EQUIVALENCE ( NDIAG(24) ,  NTVFLUX )                              
      EQUIVALENCE ( NDIAG(25) ,  NTTFLUX )                              
      EQUIVALENCE ( NDIAG(26) ,  NTQFLUX )                              
      EQUIVALENCE ( NDIAG(27) ,  NCN     )                              
      EQUIVALENCE ( NDIAG(28) ,  NWINDS  )                              
      EQUIVALENCE ( NDIAG(29) ,  NDTSRF  )                              
      EQUIVALENCE ( NDIAG(30) ,  NTG     )                              
      EQUIVALENCE ( NDIAG(31) ,  NTS     )                              
      EQUIVALENCE ( NDIAG(32) ,  NDTG    )                              
      EQUIVALENCE ( NDIAG(33) ,  NQG     )                              
      EQUIVALENCE ( NDIAG(34) ,  NQS     )                              
      EQUIVALENCE ( NDIAG(35) ,  NTGRLW  )                              
      EQUIVALENCE ( NDIAG(36) ,  NST4    )                              
      EQUIVALENCE ( NDIAG(37) ,  NOLR    )                              
      EQUIVALENCE ( NDIAG(38) ,  NOLRCLR )                              
      EQUIVALENCE ( NDIAG(39) ,  NLWGCLR )                              
      EQUIVALENCE ( NDIAG(40) ,  NLWCLR  )                              
      EQUIVALENCE ( NDIAG(41) ,  NTLW    )                              
      EQUIVALENCE ( NDIAG(42) ,  NSHLW   )                              
      EQUIVALENCE ( NDIAG(43) ,  NOZLW   )                              
      EQUIVALENCE ( NDIAG(44) ,  NCLMOLW )                              
      EQUIVALENCE ( NDIAG(45) ,  NCLROLW )                              
      EQUIVALENCE ( NDIAG(46) ,  NCLMOSW )                              
      EQUIVALENCE ( NDIAG(47) ,  NCLROSW )                              
      EQUIVALENCE ( NDIAG(48) ,  NRADSWT )                              
      EQUIVALENCE ( NDIAG(49) ,  NFREQMD )                              
      EQUIVALENCE ( NDIAG(50) ,  NFREQPN )                              
      EQUIVALENCE ( NDIAG(51) ,  NFREQLW )                              
      EQUIVALENCE ( NDIAG(52) ,  NEVAP   )                              
      EQUIVALENCE ( NDIAG(53) ,  NDPDT   )                              
      EQUIVALENCE ( NDIAG(54) ,  NANALU  )                              
      EQUIVALENCE ( NDIAG(55) ,  NANALV  )                              
      EQUIVALENCE ( NDIAG(56) ,  NANALT  )                              
      EQUIVALENCE ( NDIAG(57) ,  NANALQ  )                              
      EQUIVALENCE ( NDIAG(58) ,  NOMEGA  )                              
      EQUIVALENCE ( NDIAG(59) ,  NDUDT   )                              
      EQUIVALENCE ( NDIAG(60) ,  NDVDT   )                              
      EQUIVALENCE ( NDIAG(61) ,  NDTDT   )                              
      EQUIVALENCE ( NDIAG(62) ,  NDQDT   )                              
      EQUIVALENCE ( NDIAG(63) ,  NVORT   )                              
      EQUIVALENCE ( NDIAG(64) ,  NPGV    )                              
      EQUIVALENCE ( NDIAG(65) ,  NCORU   )                              
      EQUIVALENCE ( NDIAG(66) ,  NCORV   )                              
      EQUIVALENCE ( NDIAG(67) ,  NUSTAR  )                              
      EQUIVALENCE ( NDIAG(68) ,  NZ0     )                              
      EQUIVALENCE ( NDIAG(69) ,  NFRQTRB )                              
      EQUIVALENCE ( NDIAG(70) ,  NPBL    )                              
      EQUIVALENCE ( NDIAG(71) ,  NSWCLR  )                              
      EQUIVALENCE ( NDIAG(72) ,  NOSR    )                              
      EQUIVALENCE ( NDIAG(73) ,  NOSRCLR )                              
      EQUIVALENCE ( NDIAG(74) ,  NCLDMAS )                              
      EQUIVALENCE ( NDIAG(75) ,  NUAVE   )                              
      EQUIVALENCE ( NDIAG(76) ,  NVAVE   )                              
      EQUIVALENCE ( NDIAG(77) ,  NTAVE   )                              
      EQUIVALENCE ( NDIAG(78) ,  NQAVE   )                              
      EQUIVALENCE ( NDIAG(79) ,  NMDIV   )                              
      EQUIVALENCE ( NDIAG(80) ,  NPAVE   )                              
      EQUIVALENCE ( NDIAG(81) ,  NQQAVE  )                              
      EQUIVALENCE ( NDIAG(82) ,  NSWGCLR )                              
      EQUIVALENCE ( NDIAG(83) ,  NANALP  )                              
      EQUIVALENCE ( NDIAG(84) ,  NSDIAG1 )                              
      EQUIVALENCE ( NDIAG(85) ,  NSDIAG2 )                              
      EQUIVALENCE ( NDIAG(86) ,  NUDIAG1 )                              
      EQUIVALENCE ( NDIAG(87) ,  NUDIAG2 )                              
      EQUIVALENCE ( NDIAG(88) ,  NDIABU  )                              
      EQUIVALENCE ( NDIAG(89) ,  NDIABV  )                              
      EQUIVALENCE ( NDIAG(90) ,  NDIABT  )                              
      EQUIVALENCE ( NDIAG(91) ,  NDIABQ  )                              
      EQUIVALENCE ( NDIAG(92) ,  NPSIGDOT)                              
      EQUIVALENCE ( NDIAG(93) ,  NPGU    )                              
      EQUIVALENCE ( NDIAG(94) ,  NPUFLX  )                              
      EQUIVALENCE ( NDIAG(95) ,  NPVFLX  )                              
      EQUIVALENCE ( NDIAG(96) ,  NPTFLX  )                              
      EQUIVALENCE ( NDIAG(97) ,  NPQFLX  )                              
      EQUIVALENCE ( NDIAG(98) ,  NDPUDT  )                              
      EQUIVALENCE ( NDIAG(99) ,  NDPVDT  )                              
      EQUIVALENCE ( NDIAG(100),  NDPTDT  )                              
      EQUIVALENCE ( NDIAG(101),  NDPQDT  )                              
      EQUIVALENCE ( NDIAG(102),  NVINTUQ )                              
      EQUIVALENCE ( NDIAG(103),  NVINTVQ )                              
      EQUIVALENCE ( NDIAG(104),  NVINTUT )                              
      EQUIVALENCE ( NDIAG(105),  NVINTVT )                              
      EQUIVALENCE ( NDIAG(106),  NCLDFRC )                              
      EQUIVALENCE ( NDIAG(107),  NQINT   )                              
      EQUIVALENCE ( NDIAG(108),  NU2M    )                              
      EQUIVALENCE ( NDIAG(109),  NV2M    )                              
      EQUIVALENCE ( NDIAG(110),  NT2M    )                              
      EQUIVALENCE ( NDIAG(111),  NQ2M    )                              
      EQUIVALENCE ( NDIAG(112),  NU10M   )                              
      EQUIVALENCE ( NDIAG(113),  NV10M   )                              
      EQUIVALENCE ( NDIAG(114),  NT10M   )                              
      EQUIVALENCE ( NDIAG(115),  NQ10M   )                              
      EQUIVALENCE ( NDIAG(116),  NDTRAIN )                              
      EQUIVALENCE ( NDIAG(117),  NQFILL  )                              
C                                                                       
C  DIAGNOSTICS INPUT NAMELIST                                           
C  ==========================                                           
C                                                                       
      NAMELIST /INDIAG/ NUFLUX  , NVFLUX  , NHFLUX  , NEFLUX  ,         
     .                  NQICE   , NRADLWG , NRADSWG , NRI     ,         
     .                  NCT     , NCU     , NET     , NEU     ,         
     .                  NTURBU  , NTURBV  , NTURBT  , NTURBQ  ,         
     .                  NMOISTT , NMOISTQ , NRADLW  , NRADSW  ,         
     .                  NPREACC , NPRECON , NTUFLUX , NTVFLUX ,         
     .                  NTTFLUX , NTQFLUX , NCN     , NWINDS  ,         
     .                  NDTSRF  , NTG     , NTS     , NDTG    ,         
     .                  NQG     , NQS     , NTGRLW  , NST4    ,         
     .                  NOLR    , NOLRCLR , NLWGCLR , NLWCLR  ,         
     .                  NTLW    , NSHLW   , NOZLW   , NCLMOLW ,         
     .                  NCLROLW , NCLMOSW , NCLROSW , NRADSWT ,         
     .                  NFREQMD , NFREQPN , NFREQLW , NEVAP   ,         
     .                  NDPDT   , NANALU  , NANALV  , NANALT  ,         
     .                  NANALQ  , NOMEGA  , NDUDT   , NDVDT   ,         
     .                  NDTDT   , NDQDT   , NVORT   , NPGV    ,         
     .                  NCORU   , NCORV   , NUSTAR  , NZ0     ,         
     .                  NFRQTRB , NPBL    , NSWCLR  , NOSR    ,         
     .                  NOSRCLR , NCLDMAS , NUAVE   , NVAVE   ,         
     .                  NTAVE   , NQAVE   , NMDIV   , NPAVE   ,         
     .                  NQQAVE  , NSWGCLR , NANALP  , NSDIAG1 ,
     .                  NSDIAG2 , NUDIAG1 , NUDIAG2 , NDIABU  ,
     .                  NDIABV  , NDIABT  , NDIABQ  , NPSIGDOT,
     .                  NPGU    , NPUFLX  , NPVFLX  , NPTFLX  ,
     .                  NPQFLX  , NDPUDT  , NDPVDT  , NDPTDT  ,
     .                  NDPQDT  , NVINTUQ , NVINTVQ , NVINTUT ,
     .                  NVINTVT , NCLDFRC , NQINT   , NU2M    ,
     .                  NV2M    , NT2M    , NQ2M    , NU10M   ,
     .                  NV10M   , NT10M   , NQ10M   , NDTRAIN ,
     .                  NQFILL
                                                                        
C **********************************************************************
C ****               INITIALIZE DIAGNOSTIC LOGICALS                 ****
C **********************************************************************
                                                                        
      PRINT *                                                           
      PRINT *, 'Total Number of Diagnostics Allowed (NDIAGMX)'
      PRINT *, 'Defined by Users Application:  ',NDIAGMX
      PRINT *                                                           
                                                                        
      DO N = 1,NDIAGT                                                
      NDIAG(N) = 0
      ENDDO                                                             
                                                                        
C **********************************************************************
C ****                  READ DIAGNOSTIC NAMELIST                    ****
C **********************************************************************
                                                                        
      REWIND 11                                                         
      WRITE (6 ,1000)                                                   
      READ  (11,INDIAG)                                                 
      WRITE (6 ,INDIAG)                                                 
                                                                        
C **********************************************************************
C **** CALCULATE POINTERS FOR DIAGNOSTICS SET TO NON-ZERO FREQUENCY ****
C **********************************************************************
                                                                        
      DO   N = 1,NDIAGT                                                
           NFREQ(N) =  NDIAG(N)
      IF(  NDIAG(N).NE.0 ) THEN                                           
      CALL SETDIAG (N)                                                  
      CALL CLRDIAG (N)                                                  
      ENDIF                                                             
      ENDDO                                                             
                                                                        
 1000 FORMAT(1X,'READING DIAGNOSTIC NAMELIST:')                         
      RETURN                                                            
      END                                                               
      SUBROUTINE SETDIAG (NUM)                                                  
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     SET POINTER LOCATIONS, NAMES, LEVELS and TITLES FOR DIAGNOSTIC NUM
C  ENTRY POINTS:                                                                
C  ============                                                                 
C   ZAPDIAG (NUM):  ZAP POINTER LOCATION FOR DIAGNOSTIC NUM                     
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
                                                                                
      PARAMETER ( IMJMR   = IMR*(JMR+1) )                                       
      PARAMETER ( NLAYM1R = NLAYR-1     )                                       
                                                                                
      DATA IPOINTER / 1 /                                                       
      DATA IDIAG    / NDIAGT*0 /                                                
                                                                                
C **********************************************************************        
C ****               SET DIAGNOSTIC LEVELS AND NAMES                ****        
C **********************************************************************        
                                                                                
      DATA KDIAG( 1) /  1    /, CDIAG( 1) /'UFLUX   '/                          
      DATA KDIAG( 2) /  1    /, CDIAG( 2) /'VFLUX   '/                          
      DATA KDIAG( 3) /  1    /, CDIAG( 3) /'HFLUX   '/                          
      DATA KDIAG( 4) /  1    /, CDIAG( 4) /'EFLUX   '/                          
      DATA KDIAG( 5) /  1    /, CDIAG( 5) /'QICE    '/                          
      DATA KDIAG( 6) /  1    /, CDIAG( 6) /'RADLWG  '/                          
      DATA KDIAG( 7) /  1    /, CDIAG( 7) /'RADSWG  '/                          
      DATA KDIAG( 8) /NLAYR  /, CDIAG( 8) /'RI      '/                          
      DATA KDIAG( 9) /  1    /, CDIAG( 9) /'CT      '/                          
      DATA KDIAG(10) /  1    /, CDIAG(10) /'CU      '/                          
      DATA KDIAG(11) /NLAYR  /, CDIAG(11) /'ET      '/                          
      DATA KDIAG(12) /NLAYR  /, CDIAG(12) /'EU      '/                          
      DATA KDIAG(13) /NLAYR  /, CDIAG(13) /'TURBU   '/                          
      DATA KDIAG(14) /NLAYR  /, CDIAG(14) /'TURBV   '/                          
      DATA KDIAG(15) /NLAYR  /, CDIAG(15) /'TURBT   '/                          
      DATA KDIAG(16) /NLAYR  /, CDIAG(16) /'TURBQ   '/                          
      DATA KDIAG(17) /NLAYR  /, CDIAG(17) /'MOISTT  '/                          
      DATA KDIAG(18) /NLAYR  /, CDIAG(18) /'MOISTQ  '/                          
      DATA KDIAG(19) /NLAYR  /, CDIAG(19) /'RADLW   '/                          
      DATA KDIAG(20) /NLAYR  /, CDIAG(20) /'RADSW   '/                          
      DATA KDIAG(21) / 1     /, CDIAG(21) /'PREACC  '/                          
      DATA KDIAG(22) / 1     /, CDIAG(22) /'PRECON  '/                          
      DATA KDIAG(23) /NLAYR  /, CDIAG(23) /'TUFLUX  '/                          
      DATA KDIAG(24) /NLAYR  /, CDIAG(24) /'TVFLUX  '/                          
      DATA KDIAG(25) /NLAYR  /, CDIAG(25) /'TTFLUX  '/                          
      DATA KDIAG(26) /NLAYR  /, CDIAG(26) /'TQFLUX  '/                          
      DATA KDIAG(27) / 1     /, CDIAG(27) /'CN      '/                          
      DATA KDIAG(28) / 1     /, CDIAG(28) /'WINDS   '/                          
      DATA KDIAG(29) / 1     /, CDIAG(29) /'DTSRF   '/                          
      DATA KDIAG(30) / 1     /, CDIAG(30) /'TG      '/                          
      DATA KDIAG(31) / 1     /, CDIAG(31) /'TS      '/                          
      DATA KDIAG(32) / 1     /, CDIAG(32) /'DTG     '/                          
      DATA KDIAG(33) / 1     /, CDIAG(33) /'QG      '/                          
      DATA KDIAG(34) / 1     /, CDIAG(34) /'QS      '/                          
      DATA KDIAG(35) / 1     /, CDIAG(35) /'TGRLW   '/                          
      DATA KDIAG(36) / 1     /, CDIAG(36) /'ST4     '/                          
      DATA KDIAG(37) / 1     /, CDIAG(37) /'OLR     '/                          
      DATA KDIAG(38) / 1     /, CDIAG(38) /'OLRCLR  '/                          
      DATA KDIAG(39) / 1     /, CDIAG(39) /'LWGCLR  '/                          
      DATA KDIAG(40) / NLAYR /, CDIAG(40) /'LWCLR   '/                          
      DATA KDIAG(41) / NLAYR /, CDIAG(41) /'TLW     '/                          
      DATA KDIAG(42) / NLAYR /, CDIAG(42) /'SHLW    '/                          
      DATA KDIAG(43) / NLAYR /, CDIAG(43) /'OZLW    '/                          
      DATA KDIAG(44) / NLAYR /, CDIAG(44) /'CLMOLW  '/                          
      DATA KDIAG(45) / NLAYR /, CDIAG(45) /'CLROLW  '/                          
      DATA KDIAG(46) / NLAYR /, CDIAG(46) /'CLMOSW  '/                          
      DATA KDIAG(47) / NLAYR /, CDIAG(47) /'CLROSW  '/                          
      DATA KDIAG(48) / 1     /, CDIAG(48) /'RADSWT  '/                          
      DATA KDIAG(49) / 1     /, CDIAG(49) /'FREQMD  '/                          
      DATA KDIAG(50) / NLAYR /, CDIAG(50) /'FREQPN  '/                          
      DATA KDIAG(51) / 1     /, CDIAG(51) /'FREQLW  '/                          
      DATA KDIAG(52) / 1     /, CDIAG(52) /'EVAP    '/                          
      DATA KDIAG(53) / 1     /, CDIAG(53) /'DPDT    '/                          
      DATA KDIAG(54) / NLAYR /, CDIAG(54) /'ANALU   '/                          
      DATA KDIAG(55) / NLAYR /, CDIAG(55) /'ANALV   '/                          
      DATA KDIAG(56) / NLAYR /, CDIAG(56) /'ANALT   '/                          
      DATA KDIAG(57) / NLAYR /, CDIAG(57) /'ANALQ   '/                          
      DATA KDIAG(58) / NLAYR /, CDIAG(58) /'OMEGA   '/                          
      DATA KDIAG(59) / NLAYR /, CDIAG(59) /'DUDT    '/                          
      DATA KDIAG(60) / NLAYR /, CDIAG(60) /'DVDT    '/                          
      DATA KDIAG(61) / NLAYR /, CDIAG(61) /'DTDT    '/                          
      DATA KDIAG(62) / NLAYR /, CDIAG(62) /'DQDT    '/                          
      DATA KDIAG(63) / NLAYR /, CDIAG(63) /'VORT    '/                          
      DATA KDIAG(64) / NLAYR /, CDIAG(64) /'PGV     '/                          
      DATA KDIAG(65) / NLAYR /, CDIAG(65) /'CORU    '/                          
      DATA KDIAG(66) / NLAYR /, CDIAG(66) /'CORV    '/                          
      DATA KDIAG(67) / 1     /, CDIAG(67) /'USTAR   '/                          
      DATA KDIAG(68) / 1     /, CDIAG(68) /'Z0      '/                          
      DATA KDIAG(69) /NLAYM1R/, CDIAG(69) /'FRQTRB  '/                          
      DATA KDIAG(70) / 1     /, CDIAG(70) /'PBL     '/                          
      DATA KDIAG(71) / NLAYR /, CDIAG(71) /'SWCLR   '/                          
      DATA KDIAG(72) / 1     /, CDIAG(72) /'OSR     '/                          
      DATA KDIAG(73) / 1     /, CDIAG(73) /'OSRCLR  '/                          
      DATA KDIAG(74) / NLAYR /, CDIAG(74) /'CLDMAS  '/                          
      DATA KDIAG(75) / NLAYR /, CDIAG(75) /'UAVE    '/                          
      DATA KDIAG(76) / NLAYR /, CDIAG(76) /'VAVE    '/                          
      DATA KDIAG(77) / NLAYR /, CDIAG(77) /'TAVE    '/                          
      DATA KDIAG(78) / NLAYR /, CDIAG(78) /'QAVE    '/                          
      DATA KDIAG(79) / NLAYR /, CDIAG(79) /'MDIV    '/                          
      DATA KDIAG(80) / 1     /, CDIAG(80) /'PAVE    '/                          
      DATA KDIAG(81) / NLAYR /, CDIAG(81) /'QQAVE   '/                          
      DATA KDIAG(82) / 1     /, CDIAG(82) /'SWGCLR  '/                          
      DATA KDIAG(83) / 1     /, CDIAG(83) /'ANALP   '/                          
      DATA KDIAG(84) / 1     /, CDIAG(84) /'SDIAG1  '/                          
      DATA KDIAG(85) / 1     /, CDIAG(85) /'SDIAG2  '/                          
      DATA KDIAG(86) / NLAYR /, CDIAG(86) /'UDIAG1  '/                          
      DATA KDIAG(87) / NLAYR /, CDIAG(87) /'UDIAG2  '/                          
      DATA KDIAG(88) / NLAYR /, CDIAG(88) /'DIABU   '/                          
      DATA KDIAG(89) / NLAYR /, CDIAG(89) /'DIABV   '/                          
      DATA KDIAG(90) / NLAYR /, CDIAG(90) /'DIABT   '/                          
      DATA KDIAG(91) / NLAYR /, CDIAG(91) /'DIABQ   '/                          
      DATA KDIAG(92) / NLAYR /, CDIAG(92) /'PSIGDOT '/                          
      DATA KDIAG(93) / NLAYR /, CDIAG(93) /'PGU     '/                          
      DATA KDIAG(94) / NLAYR /, CDIAG(94) /'PUFLX   '/                          
      DATA KDIAG(95) / NLAYR /, CDIAG(95) /'PVFLX   '/                          
      DATA KDIAG(96) / NLAYR /, CDIAG(96) /'PTFLX   '/                          
      DATA KDIAG(97) / NLAYR /, CDIAG(97) /'PQFLX   '/                          
      DATA KDIAG(98) / NLAYR /, CDIAG(98) /'DPUDT   '/                          
      DATA KDIAG(99) / NLAYR /, CDIAG(99) /'DPVDT   '/                          
      DATA KDIAG(100)/ NLAYR /, CDIAG(100)/'DPTDT   '/                          
      DATA KDIAG(101)/ NLAYR /, CDIAG(101)/'DPQDT   '/                          
      DATA KDIAG(102)/ 1     /, CDIAG(102)/'VINTUQ  '/                          
      DATA KDIAG(103)/ 1     /, CDIAG(103)/'VINTVQ  '/                          
      DATA KDIAG(104)/ 1     /, CDIAG(104)/'VINTUT  '/                          
      DATA KDIAG(105)/ 1     /, CDIAG(105)/'VINTVT  '/                          
      DATA KDIAG(106)/ 1     /, CDIAG(106)/'CLDFRC  '/                          
      DATA KDIAG(107)/ 1     /, CDIAG(107)/'QINT    '/                          
      DATA KDIAG(108)/ 1     /, CDIAG(108)/'U2M     '/
      DATA KDIAG(109)/ 1     /, CDIAG(109)/'V2M     '/
      DATA KDIAG(110)/ 1     /, CDIAG(110)/'T2M     '/
      DATA KDIAG(111)/ 1     /, CDIAG(111)/'Q2M     '/
      DATA KDIAG(112)/ 1     /, CDIAG(112)/'U10M    '/
      DATA KDIAG(113)/ 1     /, CDIAG(113)/'V10M    '/
      DATA KDIAG(114)/ 1     /, CDIAG(114)/'T10M    '/
      DATA KDIAG(115)/ 1     /, CDIAG(115)/'Q10M    '/
      DATA KDIAG(116)/ NLAYR /, CDIAG(116)/'DTRAIN  '/
      DATA KDIAG(117)/ NLAYR /, CDIAG(117)/'QFILL   '/
                                                                                
      DATA TDIAG( 1)  /'U-MOMENTUM SURFACE STRESS     (N/M**2)  '/
      DATA TDIAG( 2)  /'V-MOMENTUM SURFACE STRESS     (N/M**2)  '/
      DATA TDIAG( 3)  /'SRFCE FLX OF SENSIBLE    HEAT (W/M**2)  '/
      DATA TDIAG( 4)  /'SRFCE FLX OF EVAPORATIVE HEAT (W/M**2)  '/
      DATA TDIAG( 5)  /'HEAT CONDUCTION THRU SEA-ICE  (W/M**2)  '/
      DATA TDIAG( 6)  /'NET UPWARD   LW RAD. AT GRND  (W/M**2)  '/
      DATA TDIAG( 7)  /'NET DOWNWARD SW RAD. AT GRND  (W/M**2)  '/
      DATA TDIAG( 8)  /'RICHARDSON NUMBER                       '/
      DATA TDIAG( 9)  /'SURFACE DRAG COEF. FOR T AND Q (M/S)    '/
      DATA TDIAG(10)  /'SURFACE DRAG COEF. FOR U AND V (M/S)    '/
      DATA TDIAG(11)  /'DIFFUSIVITY  COEF. FOR T AND Q (M**2/S) '/
      DATA TDIAG(12)  /'DIFFUSIVITY  COEF. FOR U AND V (M**2/S) '/
      DATA TDIAG(13)  /'U-MOMENTUM  CHNGS DUE TO TURB (M/S/DAY) '/
      DATA TDIAG(14)  /'V-MOMENTUM  CHNGS DUE TO TURB (M/S/DAY) '/
      DATA TDIAG(15)  /'TEMPERATURE CHNGS DUE TO TURB (DEG/DAY) '/
      DATA TDIAG(16)  /'MOISTURE    CHNGS DUE TO TURB (G/KG/DAY)'/
      DATA TDIAG(17)  /'TEMPERATURE CHNGS DUE TO MOIST (DEG/DAY)'/
      DATA TDIAG(18)  /'MOISTURE CHANGES DUE TO MOIST (G/KG/DAY)'/
      DATA TDIAG(19)  /'TEMPERATURE CHNGS DUE TO LW RAD  DEG/DAY'/
      DATA TDIAG(20)  /'TEMPERATURE CHNGS DUE TO SW RAD  DEG/DAY'/
      DATA TDIAG(21)  /'TOTAL PRECIPITATION       (MM/DAY)      '/
      DATA TDIAG(22)  /'CONVECTIVE PRECIPITATION  (MM/DAY)      '/
      DATA TDIAG(23)  /'TURBULENT FLUX OF U-MOMENTUM (N/M**2)   '/
      DATA TDIAG(24)  /'TURBULENT FLUX OF V-MOMENTUM (N/M**2)   '/
      DATA TDIAG(25)  /'TURBULENT FLUX OF SENS.  HEAT (W/M**2)  '/
      DATA TDIAG(26)  /'TURBULENT FLUX OF LATENT HEAT (W/M**2)  '/
      DATA TDIAG(27)  /'NEUTRAL DRAG COEFFICIENT   (M/S)        '/
      DATA TDIAG(28)  /'SURFACE WIND SPEED         (M/S)        '/
      DATA TDIAG(29)  /'AIR/SURFACE VIRTUAL TEMP. DIFF.  (DEG)  '/
      DATA TDIAG(30)  /'GROUND TEMPERATURE            (DEG)     '/
      DATA TDIAG(31)  /'SURFACE AIR TEMPERATURE       (DEG)     '/
      DATA TDIAG(32)  /'GROUND TEMPERATURE ADJUSTMENT (DEG)     '/
      DATA TDIAG(33)  /'GROUND SPECIFIC HUMIDITY      (G/KG)    '/
      DATA TDIAG(34)  /'SATURATION SURFACE SPEC. HUM. (G/KG)    '/
      DATA TDIAG(35)  /'GROUND TEMP SEEN BY LONGWAVE RAD  (DEG) '/
      DATA TDIAG(36)  /'SIGMA T**4                    (W/M**2)  '/
      DATA TDIAG(37)  /'OUTGOING LONGWAVE RADIATION   (W/M**2)  '/
      DATA TDIAG(38)  /'OUTGOING LONGWAVE RAD CLEAR SKY (W/M**2)'/
      DATA TDIAG(39)  /'SURFACE LONGWAVE FLUX CLEAR SKY (W/M**2)'/
      DATA TDIAG(40)  /'CLEAR SKY LW HEATING RATES   (DEG/DAY)  '/
      DATA TDIAG(41)  /'TEMPERATURE SEEN BY LONGWAVE RAD   (DEG)'/
      DATA TDIAG(42)  /'SPEC. HUM.  SEEN BY LONGWAVE RAD (G/G)  '/
      DATA TDIAG(43)  /'OZONE SEEN BY LONGWAVE RAD              '/
      DATA TDIAG(44)  /'MAXIMUM OVERLAP CLOUD FRACTION FOR LW   '/
      DATA TDIAG(45)  /'RANDOM  OVERLAP CLOUD FRACTION FOR LW   '/
      DATA TDIAG(46)  /'MAXIMUM OVERLAP CLOUD FRACTION FOR SW   '/
      DATA TDIAG(47)  /'RANDOM  OVERLAP CLOUD FRACTION FOR SW   '/
      DATA TDIAG(48)  /'INCIDENT SW RAD. AT TOP OF ATM. (W/M**2)'/
      DATA TDIAG(49)  /'FREQUENCY OF MID-LEVEL CONVECTION       '/
      DATA TDIAG(50)  /'FREQUENCY OF PEN-LEVEL CONVECTION       '/
      DATA TDIAG(51)  /'FREQUENCY OF LOW-LEVEL CONVECTION       '/
      DATA TDIAG(52)  /'SURFACE EVAPORATION  (MM/DAY)           '/
      DATA TDIAG(53)  /'SURFACE PRESSURE TENDENCY  (MB/DAY)     '/
      DATA TDIAG(54)  /'ANALYSIS INCREMENT OF U-WIND  (M/S/SEC) '/
      DATA TDIAG(55)  /'ANALYSIS INCREMENT OF V-WIND  (M/S/SEC) '/
      DATA TDIAG(56)  /'ANALYSIS INCREMENT OF THETA (PI*TH/SEC) '/
      DATA TDIAG(57)  /'ANALYSIS INCREMENT OF SPHU  (PI*SH/SEC) '/
      DATA TDIAG(58)  /'VERTICAL VELOCITY OMEGA   (MB/DAY)      '/
      DATA TDIAG(59)  /'TOTAL PU TENDENCY   (M/S/DAY)           '/
      DATA TDIAG(60)  /'TOTAL PV TENDENCY   (M/S/DAY)           '/
      DATA TDIAG(61)  /'TOTAL PT TENDENCY   (DEG/DAY)           '/
      DATA TDIAG(62)  /'TOTAL PQ TENDENCY  (G/KG/DAY)           '/
      DATA TDIAG(63)  /'RELATIVE VORTICITY * 10**-4   (SEC**-1) '/
      DATA TDIAG(64)  /'V PRESSURE GRADIENT TENDENCY  (M/S/DAY) '/
      DATA TDIAG(65)  /'U CORIOLIS TENDENCY           (M/S/DAY) '/
      DATA TDIAG(66)  /'V CORIOLIS TENDENCY           (M/S/DAY) '/
      DATA TDIAG(67)  /'USTAR    (M/SEC)                        '/
      DATA TDIAG(68)  /'SURFACE ROUGHNESS Z0   (M)              '/
      DATA TDIAG(69)  /'FREQUENCY OF TURBULENCE   (0-1)         '/
      DATA TDIAG(70)  /'PBL DEPTH (MB)                          '/
      DATA TDIAG(71)  /'CLEAR SKY SW HEATING RATES   (DEG/DAY)  '/
      DATA TDIAG(72)  /'OUTGOING SHORTWAVE RADIATION  (W/M**2)  '/
      DATA TDIAG(73)  /'OUTGOING SHORTWAVE RAD CLEAR  (W/M**2)  '/
      DATA TDIAG(74)  /'CLOUD MASS FLUX  (KG M/SEC)             '/
      DATA TDIAG(75)  /'TIME-AVERAGED UWND    (M/SEC)           '/
      DATA TDIAG(76)  /'TIME-AVERAGED VWND    (M/SEC)           '/
      DATA TDIAG(77)  /'TIME-AVERAGED TMPU    (DEG)             '/
      DATA TDIAG(78)  /'TIME-AVERAGED SPHU    (G/G)             '/
      DATA TDIAG(79)  /'MASS FLUX DIVERGENCE  (MB/DAY)          '/
      DATA TDIAG(80)  /'TIME-AVERAGED PS-PTOP (MB)              '/
      DATA TDIAG(81)  /'TIME-AVERAGED QQ      (M/SEC)**2        '/
      DATA TDIAG(82)  /'NET DWNWRD SWRAD AT GRND CLRSKY (W/M**2)'/
      DATA TDIAG(82)  /'SURF. SHORTWAVE FLUX CLEAR SKY (W/M**2) '/
      DATA TDIAG(83)  /'ANALYSIS INCREMENT OF PSURF    (MB/SEC) '/
      DATA TDIAG(84)  /'USER-DEFINED SURFACE DIAGNOSTIC-1       '/
      DATA TDIAG(85)  /'USER-DEFINED SURFACE DIAGNOSTIC-2       '/
      DATA TDIAG(86)  /'USER-DEFINED UPPER-AIR DIAGNOSTIC-1     '/
      DATA TDIAG(87)  /'USER-DEFINED UPPER-AIR DIAGNOSTIC-2     '/
      DATA TDIAG(88)  /'TOT DIABATIC FORCING ON U-WIND (M/S/DAY)'/
      DATA TDIAG(89)  /'TOT DIABATIC FORCING ON V-WIND (M/S/DAY)'/
      DATA TDIAG(90)  /'TOT DIABATIC FORCING ON TMPU   (DEG/DAY)'/
      DATA TDIAG(91)  /'TOT DIABATIC FORCING ON SPHU  (G/KG/DAY)'/
      DATA TDIAG(92)  /'VRT SIGMA VELCTY PI*D(SIGMA)/DT (MB/DAY)'/
      DATA TDIAG(93)  /'M/S/DAY  U PRESSURE GRAD TEND   (A-GRID)'/
      DATA TDIAG(94)  /'M/S/DAY  3-D FLUX CONVERG OF PU (A-GRID)'/
      DATA TDIAG(95)  /'M/S/DAY  3-D FLUX CONVERG OF PV (A-GRID)'/
      DATA TDIAG(96)  /'DEG/DAY  3-D FLUX CONVERG OF PT (A-GRID)'/
      DATA TDIAG(97)  /'G/KG/DAY 3-D FLUX CONVERG OF PQ (A-GRID)'/
      DATA TDIAG(98)  /'M/S/DAY  TOTAL PU TENDENCY      (A-GRID)'/
      DATA TDIAG(99)  /'M/S/DAY  TOTAL PV TENDENCY      (A-GRID)'/
      DATA TDIAG(100) /'DEG/DAY  TOTAL PT TENDENCY      (A-GRID)'/
      DATA TDIAG(101) /'G/KG/DAY  TOTAL PQ TENDENCY     (A-GRID)'/
      DATA TDIAG(102) /'VERTICALLY INTEGRATED U*Q   (M/SEC G/KG)'/
      DATA TDIAG(103) /'VERTICALLY INTEGRATED V*Q   (M/SEC G/KG)'/
      DATA TDIAG(104) /'VERTICALLY INTEGRATED U*T   (M/SEC  DEG)'/
      DATA TDIAG(105) /'VERTICALLY INTEGRATED V*T   (M/SEC  DEG)'/
      DATA TDIAG(106) /'2-DIMENSIONAL TOTAL CLOUD FRACTION (0-1)'/
      DATA TDIAG(107) /'PRECIPITABLE WATER   (GM/CM**2)         '/
      DATA TDIAG(108) /'U AT  2 METERS (M/SEC)                  '/
      DATA TDIAG(109) /'V AT  2 METERS (M/SEC)                  '/
      DATA TDIAG(110) /'T AT  2 METERS (DEG)                    '/
      DATA TDIAG(111) /'Q AT  2 METERS (KG/KG)                  '/
      DATA TDIAG(112) /'U AT 10 METERS (M/SEC)                  '/
      DATA TDIAG(113) /'V AT 10 METERS (M/SEC)                  '/
      DATA TDIAG(114) /'T AT 10 METERS (DEG)                    '/
      DATA TDIAG(115) /'Q AT 10 METERS (KG/KG)                  '/
      DATA TDIAG(116) /'CONVECTIVE CLOUD DETRAINMENT  (KG M/S)  '/
      DATA TDIAG(117) /'FILLING OF NEGATIVE SPHU      (G/KG/DAY)'/

C **********************************************************************        
C ****                SET POINTERS FOR DIAGNOSTIC NUM               ****        
C **********************************************************************        
                                                                                
      IF( IDIAG(NUM).EQ.0 ) THEN                                                
      ISPAN = IPOINTER + KDIAG(NUM) - 1                                         
        IF (ISPAN.GT.NDIAGMX) THEN                                              
            WRITE(6,1000) KDIAG(NUM), NUM, CDIAG(NUM)                           
        ELSE                                                                    
            IDIAG(NUM) = IPOINTER                                               
            IPOINTER   = IPOINTER + KDIAG(NUM)                                  
            ISPACE     = NDIAGMX  - IPOINTER + 1                                
            WRITE(6,2000) KDIAG(NUM), NUM, CDIAG(NUM), ISPACE                   
        ENDIF                                                                   
      ELSE                                                                      
            WRITE(6,3000) NUM, CDIAG(NUM)                                       
      ENDIF                                                                     
      RETURN                                                                    
                                                                                
C **********************************************************************        
C ****                ZAP POINTERS FOR DIAGNOSTIC NUM               ****        
C **********************************************************************        
                                                                                
      ENTRY ZAPDIAG (NUM)                                                       
                                                                                
      IF( IDIAG(NUM).NE.0 ) THEN                                                
          DO 10 L=IDIAG(NUM)+KDIAG(NUM),IPOINTER-1                              
          DO 10 I=1,IMJMR                                                       
          QDIAG(I,1,L-KDIAG(NUM)) = QDIAG(I,1,L)                                
   10     CONTINUE                                                              
                                                                                
          DO 20 N=1,NDIAGT                                                      
          IF ( IDIAG(N).GT.IDIAG(NUM) ) IDIAG(N) = IDIAG(N) - KDIAG(NUM)        
   20     CONTINUE                                                              
                                                                                
          IDIAG(NUM) = 0                                                        
          NDIAG(NUM) = 0                                                        
          IPOINTER   = IPOINTER - KDIAG(NUM)                                    
      ENDIF                                                                     
      WRITE(6,4000) NUM, CDIAG(NUM)                                             
      RETURN                                                                    
                                                                                
C **********************************************************************        
                                                                                
 1000 FORMAT(1X,'NO SPACE LEFT FOR ',I2,' LEVEL(S) OF DIAGNOSTIC # ',           
     .       I3,' (',A8,')')                                                    
 2000 FORMAT(1X,'ALLOCATING ',I2,' LEVEL(S) FOR DIAGNOSTIC # ',I3,              
     .          ' (',A8,'),  ',I3,' SLOTS REMAIN')                              
 3000 FORMAT(1X,'DIAGNOSTIC # ',I3,' (',A8,') HAS ALREADY BEEN SET')            
 4000 FORMAT(1X,'DIAGNOSTIC # ',I3,' (',A8,') HAS BEEN ZAPPED')                 
      END                                                                       
      SUBROUTINE GETDIAG (LEV, IPOINT, QTMP, UNDEF)                             
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     RETRIEVE MODEL DIAGNOSTICS FROM COMMON                                    
C  INPUT:                                                                       
C     LEV ..... SIGMA LEVEL                                                     
C  IPOINT ..... DIAGNOSTIC NUMBER FROM MENU                                     
C   UNDEF ..... UNDEFINED VALUE                                                 
C                                                                               
C  OUTPUT:                                                                      
C    QTMP ..... DIAGNOSTIC QUANTITY                                             
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
C                                                                               
      PARAMETER ( ZERO  = 0.0 )                                                 
      PARAMETER ( ONE   = 1.0 )                                                 
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
      DIMENSION   QTMP(IMJMR)                                                   
                                                                                
      DO 10 I=1,IMJMR                                                           
      QTMP(I) = UNDEF                                                           
   10 CONTINUE                                                                  
      IF (IPOINT.LT.1) GO TO 999                                                
                                                                                
      KLEV = KDIAG(IPOINT)                                                      
      IF(KLEV.GE.LEV) THEN                                                      
      IPNT = IDIAG(IPOINT) + LEV - 1                                            
                                FACTOR = ONE                                    
      IF( NDIAG(IPOINT).NE.0 )  FACTOR = ONE   / NDIAG(IPOINT)                  
      DO 20 I=1,IMJMR                                                           
      QTMP(I) = QDIAG(I,1,IPNT) * FACTOR                                        
   20 CONTINUE                                                                  
      ENDIF                                                                     
                                                                                
 999  RETURN                                                                    
      END                                                                       
      SUBROUTINE CLRDIAG (N)                                                    
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     INITIALIZE MODEL DIAGNOSTIC QUANTITIES                                    
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
C                                                                               
      PARAMETER ( ZERO  = 0.0 )                                                 
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
C                                                                               
C **********************************************************************        
C ****              SET DIAGNOSTIC AND COUNTER TO ZERO              ****        
C **********************************************************************        
C                                                                               
      IF( IDIAG(N).NE.0 ) THEN                                                  
         DO 10 K=1,KDIAG(N)                                                     
         DO 10 I=1,IMJMR                                                        
         QDIAG(I,1,IDIAG(N)+K-1) = ZERO                                         
   10    CONTINUE                                                               
         NDIAG(N) = 0                                                           
      ENDIF                                                                     
C                                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE CLRANAL                                                    
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     INITIALIZE ANALYSIS TENDENCIES TO ZERO                                    
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <iau.com>                                                           
                                                                                
      PARAMETER ( ZERO  = 0.0 )                                                 
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
                                                                                
      DO I=1,IMJMR                                                        
      DPANAL(I,1) = ZERO                                         
      ENDDO
      DO L=1,NLAYR
      DO I=1,IMJMR                                                        
      DUANAL(I,1,L) = ZERO                                         
      DVANAL(I,1,L) = ZERO                                         
      DTANAL(I,1,L) = ZERO                                         
      DQANAL(I,1,L) = ZERO                                         
      ENDDO
      ENDDO
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE PRTBCS (KBC,NYMD,NHMS)                                         
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     PRINTS OUT SURFACE CONDITION FROM BOUNDARY DATA SETS                      
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
C                                                                               
      PARAMETER ( IM    = IMR    )                                              
      PARAMETER ( JNP   = JMR+1  )                                              
                                                                                
      PARAMETER ( ZERO  = 0.0    )                                              
      PARAMETER ( ONE   = 1.0    )                                              
      PARAMETER ( TWO   = 2.0    )                                              
      PARAMETER ( THREE = 3.0    )                                              
      PARAMETER ( FOUR  = 4.0    )                                              
      PARAMETER ( FIVE  = 5.0    )                                              
      PARAMETER (  ZP4  = 0.4    )                                              
      PARAMETER ( TICE  = 273.16 )                                              
      PARAMETER ( GRAV  = 9.81   )                                              
      PARAMETER ( AGRAV = ONE/GRAV )                                            
C                                                                               
      COMMON /SPACE/  TOPOG(IMR,JMR+1)                                          
      COMMON /SPACE/  FLAGS(IMR,JMR+1)                                          
C                                                                               
      CHARACTER*1 FLAGS, WFLAG, IFLAG, SFLAG, FFLAG, GFLAG, LFLAG               
      DATA        WFLAG /'W'/                                                   
      DATA        IFLAG /'I'/                                                   
      DATA        SFLAG /'S'/                                                   
      DATA        FFLAG /'F'/                                                   
      DATA        GFLAG /'G'/                                                   
      DATA        LFLAG /' '/                                                   
C                                                                               
      IMJM = IM*JNP                                                             
C                                                                               
C **********************************************************************        
C ****             INITIALIZE BOUNDARY CONDITIONS                   ****        
C **********************************************************************        
C                                                                               
      CALL GETBCS (KBC, NYMD, NHMS, TGZ, PHISZ, TVAZ, RADSWT,                   
     .             SSTZ, ALBZ, GWZ, THKZ, SRZ, ZLWI, IMJM)                      
C                                                                               
C **********************************************************************        
C ****              SET SURFACE FLAGS AND TOPOGRAPHY                ****        
C **********************************************************************        
C                                                                               
      DO 10 J = 1,JNP                                                           
      DO 10 I = 1,IM                                                            
          TOPOG(I,J) =         PHISZ(I,J)*AGRAV                                 
      IF ( ZLWI(I,J).EQ.ONE   )                      FLAGS(I,J) = WFLAG         
      IF ( ZLWI(I,J).EQ.TWO   )                      FLAGS(I,J) = LFLAG         
      IF ( ZLWI(I,J).EQ.THREE )                      FLAGS(I,J) = GFLAG         
      IF ( THKZ(I,J).GT.ZERO  )                      FLAGS(I,J) = IFLAG         
      IF ( ZLWI(I,J).EQ.TWO ) THEN                                              
      IF ( ALBZ(I,J).GT.ZP4 )                        FLAGS(I,J) = SFLAG         
      IF ( ALBZ(I,J).LE.ZP4 .AND. TGZ(I,J).LE.TICE ) FLAGS(I,J) = FFLAG         
      ENDIF                                                                     
   10 CONTINUE                                                                  
C                                                                               
C **********************************************************************        
C ****                      PRINT OUTPUT                            ****        
C **********************************************************************        
C                                                                               
      WRITE(6,6052)                                                             
 6052 FORMAT(//)                                                                
C                                                                               
      WRITE(6,1001)                                                             
      NPASS = 1 + (IM-1)/130                                                    
      DO 100 N = 1,NPASS                                                        
      IBEG  = 1+(N-1)*130                                                       
      ICHK  = IM - N *130                                                       
      IF(ICHK.GT.0) THEN                                                        
      IEND  =      N *130                                                       
      ELSE                                                                      
      IEND  = IM                                                                
      ENDIF                                                                     
      DO 110 JJ = 1,JNP                                                         
      J = JNP-JJ+1                                                              
      WRITE(6,1002) (FLAGS(I,J),I=IBEG,IEND)                                    
  110 CONTINUE                                                                  
      PRINT *                                                                   
  100 CONTINUE                                                                  
 1001 FORMAT(//1X,'LAND-WATER-ICE-SNOW-FROST POINTS',/)                         
 1002 FORMAT(1X,130A1)                                                          
C                                                                               
C     CALL PRINCE( TOPOG,'PHIS',IM,JNP,'DISP')                                  
C     CALL PRINCE(  TVAZ,'TVA ',IM,JNP,'DISP')                                  
C     CALL PRINCE(   TGZ,'GT  ',IM,JNP,'DISP')                                  
C     CALL PRINCE(   GWZ,'GW  ',IM,JNP,'DISP')                                  
C     CALL PRINCE(  ALBZ,'ALB ',IM,JNP,'DISP')                                  
C     CALL PRINCE(   SRZ,'SRF ',IM,JNP,'DISP')                                  
C     CALL PRINCE(  THKZ,'ICE ',IM,JNP,'DISP')                                  
C                                                                               
      RETURN                                                                    
      END 
      SUBROUTINE TICK (NYMD,NHMS,NDT,N)
C***********************************************************************
C  PURPOSE
C     UPDATES TIME INFORMATION
C  ARGUMENTS   DESCRIPTION
C     NYMD ...... CURRENT YYMMDD
C     NHMS ...... CURRENT HHMMSS
C     NDT ....... TIME-STEP
C     N ......... CURRENT TIME-INDEX
C
C***********************************************************************
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *
C***********************************************************************

      NXTMOD(N,M) = MOD(N,M) + 1
       NSECF(N)   = N/10000*3600 + MOD(N,10000)/100* 60 + MOD(N,100)
       NHMSF(N)   = N/3600*10000 + MOD(N,3600 )/ 60*100 + MOD(N, 60)

C **********************************************************************
C ****                    UPDATE TIME INFORMATION                   ****
C **********************************************************************

      IF(NDT.NE.0) THEN
      N    = NXTMOD (N,2)
      NSEC = NSECF(NHMS) + NDT

      IF (NSEC.GT.86400)  THEN
      DO WHILE (NSEC.GT.86400)
      NSEC = NSEC - 86400
      NYMD = INCYMD (NYMD,1)
      ENDDO
      ENDIF   
               
      IF (NSEC.EQ.86400)  THEN
      NSEC = 0
      NYMD = INCYMD (NYMD,1)
      ENDIF   
               
      IF (NSEC.LT.00000)  THEN
      DO WHILE (NSEC.LT.0)
      NSEC = 86400 + NSEC
      NYMD = INCYMD (NYMD,-1)
      ENDDO
      ENDIF   
               
      NHMS = NHMSF (NSEC)
      ENDIF   
      RETURN  
      END    
      FUNCTION NALARM (MHMS,NYMD,NHMS,NYMD0,NHMS0)                              
C***********************************************************************        
C  PURPOSE                                                                      
C     COMPUTES MODULO-FRACTION BETWEEN MHHS AND TOTAL TIME                      
C  USAGE                                                                        
C  ARGUMENTS   DESCRIPTION                                                      
C     MHMS     INTERVAL FREQUENCY (HHMMSS)                                      
C     NYMD     CURRENT   YYMMDD                                                 
C     NHMS     CURRENT   HHMMSS                                                 
C     NYMD0    BEGINNING YYMMDD                                                 
C     NHMS0    BEGINNING HHMMSS                                                 
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
      PARAMETER ( NSDAY  = 86400 )                                              
      PARAMETER ( NCYCLE = 1461*24*3600 )                                       
C                                                                               
      INTEGER YEAR, DAY, SEC, YEAR0, DAY0, SEC0                                 
                                                                                
      DIMENSION  MNDY(12,4)                                                     
      DATA MNDY /0,31,60,91,121,152,182,213,244,274,305,335,366,                
     .           397,34*0 /                                                     
                                                                                
      NSECF(N) = N/10000*3600 + MOD(N,10000)/100* 60 + MOD(N,100)               
C                                                                               
C***********************************************************************        
C*                 COMPUTE # OF DAYS IN A 4-YEAR CYCLE                 *        
C***********************************************************************        
                                                                                
      DO 100 I=15,48                                                            
      MNDY(I,1) = MNDY(I-12,1) + 365                                            
100   CONTINUE                                                                  
                                                                                
C***********************************************************************        
C*                   SET CURRENT AND BEGINNING TIMES                   *        
C***********************************************************************        
C                                                                               
      YEAR   = NYMD / 10000                                                     
      MONTH  = MOD(NYMD,10000) / 100                                            
      DAY    = MOD(NYMD,100)                                                    
      SEC    = NSECF(NHMS)                                                      
C                                                                               
      YEAR0  = NYMD0 / 10000                                                    
      MONTH0 = MOD(NYMD0,10000) / 100                                           
      DAY0   = MOD(NYMD0,100)                                                   
      SEC0   = NSECF(NHMS0)                                                     
C                                                                               
C***********************************************************************        
C*      COMPUTE POSITIONS IN CYCLE FOR CURRENT AND BEGINNING TIMES     *        
C***********************************************************************        
C                                                                               
      IDAY   = (DAY -1) + MNDY( MONTH ,MOD(YEAR ,4)+1 )                         
      IDAY0  = (DAY0-1) + MNDY( MONTH0,MOD(YEAR0,4)+1 )                         
                                                                                
      NSEC   = IDAY *NSDAY + SEC                                                
      NSEC0  = IDAY0*NSDAY + SEC0                                               
                                                                                
                       NTIME  = NSEC-NSEC0                                      
      IF (NTIME.LT.0 ) NTIME  = NTIME + NCYCLE                                  
                       NALARM = NTIME                                           
      IF ( MHMS.NE.0 ) NALARM = MOD( NALARM,NSECF(MHMS) )                       
C                                                                               
      RETURN                                                                    
      END                                                                       
      FUNCTION MODHMS (NHMS, MHMS)                                              
C***********************************************************************        
C  PURPOSE                                                                      
C     TIME SINCE LAST INCREMENT TIME OCCURRED IN SECONDS                        
C  ARGUMENTS   DESCRIPTION                                                      
C     NHMS     CURRENT   TIME IN HHMMSS FORMAT                                  
C     MHMS     INCREMENT TIME IN HHMMSS FORMAT                                  
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
      NSECF(N) = N/10000*3600 + MOD(N,10000)/100* 60 + MOD(N,100)               
C                                                                               
      MSEC   = NSECF(NHMS)                                                      
      IF (MHMS.GT.0)  MSEC = MOD(MSEC,NSECF(MHMS))                              
      MODHMS = MSEC                                                             
C                                                                               
      RETURN                                                                    
      END                                                                       
      FUNCTION INCYMD (NYMD,M)                                                  
C***********************************************************************        
C  PURPOSE                                                                      
C     INCYMD:  NYMD CHANGED BY ONE DAY                                          
C     MODYMD:  NYMD CONVERTED TO JULIAN DATE                                    
C  DESCRIPTION OF PARAMETERS                                                    
C     NYMD     CURRENT DATE IN YYMMDD FORMAT                                    
C     M        +/- 1 (DAY ADJUSTMENT)                                           
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
      INTEGER NDPM(12)                                                          
      DATA    NDPM /31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/             
      LOGICAL LEAP                                                              
      DATA    NY00     / 1900 /                                                 
      LEAP(NY) = MOD(NY,4).EQ.0 .AND. (NY.NE.0 .OR. MOD(NY00,400).EQ.0)         
C                                                                               
C***********************************************************************        
C                                                                               
CVD$ NOVECTOR                                                                   
      NY = NYMD / 10000                                                         
      NM = MOD(NYMD,10000) / 100                                                
      ND = MOD(NYMD,100) + M                                                    
C                                                                               
      IF (ND.EQ.0) THEN                                                         
      NM = NM - 1                                                               
      IF (NM.EQ.0) THEN                                                         
          NM = 12                                                               
          NY = NY - 1                                                           
      ENDIF                                                                     
      ND = NDPM(NM)                                                             
      IF (NM.EQ.2 .AND. LEAP(NY))  ND = 29                                      
      ENDIF                                                                     
                                                                                
      IF (ND.EQ.29 .AND. NM.EQ.2 .AND. LEAP(NY))  GO TO 20                      
                                                                                
      IF (ND.GT.NDPM(NM)) THEN                                                  
      ND = 1                                                                    
      NM = NM + 1                                                               
      IF (NM.GT.12) THEN                                                        
          NM = 1                                                                
          NY = NY + 1                                                           
      ENDIF                                                                     
      ENDIF                                                                     
C                                                                               
   20 CONTINUE                                                                  
      INCYMD = NY*10000 + NM*100 + ND                                           
CVD$ VECTOR                                                                     
      RETURN                                                                    
C                                                                               
C***********************************************************************        
C                      E N T R Y    M O D Y M D                                 
C***********************************************************************        
C                                                                               
      ENTRY MODYMD (NYMD)                                                       
CVD$ NOVECTOR                                                                   
      NY = NYMD / 10000                                                         
      NM = MOD(NYMD,10000) / 100                                                
      ND = MOD(NYMD,100)                                                        
C                                                                               
   40 CONTINUE                                                                  
      IF (NM.LE.1)  GO TO 60                                                    
      NM = NM - 1                                                               
      ND = ND + NDPM(NM)                                                        
      IF (NM.EQ.2 .AND. LEAP(NY))  ND = ND + 1                                  
      GO TO 40                                                                  
C                                                                               
   60 CONTINUE                                                                  
      MODYMD = ND                                                               
CVD$ VECTOR                                                                     
      RETURN                                                                    
      END                                                                       
      SUBROUTINE PRINCE(Q,NAME,IM,JM,PTYPE)                                     
C                                                                               
      PARAMETER (  ONE = 1.0 )                                                  
      PARAMETER ( ZERO = 0.0 )                                                  
                                                                                
      DIMENSION  Q(IM,JM)                                                       
C                                                                               
      CHARACTER*4 NAME, PTYPE, PTD, PTG, MAGS                                   
      DATA        PTD/'DISP'/, PTG/'GRAF'/, MAGS/'MAGS'/                        
C                                                                               
      KU = 6                                                                    
      CALL SCALE(Q,IM,JM,IM,JM,MQ,QMIN,QMAX,DQ,0,IMN,JMN,IMX,JMX)               
C                                                                               
      WRITE(KU,100) NAME,NAME,IMN,JMN,QMIN,NAME,IMX,JMX,QMAX                    
  100 FORMAT(1X,A4,' FIELD',5X,'MIN: ',A4,'(',I3,',',I3,') = ',G12.5,           
     $                      5X,'MAX: ',A4,'(',I3,',',I3,') = ',G12.5)           
      IF(PTYPE.EQ.MAGS) RETURN                                                  
C                                                                               
      IF(PTYPE.NE.PTG) THEN                                                     
      CALL DISPLA (Q,QMAX,QMIN,NAME,IM,JM,IM,JM,MQ,ZERO,4,2,KU)                 
      ENDIF                                                                     
C                                                                               
      IF(PTYPE.NE.PTD) THEN                                                     
      CALL GRAFIC (Q,NAME,IM,JM,IM,JM,QMIN,QMAX,DQ,1,1,KU)                      
      ENDIF                                                                     
C                                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE DISPLA(F,FMAX,FMIN,                                            
     $                 NAMEF,IMAX,JMAX,IDIM,JDIM,M,CONST,NX,NY,K)               
      PARAMETER (  ONE =  1.0 )                                                 
      PARAMETER ( ZERO =  0.0 )                                                 
      PARAMETER (  TEN = 10.0 )                                                 
      PARAMETER (  ZP5 =  0.5 )                                                 
      CHARACTER*4  NAMEF                                                        
      CHARACTER*4  PTYPE                                                        
      CHARACTER*1  PLUS,MINUS,BLANK,SIGN(132)                                   
      INTEGER      INT(44)                                                
      DATA PLUS/'+'/,MINUS/'-'/,BLANK/' '/                                      
      DIMENSION F(IDIM,JDIM)                                                    
      FSCALE = ONE                                                              
      NYM1 = NY-1                                                               
      IC = 132/NX -1                                                            
      DO 20 I=1,132                                                             
   20 SIGN(I) = BLANK                                                           
      MX = -M                                                                   
      WRITE(K,63) MX                                                            
   63 FORMAT(//1X,'SCALING FACTOR:  10**',I3,/)                                 
      IF( CONST.NE.ZERO ) WRITE(K,66) CONST                                     
   66 FORMAT(1X,'CONSTANT OF ',1PE10.2,'   HAS BEEN SUBTRACTED ')               
C                                                                               
      EXPON = TEN**M                                                            
      IA = 1                                                                    
  100 IB = IA+IC                                                                
      IF( IB.GT.IMAX ) IB=IMAX                                                  
      WRITE(K,64) IA,IB                                                         
   64 FORMAT(1X,'I =',I3,'  TO I =',I3)                                         
      DO 10 L=1,JMAX                                                            
      J = L                                                                     
      J = JMAX-L+1                                                              
      DO 14 I=IA,IB                                                             
      II = I-IA+1                                                               
   14 INT(II) = ABS( (F(I,J)*FSCALE-CONST) * EXPON ) + ZP5                      
      GO TO (101,102,103,104,105,106), NX                                       
      WRITE(K,608)                                                              
  608 FORMAT(1X,'INVALID NX IN SUBROUTINE DISP')                                
      RETURN                                                                    
  101 WRITE(K,601) (INT(I-IA+1),I=IA,IB)                                        
  601 FORMAT(' ',132I1)                                                         
      GO TO 109                                                                 
  102 WRITE(K,602) (INT(I-IA+1),I=IA,IB)                                        
  602 FORMAT(' ', 66I2)                                                         
      GO TO 109                                                                 
  103 WRITE(K,603) (INT(I-IA+1),I=IA,IB)                                        
  603 FORMAT(' ', 44I3)                                                         
      GO TO 109                                                                 
  104 WRITE(K,604) (INT(I-IA+1),I=IA,IB)                                        
  604 FORMAT(' ', 33I4)                                                         
      GO TO 109                                                                 
  105 WRITE(K,605) (INT(I-IA+1),I=IA,IB)                                        
  605 FORMAT(' ', 26I5)                                                         
      GO TO 109                                                                 
  106 WRITE(K,606) (INT(I-IA+1),I=IA,IB)                                        
  606 FORMAT(' ', 22I6)                                                         
  109 CONTINUE                                                                  
      DO 121 I=1,132                                                            
  121 SIGN(I) = BLANK                                                           
      DO 11 I=IA,IB                                                             
      II = (I-IA+1) * NX                                                        
      IF( (F(I,J)*FSCALE-CONST).GE.ZERO ) GO TO 13                              
   12 SIGN(II) = MINUS                                                          
      GO TO 11                                                                  
   13 SIGN(II) = PLUS                                                           
   11 CONTINUE                                                                  
      II = (IB-IA+1) * NX                                                       
      WRITE(K,61) (SIGN(I),I=1,II)                                              
   61 FORMAT(' ',132A1)                                                         
      IY = 1                                                                    
   40 CONTINUE                                                                  
      IF( IY.GE.NYM1 ) GO TO 41                                                 
      WRITE(K,600)                                                              
  600 FORMAT(' ')                                                               
      IY = IY+1                                                                 
      GO TO 40                                                                  
   41 CONTINUE                                                                  
   10 CONTINUE                                                                  
      IA = IB+1                                                                 
      IF( IA.LE.IMAX ) GO TO 100                                                
      WRITE(K,9191)                                                             
 9191 FORMAT('1')                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE SCALE (F,IM,JM,IDIM,JDIM,N,FMIN,FMAX,DF,NUM,                   
     $                  IMIN,JMIN,IMAX,JMAX)                                    
      PARAMETER ( ZZMAX = -1.0E20 )                                             
      PARAMETER ( ZZMIN =  1.0E20 )                                             
      PARAMETER ( ZERO  =  0.0    )                                             
      PARAMETER (  ONE  =  1.0    )                                             
      PARAMETER (  Z18  = 18.0    )                                             
      PARAMETER (   Z2  =  2.0    )                                             
      DIMENSION F(IDIM,JDIM)                                                    
      IF(NUM.GE.9) NUM=8                                                        
      IMAX = 0                                                                  
      JMAX = 0                                                                  
      IMIN = 0                                                                  
      JMIN = 0                                                                  
      FMAX = ZZMAX                                                              
      FMIN = ZZMIN                                                              
      DO 10 J=1,JM                                                              
      DO 10 I=1,IM                                                              
      IF( F(I,J).LT.FMAX ) GOTO 5                                               
      IMAX = I                                                                  
      JMAX = J                                                                  
      FMAX = F(I,J)                                                             
    5 IF( F(I,J).GT.FMIN ) GOTO 10                                              
      IMIN = I                                                                  
      JMIN = J                                                                  
      FMIN = F(I,J)                                                             
   10 CONTINUE                                                                  
      AFMAX = ABS(FMAX)                                                         
      AFMIN = ABS(FMIN)                                                         
      EX = MAX( AFMAX, AFMIN )                                                  
      IF(EX.EQ.ZERO) GOTO 20                                                    
      X = -LOG10(EX)                                                            
      N = X                                                                     
      IF( FLOAT(N).GE.X .AND. FLOAT(N).LT.(ONE+X) ) GO TO 30                    
      N = N+1                                                                   
      GOTO 30                                                                   
   20 N = 18                                                                    
   30 CONTINUE                                                                  
      N = N + 2                                                                 
      DF = (FMAX-FMIN)/(Z18-Z2*NUM)                                             
      FMIN = FMIN - NUM*DF                                                      
      FMAX = FMAX + NUM*DF                                                      
      RETURN                                                                    
      END                                                                       
      SUBROUTINE GRAFIC(F,NAMEF,IMAX,JMAX,IDIM,JDIM,FMIN,FMAX,DF,NX,NY,         
     $K)                                                                        
      PARAMETER ( ONE = 1.0 )                                                   
      CHARACTER*4 NAMEF                                                         
      CHARACTER*1 W(132),SYM(23),BLANK,ASTER,PLUS,DOT,MINUS                     
      DIMENSION F(IDIM,JDIM)                                                    
      EQUIVALENCE (BLANK,SYM(2)),(MINUS,SYM(1)),(PLUS,SYM(22)),                 
     .            (ASTER,SYM(21)),(DOT,SYM(23))                                 
      DATA SYM/ '-',' ','1',' ','2',' ','3',' ','4',' ',                        
     .          '5',' ','6',' ','7',' ','8',' ',                                
     .          '9',' ','*','+','.' /                                           
      FX = NX                                                                   
      FY = NY                                                                   
      IC = 130/NX -1                                                            
  601 FORMAT(1X,132A1)                                                          
      IA = 1                                                                    
  100 IB = IA+IC                                                                
      IF( IB.GT.IMAX ) IB=IMAX                                                  
      WRITE(K,64) IA,IB                                                         
   64 FORMAT(//1X,3HI =,I3,8H  TO I =,I3)                                       
      INDMAX = NX*(IB-IA) + 1                                                   
      IMNX = INDMAX+2                                                           
      DO 70 NI=1,IMNX                                                           
   70 W(NI) = BLANK                                                             
      DO 71 NI=2,IMNX,NX                                                        
   71 W(NI) = DOT                                                               
      WRITE(K,601) (W(NI),NI=1,IMNX)                                            
      LA = 0                                                                    
      L = 2                                                                     
    1 J = JMAX-L+1                                                              
      IY = 0                                                                    
    2 Y1 = IY/FY                                                                
      Y = ONE-Y1                                                                
    3 I = IA                                                                    
   13 IX = 0                                                                    
   14 X = IX/FX                                                                 
      X1 = ONE-X                                                                
      F1 = X1*Y1*F(I,J) + X*Y1*F(I+1,J) + X1*Y*F(I,J+1) + X*Y*F(I+1,J+1)        
      NUM = 1                                                                   
      IF(F1-FMAX) 10,11,11                                                      
   11 W(NX*(I-IA)+1+IX) = PLUS                                                  
   12 IX = IX+1                                                                 
      IF( IX.LT.NX ) GO TO 14                                                   
      I = I+1                                                                   
      IF( I.LT.IB ) GO TO 13                                                    
      GO TO 41                                                                  
   10 IF( F1-FMIN ) 20,21,21                                                    
   20 W(NX*(I-IA)+1+IX) = SYM(NUM)                                              
      GO TO 12                                                                  
   21 F1 = F1-DF                                                                
      IF( NUM.LE.20 ) NUM = NUM+1                                               
      GO TO 10                                                                  
   41 W(INDMAX) = BLANK                                                         
      IF(IY) 42,42,43                                                           
   42 WRITE(K,601) DOT,(W(NI),NI=1,INDMAX),DOT                                  
      IF(LA) 44,44,47                                                           
   43 IF(NY.GT.1) WRITE(K,601) BLANK,(W(NI),NI=1,INDMAX)                        
      IF( IY.GE.(NY-1) ) GO TO 45                                               
   44 IY = IY+1                                                                 
      GO TO 2                                                                   
   45 L = L+1                                                                   
      IF(L-JMAX-1) 1,46,46                                                      
   46 LA = 1                                                                    
      J = 1                                                                     
      IY = 0                                                                    
      Y1 = 1.0                                                                  
      Y = 0.0                                                                   
      GO TO 3                                                                   
   47 DO 72 NI=1,IMNX                                                           
   72 W(NI) = BLANK                                                             
      DO 73 NI=2,IMNX,NX                                                        
   73 W(NI) = DOT                                                               
      WRITE(K,601) (W(NI),NI=1,IMNX)                                            
      IA = IB+1                                                                 
      IF( IA.LE.IMAX ) GO TO 100                                                
      WRITE(K,605) FMIN,FMAX,DF                                                 
  605 FORMAT(////,1X,13HLOWER LIMIT =,G11.4,5X,13HUPPER LIMIT =,G11.4,          
     $       5X,10HINTERVAL =,G11.4,/'1')                                       
      RETURN                                                                    
      END                                                                       
      FUNCTION GETCON (NAME)                                                    
C***********************************************************************        
C                                                                               
C  GENERIC FUNCTION GETCON IS A REPOSITORY OF GLOBAL VARIABLES, I.E.            
C  A MEMORY FOR SCALAR VALUES NEEDED THROUGHOUT A LARGE PROGRAM.                
C                                                                               
C  THIS SPECIFIC FUNCTION, GETCON, REMEMBERS FLOATING POINT VALUES.             
C  THE FUNCTION IS CALLED WITH A CHARACTER NAME TO INTERROGATE A VALUE.         
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
      PARAMETER (MAXCON=36)                                                     
      CHARACTER*(*) NAME                                                        
      CHARACTER*16 ANAME(MAXCON)                                                
      DIMENSION    ACON (MAXCON)                                                
                                                                                
C COMPUTATIONAL CONSTANTS                                                       
C -----------------------                                                       
      PARAMETER ( VECMAX = 65535.5    )                                         
      PARAMETER ( CALTOJ = 4184.      )                                         
      PARAMETER ( UNDEF  = 1.E15      )                                         
                                                                                
C ASTRONOMICAL CONSTANTS                                                        
C ----------------------                                                        
      PARAMETER ( OB     = 23.45      )                                         
      PARAMETER ( PER    = 102.       )                                         
      PARAMETER ( ECC    = .0167      )                                         
      PARAMETER ( AE     = 6371E3     )                                         
      PARAMETER ( EQNX   = 80.5       )                                         
      PARAMETER ( SOLS   = 176.5      )                                         
      PARAMETER ( S0     = 1380.0     )                                         
                                                                                
C TERRESTRIAL CONSTANTS                                                         
C ---------------------                                                         
      PARAMETER ( GRAV   = 9.81       )                                         
      PARAMETER ( SRFPRS = 984.7      )                                         
      PARAMETER ( PIMEAN = 984.7      )                                         
      PARAMETER ( PSTD   = 1000.0     )                                         
      PARAMETER ( TSTD   = 280.0      )                                         
      PARAMETER ( SDAY   = 86400.0    )                                         
      PARAMETER ( SSALB  = 0.99       )                                         
      PARAMETER ( CO2    = 330.0      )                                         
                                                                                
C THERMODYNAMIC CONSTANTS                                                       
C -----------------------                                                       
      PARAMETER ( CPD    = 1004.16    )                                         
      PARAMETER ( CPV    = 1869.46    )                                         
      PARAMETER ( ALHL   = 2.499E6    )                                         
      PARAMETER ( ALHS   = 2.845E6    )                                         
      PARAMETER ( STFBOL = 5.67E-8    )                                         
      PARAMETER ( AIRMW  = 28.97      )                                         
      PARAMETER ( H2OMW  = 18.01      )                                         
      PARAMETER ( RUNIV  = 8314.3     )                                         
      PARAMETER ( RGAS   = RUNIV/AIRMW)                                         
      PARAMETER ( RKAP   = RGAS/CPD   )                                         
      PARAMETER ( HEATW  = 597.2      )                                         
      PARAMETER ( HEATI  = 680.0      )                                         
                                                                                
C TURBULENCE CONSTANTS                                                          
C --------------------                                                          
      PARAMETER ( VKRM   = 0.41       )                                         
                                                                                
C MOISTURE CONSTANTS                                                            
C ------------------                                                            
      PARAMETER ( TICE   = 273.16     )                                         
      PARAMETER ( EPS    = 0.622      )                                         
      PARAMETER ( VIRTCON= 0.609      )                                         
      PARAMETER ( EPSFAC = EPS*HEATW/RGAS*CALTOJ )                              
                                                                                
C                                                                               
       DATA ANAME(1 ),ACON(1 ) / 'CP              ', CPD    /                   
       DATA ANAME(2 ),ACON(2 ) / 'RGAS            ', RGAS   /                   
       DATA ANAME(3 ),ACON(3 ) / 'KAPPA           ', RKAP   /                   
       DATA ANAME(4 ),ACON(4 ) / 'LATENT HEAT COND', ALHL   /                   
       DATA ANAME(5 ),ACON(5 ) / 'GRAVITY         ', GRAV   /                   
       DATA ANAME(6 ),ACON(6 ) / 'STEFAN-BOLTZMAN ', STFBOL /                   
       DATA ANAME(7 ),ACON(7 ) / 'VON KARMAN      ', VKRM   /                   
       DATA ANAME(8 ),ACON(8 ) / 'EARTH RADIUS    ', AE     /                   
       DATA ANAME(9 ),ACON(9 ) / 'OBLIQUITY       ', OB     /                   
       DATA ANAME(10),ACON(10) / 'ECCENTRICITY    ', ECC    /                   
       DATA ANAME(11),ACON(11) / 'PERIHELION      ', PER    /                   
       DATA ANAME(12),ACON(12) / 'VERNAL EQUINOX  ', EQNX   /                   
       DATA ANAME(13),ACON(13) / 'SUMMER SOLSTICE ', SOLS   /                   
       DATA ANAME(14),ACON(14) / 'MAX VECT LENGTH ', VECMAX /                   
       DATA ANAME(15),ACON(15) / 'MOL WT H2O      ', H2OMW  /                   
       DATA ANAME(16),ACON(16) / 'MOL WT AIR      ', AIRMW  /                   
       DATA ANAME(17),ACON(17) / 'CPV             ', CPV    /                   
       DATA ANAME(18),ACON(18) / 'CPD             ', CPD    /                   
       DATA ANAME(19),ACON(19) / 'UNIV GAS CONST  ', RUNIV  /                   
       DATA ANAME(20),ACON(20) / 'LATENT HEAT SBLM', ALHS   /                   
       DATA ANAME(21),ACON(21) / 'FREEZING-POINT  ', TICE   /                   
       DATA ANAME(23),ACON(23) / 'CALTOJ          ', CALTOJ /                   
       DATA ANAME(24),ACON(24) / 'EPS             ', EPS    /                   
       DATA ANAME(25),ACON(25) / 'HEATW           ', HEATW  /                   
       DATA ANAME(26),ACON(26) / 'EPSFAC          ', EPSFAC /                   
       DATA ANAME(27),ACON(27) / 'VIRTCON         ', VIRTCON/                   
       DATA ANAME(28),ACON(28) / 'PIMEAN          ', PIMEAN /                   
       DATA ANAME(29),ACON(29) / 'SDAY            ', SDAY   /                   
       DATA ANAME(30),ACON(30) / 'HEATI           ', HEATI  /                   
       DATA ANAME(31),ACON(31) / 'S0              ', S0     /                   
       DATA ANAME(32),ACON(32) / 'PSTD            ', PSTD   /                   
       DATA ANAME(33),ACON(33) / 'TSTD            ', TSTD   /                   
       DATA ANAME(34),ACON(34) / 'SSALB           ', SSALB  /                   
       DATA ANAME(35),ACON(35) / 'UNDEF           ', UNDEF  /                   
       DATA ANAME(36),ACON(36) / 'CO2             ', CO2    /                   
                                                                                
      DO 10 I=1,MAXCON                                                          
      IF(NAME.EQ.ANAME(I)) THEN                                                 
         GETCON = ACON(I)                                                       
         RETURN                                                                 
      ENDIF                                                                     
10    CONTINUE                                                                  
                                                                                
900   PRINT *,' CANNOT FIND FLOATING POINT CONSTANT - ',NAME                    
      PRINT *,' GETCON - CANNOT FIND CONSTANT REQUESTED'                        
      STOP                                                                      
      END                                                                       
      SUBROUTINE GETFG ( UP,VP,HP,QP,RHP,SLP,SLU,SLV,TSRF,TVSRF,
     .                   PLEV,IRUN,NLEV,NB,
     .                   PKZ,TD,THV,RH,
     .                   WRK )      
C***********************************************************************        
C                                                                               
C  PURPOSE                                                                      
C     To obtain FIRST-GUESS Fields on Pressure-Levels for Analysis
C
C  INPUT  DESCRIPTION                                                         
C     PLEV ..... Analysis Pressure Levels
C     IRUN ..... Horizontal Dimension
C     NLEV ..... Number of Analysis Pressure Levels
C     NB ....... Current Time Index
C     PKZ ...... Upper Air Work Array (IRUN,NLAY)
C     TD ....... Upper Air Work Array (IRUN,NLAY)
C     THV ...... Upper Air Work Array (IRUN,NLAY)
C     RH ....... Upper Air Work Array (IRUN,NLAY)
C     WRK ...... Misc.     Work Array (IRUN,8)
C
C  OUTPUT DESCRIPTION                                                         
C     UP ....... Zonal      U-Wind on Pressure                                       
C     VP ....... Meridional V-Wind on Pressure                                       
C     HP ....... Heights (Virtual) on Pressure                                       
C     QP ....... Mixing Ratio      on Pressure                                       
C     RHP ...... Relative Humidity on Pressure                                       
C     SLP ...... Sea-Level Pressure
C     SLU ...... Sea-Level U-Wind
C     SLV ...... Sea-Level V-Wind
C     TSRF ..... Surface   Temperature
C     TVSRF .... Surface   Temperature (Virtual)
C
C  NOTE:
C     STATE COMMON is assumed to be on A-GRID
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vstate.com>                                                           
#include <vmetric.com>                                                          
                                                                                
      PARAMETER  (IMJMR   =  IMR*(JMR+1)  )                                     
      PARAMETER  (ONE     =  1.           )                                     
      PARAMETER  (TWO     =  2.           )                                     
      PARAMETER  (THREE   =  3.           )                                     
                                                                                
      Dimension    UP (irun,nlev)
      Dimension    VP (irun,nlev)
      Dimension    HP (irun,nlev)
      Dimension    QP (irun,nlev)
      Dimension   RHP (irun,nlev)
      Dimension   SLP (irun)
      Dimension   SLU (irun)
      Dimension   SLV (irun)
      Dimension  TSRF (irun)
      Dimension TVSRF (irun)
      Dimension  PLEV (nlev)

c Work Arrays
c -----------
      Dimension    PKZ (IRUN,NLAYR)
      Dimension    TD  (IRUN,NLAYR)
      Dimension    THV (IRUN,NLAYR)
      Dimension    RH  (IRUN,NLAYR)
      Dimension    WRK (IRUN,8)

                                                                                
C *********************************************************************
C *****           CHECK HORIZONTAL CONSISTANCY                     ****
C *********************************************************************

      if( irun.ne.imjmr ) then
      Print *
      Print *, 'Inconsistant Horizontal Resolution'
      Print *
      Print *, 'GCM Compiled with IMJMR = ',imjmr
      Print *, 'GETFG called with IRUN  = ',irun
      Print *
      Stop
      endif

      UNDEF = getcon('UNDEF')
      grav  = getcon('GRAVITY')
      AKAP  = getcon('KAPPA')                                                    
                                                                                
C *********************************************************************         
C *****        CONVERT POTENTIAL TEMPERATURE TO TEMPERATURE        ****         
C *****       CONVERT SPECIFIC HUMIDITY TO RELATIVE HUMIDITY       ****         
C *********************************************************************         

      call pkappa ( pz(1,1,nb),pkht,pkz,imr,jmr+1,nlayr )

      DO 40 L = 1,NLAYR                                                         
      DO    I = 1,IMJMR                                                         
      TD (I,L) = TZ(I,1,L,nb)*PKZ(I,L)                                     
      THV(I,L) = TZ(I,1,L,nb)*(1. + 0.609*QZ(I,1,L,nb) )                        
      WRK(I,1) = PZ(I,1  ,nb)*SIG(L) + PTOP                                  
      ENDDO
                                                                                
      CALL QSTARS ( TD(1,L),wrk(1,1),                                             
     .             wrk(1,2),wrk(1,3),wrk(1,4),wrk(1,5),wrk(1,6),
     .             IMJMR,RH(1,L) )                                                  
                                                                                
      DO I = 1,IMJMR                                                         
      RH(I,L) = QZ(I,1,L,nb) / RH(I,L)                                      
      ENDDO
   40 CONTINUE                                                                  
                                                                                

C *********************************************************************         
C *****          COMPUTE SEA-LEVEL AND SURFACE QUANTITIES          ****         
C *********************************************************************         
                                                                                
      CALL SLPRS  ( SLP,PZ(1,1,NB),PTOP,PHISZ,THV,DSIG,WRK )                                                 
      CALL TSPBL  ( PZ,PTOP,TZ(1,1,1,NB),DSIG,IMR,JMR+1,NLAYR,TSRF  )
      CALL TSPBL  ( PZ,PTOP,THV         ,DSIG,IMR,JMR+1,NLAYR,TVSRF )
 
      do I=1,IMJMR
      SLU(I) = UZ(I,1,NLAYR,NB)
      SLV(I) = VZ(I,1,NLAYR,NB)
      ENDDO


C *********************************************************************         
C *****          INTERPOLATE TO ANALYSIS PRESSURE LEVELS           ****         
C *********************************************************************         
                                                                                
      DO   L = 1, NLEV
      CALL INTERP  ( up(1,L),uz(1,1,1,nb),PKHT,PKZ,PLEV(L),UNDEF )               
      CALL INTERP  ( vp(1,L),vz(1,1,1,nb),PKHT,PKZ,PLEV(L),UNDEF )               
      CALL INTERP  (rhp(1,L),rh          ,PKHT,PKZ,PLEV(L),UNDEF )               
                                                                                
      CALL INTGEOP ( hp(1,L),THV,PKHT,PKZ,                          
     .               PHISZ,PLEV(L),UNDEF,WRK )                            
                                                                               
      CALL INTERQ  ( wrk(1,8), QP(1,L), RH, TD, QZ(1,1,1,nb),                   
     .               PKHT, PKZ, PLEV(L),                                         
     .               UNDEF, wrk(1,1),wrk(1,3),wrk(1,4),
     .                      wrk(1,5),wrk(1,6),wrk(1,7) )                      


c Scale Relative Humidity and Compute Mixing Ratio
c ------------------------------------------------
      DO I=1,IMJMR
      IF( RHP(I,L).NE.UNDEF ) RHP(I,L) =  100 * RHP(I,L)
      IF(  QP(I,L).NE.UNDEF )  QP(I,L) = 1000 *  QP(I,L)/(1.-QP(I,L))
      ENDDO

      ENDDO
                                                                                

C *********************************************************************         
C *****             FILL AND MASK UNDERGROUND VALUES               ****         
C *********************************************************************         
                                                                                
      DO I=1,IMJMR
      wrk(I,1) = PZ(I,1,nb)+PTOP
      wrk(I,2) = PHISZ(I,1)/grav
      IF( ZLWI(I,1).NE.ONE ) THEN
      SLU(I) = UNDEF
      SLV(I) = UNDEF
      ENDIF
      ENDDO

      CALL ZUNDERG ( HP,TVSRF,wrk(1,1),wrk(1,2),PLEV,
     .               UNDEF,IMR,JMR+1,NLEV )

      RETURN                                                                    
      END                                                                       
      SUBROUTINE TSPBL (PZ,PTOP,THZ,DSIG,IM,JNP,NLAY,ts_new)
C***********************************************************************        
C
C  TSPBL: "New" surface temperature derived from mean PBL potential 
C         temperature
C                                                                               
C  INPUT                                                                        
C    PZ ...... SURFACE   PRESSURE (MB) - PTOP                                   
C    PTOP .... MODEL TOP PRESSURE (MB)                                          
C    THZ ..... POTENTIAL TEMPERATURE (K) ON  SIGMA LEVELS (T/P**KAPPA)
C    DSIG .... SIGMA INCREMENTS                                                 
C    IM ...... NUMBER OF LONGITUDINAL GRID POINTS
C    JNP ..... NUMBER OF LATITUDINAL GRID POINTS
C    NLAY .... NUMBER OF LEVELS
C                                                                               
C  OUTPUT                                                                       
C    ts_new .. Temperature (K) at surface pressure.  ts_new is derived
C              by computing the mean potential temperature in (approximately)
C              the lowest 100 mb of the atmosphere (this represents the PBL)
C              and calculating what temperature this mean potential 
C              temperature represents at the surface pressure. 
C
C  NOTE: TSPBL is a modification of Phoenix model subroutine SLPRS.
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
      dimension  pz     (im,jnp)
      dimension  thz    (im,jnp,nlay)
      dimension  ts_new (im,jnp)
      dimension  dsig   (nlay)
                                                                                
      ak = getcon('KAPPA')                                                      
 
      do i = 1,im*jnp
      ts_new(i,1) = 0.
      end do
                                                                                
C***********************************************************************        
C*                COMPUTE MEAN THETA IN PBL (100 MB)                   *        
C***********************************************************************        
                                                                                
      delp = 0.
      dels = 0.
 
      do L = nlay,1,-1                                                        
      delp = delp + (1000.-ptop)*dsig(L)
                                                                                
      if (delp.le.100.) then
          dels = dels + dsig(L)
          do i = 1,im*jnp
          ts_new(i,1) = ts_new(i,1) + thz(i,1,L)*dsig(L)
          end do
      endif 
                                                                                
      enddo
 
      do i = 1,im*jnp
      ts_new(i,1) = ts_new(i,1) / dels
      end do
                                                                                
C***********************************************************************        
C*                Compute new surface temperature                      *        
C***********************************************************************        
                                                                                
      do i = 1,im*jnp
      ts_new(i,1) = ts_new(i,1) * (pz(i,1)+ptop)**ak
      end do
                                                                                
      return
      end
      subroutine zunderg (z_p, tv_sfc,  p_sfc, z_sfc,
     .                    pmand, UNDEF, im, jnp, nlevoi )

      real  z_p    (im,jnp,nlevoi)
      real  tv_sfc (im,jnp)
      real  p_sfc  (im,jnp)
      real  z_sfc  (im,jnp)
      real  pmand  (nlevoi)

      grav = getcon('GRAVITY')
      rgas = getcon('RGAS')
      beta = 0.0065

      do L=1,nlevoi

      do i=1,im*jnp
      if (z_p(i,1,L).eq.undef) THEN
          z_p(i,1,L) = z_sfc(i,1) - tv_sfc(i,1)/beta*
     .               ((pmand(L)/p_sfc(i,1))**(rgas*beta/grav)-1.)
      end if
      enddo
 
      enddo

      return
      end
      SUBROUTINE SLPRS (SLP,PZ,PTOP,PHIS,THZ,DSIG,TM)                           
C***********************************************************************        
C                                                                               
C  INPUT                                                                        
C    PZ ...... SURFACE   PRESSURE (MB) - PTOP                                   
C    PTOP .... MODEL TOP PRESSURE (MB)                                          
C    PHIS .... SURFACE GEOPOTENTIAL (M2/S2)                                     
C    THZ ..... POTENTIAL TEMPERATURE (K) ON  SIGMA LEVELS                       
C    DSIG .... SIGMA INCREMENTS                                                 
C    TM ...... TEMPORARY SPACE                                                  
C                                                                               
C  OUTPUT                                                                       
C    SLP ..... SEA LEVEL PRESSURE (MB)                                          
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
                                                                                
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
                                                                                
      PARAMETER ( ZERO = 0.0   )                                                
      PARAMETER ( ONE  = 1.0   )                                                
      PARAMETER ( TWO  = 2.0   )                                                
      PARAMETER ( BETA = 0.0065)                                                
                                                                                
      DIMENSION  SLP (IMJMR),  PZ   (IMJMR)                                     
      DIMENSION  PHIS(IMJMR),  THZ  (IMJMR,NLAYR)                               
      DIMENSION  TM  (IMJMR)                                                    
      DIMENSION  DSIG(NLAYR)                                                    
                                                                                
      G  = GETCON('GRAVITY')                                                    
      R  = GETCON('RGAS')                                                       
      AK = GETCON('KAPPA')                                                      
                                                                                
      DO 10 I= 1,IMJMR                                                          
      TM(I)  = ZERO                                                             
   10 CONTINUE                                                                  
                                                                                
C***********************************************************************        
C*                COMPUTE MEAN THETA IN PBL (100 MB)                   *        
C***********************************************************************        
                                                                                
      DELP   = ZERO                                                             
      DELS   = ZERO                                                             
      DO 20 L=NLAYR,1,-1                                                        
      DELP   = DELP + (1000.-ptop)*DSIG(L)                                              
                                                                                
      IF (DELP.LE.100.) THEN                                                    
      DELS   = DELS + DSIG(L)                                                   
      DO 30 I= 1,IMJMR                                                          
      TM(I)  = TM(I) + THZ(I,L)*DSIG(L)                                         
   30 CONTINUE                                                                  
      ENDIF                                                                     
                                                                                
   20 CONTINUE                                                                  
      DO 40 I= 1,IMJMR                                                          
      TM(I)  = TM(I) / DELS                                                     
   40 CONTINUE                                                                  
                                                                                
C***********************************************************************        
C*                   COMPUTE SEA LEVEL PRESSURE                        *        
C***********************************************************************        
                                                                                
      DO 50 I=1,IMJMR                                                           
      TM (I) =   TM(I) * ( PZ(I)+PTOP )**AK + BETA * PHIS(I)/(TWO*G)            
      SLP(I) = PHIS(I) / ( R*TM(I) )                                            
      SLP(I) = ( PZ(I)+PTOP ) * EXP( SLP(I) )                                   
   50 CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE INTERP (U,UOT,PKE,PKO,PSTR,UNDEF)                              
C***********************************************************************        
C                                                                               
C INPUT                                                                         
C   UOT .... QUANTITY TO BE INTERPOLATED AT FULL LEVELS                         
C   PKE .... P TO THE KAPPA AT THE BASE OF EACH FULL LEVELS                     
C   PKO .... P TO THE KAPPA AT FULL LEVELS                                      
C   PSTR ... OUTPUT PRESSURE LEVEL (SCALAR) IN MB                               
C   UNDEF .. UNDEFINED VALUE                                                    
C                                                                               
C OUTPUT                                                                        
C   U ...... INTERPOLATED QUANTITY AT PSTR                                      
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
                                                                                
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
                                                                                
      DIMENSION UOT(IMJMR,NLAYR), PKO(IMJMR,NLAYR)                              
      DIMENSION   U(IMJMR)      , PKE(IMJMR,NLAYR)                              
                                                                                
      PARAMETER (ONE=1.0)                                                       
                                                                                
      AKP   = GETCON('KAPPA')                                                   
      PKSTR = PSTR**AKP                                                         
                                                                                
      DO 10 I=1,IMJMR                                                           
      U(I) = UNDEF                                                              
   10 CONTINUE                                                                  
                                                                                
      DO 20 L=1,NLAYR-1                                                         
      PKMIN = PKO(1,L)                                                          
      PKMAX = PKO(1,L+1)                                                        
      DO 30 I=2,IMJMR                                                           
      IF( PKO(I,L)  .LT.PKMIN ) PKMIN = PKO(I,L)                                
      IF( PKO(I,L+1).GT.PKMAX ) PKMAX = PKO(I,L+1)                              
   30 CONTINUE                                                                  
                                                                                
      IF (PKSTR .LE. PKMAX .AND. PKSTR .GE. PKMIN) THEN                         
      DO 40 I=1,IMJMR                                                           
      IF( PKSTR.LE.PKO(I,L+1) .AND. PKSTR.GE.PKO(I,L) ) THEN                    
      U(I) = (PKO(I,L)-PKSTR) / (PKO(I,L)-PKO(I,L+1))                           
      U(I) =  UOT(I,L+1)*U(I) +  UOT(I,L)*(ONE-U(I))                            
      ENDIF                                                                     
   40 CONTINUE                                                                  
      ENDIF                                                                     
   20 CONTINUE                                                                  
                                                                                
      DO 50 I=1,IMJMR                                                           
      IF( PKSTR.LT.PKO(I,1) ) THEN                                              
      U(I) = (PKO(I,1)-PKSTR) / (PKO(I,1)-PKO(I,2))                             
      U(I) =  UOT(I,2)*U(I)   +  UOT(I,1)*(ONE-U(I))                            
      ENDIF                                                                     
      IF( PKSTR.GT.PKO(I,NLAYR) .AND. PKSTR.LE.PKE(I,NLAYR) ) THEN              
      U(I) =  UOT(I,NLAYR)                                                      
      ENDIF                                                                     
   50 CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE INTERQ (T,Q,RH,TMP,QZ,PKE,PKO,PSTR,                            
     .                   UNDEF,TEM,TEM1,TEM2,TEM3,ITEM1,ITEM2)                  
C***********************************************************************        
C                                                                               
C INPUT                                                                         
C   QZ .... SPECIFIC HUMIDITY AT FULL LEVELS                                    
C   RH .... RELATIVE HUMIDITY AT FULL LEVELS                                    
C   TMP ... TEMPERATURE AT FULL LEVELS                                          
C   PKE ... P TO THE KAPPA AT THE BASE OF EACH FULL LEVELS                      
C   PKO ... P TO THE KAPPA AT FULL LEVELS                                       
C   PSTR .. OUTPUT PRESSURE LEVEL (SCALAR) IN MB                                
C   UNDEF . UNDEFINED VALUE                                                     
C   TEM ... SCRATCH ARRAY                                                       
C                                                                               
C OUTPUT                                                                        
C   T ..... TEMPERATURE AT PSTR                                                 
C   Q ..... SPECIFIC HUMIDITY (G/G) AT PSTR                                     
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
                                                                                
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
                                                                                
      DIMENSION    RH(IMJMR,NLAYR),  TMP(IMJMR,NLAYR),  PKO(IMJMR,NLAYR)        
      DIMENSION    QZ(IMJMR,NLAYR),  PKE(IMJMR,NLAYR)                           
      DIMENSION     Q(IMJMR)      ,    T(IMJMR)      ,  TEM(IMJMR,2)            
      DIMENSION  TEM1(IMJMR)      , TEM2(IMJMR)      , TEM3(IMJMR)              
      DIMENSION ITEM1(IMJMR)      ,ITEM2(IMJMR)                                 
                                                                                
      CALL INTERP (Q,RH, PKE,PKO,PSTR,UNDEF)                                    
      CALL INTERP (T,TMP,PKE,PKO,PSTR,UNDEF)                                    
                                                                                
      AKP   = GETCON('KAPPA')                                                   
      PKSTR = PSTR**AKP                                                         
                                                                                
      DO 10 I=1,IMJMR                                                           
      TEM(I,1) = PSTR                                                           
   10 CONTINUE                                                                  
                                                                                
      CALL QSTARS (T,TEM(1,1),                                                  
     .             TEM1,TEM2,TEM3,ITEM1,ITEM2,IMJMR,                            
     .             TEM(1,2) )                                                   
                                                                                
      DO 20 I=1,IMJMR                                                           
      IF( Q(I).NE.UNDEF ) Q(I) = Q(I) * TEM(I,2)                                
      IF( PKSTR.GT.PKO(I,NLAYR).AND.PKSTR.LE.PKE(I,NLAYR)) THEN                 
      Q(I) = QZ(I,NLAYR)                                                        
      ENDIF                                                                     
   20 CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE INTGEOP (PHI,THB,PKE,PKO,PHISS,PSTR,UNDEF,TEM)                 
C***********************************************************************        
C                                                                               
C INPUT                                                                         
C   THB .... POTENTIAL TEMPERATURE (T/PKO) AT FULL LEVELS                       
C   PKE .... P TO THE KAPPA AT THE BASE OF EACH LAYER                           
C   PKO .... P TO THE KAPPA AT FULL LEVELS                                      
C   PHISS .. SURFACE GEOPOTENTIAL                                               
C   PSTR ... OUTPUT PRESSURE LEVEL (SCALAR) IN MB                               
C   UNDEF .. UNDEFINED VALUE                                                    
C   TEM .... SCRATCH ARRAY                                                      
C                                                                               
C OUTPUT                                                                        
C   PHI .... GEOPOTENTIAL HEIGHT (M) AT PSTR                                    
C                                                                               
C***********************************************************************        
C*                  GODDARD LABORATORY FOR ATMOSPHERES                 *        
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
                                                                                
      PARAMETER ( IMJMR = IMR*(JMR+1) )                                         
                                                                                
      PARAMETER (HALF=.5)                                                       
      PARAMETER (ONE=1.)                                                        
      PARAMETER (TWO=2.)                                                        
                                                                                
      DIMENSION THB(IMJMR,NLAYR), PKE(IMJMR,NLAYR), PKO(IMJMR,NLAYR)            
      DIMENSION PHI(IMJMR)      , PHISS(IMJMR)    , TEM(IMJMR,5)                
                                                                                
      G   = GETCON('GRAVITY')                                                   
      CP  = GETCON('CP')                                                        
      AKP = GETCON('KAPPA')                                                     
                                                                                
      PKSTR  = PSTR ** AKP                                                      
      DO 10 I=1,IMJMR                                                           
      PHI(I) = UNDEF                                                            
   10 CONTINUE                                                                  
                                                                                
      PKMIN = PKO(1,NLAYR)                                                      
      DO 20 I=2,IMJMR                                                           
      IF( PKO(I,NLAYR).LT.PKMIN ) PKMIN = PKO(I,NLAYR)                          
   20 CONTINUE                                                                  
                                                                                
      IF( PKSTR.GE.PKMIN ) THEN                                                 
      DO 30 I=1,IMJMR                                                           
      IF (PKSTR .LE. PKE(I,NLAYR) .AND. PKSTR .GE. PKO(I,NLAYR)) THEN           
      PHI(I) = PHISS(I) + (PKE(I,NLAYR)-PKSTR) * THB(I,NLAYR)*CP                
      ENDIF                                                                     
   30 CONTINUE                                                                  
      ENDIF                                                                     
                                                                                
      DO 40 I=1,IMJMR                                                           
      TEM(I,5) = PHISS(I)                                                       
     .         +  (PKE(I,NLAYR)-PKO(I,NLAYR)) * THB(I,NLAYR)*CP                 
   40 CONTINUE                                                                  
                                                                                
      DO 310 L=NLAYR,2,-1                                                       
                                                                                
      PKMAX = PKO(1,L)                                                          
      DO 50 I=2,IMJMR                                                           
      IF( PKO(I,L).GT.PKMAX ) PKMAX = PKO(I,L)                                  
   50 CONTINUE                                                                  
      IF( PKSTR.GE.PKMAX ) GO TO 320                                            
                                                                                
      DO 60 I=1,IMJMR                                                           
      TEM(I,1) = THB(I,L-1)*(PKE(I,L-1)-PKO(I,L-1))                             
     .         + THB(I,L  )*(PKO(I,L  )-PKE(I,L-1))                             
                                                                                
      IF( PKSTR.GE.PKE(I,L-1) ) THEN                                            
      TEM(I,3) = PKO(I,L)                                                       
      TEM(I,2) = TEM(I,5)                                                       
      TEM(I,4) = THB(I,L)                                                       
      ENDIF                                                                     
                                                                                
      TEM(I,5) = TEM(I,5) + TEM(I,1) * CP                                       
                                                                                
      IF( PKSTR.LT.PKE(I,L-1) ) THEN                                            
      TEM(I,3) = PKO(I,L-1)                                                     
      TEM(I,2) = TEM(I,5  )                                                     
      TEM(I,4) = THB(I,L-1)                                                     
      ENDIF                                                                     
                                                                                
      IF( PKSTR.GE.PKO(I,L-1) .AND. PKSTR.LT.PKO(I,L) ) THEN                    
      PHI(I) = TEM(I,2)                                                         
      ENDIF                                                                     
                                                                                
      IF(L.EQ.2) THEN                                                           
      IF( PKSTR.LT.PKO(I,1) ) PHI(I) = TEM(I,2)                                 
      ENDIF                                                                     
                                                                                
      TEM(I,1) =  TEM(I,1)        / (PKO(I,L)-PKO(I,L-1))                       
      TEM(I,2) = (TEM(I,3)-PKSTR) / (TEM(I,3)-PKE(I,L-1))                       
                                                                                
      TEM(I,1) =  TEM(I,1) *      TEM(I,2)                                      
     .         +  TEM(I,4) * (TWO-TEM(I,2))                                     
      TEM(I,1) =  TEM(I,1) * (TEM(I,3)-PKSTR)                                   
                                                                                
      IF( PKSTR.GE.PKO(I,L-1) .AND. PKSTR.LT.PKO(I,L)) THEN                     
      PHI(I) = PHI(I) + TEM(I,1) * (CP*HALF)                                    
      ENDIF                                                                     
                                                                                
      IF(L.EQ.2) THEN                                                           
      IF( PKSTR.LT.PKO(I,1) ) PHI(I) = PHI(I) + TEM(I,1)*(CP*HALF)              
      ENDIF                                                                     
                                                                                
   60 CONTINUE                                                                  
  310 CONTINUE                                                                  
  320 CONTINUE                                                                  
                                                                                
      DO 70 I=1,IMJMR                                                           
      IF( PHI(I).NE.UNDEF ) PHI(I) = PHI(I) * (ONE/G)                           
   70 CONTINUE                                                                  
      RETURN                                                                    
      END                                                                       
      SUBROUTINE P2KAP ( PKE,PE,IRN,LM,PK,T1 )                                  
C***********************************************************************        
C  PURPOSE                                                                      
C     CALCULATE PHILLIPS P**KAPPA                                               
C  ARGUEMENTS                                                                   
C     PKE .... SIGMA EDGE-LEVEL PRESSURE**KAPPA                                 
C     PE ..... SIGMA EDGE-LEVEL PRESSURE                                        
C     IRN .... HORIZONTAL DIMENSION                                             
C     LM ..... VERTICAL   DIMENSION                                             
C     PK ..... SIGMA      LEVEL PRESSURE**KAPPA                                 
C     T1 ..... TEMPORARY  WORK ARRAY                                            
C                                                                               
C***********************************************************************        
C*                GODDARD LABORATORY FOR ATMOSPHERES                            
C***********************************************************************        
C                                                                               
      PARAMETER ( ONE = 1.0 )                                                   
                                                                                
      DIMENSION PKE(IRN,LM+1)                                                   
      DIMENSION  PE(IRN,LM+1)                                                   
      DIMENSION  PK(IRN,LM)                                                     
      DIMENSION  T1(IRN,LM+1)                                                   
                                                                                
      AKAP  = GETCON('KAPPA')                                                   
      MAXV  = GETCON('MAX VECT LENGTH')                                         
                                                                                
      IRNLM   = IRN*LM                                                          
      IRNLMP1 = IRN*(LM+1)                                                      
                                                                                
      IF( IRNLMP1.LT.MAXV ) THEN                                                
      DO 10 I=1,IRNLMP1                                                         
      T1(I,1) = PE(I,1)*PKE(I,1)                                                
   10 CONTINUE                                                                  
      DO 20 I=1,IRNLM                                                           
      PK(I,1) = ( T1(I,2)-T1(I,1) ) / ((ONE+AKAP)*( PE(I,2)-PE(I,1) ))          
   20 CONTINUE                                                                  
                                                                                
      ELSE                                                                      
                                                                                
      DO 30 L=1,LM+1                                                            
      DO 30 I=1,IRN                                                             
      T1(I,L) = PE(I,L)*PKE(I,L)                                                
   30 CONTINUE                                                                  
      DO 40 L=1,LM                                                              
      DO 40 I=1,IRN                                                             
      PK(I,L) = ( T1(I,L+1)-T1(I,L) )                                           
     *        / ((ONE+AKAP)*( PE(I,L+1)-PE(I,L) ))                              
   40 CONTINUE                                                                  
      ENDIF                                                                     
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE PKAPPA (PZ,PKHT,PKZ,IMAX,JMAX,KMAX)                            
C***********************************************************************        
C  PURPOSE                                                                      
C     CALCULATE PHILLIPS P**KAPPA                                               
C  ARGUMENTS                                                                    
C     PZ ..... SURFACE PRESSURE - PTOP                                          
C     PKHT ... SIGMA EDGE-LEVEL PRESSURE**KAPPA                                 
C     PKZ .... SIGMA      LEVEL PRESSURE**KAPPA                                 
C     IMAX ... LONGITUDE  DIMENSION                                             
C     JMAX ... LATITUDE   DIMENSION                                             
C     KMAX ... VERTICAL   DIMENSION                                             
C  SUBPROGRAMS NEEDED                                                           
C     EXPBYK   COMPUTES  P**KAPPA  AND KAPPA=.2861328125                        
C  RECORD OF MODIFICATIONS                                                      
C  REMARKS:                                                                     
C     NOTE:  PERTURBATION GEOPOTENTIALS ARE USED WITH                           
C             TSTD = 280 DEG. K.    ,  PSTD = 1000 MB.                          
C            PKSTD = PSTD ** KAPPA  , THSTD = TSTD / PKSTD                      
C                                                                               
C***********************************************************************        
C*                GODDARD LABORATORY FOR ATMOSPHERES                            
C***********************************************************************        
C                                                                               
#include <vrslv.com>                                                            
#include <vmetric.com>                                                          
C                                                                               
      PARAMETER (  ONE   = 1.0 )                                                
      PARAMETER (   CP   = 1004.16  )                                           
      PARAMETER ( RGAS   = 287.0   )                                            
      PARAMETER ( ROCP   = RGAS/CP )                                            
      PARAMETER ( ROCPP1 = ROCP + 1 )                                           
      PARAMETER ( TSTD   =  280.0  )                                            
      PARAMETER ( PSTD   = 1000.0  )                                            
      PARAMETER ( IMJMR  = IMR*(JMR+1) )                                        
C                                                                               
      DIMENSION PZ  (IMAX,JMAX)                                                 
      DIMENSION PKHT(IMAX,JMAX,KMAX)                                            
      DIMENSION PKZ (IMAX,JMAX,KMAX)                                            
C                                                                               
      DIMENSION PINV(IMR,JMR+1)                                                 
      DIMENSION PLE (IMR,JMR+1,2)                                               
      DIMENSION PKLE(IMR,JMR+1,2)                                               
C                                                                               
C **********************************************************************        
C ****                      INITIALIZATION                          ****        
C **********************************************************************        
C                                                                               
      DO 10 I=1,IMJMR                                                           
      PINV(I,1) = ONE / ( ROCPP1*PZ(I,1) )                                      
   10 CONTINUE                                                                  
C                                                                               
C **********************************************************************        
C ****                  COMPUTE PHILLIPS P**KAPPA                   ****        
C **********************************************************************        
C                                                                               
      LUP = 1                                                                   
      LDN = 2                                                                   
C                                                                               
      PKTOP = EXPBYK(PTOP)                                                      
      DO 20 I=1,IMJMR                                                           
      PLE (I,1,LUP) = PTOP                                                      
      PKLE(I,1,LUP) = PKTOP                                                     
   20 CONTINUE                                                                  
                                                                                
      DO 1000 L = 1,NLAY                                                        
      DSIGINV = ONE/DSIG(L)                                                     
      DO 30 I=1,IMJMR                                                           
      PLE (I,1,LDN) = SIGE(L+1)*PZ(I,1) + PTOP                                  
      PKLE(I,1,LDN) = PKHT(I,1,L)                                               
C                                                                               
      PKZ(I,1,L) = ( PLE (I,1,LDN)*PKLE(I,1,LDN)                                
     .             - PLE (I,1,LUP)*PKLE(I,1,LUP) )                              
     .           *   PINV(I,1)*DSIGINV                                          
   30 CONTINUE                                                                  
C                                                                               
      LSV = LUP                                                                 
      LUP = LDN                                                                 
      LDN = LSV                                                                 
 1000 CONTINUE                                                                  
C                                                                               
      RETURN                                                                    
      END                                                                       
      FUNCTION EXPBYK (XD)                                                      
C***********************************************************************        
C  PURPOSE                                                                      
C     COMPUTE  (EXPBYK = XD**KAPPA)                                             
C  ARGUMENTS   DESCRIPTION                                                      
C     XD       PRESSURE                                                         
C     EXPBYK   XD**0.2861328125                                                 
C***********************************************************************        
      EXPO   = GETCON('KAPPA')                                                  
      EXPBYK = XD**EXPO                                                         
C                                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE STRIP(A,B,IA,IB,L,K)                                           
      DIMENSION A(IA,L), B(IB,L)                                                
      INTEGER   OFFSET                                                          
                                                                                
      OFFSET = IB*(K-1)                                                         
      LEN    = MIN(IB,IA-OFFSET)                                                
      OFFSET = OFFSET+1                                                         
                                                                                
      IF(LEN.EQ.IB) THEN                                                        
        DO 100 J=1,L                                                            
        DO 100 I=1,LEN                                                          
        B(I,J) = A(I+OFFSET-1,J)                                                
100     CONTINUE                                                                
      ELSE                                                                      
        DO 200 J=1,L                                                            
        DO 300 I=1,LEN                                                          
        B(I,J) = A(I+OFFSET-1,J)                                                
300     CONTINUE                                                                
        DO 400 I=1,IB-LEN                                                       
        B(LEN+I,J) = A(1,J)                                            
400     CONTINUE                                                                
200     CONTINUE                                                                
      ENDIF                                                                     
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE PASTE(B,A,IB,IA,L,K)                                           
      DIMENSION A(IA,L), B(IB,L)                                                
      INTEGER   OFFSET                                                          
                                                                                
      OFFSET = IB*(K-1)                                                         
      LEN    = MIN(IB,IA-OFFSET)                                                
      OFFSET = OFFSET+1                                                         
                                                                                
      DO 100 J=1,L                                                              
      DO 100 I=1,LEN                                                            
      A(I+OFFSET-1,J) = B(I,J)                                                  
100   CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE PSTBMP(B,A,IB,IA,L,K)                                          
      DIMENSION A(IA,L), B(IB,L)                                                
      INTEGER   OFFSET                                                          
                                                                                
      OFFSET = IB*(K-1)                                                         
      LEN    = MIN(IB,IA-OFFSET)                                                
      OFFSET = OFFSET+1                                                         
                                                                                
      DO 100 J=1,L                                                              
      DO 100 I=1,LEN                                                            
      A(I+OFFSET-1,J) = A(I+OFFSET-1,J) + B(I,J)                                
100   CONTINUE                                                                  
C                                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE BDGBMP (B,A,IRN,IM,JM,LM,K)                                    
      PARAMETER ( ZERO = 0.0 )                                                  
      DIMENSION B(IRN,LM)                                                       
      REAL      A(JM,LM), TREAL(1440)                                           
      INTEGER   OFFSET                                                          
                                                                                
      OFFSET = 1                                                                
      JOFF = IRN*(K-1) + 1                                                      
                                                                                
90    J = (JOFF-1)/IM + 1                                                       
      IF(J.GT.JM) RETURN                                                        
      I = MOD(JOFF-1,IM) + 1                                                    
                                                                                
      IF(OFFSET+IM-1.LE.IRN) THEN                                               
        LEN = MIN(IM,IM+1-I)                                                    
      ELSE                                                                      
        LEN = IRN+1-OFFSET                                                      
      ENDIF                                                                     
                                                                                
      DO 100 L=1,LM                                                             
      SUM = ZERO                                                                
      DO 200 I=1,LEN                                                            
      TREAL(I) = B(I+OFFSET-1,L)                                                
      SUM      = SUM + TREAL(I)                                                 
200   CONTINUE                                                                  
      A(J,L) = A(J,L) + SUM / FLOAT(IM)                                         
100   CONTINUE                                                                  
                                                                                
      JOFF = JOFF + LEN                                                         
      OFFSET = OFFSET + LEN                                                     
      IF(OFFSET.LE.IRN) GO TO 90                                                
                                                                                
      RETURN                                                                    
      END                                                                       
      SUBROUTINE HARM (F,FNAME,L,IM)                                            
C                                                                               
      PARAMETER ( FOUR  = 4.0 )                                                 
      PARAMETER ( ONE   = 1.0 )                                                 
      PARAMETER ( AHALF = 0.5 )                                                 
      PARAMETER ( ZERO  = 0.0 )                                                 
      PARAMETER ( P001  = 0.001 )                                               
      DIMENSION IFAX (100)                                                       
      REAL      TRIGS(3*(IM+1))                                                
C                                                                               
      DIMENSION   F(IM), FZ(IM+2), C(2*(IM+2))                                    
      CHARACTER*4 FNAME                                                         
C                                                                               
      PI   = FOUR * ATAN(ONE)                                                   
      PID2 = PI   * AHALF                                                       
      CALL FFTFAX (IM,IFAX,TRIGS)                                               
      WRITE(6,100) FNAME, L                                                     
C                                                                               
      DO 10 I=1,IM                                                             
      FZ(I) = F(I)                                                              
   10 CONTINUE                                                                  
                                                                                
      CALL RFFTMLT (FZ,C,TRIGS,IFAX,1 ,IM+2,IM,1,-1)                           
C                                                                               
      DO 20 I=1,IM+2                                                           
      C(I) = FZ(I)                                                              
   20 CONTINUE                                                                  
C                                                                               
      WRITE(6,200) C(1)                                                         
      WRITE(6,300)                                                              
      KM  = IM/2                                                                
      KM1 = IM/2-1                                                              
      DO 30 K=1,KM1                                                             
      SIZE = FLOAT(IM)/FLOAT(K)                                                 
      K2   =  2*K+1                                                             
      AK   =  2*C(K2)                                                           
      BK   = -2*C(K2+1)                                                         
      AMPL = SQRT( AK**2 + BK**2 )                                              
C                                                                               
      IF(AK.EQ.ZERO) TEMP = PID2 - ( ONE - SIGN(ONE,BK) )*PID2                  
      IF(AK.NE.ZERO)                                                            
     .TEMP = ATAN(BK/AK)                                                        
     .     + SIGN(ONE,BK) * (ONE-SIGN(ONE,AK)) * PID2                           
C                                                                               
                            PHAZ = -TEMP                                        
      IF( AMPL.LT.P001) PHAZ =  ZERO                                            
      WRITE(6,400) K,SIZE,AK,BK,AMPL,PHAZ                                       
   30 CONTINUE                                                                  
C                                                                               
      AK   =  C(IM+1)                                                         
      BK   =  ZERO                                                              
      PHAZ = -(ONE-SIGN(ONE,AK)) * PID2                                         
      AMPL = ABS(AK)                                                            
C                                                                               
      IF( AMPL.LT.P001) PHAZ =  ZERO                                            
      SIZE = FLOAT(IM)/FLOAT(KM)                                                
      WRITE(6,400) KM,SIZE,AK,BK,AMPL,PHAZ                                      
C                                                                               
  100 FORMAT(//5X,'HARMONIC ANALYSIS FOR ',A4,' FIELD,  LEVEL = ',I2)           
  200 FORMAT(/1X,'MEAN VALUE:',2X,F13.4)                                        
  300 FORMAT(/1X,'WAVENUMBER',3X,'WAVELENGTH',                                  
     $ 6X,' COS ',7X,' SIN ',7X,' AMPL ',5X,' PHASE ')                          
  400 FORMAT(4X,I3,6X,F8.1,6X,F9.3,3X,F9.3,3X,F9.3,3X,F9.3)                     
      RETURN                                                                    
      END                                                                       
      SUBROUTINE ZMEAN (Q,QBAR,IMAX,JMAX,DT)                                    
#include <vrslv.com>                                                            
      PARAMETER ( ZERO = 0.0 )                                                  
      DIMENSION      Q(IMAX,JMAX)                                               
      REAL        QBAR(JMAX), SUM                                               
      REAL        QTMP(JMR+1), QFULL(IMR)                                       
C                                                                               
C *********************************************************                     
C ****          CALCULATE ZONAL MEAN                   ****                     
C *********************************************************                     
C                                                                               
      DO 10 J=1,JMAX                                                            
      SUM = ZERO                                                                
      DO 20 I=1,IMR                                                             
      QFULL(I) = Q(I,J)                                                         
           SUM = SUM + QFULL(I)                                                 
   20 CONTINUE                                                                  
      QTMP(J)  = SUM / IMR                                                      
      QBAR(J)  = QBAR(J) + QTMP(J)*DT                                           
   10 CONTINUE                                                                  
C                                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE ZMEANJ (Q,QBAR,IMAX,DT)                                        
#include <vrslv.com>                                                            
      PARAMETER ( ZERO = 0.0 )                                                  
      DIMENSION Q(IMAX)                                                         
      REAL      QFULL(IMR), QTMP, QBAR, SUM                                     
C                                                                               
C *********************************************************                     
C ****          CALCULATE ZONAL MEAN                   ****                     
C *********************************************************                     
C                                                                               
           SUM = ZERO                                                           
      DO 10 I=1,IMR                                                             
      QFULL(I) = Q(I)                                                           
           SUM = SUM + QFULL(I)                                                 
   10 CONTINUE                                                                  
      QTMP = SUM / IMR                                                          
      QBAR = QBAR + QTMP*DT                                                     
C                                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE QSTARS (T,P,E1,E2,TQ,I1,I2,N,QSAT)                             
C***********************************************************************        
C  FUNCTION VQSAT                                                               
C  PURPOSE                                                                      
C     VECTOR COMPUTATION OF SATURATION MIXING RATIO                             
C  USAGE                                                                        
C     QSATT(1;N) = VQSAT(T(1;N),P(1;N),N;QSATT(1;N))                            
C  DESCRIPTION OF PARAMETERS                                                    
C     T        TEMPERATURE VECTOR (DEG K)                                       
C     P        PRESSURE VECTOR (MB)                                             
C     VQSAT    SATURATION MIXING RATIO VECTOR                                   
C                                                                               
C***********************************************************************        
C                                                                               
      PARAMETER ( ONE    = 1.      )                                            
      PARAMETER ( ZP622  = 0.622   )                                            
      PARAMETER ( Z1P0S1 = 1.00001 )                                            
      PARAMETER ( Z1P622 = 1.622   )                                            
      PARAMETER ( Z138P9 = 138.90001 )                                          
      PARAMETER ( Z198P9 = 198.99999 )                                          
      PARAMETER ( Z200   = 200.0     )                                          
      PARAMETER ( Z337P9 = 337.9     )                                          
                                                                                
      DIMENSION T(N),P(N),E1(N),E2(N),TQ(N),I1(N),I2(N),QSAT(N)                 
C                                                                               
      DIMENSION    EST(139), EST1(67),  EST2(72)                                
      EQUIVALENCE (EST(1)  , EST1(1)), (EST(68),EST2(1))                        
C                                                                               
      DATA EST1/                                                                
     *                  0.31195E-02, 0.36135E-02, 0.41800E-02,                  
     *    0.48227E-02, 0.55571E-02, 0.63934E-02, 0.73433E-02,                   
     *    0.84286E-02, 0.96407E-02, 0.11014E-01, 0.12582E-01,                   
     *    0.14353E-01, 0.16341E-01, 0.18574E-01, 0.21095E-01,                   
     *    0.23926E-01, 0.27096E-01, 0.30652E-01, 0.34629E-01,                   
     *    0.39073E-01, 0.44028E-01, 0.49546E-01, 0.55691E-01,                   
     *    0.62508E-01, 0.70077E-01, 0.78700E-01, 0.88128E-01,                   
     *    0.98477E-01, 0.10983E+00, 0.12233E+00, 0.13608E+00,                   
     *    0.15121E+00, 0.16784E+00, 0.18615E+00, 0.20627E+00,                   
     *    0.22837E+00, 0.25263E+00, 0.27923E+00, 0.30838E+00,                   
     *    0.34030E+00, 0.37520E+00, 0.41334E+00, 0.45497E+00,                   
     *    0.50037E+00, 0.54984E+00, 0.60369E+00, 0.66225E+00,                   
     *    0.72589E+00, 0.79497E+00, 0.86991E+00, 0.95113E+00,                   
     *    0.10391E+01, 0.11343E+01, 0.12372E+01, 0.13484E+01,                   
     *    0.14684E+01, 0.15979E+01, 0.17375E+01, 0.18879E+01,                   
     *    0.20499E+01, 0.22241E+01, 0.24113E+01, 0.26126E+01,                   
     *    0.28286E+01, 0.30604E+01, 0.33091E+01, 0.35755E+01/                   
C                                                                               
      DATA EST2/                                                                
     *    0.38608E+01, 0.41663E+01, 0.44930E+01, 0.48423E+01,                   
     *    0.52155E+01, 0.56140E+01, 0.60394E+01, 0.64930E+01,                   
     *    0.69767E+01, 0.74919E+01, 0.80406E+01, 0.86246E+01,                   
     *    0.92457E+01, 0.99061E+01, 0.10608E+02, 0.11353E+02,                   
     *    0.12144E+02, 0.12983E+02, 0.13873E+02, 0.14816E+02,                   
     *    0.15815E+02, 0.16872E+02, 0.17992E+02, 0.19176E+02,                   
     *    0.20428E+02, 0.21750E+02, 0.23148E+02, 0.24623E+02,                   
     *    0.26180E+02, 0.27822E+02, 0.29553E+02, 0.31378E+02,                   
     *    0.33300E+02, 0.35324E+02, 0.37454E+02, 0.39696E+02,                   
     *    0.42053E+02, 0.44531E+02, 0.47134E+02, 0.49869E+02,                   
     *    0.52741E+02, 0.55754E+02, 0.58916E+02, 0.62232E+02,                   
     *    0.65708E+02, 0.69351E+02, 0.73168E+02, 0.77164E+02,                   
     *    0.81348E+02, 0.85725E+02, 0.90305E+02, 0.95094E+02,                   
     *    0.10010E+03, 0.10533E+03, 0.11080E+03, 0.11650E+03,                   
     *    0.12246E+03, 0.12868E+03, 0.13517E+03, 0.14193E+03,                   
     *    0.14899E+03, 0.15634E+03, 0.16400E+03, 0.17199E+03,                   
     *    0.18030E+03, 0.18895E+03, 0.19796E+03, 0.20733E+03,                   
     *    0.21708E+03, 0.22722E+03, 0.23776E+03, 0.24871E+03/                   
C                                                                               
C***********************************************************************        
C                                                                               
      A1622   = ONE  / Z1P622                                                   
      DO 10 I = 1,N                                                             
                           TQ(I) = T(I) - Z198P9                                
       IF(T(I).LT.Z200   ) TQ(I) = Z1P0S1                                       
       IF(T(I).GT.Z337P9 ) TQ(I) = Z138P9                                       
                           I1(I) = TQ(I)                                        
                           I2(I) = I1(I) + 1                                    
  10  CONTINUE                                                                  
C                                                                               
      DO 11 I = 1,N                                                             
       E1(I) =  EST(  I1(I) )                                                   
       E2(I) =  EST(  I2(I) )                                                   
  11  CONTINUE                                                                  
C                                                                               
      DO 12 I = 1,N                                                             
       QSAT(I)   = TQ(I) - I1(I)                                                
       QSAT(I)   = QSAT(I) * ( E2(I)-E1(I) )                                    
       QSAT(I)   = QSAT(I) +   E1(I)                                            
       E1(I) = P(I) * A1622                                                     
       IF( E1(I).LT.QSAT(I) ) QSAT(I) = E1(I)                                   
       QSAT(I)   = ZP622 *  QSAT(I) / P(I)                                      
  12  CONTINUE                                                                  
C                                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE PNTPRF(NN,IRUN,NLEV,NYMD,NHMS,FIELD,NAME)                      
C**********************************************************************         
C  PURPOSE                                                                      
C     STORES DIAGNOSTIC POINT BY POINT PROFILES                                 
C  INPUT PARAMETERS:                                                            
C      NN     - WHICH 'STRIP' IS BEING SENT                                     
C      IRUN   - FIRST DIMENSION OF INPUT VARIABLES                              
C      NLEV   - HOW MANY LEVELS                                                 
C      NYMD   - DATE IN YYMMDD FORMAT                                           
C      NHMS   - TIME IN HHMMSS FORMAT                                           
C      FIELD  - FIELD TO BE STORED - DIMENSION IRUN X NLEV                      
C**********************************************************************         
#include <vrslv.com>                                                            
#include <stripmine.com>
      PARAMETER( IMJMR = IMR * (JMR+1) )                                        
C                                                                               
C COMMON TO STORE DIAGNOSTIC ARRAYS - 100     POSSIBLE QUANTITIES,              
C                                     NLAYR+1 POSSIBLE LEVELS                   
C                                     20      POSSIBLE POINTS                   
C                                     50      POSSIBLE TIME PERIODS             
      COMMON /SAVPNT/ NDAYS(50),NHOURS(50)                                      
      COMMON /SAVPNT/ QNAMES(100),NUMLEVS(100)                                  
      CHARACTER * 16 QNAMES                                                     
      COMMON /SAVPNT/ INDXS(20),JNDXS(20)                                       
      COMMON /SAVPNT/ NPALL,NQUANTS,NTIMES                                      
      COMMON /SAVPNT/ PNTDIAG(1,1,1,1)                                          
C                                                                               
      DIMENSION FIELD(IRUN,NLEV)                                                
      CHARACTER * 16 NAME                                                       
      DIMENSION IJOUT(IMR,JMR+1),INDALL(IMJMR)                                  
      DIMENSION IJSTR(IRUN),INDEX(IRUN)                                         
C                                                                               
      LOGICAL FIRST                                                             
      DATA FIRST/.TRUE./                                                        
      SAVE                                                                      
      IF(FIRST)THEN                                                             
       NNM1    = 0                                                              
       NYMD1   = 0                                                              
       NHMS1   = 0                                                              
       NTIMES  = 0                                                              
       NPOINTS = 0                                                              
       NQUANTS = 0                                                              
C fill the array of which points are selected the first time here               
C  and then get the I (lon) and J (lat) indeces of the points                   
       CALL SETPRF(IMR,JMR+1,IJOUT)                                             
       CALL WHENEQ(IMJMR,IJOUT,1,1,INDALL,NPALL)                                
       DO IJ = 1,NPALL                                                          
        JNDXS(IJ) = INDALL(IJ)/IMR + 1                                          
        INDXS(IJ) = MOD(INDALL(IJ),IMR)                                         
       ENDDO                                                                    
       FIRST = .FALSE.                                                          
      ENDIF                                                                     
C                                                                               
C get the points for this strip only                                            
      CALL STRINT(IJOUT,IJSTR,IMJMR,IRUN,1,NN)                                  
C count up how many of ijstr are '1', (NPSTR) and put the indeces               
C                     into array index                                          
      CALL WHENEQ(IRUN,IJSTR,1,1,INDEX,NPSTR)                                   
C for a new time, bump the time count and zero out points and quantities        
      IF((NYMD.NE.NYMD1).OR.(NHMS.NE.NHMS1)) THEN                               
       NOTTIME = 1                                                              
       NTIMES = NTIMES + 1                                                      
       NQUANTS = 0                                                              
       NPOINTS = 0                                                              
      ELSE                                                                      
       NOTTIME = 0                                                              
      ENDIF                                                                     
C if we have any points in this strip we want, do some work                     
      IF(NPSTR.GT.0)THEN                                                        
C for a new strip, zero out the quantities counter                              
       IF((NN.NE.NNM1).OR.((NPCS.EQ.1).AND.(NOTTIME.EQ.1)))THEN                 
CCCC   IF(NN.NE.NNM1)THEN                                                       
        NQUANTS = 0                                                             
       ENDIF                                                                    
       NQUANTS = NQUANTS + 1                                                    
C if we are on the first quantity of a time period or of a strip,               
C      increment the counter for the number of points                           
       IF(NQUANTS.EQ.1)NPOINTS = NPOINTS + NPSTR                                
C FILL THE ARRAY KEEPING THE NAMES AND THE ONE KEEPING THE LEVELS               
       QNAMES(NQUANTS) = NAME                                                   
       NUMLEVS(NQUANTS) = NLEV                                                  
C fill the array keeping the date and time                                      
       NDAYS(NTIMES) = NYMD                                                     
       NHOURS(NTIMES) = NHMS                                                    
C now fill the saver array with the sent quantity for each chosen point         
       DO NS = NPOINTS-NPSTR+1,NPOINTS                                          
        II = INDEX(NS-NPOINTS+NPSTR)                                            
        DO LEV = 1,NLEV                                                         
         PNTDIAG(NQUANTS,LEV,NS,NTIMES) = FIELD(II,LEV)                         
        ENDDO                                                                   
       ENDDO                                                                    
      ENDIF                                                                     
C save the strip number and the date and time for the next call                 
      NNM1 = NN                                                                 
      NYMD1 = NYMD                                                              
      NHMS1 = NHMS                                                              
C                                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE SETPRF(IM,JM,IJOUT)                                            
      DIMENSION IJOUT(IM,JM)                                                    
      DO I = 1,IM*JM                                                            
       IJOUT(I,1) = 0                                                           
      ENDDO                                                                     
C                                                                               
       IJOUT( 5,13)=1                                                           
       IJOUT(24,14)=1                                                           
       IJOUT(66,15)=1                                                           
       IJOUT(42,16)=1                                                           
       IJOUT( 5,22)=1                                                           
       IJOUT( 5,23)=1                                                           
       IJOUT(43,26)=1                                                           
       IJOUT( 9,29)=1                                                           
       IJOUT(17,33)=1                                                           
       IJOUT(23,33)=1                                                           
       IJOUT(64,33)=1                                                           
       IJOUT(67,33)=1                                                           
       IJOUT( 9,34)=1                                                           
       IJOUT(26,34)=1                                                           
       IJOUT(39,36)=1                                                           
       IJOUT(52,37)=1                                                           
C                                                                               
      RETURN                                                                    
      END                                                                       
      SUBROUTINE PHNXDATE ( ADATE, ATIME )
      CHARACTER*8 ADATE, ATIME
      CALL DATE  (ADATE)
      CALL CLOCK (ATIME)
      RETURN
      END
      SUBROUTINE CLOCKS ( NCPU, NWCLK )
      CALL SECOND ( TCPU  )
      CALL TIMEF  ( TWCLK )
      NCPU  = - 100*TCPU
      NWCLK = - 0.1*TWCLK
      RETURN
      END
      subroutine   zeitbeg ( xname )
      character    xname*8
      return
      end
      subroutine   zeitend
      return
      end
      subroutine   zeitpri ( iunit )
      integer      iunit
      print *
      print *
      print *, '****************************************************'
      print *, '****                                            ****'
      print *, '****      ZEITSTUFF is NO LONGER SUPPORTED      ****'
      print *, '****     due to the MULTI-TASKING Environment   ****'
      print *, '****                                            ****'
      print *, '****      Please use CRAY System Utilities      ****'
      print *, '****     to monitor your program efficiency     ****'
      print *, '****                                            ****'
      print *, '****************************************************'
      print *
      print *
      return
      end
      subroutine zeitime ( nyymmdd, nss, vcpu )
      return
      end
