      SUBROUTINE MMCOV(XUV,IREG,NPF,NP2,PLON,PLAT,PSTD,GETI,GETJ,XX,    
     1                  GI,GJ,DX,Y1,Y2,DY)                              
                                                                        
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC           
CC                                                         CC           
CC    MMCOV  GENERATES THE MASS-MASS COVARIANCES           CC           
CC    USING CHORD-LENGTH DISTANCE FORMULA:                 CC           
CC                                                         CC           
CC .5*S**2 = 1-COS(Y1-Y2) + COS(Y1)*COS(Y2)*(1-COS(X1-X2)) CC           
CC                                                         CC           
CC    HORIZONTAL FCST ERROR CORRELATION OF (1) SEA LEVEL   CC           
CC    PRESSURE OR (2) GEOPOTENTIAL HEIGHT ARE INTERPOLATED CC           
CC    LINEARLY FROM A TABLE (HCORFE) INDEXED BY HORZ. DIST.CC           
CC                                                         CC           
CC                                                         CC           
CC    INPUT ARGUMENTS:                                     CC           
CC                                                         CC           
CC            GETI  -  ORDERING ARRAY, WITH GETJ, PUTS XX  CC           
CC            GETJ     INTO SYMMETRIC STORAGE FORM         CC           
CC            IREG  -  GEOGRAPHICAL REGEION (IN TABLE)     CC           
CC            NP2   -  .5*(NP+NH)(NP+NH+1)                 CC           
CC            PLAT  -  LATITUDES OF MASS POINTS            CC           
CC                     (INCLUDES GRID POINTS)              CC           
CC            PLON  -  LONGITUDES OF MASS POINTS           CC           
CC            PSTD  -  STD DEV'S OF ERROR AT MASS POINTS   CC           
CC  GI,GJ,DX,Y1,Y2  -  WORKING SPACE ARRAYS                CC           
CC            XUV   -  'PUV' (PRESSURE) OR 'ZUV' (HEIGHT)  CC           
CC                                                         CC           
CC    OUTPUT ARGUMENT:                                     CC           
CC                                                         CC           
CC            XX    -  MASS-MASS COVARIANCE                CC           
CC                                                         CC           
CC                                                         CC           
CC    ******  FORTRAN 77 VERSION *******                   CC           
CC                                                         CC           
CC                                                         CC           
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC           
                                                                        
                                                                        
      CHARACTER*3 XUV                                                   
                                                                        
      COMMON /CORTABS/ HCORFE(2000,3,20,2)                              
                                                                        

      logical first                                                     
      data first /.true./                                               
c      save first, radei, pi180
                                                                        
      DIMENSION PLON(*),PLAT(*),GETI(*),GETJ(*),XX(*)                   
      DIMENSION Y1(*),Y2(*),GI(*),GJ(*),DX(*),DY(*)                     
      DIMENSION PSTD(*)                                                 
                                                                        
      real      COSA(600000)                                               
                                                                        
      DATA   ZERO/0.0/                                                  
      DATA    ONE/1.0/                                                  
      DATA    TWO/2.0/                                                  
      DATA    TEN/10.0/                                                 
      DATA  TENTH/0.1/                                                  
      DATA   PT25/0.25/                                                 
      DATA    PT5/0.5/                                                  
      DATA SIXHUN/600.0/                                                
      DATA  ONE50/150.0/                                                
      DATA  PT001/0.001/                                                
                                                                        
              CALL GETCONH('RADE    ',RADE)                             
              CALL GETCONH('PI180   ',PI180)                            
                                                                        
C                                                                       
C     SET UP COORDINATE PAIRS                                           
C          HERE WE IDENTIFY GI WITH 'IGAT' IN OLD CODE                  
C                           GJ WITH 'JGAT' IN OLD CODE                  
C------------------------------------------------------                 
C     call zeitbeg('mmvsetup')                                          
                                                                        
      DO 10 N=1,NP2                                                     
              GI(N) = GETI(N) + TENTH                                   
              GJ(N) = GETJ(N) + TENTH                                   
 10   CONTINUE                                                          
                                                                        
                                                                        
      CALL A77HGATH(PLON,GI,Y1,NP2)                                     
      CALL A77HGATH(PLON,GJ,Y2,NP2)                                     
                                                                        
      DO 15 N=1,NP2                                                     
              DX(N) = COS((Y2(N)-Y1(N))*PI180)                          
 15   CONTINUE                                                          
                                                                        
      DO 20 N=1,NPF                                                     
              COSA(N) = COS( PLAT(N)*PI180 )                            
 20   CONTINUE                                                          
                                                                        
      CALL A77HGATH(PLAT,GI,Y1,NP2)                                     
      CALL A77HGATH(PLAT,GJ,Y2,NP2)                                     
                                                                        
      DO 25 N=1,NP2                                                     
              DY(N) = ONE                                               
              IF(Y1(N)*Y2(N) .LT. ZERO) DY(N) = -ONE                    
 25   CONTINUE                                                          
                                                                        
      CALL A77HGATH(COSA,GI,Y1,NP2)                                     
      CALL A77HGATH(COSA,GJ,Y2,NP2)                                     
C..............................................                         
C IN WHAT FOLLOWS, THE REMAINDER OF THE OLD                             
C MMCORR CODE IS PUT INTO ONE 'DO' LOOP                                 
C..............................................                         
C     call zeitend                                                      
                                                                        
      ICOR = 0                                                          
      IF(XUV.EQ.'PUV') ICOR=1                                           
      IF(XUV.EQ.'ZUV') ICOR=2                                           
      IF(XUV.EQ.'MIX') RETURN                                           
      IF(ICOR.EQ.0) then                                                
       print*,' FATAL: MMCOV>> Bad XUV encountered. '                   
       stop 'Fatal.'                                                    
      end if                                                            
                                                                        
C     call zeitbeg('mmvloop')                                           
                                                                        
      SQ2 = SQRT(TWO)                                                   
      DO 50 K=1,NP2                                                     
                                                                        
C             COSDX = COS(DX(K)*PI180)                                  
C             COSY1 = COS(Y1(K)*PI180)                                  
C             COSY2 = COS(Y2(K)*PI180)                                  
C             COSDY = COS((Y1(K)-Y2(K))*PI180)                          
                                                                        
C             S =  SQRT(TWO - TWO*DY(K) + TWO*Y1(K)*Y2(K)*(ONE-DX(K)))  
C             S =  SQRT(TWO - TWO*COSDY + TWO*COSY1*COSY2*(ONE-COSDX))  
                                                                        
              ZZ = SQRT((ONE-Y1(K)*Y1(K))*(ONE-Y2(K)*Y2(K)))            
              S =  SQ2 * SQRT( ABS(ONE - Y1(K)*Y2(K)*DX(K) - DY(K)*ZZ)) 
C             S =  SQ2 * SQRT( ONE - COSY1*COSY2*COSDX -                
C    .                  SGN*SQRT((ONE-COSY1*COSY1)*(ONE-COSY2*COSY2)))  
                                                                        
              D = S * RADE                                              
                                                                        
c             IF(D.LE.TEN) D=ZERO                                       
              if ( d .le. .5 ) d = 0.0
                                                                        
              DX(K) = D                                                 
                                                                        
              IF(D.LT.SIXHUN) S=PT5*D                                   
              IF(D.GE.SIXHUN) S=PT25*D + ONE50                          
                                                                        
              INDX = INT(S + PT001)                                     
                                                                        
              R = S - INDX                                              
                                                                        
              IND1 = INDX + 1                                           
              IND2 = IND1 + 1                                           
                                                                        
              T1 = HCORFE(IND1,1,IREG,ICOR)                             
              T2 = HCORFE(IND2,1,IREG,ICOR)                             
                                                                        
              XX(K) = T1 + (T2-T1)*R                                    
                                                                        
 50   CONTINUE                                                          
                                                                        
C     call zeitend                                                      
                                                                        
C                                                                       
C     FORM COVARIANCES BY MULTIPLYING CORRELATIONS BY VARIANCES         
                                                                        
C     call zeitbeg('mmstor')                                            
      CALL A77HGATH(PSTD,GI,Y1,NP2)                                     
      CALL A77HGATH(PSTD,GJ,Y2,NP2)                                     
                                                                        
      DO 60 K=1,NP2                                                     
                                                                        
        XX(K) = Y1(K) * Y2(K) * XX(K)                                   
                                                                        
 60   CONTINUE                                                          
C     call zeitend                                                      
      RETURN                                                            
      END                                                               
