CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC                   
CC                                                         CC                   
CC    WMCORR GENERATES THE WIND-MASS CORRELATIONS: UX,VX   CC                   
CC    USING CHORD-LENGTH DISTANCE FORMULA:                 CC                   
CC                                                         CC                   
CC .5*S**2 = 1-COS(Y1-Y2) + COS(Y1)*COS(Y2)*(1-COS(X1-X2)) CC                   
CC                                                         CC                   
CC    HORIZONTAL FCST ERROR CORRELATION OF (1) SEA LEVEL   CC                   
CC    PRESSURE OR (2) GEOPOTENTIAL HEIGHT ARE INTERPOLATED CC                   
CC    LINEARLY FROM A TABLE (HCORFE) INDEXED BY HORZ. DIST.CC                   
CC                                                         CC                   
CC                                                         CC                   
CC    INPUT ARGUMENTS:                                     CC                   
CC                                                         CC                   
CC            GI    -  'IGAT'                              CC                   
CC            GJ    -  'JGAT'                              CC                   
CC            IREG  -  GEOGRAPHICAL REGEION (IN TABLE)     CC                   
CC            NWP   -  (NW+NH) * NP                        CC                   
CC            PLAT  -  LATITUDES OF MASS POINTS            CC                   
CC                     (INCLUDES GRID POINTS)              CC                   
CC            PLON  -  LONGITUDES OF MASS POINTS           CC                   
CC           SCALE  -  DIMENSIONAL SCALE FACTOR            CC                   
CC        DX,Y1,Y2  -  WORKING SPACE ARRAYS                CC                   
CC            WLAT  -  LATITUDES OF WIND POINTS            CC                   
CC                     (INCLUDES GRID POINTS)              CC                   
CC            WLON  -  LONGITUDES OF WIND POINTS           CC                   
CC            XUV   -  'PUV' (PRESSURE) OR 'ZUV' (HEIGHT)  CC                   
CC                                                         CC                   
CC    OUTPUT ARGUMENT:                                     CC                   
CC                                                         CC                   
CC            UX    -  U WIND - MASS COVARIANCE            CC                   
CC            VX    -  V WIND - MASS COVARIANCE            CC                   
CC                     TO FORM COVARIANCES LATER)          CC                   
CC                                                         CC                   
CC                                                         CC                   
CC    ******  FORTRAN 77 VERSION *******                   CC                   
CC                                                         CC                   
CC                                                         CC                   
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC                   
                                                                                
      SUBROUTINE WMCORR0(XUV,IREG,NWP,PLON,PLAT,WLON,WLAT,UX,VX,                
     1                  SCALE,GI,GJ,DX,Y1,Y2)                                   
                                                                                
      CHARACTER*3 XUV                                                           
                                                                                
      LOGICAL FIRST                                                             
      data first /.true./                                                       
      save first
                                                                                
      DIMENSION PLON(1),PLAT(1),WLON(1),WLAT(1),UX(1),VX(1)                     
      DIMENSION GI(1),GJ(1),Y1(1),Y2(1),DX(1)                                   
                                                                                
      COMMON/CORTABS/ HCORFE(2000,3,20,2)                                       
                                                                                
      DATA   ZERO/0.0/                                                          
      DATA    ONE/1.0/                                                          
      DATA    TWO/2.0/                                                          
      DATA    TEN/10.0/                                                         
      DATA   PT25/0.25/                                                         
      DATA    PT5/0.5/                                                          
      DATA SIXHUN/600.0/                                                        
      DATA  ONE50/150.0/                                                        
      DATA  PT001/0.001/                                                        
      save rade, pi180
                                                                                
              CALL GETCONH('RADE    ',RADE)                                     
              CALL GETCONH('PI180   ',PI180)                                    
C------------------------------------------------------                         
                                                                                
C                                                                               
C     SET UP COORDINATE PAIRS                                                   
                                                                                
                                                                                
      CALL A77HGATH(WLON,GI,Y1,NWP)                                             
      CALL A77HGATH(PLON,GJ,Y2,NWP)                                             
                                                                                
      DO 15 N=1,NWP                                                             
              DX(N) = Y1(N) - Y2(N)                                             
 15   CONTINUE                                                                  
                                                                                
      CALL A77HGATH(WLAT,GI,Y1,NWP)                                             
      CALL A77HGATH(PLAT,GJ,Y2,NWP)                                             
                                                                                
C..............................................                                 
C IN WHAT FOLLOWS, THE REMAINDER OF THE OLD                                     
C WMCORR CODE IS PUT INTO ONE 'DO' LOOP                                         
C..............................................                                 
                                                                                
      ICOR = 0                                                                  
      IF(XUV.EQ.'PUV') ICOR=1                                                   
      IF(XUV.EQ.'ZUV') ICOR=2                                                   
      IF(XUV.EQ.'MIX') RETURN                                                   
      IF(ICOR.EQ.0) then                                                        
        print*,' FATAL: WMCORR>> Bad XUV passed into routine.'                  
        stop 'Fatal.'                                                           
      end if                                                                    
                                                                                
      DO 50 K=1,NWP                                                             
                                                                                
              SINDX = SIN(DX(K)*PI180)                                          
              COSDX = COS(DX(K)*PI180)                                          
                                                                                
              SINY1 = SIN(Y1(K)*PI180)                                          
              COSY1 = COS(Y1(K)*PI180)                                          
              COSY2 = COS(Y2(K)*PI180)                                          
              COSDY = COS((Y1(K)-Y2(K))*PI180)                                  
              SINDY = SIN((Y1(K)-Y2(K))*PI180)                                  
                                                                                
              S =  SQRT(TWO - TWO*COSDY + TWO*COSY1*COSY2*(ONE-COSDX))          
                                                                                
              D = S * RADE                                                      
                                                                                
              IF(D.LE.TEN) THEN                                                 
                                                                                
                 D   = ZERO                                                     
                 DY1 = ONE                                                      
                 DX1 = COSY1                                                    
                                                                                
              ELSE                                                              
                                                                                
                 SI  = ONE/S                                                    
                 DY1 = SI*(SINDY-SINY1*COSY2*(ONE-COSDX))                       
                 DX1 = SI*COSY1*COSY2*SINDX                                     
                                                                                
              ENDIF                                                             
                                                                                
              IF(D.LT.SIXHUN) S=PT5*D                                           
              IF(D.GE.SIXHUN) S=PT25*D + ONE50                                  
                                                                                
              INDX = INT(S + PT001)                                             
                                                                                
              R = S - INDX                                                      
                                                                                
              IND1 = INDX + 1                                                   
              IND2 = IND1 + 1                                                   
                                                                                
              T1 = HCORFE(IND1,2,IREG,ICOR)                                     
              T2 = HCORFE(IND2,2,IREG,ICOR)                                     
                                                                                
              XX = T1 + (T2-T1)*R                                               
                                                                                
              UX(K) = DY1 * XX                                                  
              VX(K) = DX1 * XX / COSY1                                          
                                                                                
 50   CONTINUE                                                                  
                                                                                
C                                                                               
C     USE DERIVATIVE SCALE INFORMATION FROM 2ND DERIVATIVE                      
C     AT ORIGIN - FIRST ENTRY IN 'HCORFE(,3,,)                                  
C                                                                               
C                                                                               
C     SCALE =  SQRT(-HCORFE(1,3,IREG,ICOR))                                     
                                                                                
      SCALEI = ONE/SCALE                                                        
                                                                                
      DO 60 K=1,NWP                                                             
                                                                                
        UX(K) = SCALEI * UX(K)                                                  
        VX(K) = SCALEI * VX(K)                                                  
                                                                                
 60   CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
