      SUBROUTINE WWCORR(XUV,IREG,NW2,WLON,WLAT,XU,XV,UU,UV,VU,VV,               
     1                  GI,GJ,DX,Y1,Y2)                                         
                                                                                
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC         
C                                                                     C         
C  SUBROUTINE WWCORR COMPUTES CHORD LENGTH DISTANCE FOR A MATRIX OF   C         
C  LOCATIONS <(X,Y)I,(X,Y)J> USING THE NORMAL ANGLE FORMULA:          C         
C                                                                     C         
C     S**2/2 = 1 - COS(Y1-Y2) + COS(Y1)*COS(Y2)*(1-COS(X1-X2))        C         
C                                                                     C         
C  1ST AND 2ND DERIVATIVES OF THE HORIZONTAL CORRELATION OF FORECAST  C         
C  ERROR OF EITHER SL PRESSURE OR GEOPOTENTIAL HEIGHT WITH RESPECT    C         
C  TO DISTANCE ARE INTERPOLATED FROM A TABLE AND DERIVATIVES WITH     C         
C  RESPECT TO X AND Y ARE COMPUTED                                    C         
C                                                                     C         
C  INPUT ARGUMENTS:                                                   C         
C                                                                     C         
C     GI     - IGAT                                                   C         
C     GJ     - JGAT                                                   C         
C     XUV    - EITHER 'PUV' (PRESSURE) OR 'ZUV' (HEIGHT)              C         
C     IREG   - REGION IN TABLE TO USE                                 C         
C     NW     - NUMBER OF WIND OBS                                     C         
C     NWF    - NW + NH                                                C         
C     WLAT   - LATITUDES OF WIND POINTS                               C         
C     WLON   - LONGITUDES OF WIND POINTS                              C         
C                                                                     C         
C  OUTPUT ARGUMENTS:                                                  C         
C                                                                     C         
C     XU    - FIRST  DERIVATIVE OF XX WITH RESPECT TO Y2              C         
C     XV    - FIRST  DERIVATIVE OF XX WITH RESPECT TO X2              C         
C     UU    - SECOND DERIVATIVE OF XX WITH RESPECT TO Y1 AND Y2       C         
C     UV    - SECOND DERIVATIVE OF XX WITH RESPECT TO Y1 AND X2       C         
C     VU    - SECOND DERIVATIVE OF XX WITH RESPECT TO X1 AND Y2       C         
C     VV    - SECOND DERIVATIVE OF XX WITH RESPECT TO X1 AND X2       C         
C                                                                     C         
C  PROGRAMMER: J.WOOLEN 7/85                                          C         
C              J.WOOLEN 9/86                                          C         
C                                                                     C         
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC         
                                                                                
                                                                                
      DIMENSION WLON(*),WLAT(*),XU(*),XV(*),UU(*),VV(*),UV(*),VU(*)             
      DIMENSION GI(*),GJ(*),DX(*),Y1(*),Y2(*)                                   
                                                                                
      CHARACTER*3 XUV                                                           
                                                                                
      COMMON /CORTABS/ HCORFE(2000,3,20,2)                                      
                                                                                
                                                                                
                                                                                
      CALL GETCONH('RADE    ',RADE)                                          
      CALL GETCONH('PI180   ',PI180)                                         
C                                                                               
C.... SET COORDINATE PAIRS                                                      
C                                                                               
                                                                                
      CALL A77HGATH(WLON,GI,Y1,NW2)                                             
      CALL A77HGATH(WLON,GJ,Y2,NW2)                                             
                                                                                
      DO 10 N=1,NW2                                                             
                                                                                
          DX(N) = Y1(N) - Y2(N)                                                 
                                                                                
10    CONTINUE                                                                  
                                                                                
      CALL A77HGATH(WLAT,GI,Y1,NW2)                                             
      CALL A77HGATH(WLAT,GJ,Y2,NW2)                                             
                                                                                
      ICOR = 0                                                                  
      IF(XUV.EQ.'PUV') ICOR = 1                                                 
      IF(XUV.EQ.'ZUV') ICOR = 2                                                 
      IF(ICOR.EQ.0 ) then                                                       
        print*,' FATAL: WWCORR>> Bad XUV passed into subroutine.'               
        stop 'Fatal.'                                                           
      end if                                                                    
C                                                                               
C..... OBTAIN DERIVATIVE SCALE INFORMATION FROM HCORFE                          
C                                                                               
      SCALE2  = ABS(HCORFE(1,3,IREG,ICOR))                                      
      SCALE2I = 1.0/SCALE2                                                      
      SCALEI  =  SQRT(SCALE2I)                                                  
                                                                                
C  REMAINDER OF WWCORR CODE NOW IN 'DO' LOOP                                    
C  ---------------------------------------                                      
                                                                                
      DO 50 K=1,NW2                                                             
                                                                                
          SINDX  = SIN(DX(K)*PI180)                                             
          COSDX  = COS(DX(K)*PI180)                                             
                                                                                
          SINY1  = SIN(Y1(K)*PI180)                                             
          SINY2  = SIN(Y2(K)*PI180)                                             
          COSY1  = COS(Y1(K)*PI180)                                             
          COSY2  = COS(Y2(K)*PI180)                                             
                                                                                
          SINDY  = SIN((Y1(K)-Y2(K))*PI180)                                     
          COSDY  = COS((Y1(K)-Y2(K))*PI180)                                     
                                                                                
                                                                                
          S  = SQRT(2.0 - 2.0*COSDY + 2.0*COSY1*COSY2*(1.0-COSDX))              
                                                                                
          D  = S * RADE                                                         
                                                                                
          IF(D.LE.0.5) THEN                                                     
                                                                                
              UU(K) =  1.0                                                      
              VV(K) =  1.0                                                      
                                                                                
              UV(K) =  0.0                                                      
              VU(K) =  0.0                                                      
              xu(K) =  0.0                                                      
              xv(K) =  0.0                                                      
                                                                                
          ELSE                                                                  
                                                                                
              SI  = 1.0/S                                                       
              DY1 =  SI*(SINDY-SINY1*COSY2*(1.0-COSDX))                         
              DX1 =  SI*(COSY1*COSY2*SINDX)                                     
              DY2 = -SI*(SINDY+COSY1*SINY2*(1.0-COSDX))                         
              DX2 = -DX1                                                        
                                                                                
                                                                                
C  CALCULATE THE MIXED PARTIALS FOR THE GEOSTROPHIC COVARIANCES                 
C  ------------------------------------------------------------                 
                                                                                
             DY1Y2=  SI*(SINY1*SINY2*(1.0-COSDX)-COSDY-DY1*DY2)                 
             DY1X2=  SI*(SINY1*COSY2*SINDX-DY1*DX2)                             
             DX1X2= -SI*(COSY1*COSY2*COSDX+DX1*DX2)                             
             DX1Y2= -SI*(COSY1*SINY2*SINDX+DX1*DY2)                             
                                                                                
C  LOOK UP THE CORRELATIONS AND COMPUTE APPROPRIATE DERIVATIVES                 
C  ------------------------------------------------------------                 
                                                                                
                                                                                
             IF(D.LT. 600.0) S = 0.5 * D                                        
             IF(D.GE. 600.0) S = 0.25* D + 150.0                                
                                                                                
             INDX  = INT(S+1.e-3)                                               
                                                                                
             R     = S - INDX                                                   
                                                                                
             IND1  = INDX + 1                                                   
             IND2  = IND1 + 1                                                   
                                                                                
             T1    = HCORFE(IND1,2,IREG,ICOR)                                   
             T2    = HCORFE(IND2,2,IREG,ICOR)                                   
             DMU   = T1 + (T2-T1)*R                                             
                                                                                
             XU(K) = SCALEI * DMU * DY2                                         
             XV(K) = SCALEI * DMU * DX2 /COSY2                                  
                                                                                
             T1    = HCORFE(IND1,3,IREG,ICOR)                                   
             T2    = HCORFE(IND2,3,IREG,ICOR)                                   
             D2MU  = T1 + (T2-T1)*R                                             
                                                                                
             UU(K) = SCALE2I*(DY1*DY2*D2MU + DY1Y2*DMU)                         
             UV(K) = SCALE2I*(DY1*DX2*D2MU + DY1X2*DMU)/COSY2                   
             VU(K) = SCALE2I*(DX1*DY2*D2MU + DX1Y2*DMU)/COSY1                   
             VV(K) = SCALE2I*(DX1*DX2*D2MU + DX1X2*DMU)/(COSY1*COSY2)           
                                                                                
          END IF                                                                
                                                                                
 50   CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
