CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC         
C                                                                     C         
C  SUBROUTINE WWCORR COMPUTES CHORD LENGTH DISTANCE FOR A MATRIX OF   C         
C  LOCATIONS <(X,Y)I,(X,Y)J> USING THE NORMAL ANGLE FORMULA:          C         
C                                                                     C         
C     S**2/2 = 1 - COS(Y1-Y2) + COS(Y1)*COS(Y2)*(1-COS(X1-X2))        C         
C                                                                     C         
C  1ST AND 2ND DERIVATIVES OF THE HORIZONTAL CORRELATION OF FORECAST  C         
C  ERROR OF EITHER SL PRESSURE OR GEOPOTENTIAL HEIGHT WITH RESPECT    C         
C  TO DISTANCE ARE INTERPOLATED FROM A TABLE AND DERIVATIVES WITH     C         
C  RESPECT TO X AND Y ARE COMPUTED                                    C         
C                                                                     C         
C  INPUT ARGUMENTS:                                                   C         
C                                                                     C         
C     GI     - IGAT                                                   C         
C     GJ     - JGAT                                                   C         
C     XUV    - EITHER 'PUV' (PRESSURE) OR 'ZUV' (HEIGHT)              C         
C     IREG   - REGION IN TABLE TO USE                                 C         
C     NW     - NUMBER OF WIND OBS                                     C         
C     NWF    - NW + NH                                                C         
C     WLAT   - LATITUDES OF WIND POINTS                               C         
C     WLON   - LONGITUDES OF WIND POINTS                              C         
C                                                                     C         
C  OUTPUT ARGUMENTS:                                                  C         
C                                                                     C         
C     XU    - FIRST  DERIVATIVE OF XX WITH RESPECT TO Y2              C         
C     XV    - FIRST  DERIVATIVE OF XX WITH RESPECT TO X2              C         
C     UU    - SECOND DERIVATIVE OF XX WITH RESPECT TO Y1 AND Y2       C         
C     UV    - SECOND DERIVATIVE OF XX WITH RESPECT TO Y1 AND X2       C         
C     VU    - SECOND DERIVATIVE OF XX WITH RESPECT TO X1 AND Y2       C         
C     VV    - SECOND DERIVATIVE OF XX WITH RESPECT TO X1 AND X2       C         
C                                                                     C         
C  PROGRAMMER: J.WOOLEN 7/85                                          C         
C              J.WOOLEN 9/86                                          C         
C                                                                     C         
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC         
                                                                                
      SUBROUTINE WWCORR0(XUV,IREG,NW2,WLON,WLAT,XU,XV,UU,UV,VU,VV,              
     1                  GI,GJ,DX,Y1,Y2)                                         
                                                                                
                                                                                
      DIMENSION WLON(1),WLAT(1),XU(1),XV(1),UU(1),VV(1),UV(1),VU(1)             
      DIMENSION GI(1),GJ(1),DX(1),Y1(1),Y2(1)                                   
                                                                                
      CHARACTER*3 XUV                                                           
                                                                                
      COMMON /CORTABS/ HCORFE(2000,3,20,2)                                      
                                                                                
      LOGICAL FIRST                                                             
      data first /.true./                                                       
      save first, rade, pi180
                                                                                
      DATA   ZERO/0.0/                                                          
      DATA    ONE/1.0/                                                          
      DATA    TWO/2.0/                                                          
      DATA    TEN/10.0/                                                         
      DATA   PT25/0.25/                                                         
      DATA    PT5/0.5/                                                          
      DATA SIXHUN/600./                                                         
      DATA  ONE50/150./                                                         
      DATA  PT001/0.001/                                                        
                                                                                
C----------------------------------------------------------------------         
      IF(FIRST) THEN                                                            
         CALL GETCONH('RADE    ',RADE)                                          
         CALL GETCONH('PI180   ',PI180)                                         
         FIRST = .FALSE.                                                        
      ENDIF                                                                     
C----------------------------------------------------------------------         
C                                                                               
C.... SET COORDINATE PAIRS                                                      
C                                                                               
                                                                                
      CALL A77HGATH(WLON,GI,Y1,NW2)                                             
      CALL A77HGATH(WLON,GJ,Y2,NW2)                                             
                                                                                
      DO 10 N=1,NW2                                                             
                                                                                
          DX(N) = Y1(N) - Y2(N)                                                 
                                                                                
10    CONTINUE                                                                  
                                                                                
      CALL A77HGATH(WLAT,GI,Y1,NW2)                                             
      CALL A77HGATH(WLAT,GJ,Y2,NW2)                                             
                                                                                
      ICOR = 0                                                                  
      IF(XUV.EQ.'PUV') ICOR = 1                                                 
      IF(XUV.EQ.'ZUV') ICOR = 2                                                 
      IF(ICOR.EQ.0 ) then                                                       
       print*,' FATAL: WWCOR>> Bad XUV passed into routine.'                    
       stop 'Fatal.'                                                            
      end if                                                                    
C                                                                               
C..... OBTAIN DERIVATIVE SCALE INFORMATION FROM HCORFE                          
C                                                                               
      SCALE2  = ABS(HCORFE(1,3,IREG,ICOR))                                      
      SCALE2I = ONE/SCALE2                                                      
      SCALEI  = sqrt (SCALE2I)                                                  
                                                                                
C  REMAINDER OF WWCORR CODE NOW IN 'DO' LOOP                                    
C  ---------------------------------------                                      
                                                                                
      DO 50 K=1,NW2                                                             
                                                                                
          SINDX  = SIN(DX(K)*PI180)                                             
          COSDX  = COS(DX(K)*PI180)                                             
                                                                                
          SINY1  = SIN(Y1(K)*PI180)                                             
          SINY2  = SIN(Y2(K)*PI180)                                             
          COSY1  = COS(Y1(K)*PI180)                                             
          COSY2  = COS(Y2(K)*PI180)                                             
                                                                                
          SINDY  = SIN((Y1(K)-Y2(K))*PI180)                                     
          COSDY  = COS((Y1(K)-Y2(K))*PI180)                                     
                                                                                
                                                                                
          S  = sqrt (TWO - TWO*COSDY + TWO*COSY1*COSY2*(ONE-COSDX))             
                                                                                
          D  = S * RADE                                                         
                                                                                
          IF(D.LE.TEN) THEN                                                     
                                                                                
              D   =  ZERO                                                       
              SI  =  ZERO                                                       
              DY1 =  ONE                                                        
              DY2 = -ONE                                                        
              DX1 =  COSY1                                                      
              DX2 = -COSY1                                                      
                                                                                
          ELSE                                                                  
                                                                                
              SI  = ONE/S                                                       
              DY1 =  SI*(SINDY-SINY1*COSY2*(ONE-COSDX))                         
              DX1 =  SI*(COSY1*COSY2*SINDX)                                     
              DY2 = -SI*(SINDY+COSY1*SINY2*(ONE-COSDX))                         
              DX2 = -DX1                                                        
                                                                                
          END IF                                                                
                                                                                
C  CALCULATE THE MIXED PARTIALS FOR THE GEOSTROPHIC COVARIANCES                 
C  ------------------------------------------------------------                 
                                                                                
          DY1Y2=  SI*(SINY1*SINY2*(ONE-COSDX)-COSDY-DY1*DY2)                    
          DY1X2=  SI*(SINY1*COSY2*SINDX-DY1*DX2)                                
          DX1X2= -SI*(COSY1*COSY2*COSDX+DX1*DX2)                                
          DX1Y2= -SI*(COSY1*SINY2*SINDX+DX1*DY2)                                
                                                                                
C  LOOK UP THE CORRELATIONS AND COMPUTE APPROPRIATE DERIVATIVES                 
C  ------------------------------------------------------------                 
                                                                                
                                                                                
          IF(D.LT.SIXHUN) S = PT5 * D                                           
          IF(D.GE.SIXHUN) S = PT25* D + ONE50                                   
                                                                                
          INDX   = INT(S+PT001)                                                 
                                                                                
          R      = S - INDX                                                     
                                                                                
          IND1   = INDX + 1                                                     
          IND2   = IND1 + 1                                                     
                                                                                
          T1     = HCORFE(IND1,2,IREG,ICOR)                                     
          T2     = HCORFE(IND2,2,IREG,ICOR)                                     
          DMU    = T1 + (T2-T1)*R                                               
                                                                                
          XU(K)  = SCALEI * DMU * DY2                                           
          XV(K)  = SCALEI * DMU * DX2 /COSY2                                    
                                                                                
          T1     = HCORFE(IND1,3,IREG,ICOR)                                     
          T2     = HCORFE(IND2,3,IREG,ICOR)                                     
          D2MU   = T1 + (T2-T1)*R                                               
                                                                                
          UU(K)  = SCALE2I * (DY1*DY2*D2MU + DY1Y2*DMU)                         
          UV(K)  = SCALE2I * (DY1*DX2*D2MU + DY1X2*DMU)/COSY2                   
          VU(K)  = SCALE2I * (DX1*DY2*D2MU + DX1Y2*DMU)/COSY1                   
          VV(K)  = SCALE2I * (DX1*DX2*D2MU + DX1X2*DMU)/(COSY1*COSY2)           
                                                                                
          IF(D.EQ.ZERO) THEN                                                    
                                                                                
              XU(K) = ZERO                                                      
              XV(K) = ZERO                                                      
              VU(K) = ZERO                                                      
              UV(K) = ZERO                                                      
              UU(K) = ONE                                                       
              VV(K) = ONE                                                       
                                                                                
          END IF                                                                
                                                                                
 50   CONTINUE                                                                  
                                                                                
      RETURN                                                                    
      END                                                                       
